@@ -78,7 +78,7 @@ void gettimeofday(struct timeval *tv, void /*struct timezone*/ *tz);
 
 #ifndef MACOSX
 #include <malloc.h>
-#ifndef SOLARIS
+#if !defined(SOLARIS) && !defined(__ANDROID__)
 #include <error.h>
 #endif
 #endif
@@ -499,7 +499,7 @@ int iax_get_netstats(struct iax_session *session, int *rtt, struct iax_netstat *
 {
 	jb_info stats;
 
-	if(!iax_session_valid(session)) return -1;
+	if(!session || !iax_session_valid(session)) return -1;
 
 	*rtt = session->pingtime;
 
@@ -1843,7 +1843,7 @@ static void send_ping(void *s)
 	struct iax_session *session = (struct iax_session *)s;
 
 	/* important, eh? */
-	if(!iax_session_valid(session)) return;
+	if(!session || !iax_session_valid(session)) return;
 
 	send_command(session, AST_FRAME_IAX, IAX_COMMAND_PING, 0, NULL, 0, -1);
 	session->pingid = iax_sched_add(NULL,NULL, send_ping, (void *)session, ping_time * 1000);
@@ -2456,6 +2456,14 @@ static void iax_handle_vnak(struct iax_session *session, struct ast_iax2_full_hd
 	}
 }
 
+static void
+copy_sockaddr_in(struct sockaddr_in *dest, struct sockaddr_in *src)
+{
+    dest->sin_port = src->sin_port;
+    dest->sin_addr = src->sin_addr;
+    dest->sin_family = AF_INET; /* it's always AF_INET */
+}
+
 static struct iax_event *iax_header_to_event(struct iax_session *session, struct ast_iax2_full_hdr *fh, int datalen, struct sockaddr_in *sin)
 {
 	struct iax_event *e;
@@ -2764,8 +2772,11 @@ static struct iax_event *iax_header_to_event(struct iax_session *session, struct
 					/* so a full voice frame is sent on the
 					   next voice output */
 					session->svoiceformat = -1;
-					session->transfer = *e->ies.apparent_addr;
-					session->transfer.sin_family = AF_INET;
+
+                    /* don't shallow copy struct by assignment (this avoids
+                     * unaligned accesses) */
+                    copy_sockaddr_in(&session->transfer, e->ies.apparent_addr);
+
 					session->transfercallno = e->ies.callno;
 					session->transferring = TRANSFER_BEGIN;
 					session->transferid = e->ies.transferid;
@@ -2781,7 +2792,7 @@ static struct iax_event *iax_header_to_event(struct iax_session *session, struct
 				break;
 			case IAX_COMMAND_TXCNT:
 				if (session->transferring)  {
-					session->transfer = *sin;
+					copy_sockaddr_in(&session->transfer, sin);
 					iax_send_txaccept(session);
 				}
 				free(e);
