@@ -1,5 +1,5 @@
-/* $Id: resolver_test.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: resolver_test.c 4537 2013-06-19 06:47:43Z riza $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjlib-util.h>
 #include <pjlib.h>
@@ -111,7 +111,7 @@ static int print_name(pj_uint8_t *pkt, int size,
 	return 2;
     } else {
 	if (tab->count < MAX_LABEL) {
-	    tab->a[tab->count].pos = (p-pkt);
+	    tab->a[tab->count].pos = (unsigned)(p-pkt);
 	    tab->a[tab->count].label.ptr = (char*)(p+1);
 	    tab->a[tab->count].label.slen = name->slen;
 	    ++tab->count;
@@ -136,7 +136,7 @@ static int print_name(pj_uint8_t *pkt, int size,
 	*p = (pj_uint8_t)label.slen;
 	pj_memcpy(p+1, label.ptr, label.slen);
 
-	size -= (label.slen+1);
+	size -= (int)(label.slen+1);
 	p += (label.slen+1);
 
 	if (endlabel != endname && *endlabel == '.')
@@ -149,7 +149,7 @@ static int print_name(pj_uint8_t *pkt, int size,
 
     *p++ = '\0';
 
-    return p-pos;
+    return (int)(p-pos);
 }
 
 static int print_rr(pj_uint8_t *pkt, int size, pj_uint8_t *pos,
@@ -232,7 +232,7 @@ static int print_rr(pj_uint8_t *pkt, int size, pj_uint8_t *pos,
 	return -1;
     }
 
-    return p-pos;
+    return (int)(p-pos);
 }
 
 static int print_packet(const pj_dns_parsed_packet *rec, pj_uint8_t *pkt,
@@ -319,7 +319,7 @@ static int print_packet(const pj_dns_parsed_packet *rec, pj_uint8_t *pkt,
 	size -= len;
     }
 
-    return p - pkt;
+    return (int)(p - pkt);
 }
 
 
@@ -339,13 +339,13 @@ static int server_thread(void *p)
 	PJ_FD_ZERO(&rset);
 	PJ_FD_SET(srv->sock, &rset);
 
-	rc = pj_sock_select(srv->sock+1, &rset, NULL, NULL, &timeout);
+	rc = pj_sock_select((int)(srv->sock+1), &rset, NULL, NULL, &timeout);
 	if (rc != 1)
 	    continue;
 
 	src_len = sizeof(src_addr);
 	pkt_len = sizeof(pkt);
-	rc = pj_sock_recvfrom(srv->sock, pkt, &pkt_len, 0,
+	rc = pj_sock_recvfrom(srv->sock, pkt, &pkt_len, 0, 
 			      &src_addr, &src_len);
 	if (rc != 0) {
 	    app_perror("Server error receiving packet", rc);
@@ -355,7 +355,7 @@ static int server_thread(void *p)
 	PJ_LOG(5,(THIS_FILE, "Server %d processing packet", srv - &g_server[0]));
 	srv->pkt_count++;
 
-	rc = pj_dns_parse_packet(pool, pkt, pkt_len, &req);
+	rc = pj_dns_parse_packet(pool, pkt, (unsigned)pkt_len, &req);
 	if (rc != PJ_SUCCESS) {
 	    app_perror("server error parsing packet", rc);
 	    continue;
@@ -821,8 +821,8 @@ static int dns_test(void)
 			 set.good_ns_ttl));
     pj_thread_sleep(set.good_ns_ttl * 1000);
 
-    /*
-     * Fail-over test
+    /* 
+     * Fail-over test 
      */
     PJ_LOG(3,(THIS_FILE, "  failing server0"));
     g_server[0].action = ACTION_IGNORE;
@@ -865,7 +865,7 @@ static int dns_test(void)
 
     /* Wait to allow probing period to complete */
     PJ_LOG(3,(THIS_FILE, "  waiting for probing state to end (%d sec)",
-			 set.qretr_delay *
+			 set.qretr_delay * 
 			 (set.qretr_count+2) / 1000));
     pj_thread_sleep(set.qretr_delay * (set.qretr_count + 2));
 
@@ -965,7 +965,7 @@ static void action1_1(const pj_dns_parsed_packet *pkt,
 	res->q = PJ_POOL_ZALLOC_T(pool, pj_dns_parsed_query);
     }
     if (res->ans == NULL) {
-	res->ans = (pj_dns_parsed_rr*)
+	res->ans = (pj_dns_parsed_rr*) 
 		  pj_pool_calloc(pool, 4, sizeof(pj_dns_parsed_rr));
     }
 
@@ -1029,7 +1029,7 @@ static void srv_cb_1(void *user_data,
     PJ_ASSERT_ON_FAIL(rec->entry[0].server.addr[0].s_addr == IP_ADDR1, return);
     PJ_ASSERT_ON_FAIL(rec->entry[0].port == PORT1, return);
 
-
+    
 }
 
 static void srv_cb_1b(void *user_data,
@@ -1075,7 +1075,7 @@ static int srv_resolver_test(void)
 
     /* Wait until cache expires and nameserver state moves out from STATE_PROBING */
     PJ_LOG(3,(THIS_FILE, "  waiting for cache to expire (~15 secs).."));
-    pj_thread_sleep(1000 +
+    pj_thread_sleep(1000 + 
 		    ((set.qretr_count + 2) * set.qretr_delay));
 
     /* Successful scenario */
@@ -1132,7 +1132,7 @@ static void action2_1(const pj_dns_parsed_packet *pkt,
     res = PJ_POOL_ZALLOC_T(pool, pj_dns_parsed_packet);
 
     res->q = PJ_POOL_ZALLOC_T(pool, pj_dns_parsed_query);
-    res->ans = (pj_dns_parsed_rr*)
+    res->ans = (pj_dns_parsed_rr*) 
 	       pj_pool_calloc(pool, 4, sizeof(pj_dns_parsed_rr));
 
     res->hdr.qdcount = 1;
@@ -1269,7 +1269,7 @@ static void action3_1(const pj_dns_parsed_packet *pkt,
 	pj_assert(pj_strcmp2(&pkt->q[0].name, "_sip._udp." DOMAIN3)==0);
 
 	res->hdr.anscount = SRV_COUNT3;
-	res->ans = (pj_dns_parsed_rr*)
+	res->ans = (pj_dns_parsed_rr*) 
 		   pj_pool_calloc(pool, SRV_COUNT3, sizeof(pj_dns_parsed_rr));
 
 	for (i=0; i<SRV_COUNT3; ++i) {
@@ -1293,7 +1293,7 @@ static void action3_1(const pj_dns_parsed_packet *pkt,
 	//pj_assert(pj_strcmp2(&res->q[0].name, "sip." DOMAIN3)==0);
 
 	res->hdr.anscount = A_COUNT3;
-	res->ans = (pj_dns_parsed_rr*)
+	res->ans = (pj_dns_parsed_rr*) 
 		   pj_pool_calloc(pool, A_COUNT3, sizeof(pj_dns_parsed_rr));
 
 	for (i=0; i<A_COUNT3; ++i) {
@@ -1373,7 +1373,7 @@ static int srv_resolver_many_test(void)
 int resolver_test(void)
 {
     int rc;
-
+    
     rc = init();
     if (rc != 0)
 	goto on_error;
@@ -1401,3 +1401,5 @@ on_error:
     destroy();
     return rc;
 }
+
+
