@@ -1,5 +1,5 @@
-/* $Id: dns.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: dns.c 4712 2014-01-23 08:09:29Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjlib-util/dns.h>
 #include <pjlib-util/errno.h>
@@ -59,9 +59,9 @@ PJ_DEF(pj_status_t) pj_dns_make_query( void *packet,
 				       int qtype,
 				       const pj_str_t *name)
 {
-    pj_uint8_t *query, *p = (pj_uint8_t*)packet;
+    pj_uint8_t *p = (pj_uint8_t*)packet;
     const char *startlabel, *endlabel, *endname;
-    unsigned d;
+    pj_size_t d;
 
     /* Sanity check */
     PJ_ASSERT_RETURN(packet && size && qtype && name, PJ_EINVAL);
@@ -80,7 +80,7 @@ PJ_DEF(pj_status_t) pj_dns_make_query( void *packet,
     write16(p+4, (pj_uint16_t)1);
 
     /* Initialize query */
-    query = p = ((pj_uint8_t*)packet)+sizeof(pj_dns_hdr);
+    p = ((pj_uint8_t*)packet)+sizeof(pj_dns_hdr);
 
     /* Tokenize name */
     startlabel = endlabel = name->ptr;
@@ -106,17 +106,17 @@ PJ_DEF(pj_status_t) pj_dns_make_query( void *packet,
     p += 2;
 
     /* Done, calculate length */
-    *size = p - (pj_uint8_t*)packet;
+    *size = (unsigned)(p - (pj_uint8_t*)packet);
 
     return 0;
 }
 
 
 /* Get a name length (note: name consists of multiple labels and
- * it may contain pointers when name compression is applied)
+ * it may contain pointers when name compression is applied) 
  */
-static pj_status_t get_name_len(int rec_counter, const pj_uint8_t *pkt,
-				const pj_uint8_t *start, const pj_uint8_t *max,
+static pj_status_t get_name_len(int rec_counter, const pj_uint8_t *pkt, 
+				const pj_uint8_t *start, const pj_uint8_t *max, 
 				int *parsed_len, int *name_len)
 {
     const pj_uint8_t *p;
@@ -147,7 +147,7 @@ static pj_status_t get_name_len(int rec_counter, const pj_uint8_t *pkt,
 		return PJLIB_UTIL_EDNSINNAMEPTR;
 
 	    /* Get the name length from that offset. */
-	    status = get_name_len(rec_counter+1, pkt, pkt + offset, max,
+	    status = get_name_len(rec_counter+1, pkt, pkt + offset, max, 
 				  &dummy, &ptr_len);
 	    if (status != PJ_SUCCESS)
 		return status;
@@ -168,7 +168,7 @@ static pj_status_t get_name_len(int rec_counter, const pj_uint8_t *pkt,
 
 	    if (*p != 0)
 		++label_len;
-
+	    
 	    *name_len += label_len;
 
 	    if (p >= max)
@@ -185,7 +185,7 @@ static pj_status_t get_name_len(int rec_counter, const pj_uint8_t *pkt,
 /* Parse and copy name (note: name consists of multiple labels and
  * it may contain pointers when compression is applied).
  */
-static pj_status_t get_name(int rec_counter, const pj_uint8_t *pkt,
+static pj_status_t get_name(int rec_counter, const pj_uint8_t *pkt, 
 			    const pj_uint8_t *start, const pj_uint8_t *max,
 			    pj_str_t *name)
 {
@@ -319,7 +319,7 @@ static pj_status_t parse_rr(pj_dns_parsed_rr *rr, pj_pool_t *pool,
     pj_memcpy(&rr->type, p, 2);
     rr->type = pj_ntohs(rr->type);
     p += 2;
-
+    
     /* Get the class */
     pj_memcpy(&rr->dnsclass, p, 2);
     rr->dnsclass = pj_ntohs(rr->dnsclass);
@@ -354,7 +354,7 @@ static pj_status_t parse_rr(pj_dns_parsed_rr *rr, pj_pool_t *pool,
 
     } else if (rr->type == PJ_DNS_TYPE_CNAME ||
 	       rr->type == PJ_DNS_TYPE_NS ||
-	       rr->type == PJ_DNS_TYPE_PTR)
+	       rr->type == PJ_DNS_TYPE_PTR) 
     {
 
 	/* Get the length of the target name */
@@ -389,7 +389,7 @@ static pj_status_t parse_rr(pj_dns_parsed_rr *rr, pj_pool_t *pool,
 	pj_memcpy(&rr->rdata.srv.port, p, 2);
 	rr->rdata.srv.port = pj_ntohs(rr->rdata.srv.port);
 	p += 2;
-
+	
 	/* Get the length of the target name */
 	status = get_name_len(0, pkt, p, max, &name_part_len, &name_len);
 	if (status != PJ_SUCCESS)
@@ -422,7 +422,7 @@ static pj_status_t parse_rr(pj_dns_parsed_rr *rr, pj_pool_t *pool,
  * Parse raw DNS packet into DNS packet structure.
  */
 PJ_DEF(pj_status_t) pj_dns_parse_packet( pj_pool_t *pool,
-					 const void *packet,
+				  	 const void *packet,
 					 unsigned size,
 					 pj_dns_parsed_packet **p_res)
 {
@@ -462,9 +462,9 @@ PJ_DEF(pj_status_t) pj_dns_parse_packet( pj_pool_t *pool,
 				      sizeof(pj_dns_parsed_query));
 	for (i=0; i<res->hdr.qdcount; ++i) {
 	    int parsed_len = 0;
-
-	    status = parse_query(&res->q[i], pool, (const pj_uint8_t*)packet,
-				 start, end, &parsed_len);
+	    
+	    status = parse_query(&res->q[i], pool, (const pj_uint8_t*)packet, 
+	    			 start, end, &parsed_len);
 	    if (status != PJ_SUCCESS)
 		return status;
 
@@ -475,14 +475,14 @@ PJ_DEF(pj_status_t) pj_dns_parse_packet( pj_pool_t *pool,
     /* Parse answer, if any */
     if (res->hdr.anscount) {
 	res->ans = (pj_dns_parsed_rr*)
-		   pj_pool_zalloc(pool, res->hdr.anscount *
+		   pj_pool_zalloc(pool, res->hdr.anscount * 
 					sizeof(pj_dns_parsed_rr));
 
 	for (i=0; i<res->hdr.anscount; ++i) {
 	    int parsed_len;
 
-	    status = parse_rr(&res->ans[i], pool, (const pj_uint8_t*)packet,
-			      start, end, &parsed_len);
+	    status = parse_rr(&res->ans[i], pool, (const pj_uint8_t*)packet, 
+	    		      start, end, &parsed_len);
 	    if (status != PJ_SUCCESS)
 		return status;
 
@@ -499,8 +499,8 @@ PJ_DEF(pj_status_t) pj_dns_parse_packet( pj_pool_t *pool,
 	for (i=0; i<res->hdr.nscount; ++i) {
 	    int parsed_len;
 
-	    status = parse_rr(&res->ns[i], pool, (const pj_uint8_t*)packet,
-			      start, end, &parsed_len);
+	    status = parse_rr(&res->ns[i], pool, (const pj_uint8_t*)packet, 
+	    		      start, end, &parsed_len);
 	    if (status != PJ_SUCCESS)
 		return status;
 
@@ -517,8 +517,8 @@ PJ_DEF(pj_status_t) pj_dns_parse_packet( pj_pool_t *pool,
 	for (i=0; i<res->hdr.arcount; ++i) {
 	    int parsed_len;
 
-	    status = parse_rr(&res->arr[i], pool, (const pj_uint8_t*)packet,
-			      start, end, &parsed_len);
+	    status = parse_rr(&res->arr[i], pool, (const pj_uint8_t*)packet, 
+	    		      start, end, &parsed_len);
 	    if (status != PJ_SUCCESS)
 		return status;
 
@@ -539,7 +539,7 @@ PJ_DEF(pj_status_t) pj_dns_parse_packet( pj_pool_t *pool,
  */
 static void apply_name_table( unsigned *count,
 			      pj_str_t nametable[],
-			      const pj_str_t *src,
+		    	      const pj_str_t *src,
 			      pj_pool_t *pool,
 			      pj_str_t *dst)
 {
@@ -593,7 +593,7 @@ static void copy_rr(pj_pool_t *pool, pj_dns_parsed_rr *dst,
     }
 
     if (src->type == PJ_DNS_TYPE_SRV) {
-	apply_name_table(nametable_count, nametable, &src->rdata.srv.target,
+	apply_name_table(nametable_count, nametable, &src->rdata.srv.target, 
 			 pool, &dst->rdata.srv.target);
     } else if (src->type == PJ_DNS_TYPE_A) {
 	dst->rdata.a.ip_addr.s_addr =  src->rdata.a.ip_addr.s_addr;
@@ -640,15 +640,15 @@ PJ_DEF(void) pj_dns_packet_dup(pj_pool_t *pool,
     dst->hdr.anscount = 0;
     dst->hdr.nscount = 0;
     dst->hdr.arcount = 0;
-
+	
 
     /* Copy query section */
     if (p->hdr.qdcount && (options & PJ_DNS_NO_QD)==0) {
 	dst->q = (pj_dns_parsed_query*)
-		 pj_pool_alloc(pool, p->hdr.qdcount *
+		 pj_pool_alloc(pool, p->hdr.qdcount * 
 				     sizeof(pj_dns_parsed_query));
 	for (i=0; i<p->hdr.qdcount; ++i) {
-	    copy_query(pool, &dst->q[i], &p->q[i],
+	    copy_query(pool, &dst->q[i], &p->q[i], 
 		       &nametable_count, nametable);
 	    ++dst->hdr.qdcount;
 	}
@@ -657,7 +657,7 @@ PJ_DEF(void) pj_dns_packet_dup(pj_pool_t *pool,
     /* Copy answer section */
     if (p->hdr.anscount && (options & PJ_DNS_NO_ANS)==0) {
 	dst->ans = (pj_dns_parsed_rr*)
-		   pj_pool_alloc(pool, p->hdr.anscount *
+		   pj_pool_alloc(pool, p->hdr.anscount * 
 				       sizeof(pj_dns_parsed_rr));
 	for (i=0; i<p->hdr.anscount; ++i) {
 	    copy_rr(pool, &dst->ans[i], &p->ans[i],
@@ -669,7 +669,7 @@ PJ_DEF(void) pj_dns_packet_dup(pj_pool_t *pool,
     /* Copy NS section */
     if (p->hdr.nscount && (options & PJ_DNS_NO_NS)==0) {
 	dst->ns = (pj_dns_parsed_rr*)
-		  pj_pool_alloc(pool, p->hdr.nscount *
+		  pj_pool_alloc(pool, p->hdr.nscount * 
 				      sizeof(pj_dns_parsed_rr));
 	for (i=0; i<p->hdr.nscount; ++i) {
 	    copy_rr(pool, &dst->ns[i], &p->ns[i],
@@ -681,7 +681,7 @@ PJ_DEF(void) pj_dns_packet_dup(pj_pool_t *pool,
     /* Copy additional info section */
     if (p->hdr.arcount && (options & PJ_DNS_NO_AR)==0) {
 	dst->arr = (pj_dns_parsed_rr*)
-		   pj_pool_alloc(pool, p->hdr.arcount *
+		   pj_pool_alloc(pool, p->hdr.arcount * 
 				       sizeof(pj_dns_parsed_rr));
 	for (i=0; i<p->hdr.arcount; ++i) {
 	    copy_rr(pool, &dst->arr[i], &p->arr[i],
@@ -741,3 +741,4 @@ PJ_DEF(void) pj_dns_init_a_rr( pj_dns_parsed_rr *rec,
     rec->ttl = ttl;
     rec->rdata.a.ip_addr = *ip_addr;
 }
+
