@@ -1,4 +1,4 @@
-/* $Id: resolver.c 4333 2013-01-23 09:53:39Z nanang $ */
+/* $Id: resolver.c 4649 2013-11-07 08:20:30Z nanang $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -157,6 +157,7 @@ struct cached_res
     pj_hash_entry_buf	     hbuf;	    /**< Hash buffer		    */
     pj_time_val		     expiry_time;   /**< Expiration time.	    */
     pj_dns_parsed_packet    *pkt;	    /**< The response packet.	    */
+    unsigned		     ref_cnt;	    /**< Reference counter.	    */
 };
 
 
@@ -672,7 +673,8 @@ static pj_status_t transmit_query(pj_dns_resolver *resolver,
  */
 static void init_res_key(struct res_key *key, int type, const pj_str_t *name)
 {
-    unsigned i, len;
+    unsigned i;
+    pj_size_t len;
     char *dst = key->name;
     const char *src = name->ptr;
 
@@ -699,10 +701,29 @@ static struct cached_res *alloc_entry(pj_dns_resolver *resolver)
 			  RES_BUF_SZ, 256, NULL);
     cache = PJ_POOL_ZALLOC_T(pool, struct cached_res);
     cache->pool = pool;
+    cache->ref_cnt = 1;
 
     return cache;
 }
 
+/* Re-allocate cache entry, to free cached packet */
+static void reset_entry(struct cached_res **p_cached)
+{
+    pj_pool_t *pool;
+    struct cached_res *cache = *p_cached;
+    unsigned ref_cnt;
+
+    pool = cache->pool;
+    ref_cnt = cache->ref_cnt;
+
+    pj_pool_reset(pool);
+
+    cache = PJ_POOL_ZALLOC_T(pool, struct cached_res);
+    cache->pool = pool;
+    cache->ref_cnt = ref_cnt;
+    *p_cached = cache;
+}
+
 /* Put unused/expired cached entry to the free list */
 static void free_entry(pj_dns_resolver *resolver, struct cached_res *cache)
 {
@@ -774,6 +795,13 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
 	    status = PJ_DNS_GET_RCODE(cache->pkt->hdr.flags);
 	    status = PJ_STATUS_FROM_DNS_RCODE(status);
 
+	    /* Workaround for deadlock problem. Need to increment the cache's
+	     * ref counter first before releasing mutex, so the cache won't be
+	     * destroyed by other thread while in callback.
+	     */
+	    cache->ref_cnt++;
+	    pj_mutex_unlock(resolver->mutex);
+
 	    /* This cached response is still valid. Just return this
 	     * response to caller.
 	     */
@@ -782,6 +810,14 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
 	    }
 
 	    /* Done. No host resolution is necessary */
+	    pj_mutex_lock(resolver->mutex);
+
+	    /* Decrement the ref counter. Also check if it is time to free
+	     * the cache (as it has been expired).
+	     */
+	    cache->ref_cnt--;
+	    if (cache->ref_cnt <= 0)
+		free_entry(resolver, cache);
 
 	    /* Must return PJ_SUCCESS */
 	    status = PJ_SUCCESS;
@@ -794,8 +830,10 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
 	 */
 	pj_hash_set(NULL, resolver->hrescache, &key, sizeof(key), 0, NULL);
 
-	/* Store the entry into free nodes */
-	free_entry(resolver, cache);
+	/* Also free the cache, if it is not being used (by callback). */
+	cache->ref_cnt--;
+	if (cache->ref_cnt <= 0)
+	    free_entry(resolver, cache);
 
 	/* Must continue with creating a query now */
     }
@@ -883,8 +921,8 @@ PJ_DEF(pj_status_t) pj_dns_parse_a_response(const pj_dns_parsed_packet *pkt,
 {
     enum { MAX_SEARCH = 20 };
     pj_str_t hostname, alias = {NULL, 0}, *resname;
-    unsigned bufstart = 0;
-    unsigned bufleft = sizeof(rec->buf_);
+    pj_size_t bufstart = 0;
+    pj_size_t bufleft = sizeof(rec->buf_);
     unsigned i, ansidx, search_cnt=0;
 
     PJ_ASSERT_RETURN(pkt && rec, PJ_EINVAL);
@@ -1159,9 +1197,12 @@ static void update_res_cache(pj_dns_resolver *resolver,
     if (status != PJ_SUCCESS) {
 	cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key, 
 						  sizeof(*key), &hval);
-	if (cache)
-	    free_entry(resolver, cache);
+	/* Remove the entry before releasing its pool (see ticket #1710) */
 	pj_hash_set(NULL, resolver->hrescache, key, sizeof(*key), hval, NULL);
+	
+	/* Free the entry */
+	if (cache && --cache->ref_cnt <= 0)
+	    free_entry(resolver, cache);
     }
 
 
@@ -1195,9 +1236,12 @@ static void update_res_cache(pj_dns_resolver *resolver,
     if (ttl == 0) {
 	cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key, 
 						  sizeof(*key), &hval);
-	if (cache)
-	    free_entry(resolver, cache);
+	/* Remove the entry before releasing its pool (see ticket #1710) */
 	pj_hash_set(NULL, resolver->hrescache, key, sizeof(*key), hval, NULL);
+
+	/* Free the entry */
+	if (cache && --cache->ref_cnt <= 0)
+	    free_entry(resolver, cache);
 	return;
     }
 
@@ -1206,6 +1250,19 @@ static void update_res_cache(pj_dns_resolver *resolver,
     					      sizeof(*key), &hval);
     if (cache == NULL) {
 	cache = alloc_entry(resolver);
+    } else if (cache->ref_cnt > 1) {
+	/* When cache entry is being used by callback (to app), just decrement
+	 * ref_cnt so it will be freed after the callback returns and allocate
+	 * new entry.
+	 */
+	cache->ref_cnt--;
+	cache = alloc_entry(resolver);
+    } else {
+	/* Remove the entry before resetting its pool (see ticket #1710) */
+	pj_hash_set(NULL, resolver->hrescache, key, sizeof(*key), hval, NULL);
+
+	/* Reset cache to avoid bloated cache pool */
+	reset_entry(&cache);
     }
 
     /* Duplicate the packet.
@@ -1214,7 +1271,6 @@ static void update_res_cache(pj_dns_resolver *resolver,
      * section since DNS A parser needs the query section to know
      * the name being requested.
      */
-    cache->pkt = NULL;
     pj_dns_packet_dup(cache->pool, pkt, 
 		      PJ_DNS_NO_NS | PJ_DNS_NO_AR,
 		      &cache->pkt);
@@ -1346,7 +1402,7 @@ static void on_read_complete(pj_ioqueue_key_t *key,
     if (bytes_read < 0) {
 	char errmsg[PJ_ERR_MSG_SIZE];
 
-	status = -bytes_read;
+	status = (pj_status_t)-bytes_read;
 	pj_strerror(status, errmsg, sizeof(errmsg));
 	PJ_LOG(4,(resolver->name.ptr, 
 		  "DNS resolver read error from %s:%d: %s", 
