@@ -1,4 +1,4 @@
-/* $Id: srv_resolver.c 3553 2011-05-05 06:14:19Z nanang $ */
+/* $Id: srv_resolver.c 4774 2014-02-28 10:14:24Z bennylp $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -102,7 +102,7 @@ PJ_DEF(pj_status_t) pj_dns_srv_resolve( const pj_str_t *domain_name,
 					pj_dns_srv_resolver_cb *cb,
 					pj_dns_srv_async_query **p_query)
 {
-    int len;
+    pj_size_t len;
     pj_str_t target_name;
     pj_dns_srv_async_query *query_job;
     pj_status_t status;
@@ -254,10 +254,7 @@ static void build_server_entries(pj_dns_srv_async_query *query_job,
     }
 
     /* Second pass:
-     *	pick one host among hosts with the same priority, according
-     *	to its weight. The idea is when one server fails, client should
-     *	contact the next server with higher priority rather than contacting
-     *	server with the same priority as the failed one.
+     *	Order the entry in a list.
      *
      *  The algorithm for selecting server among servers with the same
      *  priority is described in RFC 2782.
@@ -298,12 +295,15 @@ static void build_server_entries(pj_dns_srv_async_query *query_job,
 	    SWAP(struct srv_target, &query_job->srv[i], &query_job->srv[j]);
 
 	    /* Remove all other entries (of the same priority) */
+	    /* Don't need to do this.
+	     * See https://trac.pjsip.org/repos/ticket/1719
 	    while (count > 1) {
 		pj_array_erase(query_job->srv, sizeof(struct srv_target), 
 			       query_job->srv_cnt, i+1);
 		--count;
 		--query_job->srv_cnt;
 	    }
+	    */
 	}
     }
 
