@@ -1,5 +1,5 @@
-/* $Id: ssl_sock.h 4146 2012-05-30 06:35:59Z nanang $ */
-/*
+/* $Id: ssl_sock.h 4506 2013-04-26 06:01:43Z bennylp $ */
+/* 
  * Copyright (C) 2009-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJ_SSL_SOCK_H__
 #define __PJ_SSL_SOCK_H__
@@ -38,7 +38,7 @@ PJ_BEGIN_DECL
  * @ingroup PJ_IO
  * @{
  *
- * Secure socket wraps normal socket and applies security features, i.e:
+ * Secure socket wraps normal socket and applies security features, i.e: 
  * privacy and data integrity, on the socket traffic, using standard security
  * protocols such as SSL and TLS.
  *
@@ -94,7 +94,7 @@ typedef enum pj_ssl_cert_verify_flag_t
     PJ_SSL_CERT_EINVALID_PURPOSE			= (1 << 4),
 
     /**
-     * The issuer info in the certificate does not match to the (candidate)
+     * The issuer info in the certificate does not match to the (candidate) 
      * issuer certificate, e.g: issuer name not match to subject name
      * of (candidate) issuer certificate.
      */
@@ -116,9 +116,9 @@ typedef enum pj_ssl_cert_verify_flag_t
     PJ_SSL_CERT_ECHAIN_TOO_LONG				= (1 << 8),
 
     /**
-     * The server identity does not match to any identities specified in
+     * The server identity does not match to any identities specified in 
      * the certificate, e.g: subjectAltName extension, subject common name.
-     * This flag will only be set by application as SSL socket does not
+     * This flag will only be set by application as SSL socket does not 
      * perform server identity verification.
      */
     PJ_SSL_CERT_EIDENTITY_NOT_MATCH			= (1 << 30),
@@ -167,7 +167,7 @@ typedef struct pj_ssl_cert_info {
     struct {
 	pj_time_val	start;	    /**< Validity start		*/
 	pj_time_val	end;	    /**< Validity end		*/
-	pj_bool_t	gmt;	    /**< Flag if validity date/time
+	pj_bool_t	gmt;	    /**< Flag if validity date/time 
 					 use GMT		*/
     } validity;			    /**< Validity		*/
 
@@ -224,7 +224,7 @@ PJ_DECL(pj_ssize_t) pj_ssl_cert_info_dump(const pj_ssl_cert_info *ci,
  * Get SSL certificate verification error messages from verification status.
  *
  * @param verify_status	The SSL certificate verification status.
- * @param error_strings	Array of strings to receive the verification error
+ * @param error_strings	Array of strings to receive the verification error 
  *			messages.
  * @param count		On input it specifies maximum error messages should be
  *			retrieved. On output it specifies the number of error
@@ -233,12 +233,12 @@ PJ_DECL(pj_ssize_t) pj_ssl_cert_info_dump(const pj_ssl_cert_info *ci,
  * @return		PJ_SUCCESS when successful.
  */
 PJ_DECL(pj_status_t) pj_ssl_cert_get_verify_status_strings(
-						 pj_uint32_t verify_status,
+						 pj_uint32_t verify_status, 
 						 const char *error_strings[],
 						 unsigned *count);
 
 
-/**
+/** 
  * Cipher suites enumeration.
  */
 typedef enum pj_ssl_cipher {
@@ -306,7 +306,7 @@ typedef enum pj_ssl_cipher {
     PJ_SSL_FORTEZZA_KEA_WITH_NULL_SHA        	= 0x0000001C,
     PJ_SSL_FORTEZZA_KEA_WITH_FORTEZZA_CBC_SHA	= 0x0000001D,
     PJ_SSL_FORTEZZA_KEA_WITH_RC4_128_SHA     	= 0x0000001E,
-
+    
     /* SSLv2 */
     PJ_SSL_CK_RC4_128_WITH_MD5               	= 0x00010080,
     PJ_SSL_CK_RC4_128_EXPORT40_WITH_MD5      	= 0x00020080,
@@ -373,8 +373,8 @@ typedef struct pj_ssl_sock_cb
      * pj_ssl_sock_start_read().
      *
      * @param ssock	The secure socket.
-     * @param data	The buffer containing the new data, if any. If
-     *			the status argument is non-PJ_SUCCESS, this
+     * @param data	The buffer containing the new data, if any. If 
+     *			the status argument is non-PJ_SUCCESS, this 
      *			argument may be NULL.
      * @param size	The length of data in the buffer.
      * @param status	The status of the read operation. This may contain
@@ -382,14 +382,14 @@ typedef struct pj_ssl_sock_cb
      *			has been closed. In this case, the buffer may
      *			contain left over data from previous callback which
      *			the application may want to process.
-     * @param remainder	If application wishes to leave some data in the
-     *			buffer (common for TCP applications), it should
-     *			move the remainder data to the front part of the
+     * @param remainder	If application wishes to leave some data in the 
+     *			buffer (common for TCP applications), it should 
+     *			move the remainder data to the front part of the 
      *			buffer and set the remainder length here. The value
      *			of this parameter will be ignored for datagram
      *			sockets.
      *
-     * @return		PJ_TRUE if further read is desired, and PJ_FALSE
+     * @return		PJ_TRUE if further read is desired, and PJ_FALSE 
      *			when application no longer wants to receive data.
      *			Application may destroy the secure socket in the
      *			callback and return PJ_FALSE here.
@@ -404,17 +404,17 @@ typedef struct pj_ssl_sock_cb
      * pj_ssl_sock_start_recvfrom().
      *
      * @param ssock	The secure socket.
-     * @param data	The buffer containing the packet, if any. If
-     *			the status argument is non-PJ_SUCCESS, this
+     * @param data	The buffer containing the packet, if any. If 
+     *			the status argument is non-PJ_SUCCESS, this 
      *			argument will be set to NULL.
-     * @param size	The length of packet in the buffer. If
-     *			the status argument is non-PJ_SUCCESS, this
+     * @param size	The length of packet in the buffer. If 
+     *			the status argument is non-PJ_SUCCESS, this 
      *			argument will be set to zero.
      * @param src_addr	Source address of the packet.
      * @param addr_len	Length of the source address.
      * @param status	This contains
      *
-     * @return		PJ_TRUE if further read is desired, and PJ_FALSE
+     * @return		PJ_TRUE if further read is desired, and PJ_FALSE 
      *			when application no longer wants to receive data.
      *			Application may destroy the secure socket in the
      *			callback and return PJ_FALSE here.
@@ -472,7 +472,7 @@ typedef struct pj_ssl_sock_cb
      *			PJ_SUCCESS.
      *
      * @return		Application may destroy the secure socket in the
-     *			callback and return PJ_FALSE here.
+     *			callback and return PJ_FALSE here. 
      */
     pj_bool_t (*on_connect_complete)(pj_ssl_sock_t *ssock,
 				     pj_status_t status);
@@ -480,7 +480,7 @@ typedef struct pj_ssl_sock_cb
 } pj_ssl_sock_cb;
 
 
-/**
+/** 
  * Enumeration of secure socket protocol types.
  */
 typedef enum pj_ssl_sock_proto
@@ -488,7 +488,7 @@ typedef enum pj_ssl_sock_proto
     PJ_SSL_SOCK_PROTO_DEFAULT,	    /**< Default protocol of backend.	*/
     PJ_SSL_SOCK_PROTO_TLS1,	    /**< TLSv1.0 protocol.		*/
     PJ_SSL_SOCK_PROTO_SSL3,	    /**< SSLv3.0 protocol.		*/
-    PJ_SSL_SOCK_PROTO_SSL23,	    /**< SSLv3.0 but can roll back to
+    PJ_SSL_SOCK_PROTO_SSL23,	    /**< SSLv3.0 but can roll back to 
 					 SSLv2.0.			*/
     PJ_SSL_SOCK_PROTO_SSL2,	    /**< SSLv2.0 protocol.		*/
     PJ_SSL_SOCK_PROTO_DTLS1	    /**< DTLSv1.0 protocol.		*/
@@ -498,10 +498,10 @@ typedef enum pj_ssl_sock_proto
 /**
  * Definition of secure socket info structure.
  */
-typedef struct pj_ssl_sock_info
+typedef struct pj_ssl_sock_info 
 {
     /**
-     * Describes whether secure socket connection is established, i.e: TLS/SSL
+     * Describes whether secure socket connection is established, i.e: TLS/SSL 
      * handshaking has been done successfully.
      */
     pj_bool_t established;
@@ -526,12 +526,12 @@ typedef struct pj_ssl_sock_info
      * Describes remote address.
      */
     pj_sockaddr remote_addr;
-
+   
     /**
      * Describes active local certificate info.
      */
     pj_ssl_cert_info *local_cert_info;
-
+   
     /**
      * Describes active remote certificate info.
      */
@@ -577,7 +577,7 @@ typedef struct pj_ssl_sock_param
 
     /**
      * Specify the timer heap to use. Secure socket uses the timer to provide
-     * auto cancelation on asynchronous operation when it takes longer time
+     * auto cancelation on asynchronous operation when it takes longer time 
      * than specified timeout period, e.g: security negotiation timeout.
      */
     pj_timer_heap_t *timer_heap;
@@ -602,7 +602,7 @@ typedef struct pj_ssl_sock_param
     /**
      * Number of concurrent asynchronous operations that is to be supported
      * by the secure socket. This value only affects socket receive and
-     * accept operations -- the secure socket will issue one or more
+     * accept operations -- the secure socket will issue one or more 
      * asynchronous read and accept operations based on the value of this
      * field. Setting this field to more than one will allow more than one
      * incoming data or incoming connections to be processed simultaneously
@@ -614,12 +614,12 @@ typedef struct pj_ssl_sock_param
     unsigned async_cnt;
 
     /**
-     * The ioqueue concurrency to be forced on the socket when it is
+     * The ioqueue concurrency to be forced on the socket when it is 
      * registered to the ioqueue. See #pj_ioqueue_set_concurrency() for more
      * info about ioqueue concurrency.
      *
      * When this value is -1, the concurrency setting will not be forced for
-     * this socket, and the socket will inherit the concurrency setting of
+     * this socket, and the socket will inherit the concurrency setting of 
      * the ioqueue. When this value is zero, the secure socket will disable
      * concurrency for the socket. When this value is +1, the secure socket
      * will enable concurrency for the socket.
@@ -645,18 +645,18 @@ typedef struct pj_ssl_sock_param
 
     /**
      * Specify buffer size for sending operation. Buffering sending data
-     * is used for allowing application to perform multiple outstanding
+     * is used for allowing application to perform multiple outstanding 
      * send operations. Whenever application specifies this setting too
      * small, sending operation may return PJ_ENOMEM.
-     *
+     *  
      * Default value is 8192 bytes.
      */
     pj_size_t send_buffer_size;
 
     /**
      * Specify buffer size for receiving encrypted (and perhaps compressed)
-     * data on underlying socket. This setting is unused on Symbian, since
-     * SSL/TLS Symbian backend, CSecureSocket, can use application buffer
+     * data on underlying socket. This setting is unused on Symbian, since 
+     * SSL/TLS Symbian backend, CSecureSocket, can use application buffer 
      * directly.
      *
      * Default value is 1500.
@@ -664,8 +664,8 @@ typedef struct pj_ssl_sock_param
     pj_size_t read_buffer_size;
 
     /**
-     * Number of ciphers contained in the specified cipher preference.
-     * If this is set to zero, then default cipher list of the backend
+     * Number of ciphers contained in the specified cipher preference. 
+     * If this is set to zero, then default cipher list of the backend 
      * will be used.
      */
     unsigned ciphers_num;
@@ -677,7 +677,7 @@ typedef struct pj_ssl_sock_param
     pj_ssl_cipher *ciphers;
 
     /**
-     * Security negotiation timeout. If this is set to zero (both sec and
+     * Security negotiation timeout. If this is set to zero (both sec and 
      * msec), the negotiation doesn't have a timeout.
      *
      * Default value is zero.
@@ -690,7 +690,7 @@ typedef struct pj_ssl_sock_param
      * Default value is PJ_FALSE.
      */
     pj_bool_t verify_peer;
-
+    
     /**
      * When secure socket is acting as server (handles incoming connection),
      * it will require the client to provide certificate.
@@ -700,7 +700,7 @@ typedef struct pj_ssl_sock_param
     pj_bool_t require_client_cert;
 
     /**
-     * Server name indication. When secure socket is acting as client
+     * Server name indication. When secure socket is acting as client 
      * (perform outgoing connection) and the server may host multiple
      * 'virtual' servers at a single underlying network address, setting
      * this will allow client to tell the server a name of the server
@@ -710,6 +710,14 @@ typedef struct pj_ssl_sock_param
      */
     pj_str_t server_name;
 
+    /**
+     * Specify if SO_REUSEADDR should be used for listening socket. This
+     * option will only be used with accept() operation.
+     *
+     * Default is PJ_FALSE.
+     */
+    pj_bool_t reuse_addr;
+
     /**
      * QoS traffic type to be set on this transport. When application wants
      * to apply QoS tagging to the transport, it's preferable to set this
@@ -741,7 +749,7 @@ typedef struct pj_ssl_sock_param
 
 
 /**
- * Initialize the secure socket parameters for its creation with
+ * Initialize the secure socket parameters for its creation with 
  * the default values.
  *
  * @param param		The parameter to be initialized.
@@ -764,8 +772,8 @@ PJ_DECL(pj_status_t) pj_ssl_sock_create(pj_pool_t *pool,
 
 
 /**
- * Set secure socket certificate or credentials. Credentials may include
- * certificate, private key and trusted Certification Authorities list.
+ * Set secure socket certificate or credentials. Credentials may include 
+ * certificate, private key and trusted Certification Authorities list. 
  * Normally, server socket must provide certificate (and private key).
  * Socket client may also need to provide certificate in case requested
  * by the server.
@@ -837,15 +845,15 @@ PJ_DECL(pj_status_t) pj_ssl_sock_get_info(pj_ssl_sock_t *ssock,
  * Starts read operation on this secure socket. This function will create
  * \a async_cnt number of buffers (the \a async_cnt parameter was given
  * in \a pj_ssl_sock_create() function) where each buffer is \a buff_size
- * long. The buffers are allocated from the specified \a pool. Once the
+ * long. The buffers are allocated from the specified \a pool. Once the 
  * buffers are created, it then issues \a async_cnt number of asynchronous
  * \a recv() operations to the socket and returns back to caller. Incoming
- * data on the socket will be reported back to application via the
+ * data on the socket will be reported back to application via the 
  * \a on_data_read() callback.
  *
  * Application only needs to call this function once to initiate read
  * operations. Further read operations will be done automatically by the
- * secure socket when \a on_data_read() callback returns non-zero.
+ * secure socket when \a on_data_read() callback returns non-zero. 
  *
  * @param ssock		The secure socket.
  * @param pool		Pool used to allocate buffers for incoming data.
@@ -899,7 +907,7 @@ PJ_DECL(pj_status_t) pj_ssl_sock_start_recvfrom(pj_ssl_sock_t *ssock,
 						pj_uint32_t flags);
 
 /**
- * Same as #pj_ssl_sock_start_recvfrom() except that the recvfrom()
+ * Same as #pj_ssl_sock_start_recvfrom() except that the recvfrom() 
  * operation takes the buffer from the argument rather than creating
  * new ones.
  *
@@ -924,7 +932,7 @@ PJ_DECL(pj_status_t) pj_ssl_sock_start_recvfrom2(pj_ssl_sock_t *ssock,
  * @param ssock		The secure socket.
  * @param send_key	The operation key to send the data, which is useful
  *			if application wants to submit multiple pending
- *			send operations and want to track which exact data
+ *			send operations and want to track which exact data 
  *			has been sent in the \a on_data_sent() callback.
  * @param data		The data to be sent. This data must remain valid
  *			until the data has been sent.
@@ -950,7 +958,7 @@ PJ_DECL(pj_status_t) pj_ssl_sock_send(pj_ssl_sock_t *ssock,
  * @param ssock		The secure socket.
  * @param send_key	The operation key to send the data, which is useful
  *			if application wants to submit multiple pending
- *			send operations and want to track which exact data
+ *			send operations and want to track which exact data 
  *			has been sent in the \a on_data_sent() callback.
  * @param data		The data to be sent. This data must remain valid
  *			until the data has been sent.
@@ -975,14 +983,14 @@ PJ_DECL(pj_status_t) pj_ssl_sock_sendto(pj_ssl_sock_t *ssock,
 
 
 /**
- * Starts asynchronous socket accept() operations on this secure socket.
- * This function will issue \a async_cnt number of asynchronous \a accept()
+ * Starts asynchronous socket accept() operations on this secure socket. 
+ * This function will issue \a async_cnt number of asynchronous \a accept() 
  * operations to the socket and returns back to caller. Incoming
  * connection on the socket will be reported back to application via the
  * \a on_accept_complete() callback.
  *
  * Application only needs to call this function once to initiate accept()
- * operations. Further accept() operations will be done automatically by
+ * operations. Further accept() operations will be done automatically by 
  * the secure socket when \a on_accept_complete() callback returns non-zero.
  *
  * @param ssock		The secure socket.
@@ -1001,7 +1009,7 @@ PJ_DECL(pj_status_t) pj_ssl_sock_start_accept(pj_ssl_sock_t *ssock,
 
 
 /**
- * Starts asynchronous socket connect() operation and SSL/TLS handshaking
+ * Starts asynchronous socket connect() operation and SSL/TLS handshaking 
  * for this socket. Once the connection is done (either successfully or not),
  * the \a on_connect_complete() callback will be called.
  *
@@ -1013,9 +1021,9 @@ PJ_DECL(pj_status_t) pj_ssl_sock_start_accept(pj_ssl_sock_t *ssock,
  * @param addr_len	Length of buffer containing above addresses.
  *
  * @return		PJ_SUCCESS if connection can be established immediately
- *			or PJ_EPENDING if connection cannot be established
+ *			or PJ_EPENDING if connection cannot be established 
  *			immediately. In this case the \a on_connect_complete()
- *			callback will be called when connection is complete.
+ *			callback will be called when connection is complete. 
  *			Any other return value indicates error condition.
  */
 PJ_DECL(pj_status_t) pj_ssl_sock_start_connect(pj_ssl_sock_t *ssock,
@@ -1027,8 +1035,8 @@ PJ_DECL(pj_status_t) pj_ssl_sock_start_connect(pj_ssl_sock_t *ssock,
 
 /**
  * Starts SSL/TLS renegotiation over an already established SSL connection
- * for this socket. This operation is performed transparently, no callback
- * will be called once the renegotiation completed successfully. However,
+ * for this socket. This operation is performed transparently, no callback 
+ * will be called once the renegotiation completed successfully. However, 
  * when the renegotiation fails, the connection will be closed and callback
  * \a on_data_read() will be invoked with non-PJ_SUCCESS status code.
  *
@@ -1036,7 +1044,7 @@ PJ_DECL(pj_status_t) pj_ssl_sock_start_connect(pj_ssl_sock_t *ssock,
  *
  * @return		PJ_SUCCESS if renegotiation is completed immediately,
  *			or PJ_EPENDING if renegotiation has been started and
- *			waiting for completion, or the appropriate error code
+ *			waiting for completion, or the appropriate error code 
  *			on failure.
  */
 PJ_DECL(pj_status_t) pj_ssl_sock_renegotiate(pj_ssl_sock_t *ssock);
