@@ -1,5 +1,5 @@
-/* $Id: ioqueue_common_abs.c 3666 2011-07-19 08:40:20Z nanang $ */
-/*
+/* $Id: ioqueue_common_abs.c 4601 2013-09-25 04:20:01Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 
 /*
@@ -43,14 +43,14 @@ static pj_status_t ioqueue_destroy(pj_ioqueue_t *ioqueue)
 	pj_lock_release(ioqueue->lock);
         return pj_lock_destroy(ioqueue->lock);
     }
-
+    
     return PJ_SUCCESS;
 }
 
 /*
  * pj_ioqueue_set_lock()
  */
-PJ_DEF(pj_status_t) pj_ioqueue_set_lock( pj_ioqueue_t *ioqueue,
+PJ_DEF(pj_status_t) pj_ioqueue_set_lock( pj_ioqueue_t *ioqueue, 
 					 pj_lock_t *lock,
 					 pj_bool_t auto_delete )
 {
@@ -70,6 +70,7 @@ static pj_status_t ioqueue_init_key( pj_pool_t *pool,
                                      pj_ioqueue_t *ioqueue,
                                      pj_ioqueue_key_t *key,
                                      pj_sock_t sock,
+                                     pj_grp_lock_t *grp_lock,
                                      void *user_data,
                                      const pj_ioqueue_callback *cb)
 {
@@ -114,10 +115,18 @@ static pj_status_t ioqueue_init_key( pj_pool_t *pool,
 
     /* Create mutex for the key. */
 #if !PJ_IOQUEUE_HAS_SAFE_UNREG
-    rc = pj_mutex_create_simple(pool, NULL, &key->mutex);
+    rc = pj_lock_create_simple_mutex(poll, NULL, &key->lock);
 #endif
+    if (rc != PJ_SUCCESS)
+	return rc;
 
-    return rc;
+    /* Group lock */
+    key->grp_lock = grp_lock;
+    if (key->grp_lock) {
+	pj_grp_lock_add_ref_dbg(key->grp_lock, "ioqueue", 0);
+    }
+    
+    return PJ_SUCCESS;
 }
 
 /*
@@ -189,10 +198,10 @@ PJ_INLINE(int) key_has_pending_connect(pj_ioqueue_key_t *key)
 void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 {
     /* Lock the key. */
-    pj_mutex_lock(h->mutex);
+    pj_ioqueue_lock_key(h);
 
     if (IS_CLOSING(h)) {
-	pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
 	return;
     }
 
@@ -210,7 +219,7 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 
 
 #if (defined(PJ_HAS_SO_ERROR) && PJ_HAS_SO_ERROR!=0)
-	/* from connect(2):
+	/* from connect(2): 
 	 * On Linux, use getsockopt to read the SO_ERROR option at
 	 * level SOL_SOCKET to determine whether connect() completed
 	 * successfully (if SO_ERROR is zero).
@@ -218,10 +227,10 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 	{
 	  int value;
 	  int vallen = sizeof(value);
-	  int gs_rc = pj_sock_getsockopt(h->fd, SOL_SOCKET, SO_ERROR,
+	  int gs_rc = pj_sock_getsockopt(h->fd, SOL_SOCKET, SO_ERROR, 
 					 &value, &vallen);
 	  if (gs_rc != 0) {
-	    /* Argh!! What to do now???
+	    /* Argh!! What to do now??? 
 	     * Just indicate that the socket is connected. The
 	     * application will get error as soon as it tries to use
 	     * the socket to send/receive.
@@ -230,8 +239,8 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 	  } else {
 	      status = PJ_STATUS_FROM_OS(value);
 	  }
-	}
-#elif defined(PJ_WIN32) && PJ_WIN32!=0
+ 	}
+#elif (defined(PJ_WIN32) && PJ_WIN32!=0) || (defined(PJ_WIN64) && PJ_WIN64!=0) 
 	status = PJ_SUCCESS; /* success */
 #else
 	/* Excellent information in D.J. Bernstein page:
@@ -261,7 +270,7 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 	     * save it to a flag.
 	     */
 	    has_lock = PJ_FALSE;
-	    pj_mutex_unlock(h->mutex);
+	    pj_ioqueue_unlock_key(h);
 	} else {
 	    has_lock = PJ_TRUE;
 	}
@@ -272,12 +281,12 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 
 	/* Unlock if we still hold the lock */
 	if (has_lock) {
-	    pj_mutex_unlock(h->mutex);
+	    pj_ioqueue_unlock_key(h);
 	}
 
         /* Done. */
 
-    } else
+    } else 
 #endif /* PJ_HAS_TCP */
     if (key_has_pending_write(h)) {
 	/* Socket is writable. */
@@ -299,7 +308,7 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 
         }
 
-        /* Send the data.
+        /* Send the data. 
          * Unfortunately we must do this while holding key's mutex, thus
          * preventing parallel write on a single key.. :-((
          */
@@ -314,10 +323,10 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
         } else if (write_op->op == PJ_IOQUEUE_OP_SEND_TO) {
 	    int retry = 2;
 	    while (--retry >= 0) {
-		send_rc = pj_sock_sendto(h->fd,
+		send_rc = pj_sock_sendto(h->fd, 
 					 write_op->buf+write_op->written,
 					 &sent, write_op->flags,
-					 &write_op->rmt_addr,
+					 &write_op->rmt_addr, 
 					 write_op->rmt_addrlen);
 #if defined(PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT) && \
 	    PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT!=0
@@ -353,9 +362,9 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
         }
 
         /* Are we finished with this buffer? */
-        if (send_rc!=PJ_SUCCESS ||
+        if (send_rc!=PJ_SUCCESS || 
             write_op->written == (pj_ssize_t)write_op->size ||
-            h->fd_type == pj_SOCK_DGRAM())
+            h->fd_type == pj_SOCK_DGRAM()) 
         {
 	    pj_bool_t has_lock;
 
@@ -379,24 +388,25 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
 		 * save it to a flag.
 		 */
 		has_lock = PJ_FALSE;
-		pj_mutex_unlock(h->mutex);
+		pj_ioqueue_unlock_key(h);
+		PJ_RACE_ME(5);
 	    } else {
 		has_lock = PJ_TRUE;
 	    }
 
 	    /* Call callback. */
             if (h->cb.on_write_complete && !IS_CLOSING(h)) {
-	        (*h->cb.on_write_complete)(h,
+	        (*h->cb.on_write_complete)(h, 
                                            (pj_ioqueue_op_key_t*)write_op,
                                            write_op->written);
             }
 
 	    if (has_lock) {
-		pj_mutex_unlock(h->mutex);
+		pj_ioqueue_unlock_key(h);
 	    }
 
         } else {
-            pj_mutex_unlock(h->mutex);
+            pj_ioqueue_unlock_key(h);
         }
 
         /* Done. */
@@ -406,7 +416,7 @@ void ioqueue_dispatch_write_event(pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h)
          * are signalled for the same event, but only one thread eventually
          * able to process the event.
          */
-        pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
     }
 }
 
@@ -415,10 +425,10 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
     pj_status_t rc;
 
     /* Lock the key. */
-    pj_mutex_lock(h->mutex);
+    pj_ioqueue_lock_key(h);
 
     if (IS_CLOSING(h)) {
-	pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
 	return;
     }
 
@@ -427,7 +437,7 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
 
         struct accept_operation *accept_op;
 	pj_bool_t has_lock;
-
+	
         /* Get one accept operation from the list. */
 	accept_op = h->accept_list.next;
         pj_list_erase(accept_op);
@@ -437,10 +447,10 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
         if (pj_list_empty(&h->accept_list))
             ioqueue_remove_from_set(ioqueue, h, READABLE_EVENT);
 
-	rc=pj_sock_accept(h->fd, accept_op->accept_fd,
+	rc=pj_sock_accept(h->fd, accept_op->accept_fd, 
                           accept_op->rmt_addr, accept_op->addrlen);
 	if (rc==PJ_SUCCESS && accept_op->local_addr) {
-	    rc = pj_sock_getsockname(*accept_op->accept_fd,
+	    rc = pj_sock_getsockname(*accept_op->accept_fd, 
                                      accept_op->local_addr,
 				     accept_op->addrlen);
 	}
@@ -453,20 +463,21 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
 	     * save it to a flag.
 	     */
 	    has_lock = PJ_FALSE;
-	    pj_mutex_unlock(h->mutex);
+	    pj_ioqueue_unlock_key(h);
+	    PJ_RACE_ME(5);
 	} else {
 	    has_lock = PJ_TRUE;
 	}
 
 	/* Call callback. */
         if (h->cb.on_accept_complete && !IS_CLOSING(h)) {
-	    (*h->cb.on_accept_complete)(h,
+	    (*h->cb.on_accept_complete)(h, 
                                         (pj_ioqueue_op_key_t*)accept_op,
                                         *accept_op->accept_fd, rc);
 	}
 
 	if (has_lock) {
-	    pj_mutex_unlock(h->mutex);
+	    pj_ioqueue_unlock_key(h);
 	}
     }
     else
@@ -486,15 +497,15 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
 
         bytes_read = read_op->size;
 
-	if ((read_op->op == PJ_IOQUEUE_OP_RECV_FROM)) {
+	if (read_op->op == PJ_IOQUEUE_OP_RECV_FROM) {
 	    read_op->op = PJ_IOQUEUE_OP_NONE;
-	    rc = pj_sock_recvfrom(h->fd, read_op->buf, &bytes_read,
+	    rc = pj_sock_recvfrom(h->fd, read_op->buf, &bytes_read, 
 				  read_op->flags,
-				  read_op->rmt_addr,
+				  read_op->rmt_addr, 
                                   read_op->rmt_addrlen);
-	} else if ((read_op->op == PJ_IOQUEUE_OP_RECV)) {
+	} else if (read_op->op == PJ_IOQUEUE_OP_RECV) {
 	    read_op->op = PJ_IOQUEUE_OP_NONE;
-	    rc = pj_sock_recv(h->fd, read_op->buf, &bytes_read,
+	    rc = pj_sock_recv(h->fd, read_op->buf, &bytes_read, 
 			      read_op->flags);
         } else {
             pj_assert(read_op->op == PJ_IOQUEUE_OP_READ);
@@ -507,13 +518,14 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
              * just call pj_sock_recv() instead of ReadFile().
              * On Unix, user may put a file in h->fd, so we'll have
              * to call read() here.
-             * This may not compile on systems which doesn't have
+             * This may not compile on systems which doesn't have 
              * read(). That's why we only specify PJ_LINUX here so
              * that error is easier to catch.
              */
 #	    if defined(PJ_WIN32) && PJ_WIN32 != 0 || \
+	       defined(PJ_WIN64) && PJ_WIN64 != 0 || \
 	       defined(PJ_WIN32_WINCE) && PJ_WIN32_WINCE != 0
-                rc = pj_sock_recv(h->fd, read_op->buf, &bytes_read,
+                rc = pj_sock_recv(h->fd, read_op->buf, &bytes_read, 
 				  read_op->flags);
                 //rc = ReadFile((HANDLE)h->fd, read_op->buf, read_op->size,
                 //              &bytes_read, NULL);
@@ -527,20 +539,21 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
 #               error "Implement read() for this platform!"
 #           endif
         }
-
+	
 	if (rc != PJ_SUCCESS) {
-#	    if defined(PJ_WIN32) && PJ_WIN32 != 0
-	    /* On Win32, for UDP, WSAECONNRESET on the receive side
-	     * indicates that previous sending has triggered ICMP Port
+#	    if (defined(PJ_WIN32) && PJ_WIN32 != 0) || \
+	       (defined(PJ_WIN64) && PJ_WIN64 != 0) 
+	    /* On Win32, for UDP, WSAECONNRESET on the receive side 
+	     * indicates that previous sending has triggered ICMP Port 
 	     * Unreachable message.
-	     * But we wouldn't know at this point which one of previous
+	     * But we wouldn't know at this point which one of previous 
 	     * key that has triggered the error, since UDP socket can
 	     * be shared!
 	     * So we'll just ignore it!
 	     */
 
 	    if (rc == PJ_STATUS_FROM_OS(WSAECONNRESET)) {
-		//PJ_LOG(4,(THIS_FILE,
+		//PJ_LOG(4,(THIS_FILE, 
                 //          "Ignored ICMP port unreach. on key=%p", h));
 	    }
 #	    endif
@@ -567,20 +580,21 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
 	     * save it to a flag.
 	     */
 	    has_lock = PJ_FALSE;
-	    pj_mutex_unlock(h->mutex);
+	    pj_ioqueue_unlock_key(h);
+	    PJ_RACE_ME(5);
 	} else {
 	    has_lock = PJ_TRUE;
 	}
 
 	/* Call callback. */
         if (h->cb.on_read_complete && !IS_CLOSING(h)) {
-	    (*h->cb.on_read_complete)(h,
+	    (*h->cb.on_read_complete)(h, 
                                       (pj_ioqueue_op_key_t*)read_op,
                                       bytes_read);
         }
 
 	if (has_lock) {
-	    pj_mutex_unlock(h->mutex);
+	    pj_ioqueue_unlock_key(h);
 	}
 
     } else {
@@ -589,29 +603,29 @@ void ioqueue_dispatch_read_event( pj_ioqueue_t *ioqueue, pj_ioqueue_key_t *h )
          * are signalled for the same event, but only one thread eventually
          * able to process the event.
          */
-        pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
     }
 }
 
 
-void ioqueue_dispatch_exception_event( pj_ioqueue_t *ioqueue,
+void ioqueue_dispatch_exception_event( pj_ioqueue_t *ioqueue, 
                                        pj_ioqueue_key_t *h )
 {
     pj_bool_t has_lock;
 
-    pj_mutex_lock(h->mutex);
+    pj_ioqueue_lock_key(h);
 
     if (!h->connecting) {
         /* It is possible that more than one thread was woken up, thus
          * the remaining thread will see h->connecting as zero because
          * it has been processed by other thread.
          */
-        pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
         return;
     }
 
     if (IS_CLOSING(h)) {
-	pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
 	return;
     }
 
@@ -629,7 +643,8 @@ void ioqueue_dispatch_exception_event( pj_ioqueue_t *ioqueue,
 	 * save it to a flag.
 	 */
 	has_lock = PJ_FALSE;
-	pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
+	PJ_RACE_ME(5);
     } else {
 	has_lock = PJ_TRUE;
     }
@@ -640,7 +655,7 @@ void ioqueue_dispatch_exception_event( pj_ioqueue_t *ioqueue,
 #if (defined(PJ_HAS_SO_ERROR) && PJ_HAS_SO_ERROR!=0)
 	int value;
 	int vallen = sizeof(value);
-	int gs_rc = pj_sock_getsockopt(h->fd, SOL_SOCKET, SO_ERROR,
+	int gs_rc = pj_sock_getsockopt(h->fd, SOL_SOCKET, SO_ERROR, 
 				       &value, &vallen);
 	if (gs_rc == 0) {
 	    status = PJ_RETURN_OS_ERROR(value);
@@ -651,7 +666,7 @@ void ioqueue_dispatch_exception_event( pj_ioqueue_t *ioqueue,
     }
 
     if (has_lock) {
-	pj_mutex_unlock(h->mutex);
+	pj_ioqueue_unlock_key(h);
     }
 }
 
@@ -681,7 +696,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_recv(  pj_ioqueue_key_t *key,
     read_op = (struct read_operation*)op_key;
     read_op->op = PJ_IOQUEUE_OP_NONE;
 
-    /* Try to see if there's data immediately available.
+    /* Try to see if there's data immediately available. 
      */
     if ((flags & PJ_IOQUEUE_ALWAYS_ASYNC) == 0) {
 	pj_status_t status;
@@ -713,18 +728,18 @@ PJ_DEF(pj_status_t) pj_ioqueue_recv(  pj_ioqueue_key_t *key,
     read_op->size = *length;
     read_op->flags = flags;
 
-    pj_mutex_lock(key->mutex);
+    pj_ioqueue_lock_key(key);
     /* Check again. Handle may have been closed after the previous check
      * in multithreaded app. If we add bad handle to the set it will
      * corrupt the ioqueue set. See #913
      */
     if (IS_CLOSING(key)) {
-	pj_mutex_unlock(key->mutex);
+	pj_ioqueue_unlock_key(key);
 	return PJ_ECANCELLED;
     }
     pj_list_insert_before(&key->read_list, read_op);
     ioqueue_add_to_set(key->ioqueue, key, READABLE_EVENT);
-    pj_mutex_unlock(key->mutex);
+    pj_ioqueue_unlock_key(key);
 
     return PJ_EPENDING;
 }
@@ -754,7 +769,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_recvfrom( pj_ioqueue_key_t *key,
     read_op = (struct read_operation*)op_key;
     read_op->op = PJ_IOQUEUE_OP_NONE;
 
-    /* Try to see if there's data immediately available.
+    /* Try to see if there's data immediately available. 
      */
     if ((flags & PJ_IOQUEUE_ALWAYS_ASYNC) == 0) {
 	pj_status_t status;
@@ -789,18 +804,18 @@ PJ_DEF(pj_status_t) pj_ioqueue_recvfrom( pj_ioqueue_key_t *key,
     read_op->rmt_addr = addr;
     read_op->rmt_addrlen = addrlen;
 
-    pj_mutex_lock(key->mutex);
+    pj_ioqueue_lock_key(key);
     /* Check again. Handle may have been closed after the previous check
      * in multithreaded app. If we add bad handle to the set it will
      * corrupt the ioqueue set. See #913
      */
     if (IS_CLOSING(key)) {
-	pj_mutex_unlock(key->mutex);
+	pj_ioqueue_unlock_key(key);
 	return PJ_ECANCELLED;
     }
     pj_list_insert_before(&key->read_list, read_op);
     ioqueue_add_to_set(key->ioqueue, key, READABLE_EVENT);
-    pj_mutex_unlock(key->mutex);
+    pj_ioqueue_unlock_key(key);
 
     return PJ_EPENDING;
 }
@@ -888,7 +903,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_send( pj_ioqueue_key_t *key,
 	 * the sending only. If the polling thread runs on lower priority
 	 * than the sending thread, then it's possible that the pending
 	 * write flag is not cleared in-time because clearing is only done
-	 * during polling.
+	 * during polling. 
 	 *
 	 * Aplication should specify multiple write operation keys on
 	 * situation like this.
@@ -902,19 +917,19 @@ PJ_DEF(pj_status_t) pj_ioqueue_send( pj_ioqueue_key_t *key,
     write_op->size = *length;
     write_op->written = 0;
     write_op->flags = flags;
-
-    pj_mutex_lock(key->mutex);
+    
+    pj_ioqueue_lock_key(key);
     /* Check again. Handle may have been closed after the previous check
      * in multithreaded app. If we add bad handle to the set it will
      * corrupt the ioqueue set. See #913
      */
     if (IS_CLOSING(key)) {
-	pj_mutex_unlock(key->mutex);
+	pj_ioqueue_unlock_key(key);
 	return PJ_ECANCELLED;
     }
     pj_list_insert_before(&key->write_list, write_op);
     ioqueue_add_to_set(key->ioqueue, key, WRITEABLE_EVENT);
-    pj_mutex_unlock(key->mutex);
+    pj_ioqueue_unlock_key(key);
 
     return PJ_EPENDING;
 }
@@ -1001,7 +1016,6 @@ retry_on_restart:
 
                 return status;
             }
-	    status = status;
         }
     }
 
@@ -1014,7 +1028,7 @@ retry_on_restart:
      * Schedule asynchronous send.
      */
     write_op = (struct write_operation*)op_key;
-
+    
     /* Spin if write_op has pending operation */
     for (retry=0; write_op->op != 0 && retry<PENDING_RETRY; ++retry)
 	pj_thread_sleep(0);
@@ -1033,7 +1047,7 @@ retry_on_restart:
 	 * the sending only. If the polling thread runs on lower priority
 	 * than the sending thread, then it's possible that the pending
 	 * write flag is not cleared in-time because clearing is only done
-	 * during polling.
+	 * during polling. 
 	 *
 	 * Aplication should specify multiple write operation keys on
 	 * situation like this.
@@ -1049,19 +1063,19 @@ retry_on_restart:
     write_op->flags = flags;
     pj_memcpy(&write_op->rmt_addr, addr, addrlen);
     write_op->rmt_addrlen = addrlen;
-
-    pj_mutex_lock(key->mutex);
+    
+    pj_ioqueue_lock_key(key);
     /* Check again. Handle may have been closed after the previous check
      * in multithreaded app. If we add bad handle to the set it will
      * corrupt the ioqueue set. See #913
      */
     if (IS_CLOSING(key)) {
-	pj_mutex_unlock(key->mutex);
+	pj_ioqueue_unlock_key(key);
 	return PJ_ECANCELLED;
     }
     pj_list_insert_before(&key->write_list, write_op);
     ioqueue_add_to_set(key->ioqueue, key, WRITEABLE_EVENT);
-    pj_mutex_unlock(key->mutex);
+    pj_ioqueue_unlock_key(key);
 
     return PJ_EPENDING;
 }
@@ -1127,18 +1141,18 @@ PJ_DEF(pj_status_t) pj_ioqueue_accept( pj_ioqueue_key_t *key,
     accept_op->addrlen= addrlen;
     accept_op->local_addr = local;
 
-    pj_mutex_lock(key->mutex);
+    pj_ioqueue_lock_key(key);
     /* Check again. Handle may have been closed after the previous check
      * in multithreaded app. If we add bad handle to the set it will
      * corrupt the ioqueue set. See #913
      */
     if (IS_CLOSING(key)) {
-	pj_mutex_unlock(key->mutex);
+	pj_ioqueue_unlock_key(key);
 	return PJ_ECANCELLED;
     }
     pj_list_insert_before(&key->accept_list, accept_op);
     ioqueue_add_to_set(key->ioqueue, key, READABLE_EVENT);
-    pj_mutex_unlock(key->mutex);
+    pj_ioqueue_unlock_key(key);
 
     return PJ_EPENDING;
 }
@@ -1152,7 +1166,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_connect( pj_ioqueue_key_t *key,
 					int addrlen )
 {
     pj_status_t status;
-
+    
     /* check parameters. All must be specified! */
     PJ_ASSERT_RETURN(key && addr && addrlen, PJ_EINVAL);
 
@@ -1163,7 +1177,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_connect( pj_ioqueue_key_t *key,
     /* Check if socket has not been marked for connecting */
     if (key->connecting != 0)
         return PJ_EPENDING;
-
+    
     status = pj_sock_connect(key->fd, addr, addrlen);
     if (status == PJ_SUCCESS) {
 	/* Connected! */
@@ -1171,18 +1185,18 @@ PJ_DEF(pj_status_t) pj_ioqueue_connect( pj_ioqueue_key_t *key,
     } else {
 	if (status == PJ_STATUS_FROM_OS(PJ_BLOCKING_CONNECT_ERROR_VAL)) {
 	    /* Pending! */
-            pj_mutex_lock(key->mutex);
-	    /* Check again. Handle may have been closed after the previous
+	    pj_ioqueue_lock_key(key);
+	    /* Check again. Handle may have been closed after the previous 
 	     * check in multithreaded app. See #913
 	     */
 	    if (IS_CLOSING(key)) {
-		pj_mutex_unlock(key->mutex);
+		pj_ioqueue_unlock_key(key);
 		return PJ_ECANCELLED;
 	    }
 	    key->connecting = PJ_TRUE;
             ioqueue_add_to_set(key->ioqueue, key, WRITEABLE_EVENT);
             ioqueue_add_to_set(key->ioqueue, key, EXCEPTION_EVENT);
-            pj_mutex_unlock(key->mutex);
+            pj_ioqueue_unlock_key(key);
 	    return PJ_EPENDING;
 	} else {
 	    /* Error! */
@@ -1228,7 +1242,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_post_completion( pj_ioqueue_key_t *key,
      * Find the operation key in all pending operation list to
      * really make sure that it's still there; then call the callback.
      */
-    pj_mutex_lock(key->mutex);
+    pj_ioqueue_lock_key(key);
 
     /* Find the operation in the pending read list. */
     op_rec = (struct generic_operation*)key->read_list.next;
@@ -1236,7 +1250,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_post_completion( pj_ioqueue_key_t *key,
         if (op_rec == (void*)op_key) {
             pj_list_erase(op_rec);
             op_rec->op = PJ_IOQUEUE_OP_NONE;
-            pj_mutex_unlock(key->mutex);
+            pj_ioqueue_unlock_key(key);
 
             (*key->cb.on_read_complete)(key, op_key, bytes_status);
             return PJ_SUCCESS;
@@ -1250,7 +1264,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_post_completion( pj_ioqueue_key_t *key,
         if (op_rec == (void*)op_key) {
             pj_list_erase(op_rec);
             op_rec->op = PJ_IOQUEUE_OP_NONE;
-            pj_mutex_unlock(key->mutex);
+            pj_ioqueue_unlock_key(key);
 
             (*key->cb.on_write_complete)(key, op_key, bytes_status);
             return PJ_SUCCESS;
@@ -1264,18 +1278,18 @@ PJ_DEF(pj_status_t) pj_ioqueue_post_completion( pj_ioqueue_key_t *key,
         if (op_rec == (void*)op_key) {
             pj_list_erase(op_rec);
             op_rec->op = PJ_IOQUEUE_OP_NONE;
-            pj_mutex_unlock(key->mutex);
+            pj_ioqueue_unlock_key(key);
 
-            (*key->cb.on_accept_complete)(key, op_key,
+            (*key->cb.on_accept_complete)(key, op_key, 
                                           PJ_INVALID_SOCKET,
-                                          bytes_status);
+                                          (pj_status_t)bytes_status);
             return PJ_SUCCESS;
         }
         op_rec = op_rec->next;
     }
 
-    pj_mutex_unlock(key->mutex);
-
+    pj_ioqueue_unlock_key(key);
+    
     return PJ_EINVALIDOP;
 }
 
@@ -1304,10 +1318,18 @@ PJ_DEF(pj_status_t) pj_ioqueue_set_concurrency(pj_ioqueue_key_t *key,
 
 PJ_DEF(pj_status_t) pj_ioqueue_lock_key(pj_ioqueue_key_t *key)
 {
-    return pj_mutex_lock(key->mutex);
+    if (key->grp_lock)
+	return pj_grp_lock_acquire(key->grp_lock);
+    else
+	return pj_lock_acquire(key->lock);
 }
 
 PJ_DEF(pj_status_t) pj_ioqueue_unlock_key(pj_ioqueue_key_t *key)
 {
-    return pj_mutex_unlock(key->mutex);
+    if (key->grp_lock)
+	return pj_grp_lock_release(key->grp_lock);
+    else
+	return pj_lock_release(key->lock);
 }
+
+
