@@ -1,4 +1,4 @@
-/* $Id: lock.c 4412 2013-03-05 03:12:32Z riza $ */
+/* $Id: lock.c 4613 2013-10-08 09:08:13Z bennylp $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -683,7 +683,7 @@ PJ_DEF(void) pj_grp_lock_dump(pj_grp_lock_t *grp_lock)
 	int len;
 
 	len = pj_ansi_snprintf(start, max_len, "%s:%d ", ref->file, ref->line);
-	if (len < 1 || len > max_len) {
+	if (len < 1 || len >= max_len) {
 	    len = strlen(ref->file);
 	    if (len > max_len - 1)
 		len = max_len - 1;
