@@ -1,5 +1,5 @@
-/* $Id: sock_common.c 3841 2011-10-24 09:28:13Z ming $ */
-/*
+/* $Id: sock_common.c 4538 2013-06-19 09:06:55Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pj/sock.h>
 #include <pj/assert.h>
@@ -24,6 +24,7 @@
 #include <pj/ip_helper.h>
 #include <pj/os.h>
 #include <pj/addr_resolv.h>
+#include <pj/rand.h>
 #include <pj/string.h>
 #include <pj/compat/socket.h>
 
@@ -39,7 +40,7 @@
 
 /*
  * Convert address string with numbers and dots to binary IP address.
- */
+ */ 
 PJ_DEF(pj_in_addr) pj_inet_addr(const pj_str_t *cp)
 {
     pj_in_addr addr;
@@ -50,7 +51,7 @@ PJ_DEF(pj_in_addr) pj_inet_addr(const pj_str_t *cp)
 
 /*
  * Convert address string with numbers and dots to binary IP address.
- */
+ */ 
 PJ_DEF(pj_in_addr) pj_inet_addr2(const char *cp)
 {
     pj_str_t str = pj_str((char*)cp);
@@ -112,7 +113,7 @@ PJ_DEF(char*) pj_sockaddr_print( const pj_sockaddr_t *addr,
 }
 
 /*
- * Set the IP address of an IP socket address from string address,
+ * Set the IP address of an IP socket address from string address, 
  * with resolving the host if necessary. The string address may be in a
  * standard numbers and dots notation or may be a hostname. If hostname
  * is specified, then the function will resolve the host into the IP
@@ -123,7 +124,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_in_set_str_addr( pj_sockaddr_in *addr,
 {
     PJ_CHECK_STACK();
 
-    PJ_ASSERT_RETURN(!str_addr || str_addr->slen < PJ_MAX_HOSTNAME,
+    PJ_ASSERT_RETURN(!str_addr || str_addr->slen < PJ_MAX_HOSTNAME, 
                      (addr->sin_addr.s_addr=PJ_INADDR_NONE, PJ_EINVAL));
 
     PJ_SOCKADDR_RESET_LEN(addr);
@@ -133,7 +134,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_in_set_str_addr( pj_sockaddr_in *addr,
     if (str_addr && str_addr->slen) {
 	addr->sin_addr = pj_inet_addr(str_addr);
 	if (addr->sin_addr.s_addr == PJ_INADDR_NONE) {
-	    pj_hostent he;
+    	    pj_hostent he;
 	    pj_status_t rc;
 
 	    rc = pj_gethostbyname(str_addr, &he);
@@ -173,7 +174,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_set_str_addr(int af,
     if (str_addr && str_addr->slen) {
 	status = pj_inet_pton(PJ_AF_INET6, str_addr, &addr->ipv6.sin6_addr);
 	if (status != PJ_SUCCESS) {
-	    pj_addrinfo ai;
+    	    pj_addrinfo ai;
 	    unsigned count = 1;
 
 	    status = pj_getaddrinfo(PJ_AF_INET6, str_addr, &count, &ai);
@@ -191,8 +192,8 @@ PJ_DEF(pj_status_t) pj_sockaddr_set_str_addr(int af,
 
 /*
  * Set the IP address and port of an IP socket address.
- * The string address may be in a standard numbers and dots notation or
- * may be a hostname. If hostname is specified, then the function will
+ * The string address may be in a standard numbers and dots notation or 
+ * may be a hostname. If hostname is specified, then the function will 
  * resolve the host into the IP address.
  */
 PJ_DEF(pj_status_t) pj_sockaddr_in_init( pj_sockaddr_in *addr,
@@ -211,7 +212,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_in_init( pj_sockaddr_in *addr,
 /*
  * Initialize IP socket address based on the address and port info.
  */
-PJ_DEF(pj_status_t) pj_sockaddr_init(int af,
+PJ_DEF(pj_status_t) pj_sockaddr_init(int af, 
 				     pj_sockaddr *addr,
 				     const pj_str_t *cp,
 				     pj_uint16_t port)
@@ -227,7 +228,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_init(int af,
 
     pj_bzero(addr, sizeof(pj_sockaddr_in6));
     addr->addr.sa_family = PJ_AF_INET6;
-
+    
     status = pj_sockaddr_set_str_addr(af, addr, cp);
     if (status != PJ_SUCCESS)
 	return status;
@@ -270,9 +271,9 @@ PJ_DEF(int) pj_sockaddr_cmp( const pj_sockaddr_t *addr1,
 	return 1;
 
     /* TODO:
-     *	Do we need to compare flow label and scope id in IPv6?
+     *	Do we need to compare flow label and scope id in IPv6? 
      */
-
+    
     /* Looks equal */
     return 0;
 }
@@ -323,8 +324,8 @@ PJ_DEF(pj_bool_t) pj_sockaddr_has_addr(const pj_sockaddr_t *addr)
     /* It's probably not wise to raise assertion here if
      * the address doesn't contain a valid address family, and
      * just return PJ_FALSE instead.
-     *
-     * The reason is because application may need to distinguish
+     * 
+     * The reason is because application may need to distinguish 
      * these three conditions with sockaddr:
      *	a) sockaddr is not initialized. This is by convention
      *	   indicated by sa_family==0.
@@ -345,7 +346,7 @@ PJ_DEF(pj_bool_t) pj_sockaddr_has_addr(const pj_sockaddr_t *addr)
     } else if (a->addr.sa_family == PJ_AF_INET6) {
 	pj_uint8_t zero[24];
 	pj_bzero(zero, sizeof(zero));
-	return pj_memcmp(a->ipv6.sin6_addr.s6_addr, zero,
+	return pj_memcmp(a->ipv6.sin6_addr.s6_addr, zero, 
 			 sizeof(pj_in6_addr)) != 0;
     } else
 	return a->ipv4.sin_addr.s_addr != PJ_INADDR_ANY;
@@ -412,7 +413,7 @@ PJ_DEF(void) pj_sockaddr_cp(pj_sockaddr_t *dst, const pj_sockaddr_t *src)
 /*
  * Set port number of pj_sockaddr_in
  */
-PJ_DEF(void) pj_sockaddr_in_set_port(pj_sockaddr_in *addr,
+PJ_DEF(void) pj_sockaddr_in_set_port(pj_sockaddr_in *addr, 
 				     pj_uint16_t hostport)
 {
     addr->sin_port = pj_htons(hostport);
@@ -421,7 +422,7 @@ PJ_DEF(void) pj_sockaddr_in_set_port(pj_sockaddr_in *addr,
 /*
  * Set port number of pj_sockaddr
  */
-PJ_DEF(pj_status_t) pj_sockaddr_set_port(pj_sockaddr *addr,
+PJ_DEF(pj_status_t) pj_sockaddr_set_port(pj_sockaddr *addr, 
 					 pj_uint16_t hostport)
 {
     int af = addr->addr.sa_family;
@@ -639,7 +640,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_parse( int af, unsigned options,
     status = pj_sockaddr_parse2(af, options, str, &hostpart, &port, &af);
     if (status != PJ_SUCCESS)
 	return status;
-
+    
 #if !defined(PJ_HAS_IPV6) || !PJ_HAS_IPV6
     if (af==PJ_AF_INET6)
 	return PJ_EIPV6NOTSUP;
@@ -648,7 +649,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_parse( int af, unsigned options,
     status = pj_sockaddr_init(af, addr, &hostpart, port);
 #if defined(PJ_HAS_IPV6) && PJ_HAS_IPV6
     if (status != PJ_SUCCESS && af == PJ_AF_INET6) {
-	/* Parsing does not yield valid address. Try to treat the last
+	/* Parsing does not yield valid address. Try to treat the last 
 	 * portion after the colon as port number.
 	 */
 	const char *last_colon_pos=NULL, *p;
@@ -686,7 +687,7 @@ PJ_DEF(pj_status_t) pj_sockaddr_parse( int af, unsigned options,
 	status = pj_sockaddr_init(PJ_AF_INET6, addr, &hostpart, port);
     }
 #endif
-
+    
     return status;
 }
 
@@ -790,8 +791,8 @@ PJ_DEF(pj_status_t) pj_gethostip(int af, pj_sockaddr *addr)
     count = 1;
     status = pj_getaddrinfo(af, pj_gethostname(), &count, &ai);
     if (status == PJ_SUCCESS) {
-	pj_assert(ai.ai_addr.addr.sa_family == (pj_uint16_t)af);
-	pj_sockaddr_copy_addr(&cand_addr[cand_cnt], &ai.ai_addr);
+    	pj_assert(ai.ai_addr.addr.sa_family == (pj_uint16_t)af);
+    	pj_sockaddr_copy_addr(&cand_addr[cand_cnt], &ai.ai_addr);
 	pj_sockaddr_set_port(&cand_addr[cand_cnt], 0);
 	cand_weight[cand_cnt] += WEIGHT_HOSTNAME;
 	++cand_cnt;
@@ -845,7 +846,7 @@ PJ_DEF(pj_status_t) pj_gethostip(int af, pj_sockaddr *addr)
 	    for (i=0; i<cand_cnt; ++i) {
 		unsigned j;
 		for (j=0; j<count; ++j) {
-		    if (pj_sockaddr_cmp(&cand_addr[i],
+		    if (pj_sockaddr_cmp(&cand_addr[i], 
 					&cand_addr[start_if+j])==0)
 			break;
 		}
@@ -862,13 +863,13 @@ PJ_DEF(pj_status_t) pj_gethostip(int af, pj_sockaddr *addr)
 	    for (i=0; i<count; ++i) {
 		unsigned j;
 		for (j=0; j<cand_cnt; ++j) {
-		    if (pj_sockaddr_cmp(&cand_addr[start_if+i],
+		    if (pj_sockaddr_cmp(&cand_addr[start_if+i], 
 					&cand_addr[j])==0)
 			break;
 		}
 
 		if (j == cand_cnt) {
-		    pj_sockaddr_copy_addr(&cand_addr[cand_cnt],
+		    pj_sockaddr_copy_addr(&cand_addr[cand_cnt], 
 					  &cand_addr[start_if+i]);
 		    cand_weight[cand_cnt] += WEIGHT_INTERFACE;
 		    ++cand_cnt;
@@ -956,42 +957,54 @@ PJ_DEF(pj_status_t) pj_gethostip(int af, pj_sockaddr *addr)
     return PJ_SUCCESS;
 }
 
-/* Get the default IP interface */
-PJ_DEF(pj_status_t) pj_getdefaultipinterface(int af, pj_sockaddr *addr)
+/* Get IP interface for sending to the specified destination */
+PJ_DEF(pj_status_t) pj_getipinterface(int af,
+                                      const pj_str_t *dst,
+                                      pj_sockaddr *itf_addr,
+                                      pj_bool_t allow_resolve,
+                                      pj_sockaddr *p_dst_addr)
 {
+    pj_sockaddr dst_addr;
     pj_sock_t fd;
-    pj_str_t cp;
-    pj_sockaddr a;
     int len;
     pj_uint8_t zero[64];
     pj_status_t status;
 
-    addr->addr.sa_family = (pj_uint16_t)af;
+    pj_sockaddr_init(af, &dst_addr, NULL, 53);
+    status = pj_inet_pton(af, dst, pj_sockaddr_get_addr(&dst_addr));
+    if (status != PJ_SUCCESS) {
+	/* "dst" is not an IP address. */
+	if (allow_resolve) {
+	    status = pj_sockaddr_init(af, &dst_addr, dst, 53);
+	} else {
+	    pj_str_t cp;
 
+	    if (af == PJ_AF_INET) {
+		cp = pj_str("1.1.1.1");
+	    } else {
+		cp = pj_str("1::1");
+	    }
+	    status = pj_sockaddr_init(af, &dst_addr, &cp, 53);
+	}
+
+	if (status != PJ_SUCCESS)
+	    return status;
+    }
+
+    /* Create UDP socket and connect() to the destination IP */
     status = pj_sock_socket(af, pj_SOCK_DGRAM(), 0, &fd);
     if (status != PJ_SUCCESS) {
 	return status;
     }
 
-    if (af == PJ_AF_INET) {
-	cp = pj_str("1.1.1.1");
-    } else {
-	cp = pj_str("1::1");
-    }
-    status = pj_sockaddr_init(af, &a, &cp, 53);
+    status = pj_sock_connect(fd, &dst_addr, pj_sockaddr_get_len(&dst_addr));
     if (status != PJ_SUCCESS) {
 	pj_sock_close(fd);
 	return status;
     }
 
-    status = pj_sock_connect(fd, &a, pj_sockaddr_get_len(&a));
-    if (status != PJ_SUCCESS) {
-	pj_sock_close(fd);
-	return status;
-    }
-
-    len = sizeof(a);
-    status = pj_sock_getsockname(fd, &a, &len);
+    len = sizeof(*itf_addr);
+    status = pj_sock_getsockname(fd, itf_addr, &len);
     if (status != PJ_SUCCESS) {
 	pj_sock_close(fd);
 	return status;
@@ -1001,18 +1014,133 @@ PJ_DEF(pj_status_t) pj_getdefaultipinterface(int af, pj_sockaddr *addr)
 
     /* Check that the address returned is not zero */
     pj_bzero(zero, sizeof(zero));
-    if (pj_memcmp(pj_sockaddr_get_addr(&a), zero,
-		  pj_sockaddr_get_addr_len(&a))==0)
+    if (pj_memcmp(pj_sockaddr_get_addr(itf_addr), zero,
+		  pj_sockaddr_get_addr_len(itf_addr))==0)
     {
 	return PJ_ENOTFOUND;
     }
 
-    pj_sockaddr_copy_addr(addr, &a);
+    if (p_dst_addr)
+	*p_dst_addr = dst_addr;
 
-    /* Success */
     return PJ_SUCCESS;
 }
 
+/* Get the default IP interface */
+PJ_DEF(pj_status_t) pj_getdefaultipinterface(int af, pj_sockaddr *addr)
+{
+    pj_str_t cp;
+
+    if (af == PJ_AF_INET) {
+	cp = pj_str("1.1.1.1");
+    } else {
+	cp = pj_str("1::1");
+    }
+
+    return pj_getipinterface(af, &cp, addr, PJ_FALSE, NULL);
+}
+
+
+/*
+ * Bind socket at random port.
+ */
+PJ_DEF(pj_status_t) pj_sock_bind_random(  pj_sock_t sockfd,
+				          const pj_sockaddr_t *addr,
+				          pj_uint16_t port_range,
+				          pj_uint16_t max_try)
+{
+    pj_sockaddr bind_addr;
+    int addr_len;
+    pj_uint16_t base_port;
+    pj_status_t status = PJ_SUCCESS;
+
+    PJ_CHECK_STACK();
+
+    PJ_ASSERT_RETURN(addr, PJ_EINVAL);
+
+    pj_sockaddr_cp(&bind_addr, addr);
+    addr_len = pj_sockaddr_get_len(addr);
+    base_port = pj_sockaddr_get_port(addr);
+
+    if (base_port == 0 || port_range == 0) {
+	return pj_sock_bind(sockfd, &bind_addr, addr_len);
+    }
+
+    for (; max_try; --max_try) {
+	pj_uint16_t port;
+	port = (pj_uint16_t)(base_port + pj_rand() % (port_range + 1));
+	pj_sockaddr_set_port(&bind_addr, port);
+	status = pj_sock_bind(sockfd, &bind_addr, addr_len);
+	if (status == PJ_SUCCESS)
+	    break;
+    }
+
+    return status;
+}
+
+
+/*
+ * Adjust socket send/receive buffer size.
+ */
+PJ_DEF(pj_status_t) pj_sock_setsockopt_sobuf( pj_sock_t sockfd,
+					      pj_uint16_t optname,
+					      pj_bool_t auto_retry,
+					      unsigned *buf_size)
+{
+    pj_status_t status;
+    int try_size, cur_size, i, step, size_len;
+    enum { MAX_TRY = 20 };
+
+    PJ_CHECK_STACK();
+
+    PJ_ASSERT_RETURN(sockfd != PJ_INVALID_SOCKET &&
+		     buf_size &&
+		     *buf_size > 0 &&
+		     (optname == pj_SO_RCVBUF() ||
+		      optname == pj_SO_SNDBUF()),
+		     PJ_EINVAL);
+
+    size_len = sizeof(cur_size);
+    status = pj_sock_getsockopt(sockfd, pj_SOL_SOCKET(), optname,
+				&cur_size, &size_len);
+    if (status != PJ_SUCCESS)
+	return status;
+
+    try_size = *buf_size;
+    step = (try_size - cur_size) / MAX_TRY;
+    if (step < 4096)
+	step = 4096;
+
+    for (i = 0; i < (MAX_TRY-1); ++i) {
+	if (try_size <= cur_size) {
+	    /* Done, return current size */
+	    *buf_size = cur_size;
+	    break;
+	}
+
+	status = pj_sock_setsockopt(sockfd, pj_SOL_SOCKET(), optname,
+				    &try_size, sizeof(try_size));
+	if (status == PJ_SUCCESS) {
+	    status = pj_sock_getsockopt(sockfd, pj_SOL_SOCKET(), optname,
+					&cur_size, &size_len);
+	    if (status != PJ_SUCCESS) {
+		/* Ops! No info about current size, just return last try size
+		 * and quit.
+		 */
+		*buf_size = try_size;
+		break;
+	    }
+	}
+
+	if (!auto_retry)
+	    break;
+
+	try_size -= step;
+    }
+
+    return status;
+}
+
 
 /* Only need to implement these in DLL build */
 #if defined(PJ_DLL)
@@ -1193,3 +1321,4 @@ PJ_DEF(int) pj_MSG_DONTROUTE(void)
 }
 
 #endif	/* PJ_DLL */
+
