@@ -1,4 +1,4 @@
-/* $Id: ssl_sock_ossl.c 4349 2013-02-14 09:38:31Z nanang $ */
+/* $Id: ssl_sock_ossl.c 4624 2013-10-21 06:37:30Z ming $ */
 /* 
  * Copyright (C) 2009-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -52,13 +52,8 @@
 
 
 #ifdef _MSC_VER
-# ifdef _DEBUG
-#  pragma comment( lib, "libeay32MTd")
-#  pragma comment( lib, "ssleay32MTd")
-#else
-#  pragma comment( lib, "libeay32MT")
-#  pragma comment( lib, "ssleay32MT")
-# endif
+#  pragma comment( lib, "libeay32")
+#  pragma comment( lib, "ssleay32")
 #endif
 
 
@@ -277,7 +272,8 @@ static pj_str_t ssl_strerror(pj_status_t status,
     errstr.slen = pj_ansi_snprintf(buf, bufsize, 
 				   "Unknown OpenSSL error %lu",
 				   ssl_err);
-
+    if (errstr.slen < 1 || errstr.slen >= (int)bufsize)
+	errstr.slen = bufsize - 1;
     return errstr;
 }
 
@@ -385,7 +381,7 @@ static int password_cb(char *buf, int num, int rwflag, void *user_data)
 	return 0;
     
     pj_memcpy(buf, cert->privkey_pass.ptr, cert->privkey_pass.slen);
-    return cert->privkey_pass.slen;
+    return (int)cert->privkey_pass.slen;
 }
 
 
@@ -614,8 +610,8 @@ static pj_status_t create_ssl(pj_ssl_sock_t *ssock)
     /* Setup SSL BIOs */
     ssock->ossl_rbio = BIO_new(BIO_s_mem());
     ssock->ossl_wbio = BIO_new(BIO_s_mem());
-    BIO_set_close(ssock->ossl_rbio, BIO_CLOSE);
-    BIO_set_close(ssock->ossl_wbio, BIO_CLOSE);
+    (void)BIO_set_close(ssock->ossl_rbio, BIO_CLOSE);
+    (void)BIO_set_close(ssock->ossl_wbio, BIO_CLOSE);
     SSL_set_bio(ssock->ossl_ssl, ssock->ossl_rbio, ssock->ossl_wbio);
 
     return PJ_SUCCESS;
@@ -992,7 +988,7 @@ static pj_bool_t on_handshake_complete(pj_ssl_sock_t *ssock,
 		      errmsg));
 
 	    /* Workaround for ticket #985 */
-#if defined(PJ_WIN32) && PJ_WIN32!=0
+#if (defined(PJ_WIN32) && PJ_WIN32!=0) || (defined(PJ_WIN64) && PJ_WIN64!=0)
 	    if (ssock->param.timer_heap) {
 		pj_time_val interval = {0, DELAYED_CLOSE_TIMEOUT};
 
@@ -1135,7 +1131,7 @@ static void free_send_data(pj_ssl_sock_t *ssock, write_data_t *wdata)
 	    buf->len -= ((char*)wdata->next - buf->start);
 	} else {
 	    /* Overlapped */
-	    unsigned right_len, left_len;
+	    pj_size_t right_len, left_len;
 	    right_len = buf->buf + buf->max_len - (char*)wdata;
 	    left_len  = (char*)wdata->next - buf->buf;
 	    buf->len -= (right_len + left_len);
@@ -1143,13 +1139,13 @@ static void free_send_data(pj_ssl_sock_t *ssock, write_data_t *wdata)
     } else if (spl->prev == wdata) {
 	/* This is the last data, just adjust the buffer length */
 	if (wdata->prev < wdata) {
-	    unsigned jump_len;
+	    pj_size_t jump_len;
 	    jump_len = (char*)wdata -
 		       ((char*)wdata->prev + wdata->prev->record_len);
 	    buf->len -= (wdata->record_len + jump_len);
 	} else {
 	    /* Overlapped */
-	    unsigned right_len, left_len;
+	    pj_size_t right_len, left_len;
 	    right_len = buf->buf + buf->max_len -
 			((char*)wdata->prev + wdata->prev->record_len);
 	    left_len  = (char*)wdata + wdata->record_len - buf->buf;
@@ -1277,7 +1273,7 @@ static pj_status_t flush_write_bio(pj_ssl_sock_t *ssock,
     pj_memcpy(&wdata->data, data, len);
 
     /* Reset write BIO */
-    BIO_reset(ssock->ossl_wbio);
+    (void)BIO_reset(ssock->ossl_wbio);
 
     /* Ticket #1573: Don't hold mutex while calling PJLIB socket send(). */
     pj_lock_release(ssock->write_mutex);
@@ -1392,7 +1388,7 @@ static pj_bool_t asock_on_data_read (pj_activesock_t *asock,
     /* Socket error or closed */
     if (data && size > 0) {
 	/* Consume the whole data */
-	nwritten = BIO_write(ssock->ossl_rbio, data, size);
+	nwritten = BIO_write(ssock->ossl_rbio, data, (int)size);
 	if (nwritten < size) {
 	    status = GET_SSL_STATUS(ssock);
 	    goto on_error;
@@ -1418,7 +1414,7 @@ static pj_bool_t asock_on_data_read (pj_activesock_t *asock,
 	do {
 	    read_data_t *buf = *(OFFSET_OF_READ_DATA_PTR(ssock, data));
 	    void *data_ = (pj_int8_t*)buf->data + buf->len;
-	    int size_ = ssock->read_size - buf->len;
+	    int size_ = (int)(ssock->read_size - buf->len);
 
 	    /* SSL_read() may write some data to BIO write when re-negotiation
 	     * is on progress, so let's protect it with write mutex.
@@ -1460,7 +1456,7 @@ static pj_bool_t asock_on_data_read (pj_activesock_t *asock,
 
 	    } else {
 
-		int err = SSL_get_error(ssock->ossl_ssl, size);
+		int err = SSL_get_error(ssock->ossl_ssl, (int)size);
 		
 		/* SSL might just return SSL_ERROR_WANT_READ in 
 		 * re-negotiation.
@@ -1668,7 +1664,7 @@ static pj_bool_t asock_on_accept_complete (pj_activesock_t *asock,
 
     /* Start read */
     status = pj_activesock_start_read2(ssock->asock, ssock->pool, 
-				       ssock->param.read_buffer_size,
+				       (unsigned)ssock->param.read_buffer_size,
 				       ssock->asock_rbuf,
 				       PJ_IOQUEUE_ALWAYS_ASYNC);
     if (status != PJ_SUCCESS)
@@ -1747,7 +1743,7 @@ static pj_bool_t asock_on_connect_complete (pj_activesock_t *asock,
 
     /* Start read */
     status = pj_activesock_start_read2(ssock->asock, ssock->pool, 
-				       ssock->param.read_buffer_size,
+				       (unsigned)ssock->param.read_buffer_size,
 				       ssock->asock_rbuf,
 				       PJ_IOQUEUE_ALWAYS_ASYNC);
     if (status != PJ_SUCCESS)
@@ -2185,7 +2181,7 @@ static pj_status_t ssl_write(pj_ssl_sock_t *ssock,
      * until re-negotiation is completed.
      */
     pj_lock_acquire(ssock->write_mutex);
-    nwritten = SSL_write(ssock->ossl_ssl, data, size);
+    nwritten = SSL_write(ssock->ossl_ssl, data, (int)size);
     pj_lock_release(ssock->write_mutex);
     
     if (nwritten == size) {
@@ -2382,6 +2378,18 @@ PJ_DEF(pj_status_t) pj_ssl_sock_start_accept (pj_ssl_sock_t *ssock,
     if (status != PJ_SUCCESS)
 	goto on_error;
 
+    /* Apply SO_REUSEADDR */
+    if (ssock->param.reuse_addr) {
+	int enabled = 1;
+	status = pj_sock_setsockopt(ssock->sock, pj_SOL_SOCKET(),
+				    pj_SO_REUSEADDR(),
+				    &enabled, sizeof(enabled));
+	if (status != PJ_SUCCESS) {
+	    PJ_PERROR(4,(ssock->pool->obj_name, status,
+		         "Warning: error applying SO_REUSEADDR"));
+	}
+    }
+
     /* Apply QoS, if specified */
     status = pj_sock_apply_qos2(ssock->sock, ssock->param.qos_type,
 				&ssock->param.qos_params, 2, 
