@@ -1,5 +1,5 @@
-/* $Id: timer.c 4154 2012-06-05 10:41:17Z bennylp $ */
-/*
+/* $Id: timer.c 4567 2013-07-19 06:31:28Z bennylp $ */
+/* 
  * The PJLIB's timer heap is based (or more correctly, copied and modied)
  * from ACE library by Douglas C. Schmidt. ACE is an excellent OO framework
  * that implements many core patterns for concurrent communication software.
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pj/timer.h>
 #include <pj/pool.h>
@@ -35,6 +35,7 @@
 #include <pj/errno.h>
 #include <pj/lock.h>
 #include <pj/log.h>
+#include <pj/rand.h>
 
 #define THIS_FILE	"timer.c"
 
@@ -44,6 +45,13 @@
 
 #define DEFAULT_MAX_TIMED_OUT_PER_POLL  (64)
 
+enum
+{
+    F_DONT_CALL = 1,
+    F_DONT_ASSERT = 2,
+    F_SET_ID = 4
+};
+
 
 /**
  * The implementation of timer heap.
@@ -116,31 +124,32 @@ PJ_INLINE(void) unlock_timer_heap( pj_timer_heap_t *ht )
 }
 
 
-static void copy_node( pj_timer_heap_t *ht, int slot, pj_timer_entry *moved_node )
+static void copy_node( pj_timer_heap_t *ht, pj_size_t slot, 
+		       pj_timer_entry *moved_node )
 {
     PJ_CHECK_STACK();
 
     // Insert <moved_node> into its new location in the heap.
     ht->heap[slot] = moved_node;
-
+    
     // Update the corresponding slot in the parallel <timer_ids_> array.
-    ht->timer_ids[moved_node->_timer_id] = slot;
+    ht->timer_ids[moved_node->_timer_id] = (int)slot;
 }
 
 static pj_timer_id_t pop_freelist( pj_timer_heap_t *ht )
 {
     // We need to truncate this to <int> for backwards compatibility.
     pj_timer_id_t new_id = ht->timer_ids_freelist;
-
+    
     PJ_CHECK_STACK();
 
     // The freelist values in the <timer_ids_> are negative, so we need
     // to negate them to get the next freelist "pointer."
     ht->timer_ids_freelist =
 	-ht->timer_ids[ht->timer_ids_freelist];
-
+    
     return new_id;
-
+    
 }
 
 static void push_freelist (pj_timer_heap_t *ht, pj_timer_id_t old_id)
@@ -160,14 +169,14 @@ static void reheap_down(pj_timer_heap_t *ht, pj_timer_entry *moved_node,
     PJ_CHECK_STACK();
 
     // Restore the heap property after a deletion.
-
+    
     while (child < ht->cur_size)
     {
 	// Choose the smaller of the two children.
 	if (child + 1 < ht->cur_size
 	    && PJ_TIME_VAL_LT(ht->heap[child + 1]->_timer_value, ht->heap[child]->_timer_value))
 	    child++;
-
+	
 	// Perform a <copy> if the child has a larger timeout value than
 	// the <moved_node>.
 	if (PJ_TIME_VAL_LT(ht->heap[child]->_timer_value, moved_node->_timer_value))
@@ -180,7 +189,7 @@ static void reheap_down(pj_timer_heap_t *ht, pj_timer_entry *moved_node,
 	    // We've found our location in the heap.
 	    break;
     }
-
+    
     copy_node( ht, slot, moved_node);
 }
 
@@ -188,7 +197,7 @@ static void reheap_up( pj_timer_heap_t *ht, pj_timer_entry *moved_node,
 		       size_t slot, size_t parent)
 {
     // Restore the heap property after an insertion.
-
+    
     while (slot > 0)
     {
 	// If the parent node is greater than the <moved_node> we need
@@ -202,7 +211,7 @@ static void reheap_up( pj_timer_heap_t *ht, pj_timer_entry *moved_node,
 	else
 	    break;
     }
-
+    
     // Insert the new node into its proper resting place in the heap and
     // update the corresponding slot in the parallel <timer_ids> array.
     copy_node(ht, slot, moved_node);
@@ -212,38 +221,38 @@ static void reheap_up( pj_timer_heap_t *ht, pj_timer_entry *moved_node,
 static pj_timer_entry * remove_node( pj_timer_heap_t *ht, size_t slot)
 {
     pj_timer_entry *removed_node = ht->heap[slot];
-
+    
     // Return this timer id to the freelist.
     push_freelist( ht, removed_node->_timer_id );
-
+    
     // Decrement the size of the heap by one since we're removing the
     // "slot"th node.
     ht->cur_size--;
-
+    
     // Set the ID
     removed_node->_timer_id = -1;
 
     // Only try to reheapify if we're not deleting the last entry.
-
+    
     if (slot < ht->cur_size)
     {
-	int parent;
+	pj_size_t parent;
 	pj_timer_entry *moved_node = ht->heap[ht->cur_size];
-
+	
 	// Move the end node to the location being removed and update
 	// the corresponding slot in the parallel <timer_ids> array.
 	copy_node( ht, slot, moved_node);
-
+	
 	// If the <moved_node->time_value_> is great than or equal its
 	// parent it needs be moved down the heap.
 	parent = HEAP_PARENT (slot);
-
+	
 	if (PJ_TIME_VAL_GTE(moved_node->_timer_value, ht->heap[parent]->_timer_value))
 	    reheap_down( ht, moved_node, slot, HEAP_LEFT(slot));
 	else
 	    reheap_up( ht, moved_node, slot, parent);
     }
-
+    
     return removed_node;
 }
 
@@ -253,32 +262,32 @@ static void grow_heap(pj_timer_heap_t *ht)
     size_t new_size = ht->max_size * 2;
     pj_timer_id_t *new_timer_ids;
     pj_size_t i;
-
+    
     // First grow the heap itself.
-
+    
     pj_timer_entry **new_heap = 0;
-
-    new_heap = (pj_timer_entry**)
-	       pj_pool_alloc(ht->pool, sizeof(pj_timer_entry*) * new_size);
+    
+    new_heap = (pj_timer_entry**) 
+    	       pj_pool_alloc(ht->pool, sizeof(pj_timer_entry*) * new_size);
     memcpy(new_heap, ht->heap, ht->max_size * sizeof(pj_timer_entry*));
     //delete [] this->heap_;
     ht->heap = new_heap;
-
+    
     // Grow the array of timer ids.
-
+    
     new_timer_ids = 0;
     new_timer_ids = (pj_timer_id_t*)
-		    pj_pool_alloc(ht->pool, new_size * sizeof(pj_timer_id_t));
-
+    		    pj_pool_alloc(ht->pool, new_size * sizeof(pj_timer_id_t));
+    
     memcpy( new_timer_ids, ht->timer_ids, ht->max_size * sizeof(pj_timer_id_t));
-
+    
     //delete [] timer_ids_;
     ht->timer_ids = new_timer_ids;
-
+    
     // And add the new elements to the end of the "freelist".
     for (i = ht->max_size; i < new_size; i++)
 	ht->timer_ids[i] = -((pj_timer_id_t) (i + 1));
-
+    
     ht->max_size = new_size;
 }
 
@@ -286,14 +295,14 @@ static void insert_node(pj_timer_heap_t *ht, pj_timer_entry *new_node)
 {
     if (ht->cur_size + 2 >= ht->max_size)
 	grow_heap(ht);
-
+    
     reheap_up( ht, new_node, ht->cur_size, HEAP_PARENT(ht->cur_size));
     ht->cur_size++;
 }
 
 
 static pj_status_t schedule_entry( pj_timer_heap_t *ht,
-				   pj_timer_entry *entry,
+				   pj_timer_entry *entry, 
 				   const pj_time_val *future_time )
 {
     if (ht->cur_size < ht->max_size)
@@ -310,9 +319,9 @@ static pj_status_t schedule_entry( pj_timer_heap_t *ht,
 }
 
 
-static int cancel( pj_timer_heap_t *ht,
-		   pj_timer_entry *entry,
-		   int dont_call)
+static int cancel( pj_timer_heap_t *ht, 
+		   pj_timer_entry *entry, 
+		   unsigned flags)
 {
   long timer_node_slot;
 
@@ -329,14 +338,15 @@ static int cancel( pj_timer_heap_t *ht,
 
   if (entry != ht->heap[timer_node_slot])
     {
-      pj_assert(entry == ht->heap[timer_node_slot]);
+      if ((flags & F_DONT_ASSERT) == 0)
+	  pj_assert(entry == ht->heap[timer_node_slot]);
       return 0;
     }
   else
     {
       remove_node( ht, timer_node_slot);
 
-      if (dont_call == 0)
+      if ((flags & F_DONT_CALL) == 0)
         // Call the close hook.
 	(*ht->callback)(ht, entry);
       return 1;
@@ -350,7 +360,7 @@ static int cancel( pj_timer_heap_t *ht,
 PJ_DEF(pj_size_t) pj_timer_heap_mem_size(pj_size_t count)
 {
     return /* size of the timer heap itself: */
-           sizeof(pj_timer_heap_t) +
+           sizeof(pj_timer_heap_t) + 
            /* size of each entry: */
            (count+2) * (sizeof(pj_timer_entry*)+sizeof(pj_timer_id_t)) +
            /* lock, pool etc: */
@@ -392,13 +402,13 @@ PJ_DEF(pj_status_t) pj_timer_heap_create( pj_pool_t *pool,
 
     // Create the heap array.
     ht->heap = (pj_timer_entry**)
-	       pj_pool_alloc(pool, sizeof(pj_timer_entry*) * size);
+    	       pj_pool_alloc(pool, sizeof(pj_timer_entry*) * size);
     if (!ht->heap)
         return PJ_ENOMEM;
 
     // Create the parallel
     ht->timer_ids = (pj_timer_id_t *)
-		    pj_pool_alloc( pool, sizeof(pj_timer_id_t) * size);
+    		    pj_pool_alloc( pool, sizeof(pj_timer_id_t) * size);
     if (!ht->timer_ids)
         return PJ_ENOMEM;
 
@@ -451,20 +461,32 @@ PJ_DEF(pj_timer_entry*) pj_timer_entry_init( pj_timer_entry *entry,
     entry->id = id;
     entry->user_data = user_data;
     entry->cb = cb;
+    entry->_grp_lock = NULL;
 
     return entry;
 }
 
+PJ_DEF(pj_bool_t) pj_timer_entry_running( pj_timer_entry *entry )
+{
+    return (entry->_timer_id >= 1);
+}
+
 #if PJ_TIMER_DEBUG
-PJ_DEF(pj_status_t) pj_timer_heap_schedule_dbg( pj_timer_heap_t *ht,
-						pj_timer_entry *entry,
-						const pj_time_val *delay,
-						const char *src_file,
-						int src_line)
+static pj_status_t schedule_w_grp_lock_dbg(pj_timer_heap_t *ht,
+                                           pj_timer_entry *entry,
+                                           const pj_time_val *delay,
+                                           pj_bool_t set_id,
+                                           int id_val,
+					   pj_grp_lock_t *grp_lock,
+					   const char *src_file,
+					   int src_line)
 #else
-PJ_DEF(pj_status_t) pj_timer_heap_schedule( pj_timer_heap_t *ht,
-					    pj_timer_entry *entry,
-					    const pj_time_val *delay)
+static pj_status_t schedule_w_grp_lock(pj_timer_heap_t *ht,
+                                       pj_timer_entry *entry,
+                                       const pj_time_val *delay,
+                                       pj_bool_t set_id,
+                                       int id_val,
+                                       pj_grp_lock_t *grp_lock)
 #endif
 {
     pj_status_t status;
@@ -482,29 +504,103 @@ PJ_DEF(pj_status_t) pj_timer_heap_schedule( pj_timer_heap_t *ht,
 #endif
     pj_gettickcount(&expires);
     PJ_TIME_VAL_ADD(expires, *delay);
-
+    
     lock_timer_heap(ht);
     status = schedule_entry(ht, entry, &expires);
+    if (status == PJ_SUCCESS) {
+	if (set_id)
+	    entry->id = id_val;
+	entry->_grp_lock = grp_lock;
+	if (entry->_grp_lock) {
+	    pj_grp_lock_add_ref(entry->_grp_lock);
+	}
+    }
     unlock_timer_heap(ht);
 
     return status;
 }
 
-PJ_DEF(int) pj_timer_heap_cancel( pj_timer_heap_t *ht,
-				  pj_timer_entry *entry)
+
+#if PJ_TIMER_DEBUG
+PJ_DEF(pj_status_t) pj_timer_heap_schedule_dbg( pj_timer_heap_t *ht,
+						pj_timer_entry *entry,
+						const pj_time_val *delay,
+						const char *src_file,
+						int src_line)
+{
+    return schedule_w_grp_lock_dbg(ht, entry, delay, PJ_FALSE, 1, NULL,
+                                   src_file, src_line);
+}
+
+PJ_DEF(pj_status_t) pj_timer_heap_schedule_w_grp_lock_dbg(
+						pj_timer_heap_t *ht,
+						pj_timer_entry *entry,
+						const pj_time_val *delay,
+						int id_val,
+                                                pj_grp_lock_t *grp_lock,
+						const char *src_file,
+						int src_line)
+{
+    return schedule_w_grp_lock_dbg(ht, entry, delay, PJ_TRUE, id_val,
+                                   grp_lock, src_file, src_line);
+}
+
+#else
+PJ_DEF(pj_status_t) pj_timer_heap_schedule( pj_timer_heap_t *ht,
+                                            pj_timer_entry *entry,
+                                            const pj_time_val *delay)
+{
+    return schedule_w_grp_lock(ht, entry, delay, PJ_FALSE, 1, NULL);
+}
+
+PJ_DEF(pj_status_t) pj_timer_heap_schedule_w_grp_lock(pj_timer_heap_t *ht,
+                                                      pj_timer_entry *entry,
+                                                      const pj_time_val *delay,
+                                                      int id_val,
+                                                      pj_grp_lock_t *grp_lock)
+{
+    return schedule_w_grp_lock(ht, entry, delay, PJ_TRUE, id_val, grp_lock);
+}
+#endif
+
+static int cancel_timer(pj_timer_heap_t *ht,
+			pj_timer_entry *entry,
+			unsigned flags,
+			int id_val)
 {
     int count;
 
     PJ_ASSERT_RETURN(ht && entry, PJ_EINVAL);
 
     lock_timer_heap(ht);
-    count = cancel(ht, entry, 1);
+    count = cancel(ht, entry, flags | F_DONT_CALL);
+    if (flags & F_SET_ID) {
+	entry->id = id_val;
+    }
+    if (entry->_grp_lock) {
+	pj_grp_lock_t *grp_lock = entry->_grp_lock;
+	entry->_grp_lock = NULL;
+	pj_grp_lock_dec_ref(grp_lock);
+    }
     unlock_timer_heap(ht);
 
     return count;
 }
 
-PJ_DEF(unsigned) pj_timer_heap_poll( pj_timer_heap_t *ht,
+PJ_DEF(int) pj_timer_heap_cancel( pj_timer_heap_t *ht,
+				  pj_timer_entry *entry)
+{
+    return cancel_timer(ht, entry, 0, 0);
+}
+
+PJ_DEF(int) pj_timer_heap_cancel_if_active(pj_timer_heap_t *ht,
+                                           pj_timer_entry *entry,
+                                           int id_val)
+{
+    return cancel_timer(ht, entry, F_SET_ID | F_DONT_ASSERT, id_val);
+}
+
+PJ_DEF(unsigned) pj_timer_heap_poll( pj_timer_heap_t *ht, 
                                      pj_time_val *next_delay )
 {
     pj_time_val now;
@@ -512,25 +608,38 @@ PJ_DEF(unsigned) pj_timer_heap_poll( pj_timer_heap_t *ht,
 
     PJ_ASSERT_RETURN(ht, 0);
 
+    lock_timer_heap(ht);
     if (!ht->cur_size && next_delay) {
 	next_delay->sec = next_delay->msec = PJ_MAXINT32;
+        unlock_timer_heap(ht);
 	return 0;
     }
 
     count = 0;
     pj_gettickcount(&now);
 
-    lock_timer_heap(ht);
-    while ( ht->cur_size &&
+    while ( ht->cur_size && 
 	    PJ_TIME_VAL_LTE(ht->heap[0]->_timer_value, now) &&
-            count < ht->max_entries_per_poll )
+            count < ht->max_entries_per_poll ) 
     {
 	pj_timer_entry *node = remove_node(ht, 0);
+	pj_grp_lock_t *grp_lock;
+
 	++count;
 
+	grp_lock = node->_grp_lock;
+	node->_grp_lock = NULL;
+
 	unlock_timer_heap(ht);
+
+	PJ_RACE_ME(5);
+
 	if (node->cb)
 	    (*node->cb)(ht, node);
+
+	if (grp_lock)
+	    pj_grp_lock_dec_ref(grp_lock);
+
 	lock_timer_heap(ht);
     }
     if (ht->cur_size && next_delay) {
@@ -607,3 +716,4 @@ PJ_DEF(void) pj_timer_heap_dump(pj_timer_heap_t *ht)
     unlock_timer_heap(ht);
 }
 #endif
+
