@@ -1,4 +1,4 @@
-/* $Id: ioq_perf.c 3553 2011-05-05 06:14:19Z nanang $ */
+/* $Id: ioq_perf.c 4537 2013-06-19 06:47:43Z riza $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -84,7 +84,7 @@ static void on_read_complete(pj_ioqueue_key_t *key,
             return;
 
         if (bytes_read < 0) {
-            pj_status_t rc = -bytes_read;
+            pj_status_t rc = (pj_status_t)-bytes_read;
             char errmsg[PJ_ERR_MSG_SIZE];
 
 	    if (rc != last_error) {
@@ -424,7 +424,7 @@ static int perform_test(pj_bool_t allow_concur,
     /* Calculate total bytes received. */
     total_received = 0;
     for (i=0; i<sockpair_cnt; ++i) {
-        total_received = items[i].bytes_recv;
+        total_received = (pj_uint32_t)items[i].bytes_recv;
     }
 
     /* bandwidth = total_received*1000/total_elapsed_usec */
