@@ -1,5 +1,5 @@
-/* $Id: ssl_sock.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: ssl_sock.c 4537 2013-06-19 06:47:43Z riza $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include "test.h"
 #include <pjlib.h>
@@ -73,12 +73,12 @@ struct test_state
     pj_bool_t	    is_verbose;	    /* verbose flag, e.g: cert info	    */
     pj_bool_t	    echo;	    /* echo received data		    */
     pj_status_t	    err;	    /* error flag			    */
-    unsigned	    sent;	    /* bytes sent			    */
-    unsigned	    recv;	    /* bytes received			    */
+    pj_size_t	    sent;	    /* bytes sent			    */
+    pj_size_t	    recv;	    /* bytes received			    */
     pj_uint8_t	    read_buf[256];  /* read buffer			    */
     pj_bool_t	    done;	    /* test done flag			    */
     char	   *send_str;	    /* data to send once connected	    */
-    unsigned	    send_str_len;   /* send data length			    */
+    pj_size_t	    send_str_len;   /* send data length			    */
     pj_bool_t	    check_echo;	    /* flag to compare sent & echoed data   */
     const char	   *check_echo_ptr; /* pointer/cursor for comparing data    */
     struct send_key send_key;	    /* send op key			    */
@@ -95,7 +95,7 @@ static void dump_ssl_info(const pj_ssl_sock_info *si)
     PJ_LOG(3, ("", ".....Cipher: %s", tmp_st));
 
     /* Print remote certificate info and verification result */
-    if (si->remote_cert_info && si->remote_cert_info->subject.info.slen)
+    if (si->remote_cert_info && si->remote_cert_info->subject.info.slen) 
     {
 	char buf[2048];
 	const char *verif_msgs[32];
@@ -124,8 +124,8 @@ static void dump_ssl_info(const pj_ssl_sock_info *si)
 static pj_bool_t ssl_on_connect_complete(pj_ssl_sock_t *ssock,
 					 pj_status_t status)
 {
-    struct test_state *st = (struct test_state*)
-			    pj_ssl_sock_get_user_data(ssock);
+    struct test_state *st = (struct test_state*) 
+		    	    pj_ssl_sock_get_user_data(ssock);
     void *read_buf[1];
     pj_ssl_sock_info info;
     char buf1[64], buf2[64];
@@ -161,7 +161,7 @@ static pj_bool_t ssl_on_connect_complete(pj_ssl_sock_t *ssock,
 	pj_ssize_t size;
 
 	size = st->send_str_len - st->sent;
-	status = pj_ssl_sock_send(ssock, (pj_ioqueue_op_key_t*)&st->send_key,
+	status = pj_ssl_sock_send(ssock, (pj_ioqueue_op_key_t*)&st->send_key, 
 				  st->send_str + st->sent, &size, 0);
 	if (status != PJ_SUCCESS && status != PJ_EPENDING) {
 	    app_perror("...ERROR pj_ssl_sock_send()", status);
@@ -192,7 +192,7 @@ static pj_bool_t ssl_on_accept_complete(pj_ssl_sock_t *ssock,
 					const pj_sockaddr_t *src_addr,
 					int src_addr_len)
 {
-    struct test_state *parent_st = (struct test_state*)
+    struct test_state *parent_st = (struct test_state*) 
 				   pj_ssl_sock_get_user_data(ssock);
     struct test_state *st;
     void *read_buf[1];
@@ -232,7 +232,7 @@ static pj_bool_t ssl_on_accept_complete(pj_ssl_sock_t *ssock,
 	pj_ssize_t size;
 
 	size = st->send_str_len - st->sent;
-	status = pj_ssl_sock_send(newsock, (pj_ioqueue_op_key_t*)&st->send_key,
+	status = pj_ssl_sock_send(newsock, (pj_ioqueue_op_key_t*)&st->send_key, 
 				  st->send_str + st->sent, &size, 0);
 	if (status != PJ_SUCCESS && status != PJ_EPENDING) {
 	    app_perror("...ERROR pj_ssl_sock_send()", status);
@@ -262,7 +262,7 @@ static pj_bool_t ssl_on_data_read(pj_ssl_sock_t *ssock,
 				  pj_status_t status,
 				  pj_size_t *remainder)
 {
-    struct test_state *st = (struct test_state*)
+    struct test_state *st = (struct test_state*) 
 			     pj_ssl_sock_get_user_data(ssock);
 
     PJ_UNUSED_ARG(remainder);
@@ -363,7 +363,7 @@ static pj_bool_t ssl_on_data_sent(pj_ssl_sock_t *ssock,
     PJ_UNUSED_ARG(op_key);
 
     if (sent < 0) {
-	st->err = -sent;
+	st->err = (pj_status_t)-sent;
     } else {
 	st->sent += sent;
 
@@ -373,7 +373,7 @@ static pj_bool_t ssl_on_data_sent(pj_ssl_sock_t *ssock,
 	    pj_status_t status;
 
 	    size = st->send_str_len - st->sent;
-	    status = pj_ssl_sock_send(ssock, (pj_ioqueue_op_key_t*)&st->send_key,
+	    status = pj_ssl_sock_send(ssock, (pj_ioqueue_op_key_t*)&st->send_key, 
 				      st->send_str + st->sent, &size, 0);
 	    if (status != PJ_SUCCESS && status != PJ_EPENDING) {
 		app_perror("...ERROR pj_ssl_sock_send()", status);
@@ -480,7 +480,7 @@ static int https_client_test(unsigned ms_timeout)
     PJ_LOG(3, ("", ".....Sent/recv: %d/%d bytes", state.sent, state.recv));
 
 on_return:
-    if (ssock && !state.err && !state.done)
+    if (ssock && !state.err && !state.done) 
 	pj_ssl_sock_close(ssock);
     if (ioqueue)
 	pj_ioqueue_destroy(ioqueue);
@@ -551,11 +551,11 @@ static int echo_test(pj_ssl_sock_proto srv_proto, pj_ssl_sock_proto cli_proto,
     {
 	pj_str_t tmp1, tmp2, tmp3, tmp4;
 
-	status = pj_ssl_cert_load_from_files(pool,
-					     pj_strset2(&tmp1, (char*)CERT_CA_FILE),
-					     pj_strset2(&tmp2, (char*)CERT_FILE),
-					     pj_strset2(&tmp3, (char*)CERT_PRIVKEY_FILE),
-					     pj_strset2(&tmp4, (char*)CERT_PRIVKEY_PASS),
+	status = pj_ssl_cert_load_from_files(pool, 
+					     pj_strset2(&tmp1, (char*)CERT_CA_FILE), 
+					     pj_strset2(&tmp2, (char*)CERT_FILE), 
+					     pj_strset2(&tmp3, (char*)CERT_PRIVKEY_FILE), 
+					     pj_strset2(&tmp4, (char*)CERT_PRIVKEY_PASS), 
 					     &cert);
 	if (status != PJ_SUCCESS) {
 	    goto on_return;
@@ -617,7 +617,7 @@ static int echo_test(pj_ssl_sock_proto srv_proto, pj_ssl_sock_proto cli_proto,
 
 	    pj_strset2(&tmp1, (char*)CERT_CA_FILE);
 	    pj_strset2(&tmp2, NULL);
-	    status = pj_ssl_cert_load_from_files(pool,
+	    status = pj_ssl_cert_load_from_files(pool, 
 						 &tmp1, &tmp2, &tmp2, &tmp2,
 						 &cert);
 	    if (status != PJ_SUCCESS) {
@@ -672,7 +672,7 @@ static int echo_test(pj_ssl_sock_proto srv_proto, pj_ssl_sock_proto cli_proto,
 on_return:
     if (ssock_serv)
 	pj_ssl_sock_close(ssock_serv);
-    if (ssock_cli && !state_cli.err && !state_cli.done)
+    if (ssock_cli && !state_cli.err && !state_cli.done) 
 	pj_ssl_sock_close(ssock_cli);
     if (ioqueue)
 	pj_ioqueue_destroy(ioqueue);
@@ -764,7 +764,7 @@ static pj_bool_t asock_on_accept_complete(pj_activesock_t *asock,
     st = (struct test_state*) pj_activesock_get_user_data(asock);
 
     asock_cb.on_data_read = &asock_on_data_read;
-    status = pj_activesock_create(st->pool, newsock, pj_SOCK_STREAM(), NULL,
+    status = pj_activesock_create(st->pool, newsock, pj_SOCK_STREAM(), NULL, 
 				  st->ioqueue, &asock_cb, st, &new_asock);
     if (status != PJ_SUCCESS) {
 	goto on_return;
@@ -772,8 +772,8 @@ static pj_bool_t asock_on_accept_complete(pj_activesock_t *asock,
 
     /* Start reading data */
     read_buf[0] = st->read_buf;
-    status = pj_activesock_start_read2(new_asock, st->pool,
-				       sizeof(st->read_buf),
+    status = pj_activesock_start_read2(new_asock, st->pool, 
+				       sizeof(st->read_buf), 
 				       (void**)read_buf, 0);
     if (status != PJ_SUCCESS) {
 	app_perror("...ERROR pj_ssl_sock_start_read2()", status);
@@ -825,11 +825,11 @@ static int client_non_ssl(unsigned ms_timeout)
     /* Set cert */
     {
 	pj_str_t tmp1, tmp2, tmp3, tmp4;
-	status = pj_ssl_cert_load_from_files(pool,
-					     pj_strset2(&tmp1, (char*)CERT_CA_FILE),
-					     pj_strset2(&tmp2, (char*)CERT_FILE),
-					     pj_strset2(&tmp3, (char*)CERT_PRIVKEY_FILE),
-					     pj_strset2(&tmp4, (char*)CERT_PRIVKEY_PASS),
+	status = pj_ssl_cert_load_from_files(pool, 
+					     pj_strset2(&tmp1, (char*)CERT_CA_FILE), 
+					     pj_strset2(&tmp2, (char*)CERT_FILE), 
+					     pj_strset2(&tmp3, (char*)CERT_PRIVKEY_FILE), 
+					     pj_strset2(&tmp4, (char*)CERT_PRIVKEY_PASS), 
 					     &cert);
 	if (status != PJ_SUCCESS) {
 	    goto on_return;
@@ -890,13 +890,13 @@ static int client_non_ssl(unsigned ms_timeout)
 
     asock_cb.on_connect_complete = &asock_on_connect_complete;
     asock_cb.on_data_read = &asock_on_data_read;
-    status = pj_activesock_create(pool, sock, pj_SOCK_STREAM(), NULL,
+    status = pj_activesock_create(pool, sock, pj_SOCK_STREAM(), NULL, 
 				  ioqueue, &asock_cb, &state_cli, &asock_cli);
     if (status != PJ_SUCCESS) {
 	goto on_return;
     }
 
-    status = pj_activesock_start_connect(asock_cli, pool, (pj_sockaddr_t*)&listen_addr,
+    status = pj_activesock_start_connect(asock_cli, pool, (pj_sockaddr_t*)&listen_addr, 
 					 pj_sockaddr_get_len(&listen_addr));
     if (status == PJ_SUCCESS) {
 	asock_on_connect_complete(asock_cli, PJ_SUCCESS);
@@ -948,7 +948,7 @@ on_return:
 /* SSL socket try to connect to raw TCP socket server, once
  * connection established, SSL socket will try to perform SSL
  * handshake. SSL client socket should be able to close the
- * connection after specified timeout period (set ms_timeout to
+ * connection after specified timeout period (set ms_timeout to 
  * 0 to disable timer).
  */
 static int server_non_ssl(unsigned ms_timeout)
@@ -993,7 +993,7 @@ static int server_non_ssl(unsigned ms_timeout)
 	pj_sockaddr_init(PJ_AF_INET, &listen_addr, pj_strset2(&tmp_st, "127.0.0.1"), 0);
     }
 
-    status = pj_sock_bind(sock, (pj_sockaddr_t*)&listen_addr,
+    status = pj_sock_bind(sock, (pj_sockaddr_t*)&listen_addr, 
 			  pj_sockaddr_get_len((pj_sockaddr_t*)&listen_addr));
     if (status != PJ_SUCCESS) {
 	goto on_return;
@@ -1005,7 +1005,7 @@ static int server_non_ssl(unsigned ms_timeout)
     }
 
     asock_cb.on_accept_complete = &asock_on_accept_complete;
-    status = pj_activesock_create(pool, sock, pj_SOCK_STREAM(), NULL,
+    status = pj_activesock_create(pool, sock, pj_SOCK_STREAM(), NULL, 
 				  ioqueue, &asock_cb, &state_serv, &asock_serv);
     if (status != PJ_SUCCESS) {
 	goto on_return;
@@ -1050,9 +1050,9 @@ static int server_non_ssl(unsigned ms_timeout)
 	pj_sockaddr_init(PJ_AF_INET, &addr, pj_strset2(&tmp_st, "127.0.0.1"), 0);
     }
 
-    status = pj_ssl_sock_start_connect(ssock_cli, pool,
-				       (pj_sockaddr_t*)&addr,
-				       (pj_sockaddr_t*)&listen_addr,
+    status = pj_ssl_sock_start_connect(ssock_cli, pool, 
+				       (pj_sockaddr_t*)&addr, 
+				       (pj_sockaddr_t*)&listen_addr, 
 				       pj_sockaddr_get_len(&listen_addr));
     if (status != PJ_EPENDING) {
 	goto on_return;
@@ -1113,7 +1113,8 @@ static int perf_test(unsigned clients, unsigned ms_handshake_timeout)
     pj_sockaddr addr, listen_addr;
     pj_ssl_cert_t *cert = NULL;
     pj_status_t status;
-    unsigned i, cli_err = 0, tot_sent = 0, tot_recv = 0;
+    unsigned i, cli_err = 0;
+    pj_size_t tot_sent = 0, tot_recv = 0;
     pj_time_val start;
 
     pool = pj_pool_create(mem, "ssl_perf", 256, 256, NULL);
@@ -1132,11 +1133,11 @@ static int perf_test(unsigned clients, unsigned ms_handshake_timeout)
     {
 	pj_str_t tmp1, tmp2, tmp3, tmp4;
 
-	status = pj_ssl_cert_load_from_files(pool,
-					     pj_strset2(&tmp1, (char*)CERT_CA_FILE),
-					     pj_strset2(&tmp2, (char*)CERT_FILE),
-					     pj_strset2(&tmp3, (char*)CERT_PRIVKEY_FILE),
-					     pj_strset2(&tmp4, (char*)CERT_PRIVKEY_PASS),
+	status = pj_ssl_cert_load_from_files(pool, 
+					     pj_strset2(&tmp1, (char*)CERT_CA_FILE), 
+					     pj_strset2(&tmp2, (char*)CERT_FILE), 
+					     pj_strset2(&tmp3, (char*)CERT_PRIVKEY_FILE), 
+					     pj_strset2(&tmp4, (char*)CERT_PRIVKEY_PASS), 
 					     &cert);
 	if (status != PJ_SUCCESS) {
 	    goto on_return;
@@ -1294,7 +1295,7 @@ static int perf_test(unsigned clients, unsigned ms_handshake_timeout)
     /* SSL setup and data transfer duration */
     {
 	pj_time_val stop;
-
+	
 	pj_gettimeofday(&stop);
 	PJ_TIME_VAL_SUB(stop, start);
 
@@ -1314,7 +1315,7 @@ static int perf_test(unsigned clients, unsigned ms_handshake_timeout)
     PJ_LOG(3, ("", ".....Total sent/recv: %d/%d bytes", tot_sent, tot_recv));
 
 on_return:
-    if (ssock_serv)
+    if (ssock_serv) 
 	pj_ssl_sock_close(ssock_serv);
 
     for (i = 0; i < clients; ++i) {
@@ -1329,11 +1330,34 @@ on_return:
     return status;
 }
 
+#if 0 && (!defined(PJ_SYMBIAN) || PJ_SYMBIAN==0)
+pj_status_t pj_ssl_sock_ossl_test_send_buf(pj_pool_t *pool);
+static int ossl_test_send_buf()
+{
+    pj_pool_t *pool;
+    pj_status_t status;
+
+    pool = pj_pool_create(mem, "send_buf", 256, 256, NULL);
+    status = pj_ssl_sock_ossl_test_send_buf(pool);
+    pj_pool_release(pool);
+    return status;
+}
+#else
+static int ossl_test_send_buf()
+{
+    return 0;
+}
+#endif
 
 int ssl_sock_test(void)
 {
     int ret;
 
+    PJ_LOG(3,("", "..test ossl send buf"));
+    ret = ossl_test_send_buf();
+    if (ret != 0)
+	return ret;
+
     PJ_LOG(3,("", "..get cipher list test"));
     ret = get_cipher_list();
     if (ret != 0)
@@ -1346,49 +1370,49 @@ int ssl_sock_test(void)
 	//return ret;
 
 #ifndef PJ_SYMBIAN
-
-    /* On Symbian platforms, SSL socket is implemented using CSecureSocket,
+   
+    /* On Symbian platforms, SSL socket is implemented using CSecureSocket, 
      * and it hasn't supported server mode, so exclude the following tests,
      * which require SSL server, for now.
      */
 
     PJ_LOG(3,("", "..echo test w/ TLSv1 and PJ_TLS_RSA_WITH_DES_CBC_SHA cipher"));
-    ret = echo_test(PJ_SSL_SOCK_PROTO_TLS1, PJ_SSL_SOCK_PROTO_TLS1,
-		    PJ_TLS_RSA_WITH_DES_CBC_SHA, PJ_TLS_RSA_WITH_DES_CBC_SHA,
+    ret = echo_test(PJ_SSL_SOCK_PROTO_TLS1, PJ_SSL_SOCK_PROTO_TLS1, 
+		    PJ_TLS_RSA_WITH_DES_CBC_SHA, PJ_TLS_RSA_WITH_DES_CBC_SHA, 
 		    PJ_FALSE, PJ_FALSE);
     if (ret != 0)
 	return ret;
 
     PJ_LOG(3,("", "..echo test w/ SSLv23 and PJ_TLS_RSA_WITH_AES_256_CBC_SHA cipher"));
-    ret = echo_test(PJ_SSL_SOCK_PROTO_SSL23, PJ_SSL_SOCK_PROTO_SSL23,
+    ret = echo_test(PJ_SSL_SOCK_PROTO_SSL23, PJ_SSL_SOCK_PROTO_SSL23, 
 		    PJ_TLS_RSA_WITH_AES_256_CBC_SHA, PJ_TLS_RSA_WITH_AES_256_CBC_SHA,
 		    PJ_FALSE, PJ_FALSE);
     if (ret != 0)
 	return ret;
 
     PJ_LOG(3,("", "..echo test w/ incompatible proto"));
-    ret = echo_test(PJ_SSL_SOCK_PROTO_TLS1, PJ_SSL_SOCK_PROTO_SSL3,
+    ret = echo_test(PJ_SSL_SOCK_PROTO_TLS1, PJ_SSL_SOCK_PROTO_SSL3, 
 		    PJ_TLS_RSA_WITH_DES_CBC_SHA, PJ_TLS_RSA_WITH_DES_CBC_SHA,
 		    PJ_FALSE, PJ_FALSE);
     if (ret == 0)
 	return PJ_EBUG;
 
     PJ_LOG(3,("", "..echo test w/ incompatible ciphers"));
-    ret = echo_test(PJ_SSL_SOCK_PROTO_DEFAULT, PJ_SSL_SOCK_PROTO_DEFAULT,
+    ret = echo_test(PJ_SSL_SOCK_PROTO_DEFAULT, PJ_SSL_SOCK_PROTO_DEFAULT, 
 		    PJ_TLS_RSA_WITH_DES_CBC_SHA, PJ_TLS_RSA_WITH_AES_256_CBC_SHA,
 		    PJ_FALSE, PJ_FALSE);
     if (ret == 0)
 	return PJ_EBUG;
 
     PJ_LOG(3,("", "..echo test w/ client cert required but not provided"));
-    ret = echo_test(PJ_SSL_SOCK_PROTO_DEFAULT, PJ_SSL_SOCK_PROTO_DEFAULT,
+    ret = echo_test(PJ_SSL_SOCK_PROTO_DEFAULT, PJ_SSL_SOCK_PROTO_DEFAULT, 
 		    PJ_TLS_RSA_WITH_AES_256_CBC_SHA, PJ_TLS_RSA_WITH_AES_256_CBC_SHA,
 		    PJ_TRUE, PJ_FALSE);
     if (ret == 0)
 	return PJ_EBUG;
 
     PJ_LOG(3,("", "..echo test w/ client cert required and provided"));
-    ret = echo_test(PJ_SSL_SOCK_PROTO_DEFAULT, PJ_SSL_SOCK_PROTO_DEFAULT,
+    ret = echo_test(PJ_SSL_SOCK_PROTO_DEFAULT, PJ_SSL_SOCK_PROTO_DEFAULT, 
 		    PJ_TLS_RSA_WITH_AES_256_CBC_SHA, PJ_TLS_RSA_WITH_AES_256_CBC_SHA,
 		    PJ_TRUE, PJ_TRUE);
     if (ret != 0)
@@ -1417,7 +1441,8 @@ int ssl_sock_test(void)
 
 #else	/* INCLUDE_SSLSOCK_TEST */
 /* To prevent warning about "translation unit is empty"
- * when this test is disabled.
+ * when this test is disabled. 
  */
 int dummy_ssl_sock_test;
 #endif	/* INCLUDE_SSLSOCK_TEST */
+
