@@ -1,4 +1,4 @@
-/* $Id: config.h 4414 2013-03-05 08:21:02Z riza $ */
+/* $Id: config.h 4656 2013-11-23 07:13:40Z bennylp $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -52,6 +52,8 @@ PJ_BEGIN_DECL
 #endif
 
 
+#if defined(PJMEDIA_HAS_VIDEO) && (PJMEDIA_HAS_VIDEO != 0)
+
 /**
  * This setting controls whether SDL support should be included.
  *
@@ -144,6 +146,8 @@ PJ_BEGIN_DECL
 #   undef PJMEDIA_SDL_LIB
 #endif
 
+#endif /* defined(PJMEDIA_HAS_VIDEO) && (PJMEDIA_HAS_VIDEO != 0) */
+
 /**
  * @}
  */
