@@ -1,5 +1,5 @@
-/* $Id: transport.h 3664 2011-07-19 03:42:28Z nanang $ */
-/*
+/* $Id: transport.h 4345 2013-02-13 07:43:32Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJMEDIA_TRANSPORT_H__
 #define __PJMEDIA_TRANSPORT_H__
@@ -36,7 +36,7 @@
  * @{
  * The media transport (#pjmedia_transport) is the object to send and
  * receive media packets over the network. The media transport interface
- * allows the library to be extended to support different types of
+ * allows the library to be extended to support different types of 
  * transports to send and receive packets.
  *
  * The media transport is declared as #pjmedia_transport "class", which
@@ -44,7 +44,7 @@
  * structure. For the user of the media transport (normally the user of
  * media transport is media stream, see \ref PJMED_STRM), these transport
  * "methods" are wrapped with API such as #pjmedia_transport_attach(),
- * so it should not need to call the function pointer inside
+ * so it should not need to call the function pointer inside 
  * #pjmedia_transport_op directly.
  *
  * The connection between \ref PJMED_STRM and media transport is shown in
@@ -88,27 +88,27 @@
  *    two endpoints, and it may contain more than one media streams, for
  *    example, an audio stream and a video stream).
  *
- *  When stream is created, it will "attach" itself to the media
+ *  When stream is created, it will "attach" itself to the media 
  *    transport by calling #pjmedia_transport_attach(), which is a thin
- *    wrapper which calls "attach()" method of the media transport's
+ *    wrapper which calls "attach()" method of the media transport's 
  *    "virtual function pointer" (#pjmedia_transport_op). Among other things,
  *    the stream specifies two callback functions to the transport: one
  *    callback function will be called by transport when it receives RTP
- *    packet, and another callback for incoming RTCP packet. The
+ *    packet, and another callback for incoming RTCP packet. The 
  *    #pjmedia_transport_attach() function also establish the destination
  *    of the outgoing RTP and RTCP packets.
  *
  *  When the stream needs to send outgoing RTP/RTCP packets, it will
  *    call #pjmedia_transport_send_rtp() and #pjmedia_transport_send_rtcp()
- *    of the media transport API, which is a thin wrapper to call send_rtp()
- *    and send_rtcp() methods in the media transport's "virtual function
+ *    of the media transport API, which is a thin wrapper to call send_rtp() 
+ *    and send_rtcp() methods in the media transport's "virtual function 
  *    pointer"  (#pjmedia_transport_op).
  *
  *  When the stream is destroyed, it will "detach" itself from
  *    the media transport by calling #pjmedia_transport_detach(), which is
- *    a thin wrapper which calls "detach()" method of the media transport's
+ *    a thin wrapper which calls "detach()" method of the media transport's 
  *    "virtual function pointer" (#pjmedia_transport_op). After the transport
- *    is detached from its user (the stream), it will no longer report
+ *    is detached from its user (the stream), it will no longer report 
  *    incoming RTP/RTCP packets to the stream, and it will refuse to send
  *    outgoing packets since the destination has been cleared.
  *
@@ -122,18 +122,18 @@
  *    closed and re-opened). However it is up to the application to choose
  *    which method is most suitable for its uses.
  *
- *
+ * 
  * \subsection PJMEDIA_TRANSPORT_H_DESTROY Destroying the Media Transport.
  *
  *  Finally if application no longer needs the media transport, it will
- *    call #pjmedia_transport_close() function, which is thin wrapper which
- *    calls "destroy()" method of the media transport's  "virtual function
+ *    call #pjmedia_transport_close() function, which is thin wrapper which 
+ *    calls "destroy()" method of the media transport's  "virtual function 
  *    pointer" (#pjmedia_transport_op). This function releases
  *    all resources used by the transport, such as sockets and memory.
  *
  *
  * \section offer_answer Interaction with SDP Offer/Answer
-
+ 
    For basic UDP transport, the \ref PJMEDIA_TRANSPORT_H_USING above is
    sufficient to use the media transport. However, more complex media
    transports such as \ref PJMEDIA_TRANSPORT_SRTP and \ref
@@ -149,8 +149,8 @@
        have been negotiated by SDP offer answer, and
      - #pjmedia_transport_media_stop(), to deinitialize the media transport
        and reset the transport to its idle state.
-
-   The usage of these API in the context of SDP offer answer will be
+   
+   The usage of these API in the context of SDP offer answer will be 
    described below.
 
    \subsection media_create Initializing Transport for New Session
@@ -170,7 +170,7 @@
    SDP to the invite session to be sent to remote agent.
 
    When creating an answer, also call #pjmedia_transport_encode_sdp(),
-   but this time specify both local and remote SDP to the function. The
+   but this time specify both local and remote SDP to the function. The 
    media transport will once again modify the local SDP and add relevant
    attributes to the local SDP, if the appropriate attributes related to
    the transport functionality are present in remote offer. The remote
@@ -183,9 +183,9 @@
 
    \subsection media_start Offer/Answer Completion
 
-   Once both local and remote SDP have been negotiated by the
+   Once both local and remote SDP have been negotiated by the 
    \ref PJMEDIA_SDP_NEG (normally this is part of PJSIP invite session),
-   application should give both local and remote SDP to
+   application should give both local and remote SDP to 
    #pjmedia_transport_media_start() so that the settings are activated
    for the session. This function should be called for both initial and
    subsequent SDP negotiation.
@@ -203,7 +203,7 @@
  * To implement a new type of media transport, one needs to "subclass" the
  * media transport "class" (#pjmedia_transport) by providing the "methods"
  * in the media transport "interface" (#pjmedia_transport_op), and provides
- * a function to create this new type of transport (similar to
+ * a function to create this new type of transport (similar to 
  * #pjmedia_transport_udp_create() function).
  *
  * The media transport is expected to run indepently, that is there should
@@ -249,7 +249,7 @@ typedef enum pjmedia_tranport_media_option
      * When this flag is specified, the transport will not perform media
      * transport validation, this is useful when transport is stacked with
      * other transport, for example when transport UDP is stacked under
-     * transport SRTP, media transport validation only need to be done by
+     * transport SRTP, media transport validation only need to be done by 
      * transport SRTP.
      */
     PJMEDIA_TPMED_NO_TRANSPORT_CHECKING = 1
@@ -302,10 +302,10 @@ struct pjmedia_transport_op
     /**
      * This function is called by the stream when the transport is about
      * to be used by the stream for the first time, and it tells the transport
-     * about remote RTP address to send the packet and some callbacks to be
+     * about remote RTP address to send the packet and some callbacks to be 
      * called for incoming packets.
      *
-     * Application should call #pjmedia_transport_attach() instead of
+     * Application should call #pjmedia_transport_attach() instead of 
      * calling this function directly.
      */
     pj_status_t (*attach)(pjmedia_transport *tp,
@@ -325,20 +325,20 @@ struct pjmedia_transport_op
      * needs the transport (normally when the stream is about to be closed).
      * After the transport is detached, it will ignore incoming
      * RTP/RTCP packets, and will refuse to send outgoing RTP/RTCP packets.
-     * Application may re-attach the media transport to another transport
+     * Application may re-attach the media transport to another transport 
      * user (e.g. stream) after the transport has been detached.
      *
-     * Application should call #pjmedia_transport_detach() instead of
+     * Application should call #pjmedia_transport_detach() instead of 
      * calling this function directly.
      */
     void (*detach)(pjmedia_transport *tp,
 		   void *user_data);
 
     /**
-     * This function is called by the stream to send RTP packet using the
+     * This function is called by the stream to send RTP packet using the 
      * transport.
      *
-     * Application should call #pjmedia_transport_send_rtp() instead of
+     * Application should call #pjmedia_transport_send_rtp() instead of 
      * calling this function directly.
      */
     pj_status_t (*send_rtp)(pjmedia_transport *tp,
@@ -349,7 +349,7 @@ struct pjmedia_transport_op
      * This function is called by the stream to send RTCP packet using the
      * transport.
      *
-     * Application should call #pjmedia_transport_send_rtcp() instead of
+     * Application should call #pjmedia_transport_send_rtcp() instead of 
      * calling this function directly.
      */
     pj_status_t (*send_rtcp)(pjmedia_transport *tp,
@@ -361,7 +361,7 @@ struct pjmedia_transport_op
      * transport with destination address other than default specified in
      * #pjmedia_transport_attach().
      *
-     * Application should call #pjmedia_transport_send_rtcp2() instead of
+     * Application should call #pjmedia_transport_send_rtcp2() instead of 
      * calling this function directly.
      */
     pj_status_t (*send_rtcp2)(pjmedia_transport *tp,
@@ -373,7 +373,7 @@ struct pjmedia_transport_op
     /**
      * Prepare the transport for a new media session.
      *
-     * Application should call #pjmedia_transport_media_create() instead of
+     * Application should call #pjmedia_transport_media_create() instead of 
      * calling this function directly.
      */
     pj_status_t (*media_create)(pjmedia_transport *tp,
@@ -399,7 +399,7 @@ struct pjmedia_transport_op
      * This function is called by application to start the transport
      * based on local and remote SDP.
      *
-     * Application should call #pjmedia_transport_media_start() instead of
+     * Application should call #pjmedia_transport_media_start() instead of 
      * calling this function directly.
      */
     pj_status_t (*media_start) (pjmedia_transport *tp,
@@ -411,7 +411,7 @@ struct pjmedia_transport_op
     /**
      * This function is called by application to stop the transport.
      *
-     * Application should call #pjmedia_transport_media_stop() instead of
+     * Application should call #pjmedia_transport_media_stop() instead of 
      * calling this function directly.
      */
     pj_status_t (*media_stop)  (pjmedia_transport *tp);
@@ -419,7 +419,7 @@ struct pjmedia_transport_op
     /**
      * This function can be called to simulate packet lost.
      *
-     * Application should call #pjmedia_transport_simulate_lost() instead of
+     * Application should call #pjmedia_transport_simulate_lost() instead of 
      * calling this function directly.
      */
     pj_status_t (*simulate_lost)(pjmedia_transport *tp,
@@ -429,7 +429,7 @@ struct pjmedia_transport_op
     /**
      * This function can be called to destroy this transport.
      *
-     * Application should call #pjmedia_transport_close() instead of
+     * Application should call #pjmedia_transport_close() instead of 
      * calling this function directly.
      */
     pj_status_t (*destroy)(pjmedia_transport *tp);
@@ -442,7 +442,7 @@ struct pjmedia_transport_op
 typedef struct pjmedia_transport_op pjmedia_transport_op;
 
 
-/**
+/** 
  * Media transport type.
  */
 typedef enum pjmedia_transport_type
@@ -453,7 +453,7 @@ typedef enum pjmedia_transport_type
     /** Media transport using ICE */
     PJMEDIA_TRANSPORT_TYPE_ICE,
 
-    /**
+    /** 
      * Media transport SRTP, this transport is actually security adapter to be
      * stacked with other transport to enable encryption on the underlying
      * transport.
@@ -515,8 +515,8 @@ typedef struct pjmedia_transport_specific_info
 
 
 /**
- * This structure describes transport informations, including general
- * socket information and specific information of single transport or
+ * This structure describes transport informations, including general 
+ * socket information and specific information of single transport or 
  * stacked transports (e.g: SRTP stacked on top of UDP)
  */
 struct pjmedia_transport_info
@@ -528,7 +528,7 @@ struct pjmedia_transport_info
 
     /**
      * Remote address where RTP/RTCP originated from. In case this transport
-     * hasn't ever received packet, the
+     * hasn't ever received packet, the 
      */
     pj_sockaddr	    src_rtp_name;
     pj_sockaddr	    src_rtcp_name;
@@ -560,13 +560,13 @@ PJ_INLINE(void) pjmedia_transport_info_init(pjmedia_transport_info *info)
 
 
 /**
- * Get media transport info from the specified transport and all underlying
+ * Get media transport info from the specified transport and all underlying 
  * transports if any. The transport also contains information about socket info
  * which describes the local address of the transport, and would be needed
  * for example to fill in the "c=" and "m=" line of local SDP.
  *
  * @param tp	    The transport.
- * @param info	    Media socket info to be initialized.
+ * @param info	    Media transport info to be initialized.
  *
  * @return	    PJ_SUCCESS on success.
  */
@@ -575,18 +575,41 @@ PJ_INLINE(pj_status_t) pjmedia_transport_get_info(pjmedia_transport *tp,
 {
     if (tp && tp->op && tp->op->get_info)
 	return (*tp->op->get_info)(tp, info);
-
+    
     return PJ_ENOTSUP;
 }
 
 
+/**
+ * Utility API to get transport type specific info from the specified media
+ * transport info.
+ * 
+ * @param info	    Media transport info.
+ * @param type	    Media transport type.
+ *
+ * @return	    Pointer to media transport specific info, or NULL if
+ * 		    specific info for the transport type is not found.
+ */
+PJ_INLINE(void*) pjmedia_transport_info_get_spc_info(
+						pjmedia_transport_info *info,
+						pjmedia_transport_type type)
+{
+    unsigned i;
+    for (i = 0; i < info->specific_info_cnt; ++i) {
+	if (info->spc_info[i].type == type)
+	    return (void*)info->spc_info[i].buffer;
+    }
+    return NULL;
+}
+
+
 /**
  * Attach callbacks to be called on receipt of incoming RTP/RTCP packets.
- * This is just a simple wrapper which calls <tt>attach()</tt> member of
+ * This is just a simple wrapper which calls <tt>attach()</tt> member of 
  * the transport.
  *
  * @param tp	    The media transport.
- * @param user_data Arbitrary user data to be set when the callbacks are
+ * @param user_data Arbitrary user data to be set when the callbacks are 
  *		    called.
  * @param rem_addr  Remote RTP address to send RTP packet to.
  * @param rem_rtcp  Optional remote RTCP address. If the argument is NULL
@@ -613,14 +636,14 @@ PJ_INLINE(pj_status_t) pjmedia_transport_attach(pjmedia_transport *tp,
 							        void*pkt,
 							        pj_ssize_t))
 {
-    return tp->op->attach(tp, user_data, rem_addr, rem_rtcp, addr_len,
+    return tp->op->attach(tp, user_data, rem_addr, rem_rtcp, addr_len, 
 			  rtp_cb, rtcp_cb);
 }
 
 
 /**
  * Detach callbacks from the transport.
- * This is just a simple wrapper which calls <tt>detach()</tt> member of
+ * This is just a simple wrapper which calls <tt>detach()</tt> member of 
  * the transport. After the transport is detached, it will ignore incoming
  * RTP/RTCP packets, and will refuse to send outgoing RTP/RTCP packets.
  * Application may re-attach the media transport to another transport user
@@ -639,7 +662,7 @@ PJ_INLINE(void) pjmedia_transport_detach(pjmedia_transport *tp,
 
 /**
  * Send RTP packet with the specified media transport. This is just a simple
- * wrapper which calls <tt>send_rtp()</tt> member of the transport. The
+ * wrapper which calls <tt>send_rtp()</tt> member of the transport. The 
  * RTP packet will be delivered to the destination address specified in
  * #pjmedia_transport_attach() function.
  *
@@ -659,7 +682,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_send_rtp(pjmedia_transport *tp,
 
 /**
  * Send RTCP packet with the specified media transport. This is just a simple
- * wrapper which calls <tt>send_rtcp()</tt> member of the transport. The
+ * wrapper which calls <tt>send_rtcp()</tt> member of the transport. The 
  * RTCP packet will be delivered to the destination address specified in
  * #pjmedia_transport_attach() function.
  *
@@ -679,9 +702,9 @@ PJ_INLINE(pj_status_t) pjmedia_transport_send_rtcp(pjmedia_transport *tp,
 
 /**
  * Send RTCP packet with the specified media transport. This is just a simple
- * wrapper which calls <tt>send_rtcp2()</tt> member of the transport. The
+ * wrapper which calls <tt>send_rtcp2()</tt> member of the transport. The 
  * RTCP packet will be delivered to the destination address specified in
- * param addr, if addr is NULL, RTCP packet will be delivered to destination
+ * param addr, if addr is NULL, RTCP packet will be delivered to destination 
  * address specified in #pjmedia_transport_attach() function.
  *
  * @param tp	    The media transport.
@@ -707,7 +730,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_send_rtcp2(pjmedia_transport *tp,
  * call this function before starting a new media session using this
  * transport.
  *
- * This is just a simple wrapper which calls <tt>media_create()</tt> member
+ * This is just a simple wrapper which calls <tt>media_create()</tt> member 
  * of the transport.
  *
  * @param tp		The media transport.
@@ -726,7 +749,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_media_create(pjmedia_transport *tp,
 				    const pjmedia_sdp_session *rem_sdp,
 				    unsigned media_index)
 {
-    return (*tp->op->media_create)(tp, sdp_pool, options, rem_sdp,
+    return (*tp->op->media_create)(tp, sdp_pool, options, rem_sdp, 
 				   media_index);
 }
 
@@ -736,7 +759,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_media_create(pjmedia_transport *tp,
  * called to put transport specific information in the initial or
  * subsequent SDP offer or answer.
  *
- * This is just a simple wrapper which calls <tt>encode_sdp()</tt> member
+ * This is just a simple wrapper which calls <tt>encode_sdp()</tt> member 
  * of the transport.
  *
  * @param tp		The media transport.
@@ -761,13 +784,13 @@ PJ_INLINE(pj_status_t) pjmedia_transport_encode_sdp(pjmedia_transport *tp,
 
 
 /**
- * Start the transport session with the settings in both local and remote
- * SDP. The actual work that is done by this function depends on the
+ * Start the transport session with the settings in both local and remote 
+ * SDP. The actual work that is done by this function depends on the 
  * underlying transport type. For SRTP, this will activate the encryption
  * and decryption based on the keys found the SDPs. For ICE, this will
  * start ICE negotiation according to the information found in the SDPs.
  *
- * This is just a simple wrapper which calls <tt>media_start()</tt> member
+ * This is just a simple wrapper which calls <tt>media_start()</tt> member 
  * of the transport.
  *
  * @param tp		The media transport.
@@ -784,7 +807,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_media_start(pjmedia_transport *tp,
 				    const pjmedia_sdp_session *sdp_remote,
 				    unsigned media_index)
 {
-    return (*tp->op->media_start)(tp, tmp_pool, sdp_local, sdp_remote,
+    return (*tp->op->media_start)(tp, tmp_pool, sdp_local, sdp_remote, 
 				  media_index);
 }
 
@@ -793,7 +816,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_media_start(pjmedia_transport *tp,
  * This API should be called when the session is stopped, to allow the media
  * transport to release its resources used for the session.
  *
- * This is just a simple wrapper which calls <tt>media_stop()</tt> member
+ * This is just a simple wrapper which calls <tt>media_stop()</tt> member 
  * of the transport.
  *
  * @param tp		The media transport.
@@ -806,7 +829,7 @@ PJ_INLINE(pj_status_t) pjmedia_transport_media_stop(pjmedia_transport *tp)
 }
 
 /**
- * Close media transport. This is just a simple wrapper which calls
+ * Close media transport. This is just a simple wrapper which calls 
  * <tt>destroy()</tt> member of the transport. This function will free
  * all resources created by this transport (such as sockets, memory, etc.).
  *
@@ -850,3 +873,4 @@ PJ_END_DECL
 
 
 #endif	/* __PJMEDIA_TRANSPORT_H__ */
+
