@@ -1,5 +1,5 @@
-/* $Id: audiodev.c 4150 2012-06-01 04:29:56Z ming $ */
-/*
+/* $Id: audiodev.c 4435 2013-03-11 06:32:58Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia-audiodev/audiodev_imp.h>
 #include <pj/assert.h>
@@ -33,7 +33,7 @@ static struct cap_info
 {
     const char *name;
     const char *info;
-} cap_infos[] =
+} cap_infos[] = 
 {
     DEFINE_CAP("ext-fmt",     "Extended/non-PCM format"),
     DEFINE_CAP("latency-in",  "Input latency/buffer size setting"),
@@ -53,7 +53,7 @@ static struct cap_info
 
 
 /*
- * The device index seen by application and driver is different.
+ * The device index seen by application and driver is different. 
  *
  * At application level, device index is index to global list of device.
  * At driver level, device index is index to device list on that particular
@@ -78,6 +78,14 @@ pjmedia_aud_dev_factory* pjmedia_coreaudio_factory(pj_pool_factory *pf);
 pjmedia_aud_dev_factory* pjmedia_alsa_factory(pj_pool_factory *pf);
 #endif
 
+#if PJMEDIA_AUDIO_DEV_HAS_OPENSL
+pjmedia_aud_dev_factory* pjmedia_opensl_factory(pj_pool_factory *pf);
+#endif
+
+#if PJMEDIA_AUDIO_DEV_HAS_ANDROID_JNI
+pjmedia_aud_dev_factory* pjmedia_android_factory(pj_pool_factory *pf);
+#endif
+
 #if PJMEDIA_AUDIO_DEV_HAS_BB10
 pjmedia_aud_dev_factory* pjmedia_bb10_factory(pj_pool_factory *pf);
 #endif
@@ -86,6 +94,10 @@ pjmedia_aud_dev_factory* pjmedia_bb10_factory(pj_pool_factory *pf);
 pjmedia_aud_dev_factory* pjmedia_wmme_factory(pj_pool_factory *pf);
 #endif
 
+#if PJMEDIA_AUDIO_DEV_HAS_BDIMAD
+pjmedia_aud_dev_factory* pjmedia_bdimad_factory(pj_pool_factory *pf);
+#endif
+
 #if PJMEDIA_AUDIO_DEV_HAS_SYMB_VAS
 pjmedia_aud_dev_factory* pjmedia_symb_vas_factory(pj_pool_factory *pf);
 #endif
@@ -330,8 +342,8 @@ static pj_status_t init_driver(unsigned drv_idx, pj_bool_t refresh)
 	    drv->dev_idx = i;
 	}
 
-	if (drv->play_dev_idx >= 0 && drv->rec_dev_idx >= 0 &&
-	    drv->dev_idx >= 0)
+	if (drv->play_dev_idx >= 0 && drv->rec_dev_idx >= 0 && 
+	    drv->dev_idx >= 0) 
 	{
 	    /* Done. */
 	    break;
@@ -380,8 +392,8 @@ PJ_DEF(pj_status_t) pjmedia_aud_subsys_init(pj_pool_factory *pf)
     }
 
     /* Register error subsystem */
-    status = pj_register_strerror(PJMEDIA_AUDIODEV_ERRNO_START,
-				  PJ_ERRNO_SPACE_SIZE,
+    status = pj_register_strerror(PJMEDIA_AUDIODEV_ERRNO_START, 
+				  PJ_ERRNO_SPACE_SIZE, 
 				  &pjmedia_audiodev_strerror);
     pj_assert(status == PJ_SUCCESS);
 
@@ -391,6 +403,12 @@ PJ_DEF(pj_status_t) pjmedia_aud_subsys_init(pj_pool_factory *pf)
     aud_subsys.dev_cnt = 0;
 
     /* Register creation functions */
+#if PJMEDIA_AUDIO_DEV_HAS_OPENSL
+    aud_subsys.drv[aud_subsys.drv_cnt++].create = &pjmedia_opensl_factory;
+#endif
+#if PJMEDIA_AUDIO_DEV_HAS_ANDROID_JNI
+    aud_subsys.drv[aud_subsys.drv_cnt++].create = &pjmedia_android_factory;
+#endif
 #if PJMEDIA_AUDIO_DEV_HAS_BB10
     aud_subsys.drv[aud_subsys.drv_cnt++].create = &pjmedia_bb10_factory;
 #endif
@@ -406,6 +424,9 @@ PJ_DEF(pj_status_t) pjmedia_aud_subsys_init(pj_pool_factory *pf)
 #if PJMEDIA_AUDIO_DEV_HAS_WMME
     aud_subsys.drv[aud_subsys.drv_cnt++].create = &pjmedia_wmme_factory;
 #endif
+#if PJMEDIA_AUDIO_DEV_HAS_BDIMAD
+    aud_subsys.drv[aud_subsys.drv_cnt++].create = &pjmedia_bdimad_factory;
+#endif
 #if PJMEDIA_AUDIO_DEV_HAS_SYMB_VAS
     aud_subsys.drv[aud_subsys.drv_cnt++].create = &pjmedia_symb_vas_factory;
 #endif
@@ -511,11 +532,11 @@ PJ_DEF(pj_status_t) pjmedia_aud_subsys_shutdown(void)
 PJ_DEF(pj_status_t) pjmedia_aud_dev_refresh(void)
 {
     unsigned i;
-
+    
     aud_subsys.dev_cnt = 0;
     for (i=0; i<aud_subsys.drv_cnt; ++i) {
 	struct driver *drv = &aud_subsys.drv[i];
-
+	
 	if (drv->f && drv->f->op->refresh) {
 	    pj_status_t status = drv->f->op->refresh(drv->f);
 	    if (status != PJ_SUCCESS) {
@@ -535,7 +556,7 @@ PJ_DEF(unsigned) pjmedia_aud_dev_count(void)
 }
 
 /* Internal: convert local index to global device index */
-static pj_status_t make_global_index(unsigned drv_idx,
+static pj_status_t make_global_index(unsigned drv_idx, 
 				     pjmedia_aud_dev_index *id)
 {
     if (*id < 0) {
@@ -546,7 +567,7 @@ static pj_status_t make_global_index(unsigned drv_idx,
     PJ_ASSERT_RETURN(aud_subsys.drv[drv_idx].f, PJ_EBUG);
 
     /* Check that device index is valid */
-    PJ_ASSERT_RETURN(*id>=0 && *id<(int)aud_subsys.drv[drv_idx].dev_cnt,
+    PJ_ASSERT_RETURN(*id>=0 && *id<(int)aud_subsys.drv[drv_idx].dev_cnt, 
 		     PJ_EBUG);
 
     *id += aud_subsys.drv[drv_idx].start_idx;
@@ -572,14 +593,14 @@ static pj_status_t lookup_dev(pjmedia_aud_dev_index id,
 		id = drv->dev_idx;
 		make_global_index(i, &id);
 		break;
-	    } else if (id==PJMEDIA_AUD_DEFAULT_CAPTURE_DEV &&
-		drv->rec_dev_idx >= 0)
+	    } else if (id==PJMEDIA_AUD_DEFAULT_CAPTURE_DEV && 
+		drv->rec_dev_idx >= 0) 
 	    {
 		id = drv->rec_dev_idx;
 		make_global_index(i, &id);
 		break;
-	    } else if (id==PJMEDIA_AUD_DEFAULT_PLAYBACK_DEV &&
-		drv->play_dev_idx >= 0)
+	    } else if (id==PJMEDIA_AUD_DEFAULT_PLAYBACK_DEV && 
+		drv->play_dev_idx >= 0) 
 	    {
 		id = drv->play_dev_idx;
 		make_global_index(i, &id);
@@ -750,7 +771,7 @@ PJ_DEF(pj_status_t) pjmedia_aud_stream_create(const pjmedia_aud_param *prm,
     PJ_ASSERT_RETURN(f != NULL, PJ_EBUG);
 
     /* For now, rec_id and play_id must belong to the same factory */
-    PJ_ASSERT_RETURN((param.dir != PJMEDIA_DIR_CAPTURE_PLAYBACK) ||
+    PJ_ASSERT_RETURN((param.dir != PJMEDIA_DIR_CAPTURE_PLAYBACK) || 
 		     (rec_f == play_f),
 		     PJMEDIA_EAUD_INVDEV);
 
@@ -818,3 +839,5 @@ PJ_DEF(pj_status_t) pjmedia_aud_stream_destroy(pjmedia_aud_stream *strm)
 {
     return strm->op->destroy(strm);
 }
+
+
