@@ -1,4 +1,4 @@
-/* $Id: dshow_dev.c 3953 2012-02-16 08:49:33Z ming $ */
+/* $Id: dshow_dev.c 4722 2014-01-29 10:40:40Z nanang $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -23,7 +23,8 @@
 #include <pj/unicode.h>
 
 
-#if defined(PJMEDIA_VIDEO_DEV_HAS_DSHOW) && PJMEDIA_VIDEO_DEV_HAS_DSHOW != 0
+#if defined(PJMEDIA_HAS_VIDEO) && PJMEDIA_HAS_VIDEO != 0 && \
+    defined(PJMEDIA_VIDEO_DEV_HAS_DSHOW) && PJMEDIA_VIDEO_DEV_HAS_DSHOW != 0
 
 
 #ifdef _MSC_VER
@@ -33,6 +34,7 @@
 #include <windows.h>
 #define COBJMACROS
 #include <DShow.h>
+#include <wmsdkidl.h>
 
 #ifdef _MSC_VER
 #   pragma warning(pop)
@@ -64,14 +66,16 @@ typedef struct dshow_fmt_info
 {
     pjmedia_format_id    pjmedia_format;
     const GUID          *dshow_format;
+    pj_bool_t            enabled;
 } dshow_fmt_info;
 
 static dshow_fmt_info dshow_fmts[] =
 {
-    {PJMEDIA_FORMAT_YUY2, &MEDIASUBTYPE_YUY2} ,
-    {PJMEDIA_FORMAT_RGB24, &MEDIASUBTYPE_RGB24} ,
-    {PJMEDIA_FORMAT_RGB32, &MEDIASUBTYPE_RGB32} ,
-    {PJMEDIA_FORMAT_IYUV, &MEDIASUBTYPE_IYUV} ,
+    {PJMEDIA_FORMAT_YUY2, &MEDIASUBTYPE_YUY2, PJ_FALSE} ,
+    {PJMEDIA_FORMAT_RGB24, &MEDIASUBTYPE_RGB24, PJ_FALSE} ,
+    {PJMEDIA_FORMAT_RGB32, &MEDIASUBTYPE_RGB32, PJ_FALSE} ,
+    {PJMEDIA_FORMAT_IYUV, &MEDIASUBTYPE_IYUV, PJ_FALSE} ,
+    {PJMEDIA_FORMAT_I420, &WMMEDIASUBTYPE_I420, PJ_FALSE}
 };
 
 /* dshow_ device info */
@@ -317,18 +321,20 @@ static void enum_dev_cap(IBaseFilter *filter,
 				   sizeof(dshow_fmts)/sizeof(dshow_fmts[0]));
 			for (j = 0; j < nformat; j++) {
 			    const GUID *dshow_format = dshow_fmt;
-
+                            
 			    if (!dshow_format)
 				dshow_format = dshow_fmts[j].dshow_format;
-			    if (UuidCompare(&mediatype->subtype,
+			    if (UuidCompare(&mediatype->subtype, 
 					    (UUID*)dshow_format,
-					    &rpcstatus) == 0 &&
+					    &rpcstatus) == 0 && 
 				rpcstatus == RPC_S_OK &&
 				UuidCompare(&mediatype->formattype,
 					    (UUID*)&FORMAT_VideoInfo,
 					    &rpcstatus2) == 0 &&
 				rpcstatus2 == RPC_S_OK)
 			    {
+                                if (!dshow_fmt)
+                                    dshow_fmts[j].enabled = PJ_TRUE;
 				if (sup_fmt)
 				    sup_fmt[j] = PJ_TRUE;
 				if (pSrcpin) {
@@ -379,7 +385,7 @@ static pj_status_t dshow_factory_refresh(pjmedia_vid_dev_factory *f)
                           (void**)&dev_enum);
     if (FAILED(hr) ||
         ICreateDevEnum_CreateClassEnumerator(dev_enum,
-            &CLSID_VideoInputDeviceCategory, &enum_cat, 0) != S_OK)
+            &CLSID_VideoInputDeviceCategory, &enum_cat, 0) != S_OK) 
     {
 	PJ_LOG(4,(THIS_FILE, "Windows found no video input devices"));
         if (dev_enum)
@@ -394,8 +400,8 @@ static pj_status_t dshow_factory_refresh(pjmedia_vid_dev_factory *f)
     /* Add renderer device */
     dev_count += 1;
     df->dev_info = (struct dshow_dev_info*)
-		   pj_pool_calloc(df->dev_pool, dev_count,
-				  sizeof(struct dshow_dev_info));
+ 		   pj_pool_calloc(df->dev_pool, dev_count,
+ 				  sizeof(struct dshow_dev_info));
 
     if (dev_count > 1) {
         IEnumMoniker_Reset(enum_cat);
@@ -429,7 +435,7 @@ static pj_status_t dshow_factory_refresh(pjmedia_vid_dev_factory *f)
                         CoTaskMemFree(wszDisplayName);
                     }
 
-                    strncpy(ddi->info.driver, "dshow",
+                    strncpy(ddi->info.driver, "dshow", 
                             sizeof(ddi->info.driver));
                     ddi->info.driver[sizeof(ddi->info.driver)-1] = '\0';
                     ddi->info.dir = PJMEDIA_DIR_CAPTURE;
@@ -455,8 +461,8 @@ static pj_status_t dshow_factory_refresh(pjmedia_vid_dev_factory *f)
 				continue;
 			    pjmedia_format_init_video(
 				&ddi->info.fmt[ddi->info.fmt_cnt++],
-				dshow_fmts[j].pjmedia_format,
-				DEFAULT_WIDTH, DEFAULT_HEIGHT,
+				dshow_fmts[j].pjmedia_format, 
+				DEFAULT_WIDTH, DEFAULT_HEIGHT, 
 				DEFAULT_FPS, 1);
 			}
 		    }
@@ -487,15 +493,15 @@ static pj_status_t dshow_factory_refresh(pjmedia_vid_dev_factory *f)
 //    ddi->info.caps |= PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW;
 
     ddi->info.fmt_cnt = 1;
-    pjmedia_format_init_video(&ddi->info.fmt[0], dshow_fmts[0].pjmedia_format,
-			      DEFAULT_WIDTH, DEFAULT_HEIGHT,
+    pjmedia_format_init_video(&ddi->info.fmt[0], dshow_fmts[0].pjmedia_format, 
+			      DEFAULT_WIDTH, DEFAULT_HEIGHT, 
 			      DEFAULT_FPS, 1);
 #endif
 
-    PJ_LOG(4, (THIS_FILE, "DShow has %d devices:",
+    PJ_LOG(4, (THIS_FILE, "DShow has %d devices:", 
 	       df->dev_count));
     for (c = 0; c < df->dev_count; ++c) {
-	PJ_LOG(4, (THIS_FILE, " dev_id %d: %s (%s)",
+	PJ_LOG(4, (THIS_FILE, " dev_id %d: %s (%s)", 
 	       c,
 	       df->dev_info[c].info.name,
 	       df->dev_info[c].info.dir & PJMEDIA_DIR_CAPTURE ?
@@ -575,7 +581,7 @@ static void input_cb(void *user_data, IMediaSample *pMediaSample)
     {
         pj_status_t status;
 
-	status = pj_thread_register("ds_cap", strm->cap_thread_desc,
+	status = pj_thread_register("ds_cap", strm->cap_thread_desc, 
 				    &strm->cap_thread);
         if (status != PJ_SUCCESS)
             return;
@@ -594,7 +600,7 @@ static void input_cb(void *user_data, IMediaSample *pMediaSample)
         unsigned i, stride;
         BYTE *src_buf, *dst_buf;
         pjmedia_video_format_detail *vfd;
-
+        
         /* Image is bottom-up, convert it to top-down. */
         src_buf = dst_buf = (BYTE *)frame.buf;
         stride = strm->frm_buf_size;
@@ -628,7 +634,7 @@ static pj_status_t dshow_stream_put_frame(pjmedia_vid_dev_stream *strm,
     }
 
     hr = SourceFilter_Deliver(stream->dgraph.csource_filter,
-                              frame->buf, frame->size);
+                              frame->buf, (long)frame->size);
     if (FAILED(hr))
         return hr;
 
@@ -640,7 +646,7 @@ static dshow_fmt_info* get_dshow_format_info(pjmedia_format_id id)
     unsigned i;
 
     for (i = 0; i < sizeof(dshow_fmts)/sizeof(dshow_fmts[0]); i++) {
-        if (dshow_fmts[i].pjmedia_format == id)
+        if (dshow_fmts[i].pjmedia_format == id && dshow_fmts[i].enabled)
             return &dshow_fmts[i];
     }
 
@@ -772,7 +778,7 @@ static pj_status_t create_filter_graph(pjmedia_dir dir,
     if (video_info->AvgTimePerFrame == 0 ||
         (!use_def_fps && vfd->fps.num != 0))
     {
-        video_info->AvgTimePerFrame = (LONGLONG) (10000000 *
+        video_info->AvgTimePerFrame = (LONGLONG) (10000000 * 
 						  (double)vfd->fps.denum /
 						  vfd->fps.num);
     }
@@ -821,7 +827,7 @@ on_error:
     if (FAILED(hr)) {
 	char msg[80];
 	if (AMGetErrorText(hr, msg, sizeof(msg))) {
-	    PJ_LOG(4,(THIS_FILE, "Error creating filter graph: %s (hr=0x%x)",
+	    PJ_LOG(4,(THIS_FILE, "Error creating filter graph: %s (hr=0x%x)", 
 		      msg, hr));
 	}
         return PJ_EUNKNOWN;
@@ -910,7 +916,7 @@ static pj_status_t dshow_factory_create_stream(
                 goto on_error;
             pj_memcpy(param, &strm->param, sizeof(*param));
         }
-
+	
 	vfd = pjmedia_format_get_video_format_detail(&param->fmt, PJ_TRUE);
 	strm->cap_ts_inc = PJMEDIA_SPF2(param->clock_rate, &vfd->fps, 1);
     } else if (param->dir & PJMEDIA_DIR_RENDER) {
@@ -934,7 +940,7 @@ static pj_status_t dshow_factory_create_stream(
     *p_vid_strm = &strm->base;
 
     return PJ_SUCCESS;
-
+ 
 on_error:
     dshow_stream_destroy((pjmedia_vid_dev_stream *)strm);
     return status;
