@@ -1,4 +1,4 @@
-/* $Id: ios_dev.m 3979 2012-03-20 08:55:33Z ming $ */
+/* $Id: ios_dev.m 4722 2014-01-29 10:40:40Z nanang $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -21,7 +21,9 @@
 #include <pj/log.h>
 #include <pj/os.h>
 
-#if PJMEDIA_VIDEO_DEV_HAS_IOS
+#if defined(PJMEDIA_HAS_VIDEO) && PJMEDIA_HAS_VIDEO != 0 && \
+    defined(PJMEDIA_VIDEO_DEV_HAS_IOS) && PJMEDIA_VIDEO_DEV_HAS_IOS != 0
+
 #include "Availability.h"
 #ifdef __IPHONE_4_0
 
@@ -62,7 +64,7 @@ struct ios_factory
     struct ios_dev_info		*dev_info;
 };
 
-@interface VOutDelegate: NSObject
+@interface VOutDelegate: NSObject 
 			 <AVCaptureVideoDataOutputSampleBufferDelegate>
 {
 @public
@@ -84,16 +86,16 @@ struct ios_stream
     pj_uint8_t		    bpp;
     unsigned		    bytes_per_row;
     unsigned		    frame_size;
-
+    
     AVCaptureSession		*cap_session;
     AVCaptureDeviceInput	*dev_input;
     AVCaptureVideoDataOutput	*video_output;
     VOutDelegate		*vout_delegate;
-
+    
     UIImageView		*imgView;
     void		*buf;
     dispatch_queue_t     render_queue;
-
+    
     pj_timestamp	 frame_ts;
     unsigned		 ts_inc;
 };
@@ -184,48 +186,48 @@ static pj_status_t ios_factory_init(pjmedia_vid_dev_factory *f)
     struct ios_factory *qf = (struct ios_factory*)f;
     struct ios_dev_info *qdi;
     unsigned i, l;
-
+    
     /* Initialize input and output devices here */
     qf->dev_info = (struct ios_dev_info*)
 		   pj_pool_calloc(qf->pool, 2,
 				  sizeof(struct ios_dev_info));
-
+    
     qf->dev_count = 0;
     qdi = &qf->dev_info[qf->dev_count++];
     pj_bzero(qdi, sizeof(*qdi));
     strcpy(qdi->info.name, "iOS UIView");
-    strcpy(qdi->info.driver, "iOS");
+    strcpy(qdi->info.driver, "iOS");	    
     qdi->info.dir = PJMEDIA_DIR_RENDER;
     qdi->info.has_callback = PJ_FALSE;
     qdi->info.caps = PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW;
-
+    
     if (NSClassFromString(@"AVCaptureSession")) {
 	qdi = &qf->dev_info[qf->dev_count++];
 	pj_bzero(qdi, sizeof(*qdi));
 	strcpy(qdi->info.name, "iOS AVCapture");
-	strcpy(qdi->info.driver, "iOS");
+	strcpy(qdi->info.driver, "iOS");	    
 	qdi->info.dir = PJMEDIA_DIR_CAPTURE;
 	qdi->info.has_callback = PJ_TRUE;
     }
 
     for (i = 0; i < qf->dev_count; i++) {
 	qdi = &qf->dev_info[i];
-	qdi->info.fmt_cnt = PJ_ARRAY_SIZE(ios_fmts);
+	qdi->info.fmt_cnt = PJ_ARRAY_SIZE(ios_fmts);	    
 	qdi->info.caps |= PJMEDIA_VID_DEV_CAP_FORMAT;
-
+	
 	for (l = 0; l < PJ_ARRAY_SIZE(ios_fmts); l++) {
 	    pjmedia_format *fmt = &qdi->info.fmt[l];
 	    pjmedia_format_init_video(fmt,
 				      ios_fmts[l].pjmedia_format,
 				      DEFAULT_WIDTH,
 				      DEFAULT_HEIGHT,
-				      DEFAULT_FPS, 1);
+				      DEFAULT_FPS, 1);	
 	}
     }
-
+    
     PJ_LOG(4, (THIS_FILE, "iOS video initialized with %d devices",
 	       qf->dev_count));
-
+    
     return PJ_SUCCESS;
 }
 
@@ -294,7 +296,7 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
     } else {
 	return PJMEDIA_EVID_INVDEV;
     }
-
+    
     param->flags = PJMEDIA_VID_DEV_CAP_FORMAT;
     param->clock_rate = DEFAULT_CLOCK_RATE;
     pj_memcpy(&param->fmt, &di->info.fmt[0], sizeof(param->fmt));
@@ -304,47 +306,47 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
 
 @implementation VOutDelegate
 - (void)update_image
-{
+{    
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-
+    
     /* Create a device-dependent RGB color space */
-    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
-
+    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB(); 
+    
     /* Create a bitmap graphics context with the sample buffer data */
-    CGContextRef context =
+    CGContextRef context = 
 	CGBitmapContextCreate(stream->buf, stream->size.w, stream->size.h, 8,
 			      stream->bytes_per_row, colorSpace,
 			      kCGBitmapByteOrder32Little |
 			      kCGImageAlphaPremultipliedFirst);
-
+    
     /**
      * Create a Quartz image from the pixel data in the bitmap graphics
      * context
      */
-    CGImageRef quartzImage = CGBitmapContextCreateImage(context);
-
+    CGImageRef quartzImage = CGBitmapContextCreateImage(context); 
+    
     /* Free up the context and color space */
-    CGContextRelease(context);
+    CGContextRelease(context); 
     CGColorSpaceRelease(colorSpace);
-
+    
     /* Create an image object from the Quartz image */
-    UIImage *image = [UIImage imageWithCGImage:quartzImage scale:1.0
+    UIImage *image = [UIImage imageWithCGImage:quartzImage scale:1.0 
 			      orientation:UIImageOrientationRight];
-
+    
     /* Release the Quartz image */
     CGImageRelease(quartzImage);
-
+    
     dispatch_async(dispatch_get_main_queue(),
                    ^{[stream->imgView setImage:image];});
     /*
     [stream->imgView performSelectorOnMainThread:@selector(setImage:)
 		     withObject:image waitUntilDone:NO];
      */
-
+    
     [pool release];
-}
+}    
 
-- (void)captureOutput:(AVCaptureOutput *)captureOutput
+- (void)captureOutput:(AVCaptureOutput *)captureOutput 
 		      didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
 		      fromConnection:(AVCaptureConnection *)connection
 {
@@ -353,24 +355,24 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
 
     if (!sampleBuffer)
 	return;
-
+    
     /* Get a CMSampleBuffer's Core Video image buffer for the media data */
-    imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
-
+    imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer); 
+    
     /* Lock the base address of the pixel buffer */
-    CVPixelBufferLockBaseAddress(imageBuffer, 0);
-
+    CVPixelBufferLockBaseAddress(imageBuffer, 0); 
+    
     frame.type = PJMEDIA_FRAME_TYPE_VIDEO;
     frame.buf = CVPixelBufferGetBaseAddress(imageBuffer);
     frame.size = stream->frame_size;
     frame.bit_info = 0;
     frame.timestamp.u64 = stream->frame_ts.u64;
-
+    
     if (stream->vid_cb.capture_cb)
         (*stream->vid_cb.capture_cb)(&stream->base, stream->user_data, &frame);
 
     stream->frame_ts.u64 += stream->ts_inc;
-
+    
     /* Unlock the pixel buffer */
     CVPixelBufferUnlockBaseAddress(imageBuffer,0);
 }
@@ -379,12 +381,12 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
 static ios_fmt_info* get_ios_format_info(pjmedia_format_id id)
 {
     unsigned i;
-
+    
     for (i = 0; i < PJ_ARRAY_SIZE(ios_fmts); i++) {
         if (ios_fmts[i].pjmedia_format == id)
             return &ios_fmts[i];
     }
-
+    
     return NULL;
 }
 
@@ -414,7 +416,7 @@ static pj_status_t ios_factory_create_stream(
 
     if (!(ifi = get_ios_format_info(param->fmt.id)))
         return PJMEDIA_EVID_BADFORMAT;
-
+    
     vfi = pjmedia_get_video_format_info(NULL, param->fmt.id);
     if (!vfi)
         return PJMEDIA_EVID_BADFORMAT;
@@ -444,17 +446,17 @@ static pj_status_t ios_factory_create_stream(
 	    goto on_error;
 	}
 	strm->cap_session.sessionPreset = AVCaptureSessionPresetMedium;
-
+	
 	/* Open video device */
-	AVCaptureDevice *videoDevice =
+	AVCaptureDevice *videoDevice = 
 	    [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
 	if (!videoDevice) {
 	    status = PJMEDIA_EVID_SYSERR;
 	    goto on_error;
 	}
-
-	/* Add the video device to the session as a device input */
-	strm->dev_input = [AVCaptureDeviceInput
+	
+	/* Add the video device to the session as a device input */	
+	strm->dev_input = [AVCaptureDeviceInput 
 			   deviceInputWithDevice:videoDevice
 			   error: &error];
 	if (!strm->dev_input) {
@@ -462,7 +464,7 @@ static pj_status_t ios_factory_create_stream(
 	    goto on_error;
 	}
 	[strm->cap_session addInput:strm->dev_input];
-
+	
 	strm->video_output = [[[AVCaptureVideoDataOutput alloc] init]
 			      autorelease];
 	if (!strm->video_output) {
@@ -470,15 +472,15 @@ static pj_status_t ios_factory_create_stream(
 	    goto on_error;
 	}
 	[strm->cap_session addOutput:strm->video_output];
-
+	
 	/* Configure the video output */
 	strm->vout_delegate = [VOutDelegate alloc];
 	strm->vout_delegate->stream = strm;
 	dispatch_queue_t queue = dispatch_queue_create("myQueue", NULL);
 	[strm->video_output setSampleBufferDelegate:strm->vout_delegate
 			    queue:queue];
-	dispatch_release(queue);
-
+	dispatch_release(queue);	
+	
 	strm->video_output.videoSettings =
 	    [NSDictionary dictionaryWithObjectsAndKeys:
 			  [NSNumber numberWithInt:ifi->ios_format],
@@ -488,16 +490,16 @@ static pj_status_t ios_factory_create_stream(
 			  [NSNumber numberWithInt: vfd->size.h],
 			  kCVPixelBufferHeightKey, nil];
 	strm->video_output.minFrameDuration = CMTimeMake(vfd->fps.denum,
-							 vfd->fps.num);
+							 vfd->fps.num);	
     } else if (param->dir & PJMEDIA_DIR_RENDER) {
         /* Create renderer stream here */
 	/* Get the main window */
 	UIWindow *window = [[UIApplication sharedApplication] keyWindow];
-
+	
 	if (param->flags & PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW &&
             param->window.info.ios.window)
 	    window = (UIWindow *)param->window.info.ios.window;
-
+	
 	pj_assert(window);
 	strm->imgView = [[UIImageView alloc] initWithFrame:[window bounds]];
 	if (!strm->imgView) {
@@ -505,22 +507,22 @@ static pj_status_t ios_factory_create_stream(
 	    goto on_error;
 	}
 	[window addSubview:strm->imgView];
-
+	
 	if (!strm->vout_delegate) {
 	    strm->vout_delegate = [VOutDelegate alloc];
 	    strm->vout_delegate->stream = strm;
 	}
-
+        
         strm->render_queue = dispatch_queue_create("com.pjsip.render_queue",
                                                    NULL);
         if (!strm->render_queue)
             goto on_error;
-
+	
 	strm->buf = pj_pool_alloc(pool, strm->frame_size);
-    }
-
+    }    
+    
     /* Apply the remaining settings */
-    /*
+    /*    
      if (param->flags & PJMEDIA_VID_DEV_CAP_INPUT_SCALE) {
 	ios_stream_set_cap(&strm->base,
 			  PJMEDIA_VID_DEV_CAP_INPUT_SCALE,
@@ -530,12 +532,12 @@ static pj_status_t ios_factory_create_stream(
     /* Done */
     strm->base.op = &stream_op;
     *p_vid_strm = &strm->base;
-
+    
     return PJ_SUCCESS;
-
+    
 on_error:
     ios_stream_destroy((pjmedia_vid_dev_stream *)strm);
-
+    
     return status;
 }
 
@@ -608,11 +610,11 @@ static pj_status_t ios_stream_start(pjmedia_vid_dev_stream *strm)
 
     if (stream->cap_session) {
 	[stream->cap_session startRunning];
-
+    
 	if (![stream->cap_session isRunning])
 	    return PJ_EUNKNOWN;
     }
-
+    
     return PJ_SUCCESS;
 }
 
@@ -623,20 +625,20 @@ static pj_status_t ios_stream_put_frame(pjmedia_vid_dev_stream *strm,
 {
     struct ios_stream *stream = (struct ios_stream*)strm;
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-
+    
     pj_assert(stream->frame_size >= frame->size);
     pj_memcpy(stream->buf, frame->buf, frame->size);
     /* Perform video display in a background thread */
-/*
+/*   
     [stream->vout_delegate update_image];
     [NSThread detachNewThreadSelector:@selector(update_image)
 	      toTarget:stream->vout_delegate withObject:nil];
 */
     dispatch_async(stream->render_queue,
                    ^{[stream->vout_delegate update_image];});
-
+    
     [pool release];
-
+    
     return PJ_SUCCESS;
 }
 
@@ -651,7 +653,7 @@ static pj_status_t ios_stream_stop(pjmedia_vid_dev_stream *strm)
 
     if (stream->cap_session && [stream->cap_session isRunning])
 	[stream->cap_session stopRunning];
-
+    
     return PJ_SUCCESS;
 }
 
@@ -664,7 +666,7 @@ static pj_status_t ios_stream_destroy(pjmedia_vid_dev_stream *strm)
     PJ_ASSERT_RETURN(stream != NULL, PJ_EINVAL);
 
     ios_stream_stop(strm);
-
+    
     if (stream->imgView) {
 	[stream->imgView removeFromSuperview];
 	[stream->imgView release];
@@ -674,12 +676,12 @@ static pj_status_t ios_stream_destroy(pjmedia_vid_dev_stream *strm)
     if (stream->cap_session) {
 	[stream->cap_session release];
 	stream->cap_session = NULL;
-    }
+    }    
 /*    if (stream->dev_input) {
 	[stream->dev_input release];
 	stream->dev_input = NULL;
     }
-*/
+*/ 
     if (stream->vout_delegate) {
 	[stream->vout_delegate release];
 	stream->vout_delegate = NULL;
