@@ -1,4 +1,4 @@
-/* $Id: sdl_dev.c 4157 2012-06-06 09:37:25Z nanang $ */
+/* $Id: sdl_dev.c 4722 2014-01-29 10:40:40Z nanang $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -21,7 +21,8 @@
 #include <pj/log.h>
 #include <pj/os.h>
 
-#if defined(PJMEDIA_VIDEO_DEV_HAS_SDL) && PJMEDIA_VIDEO_DEV_HAS_SDL != 0
+#if defined(PJMEDIA_HAS_VIDEO) && PJMEDIA_HAS_VIDEO != 0 && \
+    defined(PJMEDIA_VIDEO_DEV_HAS_SDL) && PJMEDIA_VIDEO_DEV_HAS_SDL != 0
 
 #include <SDL.h>
 #include <SDL_syswm.h>
@@ -261,6 +262,13 @@ static pjmedia_vid_dev_stream_op stream_op =
     &sdl_stream_destroy
 };
 
+/*
+ * Util
+ */
+static void sdl_log_err(const char *op)
+{
+    PJ_LOG(1,(THIS_FILE, "%s error: %s", op, SDL_GetError()));
+}
 
 /****************************************************************************
  * Factory operations
@@ -287,7 +295,7 @@ static pj_status_t sdl_init(void * data)
     PJ_UNUSED_ARG(data);
 
     if (SDL_Init(SDL_INIT_VIDEO)) {
-        PJ_LOG(3, (THIS_FILE, "Failed initializing SDL"));
+        sdl_log_err("SDL_Init()");
         return PJMEDIA_EVID_INIT;
     }
 
@@ -309,7 +317,7 @@ static struct sdl_stream* find_stream(struct sdl_factory *sf,
             break;
         }
     }
-
+ 
     if (strm)
         pjmedia_event_init(pevent, PJMEDIA_EVENT_NONE, &strm->last_ts,
 		           strm);
@@ -441,6 +449,8 @@ static pj_status_t sdl_factory_init(pjmedia_vid_dev_factory *f)
     pj_status_t status;
     SDL_version version;
 
+    pj_list_init(&sf->streams);
+
     status = job_queue_create(sf->pool, &sf->jq);
     if (status != PJ_SUCCESS)
         return PJMEDIA_EVID_INIT;
@@ -449,7 +459,6 @@ static pj_status_t sdl_factory_init(pjmedia_vid_dev_factory *f)
     if (status != PJ_SUCCESS)
         return status;
 
-    pj_list_init(&sf->streams);
     status = pj_mutex_create_recursive(sf->pool, "sdl_factory",
 				       &sf->mutex);
     if (status != PJ_SUCCESS)
@@ -593,7 +602,7 @@ static pj_status_t sdl_factory_default_param(pj_pool_t *pool,
     struct sdl_dev_info *di = &sf->dev_info[index];
 
     PJ_ASSERT_RETURN(index < sf->dev_count, PJMEDIA_EVID_INVDEV);
-
+    
     PJ_UNUSED_ARG(pool);
 
     pj_bzero(param, sizeof(*param));
@@ -705,7 +714,7 @@ static pj_status_t sdl_create_rend(struct sdl_stream * strm,
 
     if (!strm->window) {
         Uint32 flags = 0;
-
+        
         if (strm->param.flags & PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW_FLAGS) {
             if (!(strm->param.window_flags & PJMEDIA_VID_DEV_WND_BORDER))
                 flags |= SDL_WINDOW_BORDERLESS;
@@ -733,6 +742,10 @@ static pj_status_t sdl_create_rend(struct sdl_stream * strm,
             /* Use the window supplied by the application. */
 	    strm->window = SDL_CreateWindowFrom(
                                strm->param.window.info.window);
+	    if (!strm->window) {
+		sdl_log_err("SDL_CreateWindowFrom()");
+		return PJMEDIA_EVID_SYSERR;
+	    }
         } else {
             int x, y;
 
@@ -748,9 +761,11 @@ static pj_status_t sdl_create_rend(struct sdl_stream * strm,
                                             strm->param.disp_size.w,
                                             strm->param.disp_size.h,
                                             flags);
+	    if (!strm->window) {
+		sdl_log_err("SDL_CreateWindow()");
+		return PJMEDIA_EVID_SYSERR;
+	    }
         }
-        if (!strm->window)
-            return PJMEDIA_EVID_SYSERR;
     }
 
     /**
@@ -758,32 +773,36 @@ static pj_status_t sdl_create_rend(struct sdl_stream * strm,
       * affect this window.
       */
     strm->renderer = SDL_CreateRenderer(strm->window, -1, 0);
-    if (!strm->renderer)
+    if (!strm->renderer) {
+	sdl_log_err("SDL_CreateRenderer()");
         return PJMEDIA_EVID_SYSERR;
+    }
 
 #if PJMEDIA_VIDEO_DEV_SDL_HAS_OPENGL
     if (strm->param.rend_id == OPENGL_DEV_IDX) {
         strm->gl_context = SDL_GL_CreateContext(strm->window);
-        if (!strm->gl_context)
+        if (!strm->gl_context) {
+            sdl_log_err("SDL_GL_CreateContext()");
             return PJMEDIA_EVID_SYSERR;
+        }
         SDL_GL_MakeCurrent(strm->window, strm->gl_context);
 
         /* Init some OpenGL settings */
 	glDisable(GL_DEPTH_TEST);
 	glDisable(GL_CULL_FACE);
 	glEnable(GL_TEXTURE_2D);
-
+	
 	/* Init the viewport */
 	glViewport(0, 0, strm->param.disp_size.w, strm->param.disp_size.h);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-
+	
 	glOrtho(0.0, (GLdouble)strm->param.disp_size.w,
                 (GLdouble)strm->param.disp_size.h, 0.0, 0.0, 1.0);
-
+	
 	glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity();
-
+	
 	/* Create a texture */
 	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
 	glGenTextures(1, &strm->texture);
@@ -792,13 +811,15 @@ static pj_status_t sdl_create_rend(struct sdl_stream * strm,
             return PJMEDIA_EVID_SYSERR;
     } else
 #endif /* PJMEDIA_VIDEO_DEV_SDL_HAS_OPENGL */
-    {
+    {    
         strm->scr_tex = SDL_CreateTexture(strm->renderer, sdl_info->sdl_format,
                                           SDL_TEXTUREACCESS_STREAMING,
                                           strm->rect.w, strm->rect.h);
-        if (strm->scr_tex == NULL)
+        if (strm->scr_tex == NULL) {
+            sdl_log_err("SDL_CreateTexture()");
             return PJMEDIA_EVID_SYSERR;
-
+        }
+    
         strm->pitch = strm->rect.w * SDL_BYTESPERPIXEL(sdl_info->sdl_format);
     }
 
@@ -816,7 +837,7 @@ static pj_status_t resize_disp(struct sdl_stream *strm,
 {
     pj_memcpy(&strm->param.disp_size, new_disp_size,
               sizeof(strm->param.disp_size));
-
+    
     if (strm->scr_tex) {
         strm->dstrect.x = strm->dstrect.y = 0;
         strm->dstrect.w = (Uint16)strm->param.disp_size.w;
@@ -898,7 +919,7 @@ static pj_status_t sdl_stream_put_frame(pjmedia_vid_dev_stream *strm,
 
     stream->frame = frame;
     job_queue_post_job(stream->sf->jq, put_frame, strm, 0, &status);
-
+    
     return status;
 }
 
@@ -1123,7 +1144,7 @@ static pj_status_t set_cap(void *data)
         status = change_format(strm, (pjmedia_format *)pval);
 	if (status != PJ_SUCCESS) {
 	    pj_status_t status_;
-
+	    
 	    /**
 	     * Failed to change the output format. Try to revert
 	     * to its original format.
@@ -1137,7 +1158,7 @@ static pj_status_t set_cap(void *data)
 		status = PJMEDIA_EVID_ERR;
 	    }
 	}
-
+	
 	return status;
     } else if (cap == PJMEDIA_VID_DEV_CAP_OUTPUT_RESIZE) {
 	pjmedia_rect_size *new_size = (pjmedia_rect_size *)pval;
@@ -1350,7 +1371,7 @@ static pj_status_t job_queue_post_job(job_queue *jq, job_func_ptr func,
     JQDelegate *jqd = [[JQDelegate alloc]init];
     jqd->pjob = &jb;
     [jqd performSelectorOnMainThread:@selector(run_job)
-	 withObject:nil waitUntilDone:YES];
+ 	 withObject:nil waitUntilDone:YES];
     [jqd release];
     [apool release];
 #else /* PJ_DARWINOS */
@@ -1418,7 +1439,9 @@ static pj_status_t job_queue_destroy(job_queue *jq)
 }
 
 #ifdef _MSC_VER
-#   if SDL_VERSION_ATLEAST(2,0,0)
+#   if defined(PJMEDIA_SDL_LIB)
+#	pragma comment( lib, PJMEDIA_SDL_LIB)
+#   elif SDL_VERSION_ATLEAST(2,0,0)
 #	pragma comment( lib, "sdl2.lib")
 #   elif SDL_VERSION_ATLEAST(1,3,0)
 #	pragma comment( lib, "sdl.lib")
