@@ -1,4 +1,4 @@
-/* $Id: v4l2_dev.c 3901 2011-12-07 10:43:28Z nanang $ */
+/* $Id: v4l2_dev.c 4736 2014-02-06 11:06:42Z bennylp $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -25,7 +25,8 @@
 #include <pj/os.h>
 #include <pj/rand.h>
 
-#if PJMEDIA_VIDEO_DEV_HAS_V4L2
+#if defined(PJMEDIA_HAS_VIDEO) && PJMEDIA_HAS_VIDEO != 0 && \
+    defined(PJMEDIA_VIDEO_DEV_HAS_V4L2) && PJMEDIA_VIDEO_DEV_HAS_V4L2 != 0
 
 #include <linux/videodev2.h>
 #include <libv4l2.h>
@@ -556,6 +557,7 @@ static pj_status_t vid4lin_factory_create_stream(pjmedia_vid_dev_factory *f,
 
     vdi = &cf->dev_info[param->cap_id];
     vfd = pjmedia_format_get_video_format_detail(&param->fmt, PJ_TRUE);
+    PJ_UNUSED_ARG(vfd); /* Unused for now */
 
     /* Create and Initialize stream descriptor */
     pool = pj_pool_create(cf->pf, vdi->info.name, 512, 512, NULL);
@@ -670,6 +672,7 @@ static pj_status_t vid4lin_stream_get_frame_mmap(vid4lin_stream *stream,
     struct v4l2_buffer buf;
     pj_time_val time;
     pj_status_t status = PJ_SUCCESS;
+    unsigned tmp_idx;
 
     pj_bzero(&buf, sizeof(buf));
     buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -697,9 +700,11 @@ static pj_status_t vid4lin_stream_get_frame_mmap(vid4lin_stream *stream,
     pj_memcpy(frame->buf, stream->buffers[buf.index].start, buf.bytesused);
 
 on_return:
+    tmp_idx = buf.index;
     pj_bzero(&buf, sizeof(buf));
     buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     buf.memory = V4L2_MEMORY_MMAP;
+    buf.index = tmp_idx;
     xioctl(stream->fd, VIDIOC_QBUF, &buf);
 
     return status;
