@@ -1,5 +1,5 @@
-/* $Id: tonegen.c 3664 2011-07-19 03:42:28Z nanang $ */
-/*
+/* $Id: tonegen.c 4537 2013-06-19 06:47:43Z riza $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia/tonegen.h>
 #include <pjmedia/errno.h>
@@ -61,7 +61,7 @@
     #define DATA	float
 
     /*
-     * Default floating-point based tone generation using sine wave
+     * Default floating-point based tone generation using sine wave 
      * generation from:
      *   http://www.musicdsp.org/showone.php?id=10.
      * This produces good quality tone in relatively faster time than
@@ -104,14 +104,14 @@
     #define CORDIC_NTAB		28
     #define CORDIC_LOOP		PJMEDIA_TONEGEN_FIXED_POINT_CORDIC_LOOP
 
-    static int cordic_ctab [] =
+    static int cordic_ctab [] = 
     {
-	0x03243F6A, 0x01DAC670, 0x00FADBAF, 0x007F56EA, 0x003FEAB7,
-	0x001FFD55, 0x000FFFAA, 0x0007FFF5, 0x0003FFFE, 0x0001FFFF,
-	0x0000FFFF, 0x00007FFF, 0x00003FFF, 0x00001FFF, 0x00000FFF,
-	0x000007FF, 0x000003FF, 0x000001FF, 0x000000FF, 0x0000007F,
-	0x0000003F, 0x0000001F, 0x0000000F, 0x00000007, 0x00000003,
-	0x00000001, 0x00000000, 0x00000000
+	0x03243F6A, 0x01DAC670, 0x00FADBAF, 0x007F56EA, 0x003FEAB7, 
+	0x001FFD55, 0x000FFFAA, 0x0007FFF5, 0x0003FFFE, 0x0001FFFF, 
+	0x0000FFFF, 0x00007FFF, 0x00003FFF, 0x00001FFF, 0x00000FFF, 
+	0x000007FF, 0x000003FF, 0x000001FF, 0x000000FF, 0x0000007F, 
+	0x0000003F, 0x0000001F, 0x0000000F, 0x00000007, 0x00000003, 
+	0x00000001, 0x00000000, 0x00000000 
     };
 
     static pj_int32_t cordic(pj_int32_t theta, unsigned n)
@@ -132,7 +132,7 @@
 	    y = y + (((x>>k) ^ d) - d);
 	    z = z - ((cordic_ctab[k] ^ d) - d);
 	    x = tx;
-	}
+	}  
 	return y;
     }
 
@@ -187,11 +187,11 @@
 
 #elif PJMEDIA_TONEGEN_ALG==PJMEDIA_TONEGEN_FAST_FIXED_POINT
 
-    /*
+    /* 
      * Fallback algorithm when floating point is disabled.
      * This is a very fast fixed point tone generation using sine wave
      * approximation from
-     *    http://www.audiomulch.com/~rossb/code/sinusoids/
+     *    http://www.audiomulch.com/~rossb/code/sinusoids/ 
      * Quality wise not so good, but it's blazing fast!
      * Speed = 117 usec to generate 1 second, 8KHz dual-tones (2.66GHz P4).
      *         approx. 0.95 MIPS
@@ -199,7 +199,7 @@
      *         1,449 usec/0.29 MIPS on ARM926EJ-S.
      */
     PJ_INLINE(int) approximate_sin3(unsigned x)
-    {
+    {	
 	    unsigned s=-(int)(x>>31);
 	    x+=x;
 	    x=x>>16;
@@ -234,7 +234,7 @@ struct gen_state
 
 
 static void init_generate_single_tone(struct gen_state *state,
-				      unsigned clock_rate,
+				      unsigned clock_rate, 
 				      unsigned freq,
 				      unsigned vol)
 {
@@ -245,7 +245,7 @@ static void init_generate_single_tone(struct gen_state *state,
 static void generate_single_tone(struct gen_state *state,
 				 unsigned channel_count,
 				 unsigned samples,
-				 short buf[])
+				 short buf[]) 
 {
     short *end = buf + samples;
 
@@ -267,7 +267,7 @@ static void generate_single_tone(struct gen_state *state,
 
 
 static void init_generate_dual_tone(struct gen_state *state,
-				    unsigned clock_rate,
+				    unsigned clock_rate, 
 				    unsigned freq1,
 				    unsigned freq2,
 				    unsigned vol)
@@ -281,7 +281,7 @@ static void init_generate_dual_tone(struct gen_state *state,
 static void generate_dual_tone(struct gen_state *state,
 			       unsigned channel_count,
 			       unsigned samples,
-			       short buf[])
+			       short buf[]) 
 {
     short *end = buf + samples;
 
@@ -308,7 +308,7 @@ static void generate_dual_tone(struct gen_state *state,
 
 
 static void init_generate_tone(struct gen_state *state,
-			       unsigned clock_rate,
+			       unsigned clock_rate, 
 			       unsigned freq1,
 			       unsigned freq2,
 			       unsigned vol)
@@ -377,7 +377,7 @@ struct tonegen
 
 
 /* Default digit map is DTMF */
-static pjmedia_tone_digit_map digit_map =
+static pjmedia_tone_digit_map digit_map = 
 {
     16,
     {
@@ -401,7 +401,7 @@ static pjmedia_tone_digit_map digit_map =
 };
 
 
-static pj_status_t tonegen_get_frame(pjmedia_port *this_port,
+static pj_status_t tonegen_get_frame(pjmedia_port *this_port, 
 				     pjmedia_frame *frame);
 static pj_status_t tonegen_destroy(pjmedia_port *this_port);
 
@@ -423,8 +423,8 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_create2(pj_pool_t *pool,
     struct tonegen  *tonegen;
     pj_status_t status;
 
-    PJ_ASSERT_RETURN(pool && clock_rate && channel_count &&
-		     samples_per_frame && bits_per_sample == 16 &&
+    PJ_ASSERT_RETURN(pool && clock_rate && channel_count && 
+		     samples_per_frame && bits_per_sample == 16 && 
 		     p_port != NULL, PJ_EINVAL);
 
     /* Only support mono and stereo */
@@ -433,8 +433,8 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_create2(pj_pool_t *pool,
     /* Create and initialize port */
     tonegen = PJ_POOL_ZALLOC_T(pool, struct tonegen);
     if (name == NULL || name->slen == 0) name = &STR_TONE_GEN;
-    status = pjmedia_port_info_init(&tonegen->base.info, name,
-				    SIGNATURE, clock_rate, channel_count,
+    status = pjmedia_port_info_init(&tonegen->base.info, name, 
+				    SIGNATURE, clock_rate, channel_count, 
 				    bits_per_sample, samples_per_frame);
     if (status != PJ_SUCCESS)
 	return status;
@@ -458,7 +458,7 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_create2(pj_pool_t *pool,
 	return status;
     }
 
-    TRACE_((THIS_FILE, "Tonegen created: %u/%u/%u/%u", clock_rate,
+    TRACE_((THIS_FILE, "Tonegen created: %u/%u/%u/%u", clock_rate, 
 	    channel_count, samples_per_frame, bits_per_sample));
 
     /* Done */
@@ -476,7 +476,7 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_create( pj_pool_t *pool,
 					    pjmedia_port **p_port)
 {
     return pjmedia_tonegen_create2(pool, NULL, clock_rate, channel_count,
-				   samples_per_frame, bits_per_sample,
+				   samples_per_frame, bits_per_sample, 
 				   options, p_port);
 }
 
@@ -553,7 +553,7 @@ static pj_status_t tonegen_destroy(pjmedia_port *port)
 /*
  * Fill a frame with tones.
  */
-static pj_status_t tonegen_get_frame(pjmedia_port *port,
+static pj_status_t tonegen_get_frame(pjmedia_port *port, 
 				     pjmedia_frame *frame)
 {
     struct tonegen *tonegen = (struct tonegen*) port;
@@ -601,8 +601,8 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
     }
 
     if (tonegen->cur_digit >= tonegen->count) {
-	/* After we're finished with the last digit, we have played all
-	 * the digits
+	/* After we're finished with the last digit, we have played all 
+	 * the digits 
 	 */
 	if ((tonegen->options|tonegen->playback_options)&PJMEDIA_TONEGEN_LOOP)
 	{
@@ -620,7 +620,7 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
 	    goto on_return;
 	}
     }
-
+    
     dst = (short*) frame->buf;
     end = dst + PJMEDIA_PIA_SPF(&port->info);
 
@@ -628,12 +628,12 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
 	pjmedia_tone_desc *dig = &tonegen->digits[tonegen->cur_digit];
 	unsigned required, cnt, on_samp, off_samp;
 
-	required = end - dst;
+	required = (unsigned)(end - dst);
 	on_samp = dig->on_msec * clock_rate / 1000;
 	off_samp = dig->off_msec * clock_rate / 1000;
 
 	/* Init tonegen */
-	if (tonegen->dig_samples == 0 &&
+	if (tonegen->dig_samples == 0 && 
 	    (tonegen->count!=1 || !(dig->flags & PJMEDIA_TONE_INITIALIZED)))
 	{
 	    init_generate_tone(&tonegen->state,
@@ -642,7 +642,7 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
 	    dig->flags |= PJMEDIA_TONE_INITIALIZED;
 	    if (tonegen->cur_digit > 0) {
 		/* Clear initialized flag of previous digit */
-		tonegen->digits[tonegen->cur_digit-1].flags &=
+		tonegen->digits[tonegen->cur_digit-1].flags &= 
 						(~PJMEDIA_TONE_INITIALIZED);
 	    }
 	}
@@ -660,8 +660,8 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
 	    tonegen->dig_samples += cnt;
 	    required -= cnt;
 
-	    if ((dig->flags & PJMEDIA_TONE_ENABLE_FADE) &&
-		tonegen->dig_samples == cnt)
+	    if ((dig->flags & PJMEDIA_TONE_ENABLE_FADE) && 
+		tonegen->dig_samples == cnt) 
 	    {
 		/* Fade in */
 		short *samp = (dst - cnt);
@@ -680,7 +680,7 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
 		    }
 		}
 	    } else if ((dig->flags & PJMEDIA_TONE_ENABLE_FADE) &&
-			tonegen->dig_samples==on_samp)
+			tonegen->dig_samples==on_samp) 
 	    {
 		/* Fade out */
 		if (cnt > tonegen->fade_out_len)
@@ -728,7 +728,7 @@ static pj_status_t tonegen_get_frame(pjmedia_port *port,
     }
 
     if (dst < end)
-	pjmedia_zero_samples(dst, end-dst);
+	pjmedia_zero_samples(dst, (unsigned)(end-dst));
 
     frame->type = PJMEDIA_FRAME_TYPE_AUDIO;
     frame->size = PJMEDIA_PIA_AVG_FSZ(&port->info);
@@ -781,11 +781,11 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_play( pjmedia_port *port,
 
     /* Set playback options */
     tonegen->playback_options = options;
-
+    
     /* Copy digits */
     pj_memcpy(tonegen->digits + tonegen->count,
 	      tones, count * sizeof(pjmedia_tone_desc));
-
+    
     /* Normalize volume, and check if we need to disable fading.
      * Disable fading if tone off time is zero. Application probably
      * wants to play this tone continuously (e.g. dial tone).
@@ -865,7 +865,7 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_get_digit_map(pjmedia_port *port,
 						  const pjmedia_tone_digit_map **m)
 {
     struct tonegen *tonegen = (struct tonegen*) port;
-
+    
     PJ_ASSERT_RETURN(port->info.signature == SIGNATURE, PJ_EINVAL);
     PJ_ASSERT_RETURN(m != NULL, PJ_EINVAL);
 
@@ -882,7 +882,7 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_set_digit_map(pjmedia_port *port,
 						  pjmedia_tone_digit_map *m)
 {
     struct tonegen *tonegen = (struct tonegen*) port;
-
+    
     PJ_ASSERT_RETURN(port->info.signature == SIGNATURE, PJ_EINVAL);
     PJ_ASSERT_RETURN(m != NULL, PJ_EINVAL);
 
@@ -894,3 +894,5 @@ PJ_DEF(pj_status_t) pjmedia_tonegen_set_digit_map(pjmedia_port *port,
 
     return PJ_SUCCESS;
 }
+
+
