@@ -1,5 +1,5 @@
-/* $Id: vid_stream.c 4123 2012-05-14 11:17:31Z ming $ */
-/*
+/* $Id: vid_stream.c 4613 2013-10-08 09:08:13Z bennylp $ */
+/* 
  * Copyright (C) 2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia/vid_stream.h>
 #include <pjmedia/errno.h>
@@ -68,6 +68,13 @@
 #   define PJMEDIA_VSTREAM_INC	1000
 #endif
 
+/* Due to network MTU limitation, a picture bitstream may be splitted into
+ * several chunks for RTP delivery. The chunk number may vary depend on the
+ * picture resolution and MTU. This constant specifies the minimum chunk
+ * number to be allocated to store a picture bitstream in decoding direction.
+ */
+#define MIN_CHUNKS_PER_FRM	30
+
 /* Video stream keep-alive feature is currently disabled. */
 #if defined(PJMEDIA_STREAM_ENABLE_KA) && PJMEDIA_STREAM_ENABLE_KA != 0
 #   undef PJMEDIA_STREAM_ENABLE_KA
@@ -135,7 +142,7 @@ struct pjmedia_vid_stream
     pjmedia_frame            dec_frame;	    /**< Current decoded frame.     */
     pjmedia_event            fmt_event;	    /**< Buffered fmt_changed event
                                                  to avoid deadlock	    */
-    pjmedia_event            miss_keyframe_event;
+    pjmedia_event            miss_keyframe_event; 
 					    /**< Buffered missing keyframe
                                                  event for delayed republish*/
 
@@ -226,7 +233,7 @@ PJ_INLINE(int) trace_jb_print_timestamp(char **buf, pj_ssize_t len)
     return 0;
 }
 
-PJ_INLINE(int) trace_jb_print_state(pjmedia_vid_stream *stream,
+PJ_INLINE(int) trace_jb_print_state(pjmedia_vid_stream *stream, 
 				    char **buf, pj_ssize_t len)
 {
     char *p = *buf;
@@ -452,7 +459,7 @@ static void send_keep_alive_packet(pjmedia_vid_stream *stream)
 			        pkt_len);
 
 #else
-
+    
     PJ_UNUSED_ARG(stream);
 
 #endif
@@ -493,9 +500,9 @@ static pj_status_t send_rtcp(pjmedia_vid_stream *stream,
 					      &sdes_len, &sdes);
 	if (status != PJ_SUCCESS) {
 	    PJ_PERROR(4,(stream->name.ptr, status,
-				     "Error generating RTCP SDES"));
+        			     "Error generating RTCP SDES"));
 	} else {
-	    len += sdes_len;
+	    len += (int)sdes_len;
 	}
     }
 
@@ -508,9 +515,9 @@ static pj_status_t send_rtcp(pjmedia_vid_stream *stream,
 					     &bye_len, NULL);
 	if (status != PJ_SUCCESS) {
 	    PJ_PERROR(4,(stream->name.ptr, status,
-				     "Error generating RTCP BYE"));
+        			     "Error generating RTCP BYE"));
 	} else {
-	    len += bye_len;
+	    len += (int)bye_len;
 	}
     }
 
@@ -529,23 +536,23 @@ static pj_status_t send_rtcp(pjmedia_vid_stream *stream,
  */
 static void check_tx_rtcp(pjmedia_vid_stream *stream, pj_uint32_t timestamp)
 {
-    /* Note that timestamp may represent local or remote timestamp,
+    /* Note that timestamp may represent local or remote timestamp, 
      * depending on whether this function is called from put_frame()
      * or get_frame().
      */
 
 
     if (stream->rtcp_last_tx == 0) {
-
+	
 	stream->rtcp_last_tx = timestamp;
 
     } else if (timestamp - stream->rtcp_last_tx >= stream->rtcp_interval) {
 	pj_status_t status;
-
+	
 	status = send_rtcp(stream, !stream->rtcp_sdes_bye_disabled, PJ_FALSE);
 	if (status != PJ_SUCCESS) {
 	    PJ_PERROR(4,(stream->name.ptr, status,
-			 "Error sending RTCP"));
+        		 "Error sending RTCP"));
 	}
 
 	stream->rtcp_last_tx = timestamp;
@@ -581,9 +588,9 @@ static void dump_bin(const char *buf, unsigned len)
 
 /*
  * This callback is called by stream transport on receipt of packets
- * in the RTP socket.
+ * in the RTP socket. 
  */
-static void on_rx_rtp( void *data,
+static void on_rx_rtp( void *data, 
 		       void *pkt,
                        pj_ssize_t bytes_read)
 
@@ -599,7 +606,8 @@ static void on_rx_rtp( void *data,
 
     /* Check for errors */
     if (bytes_read < 0) {
-	LOGERR_((channel->port.info.name.ptr, "RTP recv() error", -bytes_read));
+	LOGERR_((channel->port.info.name.ptr, "RTP recv() error", 
+		(pj_status_t)-bytes_read));
 	return;
     }
 
@@ -608,7 +616,7 @@ static void on_rx_rtp( void *data,
 	return;
 
     /* Update RTP and RTCP session. */
-    status = pjmedia_rtp_decode_rtp(&channel->rtp, pkt, bytes_read,
+    status = pjmedia_rtp_decode_rtp(&channel->rtp, pkt, (int)bytes_read,
 				    &hdr, &payload, &payloadlen);
     if (status != PJ_SUCCESS) {
 	LOGERR_((channel->port.info.name.ptr, "RTP decode error", status));
@@ -625,9 +633,9 @@ static void on_rx_rtp( void *data,
      */
     pjmedia_rtp_session_update2(&channel->rtp, hdr, &seq_st, PJ_TRUE);
     if (seq_st.status.value) {
-	TRC_  ((channel->port.info.name.ptr,
+	TRC_  ((channel->port.info.name.ptr, 
 		"RTP status: badpt=%d, badssrc=%d, dup=%d, "
-		"outorder=%d, probation=%d, restart=%d",
+		"outorder=%d, probation=%d, restart=%d", 
 		seq_st.status.flag.badpt,
 		seq_st.status.flag.badssrc,
 		seq_st.status.flag.dup,
@@ -709,7 +717,7 @@ static void on_rx_rtp( void *data,
 	PJ_LOG(4,(channel->port.info.name.ptr, "Jitter buffer reset"));
     } else {
 	/* Just put the payload into jitter buffer */
-	pjmedia_jbuf_put_frame3(stream->jb, payload, payloadlen, 0,
+	pjmedia_jbuf_put_frame3(stream->jb, payload, payloadlen, 0, 
 				pj_ntohs(hdr->seq), pj_ntohl(hdr->ts), NULL);
 
 #if TRACE_JB
@@ -721,7 +729,7 @@ static void on_rx_rtp( void *data,
 
 
     /* Check if now is the time to transmit RTCP SR/RR report.
-     * We only do this when stream direction is "decoding only",
+     * We only do this when stream direction is "decoding only", 
      * because otherwise check_tx_rtcp() will be handled by put_frame()
      */
     if (stream->dir == PJMEDIA_DIR_DECODING) {
@@ -729,7 +737,7 @@ static void on_rx_rtp( void *data,
     }
 
     if (status != 0) {
-	LOGERR_((channel->port.info.name.ptr, "Jitter buffer put() error",
+	LOGERR_((channel->port.info.name.ptr, "Jitter buffer put() error", 
 		status));
 	pkt_discarded = PJ_TRUE;
 	goto on_return;
@@ -749,7 +757,7 @@ on_return:
 			   PJ_FALSE);
         if (status != PJ_SUCCESS) {
             PJ_PERROR(4,(stream->name.ptr, status,
-		     "Error sending initial RTCP RR"));
+            	     "Error sending initial RTCP RR"));
 	} else {
 	    stream->initial_rr = PJ_TRUE;
 	}
@@ -759,18 +767,18 @@ on_return:
 
 /*
  * This callback is called by stream transport on receipt of packets
- * in the RTCP socket.
+ * in the RTCP socket. 
  */
 static void on_rx_rtcp( void *data,
-                        void *pkt,
+                        void *pkt, 
                         pj_ssize_t bytes_read)
 {
     pjmedia_vid_stream *stream = (pjmedia_vid_stream*) data;
 
     /* Check for errors */
     if (bytes_read < 0) {
-	LOGERR_((stream->cname.ptr, "RTCP recv() error",
-		 -bytes_read));
+	LOGERR_((stream->cname.ptr, "RTCP recv() error", 
+		 (pj_status_t)-bytes_read));
 	return;
     }
 
@@ -801,7 +809,7 @@ static pj_status_t put_frame(pjmedia_port *port,
     {
 	pj_uint32_t dtx_duration;
 
-	dtx_duration = pj_timestamp_diff32(&stream->last_frm_ts_sent,
+	dtx_duration = pj_timestamp_diff32(&stream->last_frm_ts_sent, 
 					   &frame->timestamp);
         /* Video stream keep-alive feature is currently disabled. */
         /*
@@ -854,7 +862,7 @@ static pj_status_t put_frame(pjmedia_port *port,
 			       &rtphdrlen);
 	return status;
     }
-
+    
     pj_get_timestamp(&initial_time);
 
     /* Loop while we have frame to send */
@@ -862,7 +870,7 @@ static pj_status_t put_frame(pjmedia_port *port,
 	status = pjmedia_rtp_encode_rtp(&channel->rtp,
 	                                channel->pt,
 	                                (has_more_data == PJ_FALSE ? 1 : 0),
-	                                frame_out.size,
+	                                (int)frame_out.size,
 	                                rtp_ts_len,
 	                                (const void**)&rtphdr,
 	                                &rtphdrlen);
@@ -892,7 +900,7 @@ static pj_status_t put_frame(pjmedia_port *port,
 	    /* Ignore this error */
 	}
 
-	pjmedia_rtcp_tx_rtp(&stream->rtcp, frame_out.size);
+	pjmedia_rtcp_tx_rtp(&stream->rtcp, (unsigned)frame_out.size);
 	total_sent += frame_out.size;
 	pkt_cnt++;
 
@@ -960,7 +968,7 @@ static pj_status_t put_frame(pjmedia_port *port,
     }
 #endif
 
-    /* Check if now is the time to transmit RTCP SR/RR report.
+    /* Check if now is the time to transmit RTCP SR/RR report. 
      * We only do this when stream direction is not "decoding only", because
      * when it is, check_tx_rtcp() will be handled by get_frame().
      */
@@ -1064,7 +1072,7 @@ static pj_status_t decode_frame(pjmedia_vid_stream *stream,
 	/* Decode */
 	status = pjmedia_vid_codec_decode(stream->codec, cnt,
 	                                  stream->rx_frames,
-	                                  frame->size, frame);
+	                                  (unsigned)frame->size, frame);
 	if (status != PJ_SUCCESS) {
 	    LOGERR_((channel->port.info.name.ptr, "codec decode() error",
 		     status));
@@ -1251,7 +1259,7 @@ static pj_status_t create_channel( pj_pool_t *pool,
     pjmedia_format *fmt;
     char fourcc_name[5];
     pjmedia_port_info *pi;
-
+    
     pj_assert(info->type == PJMEDIA_TYPE_VIDEO);
     pj_assert(dir == PJMEDIA_DIR_DECODING || dir == PJMEDIA_DIR_ENCODING);
 
@@ -1276,7 +1284,7 @@ static pj_status_t create_channel( pj_pool_t *pool,
     channel->dir = dir;
     channel->paused = 1;
     channel->pt = pt;
-
+    
     /* Allocate buffer for outgoing packet. */
     if (dir == PJMEDIA_DIR_ENCODING) {
 	channel->buf_size = sizeof(pjmedia_rtp_hdr) + stream->frame_size;
@@ -1284,7 +1292,7 @@ static pj_status_t create_channel( pj_pool_t *pool,
 	/* It should big enough to hold (minimally) RTCP SR with an SDES. */
 	min_out_pkt_size =  sizeof(pjmedia_rtcp_sr_pkt) +
 			    sizeof(pjmedia_rtcp_common) +
-			    (4 + stream->cname.slen) +
+			    (4 + (unsigned)stream->cname.slen) +
 			    32;
 
 	if (channel->buf_size < min_out_pkt_size)
@@ -1357,7 +1365,6 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
     int frm_ptime, chunks_per_frm;
     pjmedia_video_format_detail *vfd_enc, *vfd_dec;
     char *p;
-    unsigned dec_mtu;
     pj_status_t status;
 
     if (!pool) {
@@ -1379,11 +1386,11 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 
     /* Init stream/port name */
     stream->name.ptr = (char*) pj_pool_alloc(pool, M);
-    stream->name.slen = pj_ansi_snprintf(stream->name.ptr, M,
+    stream->name.slen = pj_ansi_snprintf(stream->name.ptr, M, 
 					 "vstrm%p", stream);
 
     /* Create and initialize codec: */
-    status = pjmedia_vid_codec_mgr_alloc_codec(stream->codec_mgr,
+    status = pjmedia_vid_codec_mgr_alloc_codec(stream->codec_mgr, 
 					       &info->codec_info,
 					       &stream->codec);
     if (status != PJ_SUCCESS)
@@ -1393,7 +1400,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
     if (!info->codec_param) {
 	pjmedia_vid_codec_param def_param;
 
-	status = pjmedia_vid_codec_mgr_get_default_param(stream->codec_mgr,
+	status = pjmedia_vid_codec_mgr_get_default_param(stream->codec_mgr, 
 						         &info->codec_info,
 						         &def_param);
 	if (status != PJ_SUCCESS)
@@ -1410,9 +1417,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
     if (info->codec_param->enc_mtu > PJMEDIA_MAX_MTU)
 	info->codec_param->enc_mtu = PJMEDIA_MAX_MTU;
 
-    /* MTU estimation for decoding direction */
-    dec_mtu = PJMEDIA_MAX_MTU;
-
+    /* Packet size estimation for decoding direction */
     vfd_enc = pjmedia_format_get_video_format_detail(
 					&info->codec_param->enc_fmt, PJ_TRUE);
     vfd_dec = pjmedia_format_get_video_format_detail(
@@ -1467,7 +1472,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 #if 0
     stream->frame_size = vfd_enc->max_bps/8 * vfd_enc->fps.denum /
 			 vfd_enc->fps.num;
-
+    
     /* As the maximum frame_size is not represented directly by maximum bps
      * (which includes intra and predicted frames), let's increase the
      * frame size value for safety.
@@ -1476,7 +1481,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 #endif
 
     /* Validate the frame size */
-    if (stream->frame_size == 0 ||
+    if (stream->frame_size == 0 || 
 	stream->frame_size > PJMEDIA_MAX_VIDEO_ENC_FRAME_SIZE)
     {
 	stream->frame_size = PJMEDIA_MAX_VIDEO_ENC_FRAME_SIZE;
@@ -1511,13 +1516,13 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
     stream->dec_max_fps = vfd_dec->fps;
 
     /* Create decoder channel */
-    status = create_channel( pool, stream, PJMEDIA_DIR_DECODING,
+    status = create_channel( pool, stream, PJMEDIA_DIR_DECODING, 
 			     info->rx_pt, info, &stream->dec);
     if (status != PJ_SUCCESS)
 	return status;
 
     /* Create encoder channel */
-    status = create_channel( pool, stream, PJMEDIA_DIR_ENCODING,
+    status = create_channel( pool, stream, PJMEDIA_DIR_ENCODING, 
 			     info->tx_pt, info, &stream->enc);
     if (status != PJ_SUCCESS)
 	return status;
@@ -1528,8 +1533,9 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 
     /* Init jitter buffer parameters: */
     frm_ptime	    = 1000 * vfd_enc->fps.denum / vfd_enc->fps.num;
-    chunks_per_frm  = stream->frame_size / dec_mtu;
-    if (chunks_per_frm == 0) chunks_per_frm = 1;
+    chunks_per_frm  = stream->frame_size / PJMEDIA_MAX_MRU;
+    if (chunks_per_frm < MIN_CHUNKS_PER_FRM)
+	chunks_per_frm = MIN_CHUNKS_PER_FRM;
 
     /* JB max count, default 500ms */
     if (info->jb_max >= frm_ptime)
@@ -1564,7 +1570,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 
     /* Create jitter buffer */
     status = pjmedia_jbuf_create(pool, &stream->dec->port.info.name,
-                                 dec_mtu + PJMEDIA_STREAM_RESV_PAYLOAD_LEN,
+                                 PJMEDIA_MAX_MRU,
 				 1000 * vfd_enc->fps.denum / vfd_enc->fps.num,
 				 jb_max, &stream->jb);
     if (status != PJ_SUCCESS)
@@ -1594,7 +1600,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
      */
     stream->out_rtcp_pkt_size =  sizeof(pjmedia_rtcp_sr_pkt) +
 				 sizeof(pjmedia_rtcp_common) +
-				 (4 + stream->cname.slen) +
+				 (4 + (unsigned)stream->cname.slen) +
 				 32;
     if (stream->out_rtcp_pkt_size > PJMEDIA_MAX_MTU)
 	stream->out_rtcp_pkt_size = PJMEDIA_MAX_MTU;
@@ -1602,9 +1608,9 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
     stream->out_rtcp_pkt = pj_pool_alloc(pool, stream->out_rtcp_pkt_size);
 
     /* Only attach transport when stream is ready. */
-    status = pjmedia_transport_attach(tp, stream, &info->rem_addr,
-				      &info->rem_rtcp,
-				      pj_sockaddr_get_len(&info->rem_addr),
+    status = pjmedia_transport_attach(tp, stream, &info->rem_addr, 
+				      &info->rem_rtcp, 
+				      pj_sockaddr_get_len(&info->rem_addr), 
                                       &on_rx_rtp, &on_rx_rtcp);
     if (status != PJ_SUCCESS)
 	return status;
@@ -1627,14 +1633,14 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 	char trace_name[PJ_MAXPATH];
 	pj_ssize_t len;
 
-	pj_ansi_snprintf(trace_name, sizeof(trace_name),
+	pj_ansi_snprintf(trace_name, sizeof(trace_name), 
 			 TRACE_JB_PATH_PREFIX "%s.csv",
 			 channel->port.info.name.ptr);
 	status = pj_file_open(pool, trace_name, PJ_O_RDWR,
 			      &stream->trace_jb_fd);
 	if (status != PJ_SUCCESS) {
 	    stream->trace_jb_fd = TRACE_JB_INVALID_FD;
-	    PJ_LOG(3,(THIS_FILE, "Failed creating RTP trace file '%s'",
+	    PJ_LOG(3,(THIS_FILE, "Failed creating RTP trace file '%s'", 
 		      trace_name));
 	} else {
 	    stream->trace_jb_buf = (char*)pj_pool_alloc(pool, PJ_LOG_MAX_SIZE);
@@ -1644,6 +1650,8 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_create(
 				   "Time, Operation, Size, Frame Count, "
 				   "Frame type, RTP Seq, RTP TS, RTP M, "
 				   "JB size, JB burst level, JB prefetch\n");
+	    if (len < 1 || len >= PJ_LOG_MAX_SIZE)
+		len = PJ_LOG_MAX_SIZE - 1;
 	    pj_file_write(stream->trace_jb_fd, stream->trace_jb_buf, &len);
 	    pj_file_flush(stream->trace_jb_fd);
 	}
@@ -1676,7 +1684,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_destroy( pjmedia_vid_stream *stream )
 	unsigned total_time;
 
 	total_time = pj_elapsed_msec(&stream->tx_start, &stream->tx_end);
-	PJ_LOG(5, (stream->name.ptr,
+	PJ_LOG(5, (stream->name.ptr, 
 		   "RC stat: pkt_cnt=%.2f/image, sleep=%.2fms/s, fps=%.2f",
 		   stream->rc_total_pkt*1.0/stream->rc_total_img,
 		   stream->rc_total_sleep*1000.0/total_time,
@@ -1689,7 +1697,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_destroy( pjmedia_vid_stream *stream )
 	send_rtcp(stream, PJ_TRUE, PJ_TRUE);
     }
 
-    /* Detach from transport
+    /* Detach from transport 
      * MUST NOT hold stream mutex while detaching from transport, as
      * it may cause deadlock. See ticket #460 for the details.
      */
@@ -1713,7 +1721,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_stream_destroy( pjmedia_vid_stream *stream )
     }
 
     /* Free mutex */
-
+    
     if (stream->jb_mutex) {
 	pj_mutex_destroy(stream->jb_mutex);
 	stream->jb_mutex = NULL;
