@@ -1,4 +1,4 @@
-/* $Id: wav_player.c 4122 2012-05-14 11:04:46Z bennylp $ */
+/* $Id: wav_player.c 4793 2014-03-14 04:09:50Z bennylp $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -60,7 +60,7 @@ struct file_reader_port
     pjmedia_wave_fmt_tag fmt_tag;
     pj_uint16_t	     bytes_per_sample;
     pj_bool_t	     eof;
-    pj_size_t	     bufsize;
+    pj_uint32_t	     bufsize;
     char	    *buf;
     char	    *readpos;
     char	    *eofpos;
@@ -107,7 +107,7 @@ static struct file_reader_port *create_file_port(pj_pool_t *pool)
  */
 static pj_status_t fill_buffer(struct file_reader_port *fport)
 {
-    pj_ssize_t size_left = fport->bufsize;
+    pj_uint32_t size_left = fport->bufsize;
     unsigned size_to_read;
     pj_ssize_t size;
     pj_status_t status;
@@ -135,8 +135,8 @@ static pj_status_t fill_buffer(struct file_reader_port *fport)
             size = (pj_ssize_t)fport->data_left;
         }
 
-	size_left -= size;
-        fport->data_left -= size;
+	size_left -= (pj_uint32_t)size;
+        fport->data_left -= (pj_uint32_t)size;
 	fport->fpos += size;
 
 	/* If size is less than size_to_read, it indicates that we've
@@ -375,7 +375,7 @@ PJ_DEF(pj_status_t) pjmedia_wav_player_port_create( pj_pool_t *pool,
 
     /* Create file buffer.
      */
-    fport->bufsize = buff_size;
+    fport->bufsize = (pj_uint32_t)buff_size;
 
 
     /* samples_per_frame must be smaller than bufsize (because get_frame()
@@ -424,6 +424,44 @@ PJ_DEF(pj_status_t) pjmedia_wav_player_port_create( pj_pool_t *pool,
 }
 
 
+/*
+ * Get additional info about the file player.
+ */
+PJ_DEF(pj_status_t) pjmedia_wav_player_get_info(
+					pjmedia_port *port,
+					pjmedia_wav_player_info *info)
+{
+    struct file_reader_port *fport;
+    PJ_ASSERT_RETURN(port && info, PJ_EINVAL);
+
+    pj_bzero(info, sizeof(*info));
+
+    /* Check that this is really a player port */
+    PJ_ASSERT_RETURN(port->info.signature == SIGNATURE, PJ_EINVALIDOP);
+
+    fport = (struct file_reader_port*) port;
+
+    if (fport->fmt_tag == PJMEDIA_WAVE_FMT_TAG_PCM) {
+	info->fmt_id = PJMEDIA_FORMAT_PCM;
+	info->payload_bits_per_sample = 16;
+    } else if (fport->fmt_tag == PJMEDIA_WAVE_FMT_TAG_ULAW) {
+	info->fmt_id = PJMEDIA_FORMAT_ULAW;
+	info->payload_bits_per_sample = 8;
+    } else if (fport->fmt_tag == PJMEDIA_WAVE_FMT_TAG_ALAW) {
+	info->fmt_id = PJMEDIA_FORMAT_ALAW;
+	info->payload_bits_per_sample = 8;
+    } else {
+	pj_assert(!"Unsupported format");
+	return PJ_ENOTSUP;
+    }
+
+    info->size_bytes = pjmedia_wav_player_get_len(port);
+    info->size_samples = info->size_bytes /
+			 (info->payload_bits_per_sample / 8);
+
+    return PJ_SUCCESS;
+}
+
 /*
  * Get the data length, in bytes.
  */
@@ -484,7 +522,7 @@ PJ_DEF(pj_status_t) pjmedia_wav_player_port_set_pos(pjmedia_port *port,
 
 
 /*
- * Get the file play position of WAV player.
+ * Get the file play position of WAV player (in bytes).
  */
 PJ_DEF(pj_ssize_t) pjmedia_wav_player_port_get_pos( pjmedia_port *port )
 {
@@ -541,7 +579,7 @@ static pj_status_t file_get_frame(pjmedia_port *this_port,
 				  pjmedia_frame *frame)
 {
     struct file_reader_port *fport = (struct file_reader_port*)this_port;
-    unsigned frame_size;
+    pj_size_t frame_size;
     pj_status_t status;
 
     pj_assert(fport->base.info.signature == SIGNATURE);
@@ -617,7 +655,7 @@ static pj_status_t file_get_frame(pjmedia_port *this_port,
 	/* Split read.
 	 * First stage: read until end of buffer. 
 	 */
-	endread = (fport->buf+fport->bufsize) - fport->readpos;
+	endread = (unsigned)((fport->buf+fport->bufsize) - fport->readpos);
 	pj_memcpy(frame->buf, fport->readpos, endread);
 
 	/* End Of Buffer and EOF and NO LOOP */
