@@ -1,5 +1,5 @@
-/* $Id: mips_test.c 3982 2012-03-22 09:56:52Z bennylp $ */
-/*
+/* $Id: mips_test.c 4335 2013-01-29 08:09:15Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include "test.h"
 #include <pjmedia-codec.h>
@@ -414,8 +414,8 @@ static pjmedia_port* create_gen_port(pj_pool_t *pool,
     pj_status_t status;
 
     if (pct_level == 100 && channel_count==1) {
-	status = pjmedia_mem_player_create(pool, ref_signal,
-					   sizeof(ref_signal), clock_rate,
+	status = pjmedia_mem_player_create(pool, ref_signal, 
+					   sizeof(ref_signal), clock_rate, 
 					   channel_count, samples_per_frame,
 					   16, 0, &port);
     } else {
@@ -434,9 +434,9 @@ static pjmedia_port* create_gen_port(pj_pool_t *pool,
 		p += channel_count;
 	    }
 	}
-	status = pjmedia_mem_player_create(pool, buf,
-					   sizeof(ref_signal)*channel_count,
-					   clock_rate,  channel_count,
+	status = pjmedia_mem_player_create(pool, buf, 
+					   sizeof(ref_signal)*channel_count, 
+					   clock_rate,  channel_count, 
 					   samples_per_frame,
 					   16, 0, &port);
     }
@@ -476,8 +476,8 @@ static pjmedia_port* init_conf_port(unsigned nb_participant,
     PJ_UNUSED_ARG(te);
 
     /* Create conf */
-    status = pjmedia_conf_create(pool, 2+nb_participant*2, clock_rate,
-				 channel_count, samples_per_frame, 16,
+    status = pjmedia_conf_create(pool, 2+nb_participant*2, clock_rate, 
+				 channel_count, samples_per_frame, 16, 
 				 PJMEDIA_CONF_NO_DEVICE, &conf);
     if (status != PJ_SUCCESS)
 	return NULL;
@@ -523,7 +523,7 @@ static pjmedia_port* init_conf_port(unsigned nb_participant,
 	status = pjmedia_conf_connect_port(conf, slot1, slot2, 0);
 	if (status != PJ_SUCCESS)
 	    return NULL;
-#endif
+#endif	
     }
 
     return pjmedia_conf_get_master_port(conf);
@@ -539,7 +539,7 @@ static pjmedia_port* conf1_test_init(pj_pool_t *pool,
 				     unsigned flags,
 				     struct test_entry *te)
 {
-    return init_conf_port(1, pool, clock_rate, channel_count,
+    return init_conf_port(1, pool, clock_rate, channel_count, 
 			  samples_per_frame, flags, te);
 }
 
@@ -553,7 +553,7 @@ static pjmedia_port* conf2_test_init(pj_pool_t *pool,
 				     unsigned flags,
 				     struct test_entry *te)
 {
-    return init_conf_port(2, pool, clock_rate, channel_count,
+    return init_conf_port(2, pool, clock_rate, channel_count, 
 			  samples_per_frame, flags, te);
 }
 
@@ -566,7 +566,7 @@ static pjmedia_port* conf4_test_init(pj_pool_t *pool,
 				     unsigned flags,
 				     struct test_entry *te)
 {
-    return init_conf_port(4, pool, clock_rate, channel_count,
+    return init_conf_port(4, pool, clock_rate, channel_count, 
 			  samples_per_frame, flags, te);
 }
 
@@ -579,7 +579,7 @@ static pjmedia_port* conf8_test_init(pj_pool_t *pool,
 				     unsigned flags,
 				     struct test_entry *te)
 {
-    return init_conf_port(8, pool, clock_rate, channel_count,
+    return init_conf_port(8, pool, clock_rate, channel_count, 
 			  samples_per_frame, flags, te);
 }
 
@@ -593,7 +593,7 @@ static pjmedia_port* conf16_test_init(pj_pool_t *pool,
 				      struct test_entry *te)
 {
     PJ_UNUSED_ARG(flags);
-    return init_conf_port(16, pool, clock_rate, channel_count,
+    return init_conf_port(16, pool, clock_rate, channel_count, 
 			  samples_per_frame, flags, te);
 }
 
@@ -682,8 +682,11 @@ struct codec_port
     pj_uint16_t	     pcm[32000 * PTIME / 1000];
 };
 
-
-static pj_status_t codec_put_frame(struct pjmedia_port *this_port,
+#if PJMEDIA_HAS_L16_CODEC || \
+    PJMEDIA_HAS_OPENCORE_AMRWB_CODEC || PJMEDIA_HAS_OPENCORE_AMRNB_CODEC || \
+    PJMEDIA_HAS_G7221_CODEC || PJMEDIA_HAS_G722_CODEC || PJMEDIA_HAS_G711_CODEC || \
+    PJMEDIA_HAS_SPEEX_CODEC || PJMEDIA_HAS_ILBC_CODEC || PJMEDIA_HAS_GSM_CODEC
+static pj_status_t codec_put_frame(struct pjmedia_port *this_port, 
 				   pjmedia_frame *frame)
 {
     struct codec_port *cp = (struct codec_port*)this_port;
@@ -701,15 +704,15 @@ static pj_status_t codec_put_frame(struct pjmedia_port *this_port,
 	unsigned frame_cnt = PJ_ARRAY_SIZE(parsed_frm);
 	unsigned i;
 
-	status = pjmedia_codec_parse(cp->codec, out_frame.buf,
+	status = pjmedia_codec_parse(cp->codec, out_frame.buf, 
 				     out_frame.size, &out_frame.timestamp,
 				     &frame_cnt, parsed_frm);
 	pj_assert(status == PJ_SUCCESS);
-
+	
 	for (i=0; i<frame_cnt; ++i) {
 	    pcm_frm.buf = cp->pcm;
 	    pcm_frm.size = sizeof(cp->pkt);
-	    status = pjmedia_codec_decode(cp->codec, &parsed_frm[i],
+	    status = pjmedia_codec_decode(cp->codec, &parsed_frm[i], 
 					  sizeof(cp->pcm), &pcm_frm);
 	    pj_assert(status == PJ_SUCCESS);
 	}
@@ -792,6 +795,7 @@ static pjmedia_port* codec_encode_decode( pj_pool_t *pool,
 
     return &cp->base;
 }
+#endif
 
 #if PJMEDIA_HAS_G711_CODEC
 /* G.711 benchmark */
@@ -802,7 +806,7 @@ static pjmedia_port* g711_encode_decode(  pj_pool_t *pool,
 					  unsigned flags,
 					  struct test_entry *te)
 {
-    return codec_encode_decode(pool, "pcmu", &pjmedia_codec_g711_init,
+    return codec_encode_decode(pool, "pcmu", &pjmedia_codec_g711_init, 
 			       &pjmedia_codec_g711_deinit,
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
@@ -818,8 +822,8 @@ static pjmedia_port* gsm_encode_decode(  pj_pool_t *pool,
 					 unsigned flags,
 					 struct test_entry *te)
 {
-    return codec_encode_decode(pool, "gsm", &pjmedia_codec_gsm_init,
-			       &pjmedia_codec_gsm_deinit,
+    return codec_encode_decode(pool, "gsm", &pjmedia_codec_gsm_init, 
+			       &pjmedia_codec_gsm_deinit, 
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
 }
@@ -840,8 +844,8 @@ static pjmedia_port* ilbc_encode_decode( pj_pool_t *pool,
 					 struct test_entry *te)
 {
     samples_per_frame = 30 * clock_rate / 1000;
-    return codec_encode_decode(pool, "ilbc", &ilbc_init,
-			       &pjmedia_codec_ilbc_deinit, clock_rate,
+    return codec_encode_decode(pool, "ilbc", &ilbc_init, 
+			       &pjmedia_codec_ilbc_deinit, clock_rate, 
 			       channel_count, samples_per_frame, flags, te);
 }
 #endif
@@ -855,9 +859,9 @@ static pjmedia_port* speex8_encode_decode(pj_pool_t *pool,
 					  unsigned flags,
 					  struct test_entry *te)
 {
-    return codec_encode_decode(pool, "speex/8000",
-			       &pjmedia_codec_speex_init_default,
-			       &pjmedia_codec_speex_deinit,
+    return codec_encode_decode(pool, "speex/8000", 
+			       &pjmedia_codec_speex_init_default, 
+			       &pjmedia_codec_speex_deinit, 
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
 }
@@ -870,9 +874,9 @@ static pjmedia_port* speex16_encode_decode(pj_pool_t *pool,
 					   unsigned flags,
 					   struct test_entry *te)
 {
-    return codec_encode_decode(pool, "speex/16000",
-			       &pjmedia_codec_speex_init_default,
-			       &pjmedia_codec_speex_deinit,
+    return codec_encode_decode(pool, "speex/16000", 
+			       &pjmedia_codec_speex_init_default, 
+			       &pjmedia_codec_speex_deinit, 
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
 }
@@ -887,8 +891,8 @@ static pjmedia_port* g722_encode_decode(pj_pool_t *pool,
 					unsigned flags,
 					struct test_entry *te)
 {
-    return codec_encode_decode(pool, "g722", &pjmedia_codec_g722_init,
-			       &pjmedia_codec_g722_deinit,
+    return codec_encode_decode(pool, "g722", &pjmedia_codec_g722_init, 
+			       &pjmedia_codec_g722_deinit, 
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
 }
@@ -903,7 +907,7 @@ static pjmedia_port* g7221_encode_decode(pj_pool_t *pool,
 					 unsigned flags,
 					 struct test_entry *te)
 {
-    return codec_encode_decode(pool, "g7221/16000",
+    return codec_encode_decode(pool, "g7221/16000", 
 			       &pjmedia_codec_g7221_init,
 			       &pjmedia_codec_g7221_deinit,
 			       clock_rate, channel_count,
@@ -918,7 +922,7 @@ static pjmedia_port* g7221c_encode_decode(pj_pool_t *pool,
 					  unsigned flags,
 					  struct test_entry *te)
 {
-    return codec_encode_decode(pool, "g7221/32000",
+    return codec_encode_decode(pool, "g7221/32000", 
 			       &pjmedia_codec_g7221_init,
 			       &pjmedia_codec_g7221_deinit,
 			       clock_rate, channel_count,
@@ -935,7 +939,7 @@ static pjmedia_port* amr_encode_decode(pj_pool_t *pool,
                                        unsigned flags,
                                        struct test_entry *te)
 {
-    return codec_encode_decode(pool, "AMR/8000",
+    return codec_encode_decode(pool, "AMR/8000", 
 			       &pjmedia_codec_opencore_amrnb_init,
 			       &pjmedia_codec_opencore_amrnb_deinit,
 			       clock_rate, channel_count,
@@ -943,6 +947,23 @@ static pjmedia_port* amr_encode_decode(pj_pool_t *pool,
 }
 #endif	/* PJMEDIA_HAS_OPENCORE_AMRNB_CODEC */
 
+#if PJMEDIA_HAS_OPENCORE_AMRWB_CODEC
+/* AMR-WB benchmark benchmark */
+static pjmedia_port* amrwb_encode_decode(pj_pool_t *pool,
+                                         unsigned clock_rate,
+                                         unsigned channel_count,
+                                         unsigned samples_per_frame,
+                                         unsigned flags,
+                                         struct test_entry *te)
+{
+    return codec_encode_decode(pool, "AMR/16000",
+			       &pjmedia_codec_opencore_amr_init_default,
+			       &pjmedia_codec_opencore_amr_deinit,
+			       clock_rate, channel_count,
+			       samples_per_frame, flags, te);
+}
+#endif	/* PJMEDIA_HAS_OPENCORE_AMRWB_CODEC */
+
 #if defined(PJMEDIA_HAS_L16_CODEC) && PJMEDIA_HAS_L16_CODEC!=0
 static pj_status_t init_l16_default(pjmedia_endpt *endpt)
 {
@@ -957,8 +978,8 @@ static pjmedia_port* l16_8_encode_decode(pj_pool_t *pool,
 					 unsigned flags,
 					 struct test_entry *te)
 {
-    return codec_encode_decode(pool, "L16/8000/1", &init_l16_default,
-			       &pjmedia_codec_l16_deinit,
+    return codec_encode_decode(pool, "L16/8000/1", &init_l16_default, 
+			       &pjmedia_codec_l16_deinit, 
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
 }
@@ -971,8 +992,8 @@ static pjmedia_port* l16_16_encode_decode(pj_pool_t *pool,
 					  unsigned flags,
 					  struct test_entry *te)
 {
-    return codec_encode_decode(pool, "L16/16000/1", &init_l16_default,
-			       &pjmedia_codec_l16_deinit,
+    return codec_encode_decode(pool, "L16/16000/1", &init_l16_default, 
+			       &pjmedia_codec_l16_deinit, 
 			       clock_rate, channel_count,
 			       samples_per_frame, flags, te);
 }
@@ -991,7 +1012,7 @@ struct wsola_plc_port
 };
 
 
-static pj_status_t wsola_plc_get_frame(struct pjmedia_port *this_port,
+static pj_status_t wsola_plc_get_frame(struct pjmedia_port *this_port, 
 				       pjmedia_frame *frame)
 {
     struct wsola_plc_port *wp = (struct wsola_plc_port*)this_port;
@@ -1002,7 +1023,7 @@ static pj_status_t wsola_plc_get_frame(struct pjmedia_port *this_port,
 	status = pjmedia_port_get_frame(wp->gen_port, frame);
 	pj_assert(status == PJ_SUCCESS);
 
-	status = pjmedia_wsola_save(wp->wsola, (short*)frame->buf,
+	status = pjmedia_wsola_save(wp->wsola, (short*)frame->buf, 
 				    wp->prev_lost);
 	pj_assert(status == PJ_SUCCESS);
 
@@ -1043,7 +1064,7 @@ static pjmedia_port* create_wsola_plc(unsigned loss_pct,
     wp->loss_pct = loss_pct;
     wp->base.get_frame = &wsola_plc_get_frame;
     wp->base.on_destroy = &wsola_plc_on_destroy;
-    pjmedia_port_info_init(&wp->base.info, &name, 0x4123, clock_rate,
+    pjmedia_port_info_init(&wp->base.info, &name, 0x4123, clock_rate, 
 			   channel_count, 16, samples_per_frame);
 
     if (loss_pct == 0)
@@ -1054,7 +1075,7 @@ static pjmedia_port* create_wsola_plc(unsigned loss_pct,
     if (status != PJ_SUCCESS)
 	return NULL;
 
-    wp->gen_port = create_gen_port(pool, clock_rate, channel_count,
+    wp->gen_port = create_gen_port(pool, clock_rate, channel_count, 
 				   samples_per_frame, 100);
     if (wp->gen_port == NULL)
 	return NULL;
@@ -1071,7 +1092,7 @@ static pjmedia_port* wsola_plc_0( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_wsola_plc(0, pool, clock_rate, channel_count,
+    return create_wsola_plc(0, pool, clock_rate, channel_count, 
 			    samples_per_frame, flags, te);
 }
 
@@ -1084,7 +1105,7 @@ static pjmedia_port* wsola_plc_2( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_wsola_plc(2, pool, clock_rate, channel_count,
+    return create_wsola_plc(2, pool, clock_rate, channel_count, 
 			    samples_per_frame, flags, te);
 }
 
@@ -1096,7 +1117,7 @@ static pjmedia_port* wsola_plc_5( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_wsola_plc(5, pool, clock_rate, channel_count,
+    return create_wsola_plc(5, pool, clock_rate, channel_count, 
 			    samples_per_frame, flags, te);
 }
 
@@ -1108,7 +1129,7 @@ static pjmedia_port* wsola_plc_10(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_wsola_plc(10, pool, clock_rate, channel_count,
+    return create_wsola_plc(10, pool, clock_rate, channel_count, 
 			    samples_per_frame, flags, te);
 }
 
@@ -1120,7 +1141,7 @@ static pjmedia_port* wsola_plc_20(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_wsola_plc(20, pool, clock_rate, channel_count,
+    return create_wsola_plc(20, pool, clock_rate, channel_count, 
 			    samples_per_frame, flags, te);
 }
 
@@ -1132,7 +1153,7 @@ static pjmedia_port* wsola_plc_50(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_wsola_plc(50, pool, clock_rate, channel_count,
+    return create_wsola_plc(50, pool, clock_rate, channel_count, 
 			    samples_per_frame, flags, te);
 }
 
@@ -1151,7 +1172,7 @@ struct wsola_discard_port
 };
 
 
-static pj_status_t wsola_discard_get_frame(struct pjmedia_port *this_port,
+static pj_status_t wsola_discard_get_frame(struct pjmedia_port *this_port, 
 					   pjmedia_frame *frame)
 {
     struct wsola_discard_port *wp = (struct wsola_discard_port*)this_port;
@@ -1163,11 +1184,11 @@ static pj_status_t wsola_discard_get_frame(struct pjmedia_port *this_port,
 	status = pjmedia_port_get_frame(wp->gen_port, frame);
 	pj_assert(status==PJ_SUCCESS);
 
-	status = pjmedia_circ_buf_write(wp->circbuf, (short*)frame->buf,
+	status = pjmedia_circ_buf_write(wp->circbuf, (short*)frame->buf, 
 					PJMEDIA_PIA_SPF(&wp->base.info));
 	pj_assert(status==PJ_SUCCESS);
     }
-
+    
     if ((pj_rand() % 100) < (int)wp->discard_pct) {
 	pj_int16_t *reg1, *reg2;
 	unsigned reg1_len, reg2_len;
@@ -1177,7 +1198,7 @@ static pj_status_t wsola_discard_get_frame(struct pjmedia_port *this_port,
 					  &reg2, &reg2_len);
 
 	del_cnt = PJMEDIA_PIA_SPF(&wp->base.info);
-	status = pjmedia_wsola_discard(wp->wsola, reg1, reg1_len, reg2,
+	status = pjmedia_wsola_discard(wp->wsola, reg1, reg1_len, reg2, 
 				       reg2_len, &del_cnt);
 	pj_assert(status==PJ_SUCCESS);
 
@@ -1217,7 +1238,7 @@ static pjmedia_port* create_wsola_discard(unsigned discard_pct,
     wp->discard_pct = discard_pct;
     wp->base.get_frame = &wsola_discard_get_frame;
     wp->base.on_destroy = &wsola_discard_on_destroy;
-    pjmedia_port_info_init(&wp->base.info, &name, 0x4123, clock_rate,
+    pjmedia_port_info_init(&wp->base.info, &name, 0x4123, clock_rate, 
 			   channel_count, 16, samples_per_frame);
 
     if (discard_pct == 0)
@@ -1228,12 +1249,12 @@ static pjmedia_port* create_wsola_discard(unsigned discard_pct,
     if (status != PJ_SUCCESS)
 	return NULL;
 
-    wp->gen_port = create_gen_port(pool, clock_rate, channel_count,
+    wp->gen_port = create_gen_port(pool, clock_rate, channel_count, 
 				   samples_per_frame, 100);
     if (wp->gen_port == NULL)
 	return NULL;
 
-    status = pjmedia_circ_buf_create(pool, samples_per_frame * CIRC_BUF_FRAME_CNT,
+    status = pjmedia_circ_buf_create(pool, samples_per_frame * CIRC_BUF_FRAME_CNT, 
 				     &wp->circbuf);
     if (status != PJ_SUCCESS)
 	return NULL;
@@ -1266,7 +1287,7 @@ static pjmedia_port* wsola_discard_2( pj_pool_t *pool,
 				      unsigned flags,
 				      struct test_entry *te)
 {
-    return create_wsola_discard(2, pool, clock_rate, channel_count,
+    return create_wsola_discard(2, pool, clock_rate, channel_count, 
 			        samples_per_frame, flags, te);
 }
 
@@ -1278,7 +1299,7 @@ static pjmedia_port* wsola_discard_5( pj_pool_t *pool,
 				      unsigned flags,
 				      struct test_entry *te)
 {
-    return create_wsola_discard(5, pool, clock_rate, channel_count,
+    return create_wsola_discard(5, pool, clock_rate, channel_count, 
 			        samples_per_frame, flags, te);
 }
 
@@ -1290,7 +1311,7 @@ static pjmedia_port* wsola_discard_10(pj_pool_t *pool,
 				      unsigned flags,
 				      struct test_entry *te)
 {
-    return create_wsola_discard(10, pool, clock_rate, channel_count,
+    return create_wsola_discard(10, pool, clock_rate, channel_count, 
 			        samples_per_frame, flags, te);
 }
 
@@ -1302,7 +1323,7 @@ static pjmedia_port* wsola_discard_20(pj_pool_t *pool,
 				      unsigned flags,
 				      struct test_entry *te)
 {
-    return create_wsola_discard(20, pool, clock_rate, channel_count,
+    return create_wsola_discard(20, pool, clock_rate, channel_count, 
 			        samples_per_frame, flags, te);
 }
 
@@ -1314,7 +1335,7 @@ static pjmedia_port* wsola_discard_50(pj_pool_t *pool,
 				      unsigned flags,
 				      struct test_entry *te)
 {
-    return create_wsola_discard(50, pool, clock_rate, channel_count,
+    return create_wsola_discard(50, pool, clock_rate, channel_count, 
 			        samples_per_frame, flags, te);
 }
 
@@ -1335,7 +1356,7 @@ static pjmedia_port* ec_create(unsigned ec_tail_msec,
 
     PJ_UNUSED_ARG(te);
 
-    gen_port = create_gen_port(pool, clock_rate, channel_count,
+    gen_port = create_gen_port(pool, clock_rate, channel_count, 
 			       samples_per_frame, 100);
     if (gen_port == NULL)
 	return NULL;
@@ -1606,7 +1627,7 @@ static pjmedia_port* create_tonegen(unsigned freq1,
     PJ_UNUSED_ARG(te);
 
     status = pjmedia_tonegen_create(pool, clock_rate, channel_count,
-				    samples_per_frame, 16,
+				    samples_per_frame, 16, 
 				    PJMEDIA_TONEGEN_LOOP, &tonegen);
     if (status != PJ_SUCCESS)
 	return NULL;
@@ -1621,7 +1642,7 @@ static pjmedia_port* create_tonegen(unsigned freq1,
     tones[1].on_msec = 400;
     tones[1].off_msec = 100;
 
-    status = pjmedia_tonegen_play(tonegen, PJ_ARRAY_SIZE(tones), tones,
+    status = pjmedia_tonegen_play(tonegen, PJ_ARRAY_SIZE(tones), tones, 
 				  PJMEDIA_TONEGEN_LOOP);
     if (status != PJ_SUCCESS)
 	return NULL;
@@ -1679,6 +1700,10 @@ static void stream_port_custom_deinit(struct test_entry *te)
 
 }
 
+#if PJMEDIA_HAS_L16_CODEC || \
+    PJMEDIA_HAS_OPENCORE_AMRWB_CODEC || PJMEDIA_HAS_OPENCORE_AMRNB_CODEC || \
+    PJMEDIA_HAS_G7221_CODEC || PJMEDIA_HAS_G722_CODEC || PJMEDIA_HAS_G711_CODEC || \
+    PJMEDIA_HAS_SPEEX_CODEC || PJMEDIA_HAS_ILBC_CODEC || PJMEDIA_HAS_GSM_CODEC
 static pjmedia_port* create_stream( pj_pool_t *pool,
 				    const char *codec,
 				    pj_status_t (*codec_init)(pjmedia_endpt*),
@@ -1789,7 +1814,7 @@ static pjmedia_port* create_stream( pj_pool_t *pool,
 #endif
 
     /* Create stream */
-    status = pjmedia_stream_create(sp->endpt, pool, &si, sp->transport, NULL,
+    status = pjmedia_stream_create(sp->endpt, pool, &si, sp->transport, NULL, 
 				   &sp->stream);
     if (status != PJ_SUCCESS)
 	return NULL;
@@ -1805,6 +1830,7 @@ static pjmedia_port* create_stream( pj_pool_t *pool,
 
     return port;
 }
+#endif
 
 #if PJMEDIA_HAS_G711_CODEC
 /* G.711 stream, no SRTP */
@@ -1815,7 +1841,7 @@ static pjmedia_port* create_stream_pcmu( pj_pool_t *pool,
 					 unsigned flags,
 					 struct test_entry *te)
 {
-    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init,
+    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init, 
 			 &pjmedia_codec_g711_deinit,
 			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
@@ -1830,7 +1856,7 @@ static pjmedia_port* create_stream_pcmu_srtp32_no_auth( pj_pool_t *pool,
 							unsigned flags,
 							struct test_entry *te)
 {
-    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init,
+    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init, 
 			 &pjmedia_codec_g711_deinit,
 			 PJ_TRUE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
@@ -1845,7 +1871,7 @@ static pjmedia_port* create_stream_pcmu_srtp32_with_auth(pj_pool_t *pool,
 							 unsigned flags,
 							 struct test_entry *te)
 {
-    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init,
+    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init, 
 			 &pjmedia_codec_g711_deinit,
 			 PJ_TRUE, PJ_FALSE, PJ_TRUE,
 			 clock_rate, channel_count,
@@ -1860,7 +1886,7 @@ static pjmedia_port* create_stream_pcmu_srtp80_no_auth( pj_pool_t *pool,
 							unsigned flags,
 							struct test_entry *te)
 {
-    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init,
+    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init, 
 			 &pjmedia_codec_g711_deinit,
 			 PJ_TRUE, PJ_TRUE, PJ_FALSE,
 			 clock_rate, channel_count,
@@ -1875,7 +1901,7 @@ static pjmedia_port* create_stream_pcmu_srtp80_with_auth(pj_pool_t *pool,
 							 unsigned flags,
 							 struct test_entry *te)
 {
-    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init,
+    return create_stream(pool, "pcmu", &pjmedia_codec_g711_init, 
 			 &pjmedia_codec_g711_deinit,
 			 PJ_TRUE, PJ_TRUE, PJ_TRUE,
 			 clock_rate, channel_count,
@@ -1892,8 +1918,8 @@ static pjmedia_port* create_stream_gsm(  pj_pool_t *pool,
 					 unsigned flags,
 					 struct test_entry *te)
 {
-    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init,
-			 &pjmedia_codec_gsm_deinit,
+    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init, 
+			 &pjmedia_codec_gsm_deinit, 
 			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -1907,8 +1933,8 @@ static pjmedia_port* create_stream_gsm_srtp32_no_auth(pj_pool_t *pool,
 						      unsigned flags,
 						      struct test_entry *te)
 {
-    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init,
-			 &pjmedia_codec_gsm_deinit,
+    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init, 
+			 &pjmedia_codec_gsm_deinit, 
 			 PJ_TRUE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -1922,8 +1948,8 @@ static pjmedia_port* create_stream_gsm_srtp32_with_auth(pj_pool_t *pool,
 						        unsigned flags,
 						        struct test_entry *te)
 {
-    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init,
-			 &pjmedia_codec_gsm_deinit,
+    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init, 
+			 &pjmedia_codec_gsm_deinit, 
 			 PJ_TRUE, PJ_FALSE, PJ_TRUE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -1937,8 +1963,8 @@ static pjmedia_port* create_stream_gsm_srtp80_no_auth(pj_pool_t *pool,
 						      unsigned flags,
 						      struct test_entry *te)
 {
-    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init,
-			 &pjmedia_codec_gsm_deinit,
+    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init, 
+			 &pjmedia_codec_gsm_deinit, 
 			 PJ_TRUE, PJ_TRUE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -1952,8 +1978,8 @@ static pjmedia_port* create_stream_gsm_srtp80_with_auth(pj_pool_t *pool,
 						        unsigned flags,
 						        struct test_entry *te)
 {
-    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init,
-			 &pjmedia_codec_gsm_deinit,
+    return create_stream(pool, "gsm", &pjmedia_codec_gsm_init, 
+			 &pjmedia_codec_gsm_deinit, 
 			 PJ_TRUE, PJ_TRUE, PJ_TRUE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -1969,8 +1995,8 @@ static pjmedia_port* create_stream_g722( pj_pool_t *pool,
 					 unsigned flags,
 					 struct test_entry *te)
 {
-    return create_stream(pool, "g722", &pjmedia_codec_g722_init,
-			 &pjmedia_codec_g722_deinit,
+    return create_stream(pool, "g722", &pjmedia_codec_g722_init, 
+			 &pjmedia_codec_g722_deinit, 
 			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -1986,8 +2012,8 @@ static pjmedia_port* create_stream_g7221( pj_pool_t *pool,
 					  unsigned flags,
 					  struct test_entry *te)
 {
-    return create_stream(pool, "g7221/16000", &pjmedia_codec_g7221_init,
-			 &pjmedia_codec_g7221_deinit,
+    return create_stream(pool, "g7221/16000", &pjmedia_codec_g7221_init, 
+			 &pjmedia_codec_g7221_deinit, 
 			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
@@ -2002,12 +2028,12 @@ static pjmedia_port* create_stream_g7221c( pj_pool_t *pool,
 					   struct test_entry *te)
 {
     return create_stream(pool, "g7221/32000", &pjmedia_codec_g7221_init,
-			 &pjmedia_codec_g7221_deinit,
+			 &pjmedia_codec_g7221_deinit, 
 			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
 }
-#endif	/* PJMEDIA_HAS_G7221_CODEC */
+#endif	/* PJMEDIA_HAS_G7221_CODEC */ 
 
 /* AMR-NB stream */
 #if PJMEDIA_HAS_OPENCORE_AMRNB_CODEC
@@ -2018,14 +2044,32 @@ static pjmedia_port* create_stream_amr( pj_pool_t *pool,
                                         unsigned flags,
                                         struct test_entry *te)
 {
-    return create_stream(pool, "AMR/8000", &pjmedia_codec_opencore_amrnb_init,
-			 &pjmedia_codec_opencore_amrnb_deinit,
+    return create_stream(pool, "AMR/8000", &pjmedia_codec_opencore_amrnb_init, 
+			 &pjmedia_codec_opencore_amrnb_deinit, 
 			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
 			 clock_rate, channel_count,
 			 samples_per_frame, flags, te);
 }
 #endif	/* PJMEDIA_HAS_OPENCORE_AMRNB_CODEC */
 
+/* AMR-WB stream */
+#if PJMEDIA_HAS_OPENCORE_AMRWB_CODEC
+static pjmedia_port* create_stream_amrwb( pj_pool_t *pool,
+                                         unsigned clock_rate,
+                                         unsigned channel_count,
+                                         unsigned samples_per_frame,
+                                         unsigned flags,
+                                         struct test_entry *te)
+{
+    return create_stream(pool, "AMR/16000",
+                         &pjmedia_codec_opencore_amr_init_default,
+			 &pjmedia_codec_opencore_amr_deinit,
+			 PJ_FALSE, PJ_FALSE, PJ_FALSE,
+			 clock_rate, channel_count,
+			 samples_per_frame, flags, te);
+}
+#endif	/* PJMEDIA_HAS_OPENCORE_AMRWB_CODEC */
+
 /***************************************************************************/
 /* Delay buffer */
 enum {DELAY_BUF_MAX_DELAY = 80};
@@ -2038,7 +2082,7 @@ struct delaybuf_port
 };
 
 
-static pj_status_t delaybuf_get_frame(struct pjmedia_port *this_port,
+static pj_status_t delaybuf_get_frame(struct pjmedia_port *this_port, 
 				      pjmedia_frame *frame)
 {
     struct delaybuf_port *dp = (struct delaybuf_port*)this_port;
@@ -2061,7 +2105,7 @@ static pj_status_t delaybuf_get_frame(struct pjmedia_port *this_port,
     return PJ_SUCCESS;
 }
 
-static pj_status_t delaybuf_put_frame(struct pjmedia_port *this_port,
+static pj_status_t delaybuf_put_frame(struct pjmedia_port *this_port, 
 				      pjmedia_frame *frame)
 {
     struct delaybuf_port *dp = (struct delaybuf_port*)this_port;
@@ -2118,17 +2162,17 @@ static pjmedia_port* create_delaybuf(int drift_pct,
     dp->base.get_frame = &delaybuf_get_frame;
     dp->base.put_frame = &delaybuf_put_frame;
     dp->base.on_destroy = &delaybuf_on_destroy;
-    pjmedia_port_info_init(&dp->base.info, &name, 0x5678, clock_rate,
+    pjmedia_port_info_init(&dp->base.info, &name, 0x5678, clock_rate, 
 			   channel_count, 16, samples_per_frame);
 
-    status = pjmedia_delay_buf_create(pool, "mips_test", clock_rate,
-				      samples_per_frame, channel_count,
+    status = pjmedia_delay_buf_create(pool, "mips_test", clock_rate, 
+				      samples_per_frame, channel_count, 
 				      DELAY_BUF_MAX_DELAY,
 				      opt, &dp->delaybuf);
     if (status != PJ_SUCCESS)
 	return NULL;
 
-    dp->gen_port = create_gen_port(pool, clock_rate, channel_count,
+    dp->gen_port = create_gen_port(pool, clock_rate, channel_count, 
 				   samples_per_frame, 100);
     if (dp->gen_port == NULL)
 	return NULL;
@@ -2145,7 +2189,7 @@ static pjmedia_port* delaybuf_0( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(0, pool, clock_rate, channel_count,
+    return create_delaybuf(0, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2158,7 +2202,7 @@ static pjmedia_port* delaybuf_p2( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(2, pool, clock_rate, channel_count,
+    return create_delaybuf(2, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2170,7 +2214,7 @@ static pjmedia_port* delaybuf_p5( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(5, pool, clock_rate, channel_count,
+    return create_delaybuf(5, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2182,7 +2226,7 @@ static pjmedia_port* delaybuf_p10(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(10, pool, clock_rate, channel_count,
+    return create_delaybuf(10, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2194,7 +2238,7 @@ static pjmedia_port* delaybuf_p20(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(20, pool, clock_rate, channel_count,
+    return create_delaybuf(20, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2206,7 +2250,7 @@ static pjmedia_port* delaybuf_n2( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(-2, pool, clock_rate, channel_count,
+    return create_delaybuf(-2, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2218,7 +2262,7 @@ static pjmedia_port* delaybuf_n5( pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(-5, pool, clock_rate, channel_count,
+    return create_delaybuf(-5, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2230,7 +2274,7 @@ static pjmedia_port* delaybuf_n10(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(-10, pool, clock_rate, channel_count,
+    return create_delaybuf(-10, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2242,7 +2286,7 @@ static pjmedia_port* delaybuf_n20(pj_pool_t *pool,
 				  unsigned flags,
 				  struct test_entry *te)
 {
-    return create_delaybuf(-20, pool, clock_rate, channel_count,
+    return create_delaybuf(-20, pool, clock_rate, channel_count, 
 			   samples_per_frame, flags, te);
 }
 
@@ -2273,7 +2317,7 @@ static pj_timestamp run_entry(unsigned clock_rate, struct test_entry *e)
     /* Port may decide to use different ptime (e.g. iLBC) */
     samples_per_frame = PJMEDIA_PIA_SPF(&port->info);
 
-    gen_port = create_gen_port(pool, clock_rate, 1,
+    gen_port = create_gen_port(pool, clock_rate, 1, 
 			       samples_per_frame, 100);
     if (gen_port == NULL) {
 	t0.u64 = 0;
@@ -2419,6 +2463,9 @@ int mips_test(void)
 #if PJMEDIA_HAS_OPENCORE_AMRNB_CODEC
 	{ "codec encode/decode - AMR-NB", OP_PUT, K8, &amr_encode_decode},
 #endif
+#if PJMEDIA_HAS_OPENCORE_AMRWB_CODEC
+	{ "codec encode/decode - AMR-WB", OP_PUT, K16, &amrwb_encode_decode},
+#endif
 #if PJMEDIA_HAS_L16_CODEC
 	{ "codec encode/decode - L16/8000/1", OP_PUT, K8, &l16_8_encode_decode},
 	{ "codec encode/decode - L16/16000/1", OP_PUT, K16, &l16_16_encode_decode},
@@ -2446,6 +2493,9 @@ int mips_test(void)
 #endif
 #if PJMEDIA_HAS_OPENCORE_AMRNB_CODEC
 	{ "stream TX/RX - AMR-NB", OP_PUT_GET, K8, &create_stream_amr},
+#endif
+#if PJMEDIA_HAS_OPENCORE_AMRWB_CODEC
+	{ "stream TX/RX - AMR-WB", OP_PUT_GET, K16, &create_stream_amrwb},
 #endif
     };
 
@@ -2458,7 +2508,7 @@ int mips_test(void)
 
     for (c=0; c<PJ_ARRAY_SIZE(clock_rates); ++c) {
 	for (i=0; i<PJ_ARRAY_SIZE(entries); ++i) {
-	    enum
+	    enum 
 	    {
 		RETRY	= 5,	/* number of test retries */
 	    };
@@ -2491,14 +2541,14 @@ int mips_test(void)
 
 	    /* Calculate usec elapsed as average of two best times */
 	    tzero.u32.hi = tzero.u32.lo = 0;
-	    usec = (pj_elapsed_usec(&tzero, &times[0]) +
+	    usec = (pj_elapsed_usec(&tzero, &times[0]) + 
 		    pj_elapsed_usec(&tzero, &times[1])) / 2;
 
 	    usec = usec / (DURATION / 1000);
 
 	    mips_val = (float)(CPU_IPS * usec / 1000000.0 / 1000000);
 	    cpu_pct = (float)(100.0 * usec / 1000000);
-	    PJ_LOG(3,(THIS_FILE, "%2dKHz %-38s % 8d %8.3f %7.2f",
+	    PJ_LOG(3,(THIS_FILE, "%2dKHz %-38s % 8d %8.3f %7.2f", 
 		      clock_rate/1000, e->title, usec, cpu_pct, mips_val));
 
 	}
@@ -2506,3 +2556,6 @@ int mips_test(void)
 
     return 0;
 }
+
+
+
