@@ -1,5 +1,5 @@
-/* $Id: server.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: server.c 4537 2013-06-19 06:47:43Z riza $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include "server.h"
 #include "test.h"
@@ -104,9 +104,9 @@ pj_status_t create_test_server(pj_stun_config *stun_cfg,
 
 	pj_sockaddr_in_init(&bound_addr.ipv4, NULL, STUN_SERVER_PORT);
 
-	status = pj_activesock_create_udp(pool, &bound_addr, NULL,
+	status = pj_activesock_create_udp(pool, &bound_addr, NULL, 
 					  test_srv->stun_cfg->ioqueue,
-					  &stun_sock_cb, test_srv,
+					  &stun_sock_cb, test_srv, 
 					  &test_srv->stun_sock, NULL);
 	if (status != PJ_SUCCESS) {
 	    destroy_test_server(test_srv);
@@ -135,7 +135,7 @@ pj_status_t create_test_server(pj_stun_config *stun_cfg,
 	    pj_ansi_snprintf(strbuf, sizeof(strbuf),
 			     "stun.%s", domain);
 	    pj_strdup2(pool, &target, strbuf);
-	    pj_dns_init_srv_rr(&rr, &res_name, PJ_DNS_CLASS_IN, 60, 0, 0,
+	    pj_dns_init_srv_rr(&rr, &res_name, PJ_DNS_CLASS_IN, 60, 0, 0, 
 			       STUN_SERVER_PORT, &target);
 	    pj_dns_server_add_rec(test_srv->dns_server, 1, &rr);
 
@@ -156,7 +156,7 @@ pj_status_t create_test_server(pj_stun_config *stun_cfg,
 
 	pj_sockaddr_in_init(&bound_addr.ipv4, NULL, TURN_SERVER_PORT);
 
-	status = pj_activesock_create_udp(pool, &bound_addr, NULL,
+	status = pj_activesock_create_udp(pool, &bound_addr, NULL, 
 					  test_srv->stun_cfg->ioqueue,
 					  &turn_sock_cb, test_srv,
 					  &test_srv->turn_sock, NULL);
@@ -187,7 +187,7 @@ pj_status_t create_test_server(pj_stun_config *stun_cfg,
 	    pj_ansi_snprintf(strbuf, sizeof(strbuf),
 			     "turn.%s", domain);
 	    pj_strdup2(pool, &target, strbuf);
-	    pj_dns_init_srv_rr(&rr, &res_name, PJ_DNS_CLASS_IN, 60, 0, 0,
+	    pj_dns_init_srv_rr(&rr, &res_name, PJ_DNS_CLASS_IN, 60, 0, 0, 
 			       TURN_SERVER_PORT, &target);
 	    pj_dns_server_add_rec(test_srv->dns_server, 1, &rr);
 
@@ -254,14 +254,14 @@ static pj_bool_t stun_on_data_recvfrom(pj_activesock_t *asock,
     test_srv = (test_server*) pj_activesock_get_user_data(asock);
     pool = pj_pool_create(test_srv->stun_cfg->pf, NULL, 512, 512, NULL);
 
-    status = pj_stun_msg_decode(pool, (pj_uint8_t*)data, size,
-				PJ_STUN_IS_DATAGRAM | PJ_STUN_CHECK_PACKET,
+    status = pj_stun_msg_decode(pool, (pj_uint8_t*)data, size, 
+				PJ_STUN_IS_DATAGRAM | PJ_STUN_CHECK_PACKET, 
 				&req, NULL, NULL);
     if (status != PJ_SUCCESS)
 	goto on_return;
 
     if (req->hdr.type != PJ_STUN_BINDING_REQUEST) {
-	pj_stun_msg_create_response(pool, req, PJ_STUN_SC_BAD_REQUEST,
+	pj_stun_msg_create_response(pool, req, PJ_STUN_SC_BAD_REQUEST, 
 				    NULL, &resp);
 	goto send_pkt;
     }
@@ -274,7 +274,7 @@ static pj_bool_t stun_on_data_recvfrom(pj_activesock_t *asock,
 				  PJ_TRUE, src_addr, addr_len);
 
 send_pkt:
-    status = pj_stun_msg_encode(resp, (pj_uint8_t*)data, MAX_STUN_PKT,
+    status = pj_stun_msg_encode(resp, (pj_uint8_t*)data, MAX_STUN_PKT, 
 				0, NULL, &size);
     if (status != PJ_SUCCESS)
 	goto on_return;
@@ -368,7 +368,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 
 	if (i==test_srv->turn_alloc_cnt) {
 	    /* Invalid data */
-	    PJ_LOG(1,(THIS_FILE,
+	    PJ_LOG(1,(THIS_FILE, 
 		      "TURN Server received strayed data"));
 	    goto on_return;
 	}
@@ -380,7 +380,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 
 	/* For UDP check the packet length */
 	if (size < cd.length+sizeof(cd)) {
-	    PJ_LOG(1,(THIS_FILE,
+	    PJ_LOG(1,(THIS_FILE, 
 		      "TURN Server: ChannelData discarded: UDP size error"));
 	    goto on_return;
 	}
@@ -392,7 +392,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 	}
 
 	if (i==alloc->perm_cnt) {
-	    PJ_LOG(1,(THIS_FILE,
+	    PJ_LOG(1,(THIS_FILE, 
 		      "TURN Server: ChannelData discarded: invalid channel number"));
 	    goto on_return;
 	}
@@ -408,9 +408,9 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 	goto on_return;
     }
 
-    status = pj_stun_msg_decode(pool, (pj_uint8_t*)data, size,
+    status = pj_stun_msg_decode(pool, (pj_uint8_t*)data, size, 
 				PJ_STUN_IS_DATAGRAM | PJ_STUN_CHECK_PACKET |
-				    PJ_STUN_NO_FINGERPRINT_CHECK,
+				    PJ_STUN_NO_FINGERPRINT_CHECK, 
 				&req, NULL, NULL);
     if (status != PJ_SUCCESS) {
 	char errmsg[PJ_ERR_MSG_SIZE];
@@ -458,7 +458,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 	 * the user is incorrect
 	 */
 	if (pj_stun_msg_find_attr(req, PJ_STUN_ATTR_MESSAGE_INTEGRITY, 0) == NULL ||
-	    uname==NULL || pj_stricmp2(&uname->value, TURN_USERNAME) != 0)
+	    uname==NULL || pj_stricmp2(&uname->value, TURN_USERNAME) != 0) 
 	{
 	    pj_str_t tmp;
 
@@ -484,9 +484,9 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 	pj_sockaddr_in_init(&alloc->alloc_addr.ipv4, NULL, 0);
 	pj_gethostip(pj_AF_INET(), &alloc->alloc_addr);
 
-	status = pj_activesock_create_udp(alloc->pool, &alloc->alloc_addr, NULL,
+	status = pj_activesock_create_udp(alloc->pool, &alloc->alloc_addr, NULL, 
 					  test_srv->stun_cfg->ioqueue,
-					  &alloc_sock_cb, alloc,
+					  &alloc_sock_cb, alloc, 
 					  &alloc->sock, &alloc->alloc_addr);
 	if (status != PJ_SUCCESS) {
 	    pj_pool_release(alloc->pool);
@@ -514,7 +514,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 	    pj_stun_msg_create_response(pool, req, PJ_STUN_SC_SERVER_ERROR, NULL, &resp);
 	    goto send_pkt;
 	}
-	pj_stun_msg_add_sockaddr_attr(alloc->pool, alloc->data_ind,
+	pj_stun_msg_add_sockaddr_attr(alloc->pool, alloc->data_ind, 
 				      PJ_STUN_ATTR_XOR_PEER_ADDR, PJ_TRUE,
 				      &alloc->alloc_addr,
 				      pj_sockaddr_get_len(&alloc->alloc_addr));
@@ -581,7 +581,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 			pj_sockaddr_cp(&alloc->perm[alloc->perm_cnt], &pa->sockaddr);
 			++alloc->perm_cnt;
 
-			PJ_LOG(5,("", "Permission %s added to client %s, perm_cnt=%d",
+			PJ_LOG(5,("", "Permission %s added to client %s, perm_cnt=%d", 
 				      peer_info, client_info, alloc->perm_cnt));
 		    }
 
@@ -611,11 +611,11 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 		}
 
 		if (j==alloc->perm_cnt) {
-		    PJ_LOG(5,("", "SendIndication to %s is rejected (no permission)",
+		    PJ_LOG(5,("", "SendIndication to %s is rejected (no permission)", 
 			          peer_info, client_info, alloc->perm_cnt));
 		} else {
 		    PJ_LOG(5,(THIS_FILE, "Relaying %d bytes data from client %s to peer %s, "
-					 "perm_cnt=%d",
+					 "perm_cnt=%d", 
 			      da->length, client_info, peer_info, alloc->perm_cnt));
 
 		    sent = da->length;
@@ -665,7 +665,7 @@ static pj_bool_t turn_on_data_recvfrom(pj_activesock_t *asock,
 
 send_pkt:
     if (resp) {
-	status = pj_stun_msg_encode(resp, (pj_uint8_t*)data, MAX_STUN_PKT,
+	status = pj_stun_msg_encode(resp, (pj_uint8_t*)data, MAX_STUN_PKT, 
 				    0, &auth_key, &size);
 	if (status != PJ_SUCCESS)
 	    goto on_return;
@@ -716,7 +716,7 @@ static pj_bool_t alloc_on_data_recvfrom(pj_activesock_t *asock,
 	}
     }
     if (i==alloc->perm_cnt) {
-	PJ_LOG(5,("", "Client %s received %d bytes unauthorized data from peer %s",
+	PJ_LOG(5,("", "Client %s received %d bytes unauthorized data from peer %s", 
 		      client_info, size, peer_info));
 	if (alloc->perm_cnt == 0)
 	    PJ_LOG(5,("", "Client %s has no permission", client_info));
@@ -732,7 +732,7 @@ static pj_bool_t alloc_on_data_recvfrom(pj_activesock_t *asock,
 
     pj_sockaddr_cp(&pa->sockaddr, src_addr);
     da->data = (pj_uint8_t*)data;
-    da->length = size;
+    da->length = (unsigned)size;
 
     /* Encode Data indication */
     status = pj_stun_msg_encode(alloc->data_ind, buffer, sizeof(buffer), 0,
@@ -742,7 +742,7 @@ static pj_bool_t alloc_on_data_recvfrom(pj_activesock_t *asock,
 
     /* Send */
     sent = size;
-    PJ_LOG(5,("", "Forwarding %d bytes data from peer %s to client %s",
+    PJ_LOG(5,("", "Forwarding %d bytes data from peer %s to client %s", 
 		   sent, peer_info, client_info));
 
     pj_activesock_sendto(alloc->test_srv->turn_sock, &alloc->send_key, buffer,
@@ -751,3 +751,4 @@ static pj_bool_t alloc_on_data_recvfrom(pj_activesock_t *asock,
 
     return PJ_TRUE;
 }
+
