@@ -1,5 +1,5 @@
-/* $Id: nat_detect.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: nat_detect.c 4573 2013-07-24 08:06:59Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjnath/nat_detect.h>
 #include <pjnath/errno.h>
@@ -75,7 +75,7 @@ enum timer_type
 typedef struct nat_detect_session
 {
     pj_pool_t		    *pool;
-    pj_mutex_t		    *mutex;
+    pj_grp_lock_t	    *grp_lock;
 
     pj_timer_heap_t	    *timer_heap;
     pj_timer_entry	     timer;
@@ -109,8 +109,8 @@ typedef struct nat_detect_session
 } nat_detect_session;
 
 
-static void on_read_complete(pj_ioqueue_key_t *key,
-                             pj_ioqueue_op_key_t *op_key,
+static void on_read_complete(pj_ioqueue_key_t *key, 
+                             pj_ioqueue_op_key_t *op_key, 
                              pj_ssize_t bytes_read);
 static void on_request_complete(pj_stun_session *sess,
 			        pj_status_t status,
@@ -133,7 +133,7 @@ static pj_status_t send_test(nat_detect_session *sess,
 static void on_sess_timer(pj_timer_heap_t *th,
 			     pj_timer_entry *te);
 static void sess_destroy(nat_detect_session *sess);
-
+static void sess_on_destroy(void *member);
 
 /*
  * Get the NAT name from the specified NAT type.
@@ -198,7 +198,7 @@ static pj_status_t get_local_interface(const pj_sockaddr_in *server,
     }
 
     local_addr->s_addr = tmp.sin_addr.s_addr;
-
+    
     pj_sock_close(sock);
     return PJ_SUCCESS;
 }
@@ -223,7 +223,7 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
     /*
      * Init NAT detection session.
      */
-    pool = pj_pool_create(stun_cfg->pf, "natck%p", PJNATH_POOL_LEN_NATCK,
+    pool = pj_pool_create(stun_cfg->pf, "natck%p", PJNATH_POOL_LEN_NATCK, 
 			  PJNATH_POOL_INC_NATCK, NULL);
     if (!pool)
 	return PJ_ENOMEM;
@@ -233,9 +233,15 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
     sess->user_data = user_data;
     sess->cb = cb;
 
-    status = pj_mutex_create_recursive(pool, pool->obj_name, &sess->mutex);
-    if (status != PJ_SUCCESS)
-	goto on_error;
+    status = pj_grp_lock_create(pool, NULL, &sess->grp_lock);
+    if (status != PJ_SUCCESS) {
+	/* Group lock not created yet, just destroy pool and return */
+	pj_pool_release(pool);
+	return status;
+    }
+
+    pj_grp_lock_add_ref(sess->grp_lock);
+    pj_grp_lock_add_handler(sess->grp_lock, pool, sess, &sess_on_destroy);
 
     pj_memcpy(&sess->server, server, sizeof(pj_sockaddr_in));
 
@@ -259,7 +265,7 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
      */
     pj_bzero(&sess->local_addr, sizeof(pj_sockaddr_in));
     sess->local_addr.sin_family = pj_AF_INET();
-    status = pj_sock_bind(sess->sock, &sess->local_addr,
+    status = pj_sock_bind(sess->sock, &sess->local_addr, 
 			  sizeof(pj_sockaddr_in));
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -280,11 +286,11 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
 	goto on_error;
 
     PJ_LOG(5,(sess->pool->obj_name, "Local address is %s:%d",
-	      pj_inet_ntoa(sess->local_addr.sin_addr),
+	      pj_inet_ntoa(sess->local_addr.sin_addr), 
 	      pj_ntohs(sess->local_addr.sin_port)));
 
     PJ_LOG(5,(sess->pool->obj_name, "Server set to %s:%d",
-	      pj_inet_ntoa(server->sin_addr),
+	      pj_inet_ntoa(server->sin_addr), 
 	      pj_ntohs(server->sin_port)));
 
     /*
@@ -294,9 +300,9 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
     pj_bzero(&ioqueue_cb, sizeof(ioqueue_cb));
     ioqueue_cb.on_read_complete = &on_read_complete;
 
-    status = pj_ioqueue_register_sock(sess->pool, stun_cfg->ioqueue,
-				      sess->sock, sess, &ioqueue_cb,
-				      &sess->key);
+    status = pj_ioqueue_register_sock2(sess->pool, stun_cfg->ioqueue, 
+				       sess->sock, sess->grp_lock, sess,
+				       &ioqueue_cb, &sess->key);
     if (status != PJ_SUCCESS)
 	goto on_error;
 
@@ -307,7 +313,7 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
     sess_cb.on_request_complete = &on_request_complete;
     sess_cb.on_send_msg = &on_send_msg;
     status = pj_stun_session_create(stun_cfg, pool->obj_name, &sess_cb,
-				    PJ_FALSE, &sess->stun_sess);
+				    PJ_FALSE, sess->grp_lock, &sess->stun_sess);
     if (status != PJ_SUCCESS)
 	goto on_error;
 
@@ -336,26 +342,33 @@ on_error:
 
 static void sess_destroy(nat_detect_session *sess)
 {
-    if (sess->stun_sess) {
+    if (sess->stun_sess) { 
 	pj_stun_session_destroy(sess->stun_sess);
+	sess->stun_sess = NULL;
     }
 
     if (sess->key) {
 	pj_ioqueue_unregister(sess->key);
+	sess->key = NULL;
+	sess->sock = PJ_INVALID_SOCKET;
     } else if (sess->sock && sess->sock != PJ_INVALID_SOCKET) {
 	pj_sock_close(sess->sock);
+	sess->sock = PJ_INVALID_SOCKET;
     }
 
-    if (sess->mutex) {
-	pj_mutex_destroy(sess->mutex);
+    if (sess->grp_lock) {
+	pj_grp_lock_dec_ref(sess->grp_lock);
     }
+}
 
+static void sess_on_destroy(void *member)
+{
+    nat_detect_session *sess = (nat_detect_session*)member;
     if (sess->pool) {
 	pj_pool_release(sess->pool);
     }
 }
 
-
 static void end_session(nat_detect_session *sess,
 			pj_status_t status,
 			pj_stun_nat_type nat_type)
@@ -392,8 +405,8 @@ static void end_session(nat_detect_session *sess,
 /*
  * Callback upon receiving packet from network.
  */
-static void on_read_complete(pj_ioqueue_key_t *key,
-                             pj_ioqueue_op_key_t *op_key,
+static void on_read_complete(pj_ioqueue_key_t *key, 
+                             pj_ioqueue_op_key_t *op_key, 
                              pj_ssize_t bytes_read)
 {
     nat_detect_session *sess;
@@ -402,22 +415,27 @@ static void on_read_complete(pj_ioqueue_key_t *key,
     sess = (nat_detect_session *) pj_ioqueue_get_user_data(key);
     pj_assert(sess != NULL);
 
-    pj_mutex_lock(sess->mutex);
+    pj_grp_lock_acquire(sess->grp_lock);
+
+    /* Ignore packet when STUN session has been destroyed */
+    if (!sess->stun_sess)
+	goto on_return;
 
     if (bytes_read < 0) {
 	if (-bytes_read != PJ_STATUS_FROM_OS(OSERR_EWOULDBLOCK) &&
-	    -bytes_read != PJ_STATUS_FROM_OS(OSERR_EINPROGRESS) &&
-	    -bytes_read != PJ_STATUS_FROM_OS(OSERR_ECONNRESET))
+	    -bytes_read != PJ_STATUS_FROM_OS(OSERR_EINPROGRESS) && 
+	    -bytes_read != PJ_STATUS_FROM_OS(OSERR_ECONNRESET)) 
 	{
 	    /* Permanent error */
-	    end_session(sess, -bytes_read, PJ_STUN_NAT_TYPE_ERR_UNKNOWN);
+	    end_session(sess, (pj_status_t)-bytes_read, 
+			PJ_STUN_NAT_TYPE_ERR_UNKNOWN);
 	    goto on_return;
 	}
 
     } else if (bytes_read > 0) {
 	pj_stun_session_on_rx_pkt(sess->stun_sess, sess->rx_pkt, bytes_read,
-				  PJ_STUN_IS_DATAGRAM|PJ_STUN_CHECK_PACKET,
-				  NULL, NULL,
+				  PJ_STUN_IS_DATAGRAM|PJ_STUN_CHECK_PACKET, 
+				  NULL, NULL, 
 				  &sess->src_addr, sess->src_addr_len);
     }
 
@@ -425,7 +443,7 @@ static void on_read_complete(pj_ioqueue_key_t *key,
     sess->rx_pkt_len = sizeof(sess->rx_pkt);
     sess->src_addr_len = sizeof(sess->src_addr);
     status = pj_ioqueue_recvfrom(key, op_key, sess->rx_pkt, &sess->rx_pkt_len,
-				 PJ_IOQUEUE_ALWAYS_ASYNC,
+				 PJ_IOQUEUE_ALWAYS_ASYNC, 
 				 &sess->src_addr, &sess->src_addr_len);
 
     if (status != PJ_EPENDING) {
@@ -434,7 +452,7 @@ static void on_read_complete(pj_ioqueue_key_t *key,
     }
 
 on_return:
-    pj_mutex_unlock(sess->mutex);
+    pj_grp_lock_release(sess->grp_lock);
 }
 
 
@@ -489,7 +507,7 @@ static void on_request_complete(pj_stun_session *stun_sess,
 
     sess = (nat_detect_session*) pj_stun_session_get_user_data(stun_sess);
 
-    pj_mutex_lock(sess->mutex);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     /* Find errors in the response */
     if (status == PJ_SUCCESS) {
@@ -562,11 +580,11 @@ static void on_request_complete(pj_stun_session *stun_sess,
     /* Send Test 1B only when Test 2 completes. Must not send Test 1B
      * before Test 2 completes to avoid creating mapping on the NAT.
      */
-    if (!sess->result[ST_TEST_1B].executed &&
+    if (!sess->result[ST_TEST_1B].executed && 
 	sess->result[ST_TEST_2].complete &&
 	sess->result[ST_TEST_2].status != PJ_SUCCESS &&
 	sess->result[ST_TEST_1].complete &&
-	sess->result[ST_TEST_1].status == PJ_SUCCESS)
+	sess->result[ST_TEST_1].status == PJ_SUCCESS) 
     {
 	cmp = pj_memcmp(&sess->local_addr, &sess->result[ST_TEST_1].ma,
 			sizeof(pj_sockaddr_in));
@@ -629,7 +647,7 @@ static void on_request_complete(pj_stun_session *stun_sess,
     switch (sess->result[ST_TEST_1].status) {
     case PJNATH_ESTUNTIMEDOUT:
 	/*
-	 * Test 1 has timed-out. Conclude with NAT_TYPE_BLOCKED.
+	 * Test 1 has timed-out. Conclude with NAT_TYPE_BLOCKED. 
 	 */
 	end_session(sess, PJ_SUCCESS, PJ_STUN_NAT_TYPE_BLOCKED);
 	break;
@@ -663,7 +681,7 @@ static void on_request_complete(pj_stun_session *stun_sess,
 		/*
 		 * We've got other error with Test 2.
 		 */
-		end_session(sess, sess->result[ST_TEST_2].status,
+		end_session(sess, sess->result[ST_TEST_2].status, 
 			    PJ_STUN_NAT_TYPE_ERR_UNKNOWN);
 		break;
 	    }
@@ -744,14 +762,14 @@ static void on_request_complete(pj_stun_session *stun_sess,
 			 * It could be that port 3489 is blocked, while the
 			 * NAT itself looks to be a Restricted one.
 			 */
-			end_session(sess, PJ_SUCCESS,
+			end_session(sess, PJ_SUCCESS, 
 				    PJ_STUN_NAT_TYPE_RESTRICTED);
 			break;
 		    default:
 			/* Can't distinguish between Symmetric and Port
 			 * Restricted, so set the type to Unknown
 			 */
-			end_session(sess, PJ_SUCCESS,
+			end_session(sess, PJ_SUCCESS, 
 				    PJ_STUN_NAT_TYPE_ERR_UNKNOWN);
 			break;
 		    }
@@ -769,7 +787,7 @@ static void on_request_complete(pj_stun_session *stun_sess,
 		/*
 		 * We've got other error with Test 2.
 		 */
-		end_session(sess, sess->result[ST_TEST_2].status,
+		end_session(sess, sess->result[ST_TEST_2].status, 
 			    PJ_STUN_NAT_TYPE_ERR_UNKNOWN);
 		break;
 	    }
@@ -779,13 +797,13 @@ static void on_request_complete(pj_stun_session *stun_sess,
 	/*
 	 * We've got other error with Test 1.
 	 */
-	end_session(sess, sess->result[ST_TEST_1].status,
+	end_session(sess, sess->result[ST_TEST_1].status, 
 		    PJ_STUN_NAT_TYPE_ERR_UNKNOWN);
 	break;
     }
 
 on_return:
-    pj_mutex_unlock(sess->mutex);
+    pj_grp_lock_release(sess->grp_lock);
 }
 
 
@@ -810,15 +828,15 @@ static pj_status_t send_test(nat_detect_session *sess,
     tsx_id[2] = test_id;
 
     /* Create BIND request */
-    status = pj_stun_session_create_req(sess->stun_sess,
+    status = pj_stun_session_create_req(sess->stun_sess, 
 					PJ_STUN_BINDING_REQUEST, magic,
-					(pj_uint8_t*)tsx_id,
+					(pj_uint8_t*)tsx_id, 
 					&sess->result[test_id].tdata);
     if (status != PJ_SUCCESS)
 	goto on_error;
 
     /* Add CHANGE-REQUEST attribute */
-    status = pj_stun_msg_add_uint_attr(sess->pool,
+    status = pj_stun_msg_add_uint_attr(sess->pool, 
 				       sess->result[test_id].tdata->msg,
 				       PJ_STUN_ATTR_CHANGE_REQUEST,
 				       change_flag);
@@ -831,15 +849,15 @@ static pj_status_t send_test(nat_detect_session *sess,
     else
 	sess->cur_server = &sess->server;
 
-    PJ_LOG(5,(sess->pool->obj_name,
-              "Performing %s to %s:%d",
+    PJ_LOG(5,(sess->pool->obj_name, 
+              "Performing %s to %s:%d", 
 	      test_names[test_id],
 	      pj_inet_ntoa(sess->cur_server->sin_addr),
 	      pj_ntohs(sess->cur_server->sin_port)));
 
     /* Send the request */
     status = pj_stun_session_send_msg(sess->stun_sess, NULL, PJ_TRUE,
-				      PJ_TRUE, sess->cur_server,
+				      PJ_TRUE, sess->cur_server, 
 				      sizeof(pj_sockaddr_in),
 				      sess->result[test_id].tdata);
     if (status != PJ_SUCCESS)
@@ -864,12 +882,12 @@ static void on_sess_timer(pj_timer_heap_t *th,
     sess = (nat_detect_session*) te->user_data;
 
     if (te->id == TIMER_DESTROY) {
-	pj_mutex_lock(sess->mutex);
+	pj_grp_lock_acquire(sess->grp_lock);
 	pj_ioqueue_unregister(sess->key);
 	sess->key = NULL;
 	sess->sock = PJ_INVALID_SOCKET;
 	te->id = 0;
-	pj_mutex_unlock(sess->mutex);
+	pj_grp_lock_release(sess->grp_lock);
 
 	sess_destroy(sess);
 
@@ -877,7 +895,7 @@ static void on_sess_timer(pj_timer_heap_t *th,
 
 	pj_bool_t next_timer;
 
-	pj_mutex_lock(sess->mutex);
+	pj_grp_lock_acquire(sess->grp_lock);
 
 	next_timer = PJ_FALSE;
 
@@ -902,9 +920,10 @@ static void on_sess_timer(pj_timer_heap_t *th,
 	    te->id = 0;
 	}
 
-	pj_mutex_unlock(sess->mutex);
+	pj_grp_lock_release(sess->grp_lock);
 
     } else {
 	pj_assert(!"Invalid timer ID");
     }
 }
+
