@@ -1,5 +1,5 @@
-/* $Id: sip_config.h 4172 2012-06-19 14:35:18Z nanang $ */
-/*
+/* $Id: sip_config.h 4770 2014-02-27 06:16:36Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJSIP_SIP_CONFIG_H__
 #define __PJSIP_SIP_CONFIG_H__
@@ -29,7 +29,7 @@
 /**
  * @defgroup PJSIP_CORE Core SIP Library
  * @brief The core framework from which all other SIP components depends on.
- *
+ * 
  * The PJSIP Core library only provides transport framework, event
  * dispatching/module framework, and SIP message representation and
  * parsing. It doesn't do anything usefull in itself!
@@ -73,21 +73,70 @@ typedef struct pjsip_cfg_t
 	/**
 	 * Specify port number should be allowed to appear in To and From
 	 * header. Note that RFC 3261 disallow this, see Table 1 in section
-	 * 19.1.1 of the RFC. Default is PJSIP_ALLOW_PORT_IN_FROMTO_HDR.
+	 * 19.1.1 of the RFC.
+	 *
+	 * Default is PJSIP_ALLOW_PORT_IN_FROMTO_HDR.
 	 */
 	pj_bool_t allow_port_in_fromto_hdr;
 
+	/**
+	 * Accept call replace in early state when invite is not initiated
+	 * by the user agent. RFC 3891 Section 3 disallows this, however,
+	 * for better interoperability reason, this might be ignored.
+	 *
+	 * Default is PJSIP_ACCEPT_REPLACE_IN_EARLY_STATE.
+	 */
+	pj_bool_t accept_replace_in_early_state;
+
+	/**
+	 * Allow hash character ('#') to appear in outgoing URIs. See
+	 * https://trac.pjsip.org/repos/ticket/1569.
+	 *
+	 * Default is PJ_FALSE.
+	 */
+	pj_bool_t allow_tx_hash_in_uri;
+
 	/**
 	 * Disable rport in request.
+	 *
+	 * Default is PJ_FALSE.
 	 */
 	pj_bool_t disable_rport;
 
 	/**
 	 * Disable automatic switching from UDP to TCP if outgoing request
-	 * is greater than 1300 bytes. See PJSIP_DONT_SWITCH_TO_TCP.
+	 * is greater than 1300 bytes.
+	 *
+	 * Default is PJSIP_DONT_SWITCH_TO_TCP.
 	 */
 	pj_bool_t disable_tcp_switch;
 
+	/**
+	 * Disable automatic switching to TLS if target-URI does not use
+	 * "sips" scheme nor TLS transport, even when request-URI uses
+	 * "sips" scheme.
+	 *
+	 * Default is PJSIP_DONT_SWITCH_TO_TLS.
+	 */
+	pj_bool_t disable_tls_switch;
+
+	/**
+	 * Enable call media session to always be updated to the latest
+	 * received early media SDP when receiving forked early media
+	 * (multiple 183 responses with different To tag).
+	 *
+	 * Default is PJSIP_FOLLOW_EARLY_MEDIA_FORK.
+	 */
+	pj_bool_t follow_early_media_fork;
+
+	/**
+	 * Specify whether "alias" param should be added to the Via header
+	 * in any outgoing request with connection oriented transport.
+	 *
+	 * Default is PJSIP_REQ_HAS_VIA_ALIAS.
+	 */
+	pj_bool_t req_has_via_alias;
+
     } endpt;
 
     /** Transaction layer settings. */
@@ -125,9 +174,9 @@ typedef struct pjsip_cfg_t
     /** Client registration settings. */
     struct {
 	/**
-	 * Specify whether client registration should check for its
-	 * registered contact in Contact header of successful REGISTER
-	 * response to determine whether registration has been successful.
+	 * Specify whether client registration should check for its 
+	 * registered contact in Contact header of successful REGISTER 
+	 * response to determine whether registration has been successful. 
 	 * This setting may be disabled if non-compliant registrar is unable
 	 * to return correct Contact header.
 	 *
@@ -138,7 +187,7 @@ typedef struct pjsip_cfg_t
 	/**
 	 * Specify whether client registration should add "x-uid" extension
 	 * parameter in all Contact URIs that it registers to assist the
-	 * matching of Contact URIs in the 200/OK REGISTER response, in
+	 * matching of Contact URIs in the 200/OK REGISTER response, in 
 	 * case the registrar is unable to return exact Contact URI in the
 	 * 200/OK response.
 	 *
@@ -212,7 +261,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 
 /**
- * Transport manager hash table size (must be 2^n-1).
+ * Transport manager hash table size (must be 2^n-1). 
  * See also PJSIP_MAX_TRANSPORTS
  */
 #ifndef PJSIP_TPMGR_HTABLE_SIZE
@@ -222,7 +271,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 /**
  * Specify maximum URL size.
- * This constant is used mainly when printing the URL for logging purpose
+ * This constant is used mainly when printing the URL for logging purpose 
  * only.
  */
 #ifndef PJSIP_MAX_URL_SIZE
@@ -268,6 +317,68 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 #endif
 
 
+/**
+ * As specified RFC 3261 section 8.1.2, when request-URI uses "sips" scheme,
+ * TLS must always be used regardless of the target-URI scheme or transport
+ * type.
+ *
+ * This option will specify whether the behavior of automatic switching to TLS
+ * should be disabled, i.e: regard the target-URI scheme or transport type.
+ *
+ * This option can also be controlled at run-time by the \a disable_tls_switch
+ * setting in pjsip_cfg_t.
+ *
+ * Default is 0 (no).
+ */
+#ifndef PJSIP_DONT_SWITCH_TO_TLS
+#   define PJSIP_DONT_SWITCH_TO_TLS	0
+#endif
+
+
+/**
+ * Specify whether the call media session should be updated to the latest
+ * received early media SDP when receiving forked early media (multiple 183
+ * responses with different To tag).
+ *
+ * This option can also be controlled at run-time by the
+ * \a follow_early_media_fork setting in pjsip_cfg_t.
+ *
+ * Default is PJ_TRUE.
+ */
+#ifndef PJSIP_FOLLOW_EARLY_MEDIA_FORK
+#   define PJSIP_FOLLOW_EARLY_MEDIA_FORK	    PJ_TRUE
+#endif
+
+
+/**
+ * Specify whether "alias" param should be added to the Via header
+ * in any outgoing request with connection oriented transport.
+ *
+ * This option can also be controlled at run-time by the
+ * \a req_has_via_alias setting in pjsip_cfg_t.
+ *
+ * Default is PJ_TRUE.
+ */
+#ifndef PJSIP_REQ_HAS_VIA_ALIAS
+#   define PJSIP_REQ_HAS_VIA_ALIAS		    PJ_TRUE
+#endif
+
+
+/**
+ * Accept call replace in early state when invite is not initiated
+ * by the user agent. RFC 3891 Section 3 disallows this, however,
+ * for better interoperability reason, this might be ignored.
+ *
+ * This option can also be controlled at run-time by the
+ * \a accept_replace_in_early_state setting in pjsip_cfg_t.
+ *
+ * Default is 0 (no).
+ */
+#ifndef PJSIP_ACCEPT_REPLACE_IN_EARLY_STATE
+#   define PJSIP_ACCEPT_REPLACE_IN_EARLY_STATE	    0
+#endif
+
+
 /**
  * This setting controls the threshold of the UDP packet, which if it's
  * larger than this value the request will be sent with TCP. This setting
@@ -282,9 +393,9 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 /**
  * Encode SIP headers in their short forms to reduce size. By default,
- * SIP headers in outgoing messages will be encoded in their full names.
+ * SIP headers in outgoing messages will be encoded in their full names. 
  * If this option is enabled, then SIP headers for outgoing messages
- * will be encoded in their short forms, to reduce message size.
+ * will be encoded in their short forms, to reduce message size. 
  * Note that this does not affect the ability of PJSIP to parse incoming
  * SIP messages, as the parser always supports parsing both the long
  * and short version of the headers.
@@ -292,12 +403,12 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
  * Note that there is also an undocumented variable defined in sip_msg.c
  * to control whether compact form should be used for encoding SIP
  * headers. The default value of this variable is PJSIP_ENCODE_SHORT_HNAME.
- * To change PJSIP behavior during run-time, application can use the
+ * To change PJSIP behavior during run-time, application can use the 
  * following construct:
  *
  \verbatim
    extern pj_bool_t pjsip_use_compact_form;
-
+ 
    // enable compact form
    pjsip_use_compact_form = PJ_TRUE;
  \endverbatim
@@ -316,14 +427,14 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
  * allowed within dialog.
  *
  * Note that there is also an undocumented variable defined in sip_dialog.c
- * to control whether Allow header should be included. The default value
+ * to control whether Allow header should be included. The default value 
  * of this variable is PJSIP_INCLUDE_ALLOW_HDR_IN_DLG.
- * To change PJSIP behavior during run-time, application can use the
+ * To change PJSIP behavior during run-time, application can use the 
  * following construct:
  *
  \verbatim
    extern pj_bool_t pjsip_include_allow_hdr_in_dlg;
-
+ 
    // do not transmit Allow header
    pjsip_include_allow_hdr_in_dlg = PJ_FALSE;
  \endverbatim
@@ -358,13 +469,17 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
  *
  * The default behavior is yes, but when the UA supports IP address change
  * for the SIP transport, it will need to turn this checking off since
- * when the transport address is changed between request is sent and
+ * when the transport address is changed between request is sent and 
  * response is received, the response will be discarded since its Via
  * sent-by now contains address that is different than the transport
  * address.
+ *
+ * Update:
+ * As of version 2.1, the default value is 0. This change was part of
+ * https://trac.pjsip.org/repos/ticket/1412
  */
 #ifndef PJSIP_CHECK_VIA_SENT_BY
-#   define PJSIP_CHECK_VIA_SENT_BY	1
+#   define PJSIP_CHECK_VIA_SENT_BY	0
 #endif
 
 
@@ -417,8 +532,8 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 
 /**
- * Max entries to process in timer heap per poll.
- *
+ * Max entries to process in timer heap per poll. 
+ * 
  * Default: 10
  */
 #ifndef PJSIP_MAX_TIMED_OUT_ENTRIES
@@ -460,7 +575,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
  * created. This only applies for ephemeral transports such as TCP.
  *
  * Currently this is not used.
- *
+ * 
  * Default: -1
  */
 #ifndef PJSIP_MAX_TRANSPORT_USAGE
@@ -480,6 +595,24 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 #endif
 
 
+/**
+ * Specify whether TCP listener should use SO_REUSEADDR option. This constant
+ * will be used as the default value for the "reuse_addr" field in the
+ * pjsip_tcp_transport_cfg structure.
+ *
+ * Default is FALSE on Windows and TRUE on non-Windows.
+ *
+ * @see PJSIP_TLS_TRANSPORT_REUSEADDR
+ */
+#ifndef PJSIP_TCP_TRANSPORT_REUSEADDR
+# if (defined(PJ_WIN32) && PJ_WIN32) || (defined(PJ_WIN64) && PJ_WIN64)
+#   define PJSIP_TCP_TRANSPORT_REUSEADDR	0
+# else
+#   define PJSIP_TCP_TRANSPORT_REUSEADDR	1
+# endif
+#endif
+
+
 /**
  * Set the interval to send keep-alive packet for TCP transports.
  * If the value is zero, keep-alive will be disabled for TCP.
@@ -551,8 +684,8 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 #endif
 
 
-/**
- * Maximum number of addresses returned by the resolver. The number here
+/** 
+ * Maximum number of addresses returned by the resolver. The number here 
  * will slightly affect stack usage, since each entry will occupy about
  * 32 bytes of stack memory.
  *
@@ -588,6 +721,21 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 #endif
 
 
+/**
+ * Specify whether TLS listener should use SO_REUSEADDR option.
+ *
+ * Default is FALSE on Windows and TRUE on non-Windows.
+ *
+ * @see PJSIP_TCP_TRANSPORT_REUSEADDR
+ */
+#ifndef PJSIP_TLS_TRANSPORT_REUSEADDR
+# if (defined(PJ_WIN32) && PJ_WIN32) || (defined(PJ_WIN64) && PJ_WIN64)
+#   define PJSIP_TLS_TRANSPORT_REUSEADDR	0
+# else
+#   define PJSIP_TLS_TRANSPORT_REUSEADDR	1
+# endif
+#endif
+
 
 /* Endpoint. */
 #define PJSIP_MAX_TIMER_COUNT		(2*pjsip_cfg()->tsx.max_count + \
@@ -664,7 +812,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 /**
  * Initial memory size for transaction layer. The bulk of pool usage
- * for transaction layer will be used to create the hash table, so
+ * for transaction layer will be used to create the hash table, so 
  * setting this value too high will not help too much with reducing
  * fragmentation and the memory will most likely be wasted.
  */
@@ -674,7 +822,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 /**
  * Memory increment for transaction layer. The bulk of pool usage
- * for transaction layer will be used to create the hash table, so
+ * for transaction layer will be used to create the hash table, so 
  * setting this value too high will not help too much with reducing
  * fragmentation and the memory will most likely be wasted.
  */
@@ -729,7 +877,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 #define PJSIP_MAX_URI_TYPES		4
 
 /*****************************************************************************
- *  Default timeout settings, in miliseconds.
+ *  Default timeout settings, in miliseconds. 
  */
 
 /** Transaction T1 timeout value. */
@@ -821,6 +969,9 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
  * Specify support for IMS/3GPP digest AKA authentication version 1 and 2
  * (AKAv1-MD5 and AKAv2-MD5 respectively).
  *
+ * Note that if this is enabled, application would need to link with
+ * <b>libmilenage</b> library from \a third_party directory.
+ *
  * Default: 0 (for now)
  */
 #ifndef PJSIP_HAS_DIGEST_AKA_AUTH
@@ -858,11 +1009,11 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 /**
  * Specify whether client registration should add "x-uid" extension
  * parameter in all Contact URIs that it registers to assist the
- * matching of Contact URIs in the 200/OK REGISTER response, in
+ * matching of Contact URIs in the 200/OK REGISTER response, in 
  * case the registrar is unable to return exact Contact URI in the
  * 200/OK response.
  *
- * This setting can be changed in run-time by setting
+ * This setting can be changed in run-time by setting 
  * \a regc.add_xuid_param field of pjsip_cfg().
  *
  * Default is 0.
@@ -877,7 +1028,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
  */
 
 /**
- * Specify the time (in seconds) to send SUBSCRIBE to refresh client
+ * Specify the time (in seconds) to send SUBSCRIBE to refresh client 
  * subscription before the actual interval expires.
  *
  * Default: 5 seconds
@@ -888,7 +1039,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 
 /**
- * Specify the time (in seconds) to send PUBLISH to refresh client
+ * Specify the time (in seconds) to send PUBLISH to refresh client 
  * publication before the actual interval expires.
  *
  * Default: 5 seconds
@@ -925,10 +1076,10 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 /**
  * Specify the default expiration time for presence event subscription, for
  * both client and server subscription. For client subscription, application
- * can override this by specifying positive non-zero value in "expires"
+ * can override this by specifying positive non-zero value in "expires" 
  * parameter when calling #pjsip_pres_initiate(). For server subscription,
  * we would take the expiration value from the Expires header sent by client
- * in the SUBSCRIBE request if the header exists and its value is less than
+ * in the SUBSCRIBE request if the header exists and its value is less than 
  * this setting, otherwise this setting will be used.
  *
  * Default: 600 seconds (10 minutes)
@@ -969,7 +1120,7 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 
 /**
  * Default session interval for Session Timer (RFC 4028) extension, in
- * seconds. As specified in RFC 4028 Section 4, this value must not be
+ * seconds. As specified in RFC 4028 Section 4, this value must not be 
  * less than the absolute minimum for the Session-Expires header field
  * 90 seconds, and the recommended value is 1800 seconds.
  *
@@ -1011,6 +1162,21 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
 #endif
 
 
+/**
+ * Specify whether transport manager should maintain a list of transmit
+ * buffer instances, so any possible dangling instance can be cleaned up
+ * when the transport manager is shutdown (see also ticket #1671).
+ * Note that this feature will have slight impact on the performance as
+ * mutex is employed in updating the list, i.e: on creation and destruction
+ * of transmit data.
+ *
+ * Default: 0 (no)
+ */
+#ifndef PJSIP_HAS_TX_DATA_LIST
+#   define PJSIP_HAS_TX_DATA_LIST		0
+#endif
+
+
 PJ_END_DECL
 
 /**
@@ -1022,3 +1188,4 @@ PJ_END_DECL
 
 
 #endif	/* __PJSIP_SIP_CONFIG_H__ */
+
