@@ -1,5 +1,5 @@
-/* $Id: pjsua_internal.h 4175 2012-06-22 08:53:11Z nanang $ */
-/*
+/* $Id: pjsua_internal.h 4750 2014-02-19 04:11:43Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,13 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJSUA_INTERNAL_H__
 #define __PJSUA_INTERNAL_H__
 
-/**
- * This is the private header used by pjsua library implementation.
+/** 
+ * This is the private header used by pjsua library implementation. 
  * Applications should not include this file.
  */
 
@@ -112,8 +112,8 @@ typedef struct call_answer
 } call_answer;
 
 
-/**
- * Structure to be attached to invite dialog.
+/** 
+ * Structure to be attached to invite dialog. 
  * Given a dialog "dlg", application can retrieve this structure
  * by accessing dlg->mod_data[pjsua.mod.id].
  */
@@ -155,10 +155,7 @@ struct pjsua_call
     char    last_text_buf_[128];    /**< Buffer for last_text.		    */
 
     struct {
-	pj_timer_entry	 reinv_timer;/**< Reinvite retry timer.		    */
-	pj_uint32_t	 sdp_ver;    /**< SDP version of the bad answer     */
 	int		 retry_cnt;  /**< Retry count.			    */
-        pj_bool_t        pending;    /**< Pending until CONFIRMED state     */
     } lock_codec;		     /**< Data for codec locking when answer
 					  contains multiple codecs.	    */
 
@@ -170,6 +167,7 @@ struct pjsua_call
         union {
             struct {
                 pjsua_msg_data  *msg_data;/**< Headers for outgoing INVITE. */
+                pj_bool_t        hangup;  /**< Call is hangup?              */
             } out_call;
             struct {
                 call_answer      answers;/**< A list of call answers.       */
@@ -184,6 +182,10 @@ struct pjsua_call
 					    offer.			    */
     unsigned		 rem_vid_cnt;  /**< No of active video in last remote
 					    offer.			    */
+    
+    pj_timer_entry	 reinv_timer;  /**< Reinvite retry timer.	    */
+    pj_bool_t	 	 reinv_pending;/**< Pending until CONFIRMED state.  */
+    pj_bool_t	 	 reinv_ice_sent;/**< Has reinvite for ICE upd sent? */
 };
 
 
@@ -212,6 +214,7 @@ typedef struct pjsua_acc
     int		     index;	    /**< Index in accounts array.	*/
     pj_str_t	     display;	    /**< Display name, if any.		*/
     pj_str_t	     user_part;	    /**< User part of local URI.	*/
+    pj_bool_t	     is_sips;	    /**< Local URI uses "sips"?		*/
     pj_str_t	     contact;	    /**< Our Contact header.		*/
     pj_str_t         reg_contact;   /**< Contact header for REGISTER.
 				         It may be different than acc
@@ -227,6 +230,10 @@ typedef struct pjsua_acc
     pj_status_t	     reg_last_err;  /**< Last registration error.	*/
     int		     reg_last_code; /**< Last status last register.	*/
 
+    pj_str_t         reg_mapped_addr;/**< Our addr as seen by reg srv.
+                                          Only if allow_sdp_nat_rewrite
+                                          is set                        */
+
     struct {
 	pj_bool_t	 active;    /**< Flag of reregister status.	*/
 	pj_timer_entry   timer;	    /**< Timer for reregistration.	*/
@@ -261,6 +268,8 @@ typedef struct pjsua_acc
 
     pjsip_evsub	    *mwi_sub;	    /**< MWI client subscription	*/
     pjsip_dialog    *mwi_dlg;	    /**< Dialog for MWI sub.		*/
+
+    pj_uint16_t      next_rtp_port; /**< Next RTP port to be used.      */
 } pjsua_acc;
 
 
@@ -336,6 +345,9 @@ typedef struct pjsua_stun_resolve
     PJ_DECL_LIST_MEMBER(struct pjsua_stun_resolve);
 
     pj_pool_t		*pool;	    /**< Pool		    */
+    int			 ref_cnt;   /**< Reference count    */
+    pj_bool_t		 destroy_flag; /**< To be destroyed */
+    pj_bool_t		 has_result;
     unsigned		 count;	    /**< # of entries	    */
     pj_str_t		*srv;	    /**< Array of entries   */
     unsigned		 idx;	    /**< Current index	    */
@@ -513,7 +525,7 @@ pj_status_t pjsua_media_apply_xml_control(pjsua_call_id call_id,
 /**
  * Duplicate IM data.
  */
-PJ_INLINE(pjsua_im_data*) pjsua_im_data_dup(pj_pool_t *pool,
+PJ_INLINE(pjsua_im_data*) pjsua_im_data_dup(pj_pool_t *pool, 
 					    const pjsua_im_data *src)
 {
     pjsua_im_data *dst;
@@ -577,12 +589,26 @@ void pjsua_set_state(pjsua_state new_state);
 /* Resolve the STUN server */
 pj_status_t resolve_stun_server(pj_bool_t wait);
 
-/**
+/** 
  * Normalize route URI (check for ";lr" and append one if it doesn't
  * exist and pjsua_config.force_lr is set.
  */
 pj_status_t normalize_route_uri(pj_pool_t *pool, pj_str_t *uri);
 
+/* acc use stun? */
+pj_bool_t pjsua_sip_acc_is_using_stun(pjsua_acc_id acc_id);
+
+/* Get local transport address suitable to be used for Via or Contact address
+ * to send request to the specified destination URI.
+ */
+pj_status_t pjsua_acc_get_uac_addr(pjsua_acc_id acc_id,
+				   pj_pool_t *pool,
+				   const pj_str_t *dst_uri,
+				   pjsip_host_port *addr,
+				   pjsip_transport_type_e *p_tp_type,
+				   int *p_secure,
+				   const void **p_tp);
+
 /**
  * Handle incoming invite request.
  */
@@ -599,7 +625,7 @@ pj_status_t pjsua_media_channel_init(pjsua_call_id call_id,
 				     int *sip_err_code,
                                      pj_bool_t async,
                                      pjsua_med_tp_state_cb cb);
-pj_status_t pjsua_media_channel_create_sdp(pjsua_call_id call_id,
+pj_status_t pjsua_media_channel_create_sdp(pjsua_call_id call_id, 
 					   pj_pool_t *pool,
 					   const pjmedia_sdp_session *rem_sdp,
 					   pjmedia_sdp_session **p_sdp,
@@ -609,6 +635,12 @@ pj_status_t pjsua_media_channel_update(pjsua_call_id call_id,
 				       const pjmedia_sdp_session *remote_sdp);
 pj_status_t pjsua_media_channel_deinit(pjsua_call_id call_id);
 
+/*
+ * Error message when media operation is requested while another is in progress
+ */
+#define ERR_MEDIA_CHANGING  " because another media operation is in progress"
+
+pj_bool_t   pjsua_call_media_is_changing(pjsua_call *call);
 pj_status_t pjsua_call_media_init(pjsua_call_media *call_med,
                                   pjmedia_type type,
 				  const pjsua_transport_config *tcfg,
@@ -665,7 +697,7 @@ pj_status_t pjsua_pres_init_publish_acc(int acc_id);
 void pjsua_pres_unpublish(pjsua_acc *acc, unsigned flags);
 
 /**
- * Terminate server subscription for the account
+ * Terminate server subscription for the account 
  */
 void pjsua_pres_delete_acc(int acc_id, unsigned flags);
 
@@ -798,7 +830,12 @@ PJ_DECL(void) pjsua_vid_win_reset(pjsua_vid_win_id wid);
 #  define pjsua_vid_win_reset(wid)
 #endif
 
+/*
+ * Schedule check for the need of re-INVITE/UPDATE after media update
+ */
+void pjsua_call_schedule_reinvite_check(pjsua_call *call, unsigned delay_ms);
 
 PJ_END_DECL
 
 #endif	/* __PJSUA_INTERNAL_H__ */
+
