@@ -1,5 +1,5 @@
-/* $Id: publishc.c 4173 2012-06-20 10:39:05Z ming $ */
-/*
+/* $Id: publishc.c 4530 2013-05-30 09:27:49Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip-simple/publish.h>
 #include <pjsip/sip_auth.h>
@@ -49,7 +49,7 @@ enum
     PJSIP_PUBLISH_METHOD = PJSIP_OTHER_METHOD,
 };
 
-const pjsip_method pjsip_publish_method =
+const pjsip_method pjsip_publish_method = 
 {
     (pjsip_method_e)PJSIP_PUBLISH_METHOD,
     { "PUBLISH", 7 }
@@ -61,7 +61,8 @@ const pjsip_method pjsip_publish_method =
  */
 typedef struct pending_publish
 {
-    PJ_DECL_LIST_MEMBER(pjsip_tx_data);
+    PJ_DECL_LIST_MEMBER(struct pending_publish);
+    pjsip_tx_data		*tdata;
 } pending_publish;
 
 
@@ -108,6 +109,7 @@ struct pjsip_publishc
 
     /* Pending PUBLISH request */
     pending_publish		 pending_reqs;
+    pending_publish		 pending_reqs_empty;
 };
 
 
@@ -129,7 +131,7 @@ PJ_DEF(pj_status_t) pjsip_publishc_init_module(pjsip_endpoint *endpt)
 	responses.
 
 	13.2.1 Creating the Initial INVITE
-	  An Allow header field (Section 20.5) SHOULD be present in the
+	  An Allow header field (Section 20.5) SHOULD be present in the 
 	  INVITE. It indicates what methods can be invoked within a dialog
 
 	20.5 Allow
@@ -149,10 +151,10 @@ PJ_DEF(pj_status_t) pjsip_publishc_init_module(pjsip_endpoint *endpt)
 }
 
 
-PJ_DEF(pj_status_t) pjsip_publishc_create( pjsip_endpoint *endpt,
+PJ_DEF(pj_status_t) pjsip_publishc_create( pjsip_endpoint *endpt, 
 					   const pjsip_publishc_opt *opt,
 					   void *token,
-					   pjsip_publishc_cb *cb,
+					   pjsip_publishc_cb *cb,	
 					   pjsip_publishc **p_pubc)
 {
     pj_pool_t *pool;
@@ -180,6 +182,7 @@ PJ_DEF(pj_status_t) pjsip_publishc_create( pjsip_endpoint *endpt,
     }
     pj_memcpy(&pubc->opt, opt, sizeof(*opt));
     pj_list_init(&pubc->pending_reqs);
+    pj_list_init(&pubc->pending_reqs_empty);
 
     status = pj_mutex_create_recursive(pubc->pool, "pubc%p", &pubc->mutex);
     if (status != PJ_SUCCESS) {
@@ -233,8 +236,8 @@ PJ_DEF(pj_pool_t*) pjsip_publishc_get_pool(pjsip_publishc *pubc)
 
 static void set_expires( pjsip_publishc *pubc, pj_uint32_t expires)
 {
-    if (expires != pubc->expires &&
-	expires != PJSIP_PUBC_EXPIRATION_NOT_SPECIFIED)
+    if (expires != pubc->expires && 
+	expires != PJSIP_PUBC_EXPIRATION_NOT_SPECIFIED) 
     {
 	pubc->expires_hdr = pjsip_expires_hdr_create(pubc->pool, expires);
     } else {
@@ -252,7 +255,7 @@ PJ_DEF(pj_status_t) pjsip_publishc_init(pjsip_publishc *pubc,
 {
     pj_str_t tmp;
 
-    PJ_ASSERT_RETURN(pubc && event && target_uri && from_uri && to_uri &&
+    PJ_ASSERT_RETURN(pubc && event && target_uri && from_uri && to_uri && 
 		     expires, PJ_EINVAL);
 
     /* Copy event type */
@@ -272,7 +275,7 @@ PJ_DEF(pj_status_t) pjsip_publishc_init(pjsip_publishc *pubc,
     pj_strdup_with_null(pubc->pool, &pubc->from_uri, from_uri);
     tmp = pubc->from_uri;
     pubc->from_hdr = pjsip_from_hdr_create(pubc->pool);
-    pubc->from_hdr->uri = pjsip_parse_uri(pubc->pool, tmp.ptr, tmp.slen,
+    pubc->from_hdr->uri = pjsip_parse_uri(pubc->pool, tmp.ptr, tmp.slen, 
 					  PJSIP_PARSE_URI_AS_NAMEADDR);
     if (!pubc->from_hdr->uri) {
 	return PJSIP_EINVALIDURI;
@@ -281,7 +284,7 @@ PJ_DEF(pj_status_t) pjsip_publishc_init(pjsip_publishc *pubc,
     /* Set "To" header. */
     pj_strdup_with_null(pubc->pool, &tmp, to_uri);
     pubc->to_hdr = pjsip_to_hdr_create(pubc->pool);
-    pubc->to_hdr->uri = pjsip_parse_uri(pubc->pool, tmp.ptr, tmp.slen,
+    pubc->to_hdr->uri = pjsip_parse_uri(pubc->pool, tmp.ptr, tmp.slen, 
 					PJSIP_PARSE_URI_AS_NAMEADDR);
     if (!pubc->to_hdr->uri) {
 	return PJSIP_EINVALIDURI;
@@ -355,14 +358,17 @@ PJ_DEF(pj_status_t) pjsip_publishc_set_via_sent_by(pjsip_publishc *pubc,
 
     if (!via_addr)
         pj_bzero(&pubc->via_addr, sizeof(pubc->via_addr));
-    else
-        pubc->via_addr = *via_addr;
+    else {
+        if (pj_strcmp(&pubc->via_addr.host, &via_addr->host))
+            pj_strdup(pubc->pool, &pubc->via_addr.host, &via_addr->host);
+        pubc->via_addr.port = via_addr->port;
+    }
     pubc->via_tp = via_tp;
 
     return PJ_SUCCESS;
 }
 
-static pj_status_t create_request(pjsip_publishc *pubc,
+static pj_status_t create_request(pjsip_publishc *pubc, 
 				  pjsip_tx_data **p_tdata)
 {
     const pj_str_t STR_EVENT = { "Event", 5 };
@@ -373,7 +379,7 @@ static pj_status_t create_request(pjsip_publishc *pubc,
     PJ_ASSERT_RETURN(pubc && p_tdata, PJ_EINVAL);
 
     /* Create the request. */
-    status = pjsip_endpt_create_request_from_hdr( pubc->endpt,
+    status = pjsip_endpt_create_request_from_hdr( pubc->endpt, 
 						  &pjsip_publish_method,
 						  pubc->target_uri,
 						  pubc->from_hdr,
@@ -402,7 +408,7 @@ static pj_status_t create_request(pjsip_publishc *pubc,
 	route = pubc->route_set.next;
 	while (route != &pubc->route_set) {
 	    pjsip_hdr *new_hdr = (pjsip_hdr*)
-				 pjsip_hdr_shallow_clone(tdata->pool, route);
+	    			 pjsip_hdr_shallow_clone(tdata->pool, route);
 	    pj_list_insert_after(route_pos, new_hdr);
 	    route_pos = new_hdr;
 	    route = route->next;
@@ -433,7 +439,7 @@ static pj_status_t create_request(pjsip_publishc *pubc,
 	hdr = pubc->usr_hdr.next;
 	while (hdr != &pubc->usr_hdr) {
 	    pjsip_hdr *new_hdr = (pjsip_hdr*)
-				 pjsip_hdr_shallow_clone(tdata->pool, hdr);
+	    			 pjsip_hdr_shallow_clone(tdata->pool, hdr);
 	    pjsip_msg_add_hdr(tdata->msg, new_hdr);
 	    hdr = hdr->next;
 	}
@@ -446,7 +452,7 @@ static pj_status_t create_request(pjsip_publishc *pubc,
 }
 
 
-PJ_DEF(pj_status_t) pjsip_publishc_publish(pjsip_publishc *pubc,
+PJ_DEF(pj_status_t) pjsip_publishc_publish(pjsip_publishc *pubc, 
 					   pj_bool_t auto_refresh,
 					   pjsip_tx_data **p_tdata)
 {
@@ -522,7 +528,7 @@ PJ_DEF(pj_status_t) pjsip_publishc_update_expires( pjsip_publishc *pubc,
 }
 
 
-static void call_callback(pjsip_publishc *pubc, pj_status_t status,
+static void call_callback(pjsip_publishc *pubc, pj_status_t status, 
 			  int st_code, const pj_str_t *reason,
 			  pjsip_rx_data *rdata, pj_int32_t expiration)
 {
@@ -546,7 +552,7 @@ static void pubc_refresh_timer_cb( pj_timer_heap_t *timer_heap,
     pjsip_publishc *pubc = (pjsip_publishc*) entry->user_data;
     pjsip_tx_data *tdata;
     pj_status_t status;
-
+    
     PJ_UNUSED_ARG(timer_heap);
 
     entry->id = 0;
@@ -567,7 +573,7 @@ static void tsx_callback(void *token, pjsip_event *event)
     pj_status_t status;
     pjsip_publishc *pubc = (pjsip_publishc*) token;
     pjsip_transaction *tsx = event->body.tsx_state.tsx;
-
+    
     /* Decrement pending transaction counter. */
     pj_assert(pubc->pending_tsx > 0);
     --pubc->pending_tsx;
@@ -575,7 +581,7 @@ static void tsx_callback(void *token, pjsip_event *event)
     /* Mark that we're in callback to prevent deletion (#1164) */
     ++pubc->in_callback;
 
-    /* If publication data has been deleted by user then remove publication
+    /* If publication data has been deleted by user then remove publication 
      * data from transaction's callback, and don't call callback.
      */
     if (pubc->_delete_flag) {
@@ -590,15 +596,15 @@ static void tsx_callback(void *token, pjsip_event *event)
 	pjsip_tx_data *tdata;
 
 	status = pjsip_auth_clt_reinit_req( &pubc->auth_sess,
-					    rdata,
-					    tsx->last_tx,
+					    rdata, 
+					    tsx->last_tx,  
 					    &tdata);
 	if (status != PJ_SUCCESS) {
-	    call_callback(pubc, status, tsx->status_code,
+	    call_callback(pubc, status, tsx->status_code, 
 			  &rdata->msg_info.msg->line.status.reason,
 			  rdata, -1);
 	} else {
-	    status = pjsip_publishc_send(pubc, tdata);
+    	    status = pjsip_publishc_send(pubc, tdata);
 	}
 
     } else {
@@ -625,11 +631,11 @@ static void tsx_callback(void *token, pjsip_event *event)
 
 	    /* Update expires value */
 	    expires = (pjsip_expires_hdr*)
-		      pjsip_msg_find_hdr(msg, PJSIP_H_EXPIRES, NULL);
+	    	      pjsip_msg_find_hdr(msg, PJSIP_H_EXPIRES, NULL);
 
 	    if (pubc->auto_refresh && expires)
 		expiration = expires->ivalue;
-
+	    
 	    if (pubc->auto_refresh && expiration!=0 && expiration!=0xFFFF) {
 		pj_time_val delay = { 0, 0};
 
@@ -640,12 +646,12 @@ static void tsx_callback(void *token, pjsip_event *event)
 		}
 
 		delay.sec = expiration - DELAY_BEFORE_REFRESH;
-		if (pubc->expires != PJSIP_PUBC_EXPIRATION_NOT_SPECIFIED &&
-		    delay.sec > (pj_int32_t)pubc->expires)
+		if (pubc->expires != PJSIP_PUBC_EXPIRATION_NOT_SPECIFIED && 
+		    delay.sec > (pj_int32_t)pubc->expires) 
 		{
 		    delay.sec = pubc->expires;
 		}
-		if (delay.sec < DELAY_BEFORE_REFRESH)
+		if (delay.sec < DELAY_BEFORE_REFRESH) 
 		    delay.sec = DELAY_BEFORE_REFRESH;
 		pubc->timer.cb = &pubc_refresh_timer_cb;
 		pubc->timer.id = REFRESH_TIMER;
@@ -657,7 +663,7 @@ static void tsx_callback(void *token, pjsip_event *event)
 	    }
 
 	} else {
-	    rdata = (event->body.tsx_state.type==PJSIP_EVENT_RX_MSG) ?
+	    rdata = (event->body.tsx_state.type==PJSIP_EVENT_RX_MSG) ? 
 			event->body.tsx_state.src.rdata : NULL;
 	}
 
@@ -670,8 +676,8 @@ static void tsx_callback(void *token, pjsip_event *event)
 	 */
 	++pubc->pending_tsx;
 
-	call_callback(pubc, PJ_SUCCESS, tsx->status_code,
-		      (rdata ? &rdata->msg_info.msg->line.status.reason
+	call_callback(pubc, PJ_SUCCESS, tsx->status_code, 
+		      (rdata ? &rdata->msg_info.msg->line.status.reason 
 			: pjsip_get_status_text(tsx->status_code)),
 		      rdata, expiration);
 
@@ -680,8 +686,14 @@ static void tsx_callback(void *token, pjsip_event *event)
 	/* If we have pending request(s), send them now */
 	pj_mutex_lock(pubc->mutex);
 	while (!pj_list_empty(&pubc->pending_reqs)) {
-	    pjsip_tx_data *tdata = pubc->pending_reqs.next;
-	    pj_list_erase(tdata);
+	    pending_publish *pp = pubc->pending_reqs.next;
+	    pjsip_tx_data *tdata = pp->tdata;
+
+	    /* Remove the request from pending request list,
+	     * and keep the unused entry into pending_reqs_empty pool.
+	     */
+	    pj_list_erase(pp);
+	    pj_list_push_back(&pubc->pending_reqs_empty, pp);
 
 	    /* Add SIP-If-Match if we have etag and the request doesn't have
 	     * one (http://trac.pjsip.org/repos/ticket/996)
@@ -709,7 +721,6 @@ static void tsx_callback(void *token, pjsip_event *event)
 	    status = pjsip_publishc_send(pubc, tdata);
 	    if (status == PJ_EPENDING) {
 		pj_assert(!"Not expected");
-		pj_list_erase(tdata);
 		pjsip_tx_data_dec_ref(tdata);
 	    } else if (status == PJ_SUCCESS) {
 		break;
@@ -728,7 +739,7 @@ static void tsx_callback(void *token, pjsip_event *event)
 }
 
 
-PJ_DEF(pj_status_t) pjsip_publishc_send(pjsip_publishc *pubc,
+PJ_DEF(pj_status_t) pjsip_publishc_send(pjsip_publishc *pubc, 
 					pjsip_tx_data *tdata)
 {
     pj_status_t status;
@@ -741,7 +752,15 @@ PJ_DEF(pj_status_t) pjsip_publishc_send(pjsip_publishc *pubc,
     pj_mutex_lock(pubc->mutex);
     if (pubc->pending_tsx) {
 	if (pubc->opt.queue_request) {
-	    pj_list_push_back(&pubc->pending_reqs, tdata);
+	    pending_publish *pp = NULL;
+	    if (pj_list_empty(&pubc->pending_reqs_empty)) {
+		pp = PJ_POOL_ZALLOC_T(pubc->pool, pending_publish);
+	    } else {
+		pp = pubc->pending_reqs_empty.next;
+		pj_list_erase(pp);
+	    }
+	    pp->tdata = tdata;
+	    pj_list_push_back(&pubc->pending_reqs, pp);
 	    pj_mutex_unlock(pubc->mutex);
 	    PJ_LOG(4,(THIS_FILE, "Request is queued, pubc has another "
 				 "transaction pending"));
@@ -768,18 +787,18 @@ PJ_DEF(pj_status_t) pjsip_publishc_send(pjsip_publishc *pubc,
     /* Increment CSeq */
     cseq = ++pubc->cseq_hdr->cseq;
     cseq_hdr = (pjsip_cseq_hdr*)
-	       pjsip_msg_find_hdr(tdata->msg, PJSIP_H_CSEQ, NULL);
+    	       pjsip_msg_find_hdr(tdata->msg, PJSIP_H_CSEQ, NULL);
     cseq_hdr->cseq = cseq;
 
     /* Increment pending transaction first, since transaction callback
      * may be called even before send_request() returns!
      */
     ++pubc->pending_tsx;
-    status = pjsip_endpt_send_request(pubc->endpt, tdata, -1, pubc,
+    status = pjsip_endpt_send_request(pubc->endpt, tdata, -1, pubc, 
 				      &tsx_callback);
     if (status!=PJ_SUCCESS) {
 	// no need to decrement, callback has been called and it should
-	// already decremented pending_tsx. Decrementing this here may
+	// already decremented pending_tsx. Decrementing this here may 
 	// cause accessing freed memory location.
 	//--pubc->pending_tsx;
 	PJ_LOG(4,(THIS_FILE, "Error sending request, status=%d", status));
@@ -787,3 +806,4 @@ PJ_DEF(pj_status_t) pjsip_publishc_send(pjsip_publishc *pubc,
 
     return status;
 }
+
