@@ -1,4 +1,4 @@
-/* $Id: sip_dialog.c 4173 2012-06-20 10:39:05Z ming $ */
+/* $Id: sip_dialog.c 4728 2014-02-04 10:13:56Z bennylp $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -168,7 +168,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uac( pjsip_user_agent *ua,
 	    param->value.ptr[param->value.slen] = '\0';
 
 	    hdr = (pjsip_hdr*)
-		  pjsip_parse_hdr(dlg->pool, &param->name, param->value.ptr,
+	    	  pjsip_parse_hdr(dlg->pool, &param->name, param->value.ptr,
 				  param->value.slen, NULL);
 
 	    param->value.ptr[param->value.slen] = (char)c;
@@ -206,8 +206,8 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uac( pjsip_user_agent *ua,
     pj_create_unique_string(dlg->pool, &dlg->local.info->tag);
 
     /* Calculate hash value of local tag. */
-    dlg->local.tag_hval = pj_hash_calc(0, dlg->local.info->tag.ptr,
-				       dlg->local.info->tag.slen);
+    dlg->local.tag_hval = pj_hash_calc_tolower(0, NULL,
+                                               &dlg->local.info->tag);
 
     /* Randomize local CSeq. */
     dlg->local.first_cseq = pj_rand() & 0x7FFF;
@@ -249,7 +249,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uac( pjsip_user_agent *ua,
 
 	    /* Print URI */
 	    tmp.ptr = (char*) pj_pool_alloc(dlg->pool,
-					    dlg->remote.info_str.slen);
+	    				    dlg->remote.info_str.slen);
 	    tmp.slen = pjsip_uri_print(PJSIP_URI_IN_FROMTO_HDR,
 				       sip_uri, tmp.ptr,
 				       dlg->remote.info_str.slen);
@@ -354,7 +354,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uas(   pjsip_user_agent *ua,
 
     /* Init local info from the To header. */
     dlg->local.info = (pjsip_fromto_hdr*)
-		      pjsip_hdr_clone(dlg->pool, rdata->msg_info.to);
+    		      pjsip_hdr_clone(dlg->pool, rdata->msg_info.to);
     pjsip_fromto_hdr_set_from(dlg->local.info);
 
     /* Generate local tag. */
@@ -374,8 +374,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uas(   pjsip_user_agent *ua,
     pj_strdup(dlg->pool, &dlg->local.info_str, &tmp);
 
     /* Calculate hash value of local tag. */
-    dlg->local.tag_hval = pj_hash_calc(0, dlg->local.info->tag.ptr,
-				       dlg->local.info->tag.slen);
+    dlg->local.tag_hval = pj_hash_calc_tolower(0, NULL, &dlg->local.info->tag);
 
 
     /* Randomize local cseq */
@@ -410,7 +409,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uas(   pjsip_user_agent *ua,
 
     /* Init remote info from the From header. */
     dlg->remote.info = (pjsip_fromto_hdr*)
-		       pjsip_hdr_clone(dlg->pool, rdata->msg_info.from);
+    		       pjsip_hdr_clone(dlg->pool, rdata->msg_info.from);
     pjsip_fromto_hdr_set_to(dlg->remote.info);
 
     /* Print the remote info. */
@@ -453,7 +452,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uas(   pjsip_user_agent *ua,
     }
 
     dlg->remote.contact = (pjsip_contact_hdr*)
-			  pjsip_hdr_clone(dlg->pool, (pjsip_hdr*)contact_hdr);
+    			  pjsip_hdr_clone(dlg->pool, (pjsip_hdr*)contact_hdr);
 
     /* Init remote's CSeq from CSeq header */
     dlg->remote.cseq = dlg->remote.first_cseq = rdata->msg_info.cseq->cseq;
@@ -474,7 +473,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uas(   pjsip_user_agent *ua,
 
     /* Call-ID */
     dlg->call_id = (pjsip_cid_hdr*)
-		   pjsip_hdr_clone(dlg->pool, rdata->msg_info.cid);
+    		   pjsip_hdr_clone(dlg->pool, rdata->msg_info.cid);
 
     /* Route set.
      *  RFC 3261 Section 12.1.1:
@@ -522,8 +521,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uas(   pjsip_user_agent *ua,
     ++dlg->tsx_count;
 
     /* Calculate hash value of remote tag. */
-    dlg->remote.tag_hval = pj_hash_calc(0, dlg->remote.info->tag.ptr,
-					dlg->remote.info->tag.slen);
+    dlg->remote.tag_hval = pj_hash_calc_tolower(0, NULL, &dlg->remote.info->tag);
 
     /* Update remote capabilities info */
     pjsip_dlg_update_remote_cap(dlg, rdata->msg_info.msg, PJ_TRUE);
@@ -596,8 +594,11 @@ PJ_DEF(pj_status_t) pjsip_dlg_set_via_sent_by( pjsip_dialog *dlg,
 
     if (!via_addr)
         pj_bzero(&dlg->via_addr, sizeof(dlg->via_addr));
-    else
-        dlg->via_addr = *via_addr;
+    else {
+        if (pj_strcmp(&dlg->via_addr.host, &via_addr->host))
+            pj_strdup(dlg->pool, &dlg->via_addr.host, &via_addr->host);
+        dlg->via_addr.port = via_addr->port;
+    }
     dlg->via_tp = via_tp;
 
     return PJ_SUCCESS;
@@ -648,7 +649,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_fork( const pjsip_dialog *first_dlg,
 
     /* Clone local info. */
     dlg->local.info = (pjsip_fromto_hdr*)
-		      pjsip_hdr_clone(dlg->pool, first_dlg->local.info);
+    		      pjsip_hdr_clone(dlg->pool, first_dlg->local.info);
 
     /* Clone local tag. */
     pj_strdup(dlg->pool, &dlg->local.info->tag, &first_dlg->local.info->tag);
@@ -660,11 +661,11 @@ PJ_DEF(pj_status_t) pjsip_dlg_fork( const pjsip_dialog *first_dlg,
 
     /* Clone local Contact. */
     dlg->local.contact = (pjsip_contact_hdr*)
-			 pjsip_hdr_clone(dlg->pool, first_dlg->local.contact);
+    			 pjsip_hdr_clone(dlg->pool, first_dlg->local.contact);
 
     /* Clone remote info. */
     dlg->remote.info = (pjsip_fromto_hdr*)
-		       pjsip_hdr_clone(dlg->pool, first_dlg->remote.info);
+    		       pjsip_hdr_clone(dlg->pool, first_dlg->remote.info);
 
     /* Set remote tag from the response. */
     pj_strdup(dlg->pool, &dlg->remote.info->tag, &rdata->msg_info.to->tag);
@@ -689,7 +690,7 @@ PJ_DEF(pj_status_t) pjsip_dlg_fork( const pjsip_dialog *first_dlg,
 
     /* Clone Call-ID header. */
     dlg->call_id = (pjsip_cid_hdr*)
-		   pjsip_hdr_clone(dlg->pool, first_dlg->call_id);
+    		   pjsip_hdr_clone(dlg->pool, first_dlg->call_id);
 
     /* Get route-set from the response. */
     pj_list_init(&dlg->route_set);
@@ -934,10 +935,10 @@ PJ_DEF(pj_bool_t) pjsip_dlg_has_usage( pjsip_dialog *dlg,
 
     pjsip_dlg_inc_lock(dlg);
     for (index=0; index<dlg->usage_cnt; ++index) {
-	if (dlg->usage[index] == mod) {
-	    found = PJ_TRUE;
-	    break;
-	}
+    	if (dlg->usage[index] == mod) {
+    	    found = PJ_TRUE;
+    	    break;
+    	}
     }
     pjsip_dlg_dec_lock(dlg);
 
@@ -1632,12 +1633,12 @@ void pjsip_dlg_on_rx_request( pjsip_dialog *dlg, pjsip_rx_data *rdata )
 				     NULL);
 	if (contact && contact->uri &&
 	    (dlg->remote.contact==NULL ||
-	     pjsip_uri_cmp(PJSIP_URI_IN_REQ_URI,
+ 	     pjsip_uri_cmp(PJSIP_URI_IN_REQ_URI,
 			   dlg->remote.contact->uri,
 			   contact->uri)))
 	{
 	    dlg->remote.contact = (pjsip_contact_hdr*)
-				  pjsip_hdr_clone(dlg->pool, contact);
+	    			  pjsip_hdr_clone(dlg->pool, contact);
 	    dlg->target = dlg->remote.contact->uri;
 	}
     }
@@ -1686,11 +1687,11 @@ on_return:
 static void dlg_update_routeset(pjsip_dialog *dlg, const pjsip_rx_data *rdata)
 {
     const pjsip_hdr *hdr, *end_hdr;
-    pj_int32_t msg_cseq;
+    //pj_int32_t msg_cseq;
     const pjsip_msg *msg;
 
     msg = rdata->msg_info.msg;
-    msg_cseq = rdata->msg_info.cseq->cseq;
+    //msg_cseq = rdata->msg_info.cseq->cseq;
 
     /* Ignore if route set has been frozen */
     if (dlg->route_set_frozen)
@@ -1814,7 +1815,7 @@ void pjsip_dlg_on_rx_response( pjsip_dialog *dlg, pjsip_rx_data *rdata )
 	 res_code > 100 &&
 	 res_code/100 <= 2 &&
 	 pjsip_method_creates_dialog(&rdata->msg_info.cseq->method) &&
-	 pj_strcmp(&dlg->remote.info->tag, &rdata->msg_info.to->tag)))
+	 pj_stricmp(&dlg->remote.info->tag, &rdata->msg_info.to->tag)))
     {
 	pjsip_contact_hdr *contact;
 
@@ -1823,7 +1824,7 @@ void pjsip_dlg_on_rx_response( pjsip_dialog *dlg, pjsip_rx_data *rdata )
 	 * with To-tag or forking, apply strict update.
 	 */
 	pjsip_dlg_update_remote_cap(dlg, rdata->msg_info.msg,
-				    pj_strcmp(&dlg->remote.info->tag,
+				    pj_stricmp(&dlg->remote.info->tag,
 					      &rdata->msg_info.to->tag));
 
 	/* Update To tag. */
@@ -1853,7 +1854,7 @@ void pjsip_dlg_on_rx_response( pjsip_dialog *dlg, pjsip_rx_data *rdata )
 			   contact->uri)))
 	{
 	    dlg->remote.contact = (pjsip_contact_hdr*)
-				  pjsip_hdr_clone(dlg->pool, contact);
+	    			  pjsip_hdr_clone(dlg->pool, contact);
 	    dlg->target = dlg->remote.contact->uri;
 	}
 
@@ -1896,7 +1897,7 @@ void pjsip_dlg_on_rx_response( pjsip_dialog *dlg, pjsip_rx_data *rdata )
 
 	contact = (pjsip_contact_hdr*) pjsip_msg_find_hdr(rdata->msg_info.msg,
 							  PJSIP_H_CONTACT,
-							  NULL);
+				     			  NULL);
 	if (contact && contact->uri &&
 	    (dlg->remote.contact==NULL ||
 	     pjsip_uri_cmp(PJSIP_URI_IN_REQ_URI,
@@ -1904,7 +1905,7 @@ void pjsip_dlg_on_rx_response( pjsip_dialog *dlg, pjsip_rx_data *rdata )
 			   contact->uri)))
 	{
 	    dlg->remote.contact = (pjsip_contact_hdr*)
-				  pjsip_hdr_clone(dlg->pool, contact);
+	    			  pjsip_hdr_clone(dlg->pool, contact);
 	    dlg->target = dlg->remote.contact->uri;
 	}
 
