@@ -1,5 +1,5 @@
-/* $Id: sip_multipart.c 3841 2011-10-24 09:28:13Z ming $ */
-/*
+/* $Id: sip_multipart.c 4792 2014-03-13 09:33:32Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip/sip_multipart.h>
 #include <pjsip/sip_parser.h>
@@ -109,7 +109,7 @@ static int multipart_print_body(struct pjsip_msg_body *msg_body,
 	    }
 	    pj_memcpy(p, ctype_hdr.ptr, ctype_hdr.slen);
 	    p += ctype_hdr.slen;
-	    p += pjsip_media_type_print(p, end-p, media);
+	    p += pjsip_media_type_print(p, (unsigned)(end-p), media);
 	    *p++ = '\r';
 	    *p++ = '\n';
 
@@ -168,7 +168,7 @@ static int multipart_print_body(struct pjsip_msg_body *msg_body,
 
 #undef SIZE_LEFT
 
-    return p - buf;
+    return (int)(p - buf);
 }
 
 static void* multipart_clone_data(pj_pool_t *pool, const void *data,
@@ -182,6 +182,7 @@ static void* multipart_clone_data(pj_pool_t *pool, const void *data,
 
     src = (const struct multipart_data*) data;
     dst = PJ_POOL_ALLOC_T(pool, struct multipart_data);
+    pj_list_init(&dst->part_head);
 
     pj_strdup(pool, &dst->boundary, &src->boundary);
 
@@ -456,7 +457,7 @@ static pjsip_multipart_part *parse_multipart_part(pj_pool_t *pool,
 	pjsip_hdr *hdr;
 	pj_status_t status;
 
-	status = pjsip_parse_headers(pool, start, end_hdr-start,
+	status = pjsip_parse_headers(pool, start, end_hdr-start, 
 				     &part->hdr, 0);
 	if (status != PJ_SUCCESS) {
 	    PJ_PERROR(2,(THIS_FILE, status, "Warning: error parsing multipart"
@@ -489,7 +490,7 @@ static pjsip_multipart_part *parse_multipart_part(pj_pool_t *pool,
 
     if (start_body < end) {
 	part->body->data = start_body;
-	part->body->len = end - start_body;
+	part->body->len = (unsigned)(end - start_body);
     } else {
 	part->body->data = (void*)"";
 	part->body->len = 0;
@@ -655,3 +656,4 @@ PJ_DEF(pjsip_msg_body*) pjsip_multipart_parse(pj_pool_t *pool,
 
     return body;
 }
+
