@@ -1,5 +1,5 @@
-/* $Id: sip_parser.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: sip_parser.c 4537 2013-06-19 06:47:43Z riza $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip/sip_parser.h>
 #include <pjsip/sip_uri.h>
@@ -44,7 +44,7 @@
 #define ESCAPED		    "%"
 #define USER_UNRESERVED	    "&=+$,;?/"
 #define PASS		    "&=+$,"
-#define TOKEN		    "-.!%*_`'~+"   /* '=' was removed for parsing
+#define TOKEN		    "-.!%*_`'~+"   /* '=' was removed for parsing 
 					    * param */
 #define HOST		    "_-."
 #define HEX_DIGIT	    "abcdefABCDEF"
@@ -121,45 +121,45 @@ static pj_cis_buf_t cis_buf;
 /*
  * Forward decl.
  */
-static pjsip_msg *  int_parse_msg( pjsip_parse_ctx *ctx,
+static pjsip_msg *  int_parse_msg( pjsip_parse_ctx *ctx, 
 				   pjsip_parser_err_report *err_list);
-static void	    int_parse_param( pj_scanner *scanner,
+static void	    int_parse_param( pj_scanner *scanner, 
 				     pj_pool_t *pool,
-				     pj_str_t *pname,
+				     pj_str_t *pname, 
 				     pj_str_t *pvalue,
 				     unsigned option);
-static void	    int_parse_uri_param( pj_scanner *scanner,
+static void	    int_parse_uri_param( pj_scanner *scanner, 
 					 pj_pool_t *pool,
-					 pj_str_t *pname,
+					 pj_str_t *pname, 
 					 pj_str_t *pvalue,
 					 unsigned option);
 static void	    int_parse_hparam( pj_scanner *scanner,
 				      pj_pool_t *pool,
 				      pj_str_t *hname,
 				      pj_str_t *hvalue );
-static void         int_parse_req_line( pj_scanner *scanner,
+static void         int_parse_req_line( pj_scanner *scanner, 
 					pj_pool_t *pool,
 					pjsip_request_line *req_line);
 static int          int_is_next_user( pj_scanner *scanner);
-static void	    int_parse_status_line( pj_scanner *scanner,
+static void	    int_parse_status_line( pj_scanner *scanner, 
 					   pjsip_status_line *line);
-static void	    int_parse_user_pass( pj_scanner *scanner,
+static void	    int_parse_user_pass( pj_scanner *scanner, 
 					 pj_pool_t *pool,
-					 pj_str_t *user,
+					 pj_str_t *user, 
 					 pj_str_t *pass);
-static void	    int_parse_uri_host_port( pj_scanner *scanner,
-					     pj_str_t *p_host,
+static void	    int_parse_uri_host_port( pj_scanner *scanner, 
+					     pj_str_t *p_host, 
 					     int *p_port);
-static pjsip_uri *  int_parse_uri_or_name_addr( pj_scanner *scanner,
-					        pj_pool_t *pool,
+static pjsip_uri *  int_parse_uri_or_name_addr( pj_scanner *scanner, 
+					        pj_pool_t *pool, 
                                                 unsigned option);
-static void*	    int_parse_sip_url( pj_scanner *scanner,
+static void*	    int_parse_sip_url( pj_scanner *scanner, 
 				         pj_pool_t *pool,
 				         pj_bool_t parse_params);
 static pjsip_name_addr *
-                    int_parse_name_addr( pj_scanner *scanner,
+                    int_parse_name_addr( pj_scanner *scanner, 
 					 pj_pool_t *pool );
-static void*	    int_parse_other_uri(pj_scanner *scanner,
+static void*	    int_parse_other_uri(pj_scanner *scanner, 
 					pj_pool_t *pool,
 					pj_bool_t parse_params);
 static void	    parse_hdr_end( pj_scanner *scanner );
@@ -186,7 +186,7 @@ static pjsip_hdr*   parse_hdr_via( pjsip_parse_ctx *ctx );
 static pjsip_hdr*   parse_hdr_generic_string( pjsip_parse_ctx *ctx);
 
 /* Convert non NULL terminated string to integer. */
-static unsigned long pj_strtoul_mindigit(const pj_str_t *str,
+static unsigned long pj_strtoul_mindigit(const pj_str_t *str, 
                                          unsigned mindig)
 {
     unsigned long value;
@@ -208,7 +208,7 @@ static unsigned long pj_strtoul_mindigit(const pj_str_t *str,
 
 /* Get a token and unescape */
 PJ_INLINE(void) parser_get_and_unescape(pj_scanner *scanner, pj_pool_t *pool,
-					const pj_cis_t *spec,
+					const pj_cis_t *spec, 
 					const pj_cis_t *unesc_spec,
 					pj_str_t *token)
 {
@@ -239,19 +239,19 @@ PJ_DEF(const pjsip_parser_const_t*) pjsip_parser_const(void)
 }
 
 /* Concatenate unrecognized params into single string. */
-PJ_DEF(void) pjsip_concat_param_imp(pj_str_t *param, pj_pool_t *pool,
-				    const pj_str_t *pname,
-				    const pj_str_t *pvalue,
-				    int sepchar)
+PJ_DEF(void) pjsip_concat_param_imp(pj_str_t *param, pj_pool_t *pool, 
+			     	    const pj_str_t *pname, 
+				    const pj_str_t *pvalue, 
+                             	    int sepchar)
 {
     char *new_param, *p;
-    int len;
+    pj_size_t len;
 
     len = param->slen + pname->slen + pvalue->slen + 3;
     p = new_param = (char*) pj_pool_alloc(pool, len);
-
+    
     if (param->slen) {
-	int old_len = param->slen;
+	pj_size_t old_len = param->slen;
 	pj_memcpy(p, param->ptr, old_len);
 	p += old_len;
     }
@@ -266,7 +266,7 @@ PJ_DEF(void) pjsip_concat_param_imp(pj_str_t *param, pj_pool_t *pool,
     }
 
     *p = '\0';
-
+    
     param->ptr = new_param;
     param->slen = p - new_param;
 }
@@ -280,7 +280,7 @@ static pj_status_t init_parser()
      * Syntax error exception number.
      */
     pj_assert (PJSIP_SYN_ERR_EXCEPTION == -1);
-    status = pj_exception_id_alloc("PJSIP syntax error",
+    status = pj_exception_id_alloc("PJSIP syntax error", 
 				   &PJSIP_SYN_ERR_EXCEPTION);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
@@ -293,11 +293,11 @@ static pj_status_t init_parser()
     status = pj_cis_init(&cis_buf, &pconst.pjsip_DIGIT_SPEC);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
     pj_cis_add_num(&pconst.pjsip_DIGIT_SPEC);
-
+    
     status = pj_cis_init(&cis_buf, &pconst.pjsip_ALPHA_SPEC);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
     pj_cis_add_alpha( &pconst.pjsip_ALPHA_SPEC );
-
+    
     status = pj_cis_init(&cis_buf, &pconst.pjsip_ALNUM_SPEC);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
     pj_cis_add_alpha( &pconst.pjsip_ALNUM_SPEC );
@@ -417,11 +417,11 @@ static pj_status_t init_parser()
     status = pjsip_register_hdr_parser( "Contact", "m", &parse_hdr_contact);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Content-Length", "l",
+    status = pjsip_register_hdr_parser( "Content-Length", "l", 
                                         &parse_hdr_content_len);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Content-Type", "c",
+    status = pjsip_register_hdr_parser( "Content-Type", "c", 
                                         &parse_hdr_content_type);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
@@ -434,11 +434,11 @@ static pj_status_t init_parser()
     status = pjsip_register_hdr_parser( "From", "f", &parse_hdr_from);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Max-Forwards", NULL,
+    status = pjsip_register_hdr_parser( "Max-Forwards", NULL, 
                                         &parse_hdr_max_forwards);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Min-Expires", NULL,
+    status = pjsip_register_hdr_parser( "Min-Expires", NULL, 
                                         &parse_hdr_min_expires);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
@@ -451,26 +451,26 @@ static pj_status_t init_parser()
     status = pjsip_register_hdr_parser( "Require", NULL, &parse_hdr_require);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Retry-After", NULL,
+    status = pjsip_register_hdr_parser( "Retry-After", NULL, 
                                         &parse_hdr_retry_after);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Supported", "k",
+    status = pjsip_register_hdr_parser( "Supported", "k", 
                                         &parse_hdr_supported);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
     status = pjsip_register_hdr_parser( "To", "t", &parse_hdr_to);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    status = pjsip_register_hdr_parser( "Unsupported", NULL,
+    status = pjsip_register_hdr_parser( "Unsupported", NULL, 
                                         &parse_hdr_unsupported);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
     status = pjsip_register_hdr_parser( "Via", "v", &parse_hdr_via);
     PJ_ASSERT_RETURN(status == PJ_SUCCESS, status);
 
-    /*
-     * Register auth parser.
+    /* 
+     * Register auth parser. 
      */
 
     status = pjsip_auth_init_parser();
@@ -511,8 +511,8 @@ void deinit_sip_parser(void)
  * - <0 if handler is 'less' than the header name.
  * - >0 if handler is 'greater' than header name.
  */
-PJ_INLINE(int) compare_handler( const handler_rec *r1,
-				const char *name,
+PJ_INLINE(int) compare_handler( const handler_rec *r1, 
+				const char *name, 
 				pj_size_t name_len,
 				pj_uint32_t hash )
 {
@@ -537,7 +537,7 @@ PJ_INLINE(int) compare_handler( const handler_rec *r1,
 }
 
 /* Register one handler for one header name. */
-static pj_status_t int_register_parser( const char *name,
+static pj_status_t int_register_parser( const char *name, 
                                         pjsip_parse_hdr_func *fptr )
 {
     unsigned	pos;
@@ -560,12 +560,12 @@ static pj_status_t int_register_parser( const char *name,
     rec.hname[rec.hname_len] = '\0';
 
     /* Calculate hash value. */
-    rec.hname_hash = pj_hash_calc(0, rec.hname, rec.hname_len);
+    rec.hname_hash = pj_hash_calc(0, rec.hname, (unsigned)rec.hname_len);
 
     /* Get the pos to insert the new handler. */
     for (pos=0; pos < handler_count; ++pos) {
 	int d;
-	d = compare_handler(&handler[pos], rec.hname, rec.hname_len,
+	d = compare_handler(&handler[pos], rec.hname, rec.hname_len, 
                             rec.hname_hash);
 	if (d == 0) {
 	    pj_assert(0);
@@ -578,7 +578,7 @@ static pj_status_t int_register_parser( const char *name,
 
     /* Shift handlers. */
     if (pos != handler_count) {
-	pj_memmove( &handler[pos+1], &handler[pos],
+	pj_memmove( &handler[pos+1], &handler[pos], 
                     (handler_count-pos)*sizeof(handler_rec));
     }
     /* Add new handler. */
@@ -595,7 +595,8 @@ PJ_DEF(pj_status_t) pjsip_register_hdr_parser( const char *hname,
 					       const char *hshortname,
 					       pjsip_parse_hdr_func *fptr)
 {
-    unsigned i, len;
+    unsigned i;
+    pj_size_t len;
     char hname_lcase[PJSIP_MAX_HNAME_LEN+1];
     pj_status_t status;
 
@@ -623,12 +624,12 @@ PJ_DEF(pj_status_t) pjsip_register_hdr_parser( const char *hname,
     if (status != PJ_SUCCESS) {
 	return status;
     }
-
+    
 
     /* Register the shortname version of the name */
     if (hshortname) {
         status = int_register_parser(hshortname, fptr);
-        if (status != PJ_SUCCESS)
+        if (status != PJ_SUCCESS) 
 	    return status;
     }
     return PJ_SUCCESS;
@@ -636,7 +637,7 @@ PJ_DEF(pj_status_t) pjsip_register_hdr_parser( const char *hname,
 
 
 /* Find handler to parse the header name. */
-static pjsip_parse_hdr_func * find_handler_imp(pj_uint32_t  hash,
+static pjsip_parse_hdr_func * find_handler_imp(pj_uint32_t  hash, 
 					       const pj_str_t *hname)
 {
     handler_rec *first;
@@ -681,7 +682,7 @@ static pjsip_parse_hdr_func* find_handler(const pj_str_t *hname)
     }
 
     /* First, common case, try to find handler with exact name */
-    hash = pj_hash_calc(0, hname->ptr, hname->slen);
+    hash = pj_hash_calc(0, hname->ptr, (unsigned)hname->slen);
     handler = find_handler_imp(hash, hname);
     if (handler)
 	return handler;
@@ -723,7 +724,7 @@ PJ_DEF(pj_status_t) pjsip_register_uri_parser( char *scheme,
 }
 
 /* Public function to parse SIP message. */
-PJ_DEF(pjsip_msg*) pjsip_parse_msg( pj_pool_t *pool,
+PJ_DEF(pjsip_msg*) pjsip_parse_msg( pj_pool_t *pool, 
                                     char *buf, pj_size_t size,
 				    pjsip_parser_err_report *err_list)
 {
@@ -731,7 +732,7 @@ PJ_DEF(pjsip_msg*) pjsip_parse_msg( pj_pool_t *pool,
     pj_scanner scanner;
     pjsip_parse_ctx context;
 
-    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER,
+    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER, 
                  &on_syntax_error);
 
     context.scanner = &scanner;
@@ -751,7 +752,7 @@ PJ_DEF(pjsip_msg *) pjsip_parse_rdata( char *buf, pj_size_t size,
     pj_scanner scanner;
     pjsip_parse_ctx context;
 
-    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER,
+    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER, 
                  &on_syntax_error);
 
     context.scanner = &scanner;
@@ -765,7 +766,7 @@ PJ_DEF(pjsip_msg *) pjsip_parse_rdata( char *buf, pj_size_t size,
 }
 
 /* Determine if a message has been received. */
-PJ_DEF(pj_bool_t) pjsip_find_msg( const char *buf, pj_size_t size,
+PJ_DEF(pj_bool_t) pjsip_find_msg( const char *buf, pj_size_t size, 
 				  pj_bool_t is_datagram, pj_size_t *msg_size)
 {
 #if PJ_HAS_TCP
@@ -785,7 +786,7 @@ PJ_DEF(pj_bool_t) pjsip_find_msg( const char *buf, pj_size_t size,
     }
 
 
-    /* Find the end of header area by finding an empty line.
+    /* Find the end of header area by finding an empty line. 
      * Don't use plain strstr() since we want to be able to handle
      * NULL character in the message
      */
@@ -794,7 +795,7 @@ PJ_DEF(pj_bool_t) pjsip_find_msg( const char *buf, pj_size_t size,
     if (pos == NULL) {
 	return PJSIP_EPARTIALMSG;
     }
-
+ 
     hdr_end = pos+1;
     body_start = pos+3;
 
@@ -802,16 +803,16 @@ PJ_DEF(pj_bool_t) pjsip_find_msg( const char *buf, pj_size_t size,
     line = pj_strchr(&cur_msg, '\n');
     while (line && line < hdr_end) {
 	++line;
-	if ( ((*line=='C' || *line=='c') &&
+	if ( ((*line=='C' || *line=='c') && 
               strnicmp_alnum(line, "Content-Length", 14) == 0) ||
-	     ((*line=='l' || *line=='L') &&
+	     ((*line=='l' || *line=='L') && 
               (*(line+1)==' ' || *(line+1)=='\t' || *(line+1)==':')))
 	{
 	    /* Try to parse the header. */
 	    pj_scanner scanner;
 	    PJ_USE_EXCEPTION;
 
-	    pj_scan_init(&scanner, (char*)line, hdr_end-line,
+	    pj_scan_init(&scanner, (char*)line, hdr_end-line, 
 			 PJ_SCAN_AUTOSKIP_WS_HEADER, &on_syntax_error);
 
 	    PJ_TRY {
@@ -872,7 +873,7 @@ PJ_DEF(pj_bool_t) pjsip_find_msg( const char *buf, pj_size_t size,
 }
 
 /* Public function to parse URI */
-PJ_DEF(pjsip_uri*) pjsip_parse_uri( pj_pool_t *pool,
+PJ_DEF(pjsip_uri*) pjsip_parse_uri( pj_pool_t *pool, 
 					 char *buf, pj_size_t size,
 					 unsigned option)
 {
@@ -882,7 +883,7 @@ PJ_DEF(pjsip_uri*) pjsip_parse_uri( pj_pool_t *pool,
 
     pj_scan_init(&scanner, buf, size, 0, &on_syntax_error);
 
-
+    
     PJ_TRY {
 	uri = int_parse_uri_or_name_addr(&scanner, pool, option);
     }
@@ -948,7 +949,7 @@ static pjsip_msg *int_parse_msg( pjsip_parse_ctx *ctx,
     parsing_headers = PJ_FALSE;
 
 retry_parse:
-    PJ_TRY
+    PJ_TRY 
     {
 	if (parsing_headers)
 	    goto parse_headers;
@@ -986,16 +987,16 @@ parse_headers:
 	     * Ref: PROTOS #2412
 	     */
 	    hname.slen = 0;
-
+	    
 	    /* Get hname. */
 	    pj_scan_get( scanner, &pconst.pjsip_TOKEN_SPEC, &hname);
 	    if (pj_scan_get_char( scanner ) != ':') {
 		PJ_THROW(PJSIP_SYN_ERR_EXCEPTION);
 	    }
-
+	    
 	    /* Find handler. */
 	    handler = find_handler(&hname);
-
+	    
 	    /* Call the handler if found.
 	     * If no handler is found, then treat the header as generic
 	     * hname/hvalue pair.
@@ -1009,8 +1010,8 @@ parse_headers:
 		 *  header. See http://trac.pjsip.org/repos/ticket/940
 		 */
 
-		/* Check if we've just parsed a Content-Type header.
-		 * We will check for a message body if we've got Content-Type
+		/* Check if we've just parsed a Content-Type header. 
+		 * We will check for a message body if we've got Content-Type 
 		 * header.
 		 */
 		if (hdr && hdr->type == PJSIP_H_CONTENT_TYPE) {
@@ -1021,8 +1022,8 @@ parse_headers:
 		hdr = parse_hdr_generic_string(ctx);
 		hdr->name = hdr->sname = hname;
 	    }
-
-
+	    
+	
 	    /* Single parse of header line can produce multiple headers.
 	     * For example, if one Contact: header contains Contact list
 	     * separated by comma, then these Contacts will be split into
@@ -1031,10 +1032,10 @@ parse_headers:
 	     */
 	    if (hdr)
 		pj_list_insert_nodes_before(&msg->hdr, hdr);
-
+	    
 	    /* Parse until EOF or an empty line is found. */
 	} while (!pj_scan_is_eof(scanner) && !IS_NEWLINE(*scanner->curptr));
-
+	
 	parsing_headers = PJ_FALSE;
 
 	/* If empty line is found, eat it. */
@@ -1044,7 +1045,7 @@ parse_headers:
 	    }
 	}
 
-	/* If we have Content-Type header, treat the rest of the message
+	/* If we have Content-Type header, treat the rest of the message 
 	 * as body.
 	 */
 	if (ctype_hdr && scanner->curptr!=scanner->end) {
@@ -1064,7 +1065,7 @@ parse_headers:
 		                    &ctype_hdr->media);
 
 		body->data = scanner->curptr;
-		body->len = scanner->end - scanner->curptr;
+		body->len = (unsigned)(scanner->end - scanner->curptr);
 		body->print_body = &pjsip_print_text_body;
 		body->clone_data = &pjsip_clone_text_data;
 	    }
@@ -1072,14 +1073,14 @@ parse_headers:
 	    msg->body = body;
 	}
     }
-    PJ_CATCH_ANY
+    PJ_CATCH_ANY 
     {
-	/* Exception was thrown during parsing.
-	 * Skip until newline, and parse next header.
+	/* Exception was thrown during parsing. 
+	 * Skip until newline, and parse next header. 
 	 */
 	if (err_list) {
 	    pjsip_parser_err_report *err_info;
-
+	    
 	    err_info = PJ_POOL_ALLOC_T(pool, pjsip_parser_err_report);
 	    err_info->except_code = PJ_GET_EXCEPTION();
 	    err_info->line = scanner->line;
@@ -1093,10 +1094,10 @@ parse_headers:
 		err_info->hname = pj_str("Status Line");
 	    else
 		err_info->hname.slen = 0;
-
+	    
 	    pj_list_insert_before(err_list, err_info);
 	}
-
+	
 	if (parsing_headers) {
 	    if (!pj_scan_is_eof(scanner)) {
 		/* Skip until next line.
@@ -1151,7 +1152,7 @@ static void parse_param_imp( pj_scanner *scanner, pj_pool_t *pool,
 	    } else if (*scanner->curptr == '[') {
 		/* pvalue can be a quoted IPv6; in this case, the
 		 * '[' and ']' quote characters are to be removed
-		 * from the pvalue.
+		 * from the pvalue. 
 		 */
 		pj_scan_get_char(scanner);
 		pj_scan_get_until_ch(scanner, ']', pvalue);
@@ -1165,8 +1166,8 @@ static void parse_param_imp( pj_scanner *scanner, pj_pool_t *pool,
 
 /* Parse parameter (pname ["=" pvalue]) using token. */
 PJ_DEF(void) pjsip_parse_param_imp(pj_scanner *scanner, pj_pool_t *pool,
-				   pj_str_t *pname, pj_str_t *pvalue,
-				   unsigned option)
+			     	   pj_str_t *pname, pj_str_t *pvalue,
+			     	   unsigned option)
 {
     parse_param_imp(scanner, pool, pname, pvalue, &pconst.pjsip_TOKEN_SPEC,
 		    &pconst.pjsip_TOKEN_SPEC_ESC, option);
@@ -1175,8 +1176,8 @@ PJ_DEF(void) pjsip_parse_param_imp(pj_scanner *scanner, pj_pool_t *pool,
 
 /* Parse parameter (pname ["=" pvalue]) using paramchar. */
 PJ_DEF(void) pjsip_parse_uri_param_imp( pj_scanner *scanner, pj_pool_t *pool,
-					pj_str_t *pname, pj_str_t *pvalue,
-					unsigned option)
+			       		pj_str_t *pname, pj_str_t *pvalue,
+			       		unsigned option)
 {
     parse_param_imp(scanner,pool, pname, pvalue, &pconst.pjsip_PARAM_CHAR_SPEC,
 		    &pconst.pjsip_PARAM_CHAR_SPEC_ESC, option);
@@ -1204,7 +1205,7 @@ static void int_parse_uri_param( pj_scanner *scanner, pj_pool_t *pool,
     pj_scan_get_char(scanner);
 
     /* Get pname and optionally pvalue */
-    pjsip_parse_uri_param_imp(scanner, pool, pname, pvalue,
+    pjsip_parse_uri_param_imp(scanner, pool, pname, pvalue, 
 			      option);
 }
 
@@ -1217,7 +1218,7 @@ static void int_parse_hparam( pj_scanner *scanner, pj_pool_t *pool,
     pj_scan_get_char(scanner);
 
     /* hname */
-    parser_get_and_unescape(scanner, pool, &pconst.pjsip_HDR_CHAR_SPEC,
+    parser_get_and_unescape(scanner, pool, &pconst.pjsip_HDR_CHAR_SPEC, 
 			    &pconst.pjsip_HDR_CHAR_SPEC_ESC, hname);
 
     /* Init hvalue */
@@ -1227,7 +1228,7 @@ static void int_parse_hparam( pj_scanner *scanner, pj_pool_t *pool,
     /* pvalue, if any */
     if (*scanner->curptr == '=') {
 	pj_scan_get_char(scanner);
-	if (!pj_scan_is_eof(scanner) &&
+	if (!pj_scan_is_eof(scanner) && 
 	    pj_cis_match(&pconst.pjsip_HDR_CHAR_SPEC, *scanner->curptr))
 	{
 	    parser_get_and_unescape(scanner, pool, &pconst.pjsip_HDR_CHAR_SPEC,
@@ -1252,7 +1253,7 @@ static void int_parse_host(pj_scanner *scanner, pj_str_t *host)
 }
 
 /* Parse host:port in URI. */
-static void int_parse_uri_host_port( pj_scanner *scanner,
+static void int_parse_uri_host_port( pj_scanner *scanner, 
 				     pj_str_t *host, int *p_port)
 {
     int_parse_host(scanner, host);
@@ -1289,7 +1290,7 @@ static int int_is_next_user(pj_scanner *scanner)
 static void int_parse_user_pass( pj_scanner *scanner, pj_pool_t *pool,
 				 pj_str_t *user, pj_str_t *pass)
 {
-    parser_get_and_unescape(scanner, pool, &pconst.pjsip_USER_SPEC_LENIENT,
+    parser_get_and_unescape(scanner, pool, &pconst.pjsip_USER_SPEC_LENIENT, 
 			    &pconst.pjsip_USER_SPEC_LENIENT_ESC, user);
 
     if ( *scanner->curptr == ':') {
@@ -1333,7 +1334,7 @@ static pjsip_uri *int_parse_uri_or_name_addr( pj_scanner *scanner, pj_pool_t *po
 	    }
 
 	    uri = (pjsip_uri*)
-		  (*func)(scanner, pool,
+	    	  (*func)(scanner, pool, 
 			  (opt & PJSIP_PARSE_URI_IN_FROM_TO_HDR)==0);
 
 
@@ -1359,7 +1360,7 @@ static pjsip_uri *int_parse_uri_or_name_addr( pj_scanner *scanner, pj_pool_t *po
 }
 
 /* Parse URI. */
-static pjsip_uri *int_parse_uri(pj_scanner *scanner, pj_pool_t *pool,
+static pjsip_uri *int_parse_uri(pj_scanner *scanner, pj_pool_t *pool, 
 				pj_bool_t parse_params)
 {
     /* Bug:
@@ -1395,10 +1396,10 @@ static pjsip_uri *int_parse_uri(pj_scanner *scanner, pj_pool_t *pool,
     */
 }
 
-/* Parse "sip:" and "sips:" URI.
+/* Parse "sip:" and "sips:" URI. 
  * This actually returns (pjsip_sip_uri*) type,
  */
-static void* int_parse_sip_url( pj_scanner *scanner,
+static void* int_parse_sip_url( pj_scanner *scanner, 
 				pj_pool_t *pool,
 				pj_bool_t parse_params)
 {
@@ -1486,7 +1487,7 @@ static void* int_parse_sip_url( pj_scanner *scanner,
 }
 
 /* Parse nameaddr. */
-static pjsip_name_addr *int_parse_name_addr( pj_scanner *scanner,
+static pjsip_name_addr *int_parse_name_addr( pj_scanner *scanner, 
 					     pj_pool_t *pool )
 {
     int has_bracket;
@@ -1522,8 +1523,15 @@ static pjsip_name_addr *int_parse_name_addr( pj_scanner *scanner,
 
     /* Get the SIP-URL */
     has_bracket = (*scanner->curptr == '<');
-    if (has_bracket)
+    if (has_bracket) {
 	pj_scan_get_char(scanner);
+    } else if (name_addr->display.slen) {
+	/* Must have bracket now (2012-10-26).
+	 * Allowing (invalid) name-addr to pass URI verification will
+	 * cause us to send invalid URI to the wire.
+	 */
+	PJ_THROW( PJSIP_SYN_ERR_EXCEPTION);
+    }
     name_addr->uri = int_parse_uri( scanner, pool, PJ_TRUE );
     if (has_bracket) {
 	if (pj_scan_get_char(scanner) != '>')
@@ -1535,7 +1543,7 @@ static pjsip_name_addr *int_parse_name_addr( pj_scanner *scanner,
 
 
 /* Parse other URI */
-static void* int_parse_other_uri(pj_scanner *scanner,
+static void* int_parse_other_uri(pj_scanner *scanner, 
 				 pj_pool_t *pool,
 				 pj_bool_t parse_params)
 {
@@ -1546,17 +1554,17 @@ static void* int_parse_other_uri(pj_scanner *scanner,
     PJ_UNUSED_ARG(parse_params);
 
     scanner->skip_ws = 0;
-
-    uri = pjsip_other_uri_create(pool);
-
+    
+    uri = pjsip_other_uri_create(pool); 
+    
     pj_scan_get(scanner, &pc->pjsip_TOKEN_SPEC, &uri->scheme);
     if (pj_scan_get_char(scanner) != ':') {
 	PJ_THROW(PJSIP_SYN_ERR_EXCEPTION);
     }
-
+    
     pj_scan_get(scanner, &pc->pjsip_OTHER_URI_CONTENT, &uri->content);
     scanner->skip_ws = skip_ws;
-
+    
     return uri;
 }
 
@@ -1576,7 +1584,7 @@ static void int_parse_req_line( pj_scanner *scanner, pj_pool_t *pool,
 }
 
 /* Parse status line. */
-static void int_parse_status_line( pj_scanner *scanner,
+static void int_parse_status_line( pj_scanner *scanner, 
 				   pjsip_status_line *status_line)
 {
     pj_str_t token;
@@ -1602,12 +1610,12 @@ PJ_DEF(pj_status_t) pjsip_parse_status_line( char *buf, pj_size_t size,
     PJ_USE_EXCEPTION;
 
     pj_bzero(status_line, sizeof(*status_line));
-    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER,
+    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER, 
 		 &on_syntax_error);
 
     PJ_TRY {
 	int_parse_status_line(&scanner, status_line);
-    }
+    } 
     PJ_CATCH_ANY {
 	/* Tolerate the error if it is caused only by missing newline */
 	if (status_line->code == 0 && status_line->reason.slen == 0) {
@@ -1647,8 +1655,8 @@ static void parse_generic_array_hdr( pjsip_generic_array_hdr *hdr,
     /* Some header fields allow empty elements in the value:
      *   Accept, Allow, Supported
      */
-    if (pj_scan_is_eof(scanner) ||
-	*scanner->curptr == '\r' || *scanner->curptr == '\n')
+    if (pj_scan_is_eof(scanner) || 
+	*scanner->curptr == '\r' || *scanner->curptr == '\n') 
     {
 	goto end;
     }
@@ -1659,13 +1667,13 @@ static void parse_generic_array_hdr( pjsip_generic_array_hdr *hdr,
 	return;
     }
 
-    pj_scan_get( scanner, &pconst.pjsip_NOT_COMMA_OR_NEWLINE,
+    pj_scan_get( scanner, &pconst.pjsip_NOT_COMMA_OR_NEWLINE, 
 		 &hdr->values[hdr->count]);
     hdr->count++;
 
     while (*scanner->curptr == ',') {
 	pj_scan_get_char(scanner);
-	pj_scan_get( scanner, &pconst.pjsip_NOT_COMMA_OR_NEWLINE,
+	pj_scan_get( scanner, &pconst.pjsip_NOT_COMMA_OR_NEWLINE, 
 		     &hdr->values[hdr->count]);
 	hdr->count++;
 
@@ -1677,6 +1685,14 @@ end:
     parse_hdr_end(scanner);
 }
 
+/* Parse generic array header. */
+PJ_DEF(void) pjsip_parse_generic_array_hdr_imp( pjsip_generic_array_hdr *hdr,
+						pj_scanner *scanner)
+{
+    parse_generic_array_hdr(hdr, scanner);
+}
+
+
 /* Parse generic string header. */
 static void parse_generic_string_hdr( pjsip_generic_string_hdr *hdr,
 				      pjsip_parse_ctx *ctx)
@@ -1695,7 +1711,7 @@ static void parse_generic_string_hdr( pjsip_generic_string_hdr *hdr,
 	/* mangled, get next fraction */
 	pj_scan_get( scanner, &pconst.pjsip_NOT_NEWLINE, &next);
 	/* concatenate */
-	tmp.ptr = (char*)pj_pool_alloc(ctx->pool,
+	tmp.ptr = (char*)pj_pool_alloc(ctx->pool, 
 				       hdr->hvalue.slen + next.slen + 2);
 	tmp.slen = 0;
 	pj_strcpy(&tmp, &hdr->hvalue);
@@ -1750,7 +1766,7 @@ static pjsip_hdr* parse_hdr_call_id(pjsip_parse_ctx *ctx)
 }
 
 /* Parse and interpret Contact param. */
-static void int_parse_contact_param( pjsip_contact_hdr *hdr,
+static void int_parse_contact_param( pjsip_contact_hdr *hdr, 
 				     pj_scanner *scanner,
 				     pj_pool_t *pool)
 {
@@ -1771,7 +1787,7 @@ static void int_parse_contact_param( pjsip_contact_hdr *hdr,
 		pvalue.slen = (pvalue.ptr+pvalue.slen) - (dot_pos+1);
 		pvalue.ptr = dot_pos + 1;
 		hdr->q1000 += pj_strtoul_mindigit(&pvalue, 3);
-	    }
+	    }    
 	} else if (!parser_stricmp(pname, pconst.pjsip_EXPIRES_STR) && pvalue.slen) {
 	    hdr->expires = pj_strtoul(&pvalue);
 
@@ -1789,7 +1805,7 @@ static pjsip_hdr* parse_hdr_contact( pjsip_parse_ctx *ctx )
 {
     pjsip_contact_hdr *first = NULL;
     pj_scanner *scanner = ctx->scanner;
-
+    
     do {
 	pjsip_contact_hdr *hdr = pjsip_contact_hdr_create(ctx->pool);
 	if (first == NULL)
@@ -1803,7 +1819,7 @@ static pjsip_hdr* parse_hdr_contact( pjsip_parse_ctx *ctx )
 
 	} else {
 	    hdr->star = 0;
-	    hdr->uri = int_parse_uri_or_name_addr(scanner, ctx->pool,
+	    hdr->uri = int_parse_uri_or_name_addr(scanner, ctx->pool, 
                                                   PJSIP_PARSE_URI_AS_NAMEADDR |
 						  PJSIP_PARSE_URI_IN_FROM_TO_HDR);
 
@@ -1846,7 +1862,7 @@ static pjsip_hdr* parse_hdr_content_type( pjsip_parse_ctx *ctx )
     pj_scanner *scanner = ctx->scanner;
 
     hdr = pjsip_ctype_hdr_create(ctx->pool);
-
+    
     /* Parse media type and subtype. */
     pj_scan_get(scanner, &pconst.pjsip_TOKEN_SPEC, &hdr->media.type);
     pj_scan_get_char(scanner);
@@ -1897,11 +1913,11 @@ static pjsip_hdr* parse_hdr_expires(pjsip_parse_ctx *ctx)
 }
 
 /* Parse From: or To: header. */
-static void parse_hdr_fromto( pj_scanner *scanner,
-			      pj_pool_t *pool,
+static void parse_hdr_fromto( pj_scanner *scanner, 
+			      pj_pool_t *pool, 
 			      pjsip_from_hdr *hdr)
 {
-    hdr->uri = int_parse_uri_or_name_addr(scanner, pool,
+    hdr->uri = int_parse_uri_or_name_addr(scanner, pool, 
 					  PJSIP_PARSE_URI_AS_NAMEADDR |
 					  PJSIP_PARSE_URI_IN_FROM_TO_HDR);
 
@@ -1912,7 +1928,7 @@ static void parse_hdr_fromto( pj_scanner *scanner,
 
 	if (!parser_stricmp(pname, pconst.pjsip_TAG_STR)) {
 	    hdr->tag = pvalue;
-
+	    
 	} else {
 	    pjsip_param *p = PJ_POOL_ALLOC_T(pool, pjsip_param);
 	    p->name = pname;
@@ -1941,7 +1957,7 @@ static pjsip_hdr* parse_hdr_require( pjsip_parse_ctx *ctx )
     pjsip_require_hdr *hdr;
     pj_bool_t new_hdr = (ctx->rdata==NULL ||
 			 ctx->rdata->msg_info.require == NULL);
-
+    
     if (ctx->rdata && ctx->rdata->msg_info.require) {
 	hdr = ctx->rdata->msg_info.require;
     } else {
@@ -1963,7 +1979,7 @@ static pjsip_hdr* parse_hdr_retry_after(pjsip_parse_ctx *ctx)
     pj_str_t tmp;
 
     hdr = pjsip_retry_after_hdr_create(ctx->pool, 0);
-
+    
     pj_scan_get(scanner, &pconst.pjsip_DIGIT_SPEC, &tmp);
     hdr->ivalue = pj_strtoul(&tmp);
 
@@ -1990,7 +2006,7 @@ static pjsip_hdr* parse_hdr_retry_after(pjsip_parse_ctx *ctx)
 static pjsip_hdr* parse_hdr_supported(pjsip_parse_ctx *ctx)
 {
     pjsip_supported_hdr *hdr;
-    pj_bool_t new_hdr = (ctx->rdata==NULL ||
+    pj_bool_t new_hdr = (ctx->rdata==NULL || 
 		         ctx->rdata->msg_info.supported == NULL);
 
     if (ctx->rdata && ctx->rdata->msg_info.supported) {
@@ -2035,7 +2051,7 @@ static void int_parse_via_param( pjsip_via_hdr *hdr, pj_scanner *scanner,
 	//Parse with PARAM_CHAR instead, to allow IPv6
 	//No, back to using int_parse_param() for the "`" character!
 	//int_parse_param( scanner, pool, &pname, &pvalue, 0);
-	//parse_param_imp(scanner, pool, &pname, &pvalue,
+	//parse_param_imp(scanner, pool, &pname, &pvalue, 
 	//		&pconst.pjsip_TOKEN_SPEC,
 	//		&pconst.pjsip_TOKEN_SPEC_ESC, 0);
 	//int_parse_param(scanner, pool, &pname, &pvalue, 0);
@@ -2053,7 +2069,7 @@ static void int_parse_via_param( pjsip_via_hdr *hdr, pj_scanner *scanner,
 
 	} else if (!parser_stricmp(pname, pconst.pjsip_TTL_STR) && pvalue.slen) {
 	    hdr->ttl_param = pj_strtoul(&pvalue);
-
+	    
 	} else if (!parser_stricmp(pname, pconst.pjsip_MADDR_STR) && pvalue.slen) {
 	    hdr->maddr_param = pvalue;
 
@@ -2194,7 +2210,7 @@ static pjsip_hdr* parse_hdr_via( pjsip_parse_ctx *ctx )
 	    pj_scan_get(scanner, &pconst.pjsip_DIGIT_SPEC, &digit);
 	    hdr->sent_by.port = pj_strtoul(&digit);
 	}
-
+	
 	int_parse_via_param(hdr, scanner, ctx->pool);
 
 	if (*scanner->curptr == '(') {
@@ -2238,7 +2254,7 @@ PJ_DEF(void*) pjsip_parse_hdr( pj_pool_t *pool, const pj_str_t *hname,
     pjsip_parse_ctx context;
     PJ_USE_EXCEPTION;
 
-    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER,
+    pj_scan_init(&scanner, buf, size, PJ_SCAN_AUTOSKIP_WS_HEADER, 
                  &on_syntax_error);
 
     context.scanner = &scanner;
@@ -2256,14 +2272,14 @@ PJ_DEF(void*) pjsip_parse_hdr( pj_pool_t *pool, const pj_str_t *hname,
 	    hdr->sname = hdr->name;
 	}
 
-    }
+    } 
     PJ_CATCH_ANY {
 	hdr = NULL;
     }
     PJ_END
 
     if (parsed_len) {
-	*parsed_len = (scanner.curptr - scanner.begin);
+	*parsed_len = (unsigned)(scanner.curptr - scanner.begin);
     }
 
     pj_scan_fini(&scanner);
@@ -2376,3 +2392,4 @@ retry_parse:
 
     return PJ_SUCCESS;
 }
+
