@@ -1,5 +1,5 @@
-/* $Id: sip_transport.c 4094 2012-04-26 09:31:00Z bennylp $ */
-/*
+/* $Id: sip_transport.c 4775 2014-03-04 02:18:51Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip/sip_transport.h>
 #include <pjsip/sip_endpoint.h>
@@ -24,6 +24,7 @@
 #include <pjsip/sip_private.h>
 #include <pjsip/sip_errno.h>
 #include <pjsip/sip_module.h>
+#include <pj/addr_resolv.h>
 #include <pj/except.h>
 #include <pj/os.h>
 #include <pj/log.h>
@@ -44,7 +45,7 @@
 static const char *addr_string(const pj_sockaddr_t *addr)
 {
     static char str[PJ_INET6_ADDRSTRLEN];
-    pj_inet_ntop(((const pj_sockaddr*)addr)->addr.sa_family,
+    pj_inet_ntop(((const pj_sockaddr*)addr)->addr.sa_family, 
 		 pj_sockaddr_get_addr(addr),
 		 str, sizeof(str));
     return str;
@@ -57,9 +58,9 @@ static const char *addr_string(const pj_sockaddr_t *addr)
 static pj_status_t mod_on_tx_msg(pjsip_tx_data *tdata);
 
 /* This module has sole purpose to print transmit data to contigous buffer
- * before actually transmitted to the wire.
+ * before actually transmitted to the wire. 
  */
-static pjsip_module mod_msg_print =
+static pjsip_module mod_msg_print = 
 {
     NULL, NULL,				/* prev and next		    */
     { "mod-msg-print", 13},		/* Name.			    */
@@ -79,7 +80,7 @@ static pjsip_module mod_msg_print =
 /*
  * Transport manager.
  */
-struct pjsip_tpmgr
+struct pjsip_tpmgr 
 {
     pj_hash_table_t *table;
     pj_lock_t	    *lock;
@@ -91,6 +92,11 @@ struct pjsip_tpmgr
     void           (*on_rx_msg)(pjsip_endpoint*, pj_status_t, pjsip_rx_data*);
     pj_status_t	   (*on_tx_msg)(pjsip_endpoint*, pjsip_tx_data*);
     pjsip_tp_state_callback tp_state_cb;
+
+    /* Transmit data list, for transmit data cleanup when transport manager
+     * is destroyed.
+     */
+    pjsip_tx_data    tdata_list;
 };
 
 
@@ -124,7 +130,7 @@ typedef struct transport_data
 /*
  * Transport names.
  */
-struct transport_names_t
+static struct transport_names_t
 {
     pjsip_transport_type_e type;	    /* Transport type	    */
     pj_uint16_t		   port;	    /* Default port number  */
@@ -132,71 +138,78 @@ struct transport_names_t
     const char		  *description;	    /* Longer description   */
     unsigned		   flag;	    /* Flags		    */
     char		   name_buf[16];    /* For user's transport */
-} transport_names[16] =
+} transport_names[16] = 
 {
-    {
-	PJSIP_TRANSPORT_UNSPECIFIED,
-	0,
-	{"Unspecified", 11},
-	"Unspecified",
+    { 
+	PJSIP_TRANSPORT_UNSPECIFIED, 
+	0, 
+	{"Unspecified", 11}, 
+	"Unspecified", 
 	0
     },
-    {
-	PJSIP_TRANSPORT_UDP,
-	5060,
-	{"UDP", 3},
-	"UDP transport",
+    { 
+	PJSIP_TRANSPORT_UDP, 
+	5060, 
+	{"UDP", 3}, 
+	"UDP transport", 
 	PJSIP_TRANSPORT_DATAGRAM
     },
-    {
-	PJSIP_TRANSPORT_TCP,
-	5060,
-	{"TCP", 3},
-	"TCP transport",
+    { 
+	PJSIP_TRANSPORT_TCP, 
+	5060, 
+	{"TCP", 3}, 
+	"TCP transport", 
 	PJSIP_TRANSPORT_RELIABLE
     },
-    {
-	PJSIP_TRANSPORT_TLS,
-	5061,
-	{"TLS", 3},
-	"TLS transport",
+    { 
+	PJSIP_TRANSPORT_TLS, 
+	5061, 
+	{"TLS", 3}, 
+	"TLS transport", 
 	PJSIP_TRANSPORT_RELIABLE | PJSIP_TRANSPORT_SECURE
     },
-    {
-	PJSIP_TRANSPORT_SCTP,
-	5060,
-	{"SCTP", 4},
-	"SCTP transport",
+    { 
+	PJSIP_TRANSPORT_SCTP, 
+	5060, 
+	{"SCTP", 4}, 
+	"SCTP transport", 
 	PJSIP_TRANSPORT_RELIABLE
     },
-    {
-	PJSIP_TRANSPORT_LOOP,
-	15060,
-	{"LOOP", 4},
-	"Loopback transport",
+    { 
+	PJSIP_TRANSPORT_LOOP, 
+	15060, 
+	{"LOOP", 4}, 
+	"Loopback transport", 
 	PJSIP_TRANSPORT_RELIABLE
-    },
-    {
-	PJSIP_TRANSPORT_LOOP_DGRAM,
-	15060,
-	{"LOOP-DGRAM", 10},
-	"Loopback datagram transport",
+    }, 
+    { 
+	PJSIP_TRANSPORT_LOOP_DGRAM, 
+	15060, 
+	{"LOOP-DGRAM", 10}, 
+	"Loopback datagram transport", 
 	PJSIP_TRANSPORT_DATAGRAM
     },
-    {
-	PJSIP_TRANSPORT_UDP6,
-	5060,
-	{"UDP", 3},
-	"UDP IPv6 transport",
+    { 
+	PJSIP_TRANSPORT_UDP6, 
+	5060, 
+	{"UDP", 3}, 
+	"UDP IPv6 transport", 
 	PJSIP_TRANSPORT_DATAGRAM
     },
-    {
-	PJSIP_TRANSPORT_TCP6,
-	5060,
-	{"TCP", 3},
-	"TCP IPv6 transport",
+    { 
+	PJSIP_TRANSPORT_TCP6, 
+	5060, 
+	{"TCP", 3}, 
+	"TCP IPv6 transport", 
 	PJSIP_TRANSPORT_RELIABLE
     },
+    {
+	PJSIP_TRANSPORT_TLS6,
+	5061,
+	{"TLS", 3},
+	"TLS IPv6 transport",
+	PJSIP_TRANSPORT_RELIABLE | PJSIP_TRANSPORT_SECURE
+    },
 };
 
 static void tp_state_callback(pjsip_transport *tp,
@@ -204,7 +217,7 @@ static void tp_state_callback(pjsip_transport *tp,
 			      const pjsip_transport_state_info *info);
 
 
-struct transport_names_t *get_tpname(pjsip_transport_type_e type)
+static struct transport_names_t *get_tpname(pjsip_transport_type_e type)
 {
     unsigned i;
     for (i=0; i<PJ_ARRAY_SIZE(transport_names); ++i) {
@@ -228,8 +241,8 @@ PJ_DEF(pj_status_t) pjsip_transport_register_type( unsigned tp_flag,
     unsigned i;
 
     PJ_ASSERT_RETURN(tp_flag && tp_name && def_port, PJ_EINVAL);
-    PJ_ASSERT_RETURN(pj_ansi_strlen(tp_name) <
-			PJ_ARRAY_SIZE(transport_names[0].name_buf),
+    PJ_ASSERT_RETURN(pj_ansi_strlen(tp_name) < 
+			PJ_ARRAY_SIZE(transport_names[0].name_buf), 
 		     PJ_ENAMETOOLONG);
 
     for (i=1; i<PJ_ARRAY_SIZE(transport_names); ++i) {
@@ -402,7 +415,7 @@ PJ_DEF(pj_status_t) pjsip_tx_data_create( pjsip_tpmgr *mgr,
 	pjsip_endpt_release_pool( mgr->endpt, tdata->pool );
 	return status;
     }
-
+    
     //status = pj_lock_create_simple_mutex(pool, "tdta%p", &tdata->lock);
     status = pj_lock_create_null_mutex(pool, "tdta%p", &tdata->lock);
     if (status != PJ_SUCCESS) {
@@ -411,6 +424,14 @@ PJ_DEF(pj_status_t) pjsip_tx_data_create( pjsip_tpmgr *mgr,
     }
 
     pj_ioqueue_op_key_init(&tdata->op_key.key, sizeof(tdata->op_key.key));
+    pj_list_init(tdata);
+
+#if defined(PJSIP_HAS_TX_DATA_LIST) && PJSIP_HAS_TX_DATA_LIST!=0
+    /* Append this just created tdata to transmit buffer list */
+    pj_lock_acquire(mgr->lock);
+    pj_list_push_back(&mgr->tdata_list, tdata);
+    pj_lock_release(mgr->lock);
+#endif
 
 #if defined(PJ_DEBUG) && PJ_DEBUG!=0
     pj_atomic_inc( tdata->mgr->tdata_counter );
@@ -429,6 +450,27 @@ PJ_DEF(void) pjsip_tx_data_add_ref( pjsip_tx_data *tdata )
     pj_atomic_inc(tdata->ref_cnt);
 }
 
+static void tx_data_destroy(pjsip_tx_data *tdata)
+{
+    PJ_LOG(5,(tdata->obj_name, "Destroying txdata %s",
+	      pjsip_tx_data_get_info(tdata)));
+    pjsip_tpselector_dec_ref(&tdata->tp_sel);
+#if defined(PJ_DEBUG) && PJ_DEBUG!=0
+    pj_atomic_dec( tdata->mgr->tdata_counter );
+#endif
+
+#if defined(PJSIP_HAS_TX_DATA_LIST) && PJSIP_HAS_TX_DATA_LIST!=0
+    /* Remove this tdata from transmit buffer list */
+    pj_lock_acquire(tdata->mgr->lock);
+    pj_list_erase(tdata);
+    pj_lock_release(tdata->mgr->lock);
+#endif
+
+    pj_atomic_destroy( tdata->ref_cnt );
+    pj_lock_destroy( tdata->lock );
+    pjsip_endpt_release_pool( tdata->mgr->endpt, tdata->pool );
+}
+
 /*
  * Decrease transport data reference, destroy it when the reference count
  * reaches zero.
@@ -437,15 +479,7 @@ PJ_DEF(pj_status_t) pjsip_tx_data_dec_ref( pjsip_tx_data *tdata )
 {
     pj_assert( pj_atomic_get(tdata->ref_cnt) > 0);
     if (pj_atomic_dec_and_get(tdata->ref_cnt) <= 0) {
-	PJ_LOG(5,(tdata->obj_name, "Destroying txdata %s",
-		  pjsip_tx_data_get_info(tdata)));
-	pjsip_tpselector_dec_ref(&tdata->tp_sel);
-#if defined(PJ_DEBUG) && PJ_DEBUG!=0
-	pj_atomic_dec( tdata->mgr->tdata_counter );
-#endif
-	pj_atomic_destroy( tdata->ref_cnt );
-	pj_lock_destroy( tdata->lock );
-	pjsip_endpt_release_pool( tdata->mgr->endpt, tdata->pool );
+	tx_data_destroy(tdata);
 	return PJSIP_EBUFDESTROYED;
     } else {
 	return PJ_SUCCESS;
@@ -472,7 +506,7 @@ PJ_DEF(pj_status_t) pjsip_tx_data_encode(pjsip_tx_data *tdata)
 	PJ_USE_EXCEPTION;
 
 	PJ_TRY {
-	    tdata->buf.start = (char*)
+	    tdata->buf.start = (char*) 
 			       pj_pool_alloc(tdata->pool, PJSIP_MAX_PKT_LEN);
 	}
 	PJ_CATCH_ANY {
@@ -488,7 +522,7 @@ PJ_DEF(pj_status_t) pjsip_tx_data_encode(pjsip_tx_data *tdata)
     if (!pjsip_tx_data_is_valid(tdata)) {
 	pj_ssize_t size;
 
-	size = pjsip_msg_print( tdata->msg, tdata->buf.start,
+	size = pjsip_msg_print( tdata->msg, tdata->buf.start, 
 			        tdata->buf.end - tdata->buf.start);
 	if (size < 0) {
 	    return PJSIP_EMSGTOOLONG;
@@ -517,7 +551,7 @@ static char *get_msg_info(pj_pool_t *pool, const char *obj_name,
     PJ_ASSERT_RETURN(cseq != NULL, "INVALID MSG");
 
     if (msg->type == PJSIP_REQUEST_MSG) {
-	len = pj_ansi_snprintf(info_buf, sizeof(info_buf),
+	len = pj_ansi_snprintf(info_buf, sizeof(info_buf), 
 			       "Request msg %.*s/cseq=%d (%s)",
 			       (int)msg->line.req.method.name.slen,
 			       msg->line.req.method.name.ptr,
@@ -543,13 +577,15 @@ static char *get_msg_info(pj_pool_t *pool, const char *obj_name,
 
 PJ_DEF(char*) pjsip_tx_data_get_info( pjsip_tx_data *tdata )
 {
+    PJ_ASSERT_RETURN(tdata, "NULL");
+
     /* tdata->info may be assigned by application so if it exists
      * just return it.
      */
     if (tdata->info)
 	return tdata->info;
 
-    if (tdata==NULL || tdata->msg==NULL)
+    if (tdata->msg==NULL)
 	return "NULL";
 
     pj_lock_acquire(tdata->lock);
@@ -594,6 +630,89 @@ PJ_DEF(char*) pjsip_rx_data_get_info(pjsip_rx_data *rdata)
     return rdata->msg_info.info;
 }
 
+/* Clone pjsip_rx_data. */
+PJ_DEF(pj_status_t) pjsip_rx_data_clone( const pjsip_rx_data *src,
+                                         unsigned flags,
+                                         pjsip_rx_data **p_rdata)
+{
+    pj_pool_t *pool;
+    pjsip_rx_data *dst;
+    pjsip_hdr *hdr;
+
+    PJ_ASSERT_RETURN(src && flags==0 && p_rdata, PJ_EINVAL);
+
+    pool = pj_pool_create(src->tp_info.pool->factory,
+                          "rtd%p",
+                          PJSIP_POOL_RDATA_LEN,
+                          PJSIP_POOL_RDATA_INC,
+                          NULL);
+    if (!pool)
+	return PJ_ENOMEM;
+
+    dst = PJ_POOL_ZALLOC_T(pool, pjsip_rx_data);
+
+    /* Parts of tp_info */
+    dst->tp_info.pool = pool;
+    dst->tp_info.transport = (pjsip_transport*)src->tp_info.transport;
+
+    /* pkt_info can be memcopied */
+    pj_memcpy(&dst->pkt_info, &src->pkt_info, sizeof(src->pkt_info));
+
+    /* msg_info needs deep clone */
+    dst->msg_info.msg_buf = dst->pkt_info.packet;
+    dst->msg_info.len = src->msg_info.len;
+    dst->msg_info.msg = pjsip_msg_clone(pool, src->msg_info.msg);
+    pj_list_init(&dst->msg_info.parse_err);
+
+#define GET_MSG_HDR2(TYPE, type, var)	\
+			case PJSIP_H_##TYPE: \
+			    if (!dst->msg_info.var) { \
+				dst->msg_info.var = (pjsip_##type##_hdr*)hdr; \
+			    } \
+			    break
+#define GET_MSG_HDR(TYPE, var_type)	GET_MSG_HDR2(TYPE, var_type, var_type)
+
+    hdr = dst->msg_info.msg->hdr.next;
+    while (hdr != &dst->msg_info.msg->hdr) {
+	switch (hdr->type) {
+	GET_MSG_HDR(CALL_ID, cid);
+	GET_MSG_HDR(FROM, from);
+	GET_MSG_HDR(TO, to);
+	GET_MSG_HDR(VIA, via);
+	GET_MSG_HDR(CSEQ, cseq);
+	GET_MSG_HDR(MAX_FORWARDS, max_fwd);
+	GET_MSG_HDR(ROUTE, route);
+	GET_MSG_HDR2(RECORD_ROUTE, rr, record_route);
+	GET_MSG_HDR(CONTENT_TYPE, ctype);
+	GET_MSG_HDR(CONTENT_LENGTH, clen);
+	GET_MSG_HDR(REQUIRE, require);
+	GET_MSG_HDR(SUPPORTED, supported);
+	default:
+	    break;
+	}
+	hdr = hdr->next;
+    }
+
+#undef GET_MSG_HDR
+#undef GET_MSG_HDR2
+
+    *p_rdata = dst;
+
+    /* Finally add transport ref */
+    return pjsip_transport_add_ref(dst->tp_info.transport);
+}
+
+/* Free previously cloned pjsip_rx_data. */
+PJ_DEF(pj_status_t) pjsip_rx_data_free_cloned(pjsip_rx_data *rdata)
+{
+    PJ_ASSERT_RETURN(rdata, PJ_EINVAL);
+
+    pjsip_transport_dec_ref(rdata->tp_info.transport);
+    pj_pool_release(rdata->tp_info.pool);
+
+    return PJ_SUCCESS;
+}
+
 /*****************************************************************************
  *
  * TRANSPORT KEY
@@ -640,7 +759,7 @@ static pj_status_t mod_on_tx_msg(pjsip_tx_data *tdata)
 /*
  * Send a SIP message using the specified transport.
  */
-PJ_DEF(pj_status_t) pjsip_transport_send(  pjsip_transport *tr,
+PJ_DEF(pj_status_t) pjsip_transport_send(  pjsip_transport *tr, 
 					   pjsip_tx_data *tdata,
 					   const pj_sockaddr_t *addr,
 					   int addr_len,
@@ -654,7 +773,7 @@ PJ_DEF(pj_status_t) pjsip_transport_send(  pjsip_transport *tr,
     /* Is it currently being sent? */
     if (tdata->is_pending) {
 	pj_assert(!"Invalid operation step!");
-	PJ_LOG(2,(THIS_FILE, "Unable to send %s: message is pending",
+	PJ_LOG(2,(THIS_FILE, "Unable to send %s: message is pending", 
 			     pjsip_tx_data_get_info(tdata)));
 	return PJSIP_EPENDINGTX;
     }
@@ -675,7 +794,7 @@ PJ_DEF(pj_status_t) pjsip_transport_send(  pjsip_transport *tr,
 		 sizeof(tdata->tp_info.dst_name));
     tdata->tp_info.dst_port = pj_sockaddr_get_port(addr);
 
-    /* Distribute to modules.
+    /* Distribute to modules. 
      * When the message reach mod_msg_print, the contents of the message will
      * be "printed" to contiguous buffer.
      */
@@ -698,7 +817,7 @@ PJ_DEF(pj_status_t) pjsip_transport_send(  pjsip_transport *tr,
     tdata->is_pending = 1;
 
     /* Send to transport. */
-    status = (*tr->send_msg)(tr, tdata,  addr, addr_len, (void*)tdata,
+    status = (*tr->send_msg)(tr, tdata,  addr, addr_len, (void*)tdata, 
 			     &transport_send_callback);
 
     if (status != PJ_EPENDING) {
@@ -750,7 +869,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_send_raw(pjsip_tpmgr *mgr,
 {
     pjsip_transport *tr;
     pj_status_t status;
-
+ 
     /* Acquire the transport */
     status = pjsip_tpmgr_acquire_transport(mgr, tp_type, addr, addr_len,
 					   sel, &tr);
@@ -779,13 +898,13 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_send_raw(pjsip_tpmgr *mgr,
 	tdata->buf.start = (char*) pj_pool_alloc(tdata->pool, data_len+1);
 	tdata->buf.end = tdata->buf.start + data_len + 1;
     }
-
+ 
     /* Copy data, if any! (application may send zero len packet) */
     if (data_len) {
 	pj_memcpy(tdata->buf.start, raw_data, data_len);
     }
     tdata->buf.cur = tdata->buf.start + data_len;
-
+ 
     /* Save callback data. */
     tdata->token = token;
     tdata->cb = cb;
@@ -796,7 +915,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_send_raw(pjsip_tpmgr *mgr,
     /* Send to transport */
     status = tr->send_msg(tr, tdata, addr, addr_len,
 			  tdata, &send_raw_callback);
-
+ 
     if (status != PJ_EPENDING) {
 	/* callback will not be called, so destroy tdata now. */
 	pjsip_tx_data_dec_ref(tdata);
@@ -857,7 +976,7 @@ PJ_DEF(pj_status_t) pjsip_transport_dec_ref( pjsip_transport *tp )
 	 */
 	if (pj_atomic_get(tp->ref_cnt) == 0 && !tp->is_destroying) {
 	    pj_time_val delay;
-
+	    
 	    /* If transport is in graceful shutdown, then this is the
 	     * last user who uses the transport. Schedule to destroy the
 	     * transport immediately. Otherwise schedule idle timer.
@@ -873,7 +992,7 @@ PJ_DEF(pj_status_t) pjsip_transport_dec_ref( pjsip_transport *tp )
 
 	    pj_assert(tp->idle_timer.id == 0);
 	    tp->idle_timer.id = PJ_TRUE;
-	    pjsip_endpt_schedule_timer(tp->tpmgr->endpt, &tp->idle_timer,
+	    pjsip_endpt_schedule_timer(tp->tpmgr->endpt, &tp->idle_timer, 
 				       &delay);
 	}
 	pj_lock_release(tp->tpmgr->lock);
@@ -899,7 +1018,7 @@ PJ_DEF(pj_status_t) pjsip_transport_register( pjsip_tpmgr *mgr,
     tp->idle_timer.user_data = tp;
     tp->idle_timer.cb = &transport_idle_callback;
 
-    /*
+    /* 
      * Register to hash table (see Trac ticket #42).
      */
     key_len = sizeof(tp->key.type) + tp->addr_len;
@@ -966,12 +1085,13 @@ static pj_status_t destroy_transport( pjsip_tpmgr *mgr,
 
 
 /*
- * Start graceful shutdown procedure for this transport.
+ * Start graceful shutdown procedure for this transport. 
  */
 PJ_DEF(pj_status_t) pjsip_transport_shutdown(pjsip_transport *tp)
 {
     pjsip_tpmgr *mgr;
     pj_status_t status;
+    pjsip_tp_state_callback state_cb;
 
     TRACE_((THIS_FILE, "Transport %s shutting down", tp->obj_name));
 
@@ -993,6 +1113,16 @@ PJ_DEF(pj_status_t) pjsip_transport_shutdown(pjsip_transport *tp)
     if (tp->do_shutdown)
 	status = tp->do_shutdown(tp);
 
+    /* Notify application of transport shutdown */
+    state_cb = pjsip_tpmgr_get_state_cb(tp->tpmgr);
+    if (state_cb) {
+	pjsip_transport_state_info state_info;
+
+	pj_bzero(&state_info, sizeof(state_info));
+	state_info.status = status;
+        (*state_cb)(tp, PJSIP_TP_STATE_SHUTDOWN, &state_info);
+    }
+
     if (status == PJ_SUCCESS)
 	tp->is_shutdown = PJ_TRUE;
 
@@ -1014,9 +1144,20 @@ PJ_DEF(pj_status_t) pjsip_transport_shutdown(pjsip_transport *tp)
  */
 PJ_DEF(pj_status_t) pjsip_transport_destroy( pjsip_transport *tp)
 {
+    pjsip_tp_state_callback state_cb;
+
     /* Must have no user. */
     PJ_ASSERT_RETURN(pj_atomic_get(tp->ref_cnt) == 0, PJSIP_EBUSY);
 
+    /* Notify application of transport destroy */
+    state_cb = pjsip_tpmgr_get_state_cb(tp->tpmgr);
+    if (state_cb) {
+	pjsip_transport_state_info state_info;
+
+	pj_bzero(&state_info, sizeof(state_info));
+        (*state_cb)(tp, PJSIP_TP_STATE_DESTROY, &state_info);
+    }
+
     /* Destroy. */
     return destroy_transport(tp->tpmgr, tp);
 }
@@ -1080,6 +1221,10 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_unregister_tpfactory( pjsip_tpmgr *mgr,
     return PJ_SUCCESS;
 }
 
+PJ_DECL(void) pjsip_tpmgr_fla2_param_default(pjsip_tpmgr_fla2_param *prm)
+{
+    pj_bzero(prm, sizeof(*prm));
+}
 
 /*****************************************************************************
  *
@@ -1112,6 +1257,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_create( pj_pool_t *pool,
     mgr->on_rx_msg = rx_cb;
     mgr->on_tx_msg = tx_cb;
     pj_list_init(&mgr->factory_list);
+    pj_list_init(&mgr->tdata_list);
 
     mgr->table = pj_hash_create(pool, PJSIP_TPMGR_HTABLE_SIZE);
     if (!mgr->table)
@@ -1123,14 +1269,14 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_create( pj_pool_t *pool,
 
 #if defined(PJ_DEBUG) && PJ_DEBUG!=0
     status = pj_atomic_create(pool, 0, &mgr->tdata_counter);
-    if (status != PJ_SUCCESS)
-	return status;
+    if (status != PJ_SUCCESS) {
+    	pj_lock_destroy(mgr->lock);
+    	return status;
+    }
 #endif
 
     /* Set transport state callback */
-    status = pjsip_tpmgr_set_state_cb(mgr, &tp_state_callback);
-    if (status != PJ_SUCCESS)
-	return status;
+    pjsip_tpmgr_set_state_cb(mgr, &tp_state_callback);
 
     PJ_LOG(5, (THIS_FILE, "Transport manager created."));
 
@@ -1138,57 +1284,105 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_create( pj_pool_t *pool,
     return PJ_SUCCESS;
 }
 
+/* Get the interface to send packet to the specified address */
+static pj_status_t get_net_interface(pjsip_transport_type_e tp_type,
+				     const pj_str_t *dst,
+                                     pj_str_t *itf_str_addr)
+{
+    int af;
+    pj_sockaddr itf_addr;
+    pj_status_t status;
 
+    af = (tp_type & PJSIP_TRANSPORT_IPV6)? PJ_AF_INET6 : PJ_AF_INET;
+    status = pj_getipinterface(af, dst, &itf_addr, PJ_FALSE, NULL);
+    if (status != PJ_SUCCESS) {
+	/* If it fails, e.g: on WM6 (http://support.microsoft.com/kb/129065),
+	 * just fallback using pj_gethostip(), see ticket #1660.
+	 */
+	PJ_LOG(5,(THIS_FILE,"Warning: unable to determine local "
+			    "interface, fallback to default interface!"));
+	status = pj_gethostip(af, &itf_addr);
+	if (status != PJ_SUCCESS)
+	    return status;
+    }
+
+    /* Print address */
+    pj_sockaddr_print(&itf_addr, itf_str_addr->ptr,
+		      PJ_INET6_ADDRSTRLEN, 0);
+    itf_str_addr->slen = pj_ansi_strlen(itf_str_addr->ptr);
+
+    return PJ_SUCCESS;
+}
 
 /*
  * Find out the appropriate local address info (IP address and port) to
- * advertise in Contact header based on the remote address to be
+ * advertise in Contact header based on the remote address to be 
  * contacted. The local address info would be the address name of the
  * transport or listener which will be used to send the request.
  *
  * In this implementation, it will only select the transport based on
  * the transport type in the request.
  */
-PJ_DEF(pj_status_t) pjsip_tpmgr_find_local_addr( pjsip_tpmgr *tpmgr,
+PJ_DEF(pj_status_t) pjsip_tpmgr_find_local_addr2(pjsip_tpmgr *tpmgr,
 						 pj_pool_t *pool,
-						 pjsip_transport_type_e type,
-						 const pjsip_tpselector *sel,
-						 pj_str_t *ip_addr,
-						 int *port)
+						 pjsip_tpmgr_fla2_param *prm)
 {
+    char tmp_buf[PJ_INET6_ADDRSTRLEN+10];
+    pj_str_t tmp_str;
     pj_status_t status = PJSIP_EUNSUPTRANSPORT;
     unsigned flag;
 
     /* Sanity checks */
-    PJ_ASSERT_RETURN(tpmgr && pool && ip_addr && port, PJ_EINVAL);
+    PJ_ASSERT_RETURN(tpmgr && pool && prm, PJ_EINVAL);
 
-    ip_addr->slen = 0;
-    *port = 0;
+    pj_strset(&tmp_str, tmp_buf, 0);
+    prm->ret_addr.slen = 0;
+    prm->ret_port = 0;
+    prm->ret_tp = NULL;
 
-    flag = pjsip_transport_get_flag_from_type(type);
+    flag = pjsip_transport_get_flag_from_type(prm->tp_type);
 
-    if (sel && sel->type == PJSIP_TPSELECTOR_TRANSPORT &&
-	sel->u.transport)
+    if (prm->tp_sel && prm->tp_sel->type == PJSIP_TPSELECTOR_TRANSPORT &&
+	prm->tp_sel->u.transport)
     {
-	pj_strdup(pool, ip_addr, &sel->u.transport->local_name.host);
-	*port = sel->u.transport->local_name.port;
+	const pjsip_transport *tp = prm->tp_sel->u.transport;
+	if (prm->local_if) {
+	    status = get_net_interface((pjsip_transport_type_e)tp->key.type,
+				       &prm->dst_host, &tmp_str);
+	    if (status != PJ_SUCCESS)
+		goto on_return;
+	    pj_strdup(pool, &prm->ret_addr, &tmp_str);
+	    prm->ret_port = pj_sockaddr_get_port(&tp->local_addr);
+	    prm->ret_tp = tp;
+	} else {
+	    pj_strdup(pool, &prm->ret_addr, &tp->local_name.host);
+	    prm->ret_port = (pj_uint16_t)tp->local_name.port;
+	}
 	status = PJ_SUCCESS;
 
-    } else if (sel && sel->type == PJSIP_TPSELECTOR_LISTENER &&
-	       sel->u.listener)
+    } else if (prm->tp_sel && prm->tp_sel->type == PJSIP_TPSELECTOR_LISTENER &&
+	       prm->tp_sel->u.listener)
     {
-	pj_strdup(pool, ip_addr, &sel->u.listener->addr_name.host);
-	*port = sel->u.listener->addr_name.port;
+	if (prm->local_if) {
+	    status = get_net_interface(prm->tp_sel->u.listener->type,
+	                               &prm->dst_host, &tmp_str);
+	    if (status != PJ_SUCCESS)
+		goto on_return;
+	    pj_strdup(pool, &prm->ret_addr, &tmp_str);
+	} else {
+	    pj_strdup(pool, &prm->ret_addr,
+		      &prm->tp_sel->u.listener->addr_name.host);
+	}
+	prm->ret_port = (pj_uint16_t)prm->tp_sel->u.listener->addr_name.port;
 	status = PJ_SUCCESS;
 
     } else if ((flag & PJSIP_TRANSPORT_DATAGRAM) != 0) {
-
 	pj_sockaddr remote;
 	int addr_len;
 	pjsip_transport *tp;
 
 	pj_bzero(&remote, sizeof(remote));
-	if (type & PJSIP_TRANSPORT_IPV6) {
+	if (prm->tp_type & PJSIP_TRANSPORT_IPV6) {
 	    addr_len = sizeof(pj_sockaddr_in6);
 	    remote.addr.sa_family = pj_AF_INET6();
 	} else {
@@ -1196,13 +1390,23 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_find_local_addr( pjsip_tpmgr *tpmgr,
 	    remote.addr.sa_family = pj_AF_INET();
 	}
 
-	status = pjsip_tpmgr_acquire_transport(tpmgr, type, &remote,
+	status = pjsip_tpmgr_acquire_transport(tpmgr, prm->tp_type, &remote,
 					       addr_len, NULL, &tp);
 
 	if (status == PJ_SUCCESS) {
-	    pj_strdup(pool, ip_addr, &tp->local_name.host);
-	    *port = tp->local_name.port;
-	    status = PJ_SUCCESS;
+	    if (prm->local_if) {
+		status = get_net_interface((pjsip_transport_type_e)
+					   tp->key.type,
+					   &prm->dst_host, &tmp_str);
+		if (status != PJ_SUCCESS)
+		    goto on_return;
+		pj_strdup(pool, &prm->ret_addr, &tmp_str);
+		prm->ret_port = pj_sockaddr_get_port(&tp->local_addr);
+		prm->ret_tp = tp;
+	    } else {
+		pj_strdup(pool, &prm->ret_addr, &tp->local_name.host);
+		prm->ret_port = (pj_uint16_t)tp->local_name.port;
+	    }
 
 	    pjsip_transport_dec_ref(tp);
 	}
@@ -1215,22 +1419,66 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_find_local_addr( pjsip_tpmgr *tpmgr,
 
 	f = tpmgr->factory_list.next;
 	while (f != &tpmgr->factory_list) {
-	    if (f->type == type)
+	    if (f->type == prm->tp_type)
 		break;
 	    f = f->next;
 	}
 
 	if (f != &tpmgr->factory_list) {
-	    pj_strdup(pool, ip_addr, &f->addr_name.host);
-	    *port = f->addr_name.port;
+	    if (prm->local_if) {
+		status = get_net_interface(f->type, &prm->dst_host,
+					   &tmp_str);
+		if (status == PJ_SUCCESS) {
+		    pj_strdup(pool, &prm->ret_addr, &tmp_str);
+		} else {
+		    /* It could fail "normally" on certain cases, e.g.
+		     * when connecting to IPv6 link local address, it
+		     * will wail with EINVAL.
+		     * In this case, fallback to use the default interface
+		     * rather than failing the call.
+		     */
+		    PJ_PERROR(5,(THIS_FILE, status, "Warning: unable to "
+			         "determine local interface"));
+		    pj_strdup(pool, &prm->ret_addr, &f->addr_name.host);
+		    status = PJ_SUCCESS;
+		}
+	    } else {
+		pj_strdup(pool, &prm->ret_addr, &f->addr_name.host);
+	    }
+	    prm->ret_port = (pj_uint16_t)f->addr_name.port;
 	    status = PJ_SUCCESS;
 	}
 	pj_lock_release(tpmgr->lock);
     }
 
+on_return:
     return status;
 }
 
+PJ_DEF(pj_status_t) pjsip_tpmgr_find_local_addr( pjsip_tpmgr *tpmgr,
+						 pj_pool_t *pool,
+						 pjsip_transport_type_e type,
+						 const pjsip_tpselector *sel,
+						 pj_str_t *ip_addr,
+						 int *port)
+{
+    pjsip_tpmgr_fla2_param prm;
+    pj_status_t status;
+
+    pjsip_tpmgr_fla2_param_default(&prm);
+    prm.tp_type = type;
+    prm.tp_sel = sel;
+
+    status = pjsip_tpmgr_find_local_addr2(tpmgr, pool, &prm);
+    if (status != PJ_SUCCESS)
+	return status;
+
+    *ip_addr = prm.ret_addr;
+    *port = prm.ret_port;
+
+    return PJ_SUCCESS;
+}
+
 /*
  * Return number of transports currently registered to the transport
  * manager.
@@ -1240,15 +1488,15 @@ PJ_DEF(unsigned) pjsip_tpmgr_get_transport_count(pjsip_tpmgr *mgr)
     pj_hash_iterator_t itr_val;
     pj_hash_iterator_t *itr;
     int nr_of_transports = 0;
-
+    
     pj_lock_acquire(mgr->lock);
-
+    
     itr = pj_hash_first(mgr->table, &itr_val);
     while (itr) {
 	nr_of_transports++;
 	itr = pj_hash_next(mgr->table, itr);
     }
-
+    
     pj_lock_release(mgr->lock);
 
     return nr_of_transports;
@@ -1265,7 +1513,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_destroy( pjsip_tpmgr *mgr )
     pj_hash_iterator_t *itr;
     pjsip_tpfactory *factory;
     pjsip_endpoint *endpt = mgr->endpt;
-
+    
     PJ_LOG(5, (THIS_FILE, "Destroying transport manager"));
 
     pj_lock_acquire(mgr->lock);
@@ -1277,7 +1525,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_destroy( pjsip_tpmgr *mgr )
     while (itr != NULL) {
 	pj_hash_iterator_t *next;
 	pjsip_transport *transport;
-
+	
 	transport = (pjsip_transport*) pj_hash_this(mgr->table, itr);
 
 	next = pj_hash_next(mgr->table, itr);
@@ -1293,19 +1541,13 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_destroy( pjsip_tpmgr *mgr )
     factory = mgr->factory_list.next;
     while (factory != &mgr->factory_list) {
 	pjsip_tpfactory *next = factory->next;
-
+	
 	factory->destroy(factory);
 
 	factory = next;
     }
 
     pj_lock_release(mgr->lock);
-    pj_lock_destroy(mgr->lock);
-
-    /* Unregister mod_msg_print. */
-    if (mod_msg_print.id != -1) {
-	pjsip_endpt_unregister_module(endpt, &mod_msg_print);
-    }
 
 #if defined(PJ_DEBUG) && PJ_DEBUG!=0
     /* If you encounter assert error on this line, it means there are
@@ -1319,6 +1561,30 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_destroy( pjsip_tpmgr *mgr )
     }
 #endif
 
+    /*
+     * Destroy any dangling transmit buffer.
+     */
+    if (!pj_list_empty(&mgr->tdata_list)) {
+	pjsip_tx_data *tdata = mgr->tdata_list.next;
+	while (tdata != &mgr->tdata_list) {
+	    pjsip_tx_data *next = tdata->next;
+	    tx_data_destroy(tdata);
+	    tdata = next;
+	}
+	PJ_LOG(3,(THIS_FILE, "Cleaned up dangling transmit buffer(s)."));
+    }
+
+#if defined(PJ_DEBUG) && PJ_DEBUG!=0
+    pj_atomic_destroy(mgr->tdata_counter);
+#endif
+
+    pj_lock_destroy(mgr->lock);
+
+    /* Unregister mod_msg_print. */
+    if (mod_msg_print.id != -1) {
+	pjsip_endpt_unregister_module(endpt, &mod_msg_print);
+    }
+
     return PJ_SUCCESS;
 }
 
@@ -1345,8 +1611,11 @@ PJ_DEF(pj_ssize_t) pjsip_tpmgr_receive_packet( pjsip_tpmgr *mgr,
     current_pkt = rdata->pkt_info.packet;
     remaining_len = rdata->pkt_info.len;
 
-    /* Must NULL terminate buffer. This is the requirement of the
-     * parser etc.
+    tr->last_recv_len = rdata->pkt_info.len;
+    pj_get_timestamp(&tr->last_recv_ts);
+    
+    /* Must NULL terminate buffer. This is the requirement of the 
+     * parser etc. 
      */
     current_pkt[remaining_len] = '\0';
 
@@ -1377,19 +1646,19 @@ PJ_DEF(pj_ssize_t) pjsip_tpmgr_receive_packet( pjsip_tpmgr *mgr,
 	/* Initialize default fragment size. */
 	msg_fragment_size = remaining_len;
 
-	/* Clear and init msg_info in rdata.
+	/* Clear and init msg_info in rdata. 
 	 * Endpoint might inspect the values there when we call the callback
 	 * to report some errors.
 	 */
 	pj_bzero(&rdata->msg_info, sizeof(rdata->msg_info));
 	pj_list_init(&rdata->msg_info.parse_err);
 	rdata->msg_info.msg_buf = current_pkt;
-	rdata->msg_info.len = remaining_len;
+	rdata->msg_info.len = (int)remaining_len;
 
 	/* For TCP transport, check if the whole message has been received. */
 	if ((tr->flag & PJSIP_TRANSPORT_DATAGRAM) == 0) {
 	    pj_status_t msg_status;
-	    msg_status = pjsip_find_msg(current_pkt, remaining_len, PJ_FALSE,
+	    msg_status = pjsip_find_msg(current_pkt, remaining_len, PJ_FALSE, 
                                         &msg_fragment_size);
 	    if (msg_status != PJ_SUCCESS) {
 		if (remaining_len == PJSIP_MAX_PKT_LEN) {
@@ -1404,14 +1673,14 @@ PJ_DEF(pj_ssize_t) pjsip_tpmgr_receive_packet( pjsip_tpmgr *mgr,
 	}
 
 	/* Update msg_info. */
-	rdata->msg_info.len = msg_fragment_size;
+	rdata->msg_info.len = (int)msg_fragment_size;
 
 	/* Null terminate packet */
 	saved = current_pkt[msg_fragment_size];
 	current_pkt[msg_fragment_size] = '\0';
 
 	/* Parse the message. */
-	rdata->msg_info.msg = msg =
+	rdata->msg_info.msg = msg = 
 	    pjsip_parse_rdata( current_pkt, msg_fragment_size, rdata);
 
 	/* Restore null termination */
@@ -1445,13 +1714,13 @@ PJ_DEF(pj_ssize_t) pjsip_tpmgr_receive_packet( pjsip_tpmgr *mgr,
 	     * which were sent to keep NAT bindings.
 	     */
 	    if (tmp.slen) {
-		PJ_LOG(1, (THIS_FILE,
+		PJ_LOG(1, (THIS_FILE, 
 		      "Error processing %d bytes packet from %s %s:%d %.*s:\n"
 		      "%.*s\n"
 		      "-- end of packet.",
 		      msg_fragment_size,
 		      rdata->tp_info.transport->type_name,
-		      rdata->pkt_info.src_name,
+		      rdata->pkt_info.src_name, 
 		      rdata->pkt_info.src_port,
 		      (int)tmp.slen, tmp.ptr,
 		      (int)msg_fragment_size,
@@ -1463,11 +1732,11 @@ PJ_DEF(pj_ssize_t) pjsip_tpmgr_receive_packet( pjsip_tpmgr *mgr,
 
 	/* Perform basic header checking. */
 	if (rdata->msg_info.cid == NULL ||
-	    rdata->msg_info.cid->id.slen == 0 ||
-	    rdata->msg_info.from == NULL ||
-	    rdata->msg_info.to == NULL ||
-	    rdata->msg_info.via == NULL ||
-	    rdata->msg_info.cseq == NULL)
+	    rdata->msg_info.cid->id.slen == 0 || 
+	    rdata->msg_info.from == NULL || 
+	    rdata->msg_info.to == NULL || 
+	    rdata->msg_info.via == NULL || 
+	    rdata->msg_info.cseq == NULL) 
 	{
 	    mgr->on_rx_msg(mgr->endpt, PJSIP_EMISSINGHDR, rdata);
 	    goto finish_process_fragment;
@@ -1476,8 +1745,8 @@ PJ_DEF(pj_ssize_t) pjsip_tpmgr_receive_packet( pjsip_tpmgr *mgr,
 	/* For request: */
 	if (rdata->msg_info.msg->type == PJSIP_REQUEST_MSG) {
 	    /* always add received parameter to the via. */
-	    pj_strdup2(rdata->tp_info.pool,
-		       &rdata->msg_info.via->recvd_param,
+	    pj_strdup2(rdata->tp_info.pool, 
+		       &rdata->msg_info.via->recvd_param, 
 		       rdata->pkt_info.src_name);
 
 	    /* RFC 3581:
@@ -1577,7 +1846,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_acquire_transport2(pjsip_tpmgr *mgr,
      * for the destination.
      */
     if (sel && sel->type == PJSIP_TPSELECTOR_TRANSPORT &&
-	sel->u.transport)
+	sel->u.transport) 
     {
 	pjsip_transport *seltp = sel->u.transport;
 
@@ -1655,7 +1924,7 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_acquire_transport2(pjsip_tpmgr *mgr,
 
 		pj_bzero(addr, addr_len);
 		key_len = sizeof(key.type) + addr_len;
-		transport = (pjsip_transport*)
+		transport = (pjsip_transport*) 
 			    pj_hash_get(mgr->table, &key, key_len, NULL);
 	    }
 	    /* For datagram transports, try lookup with zero address.
@@ -1709,15 +1978,15 @@ PJ_DEF(pj_status_t) pjsip_tpmgr_acquire_transport2(pjsip_tpmgr *mgr,
     /* Request factory to create transport. */
     if (factory->create_transport2) {
 	status = factory->create_transport2(factory, mgr, mgr->endpt,
-					    (const pj_sockaddr*) remote,
+					    (const pj_sockaddr*) remote, 
 					    addr_len, tdata, tp);
     } else {
 	status = factory->create_transport(factory, mgr, mgr->endpt,
-					   (const pj_sockaddr*) remote,
+					   (const pj_sockaddr*) remote, 
 					   addr_len, tp);
     }
     if (status == PJ_SUCCESS) {
-	PJ_ASSERT_ON_FAIL(tp!=NULL,
+	PJ_ASSERT_ON_FAIL(tp!=NULL, 
 	    {pj_lock_release(mgr->lock); return PJ_EBUG;});
 	pjsip_transport_add_ref(*tp);
     }
@@ -1745,7 +2014,7 @@ PJ_DEF(void) pjsip_tpmgr_dump_transports(pjsip_tpmgr *mgr)
     PJ_LOG(3, (THIS_FILE, " Dumping listeners:"));
     factory = mgr->factory_list.next;
     while (factory != &mgr->factory_list) {
-	PJ_LOG(3, (THIS_FILE, "  %s %s:%.*s:%d",
+	PJ_LOG(3, (THIS_FILE, "  %s %s:%.*s:%d", 
 		   factory->obj_name,
 		   factory->type_name,
 		   (int)factory->addr_name.host.slen,
@@ -1759,10 +2028,10 @@ PJ_DEF(void) pjsip_tpmgr_dump_transports(pjsip_tpmgr *mgr)
 	PJ_LOG(3, (THIS_FILE, " Dumping transports:"));
 
 	do {
-	    pjsip_transport *t = (pjsip_transport*)
-				 pj_hash_this(mgr->table, itr);
+	    pjsip_transport *t = (pjsip_transport*) 
+	    			 pj_hash_this(mgr->table, itr);
 
-	    PJ_LOG(3, (THIS_FILE, "  %s %s (refcnt=%d%s)",
+	    PJ_LOG(3, (THIS_FILE, "  %s %s (refcnt=%d%s)", 
 		       t->obj_name,
 		       t->info,
 		       pj_atomic_get(t->ref_cnt),
@@ -1898,7 +2167,7 @@ PJ_DEF(pj_status_t) pjsip_transport_add_state_listener (
 }
 
 /**
- * Remove a listener from the specified transport for transport state
+ * Remove a listener from the specified transport for transport state 
  * notification.
  */
 PJ_DEF(pj_status_t) pjsip_transport_remove_state_listener (
