@@ -1,5 +1,5 @@
-/* $Id: sip_transport_tls.c 4146 2012-05-30 06:35:59Z nanang $ */
-/*
+/* $Id: sip_transport_tls.c 4726 2014-02-04 04:56:50Z ming $ */
+/* 
  * Copyright (C) 2009-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 
 #include <pjsip/sip_transport_tls.h>
@@ -55,6 +55,7 @@ struct tls_listener
     pjsip_endpoint	    *endpt;
     pjsip_tpmgr		    *tpmgr;
     pj_ssl_sock_t	    *ssock;
+    pj_sockaddr		     bound_addr;
     pj_ssl_cert_t	    *cert;
     pjsip_tls_setting	     tls_setting;
 };
@@ -71,6 +72,7 @@ struct delayed_tdata
 {
     PJ_DECL_LIST_MEMBER(struct delayed_tdata);
     pjsip_tx_data_op_key    *tdata_op_key;
+    pj_time_val              timeout;
 };
 
 
@@ -144,10 +146,10 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
 /* Common function to create and initialize transport */
 static pj_status_t tls_create(struct tls_listener *listener,
 			      pj_pool_t *pool,
-			      pj_ssl_sock_t *ssock,
+			      pj_ssl_sock_t *ssock, 
 			      pj_bool_t is_server,
-			      const pj_sockaddr_in *local,
-			      const pj_sockaddr_in *remote,
+			      const pj_sockaddr *local,
+			      const pj_sockaddr *remote,
 			      const pj_str_t *remote_name,
 			      struct tls_transport **p_tls);
 
@@ -165,10 +167,10 @@ static void tls_perror(const char *sender, const char *title,
 
 static void sockaddr_to_host_port( pj_pool_t *pool,
 				   pjsip_host_port *host_port,
-				   const pj_sockaddr_in *addr )
+				   const pj_sockaddr *addr )
 {
     host_port->host.ptr = (char*) pj_pool_alloc(pool, PJ_INET6_ADDRSTRLEN+4);
-    pj_sockaddr_print(addr, host_port->host.ptr, PJ_INET6_ADDRSTRLEN+4, 2);
+    pj_sockaddr_print(addr, host_port->host.ptr, PJ_INET6_ADDRSTRLEN+4, 0);
     host_port->host.slen = pj_ansi_strlen(host_port->host.ptr);
     host_port->port = pj_sockaddr_get_port(addr);
 }
@@ -181,7 +183,7 @@ static void tls_init_shutdown(struct tls_transport *tls, pj_status_t status)
     if (tls->close_reason == PJ_SUCCESS)
 	tls->close_reason = status;
 
-    if (tls->base.is_shutdown)
+    if (tls->base.is_shutdown || tls->base.is_destroying)
 	return;
 
     /* Prevent immediate transport destroy by application, as transport
@@ -196,12 +198,12 @@ static void tls_init_shutdown(struct tls_transport *tls, pj_status_t status)
 	pjsip_transport_state_info state_info;
 	pjsip_tls_state_info tls_info;
 	pj_ssl_sock_info ssl_info;
-
+	
 	/* Init transport state info */
 	pj_bzero(&state_info, sizeof(state_info));
 	state_info.status = tls->close_reason;
 
-	if (tls->ssock &&
+	if (tls->ssock && 
 	    pj_ssl_sock_get_info(tls->ssock, &ssl_info) == PJ_SUCCESS)
 	{
 	    pj_bzero(&tls_info, sizeof(tls_info));
@@ -212,8 +214,14 @@ static void tls_init_shutdown(struct tls_transport *tls, pj_status_t status)
 	(*state_cb)(&tls->base, PJSIP_TP_STATE_DISCONNECTED, &state_info);
     }
 
+    /* check again */
+    if (tls->base.is_shutdown || tls->base.is_destroying) {
+        pjsip_transport_dec_ref(&tls->base);
+	return;
+    }
+
     /* We can not destroy the transport since high level objects may
-     * still keep reference to this transport. So we can only
+     * still keep reference to this transport. So we can only 
      * instruct transport manager to gracefully start the shutdown
      * procedure for this transport.
      */
@@ -234,63 +242,91 @@ static void tls_init_shutdown(struct tls_transport *tls, pj_status_t status)
  */
 PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
 					       const pjsip_tls_setting *opt,
-					       const pj_sockaddr_in *local,
+					       const pj_sockaddr_in *local_in,
 					       const pjsip_host_port *a_name,
 					       unsigned async_cnt,
 					       pjsip_tpfactory **p_factory)
+{
+    pj_sockaddr local;
+
+    if (local_in)
+	pj_sockaddr_cp(&local, local_in);
+
+    return pjsip_tls_transport_start2(endpt, opt, (local_in? &local : NULL),
+                                      a_name, async_cnt, p_factory);
+}
+
+PJ_DEF(pj_status_t) pjsip_tls_transport_start2( pjsip_endpoint *endpt,
+ 					        const pjsip_tls_setting *opt,
+					        const pj_sockaddr *local,
+					        const pjsip_host_port *a_name,
+					        unsigned async_cnt,
+					        pjsip_tpfactory **p_factory)
 {
     pj_pool_t *pool;
+    pj_bool_t is_ipv6;
+    int af;
     struct tls_listener *listener;
     pj_ssl_sock_param ssock_param;
-    pj_sockaddr_in *listener_addr;
+    pj_sockaddr *listener_addr;
     pj_bool_t has_listener;
     pj_status_t status;
 
     /* Sanity check */
     PJ_ASSERT_RETURN(endpt && async_cnt, PJ_EINVAL);
 
+    is_ipv6 = (local && local->addr.sa_family == pj_AF_INET6());
+    af = is_ipv6 ? pj_AF_INET6() : pj_AF_INET();
+
     /* Verify that address given in a_name (if any) is valid */
     if (a_name && a_name->host.slen) {
-	pj_sockaddr_in tmp;
+	pj_sockaddr tmp;
 
-	status = pj_sockaddr_in_init(&tmp, &a_name->host,
-				     (pj_uint16_t)a_name->port);
-	if (status != PJ_SUCCESS || tmp.sin_addr.s_addr == PJ_INADDR_ANY ||
-	    tmp.sin_addr.s_addr == PJ_INADDR_NONE)
+	status = pj_sockaddr_init(af, &tmp, &a_name->host,
+				  (pj_uint16_t)a_name->port);
+	if (status != PJ_SUCCESS || !pj_sockaddr_has_addr(&tmp) ||
+	    (!is_ipv6 && tmp.ipv4.sin_addr.s_addr == PJ_INADDR_NONE))
 	{
 	    /* Invalid address */
 	    return PJ_EINVAL;
 	}
     }
 
-    pool = pjsip_endpt_create_pool(endpt, "tlslis", POOL_LIS_INIT,
+    pool = pjsip_endpt_create_pool(endpt, "tlslis", POOL_LIS_INIT, 
 				   POOL_LIS_INC);
     PJ_ASSERT_RETURN(pool, PJ_ENOMEM);
 
     listener = PJ_POOL_ZALLOC_T(pool, struct tls_listener);
     listener->factory.pool = pool;
-    listener->factory.type = PJSIP_TRANSPORT_TLS;
-    listener->factory.type_name = "tls";
-    listener->factory.flag =
-	pjsip_transport_get_flag_from_type(PJSIP_TRANSPORT_TLS);
+    if (is_ipv6)
+	listener->factory.type = PJSIP_TRANSPORT_TLS6;
+    else
+	listener->factory.type = PJSIP_TRANSPORT_TLS;
+    listener->factory.type_name = (char*)
+		pjsip_transport_get_type_name(listener->factory.type);
+    listener->factory.flag = 
+	pjsip_transport_get_flag_from_type(listener->factory.type);
 
     pj_ansi_strcpy(listener->factory.obj_name, "tlslis");
+    if (is_ipv6)
+	pj_ansi_strcat(listener->factory.obj_name, "6");
 
     if (opt)
 	pjsip_tls_setting_copy(pool, &listener->tls_setting, opt);
     else
 	pjsip_tls_setting_default(&listener->tls_setting);
 
-    status = pj_lock_create_recursive_mutex(pool, "tlslis",
+    status = pj_lock_create_recursive_mutex(pool, listener->factory.obj_name,
 					    &listener->factory.lock);
     if (status != PJ_SUCCESS)
 	goto on_error;
 
-    if (async_cnt > MAX_ASYNC_CNT)
+    if (async_cnt > MAX_ASYNC_CNT) 
 	async_cnt = MAX_ASYNC_CNT;
 
     /* Build SSL socket param */
     pj_ssl_sock_param_default(&ssock_param);
+    ssock_param.sock_af = af;
     ssock_param.cb.on_accept_complete = &on_accept_complete;
     ssock_param.cb.on_data_read = &on_data_read;
     ssock_param.cb.on_data_sent = &on_data_sent;
@@ -307,6 +343,7 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
 	ssock_param.read_buffer_size = PJSIP_MAX_PKT_LEN;
     ssock_param.ciphers_num = listener->tls_setting.ciphers_num;
     ssock_param.ciphers = listener->tls_setting.ciphers;
+    ssock_param.reuse_addr = listener->tls_setting.reuse_addr;
     ssock_param.qos_type = listener->tls_setting.qos_type;
     ssock_param.qos_ignore_error = listener->tls_setting.qos_ignore_error;
     pj_memcpy(&ssock_param.qos_params, &listener->tls_setting.qos_params,
@@ -337,17 +374,22 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
     if (status != PJ_SUCCESS)
 	goto on_error;
 
-    listener_addr = (pj_sockaddr_in*)&listener->factory.local_addr;
+    /* Bind address may be different than factory.local_addr because
+     * factory.local_addr will be resolved below.
+     */
+    listener_addr = &listener->factory.local_addr;
     if (local) {
-	pj_sockaddr_cp((pj_sockaddr_t*)listener_addr,
+	pj_sockaddr_cp((pj_sockaddr_t*)listener_addr, 
 		       (const pj_sockaddr_t*)local);
+	pj_sockaddr_cp(&listener->bound_addr, local);
     } else {
-	pj_sockaddr_in_init(listener_addr, NULL, 0);
+	pj_sockaddr_init(af, listener_addr, NULL, 0);
+	pj_sockaddr_init(af, &listener->bound_addr, NULL, 0);
     }
 
     /* Check if certificate/CA list for SSL socket is set */
     if (listener->tls_setting.cert_file.slen ||
-	listener->tls_setting.ca_list_file.slen)
+	listener->tls_setting.ca_list_file.slen) 
     {
 	status = pj_ssl_cert_load_from_files(pool,
 			&listener->tls_setting.ca_list_file,
@@ -358,7 +400,7 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
 	if (status != PJ_SUCCESS)
 	    goto on_error;
 
-	status = pj_ssl_sock_set_certificate(listener->ssock, pool,
+	status = pj_ssl_sock_set_certificate(listener->ssock, pool, 
 					     listener->cert);
 	if (status != PJ_SUCCESS)
 	    goto on_error;
@@ -369,8 +411,8 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
      */
     has_listener = PJ_FALSE;
 
-    status = pj_ssl_sock_start_accept(listener->ssock, pool,
-			  (pj_sockaddr_t*)listener_addr,
+    status = pj_ssl_sock_start_accept(listener->ssock, pool, 
+			  (pj_sockaddr_t*)listener_addr, 
 			  pj_sockaddr_get_len((pj_sockaddr_t*)listener_addr));
     if (status == PJ_SUCCESS || status == PJ_EPENDING) {
 	pj_ssl_sock_info info;
@@ -390,7 +432,7 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
     if (a_name && a_name->host.slen) {
 	/* Copy the address */
 	listener->factory.addr_name = *a_name;
-	pj_strdup(listener->factory.pool, &listener->factory.addr_name.host,
+	pj_strdup(listener->factory.pool, &listener->factory.addr_name.host, 
 		  &a_name->host);
 	listener->factory.addr_name.port = a_name->port;
 
@@ -400,27 +442,27 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
 	/* If the address returns 0.0.0.0, use the default
 	 * interface address as the transport's address.
 	 */
-	if (listener_addr->sin_addr.s_addr == 0) {
+	if (!pj_sockaddr_has_addr(listener_addr)) {
 	    pj_sockaddr hostip;
 
-	    status = pj_gethostip(pj_AF_INET(), &hostip);
+	    status = pj_gethostip(af, &hostip);
 	    if (status != PJ_SUCCESS)
 		goto on_error;
 
-	    listener_addr->sin_addr.s_addr = hostip.ipv4.sin_addr.s_addr;
+	    pj_sockaddr_copy_addr(listener_addr, &hostip);
 	}
 
 	/* Save the address name */
-	sockaddr_to_host_port(listener->factory.pool,
+	sockaddr_to_host_port(listener->factory.pool, 
 			      &listener->factory.addr_name, listener_addr);
     }
 
     /* If port is zero, get the bound port */
     if (listener->factory.addr_name.port == 0) {
-	listener->factory.addr_name.port = pj_ntohs(listener_addr->sin_port);
+	listener->factory.addr_name.port = pj_sockaddr_get_port(listener_addr);
     }
 
-    pj_ansi_snprintf(listener->factory.obj_name,
+    pj_ansi_snprintf(listener->factory.obj_name, 
 		     sizeof(listener->factory.obj_name),
 		     "tlslis:%d",  listener->factory.addr_name.port);
 
@@ -438,7 +480,7 @@ PJ_DEF(pj_status_t) pjsip_tls_transport_start (pjsip_endpoint *endpt,
     }
 
     if (has_listener) {
-	PJ_LOG(4,(listener->factory.obj_name,
+	PJ_LOG(4,(listener->factory.obj_name, 
 		 "SIP TLS listener is ready for incoming connections "
 		 "at %.*s:%d",
 		 (int)listener->factory.addr_name.host.slen,
@@ -502,7 +544,7 @@ static pj_status_t lis_destroy(pjsip_tpfactory *factory)
  * Prototypes.
  */
 /* Called by transport manager to send message */
-static pj_status_t tls_send_msg(pjsip_transport *transport,
+static pj_status_t tls_send_msg(pjsip_transport *transport, 
 				pjsip_tx_data *tdata,
 				const pj_sockaddr_t *rem_addr,
 				int addr_len,
@@ -534,15 +576,16 @@ static pj_status_t tls_create( struct tls_listener *listener,
 			       pj_pool_t *pool,
 			       pj_ssl_sock_t *ssock,
 			       pj_bool_t is_server,
-			       const pj_sockaddr_in *local,
-			       const pj_sockaddr_in *remote,
+			       const pj_sockaddr *local,
+			       const pj_sockaddr *remote,
 			       const pj_str_t *remote_name,
 			       struct tls_transport **p_tls)
 {
     struct tls_transport *tls;
     const pj_str_t ka_pkt = PJSIP_TLS_KEEP_ALIVE_DATA;
+    char print_addr[PJ_INET6_ADDRSTRLEN+10];
     pj_status_t status;
-
+    
 
     PJ_ASSERT_RETURN(listener && ssock && local && remote && p_tls, PJ_EINVAL);
 
@@ -551,7 +594,7 @@ static pj_status_t tls_create( struct tls_listener *listener,
 	pool = pjsip_endpt_create_pool(listener->endpt, "tls",
 				       POOL_TP_INIT, POOL_TP_INC);
 	PJ_ASSERT_RETURN(pool != NULL, PJ_ENOMEM);
-    }
+    }    
 
     /*
      * Create and initialize basic transport structure.
@@ -562,7 +605,7 @@ static pj_status_t tls_create( struct tls_listener *listener,
     pj_list_init(&tls->delayed_list);
     tls->base.pool = pool;
 
-    pj_ansi_snprintf(tls->base.obj_name, PJ_MAX_OBJ_NAME,
+    pj_ansi_snprintf(tls->base.obj_name, PJ_MAX_OBJ_NAME, 
 		     (is_server ? "tlss%p" :"tlsc%p"), tls);
 
     status = pj_atomic_create(pool, 0, &tls->base.ref_cnt);
@@ -578,19 +621,23 @@ static pj_status_t tls_create( struct tls_listener *listener,
     if (remote_name)
 	pj_strdup(pool, &tls->remote_name, remote_name);
 
-    tls->base.key.type = PJSIP_TRANSPORT_TLS;
-    pj_memcpy(&tls->base.key.rem_addr, remote, sizeof(pj_sockaddr_in));
-    tls->base.type_name = "tls";
-    tls->base.flag = pjsip_transport_get_flag_from_type(PJSIP_TRANSPORT_TLS);
+    tls->base.key.type = listener->factory.type;
+    pj_sockaddr_cp(&tls->base.key.rem_addr, remote);
+    tls->base.type_name = (char*)pjsip_transport_get_type_name(
+				   (pjsip_transport_type_e)tls->base.key.type);
+    tls->base.flag = pjsip_transport_get_flag_from_type(
+				   (pjsip_transport_type_e)tls->base.key.type);
 
     tls->base.info = (char*) pj_pool_alloc(pool, 64);
-    pj_ansi_snprintf(tls->base.info, 64, "TLS to %s:%d",
-		     pj_inet_ntoa(remote->sin_addr),
-		     (int)pj_ntohs(remote->sin_port));
+    pj_ansi_snprintf(tls->base.info, 64, "%s to %s",
+                     tls->base.type_name,
+                     pj_sockaddr_print(remote, print_addr,
+                                       sizeof(print_addr), 3));
 
-    tls->base.addr_len = sizeof(pj_sockaddr_in);
+
+    tls->base.addr_len = pj_sockaddr_get_len(remote);
     tls->base.dir = is_server? PJSIP_TP_DIR_INCOMING : PJSIP_TP_DIR_OUTGOING;
-
+    
     /* Set initial local address */
     if (!pj_sockaddr_has_addr(local)) {
         pj_sockaddr_cp(&tls->base.local_addr,
@@ -598,12 +645,11 @@ static pj_status_t tls_create( struct tls_listener *listener,
     } else {
 	pj_sockaddr_cp(&tls->base.local_addr, local);
     }
-
-    sockaddr_to_host_port(pool, &tls->base.local_name,
-			  (pj_sockaddr_in*)&tls->base.local_addr);
+    
+    sockaddr_to_host_port(pool, &tls->base.local_name, &tls->base.local_addr);
     if (tls->remote_name.slen) {
 	tls->base.remote_name.host = tls->remote_name;
-	tls->base.remote_name.port = pj_sockaddr_in_get_port(remote);
+	tls->base.remote_name.port = pj_sockaddr_get_port(remote);
     } else {
 	sockaddr_to_host_port(pool, &tls->base.remote_name, remote);
     }
@@ -629,7 +675,7 @@ static pj_status_t tls_create( struct tls_listener *listener,
     tls->ka_timer.cb = &tls_keep_alive_timer;
     pj_ioqueue_op_key_init(&tls->ka_op_key.key, sizeof(pj_ioqueue_op_key_t));
     pj_strdup(tls->base.pool, &tls->ka_pkt, &ka_pkt);
-
+    
     /* Done setting up basic transport. */
     *p_tls = tls;
 
@@ -647,6 +693,9 @@ on_error:
 /* Flush all delayed transmision once the socket is connected. */
 static void tls_flush_pending_tx(struct tls_transport *tls)
 {
+    pj_time_val now;
+
+    pj_gettickcount(&now);
     pj_lock_acquire(tls->base.lock);
     while (!pj_list_empty(&tls->delayed_list)) {
 	struct delayed_tdata *pending_tx;
@@ -661,13 +710,21 @@ static void tls_flush_pending_tx(struct tls_transport *tls)
 	tdata = pending_tx->tdata_op_key->tdata;
 	op_key = (pj_ioqueue_op_key_t*)pending_tx->tdata_op_key;
 
+        if (pending_tx->timeout.sec > 0 &&
+            PJ_TIME_VAL_GT(now, pending_tx->timeout))
+        {
+            continue;
+        }
+
 	/* send! */
 	size = tdata->buf.cur - tdata->buf.start;
-	status = pj_ssl_sock_send(tls->ssock, op_key, tdata->buf.start,
+	status = pj_ssl_sock_send(tls->ssock, op_key, tdata->buf.start, 
 				  &size, 0);
 
 	if (status != PJ_EPENDING) {
+            pj_lock_release(tls->base.lock);
 	    on_data_sent(tls->ssock, op_key, size);
+            pj_lock_acquire(tls->base.lock);
 	}
     }
     pj_lock_release(tls->base.lock);
@@ -689,7 +746,7 @@ static pj_status_t tls_destroy_transport(pjsip_transport *transport)
 
 
 /* Destroy TLS transport */
-static pj_status_t tls_destroy(pjsip_transport *transport,
+static pj_status_t tls_destroy(pjsip_transport *transport, 
 			       pj_status_t reason)
 {
     struct tls_transport *tls = (struct tls_transport*)transport;
@@ -755,13 +812,13 @@ static pj_status_t tls_destroy(pjsip_transport *transport,
 	    char errmsg[PJ_ERR_MSG_SIZE];
 
 	    pj_strerror(reason, errmsg, sizeof(errmsg));
-	    PJ_LOG(4,(tls->base.obj_name,
-		      "TLS transport destroyed with reason %d: %s",
+	    PJ_LOG(4,(tls->base.obj_name, 
+		      "TLS transport destroyed with reason %d: %s", 
 		      reason, errmsg));
 
 	} else {
 
-	    PJ_LOG(4,(tls->base.obj_name,
+	    PJ_LOG(4,(tls->base.obj_name, 
 		      "TLS transport destroyed normally"));
 
 	}
@@ -783,8 +840,8 @@ static pj_status_t tls_destroy(pjsip_transport *transport,
 static pj_status_t tls_start_read(struct tls_transport *tls)
 {
     pj_pool_t *pool;
-    pj_ssize_t size;
-    pj_sockaddr_in *rem_addr;
+    pj_uint32_t size;
+    pj_sockaddr *rem_addr;
     void *readbuf[1];
     pj_status_t status;
 
@@ -803,23 +860,23 @@ static pj_status_t tls_start_read(struct tls_transport *tls)
     tls->rdata.tp_info.transport = &tls->base;
     tls->rdata.tp_info.tp_data = tls;
     tls->rdata.tp_info.op_key.rdata = &tls->rdata;
-    pj_ioqueue_op_key_init(&tls->rdata.tp_info.op_key.op_key,
+    pj_ioqueue_op_key_init(&tls->rdata.tp_info.op_key.op_key, 
 			   sizeof(pj_ioqueue_op_key_t));
 
     tls->rdata.pkt_info.src_addr = tls->base.key.rem_addr;
-    tls->rdata.pkt_info.src_addr_len = sizeof(pj_sockaddr_in);
-    rem_addr = (pj_sockaddr_in*) &tls->base.key.rem_addr;
-    pj_ansi_strcpy(tls->rdata.pkt_info.src_name,
-		   pj_inet_ntoa(rem_addr->sin_addr));
-    tls->rdata.pkt_info.src_port = pj_ntohs(rem_addr->sin_port);
+    tls->rdata.pkt_info.src_addr_len = sizeof(tls->rdata.pkt_info.src_addr);
+    rem_addr = &tls->base.key.rem_addr;
+    pj_sockaddr_print(rem_addr, tls->rdata.pkt_info.src_name,
+                          sizeof(tls->rdata.pkt_info.src_name), 0);
+    tls->rdata.pkt_info.src_port = pj_sockaddr_get_port(rem_addr);
 
     size = sizeof(tls->rdata.pkt_info.packet);
     readbuf[0] = tls->rdata.pkt_info.packet;
     status = pj_ssl_sock_start_read2(tls->ssock, tls->base.pool, size,
 				     readbuf, 0);
     if (status != PJ_SUCCESS && status != PJ_EPENDING) {
-	PJ_LOG(4, (tls->base.obj_name,
-		   "pj_ssl_sock_start_read() error, status=%d",
+	PJ_LOG(4, (tls->base.obj_name, 
+		   "pj_ssl_sock_start_read() error, status=%d", 
 		   status));
 	return status;
     }
@@ -844,7 +901,7 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
     pj_pool_t *pool;
     pj_ssl_sock_t *ssock;
     pj_ssl_sock_param ssock_param;
-    pj_sockaddr_in local_addr;
+    pj_sockaddr local_addr;
     pj_str_t remote_name;
     pj_status_t status;
 
@@ -852,9 +909,11 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
     PJ_ASSERT_RETURN(factory && mgr && endpt && rem_addr &&
 		     addr_len && p_transport, PJ_EINVAL);
 
-    /* Check that address is a sockaddr_in */
-    PJ_ASSERT_RETURN(rem_addr->addr.sa_family == pj_AF_INET() &&
-		     addr_len == sizeof(pj_sockaddr_in), PJ_EINVAL);
+    /* Check that address is a sockaddr_in or sockaddr_in6*/
+    PJ_ASSERT_RETURN((rem_addr->addr.sa_family == pj_AF_INET() &&
+		      addr_len == sizeof(pj_sockaddr_in)) ||
+		     (rem_addr->addr.sa_family == pj_AF_INET6() &&
+		      addr_len == sizeof(pj_sockaddr_in6)), PJ_EINVAL);
 
 
     listener = (struct tls_listener*)factory;
@@ -871,6 +930,8 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
 
     /* Build SSL socket param */
     pj_ssl_sock_param_default(&ssock_param);
+    ssock_param.sock_af = (factory->type & PJSIP_TRANSPORT_IPV6) ?
+			    pj_AF_INET6() : pj_AF_INET();
     ssock_param.cb.on_connect_complete = &on_connect_complete;
     ssock_param.cb.on_data_read = &on_data_read;
     ssock_param.cb.on_data_sent = &on_data_sent;
@@ -921,12 +982,14 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
 	    return status;
     }
 
-    /* Initially set bind address to PJ_INADDR_ANY port 0 */
-    pj_sockaddr_in_init(&local_addr, NULL, 0);
+    /* Initially set bind address to listener's bind address */
+    pj_sockaddr_init(listener->bound_addr.addr.sa_family,
+		     &local_addr, NULL, 0);
+    pj_sockaddr_copy_addr(&local_addr, &listener->bound_addr);
 
     /* Create the transport descriptor */
-    status = tls_create(listener, pool, ssock, PJ_FALSE, &local_addr,
-			(pj_sockaddr_in*)rem_addr, &remote_name, &tls);
+    status = tls_create(listener, pool, ssock, PJ_FALSE, &local_addr, 
+			rem_addr, &remote_name, &tls);
     if (status != PJ_SUCCESS)
 	return status;
 
@@ -935,7 +998,7 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
 
     /* Start asynchronous connect() operation */
     tls->has_pending_connect = PJ_TRUE;
-    status = pj_ssl_sock_start_connect(tls->ssock, tls->base.pool,
+    status = pj_ssl_sock_start_connect(tls->ssock, tls->base.pool, 
 				       (pj_sockaddr_t*)&local_addr,
 				       (pj_sockaddr_t*)rem_addr,
 				       addr_len);
@@ -949,7 +1012,7 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
     if (tls->has_pending_connect) {
 	pj_ssl_sock_info info;
 
-	/* Update local address, just in case local address currently set is
+	/* Update local address, just in case local address currently set is 
 	 * different now that asynchronous connect() is started.
 	 */
 
@@ -968,15 +1031,15 @@ static pj_status_t lis_create_transport(pjsip_tpfactory *factory,
 					(pj_sockaddr_t*)&tls->base.local_addr))
 	    {
 		/* Update port only */
-		pj_sockaddr_set_port(&tls->base.local_addr,
+		pj_sockaddr_set_port(&tls->base.local_addr, 
 				     new_port);
 	    }
 
 	    sockaddr_to_host_port(tls->base.pool, &tls->base.local_name,
-				  (pj_sockaddr_in*)&tls->base.local_addr);
+				  &tls->base.local_addr);
 	}
 
-	PJ_LOG(4,(tls->base.obj_name,
+	PJ_LOG(4,(tls->base.obj_name, 
 		  "TLS transport %.*s:%d is connecting to %.*s:%d...",
 		  (int)tls->base.local_name.host.slen,
 		  tls->base.local_name.host.ptr,
@@ -1007,6 +1070,7 @@ static pj_bool_t on_accept_complete(pj_ssl_sock_t *ssock,
     pj_ssl_sock_info ssl_info;
     char addr[PJ_INET6_ADDRSTRLEN+10];
     pjsip_tp_state_callback state_cb;
+    pj_sockaddr tmp_src_addr;
     pj_bool_t is_shutdown;
     pj_status_t status;
 
@@ -1016,7 +1080,7 @@ static pj_bool_t on_accept_complete(pj_ssl_sock_t *ssock,
 
     PJ_ASSERT_RETURN(new_ssock, PJ_TRUE);
 
-    PJ_LOG(4,(listener->factory.obj_name,
+    PJ_LOG(4,(listener->factory.obj_name, 
 	      "TLS listener %.*s:%d: got incoming TLS connection "
 	      "from %s, sock=%d",
 	      (int)listener->factory.addr_name.host.slen,
@@ -1034,21 +1098,25 @@ static pj_bool_t on_accept_complete(pj_ssl_sock_t *ssock,
 	return PJ_TRUE;
     }
 
-    /*
+    /* Copy to larger buffer, just in case */
+    pj_bzero(&tmp_src_addr, sizeof(tmp_src_addr));
+    pj_sockaddr_cp(&tmp_src_addr, src_addr);
+
+    /* 
      * Incoming connection!
      * Create TLS transport for the new socket.
      */
     status = tls_create( listener, NULL, new_ssock, PJ_TRUE,
-			 (const pj_sockaddr_in*)&listener->factory.local_addr,
-			 (const pj_sockaddr_in*)src_addr, NULL, &tls);
-
+			 &listener->factory.local_addr,
+			 &tmp_src_addr, NULL, &tls);
+    
     if (status != PJ_SUCCESS)
 	return PJ_TRUE;
 
     /* Set the "pending" SSL socket user data */
     pj_ssl_sock_set_user_data(new_ssock, tls);
 
-    /* Prevent immediate transport destroy as application may access it
+    /* Prevent immediate transport destroy as application may access it 
      * (getting info, etc) in transport state notification callback.
      */
     pjsip_transport_add_ref(&tls->base);
@@ -1057,7 +1125,7 @@ static pj_bool_t on_accept_complete(pj_ssl_sock_t *ssock,
      * and destroy the transport.
      */
     if (ssl_info.verify_status && listener->tls_setting.verify_client) {
-	if (tls->close_reason == PJ_SUCCESS)
+	if (tls->close_reason == PJ_SUCCESS) 
 	    tls->close_reason = PJSIP_TLS_ECERTVERIF;
 	pjsip_transport_shutdown(&tls->base);
     }
@@ -1106,8 +1174,8 @@ static pj_bool_t on_accept_complete(pj_ssl_sock_t *ssock,
 	/* Start keep-alive timer */
 	if (PJSIP_TLS_KEEP_ALIVE_INTERVAL) {
 	    pj_time_val delay = {PJSIP_TLS_KEEP_ALIVE_INTERVAL, 0};
-	    pjsip_endpt_schedule_timer(listener->endpt,
-				       &tls->ka_timer,
+	    pjsip_endpt_schedule_timer(listener->endpt, 
+				       &tls->ka_timer, 
 				       &delay);
 	    tls->ka_timer.id = PJ_TRUE;
 	    pj_gettimeofday(&tls->last_activity);
@@ -1118,15 +1186,15 @@ static pj_bool_t on_accept_complete(pj_ssl_sock_t *ssock,
 }
 
 
-/*
+/* 
  * Callback from ioqueue when packet is sent.
  */
 static pj_bool_t on_data_sent(pj_ssl_sock_t *ssock,
 			      pj_ioqueue_op_key_t *op_key,
 			      pj_ssize_t bytes_sent)
 {
-    struct tls_transport *tls = (struct tls_transport*)
-				pj_ssl_sock_get_user_data(ssock);
+    struct tls_transport *tls = (struct tls_transport*) 
+    				pj_ssl_sock_get_user_data(ssock);
     pjsip_tx_data_op_key *tdata_op_key = (pjsip_tx_data_op_key*)op_key;
 
     /* Note that op_key may be the op_key from keep-alive, thus
@@ -1153,25 +1221,25 @@ static pj_bool_t on_data_sent(pj_ssl_sock_t *ssock,
     if (bytes_sent <= 0) {
 	pj_status_t status;
 
-	PJ_LOG(5,(tls->base.obj_name, "TLS send() error, sent=%d",
+	PJ_LOG(5,(tls->base.obj_name, "TLS send() error, sent=%d", 
 		  bytes_sent));
 
 	status = (bytes_sent == 0) ? PJ_RETURN_OS_ERROR(OSERR_ENOTCONN) :
-				     -bytes_sent;
+				     (pj_status_t)-bytes_sent;
 
 	tls_init_shutdown(tls, status);
 
 	return PJ_FALSE;
     }
-
+    
     return PJ_TRUE;
 }
 
 
-/*
- * This callback is called by transport manager to send SIP message
+/* 
+ * This callback is called by transport manager to send SIP message 
  */
-static pj_status_t tls_send_msg(pjsip_transport *transport,
+static pj_status_t tls_send_msg(pjsip_transport *transport, 
 				pjsip_tx_data *tdata,
 				const pj_sockaddr_t *rem_addr,
 				int addr_len,
@@ -1188,11 +1256,11 @@ static pj_status_t tls_send_msg(pjsip_transport *transport,
 
     /* Check that there's no pending operation associated with the tdata */
     PJ_ASSERT_RETURN(tdata->op_key.tdata == NULL, PJSIP_EPENDINGTX);
-
+    
     /* Check the address is supported */
-    PJ_ASSERT_RETURN(rem_addr && addr_len==sizeof(pj_sockaddr_in), PJ_EINVAL);
-
-
+    PJ_ASSERT_RETURN(rem_addr && (addr_len==sizeof(pj_sockaddr_in) ||
+	                          addr_len==sizeof(pj_sockaddr_in6)),
+	             PJ_EINVAL);
 
     /* Init op key. */
     tdata->op_key.tdata = tdata;
@@ -1217,10 +1285,19 @@ static pj_status_t tls_send_msg(pjsip_transport *transport,
 	    /*
 	     * connect() is still in progress. Put the transmit data to
 	     * the delayed list.
+             * Starting from #1583 (https://trac.pjsip.org/repos/ticket/1583),
+             * we also add timeout value for the transmit data. When the
+             * connect() is completed, the timeout value will be checked to
+             * determine whether the transmit data needs to be sent.
 	     */
-	    delayed_tdata = PJ_POOL_ALLOC_T(tdata->pool,
-					    struct delayed_tdata);
+	    delayed_tdata = PJ_POOL_ZALLOC_T(tdata->pool, 
+					     struct delayed_tdata);
 	    delayed_tdata->tdata_op_key = &tdata->op_key;
+            if (tdata->msg && tdata->msg->type == PJSIP_REQUEST_MSG) {
+                pj_gettickcount(&delayed_tdata->timeout);
+                delayed_tdata->timeout.msec += pjsip_cfg()->tsx.td;
+                pj_time_val_normalize(&delayed_tdata->timeout);
+            }
 
 	    pj_list_push_back(&tls->delayed_list, delayed_tdata);
 	    status = PJ_EPENDING;
@@ -1230,15 +1307,15 @@ static pj_status_t tls_send_msg(pjsip_transport *transport,
 	}
 
 	pj_lock_release(tls->base.lock);
-    }
-
+    } 
+    
     if (!delayed) {
 	/*
 	 * Transport is ready to go. Send the packet to ioqueue to be
 	 * sent asynchronously.
 	 */
 	size = tdata->buf.cur - tdata->buf.start;
-	status = pj_ssl_sock_send(tls->ssock,
+	status = pj_ssl_sock_send(tls->ssock, 
 				    (pj_ioqueue_op_key_t*)&tdata->op_key,
 				    tdata->buf.start, &size, 0);
 
@@ -1249,10 +1326,10 @@ static pj_status_t tls_send_msg(pjsip_transport *transport,
 	    /* Shutdown transport on closure/errors */
 	    if (size <= 0) {
 
-		PJ_LOG(5,(tls->base.obj_name, "TLS send() error, sent=%d",
+		PJ_LOG(5,(tls->base.obj_name, "TLS send() error, sent=%d", 
 			  size));
 
-		if (status == PJ_SUCCESS)
+		if (status == PJ_SUCCESS) 
 		    status = PJ_RETURN_OS_ERROR(OSERR_ENOTCONN);
 
 		tls_init_shutdown(tls, status);
@@ -1264,13 +1341,13 @@ static pj_status_t tls_send_msg(pjsip_transport *transport,
 }
 
 
-/*
+/* 
  * This callback is called by transport manager to shutdown transport.
  */
 static pj_status_t tls_shutdown(pjsip_transport *transport)
 {
     struct tls_transport *tls = (struct tls_transport*)transport;
-
+    
     /* Stop keep-alive timer. */
     if (tls->ka_timer.id) {
 	pjsip_endpt_cancel_timer(tls->base.endpt, &tls->ka_timer);
@@ -1281,7 +1358,7 @@ static pj_status_t tls_shutdown(pjsip_transport *transport)
 }
 
 
-/*
+/* 
  * Callback from ioqueue that an incoming data is received from the socket.
  */
 static pj_bool_t on_data_read(pj_ssl_sock_t *ssock,
@@ -1325,8 +1402,8 @@ static pj_bool_t on_data_read(pj_ssl_sock_t *ssock,
 	 * The transport manager will tell us how many bytes of the packet
 	 * have been processed (as valid SIP message).
 	 */
-	size_eaten =
-	    pjsip_tpmgr_receive_packet(rdata->tp_info.transport->tpmgr,
+	size_eaten = 
+	    pjsip_tpmgr_receive_packet(rdata->tp_info.transport->tpmgr, 
 				       rdata);
 
 	pj_assert(size_eaten <= (pj_size_t)rdata->pkt_info.len);
@@ -1357,7 +1434,7 @@ static pj_bool_t on_data_read(pj_ssl_sock_t *ssock,
 }
 
 
-/*
+/* 
  * Callback from ioqueue when asynchronous connect() operation completes.
  */
 static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
@@ -1365,7 +1442,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
 {
     struct tls_transport *tls;
     pj_ssl_sock_info ssl_info;
-    pj_sockaddr_in addr, *tp_addr;
+    pj_sockaddr addr, *tp_addr;
     pjsip_tp_state_callback state_cb;
     pj_bool_t is_shutdown;
 
@@ -1403,12 +1480,11 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
      * set is different now that the socket is connected (could happen
      * on some systems, like old Win32 probably?).
      */
-    tp_addr = (pj_sockaddr_in*)&tls->base.local_addr;
-    pj_sockaddr_cp((pj_sockaddr_t*)&addr,
+    tp_addr = &tls->base.local_addr;
+    pj_sockaddr_cp((pj_sockaddr_t*)&addr, 
 		   (pj_sockaddr_t*)&ssl_info.local_addr);
-    if (tp_addr->sin_addr.s_addr != addr.sin_addr.s_addr) {
-	tp_addr->sin_addr.s_addr = addr.sin_addr.s_addr;
-	tp_addr->sin_port = addr.sin_port;
+    if (pj_sockaddr_cmp(tp_addr, &addr) != 0) {
+	pj_sockaddr_cp(tp_addr, &addr);
 	sockaddr_to_host_port(tls->base.pool, &tls->base.local_name,
 			      tp_addr);
     }
@@ -1426,7 +1502,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
 	else
 	    remote_name = &tls->base.remote_name.host;
 
-	/* Start matching remote name with SubjectAltName fields of
+	/* Start matching remote name with SubjectAltName fields of 
 	 * server certificate.
 	 */
 	for (i = 0; i < serv_cert->subj_alt_name.cnt && !matched; ++i) {
@@ -1445,7 +1521,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
 		    char *p;
 
 		    p = pj_strchr(cert_name, ':') + 1;
-		    pj_strset(&host_part, p, cert_name->slen -
+		    pj_strset(&host_part, p, cert_name->slen - 
 					     (p - cert_name->ptr));
 		    matched = !pj_stricmp(remote_name, &host_part);
 		}
@@ -1454,7 +1530,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
 		break;
 	    }
 	}
-
+    	
 	/* When still not matched or no SubjectAltName fields in server
 	 * certificate, try with Common Name of Subject field.
 	 */
@@ -1466,7 +1542,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
 	    ssl_info.verify_status |= PJ_SSL_CERT_EIDENTITY_NOT_MATCH;
     }
 
-    /* Prevent immediate transport destroy as application may access it
+    /* Prevent immediate transport destroy as application may access it 
      * (getting info, etc) in transport state notification callback.
      */
     pjsip_transport_add_ref(&tls->base);
@@ -1475,7 +1551,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
      * and destroy the transport.
      */
     if (ssl_info.verify_status && tls->verify_server) {
-	if (tls->close_reason == PJ_SUCCESS)
+	if (tls->close_reason == PJ_SUCCESS) 
 	    tls->close_reason = PJSIP_TLS_ECERTVERIF;
 	pjsip_transport_shutdown(&tls->base);
     }
@@ -1511,14 +1587,31 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
      */
     is_shutdown = tls->base.is_shutdown;
     pjsip_transport_dec_ref(&tls->base);
-    if (is_shutdown)
+    if (is_shutdown) {
+	status = tls->close_reason;
+	tls_perror(tls->base.obj_name, "TLS connect() error", status);
+
+	/* Cancel all delayed transmits */
+	while (!pj_list_empty(&tls->delayed_list)) {
+	    struct delayed_tdata *pending_tx;
+	    pj_ioqueue_op_key_t *op_key;
+
+	    pending_tx = tls->delayed_list.next;
+	    pj_list_erase(pending_tx);
+
+	    op_key = (pj_ioqueue_op_key_t*)pending_tx->tdata_op_key;
+
+	    on_data_sent(tls->ssock, op_key, -status);
+	}
+
 	return PJ_FALSE;
+    }
 
 
     /* Mark that pending connect() operation has completed. */
     tls->has_pending_connect = PJ_FALSE;
 
-    PJ_LOG(4,(tls->base.obj_name,
+    PJ_LOG(4,(tls->base.obj_name, 
 	      "TLS transport %.*s:%d is connected to %.*s:%d",
 	      (int)tls->base.local_name.host.slen,
 	      tls->base.local_name.host.ptr,
@@ -1538,7 +1631,7 @@ static pj_bool_t on_connect_complete(pj_ssl_sock_t *ssock,
     /* Start keep-alive timer */
     if (PJSIP_TLS_KEEP_ALIVE_INTERVAL) {
 	pj_time_val delay = { PJSIP_TLS_KEEP_ALIVE_INTERVAL, 0 };
-	pjsip_endpt_schedule_timer(tls->base.endpt, &tls->ka_timer,
+	pjsip_endpt_schedule_timer(tls->base.endpt, &tls->ka_timer, 
 				   &delay);
 	tls->ka_timer.id = PJ_TRUE;
 	pj_gettimeofday(&tls->last_activity);
@@ -1574,13 +1667,13 @@ static void tls_keep_alive_timer(pj_timer_heap_t *th, pj_timer_entry *e)
 	delay.sec = PJSIP_TLS_KEEP_ALIVE_INTERVAL - now.sec;
 	delay.msec = 0;
 
-	pjsip_endpt_schedule_timer(tls->base.endpt, &tls->ka_timer,
+	pjsip_endpt_schedule_timer(tls->base.endpt, &tls->ka_timer, 
 				   &delay);
 	tls->ka_timer.id = PJ_TRUE;
 	return;
     }
 
-    PJ_LOG(5,(tls->base.obj_name, "Sending %d byte(s) keep-alive to %.*s:%d",
+    PJ_LOG(5,(tls->base.obj_name, "Sending %d byte(s) keep-alive to %.*s:%d", 
 	      (int)tls->ka_pkt.slen, (int)tls->base.remote_name.host.slen,
 	      tls->base.remote_name.host.ptr,
 	      tls->base.remote_name.port));
@@ -1591,7 +1684,7 @@ static void tls_keep_alive_timer(pj_timer_heap_t *th, pj_timer_entry *e)
 			      tls->ka_pkt.ptr, &size, 0);
 
     if (status != PJ_SUCCESS && status != PJ_EPENDING) {
-	tls_perror(tls->base.obj_name,
+	tls_perror(tls->base.obj_name, 
 		   "Error sending keep-alive packet", status);
 
 	tls_init_shutdown(tls, status);
@@ -1602,7 +1695,7 @@ static void tls_keep_alive_timer(pj_timer_heap_t *th, pj_timer_entry *e)
     delay.sec = PJSIP_TLS_KEEP_ALIVE_INTERVAL;
     delay.msec = 0;
 
-    pjsip_endpt_schedule_timer(tls->base.endpt, &tls->ka_timer,
+    pjsip_endpt_schedule_timer(tls->base.endpt, &tls->ka_timer, 
 			       &delay);
     tls->ka_timer.id = PJ_TRUE;
 }
