@@ -1,5 +1,5 @@
-/* $Id: sip_transport_udp.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: sip_transport_udp.c 4712 2014-01-23 08:09:29Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip/sip_transport_udp.h>
 #include <pjsip/sip_endpoint.h>
@@ -42,7 +42,7 @@
  * The buffer size is important, especially in WinXP/2000 machines.
  * Basicly the lower the size, the more packets will be lost (dropped?)
  * when we're sending (receiving?) packets in large volumes.
- *
+ * 
  * The figure here is taken based on my experiment on WinXP/2000 machine,
  * and with this value, the rate of dropped packet is about 8% when
  * sending 1800 requests simultaneously (percentage taken as average
@@ -96,9 +96,9 @@ static void init_rdata(struct udp_transport *tp, unsigned rdata_index,
     /* Init tp_info part. */
     rdata->tp_info.pool = pool;
     rdata->tp_info.transport = &tp->base;
-    rdata->tp_info.tp_data = (void*)(long)rdata_index;
+    rdata->tp_info.tp_data = (void*)(pj_ssize_t)rdata_index;
     rdata->tp_info.op_key.rdata = rdata;
-    pj_ioqueue_op_key_init(&rdata->tp_info.op_key.op_key,
+    pj_ioqueue_op_key_init(&rdata->tp_info.op_key.op_key, 
 			   sizeof(pj_ioqueue_op_key_t));
 
     tp->rdata[rdata_index] = rdata;
@@ -114,8 +114,8 @@ static void init_rdata(struct udp_transport *tp, unsigned rdata_index,
  * This is callback notification from ioqueue that a pending recvfrom()
  * operation has completed.
  */
-static void udp_on_read_complete( pj_ioqueue_key_t *key,
-				  pj_ioqueue_op_key_t *op_key,
+static void udp_on_read_complete( pj_ioqueue_key_t *key, 
+				  pj_ioqueue_op_key_t *op_key, 
 				  pj_ssize_t bytes_read)
 {
     /* See https://trac.pjsip.org/repos/ticket/1197 */
@@ -143,7 +143,7 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
      * complete asynchronously, to allow other sockets to get their data.
      */
     for (i=0;; ++i) {
-	enum { MIN_SIZE = 32 };
+    	enum { MIN_SIZE = 32 };
 	pj_uint32_t flags;
 
 	/* Report the packet to transport manager. Only do so if packet size
@@ -162,22 +162,17 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 			       pj_inet_ntoa(src_addr->ipv4.sin_addr));
 		rdata->pkt_info.src_port = pj_ntohs(src_addr->ipv4.sin_port);
 	    } else {
-		pj_inet_ntop(pj_AF_INET6(),
+		pj_inet_ntop(pj_AF_INET6(), 
 			     pj_sockaddr_get_addr(&rdata->pkt_info.src_addr),
 			     rdata->pkt_info.src_name,
 			     sizeof(rdata->pkt_info.src_name));
 		rdata->pkt_info.src_port = pj_ntohs(src_addr->ipv6.sin6_port);
 	    }
 
-	    size_eaten =
-		pjsip_tpmgr_receive_packet(rdata->tp_info.transport->tpmgr,
+	    size_eaten = 
+		pjsip_tpmgr_receive_packet(rdata->tp_info.transport->tpmgr, 
 					   rdata);
 
-	    if (size_eaten < 0) {
-		pj_assert(!"It shouldn't happen!");
-		size_eaten = rdata->pkt_info.len;
-	    }
-
 	    /* Since this is UDP, the whole buffer is the message. */
 	    rdata->pkt_info.len = 0;
 
@@ -186,14 +181,14 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	    /* TODO: */
 
 	} else if (-bytes_read != PJ_STATUS_FROM_OS(OSERR_EWOULDBLOCK) &&
-		   -bytes_read != PJ_STATUS_FROM_OS(OSERR_EINPROGRESS) &&
-		   -bytes_read != PJ_STATUS_FROM_OS(OSERR_ECONNRESET))
+		   -bytes_read != PJ_STATUS_FROM_OS(OSERR_EINPROGRESS) && 
+		   -bytes_read != PJ_STATUS_FROM_OS(OSERR_ECONNRESET)) 
 	{
 
 	    /* Report error to endpoint. */
 	    PJSIP_ENDPT_LOG_ERROR((rdata->tp_info.transport->endpt,
 				   rdata->tp_info.transport->obj_name,
-				   -bytes_read,
+				   (pj_status_t)-bytes_read, 
 				   "Warning: pj_ioqueue_recvfrom()"
 				   " callback error"));
 	}
@@ -205,7 +200,7 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	    flags = 0;
 	}
 
-	/* Reset pool.
+	/* Reset pool. 
 	 * Need to copy rdata fields to temp variable because they will
 	 * be invalid after pj_pool_reset().
 	 */
@@ -215,7 +210,8 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	    unsigned rdata_index;
 
 	    rdata_tp = (struct udp_transport*)rdata->tp_info.transport;
-	    rdata_index = (unsigned)(unsigned long)rdata->tp_info.tp_data;
+	    rdata_index = (unsigned)(unsigned long)(pj_ssize_t)
+			  rdata->tp_info.tp_data;
 
 	    pj_pool_reset(rdata_pool);
 	    init_rdata(rdata_tp, rdata_index, rdata_pool, &rdata);
@@ -236,10 +232,10 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	/* Read next packet. */
 	bytes_read = sizeof(rdata->pkt_info.packet);
 	rdata->pkt_info.src_addr_len = sizeof(rdata->pkt_info.src_addr);
-	status = pj_ioqueue_recvfrom(key, op_key,
+	status = pj_ioqueue_recvfrom(key, op_key, 
 				     rdata->pkt_info.packet,
 				     &bytes_read, flags,
-				     &rdata->pkt_info.src_addr,
+				     &rdata->pkt_info.src_addr, 
 				     &rdata->pkt_info.src_addr_len);
 
 	if (status == PJ_SUCCESS) {
@@ -255,13 +251,13 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 
 		/* Report error to endpoint if this is not EWOULDBLOCK error.*/
 		if (status != PJ_STATUS_FROM_OS(OSERR_EWOULDBLOCK) &&
-		    status != PJ_STATUS_FROM_OS(OSERR_EINPROGRESS) &&
-		    status != PJ_STATUS_FROM_OS(OSERR_ECONNRESET))
+		    status != PJ_STATUS_FROM_OS(OSERR_EINPROGRESS) && 
+		    status != PJ_STATUS_FROM_OS(OSERR_ECONNRESET)) 
 		{
 
 		    PJSIP_ENDPT_LOG_ERROR((rdata->tp_info.transport->endpt,
 					   rdata->tp_info.transport->obj_name,
-					   status,
+					   status, 
 					   "Warning: pj_ioqueue_recvfrom"));
 		}
 
@@ -273,7 +269,7 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 		 */
 		PJSIP_ENDPT_LOG_ERROR((rdata->tp_info.transport->endpt,
 				       rdata->tp_info.transport->obj_name,
-				       status,
+				       status, 
 				       "FATAL: pj_ioqueue_recvfrom() error, "
 				       "UDP transport stopping! Error"));
 		break;
@@ -288,12 +284,12 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
  * This is callback notification from ioqueue that a pending sendto()
  * operation has completed.
  */
-static void udp_on_write_complete( pj_ioqueue_key_t *key,
+static void udp_on_write_complete( pj_ioqueue_key_t *key, 
 				   pj_ioqueue_op_key_t *op_key,
 				   pj_ssize_t bytes_sent)
 {
-    struct udp_transport *tp = (struct udp_transport*)
-			       pj_ioqueue_get_user_data(key);
+    struct udp_transport *tp = (struct udp_transport*) 
+    			       pj_ioqueue_get_user_data(key);
     pjsip_tx_data_op_key *tdata_op_key = (pjsip_tx_data_op_key*)op_key;
 
     tdata_op_key->tdata = NULL;
@@ -322,7 +318,7 @@ static pj_status_t udp_send_msg( pjsip_transport *transport,
 
     PJ_ASSERT_RETURN(transport && tdata, PJ_EINVAL);
     PJ_ASSERT_RETURN(tdata->op_key.tdata == NULL, PJSIP_EPENDINGTX);
-
+    
     /* Return error if transport is paused */
     if (tp->is_paused)
 	return PJSIP_ETPNOTAVAIL;
@@ -365,7 +361,7 @@ static pj_status_t udp_destroy( pjsip_transport *transport )
      *      and another one for closing the socket.
      *
     for (i=0; i<tp->rdata_cnt; ++i) {
-	pj_ioqueue_post_completion(tp->key,
+	pj_ioqueue_post_completion(tp->key, 
 				   &tp->rdata[i]->tp_info.op_key.op_key, -1);
     }
     */
@@ -383,14 +379,14 @@ static pj_status_t udp_destroy( pjsip_transport *transport )
     }
 
     /* Must poll ioqueue because IOCP calls the callback when socket
-     * is closed. We poll the ioqueue until all pending callbacks
+     * is closed. We poll the ioqueue until all pending callbacks 
      * have been called.
      */
     for (i=0; i<50 && tp->is_closing < 1+tp->rdata_cnt; ++i) {
 	int cnt;
 	pj_time_val timeout = {0, 1};
 
-	cnt = pj_ioqueue_poll(pjsip_endpt_get_ioqueue(transport->endpt),
+	cnt = pj_ioqueue_poll(pjsip_endpt_get_ioqueue(transport->endpt), 
 			      &timeout);
 	if (cnt == 0)
 	    break;
@@ -496,7 +492,7 @@ static pj_status_t get_published_name(pj_sock_t sock,
 	    pj_strcpy2(&bound_name->host, pj_inet_ntoa(hostip.ipv4.sin_addr));
 	} else {
 	    /* Otherwise use bound address. */
-	    pj_strcpy2(&bound_name->host,
+	    pj_strcpy2(&bound_name->host, 
 		       pj_inet_ntoa(tmp_addr.ipv4.sin_addr));
 	    status = PJ_SUCCESS;
 	}
@@ -515,7 +511,7 @@ static pj_status_t get_published_name(pj_sock_t sock,
 		return status;
 	}
 
-	status = pj_inet_ntop(tmp_addr.addr.sa_family,
+	status = pj_inet_ntop(tmp_addr.addr.sa_family, 
 			      pj_sockaddr_get_addr(&tmp_addr),
 			      hostbuf, hostbufsz);
 	if (status == PJ_SUCCESS) {
@@ -535,7 +531,7 @@ static void udp_set_pub_name(struct udp_transport *tp,
     char local_addr[PJ_INET6_ADDRSTRLEN+10];
 
     pj_assert(a_name->host.slen != 0);
-    pj_strdup_with_null(tp->base.pool, &tp->base.local_name.host,
+    pj_strdup_with_null(tp->base.pool, &tp->base.local_name.host, 
 			&a_name->host);
     tp->base.local_name.port = a_name->port;
 
@@ -546,7 +542,7 @@ static void udp_set_pub_name(struct udp_transport *tp,
 
     pj_sockaddr_print(&tp->base.local_addr, local_addr, sizeof(local_addr), 3);
 
-    pj_ansi_snprintf(
+    pj_ansi_snprintf( 
 	tp->base.info, INFO_LEN, "udp %s [published as %s:%d]",
 	local_addr,
 	tp->base.local_name.host.ptr,
@@ -604,8 +600,8 @@ static pj_status_t register_to_ioqueue(struct udp_transport *tp)
 
     /* Ignore if already registered */
     if (tp->key != NULL)
-	return PJ_SUCCESS;
-
+    	return PJ_SUCCESS;
+    
     /* Register to ioqueue. */
     ioqueue = pjsip_endpt_get_ioqueue(tp->base.endpt);
     pj_memset(&ioqueue_cb, 0, sizeof(ioqueue_cb));
@@ -619,19 +615,16 @@ static pj_status_t register_to_ioqueue(struct udp_transport *tp)
 /* Start ioqueue asynchronous reading to all rdata */
 static pj_status_t start_async_read(struct udp_transport *tp)
 {
-    pj_ioqueue_t *ioqueue;
     int i;
     pj_status_t status;
 
-    ioqueue = pjsip_endpt_get_ioqueue(tp->base.endpt);
-
     /* Start reading the ioqueue. */
     for (i=0; i<tp->rdata_cnt; ++i) {
 	pj_ssize_t size;
 
 	size = sizeof(tp->rdata[i]->pkt_info.packet);
 	tp->rdata[i]->pkt_info.src_addr_len = sizeof(tp->rdata[i]->pkt_info.src_addr);
-	status = pj_ioqueue_recvfrom(tp->key,
+	status = pj_ioqueue_recvfrom(tp->key, 
 				     &tp->rdata[i]->tp_info.op_key.op_key,
 				     tp->rdata[i]->pkt_info.packet,
 				     &size, PJ_IOQUEUE_ALWAYS_ASYNC,
@@ -683,7 +676,7 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
     }
 
     /* Create pool. */
-    pool = pjsip_endpt_create_pool(endpt, format, PJSIP_POOL_LEN_TRANSPORT,
+    pool = pjsip_endpt_create_pool(endpt, format, PJSIP_POOL_LEN_TRANSPORT, 
 				   PJSIP_POOL_INC_TRANSPORT);
     if (!pool)
 	return PJ_ENOMEM;
@@ -702,7 +695,7 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
 	goto on_error;
 
     /* Init lock. */
-    status = pj_lock_create_recursive_mutex(pool, pool->obj_name,
+    status = pj_lock_create_recursive_mutex(pool, pool->obj_name, 
 					    &tp->base.lock);
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -725,7 +718,7 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
     tp->base.addr_len = sizeof(tp->base.local_addr);
 
     /* Init local address. */
-    status = pj_sock_getsockname(sock, &tp->base.local_addr,
+    status = pj_sock_getsockname(sock, &tp->base.local_addr, 
 				 &tp->base.addr_len);
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -774,10 +767,10 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
     /* Create rdata and put it in the array. */
     tp->rdata_cnt = 0;
     tp->rdata = (pjsip_rx_data**)
-		pj_pool_calloc(tp->base.pool, async_cnt,
+    		pj_pool_calloc(tp->base.pool, async_cnt, 
 			       sizeof(pjsip_rx_data*));
     for (i=0; i<async_cnt; ++i) {
-	pj_pool_t *rdata_pool = pjsip_endpt_create_pool(endpt, "rtd%p",
+	pj_pool_t *rdata_pool = pjsip_endpt_create_pool(endpt, "rtd%p", 
 							PJSIP_POOL_RDATA_LEN,
 							PJSIP_POOL_RDATA_INC);
 	if (!rdata_pool) {
@@ -800,8 +793,8 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
     /* Done. */
     if (p_transport)
 	*p_transport = &tp->base;
-
-    PJ_LOG(4,(tp->base.obj_name,
+    
+    PJ_LOG(4,(tp->base.obj_name, 
 	      "SIP %s started, published address is %s%.*s%s:%d",
 	      pjsip_transport_get_type_desc((pjsip_transport_type_e)tp->base.key.type),
 	      ipv6_quoteb,
@@ -857,16 +850,16 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_start( pjsip_endpoint *endpt,
 
     PJ_ASSERT_RETURN(endpt && async_cnt, PJ_EINVAL);
 
-    status = create_socket(pj_AF_INET(), local_a, sizeof(pj_sockaddr_in),
+    status = create_socket(pj_AF_INET(), local_a, sizeof(pj_sockaddr_in), 
 			   &sock);
     if (status != PJ_SUCCESS)
 	return status;
 
     if (a_name == NULL) {
-	/* Address name is not specified.
+	/* Address name is not specified. 
 	 * Build a name based on bound address.
 	 */
-	status = get_published_name(sock, addr_buf, sizeof(addr_buf),
+	status = get_published_name(sock, addr_buf, sizeof(addr_buf), 
 				    &bound_name);
 	if (status != PJ_SUCCESS) {
 	    pj_sock_close(sock);
@@ -876,7 +869,7 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_start( pjsip_endpoint *endpt,
 	a_name = &bound_name;
     }
 
-    return pjsip_udp_transport_attach( endpt, sock, a_name, async_cnt,
+    return pjsip_udp_transport_attach( endpt, sock, a_name, async_cnt, 
 				       p_transport );
 }
 
@@ -899,16 +892,16 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_start6(pjsip_endpoint *endpt,
 
     PJ_ASSERT_RETURN(endpt && async_cnt, PJ_EINVAL);
 
-    status = create_socket(pj_AF_INET6(), local_a, sizeof(pj_sockaddr_in6),
+    status = create_socket(pj_AF_INET6(), local_a, sizeof(pj_sockaddr_in6), 
 			   &sock);
     if (status != PJ_SUCCESS)
 	return status;
 
     if (a_name == NULL) {
-	/* Address name is not specified.
+	/* Address name is not specified. 
 	 * Build a name based on bound address.
 	 */
-	status = get_published_name(sock, addr_buf, sizeof(addr_buf),
+	status = get_published_name(sock, addr_buf, sizeof(addr_buf), 
 				    &bound_name);
 	if (status != PJ_SUCCESS) {
 	    pj_sock_close(sock);
@@ -938,7 +931,7 @@ PJ_DEF(pj_sock_t) pjsip_udp_transport_get_socket(pjsip_transport *transport)
 
 
 /*
- * Temporarily pause or shutdown the transport.
+ * Temporarily pause or shutdown the transport. 
  */
 PJ_DEF(pj_status_t) pjsip_udp_transport_pause(pjsip_transport *transport,
 					      unsigned option)
@@ -956,7 +949,7 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_pause(pjsip_transport *transport,
     /* Transport must not have been paused */
     PJ_ASSERT_RETURN(tp->is_paused==0, PJ_EINVALIDOP);
 
-    /* Set transport to paused first, so that when the read callback is
+    /* Set transport to paused first, so that when the read callback is 
      * called by pj_ioqueue_post_completion() it will not try to
      * re-register the rdata.
      */
@@ -964,7 +957,7 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_pause(pjsip_transport *transport,
 
     /* Cancel the ioqueue operation. */
     for (i=0; i<(unsigned)tp->rdata_cnt; ++i) {
-	pj_ioqueue_post_completion(tp->key,
+	pj_ioqueue_post_completion(tp->key, 
 				   &tp->rdata[i]->tp_info.op_key.op_key, -1);
     }
 
@@ -982,7 +975,7 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_pause(pjsip_transport *transport,
 	    }
 	}
 	tp->sock = PJ_INVALID_SOCKET;
-
+	
     }
 
     PJ_LOG(4,(tp->base.obj_name, "SIP UDP transport paused"));
@@ -1037,7 +1030,7 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 
 	/* Create the socket if it's not specified */
 	if (sock == PJ_INVALID_SOCKET) {
-	    status = create_socket(pj_AF_INET(), local,
+	    status = create_socket(pj_AF_INET(), local, 
 				   sizeof(pj_sockaddr_in), &sock);
 	    if (status != PJ_SUCCESS)
 		return status;
@@ -1057,6 +1050,12 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 	    a_name = &bound_name;
 	}
 
+        /* Init local address. */
+        status = pj_sock_getsockname(sock, &tp->base.local_addr, 
+				     &tp->base.addr_len);
+        if (status != PJ_SUCCESS)
+	    return status;
+
 	/* Assign the socket and published address to transport. */
 	udp_set_socket(tp, sock, a_name);
 
@@ -1084,7 +1083,7 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
     /* Everything has been set up */
     tp->is_paused = PJ_FALSE;
 
-    PJ_LOG(4,(tp->base.obj_name,
+    PJ_LOG(4,(tp->base.obj_name, 
 	      "SIP UDP transport restarted, published address is %.*s:%d",
 	      (int)tp->base.local_name.host.slen,
 	      tp->base.local_name.host.ptr,
@@ -1092,3 +1091,4 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 
     return PJ_SUCCESS;
 }
+
