@@ -1,4 +1,4 @@
-/* $Id: sip_ua_layer.c 4208 2012-07-18 07:52:33Z ming $ */
+/* $Id: sip_ua_layer.c 4537 2013-06-19 06:47:43Z riza $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -304,7 +304,7 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 	dlg_set = (struct dlg_set*)
 		  pj_hash_get_lower( mod_ua.dlg_table,
                                      dlg->local.info->tag.ptr, 
-			             dlg->local.info->tag.slen,
+			             (unsigned)dlg->local.info->tag.slen,
 			             &dlg->local.tag_hval);
 
 	if (dlg_set) {
@@ -329,7 +329,7 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 	    /* Register the dialog set in the hash table. */
 	    pj_hash_set_np_lower(mod_ua.dlg_table, 
 			         dlg->local.info->tag.ptr,
-                                 dlg->local.info->tag.slen,
+                                 (unsigned)dlg->local.info->tag.slen,
 			         dlg->local.tag_hval, dlg_set->ht_entry,
                                  dlg_set);
 	}
@@ -346,7 +346,7 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 
 	pj_hash_set_np_lower(mod_ua.dlg_table, 
 		             dlg->local.info->tag.ptr,
-                             dlg->local.info->tag.slen,
+                             (unsigned)dlg->local.info->tag.slen,
 		             dlg->local.tag_hval, dlg_set->ht_entry, dlg_set);
     }
 
@@ -392,8 +392,8 @@ PJ_DEF(pj_status_t) pjsip_ua_unregister_dlg( pjsip_user_agent *ua,
     /* If dialog list is empty, remove the dialog set from the hash table. */
     if (pj_list_empty(&dlg_set->dlg_list)) {
 	pj_hash_set_lower(NULL, mod_ua.dlg_table, dlg->local.info->tag.ptr,
-		          dlg->local.info->tag.slen, dlg->local.tag_hval,
-                          NULL);
+		          (unsigned)dlg->local.info->tag.slen, 
+			  dlg->local.tag_hval, NULL);
 
 	/* Return dlg_set to free nodes. */
 	pj_list_push_back(&mod_ua.free_dlgset_nodes, dlg_set);
@@ -455,7 +455,7 @@ PJ_DEF(pjsip_dialog*) pjsip_ua_find_dialog(const pj_str_t *call_id,
     /* Lookup the dialog set. */
     dlg_set = (struct dlg_set*)
     	      pj_hash_get_lower(mod_ua.dlg_table, local_tag->ptr,
-                                local_tag->slen, NULL);
+                                (unsigned)local_tag->slen, NULL);
     if (dlg_set == NULL) {
 	/* Not found */
 	pj_mutex_unlock(mod_ua.mutex);
@@ -545,7 +545,7 @@ static struct dlg_set *find_dlg_set_for_msg( pjsip_rx_data *rdata )
 	/* We should find the dialog attached to the INVITE transaction */
 	if (tsx) {
 	    dlg = (pjsip_dialog*) tsx->mod_data[mod_ua.mod.id];
-	    pj_mutex_unlock(tsx->mutex);
+	    pj_grp_lock_release(tsx->grp_lock);
 
 	    /* Dlg may be NULL on some extreme condition
 	     * (e.g. during debugging where initially there is a dialog)
@@ -568,8 +568,8 @@ static struct dlg_set *find_dlg_set_for_msg( pjsip_rx_data *rdata )
 
 	/* Lookup the dialog set. */
 	dlg_set = (struct dlg_set*)
-		  pj_hash_get_lower(mod_ua.dlg_table, tag->ptr, tag->slen,
-                                    NULL);
+		  pj_hash_get_lower(mod_ua.dlg_table, tag->ptr, 
+				    (unsigned)tag->slen, NULL);
 	return dlg_set;
     }
 }
@@ -769,7 +769,7 @@ retry_on_deadlock:
 	dlg_set = (struct dlg_set*)
 		  pj_hash_get_lower(mod_ua.dlg_table, 
 			            rdata->msg_info.from->tag.ptr,
-			            rdata->msg_info.from->tag.slen,
+			            (unsigned)rdata->msg_info.from->tag.slen,
 			            NULL);
 
 	if (!dlg_set) {
