@@ -1,4 +1,4 @@
-/* $Id: sip_util.c 4173 2012-06-20 10:39:05Z ming $ */
+/* $Id: sip_util.c 4770 2014-02-27 06:16:36Z nanang $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -274,7 +274,7 @@ static void init_request_throw( pjsip_endpoint *endpt,
 	body->content_type.subtype = str_PLAIN;
 	body->data = pj_pool_alloc(tdata->pool, param_text->slen );
 	pj_memcpy(body->data, param_text->ptr, param_text->slen);
-	body->len = param_text->slen;
+	body->len = (unsigned)param_text->slen;
 	body->print_body = &pjsip_print_text_body;
 	msg->body = body;
     }
@@ -802,6 +802,7 @@ on_missing_hdr:
 
 /* Fill-up destination information from a target URI */
 static pj_status_t get_dest_info(const pjsip_uri *target_uri, 
+				 const pjsip_uri *request_uri, 
 				 pj_pool_t *pool,
 				 pjsip_host_info *dest_info)
 {
@@ -810,11 +811,22 @@ static pj_status_t get_dest_info(const pjsip_uri *target_uri,
      */
     pj_bzero(dest_info, sizeof(*dest_info));
 
-    if (PJSIP_URI_SCHEME_IS_SIPS(target_uri)) {
+    /* When request URI uses sips scheme, TLS must always be used regardless
+     * of the target scheme or transport type (see ticket #1740).
+     */
+    if (PJSIP_URI_SCHEME_IS_SIPS(target_uri) || 
+	(pjsip_cfg()->endpt.disable_tls_switch == 0 && request_uri &&
+	 PJSIP_URI_SCHEME_IS_SIPS(request_uri)))
+    {
 	pjsip_uri *uri = (pjsip_uri*) target_uri;
 	const pjsip_sip_uri *url=(const pjsip_sip_uri*)pjsip_uri_get_uri(uri);
 	unsigned flag;
 
+	if (!PJSIP_URI_SCHEME_IS_SIPS(target_uri)) {
+	    PJ_LOG(4,(THIS_FILE, "Automatic switch to TLS transport as "
+				 "request-URI uses ""sips"" scheme."));
+	}
+
 	dest_info->flag |= (PJSIP_TRANSPORT_SECURE | PJSIP_TRANSPORT_RELIABLE);
 	if (url->maddr_param.slen)
 	    pj_strdup(pool, &dest_info->addr.host, &url->maddr_param);
@@ -895,7 +907,8 @@ PJ_DEF(pj_status_t) pjsip_get_request_dest(const pjsip_tx_data *tdata,
 	target_uri = tdata->msg->line.req.uri;
     }
 
-    return get_dest_info(target_uri, (pj_pool_t*)tdata->pool, dest_info);
+    return get_dest_info(target_uri, tdata->msg->line.req.uri,
+			 (pj_pool_t*)tdata->pool, dest_info);
 }
 
 
@@ -998,7 +1011,8 @@ PJ_DEF(pj_status_t) pjsip_process_route_set(pjsip_tx_data *tdata,
     }
 
     /* Fill up the destination host/port from the URI. */
-    status = get_dest_info(target_uri, tdata->pool, dest_info);
+    status = get_dest_info(target_uri, new_request_uri, tdata->pool,
+			   dest_info);
     if (status != PJ_SUCCESS)
 	return status;
 
@@ -1201,6 +1215,28 @@ static void stateless_send_transport_cb( void *token,
         }
 	via->rport_param = pjsip_cfg()->endpt.disable_rport ? -1 : 0;
 
+	/* Add/remove "alias" param to/from Via header on connection 
+	 * oriented/less transport, if configured.
+	 */
+	if (pjsip_cfg()->endpt.req_has_via_alias &&
+	    tdata->msg->type == PJSIP_REQUEST_MSG)
+	{
+	    const pj_str_t ALIAS_STR = {"alias", 5};
+	    pjsip_param *alias_param;
+	    pj_bool_t is_datagram;
+
+	    alias_param = pjsip_param_find(&via->other_param, &ALIAS_STR);
+	    is_datagram = (stateless_data->cur_transport->flag & 
+			   PJSIP_TRANSPORT_DATAGRAM);
+	    if (!is_datagram && !alias_param) {
+		alias_param = PJ_POOL_ZALLOC_T(tdata->pool, pjsip_param);
+		alias_param->name = ALIAS_STR;
+		pj_list_push_back(&via->other_param, alias_param);
+	    } else if (is_datagram && alias_param) {
+		pj_list_erase(alias_param);
+	    }
+	}
+
 	pjsip_tx_data_invalidate_msg(tdata);
 
 	/* Send message using this transport. */
@@ -1279,7 +1315,7 @@ stateless_send_resolver_callback( pj_status_t status,
 	}
 
 	/* Check if request message is larger than 1300 bytes. */
-	len = tdata->buf.cur - tdata->buf.start;
+	len = (int)(tdata->buf.cur - tdata->buf.start);
 	if (len >= PJSIP_UDP_SIZE_THRESHOLD) {
 	    int i;
 	    int count = tdata->dest_info.addr.count;
@@ -1473,7 +1509,7 @@ PJ_DEF(pj_status_t) pjsip_endpt_send_raw_to_uri(pjsip_endpoint *endpt,
     }
 
     /* Build destination info. */
-    status = get_dest_info(uri, tdata->pool, &dest_info);
+    status = get_dest_info(uri, NULL, tdata->pool, &dest_info);
     if (status != PJ_SUCCESS) {
 	pjsip_tx_data_dec_ref(tdata);
 	return status;
