@@ -1,5 +1,5 @@
-/* $Id: pjsua_vid.c 4071 2012-04-24 05:40:32Z nanang $ */
-/*
+/* $Id: pjsua_vid.c 4750 2014-02-19 04:11:43Z bennylp $ */
+/* 
  * Copyright (C) 2011-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsua-lib/pjsua.h>
 #include <pjsua-lib/pjsua_internal.h>
@@ -236,7 +236,7 @@ static pj_status_t find_codecs_with_rtp_packing(
     const pjmedia_vid_codec_info *info[32];
     unsigned i, j, count_ = PJ_ARRAY_SIZE(info);
     pj_status_t status;
-
+    
     status = pjmedia_vid_codec_mgr_find_codecs_by_id(NULL, codec_id,
 						     &count_, info, NULL);
     if (status != PJ_SUCCESS)
@@ -635,7 +635,7 @@ on_error:
 static void free_vid_win(pjsua_vid_win_id wid)
 {
     pjsua_vid_win *w = &pjsua_var.win[wid];
-
+    
     PJ_LOG(4,(THIS_FILE, "Window %d: destroying..", wid));
     pj_log_push_indent();
 
@@ -664,7 +664,7 @@ static void free_vid_win(pjsua_vid_win_id wid)
 static void inc_vid_win(pjsua_vid_win_id wid)
 {
     pjsua_vid_win *w;
-
+    
     pj_assert(wid >= 0 && wid < PJSUA_MAX_VID_WINS);
 
     w = &pjsua_var.win[wid];
@@ -675,7 +675,7 @@ static void inc_vid_win(pjsua_vid_win_id wid)
 static void dec_vid_win(pjsua_vid_win_id wid)
 {
     pjsua_vid_win *w;
-
+    
     pj_assert(wid >= 0 && wid < PJSUA_MAX_VID_WINS);
 
     w = &pjsua_var.win[wid];
@@ -705,7 +705,12 @@ pj_status_t pjsua_vid_channel_init(pjsua_call_media *call_med)
     return PJ_SUCCESS;
 }
 
-/* Internal function: update video channel after SDP negotiation */
+/* Internal function: update video channel after SDP negotiation.
+ * Warning: do not use temporary/flip-flop pool, e.g: inv->pool_prov,
+ *          for creating stream, etc, as after SDP negotiation and when
+ *	    the SDP media is not changed, the stream should remain running
+ *          while the temporary/flip-flop pool may be released.
+ */
 pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 				     pj_pool_t *tmp_pool,
 				     pjmedia_vid_stream_info *si,
@@ -716,7 +721,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
     pjsua_acc  *acc  = &pjsua_var.acc[call->acc_id];
     pjmedia_port *media_port;
     pj_status_t status;
-
+ 
     PJ_UNUSED_ARG(tmp_pool);
     PJ_UNUSED_ARG(local_sdp);
     PJ_UNUSED_ARG(remote_sdp);
@@ -724,7 +729,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
     PJ_LOG(4,(THIS_FILE, "Video channel update.."));
     pj_log_push_indent();
 
-    si->rtcp_sdes_bye_disabled = PJ_TRUE;
+    si->rtcp_sdes_bye_disabled = pjsua_var.media_cfg.no_rtcp_sdes_bye;;
 
     /* Check if no media is active */
     if (si->dir != PJMEDIA_DIR_NONE) {
@@ -756,7 +761,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 	si->use_ka = pjsua_var.acc[call->acc_id].cfg.use_stream_ka;
 #endif
 
-	/* Try to get shared format ID between the capture device and
+	/* Try to get shared format ID between the capture device and 
 	 * the encoder to avoid format conversion in the capture device.
 	 */
 	if (si->dir & PJMEDIA_DIR_ENCODING) {
@@ -772,11 +777,11 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 	    /* Find matched format ID */
 	    for (i = 0; i < codec_info->dec_fmt_id_cnt; ++i) {
 		for (j = 0; j < dev_info.fmt_cnt; ++j) {
-		    if (codec_info->dec_fmt_id[i] ==
+		    if (codec_info->dec_fmt_id[i] == 
 			(pjmedia_format_id)dev_info.fmt[j].id)
 		    {
 			/* Apply the matched format ID to the codec */
-			si->codec_param->dec_fmt.id =
+			si->codec_param->dec_fmt.id = 
 						codec_info->dec_fmt_id[i];
 
 			/* Force outer loop to break */
@@ -799,6 +804,9 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 	if (status != PJ_SUCCESS)
 	    goto on_error;
 
+        if (call_med->prev_state == PJSUA_CALL_MEDIA_NONE)
+            pjmedia_vid_stream_send_rtcp_sdes(call_med->strm.v.stream);
+
 	/* Setup decoding direction */
 	if (si->dir & PJMEDIA_DIR_DECODING)
 	{
@@ -837,7 +845,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 	    pjmedia_event_subscribe(NULL, &call_media_on_event,
                                     call_med, w->vp_rend);
 #endif
-
+	    
 	    /* Connect renderer to stream */
 	    status = pjmedia_vid_port_connect(w->vp_rend, media_port,
 					      PJ_FALSE);
@@ -907,7 +915,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
             pjmedia_event_subscribe(NULL, &call_media_on_event,
                                     call_med, w->vp_cap);
 #endif
-
+	    
 	    /* Connect stream to capturer (via video window tee) */
 	    status = pjmedia_vid_tee_add_dst_port2(w->tee, 0, media_port);
 	    if (status != PJ_SUCCESS) {
@@ -961,6 +969,8 @@ void pjsua_vid_stop_stream(pjsua_call_media *call_med)
 
     PJ_LOG(4,(THIS_FILE, "Stopping video stream.."));
     pj_log_push_indent();
+    
+    pjmedia_vid_stream_send_rtcp_bye(strm);
 
     if (call_med->strm.v.cap_win_id != PJSUA_INVALID_ID) {
 	pjmedia_port *media_port;
@@ -1525,6 +1535,11 @@ static pj_status_t call_add_video(pjsua_call *call,
     if (call->med_cnt == PJSUA_MAX_CALL_MEDIA)
 	return PJ_ETOOMANY;
 
+    if (pjsua_call_media_is_changing(call)) {
+	PJ_LOG(1,(THIS_FILE, "Unable to add video" ERR_MEDIA_CHANGING));
+	return PJ_EINVALIDOP;
+    }
+
     /* Get active local SDP and clone it */
     status = pjmedia_sdp_neg_get_active_local(call->inv->neg, &current_sdp);
     if (status != PJ_SUCCESS)
@@ -1626,10 +1641,15 @@ static pj_status_t call_modify_video(pjsua_call *call,
     pjmedia_sdp_session *sdp;
     pj_status_t status;
 
+    if (pjsua_call_media_is_changing(call)) {
+	PJ_LOG(1,(THIS_FILE, "Unable to modify video" ERR_MEDIA_CHANGING));
+	return PJ_EINVALIDOP;
+    }
+
     /* Verify and normalize media index */
     if (med_idx == -1) {
 	int first_active;
-
+	
 	call_get_vid_strm_info(call, &first_active, NULL, NULL, NULL);
 	if (first_active == -1)
 	    return PJ_ENOTFOUND;
@@ -1743,18 +1763,20 @@ static pj_status_t call_modify_video(pjsua_call *call,
 
 	sdp->media[med_idx] = sdp_m;
 
-	/* Update SDP media line by media transport */
-	status = pjmedia_transport_encode_sdp(call_med->tp, pool,
-					      sdp, NULL, call_med->idx);
-	if (status != PJ_SUCCESS)
-	    goto on_error;
+        if (call_med->dir == PJMEDIA_DIR_NONE) {
+	    /* Update SDP media line by media transport */
+	    status = pjmedia_transport_encode_sdp(call_med->tp, pool,
+					          sdp, NULL, call_med->idx);
+	    if (status != PJ_SUCCESS)
+	        goto on_error;
+        }
 
 on_error:
 	if (status != PJ_SUCCESS) {
 	    pjsua_media_prov_clean_up(call->index);
 	    return status;
 	}
-
+    
     } else {
 
 	pj_pool_t *pool = call->inv->pool_prov;
@@ -1795,7 +1817,7 @@ static pj_status_t call_change_cap_dev(pjsua_call *call,
     /* Verify and normalize media index */
     if (med_idx == -1) {
 	int first_active;
-
+	
 	call_get_vid_strm_info(call, &first_active, NULL, NULL, NULL);
 	if (first_active == -1)
 	    return PJ_ENOTFOUND;
@@ -1845,7 +1867,7 @@ static pj_status_t call_change_cap_dev(pjsua_call *call,
 
     pjmedia_event_unsubscribe(NULL, &call_media_on_event, call_med,
                               w->vp_cap);
-
+    
     /* temporarily disconnect while we operate on the tee. */
     pjmedia_vid_port_disconnect(w->vp_cap);
 
@@ -1888,7 +1910,7 @@ static pj_status_t call_change_cap_dev(pjsua_call *call,
 
     inc_vid_win(new_wid);
     new_w = &pjsua_var.win[new_wid];
-
+    
     /* Connect stream to capturer (via video window tee) */
     status = pjmedia_vid_tee_add_dst_port2(new_w->tee, 0, media_port);
     if (status != PJ_SUCCESS)
@@ -1963,7 +1985,7 @@ static pj_status_t call_set_tx_video(pjsua_call *call,
     /* Verify and normalize media index */
     if (med_idx == -1) {
 	int first_active;
-
+	
 	call_get_vid_strm_info(call, &first_active, NULL, NULL, NULL);
 	if (first_active == -1)
 	    return PJ_ENOTFOUND;
@@ -1974,7 +1996,7 @@ static pj_status_t call_set_tx_video(pjsua_call *call,
     call_med = &call->media[med_idx];
 
     /* Verify if the stream is transmitting video */
-    if (call_med->type != PJMEDIA_TYPE_VIDEO ||
+    if (call_med->type != PJMEDIA_TYPE_VIDEO || 
 	(enable && (call_med->dir & PJMEDIA_DIR_ENCODING) == 0))
     {
 	return PJ_EINVAL;
@@ -2002,7 +2024,7 @@ static pj_status_t call_send_vid_keyframe(pjsua_call *call,
     /* Verify and normalize media index */
     if (med_idx == -1) {
 	int first_active;
-
+	
 	call_get_vid_strm_info(call, &first_active, NULL, NULL, NULL);
 	if (first_active == -1)
 	    return PJ_ENOTFOUND;
@@ -2029,6 +2051,7 @@ PJ_DEF(pj_status_t) pjsua_call_set_vid_strm (
 				const pjsua_call_vid_strm_op_param *param)
 {
     pjsua_call *call;
+    pjsip_dialog *dlg = NULL;
     pjsua_call_vid_strm_op_param param_;
     pj_status_t status;
 
@@ -2040,9 +2063,9 @@ PJ_DEF(pj_status_t) pjsua_call_set_vid_strm (
 	      call_id, op));
     pj_log_push_indent();
 
-    PJSUA_LOCK();
-
-    call = &pjsua_var.calls[call_id];
+    status = acquire_call("pjsua_call_set_vid_strm()", call_id, &call, &dlg);
+    if (status != PJ_SUCCESS)
+	goto on_return;
 
     if (param) {
 	param_ = *param;
@@ -2056,7 +2079,7 @@ PJ_DEF(pj_status_t) pjsua_call_set_vid_strm (
     if (param_.cap_dev == PJMEDIA_VID_DEFAULT_CAPTURE_DEV) {
 	pjsua_acc_config *acc_cfg = &pjsua_var.acc[call->acc_id].cfg;
 	param_.cap_dev = acc_cfg->vid_cap_dev;
-
+	
 	/* If the account default video capture device is
 	 * PJMEDIA_VID_DEFAULT_CAPTURE_DEV, replace it with
 	 * global default video capture device.
@@ -2097,9 +2120,9 @@ PJ_DEF(pj_status_t) pjsua_call_set_vid_strm (
 	break;
     }
 
-    PJSUA_UNLOCK();
+on_return:
+    if (dlg) pjsip_dlg_dec_lock(dlg);
     pj_log_pop_indent();
-
     return status;
 }
 
