@@ -32,14 +32,15 @@
 #ifndef ACCOUNT_H
 #define ACCOUNT_H
 
-#include <string>
-#include <vector>
-
 #include "noncopyable.h"
 #include "config/sfl_config.h"
 #include "config/serializable.h"
 #include "registration_states.h"
 
+#include <functional>
+#include <string>
+#include <vector>
+
 class Account;
 class VoIPLink;
 
@@ -97,7 +98,7 @@ class Account : public Serializable {
          * Unregister the underlying VoIPLink. Stop the event listener.
          * This should update the getRegistrationState() return value.
          */
-        virtual void unregisterVoIPLink() = 0;
+        virtual void unregisterVoIPLink(std::function<void(bool)> cb = std::function<void(bool)>()) = 0;
 
         /**
          * Tell if the account is enable or not.
@@ -116,7 +117,7 @@ class Account : public Serializable {
          * Set the registration state of the specified link
          * @param state	The registration state of underlying VoIPLink
          */
-        void setRegistrationState(const RegistrationState &state);
+        void setRegistrationState(RegistrationState state);
 
         /* They should be treated like macro definitions by the C++ compiler */
         std::string getUsername() const {
@@ -215,6 +216,7 @@ class Account : public Serializable {
         static const char * const VIDEO_CODECS_KEY;
         static const char * const RINGTONE_PATH_KEY;
         static const char * const RINGTONE_ENABLED_KEY;
+        static const char * const VIDEO_ENABLED_KEY;
         static const char * const DISPLAY_NAME_KEY;
         static const char * const ALIAS_KEY;
         static const char * const TYPE_KEY;
@@ -227,6 +229,7 @@ class Account : public Serializable {
         static const char * const ACCOUNT_AUTOANSWER_KEY;
         static const char * const MAILBOX_KEY;
         static const char * const USER_AGENT_KEY;
+        static const char * const HAS_CUSTOM_USER_AGENT_KEY;
         static const char * const DEFAULT_USER_AGENT;
         static const char * const PRESENCE_MODULE_ENABLED_KEY;
 
@@ -289,6 +292,12 @@ class Account : public Serializable {
          */
         std::string ringtonePath_;
 
+        /**
+         * Allows user to temporarily disable video calling
+         */
+
+        bool videoEnabled_ = true;
+
         /**
          * Play ringtone when receiving a call
          */
@@ -304,6 +313,9 @@ class Account : public Serializable {
          */
         std::string userAgent_;
 
+        //  true if user has overridden default
+        bool hasCustomUserAgent_;
+
         /**
              * Account mail box
          */
