@@ -31,18 +31,13 @@
  */
 
 #include "alsalayer.h"
-#include "audio/dcblocker.h"
-#include "eventthread.h"
-#include "audio/samplerateconverter.h"
 #include "logger.h"
 #include "manager.h"
 #include "noncopyable.h"
 #include "client/configurationmanager.h"
-#include <ctime>
 
-#define SFL_ALSA_PERIOD_SIZE 160
-#define SFL_ALSA_NB_PERIOD 8
-#define SFL_ALSA_BUFFER_SIZE SFL_ALSA_PERIOD_SIZE * SFL_ALSA_NB_PERIOD
+#include <thread>
+#include <atomic>
 
 class AlsaThread {
     public:
@@ -54,16 +49,15 @@ class AlsaThread {
 
     private:
         void run();
-        static void *runCallback(void *context);
 
         NON_COPYABLE(AlsaThread);
-        pthread_t thread_;
+        std::thread thread_;
         AlsaLayer* alsa_;
-        bool running_;
+        std::atomic<bool> running_;
 };
 
 AlsaThread::AlsaThread(AlsaLayer *alsa)
-    : thread_(0), alsa_(alsa), running_(false)
+    : thread_(), alsa_(alsa), running_(false)
 {}
 
 bool AlsaThread::isRunning() const
@@ -74,23 +68,14 @@ bool AlsaThread::isRunning() const
 AlsaThread::~AlsaThread()
 {
     running_ = false;
-
-    if (thread_)
-        pthread_join(thread_, NULL);
+    if (thread_.joinable())
+        thread_.join();
 }
 
 void AlsaThread::start()
 {
     running_ = true;
-    pthread_create(&thread_, NULL, &runCallback, this);
-}
-
-void *
-AlsaThread::runCallback(void *data)
-{
-    AlsaThread *context = static_cast<AlsaThread*>(data);
-    context->run();
-    return NULL;
+    thread_ = std::thread(&AlsaThread::run, this);
 }
 
 void AlsaThread::initAudioLayer(void)
@@ -127,6 +112,8 @@ void AlsaThread::initAudioLayer(void)
                 Manager::instance().getClient()->getConfigurationManager()->errorAlert(ALSA_PLAYBACK_DEVICE);
     }
 
+    alsa_->hardwareFormatAvailable(alsa_->getFormat());
+
     alsa_->prepareCaptureStream();
     alsa_->preparePlaybackStream();
 
@@ -147,7 +134,6 @@ void AlsaThread::run()
 
     while (alsa_->isStarted_ and running_) {
         alsa_->audioCallback();
-        usleep(20000); // 20 ms
     }
 }
 
@@ -156,19 +142,21 @@ AlsaLayer::AlsaLayer(const AudioPreference &pref)
     , indexIn_(pref.getAlsaCardin())
     , indexOut_(pref.getAlsaCardout())
     , indexRing_(pref.getAlsaCardring())
-    , watchdogTotalCount_(0)
-    , watchdogTotalErr_(0)
-    , playbackHandle_(NULL)
-    , ringtoneHandle_(NULL)
-    , captureHandle_(NULL)
+    , playbackHandle_(nullptr)
+    , ringtoneHandle_(nullptr)
+    , captureHandle_(nullptr)
     , audioPlugin_(pref.getAlsaPlugin())
+    , playbackBuff_(0, audioFormat_)
+    , captureBuff_(0, audioFormat_)
+    , playbackIBuff_(1024)
+    , captureIBuff_(1024)
     , is_playback_prepared_(false)
     , is_capture_prepared_(false)
     , is_playback_running_(false)
     , is_capture_running_(false)
     , is_playback_open_(false)
     , is_capture_open_(false)
-    , audioThread_(NULL)
+    , audioThread_(nullptr)
 {}
 
 AlsaLayer::~AlsaLayer()
@@ -184,6 +172,8 @@ AlsaLayer::~AlsaLayer()
 // Retry approach taken from pa_linux_alsa.c, part of PortAudio
 bool AlsaLayer::openDevice(snd_pcm_t **pcm, const std::string &dev, snd_pcm_stream_t stream)
 {
+    DEBUG("Alsa: Opening %s",  dev.c_str());
+
     static const int MAX_RETRIES = 100;
     int err = snd_pcm_open(pcm, dev.c_str(), stream, 0);
 
@@ -339,6 +329,10 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle)
     snd_pcm_hw_params_t *hwparams;
     snd_pcm_hw_params_alloca(&hwparams);
 
+    const unsigned SFL_ALSA_PERIOD_SIZE = 160;
+    const unsigned SFL_ALSA_NB_PERIOD = 8;
+    const unsigned SFL_ALSA_BUFFER_SIZE = SFL_ALSA_PERIOD_SIZE * SFL_ALSA_NB_PERIOD;
+
     snd_pcm_uframes_t period_size = SFL_ALSA_PERIOD_SIZE;
     snd_pcm_uframes_t buffer_size = SFL_ALSA_BUFFER_SIZE;
     unsigned int periods = SFL_ALSA_NB_PERIOD;
@@ -350,15 +344,21 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle)
 
 #define HW pcm_handle, hwparams /* hardware parameters */
     TRY(snd_pcm_hw_params_any(HW), "hwparams init");
+
     TRY(snd_pcm_hw_params_set_access(HW, SND_PCM_ACCESS_RW_INTERLEAVED), "access type");
     TRY(snd_pcm_hw_params_set_format(HW, SND_PCM_FORMAT_S16_LE), "sample format");
-    TRY(snd_pcm_hw_params_set_rate_near(HW, &sampleRate_, NULL), "sample rate");
-    TRY(snd_pcm_hw_params_set_channels(HW, 1), "channel count");
+
+    TRY(snd_pcm_hw_params_set_rate_resample(HW, 0), "hardware sample rate"); /* prevent software resampling */
+    TRY(snd_pcm_hw_params_set_rate_near(HW, &audioFormat_.sample_rate, nullptr), "sample rate");
+
+    // TODO: use snd_pcm_query_chmaps or similar to get hardware channel num
+    audioFormat_.nb_channels = 2;
+    TRY(snd_pcm_hw_params_set_channels_near(HW, &audioFormat_.nb_channels), "channel count");
 
     snd_pcm_hw_params_get_buffer_size_min(hwparams, &buffer_size_min);
     snd_pcm_hw_params_get_buffer_size_max(hwparams, &buffer_size_max);
-    snd_pcm_hw_params_get_period_size_min(hwparams, &period_size_min, NULL);
-    snd_pcm_hw_params_get_period_size_max(hwparams, &period_size_max, NULL);
+    snd_pcm_hw_params_get_period_size_min(hwparams, &period_size_min, nullptr);
+    snd_pcm_hw_params_get_period_size_max(hwparams, &period_size_max, nullptr);
     DEBUG("Buffer size range from %lu to %lu", buffer_size_min, buffer_size_max);
     DEBUG("Period size range from %lu to %lu", period_size_min, period_size_max);
     buffer_size = buffer_size > buffer_size_max ? buffer_size_max : buffer_size;
@@ -367,12 +367,14 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle)
     period_size = period_size < period_size_min ? period_size_min : period_size;
 
     TRY(snd_pcm_hw_params_set_buffer_size_near(HW, &buffer_size), "Unable to set buffer size for playback");
-    TRY(snd_pcm_hw_params_set_period_size_near(HW, &period_size, NULL), "Unable to set period size for playback");
-    TRY(snd_pcm_hw_params_set_periods_near(HW, &periods, NULL), "Unable to set number of periods for playback");
+    TRY(snd_pcm_hw_params_set_period_size_near(HW, &period_size, nullptr), "Unable to set period size for playback");
+    TRY(snd_pcm_hw_params_set_periods_near(HW, &periods, nullptr), "Unable to set number of periods for playback");
     TRY(snd_pcm_hw_params(HW), "hwparams");
 
     snd_pcm_hw_params_get_buffer_size(hwparams, &buffer_size);
-    snd_pcm_hw_params_get_period_size(hwparams, &period_size, NULL);
+    snd_pcm_hw_params_get_period_size(hwparams, &period_size, nullptr);
+    snd_pcm_hw_params_get_rate(hwparams, &audioFormat_.sample_rate, nullptr);
+    snd_pcm_hw_params_get_channels(hwparams, &audioFormat_.nb_channels);
     DEBUG("Was set period_size = %lu", period_size);
     DEBUG("Was set buffer_size = %lu", buffer_size);
 
@@ -383,9 +385,9 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle)
 
 #undef HW
 
-    DEBUG("%s using sampling rate %dHz",
+    DEBUG("%s using format %s",
           (snd_pcm_stream(pcm_handle) == SND_PCM_STREAM_PLAYBACK) ? "playback" : "capture",
-          sampleRate_);
+          audioFormat_.toString().c_str() );
 
     snd_pcm_sw_params_t *swparams = NULL;
     snd_pcm_sw_params_alloca(&swparams);
@@ -404,16 +406,13 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle)
 // TODO first frame causes broken pipe (underrun) because not enough data is sent
 // we should wait until the handle is ready
 void
-AlsaLayer::write(void* buffer, int length, snd_pcm_t * handle)
+AlsaLayer::write(SFLAudioSample* buffer, int frames, snd_pcm_t * handle)
 {
     // Skip empty buffers
-    if (!length)
+    if (!frames)
         return;
 
-    snd_pcm_uframes_t frames = snd_pcm_bytes_to_frames(handle, length);
-    watchdogTotalCount_++;
-
-    int err = snd_pcm_writei(handle, buffer, frames);
+    int err = snd_pcm_writei(handle, (const void*)buffer, frames);
 
     if (err < 0)
         snd_pcm_recover(handle, err, 0);
@@ -421,8 +420,6 @@ AlsaLayer::write(void* buffer, int length, snd_pcm_t * handle)
     if (err >= 0)
         return;
 
-    watchdogTotalErr_++;
-
     switch (err) {
 
         case -EPIPE:
@@ -438,7 +435,7 @@ AlsaLayer::write(void* buffer, int length, snd_pcm_t * handle)
                     startPlaybackStream();
                 }
 
-            ALSA_CALL(snd_pcm_writei(handle, buffer, frames), "XRUN handling failed");
+            ALSA_CALL(snd_pcm_writei(handle, (const void*)buffer, frames), "XRUN handling failed");
             break;
         }
 
@@ -467,31 +464,20 @@ AlsaLayer::write(void* buffer, int length, snd_pcm_t * handle)
             stopPlaybackStream();
             break;
     }
-
-    // Detect when something is going wrong. This can be caused by alsa bugs or
-    // faulty encoder on the other side
-    // TODO do something useful instead of just warning and flushing buffers
-    if (watchdogTotalErr_ > 0 && watchdogTotalCount_ / watchdogTotalErr_ >= 4 and watchdogTotalCount_ > 50) {
-        ERROR("%d errors out of %d frames", watchdogTotalErr_, watchdogTotalCount_);
-        flushUrgent();
-        flushMain();
-    }
 }
 
 int
-AlsaLayer::read(void* buffer, int toCopy)
+AlsaLayer::read(SFLAudioSample* buffer, int frames)
 {
     if (snd_pcm_state(captureHandle_) == SND_PCM_STATE_XRUN) {
         prepareCaptureStream();
         startCaptureStream();
     }
 
-    snd_pcm_uframes_t frames = snd_pcm_bytes_to_frames(captureHandle_, toCopy);
-
-    int err = snd_pcm_readi(captureHandle_, buffer, frames);
+    int err = snd_pcm_readi(captureHandle_, (void*)buffer, frames);
 
     if (err >= 0)
-        return snd_pcm_frames_to_bytes(captureHandle_, frames);
+        return err;
 
     switch (err) {
         case -EPIPE:
@@ -535,8 +521,8 @@ AlsaLayer::buildDeviceTopo(const std::string &plugin, int card)
     return pcm + ss.str();
 }
 
-namespace {
-bool safeUpdate(snd_pcm_t *handle, int &samples)
+static bool
+safeUpdate(snd_pcm_t *handle, int &samples)
 {
     samples = snd_pcm_avail_update(handle);
 
@@ -552,7 +538,7 @@ bool safeUpdate(snd_pcm_t *handle, int &samples)
     return true;
 }
 
-std::vector<std::string>
+static std::vector<std::string>
 getValues(const std::vector<HwIDPair> &deviceMap)
 {
     std::vector<std::string> audioDeviceList;
@@ -562,7 +548,6 @@ getValues(const std::vector<HwIDPair> &deviceMap)
 
     return audioDeviceList;
 }
-}
 
 std::vector<std::string>
 AlsaLayer::getCaptureDeviceList() const
@@ -599,7 +584,7 @@ AlsaLayer::getAudioDeviceIndexMap(bool getCapture) const
 
         if (snd_ctl_open(&handle, name.c_str(), 0) == 0) {
             if (snd_ctl_card_info(handle, info) == 0) {
-                snd_pcm_info_set_device(pcminfo , 0);
+                snd_pcm_info_set_device(pcminfo, 0);
                 snd_pcm_info_set_stream(pcminfo, getCapture ? SND_PCM_STREAM_CAPTURE : SND_PCM_STREAM_PLAYBACK);
 
                 int err;
@@ -618,7 +603,7 @@ AlsaLayer::getAudioDeviceIndexMap(bool getCapture) const
                     description.append(snd_pcm_info_get_name(pcminfo));
 
                     // The number of the sound card is associated with a string description
-                    audioDevice.push_back(HwIDPair(numCard , description));
+                    audioDevice.push_back(HwIDPair(numCard, description));
                 }
             }
 
@@ -632,7 +617,7 @@ AlsaLayer::getAudioDeviceIndexMap(bool getCapture) const
 
 
 bool
-AlsaLayer::soundCardIndexExists(int card, PCMType stream)
+AlsaLayer::soundCardIndexExists(int card, DeviceType stream)
 {
     snd_pcm_info_t *pcminfo;
     snd_pcm_info_alloca(&pcminfo);
@@ -646,23 +631,18 @@ AlsaLayer::soundCardIndexExists(int card, PCMType stream)
     if (snd_ctl_open(&handle, name.c_str(), 0) != 0)
         return false;
 
-    snd_pcm_info_set_stream(pcminfo , (stream == SFL_PCM_PLAYBACK) ? SND_PCM_STREAM_PLAYBACK : SND_PCM_STREAM_CAPTURE);
-    bool ret = snd_ctl_pcm_info(handle , pcminfo) >= 0;
+    snd_pcm_info_set_stream(pcminfo, stream == DeviceType::PLAYBACK ?  SND_PCM_STREAM_PLAYBACK : SND_PCM_STREAM_CAPTURE);
+    bool ret = snd_ctl_pcm_info(handle, pcminfo) >= 0;
     snd_ctl_close(handle);
     return ret;
 }
 
 int
-AlsaLayer::getAudioDeviceIndex(const std::string &description) const
+AlsaLayer::getAudioDeviceIndex(const std::string &description, DeviceType type) const
 {
-    std::vector<HwIDPair> captureDevice(getAudioDeviceIndexMap(true));
-    std::vector<HwIDPair> playbackDevice(getAudioDeviceIndexMap(false));
+    std::vector<HwIDPair> devices = getAudioDeviceIndexMap(type == DeviceType::CAPTURE);
 
-    std::vector<HwIDPair> audioDeviceIndexMap;
-    audioDeviceIndexMap.insert(audioDeviceIndexMap.end(), captureDevice.begin(), captureDevice.end());
-    audioDeviceIndexMap.insert(audioDeviceIndexMap.end(), playbackDevice.begin(), playbackDevice.end());
-
-    for (const auto & dev : audioDeviceIndexMap)
+    for (const auto & dev : devices)
         if (dev.second == description)
             return dev.first;
 
@@ -671,117 +651,120 @@ AlsaLayer::getAudioDeviceIndex(const std::string &description) const
 }
 
 std::string
-AlsaLayer::getAudioDeviceName(int index, PCMType type) const
+AlsaLayer::getAudioDeviceName(int index, DeviceType type) const
 {
     // a bit ugly and wrong.. i do not know how to implement it better in alsalayer.
     // in addition, for now it is used in pulselayer only due to alsa and pulse layers api differences.
     // but after some tweaking in alsalayer, it could be used in it too.
     switch (type) {
-        case SFL_PCM_PLAYBACK:
-        case SFL_PCM_RINGTONE:
+        case DeviceType::PLAYBACK:
+        case DeviceType::RINGTONE:
             return getPlaybackDeviceList().at(index);
 
-        case SFL_PCM_CAPTURE:
+        case DeviceType::CAPTURE:
             return getCaptureDeviceList().at(index);
-
         default:
-            ERROR("Unexpected type %d", type);
+            // Should never happen
+            ERROR("Unexpected type");
             return "";
     }
 }
 
 void AlsaLayer::capture()
 {
-    unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
-    bool resample = sampleRate_ != mainBufferSampleRate;
+    AudioFormat mainBufferFormat = Manager::instance().getMainBuffer().getInternalAudioFormat();
 
-    int toGetSamples = snd_pcm_avail_update(captureHandle_);
+    int toGetFrames = snd_pcm_avail_update(captureHandle_);
 
-    if (toGetSamples < 0)
-        ERROR("Audio: Mic error: %s", snd_strerror(toGetSamples));
+    if (toGetFrames < 0)
+        ERROR("Audio: Mic error: %s", snd_strerror(toGetFrames));
 
-    if (toGetSamples <= 0)
+    if (toGetFrames <= 0)
         return;
 
     const int framesPerBufferAlsa = 2048;
-    toGetSamples = std::min(framesPerBufferAlsa, toGetSamples);
+    toGetFrames = std::min(framesPerBufferAlsa, toGetFrames);
+    captureIBuff_.resize(toGetFrames * audioFormat_.nb_channels);
 
-    AudioBuffer in(toGetSamples, 1, sampleRate_);
-
-    // TODO: handle ALSA multichannel capture
-    const int toGetBytes = in.frames() * sizeof(SFLAudioSample);
-    SFLAudioSample * const in_ptr = in.getChannel(0)->data();
-
-    if (read(in_ptr, toGetBytes) != toGetBytes) {
+    if (read(captureIBuff_.data(), toGetFrames) != toGetFrames) {
         ERROR("ALSA MIC : Couldn't read!");
         return;
     }
 
-    in.applyGain(isCaptureMuted_ ? 0.0 : captureGain_);
+    captureBuff_.deinterleave(captureIBuff_, audioFormat_);
+    captureBuff_.applyGain(isCaptureMuted_ ? 0.0 : captureGain_);
 
-    if (resample) {
-        int outSamples = toGetSamples * (static_cast<double>(sampleRate_) / mainBufferSampleRate);
-        AudioBuffer rsmpl_out(outSamples, 1, mainBufferSampleRate);
-        converter_.resample(in, rsmpl_out);
-        dcblocker_.process(rsmpl_out);
-        Manager::instance().getMainBuffer().putData(rsmpl_out, MainBuffer::DEFAULT_ID);
+    if (audioFormat_.nb_channels != mainBufferFormat.nb_channels) {
+        captureBuff_.setChannelNum(mainBufferFormat.nb_channels, true);
+    }
+    if (audioFormat_.sample_rate != mainBufferFormat.sample_rate) {
+        int outFrames = toGetFrames * (static_cast<double>(audioFormat_.sample_rate) / mainBufferFormat.sample_rate);
+        AudioBuffer rsmpl_in(outFrames, mainBufferFormat);
+        resampler_.resample(captureBuff_, rsmpl_in);
+        dcblocker_.process(rsmpl_in);
+        Manager::instance().getMainBuffer().putData(rsmpl_in, MainBuffer::DEFAULT_ID);
     } else {
-        dcblocker_.process(in);
-        Manager::instance().getMainBuffer().putData(in, MainBuffer::DEFAULT_ID);
+        dcblocker_.process(captureBuff_);
+        Manager::instance().getMainBuffer().putData(captureBuff_, MainBuffer::DEFAULT_ID);
     }
 }
 
-void AlsaLayer::playback(int maxSamples)
+void AlsaLayer::playback(int maxFrames)
 {
-    size_t bytesToGet = Manager::instance().getMainBuffer().availableForGet(MainBuffer::DEFAULT_ID);
-
-    const size_t bytesToPut = maxSamples * sizeof(SFLAudioSample);
+    unsigned framesToGet = Manager::instance().getMainBuffer().availableForGet(MainBuffer::DEFAULT_ID);
 
     // no audio available, play tone or silence
-    if (bytesToGet <= 0) {
+    if (framesToGet <= 0) {
         // FIXME: not thread safe! we only lock the mutex when we get the
         // pointer, we have no guarantee that it will stay safe to use
         AudioLoop *tone = Manager::instance().getTelephoneTone();
         AudioLoop *file_tone = Manager::instance().getTelephoneFile();
 
-        AudioBuffer out(maxSamples, 1, sampleRate_);
+        playbackBuff_.setFormat(audioFormat_);
+        playbackBuff_.resize(maxFrames);
 
         if (tone)
-            tone->getNext(out, playbackGain_);
+            tone->getNext(playbackBuff_, playbackGain_);
         else if (file_tone && !ringtoneHandle_)
-            file_tone->getNext(out, playbackGain_);
+            file_tone->getNext(playbackBuff_, playbackGain_);
+        else
+            playbackBuff_.reset();
 
-        write(out.getChannel(0)->data(), bytesToPut, playbackHandle_);
+        playbackBuff_.interleave(playbackIBuff_);
+        write(playbackIBuff_.data(), playbackBuff_.frames(), playbackHandle_);
     } else {
         // play the regular sound samples
-
-        const size_t mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
-        const bool resample = sampleRate_ != mainBufferSampleRate;
+        const AudioFormat mainBufferFormat = Manager::instance().getMainBuffer().getInternalAudioFormat();
+        const bool resample = audioFormat_.sample_rate != mainBufferFormat.sample_rate;
 
         double resampleFactor = 1.0;
-        size_t maxNbBytesToGet = bytesToPut;
+        unsigned maxNbFramesToGet = maxFrames;
 
         if (resample) {
-            resampleFactor = static_cast<double>(sampleRate_) / mainBufferSampleRate;
-            maxNbBytesToGet = bytesToGet / resampleFactor;
+            resampleFactor = static_cast<double>(audioFormat_.sample_rate) / mainBufferFormat.sample_rate;
+            maxNbFramesToGet = maxFrames / resampleFactor;
         }
 
-        bytesToGet = std::min(maxNbBytesToGet, bytesToGet);
+        framesToGet = std::min(framesToGet, maxNbFramesToGet);
 
-        const size_t samplesToGet = bytesToGet / sizeof(SFLAudioSample);
-        AudioBuffer out(samplesToGet, 1, mainBufferSampleRate);
+        playbackBuff_.setFormat(mainBufferFormat);
+        playbackBuff_.resize(framesToGet);
 
-        Manager::instance().getMainBuffer().getData(out, MainBuffer::DEFAULT_ID);
-        out.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
+        Manager::instance().getMainBuffer().getData(playbackBuff_, MainBuffer::DEFAULT_ID);
+        playbackBuff_.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
 
+        if (audioFormat_.nb_channels != mainBufferFormat.nb_channels) {
+            playbackBuff_.setChannelNum(audioFormat_.nb_channels, true);
+        }
         if (resample) {
-            const size_t outSamples = samplesToGet * resampleFactor;
-            const size_t outBytes = outSamples * sizeof(SFLAudioSample);
-            AudioBuffer rsmpl_out(outSamples, 1, sampleRate_);
-            converter_.resample(out, rsmpl_out);
-            write(rsmpl_out.getChannel(0)->data(), outBytes, playbackHandle_);
+            const size_t outFrames = framesToGet * resampleFactor;
+            AudioBuffer rsmpl_out(outFrames, audioFormat_);
+            resampler_.resample(playbackBuff_, rsmpl_out);
+            rsmpl_out.interleave(playbackIBuff_);
+            write(playbackIBuff_.data(), outFrames, playbackHandle_);
         } else {
-            write(out.getChannel(0)->data(), bytesToGet, playbackHandle_);
+            playbackBuff_.interleave(playbackIBuff_);
+            write(playbackIBuff_.data(), framesToGet, playbackHandle_);
         }
     }
 }
@@ -795,47 +778,46 @@ void AlsaLayer::audioCallback()
 
     snd_pcm_wait(playbackHandle_, 20);
 
-    int playbackAvailSmpl = 0;
+    int playbackAvailFrames = 0;
 
-    if (not safeUpdate(playbackHandle_, playbackAvailSmpl))
+    if (not safeUpdate(playbackHandle_, playbackAvailFrames))
         return;
 
-    unsigned samplesToGet = urgentRingBuffer_.availableForGet(MainBuffer::DEFAULT_ID);
+    unsigned framesToGet = urgentRingBuffer_.availableForGet(MainBuffer::DEFAULT_ID);
 
-    if (samplesToGet > 0) {
+    if (framesToGet > 0) {
         // Urgent data (dtmf, incoming call signal) come first.
-        samplesToGet = std::min(samplesToGet, (unsigned)playbackAvailSmpl);
-        // FIXME: should this sample rate really be hardcoded?
-        AudioBuffer out(samplesToGet, 1, 8000);
-        urgentRingBuffer_.get(out, MainBuffer::DEFAULT_ID);
-        out.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
-
-        write(out.getChannel(0)->data(), samplesToGet * sizeof(SFLAudioSample), playbackHandle_);
-        // Consume the regular one as well (same amount of bytes)
-        Manager::instance().getMainBuffer().discard(samplesToGet, MainBuffer::DEFAULT_ID);
+        framesToGet = std::min(framesToGet, (unsigned)playbackAvailFrames);
+        playbackBuff_.setFormat(audioFormat_);
+        playbackBuff_.resize(framesToGet);
+        urgentRingBuffer_.get(playbackBuff_, MainBuffer::DEFAULT_ID);
+        playbackBuff_.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
+        playbackBuff_.interleave(playbackIBuff_);
+        write(playbackIBuff_.data(), framesToGet, playbackHandle_);
+        // Consume the regular one as well (same amount of frames)
+        Manager::instance().getMainBuffer().discard(framesToGet, MainBuffer::DEFAULT_ID);
     } else {
         // regular audio data
-        playback(playbackAvailSmpl);
+        playback(playbackAvailFrames);
     }
 
     if (ringtoneHandle_) {
         AudioLoop *file_tone = Manager::instance().getTelephoneFile();
-        int ringtoneAvailSmpl = 0;
+        int ringtoneAvailFrames = 0;
 
-        if (not safeUpdate(ringtoneHandle_, ringtoneAvailSmpl))
+        if (not safeUpdate(ringtoneHandle_, ringtoneAvailFrames))
             return;
 
-        int ringtoneAvailBytes = ringtoneAvailSmpl * sizeof(SFLAudioSample);
-
-        // FIXME: should this sample rate really be hardcoded?
-        AudioBuffer out(ringtoneAvailSmpl, 1, 8000);
+        playbackBuff_.setFormat(audioFormat_);
+        playbackBuff_.resize(ringtoneAvailFrames);
 
         if (file_tone) {
             DEBUG("playback gain %d", playbackGain_);
-            file_tone->getNext(out, playbackGain_);
+            file_tone->getNext(playbackBuff_, playbackGain_);
         }
 
-        write(out.getChannel(0)->data(), ringtoneAvailBytes, ringtoneHandle_);
+        playbackBuff_.interleave(playbackIBuff_);
+        write(playbackIBuff_.data(), ringtoneAvailFrames, ringtoneHandle_);
     }
 
     // Additionally handle the mic's audio stream
@@ -843,18 +825,18 @@ void AlsaLayer::audioCallback()
         capture();
 }
 
-void AlsaLayer::updatePreference(AudioPreference &preference, int index, PCMType type)
+void AlsaLayer::updatePreference(AudioPreference &preference, int index, DeviceType type)
 {
     switch (type) {
-        case SFL_PCM_PLAYBACK:
+        case DeviceType::PLAYBACK:
             preference.setAlsaCardout(index);
             break;
 
-        case AudioLayer::SFL_PCM_CAPTURE:
+        case DeviceType::CAPTURE:
             preference.setAlsaCardin(index);
             break;
 
-        case AudioLayer::SFL_PCM_RINGTONE:
+        case DeviceType::RINGTONE:
             preference.setAlsaCardring(index);
             break;
 
