@@ -34,14 +34,16 @@
 #ifndef AUDIO_LAYER_H_
 #define AUDIO_LAYER_H_
 
-#include <mutex>
-#include <sys/time.h>
-#include <vector>
+
 #include "ringbuffer.h"
 #include "dcblocker.h"
-#include "samplerateconverter.h"
+#include "resampler.h"
 #include "noncopyable.h"
 
+#include <sys/time.h>
+#include <mutex>
+#include <vector>
+
 /**
  * @file  audiolayer.h
  * @brief Main sound class. Manages the data transfers between the application and the hardware.
@@ -54,18 +56,18 @@ namespace ost {
 class Time;
 }
 
+enum class DeviceType {
+    PLAYBACK,      /** To open playback device only */
+    CAPTURE,       /** To open capture device only */
+    RINGTONE       /** To open the ringtone device only */
+};
+
 class AudioLayer {
 
     private:
         NON_COPYABLE(AudioLayer);
 
     public:
-        enum PCMType {
-            SFL_PCM_BOTH = 0x0021,          /** To open both playback and capture devices */
-            SFL_PCM_PLAYBACK = 0x0022,      /** To open playback device only */
-            SFL_PCM_CAPTURE = 0x0023,       /** To open capture device only */
-            SFL_PCM_RINGTONE = 0x0024       /** To open the ringtone device only */
-        };
 
         AudioLayer(const AudioPreference &);
         virtual ~AudioLayer();
@@ -73,8 +75,8 @@ class AudioLayer {
         virtual std::vector<std::string> getCaptureDeviceList() const = 0;
         virtual std::vector<std::string> getPlaybackDeviceList() const = 0;
 
-        virtual int getAudioDeviceIndex(const std::string& name) const = 0;
-        virtual std::string getAudioDeviceName(int index, PCMType type) const = 0;
+        virtual int getAudioDeviceIndex(const std::string& name, DeviceType type) const = 0;
+        virtual std::string getAudioDeviceName(int index, DeviceType type) const = 0;
         virtual int getIndexCapture() const = 0;
         virtual int getIndexPlayback() const = 0;
         virtual int getIndexRingtone() const = 0;
@@ -175,7 +177,14 @@ class AudioLayer {
          *			    default: 44100 HZ
          */
         unsigned int getSampleRate() const {
-            return sampleRate_;
+            return audioFormat_.sample_rate;
+        }
+
+        /**
+         * Get the audio format of the layer (sample rate & channel number).
+         */
+        AudioFormat getFormat() const {
+            return audioFormat_;
         }
 
         /**
@@ -183,9 +192,13 @@ class AudioLayer {
          */
         void notifyIncomingCall();
 
-        virtual void updatePreference(AudioPreference &pref, int index, PCMType type) = 0;
+        virtual void updatePreference(AudioPreference &pref, int index, DeviceType type) = 0;
 
     protected:
+        /**
+         * Callback to be called by derived classes when the audio output is opened.
+         */
+        void hardwareFormatAvailable(AudioFormat playback);
 
         /**
          * True if capture is not to be used
@@ -212,17 +225,16 @@ class AudioLayer {
          */
         bool isStarted_;
 
+        /**
+         * Sample Rate SFLphone should send sound data to the sound card
+         */
+        AudioFormat audioFormat_;
+
         /**
          * Urgent ring buffer used for ringtones
          */
         RingBuffer urgentRingBuffer_;
 
-        /**
-         * Sample Rate SFLphone should send sound data to the sound card
-         * The value can be set in the user config file- now: 44100HZ
-         */
-        unsigned int sampleRate_;
-
         /**
          * Lock for the entire audio layer
          */
@@ -236,7 +248,7 @@ class AudioLayer {
         /**
          * Manage sampling rate conversion
          */
-        SamplerateConverter converter_;
+        Resampler resampler_;
 
     private:
 
