@@ -45,7 +45,7 @@
 #include "logger.h"
 
 AudioLoop::AudioLoop(unsigned int sampleRate) :
-    buffer_(new AudioBuffer(0, 1, sampleRate)), pos_(0)
+    buffer_(new AudioBuffer(0, AudioFormat(sampleRate, 1))), pos_(0)
 {}
 
 AudioLoop::~AudioLoop()
@@ -80,7 +80,7 @@ AudioLoop::getNext(AudioBuffer& output, double gain)
         return;
     }
 
-    while (total_samples > 0) {
+    while (total_samples != 0) {
         size_t samples = std::min(total_samples, buf_samples - pos);
 
         output.copy(*buffer_, samples, pos, output_pos);
