@@ -33,16 +33,17 @@
 #endif
 
 #include "audiorecord.h"
-#include <sndfile.hh>
-#include <unistd.h>
-#include <sstream> // for stringstream
-#include <algorithm>
-#include <cstdio>
 #include "logger.h"
 #include "fileutils.h"
 
-namespace {
-std::string
+#include <sndfile.hh>
+
+#include <algorithm>
+#include <sstream> // for stringstream
+#include <cstdio>
+#include <unistd.h>
+
+static std::string
 createFilename()
 {
     time_t rawtime = time(NULL);
@@ -86,12 +87,9 @@ createFilename()
     out << timeinfo->tm_sec;
     return out.str();
 }
-}
 
-
-AudioRecord::AudioRecord() : fileHandle_(0)
-    , channels_(1)
-    , sndSmplRate_(8000)
+AudioRecord::AudioRecord() : fileHandle_(nullptr)
+    , sndFormat_(AudioFormat::MONO)
     , recordingEnabled_(false)
     , filename_(createFilename())
     , savePath_()
@@ -104,12 +102,12 @@ AudioRecord::~AudioRecord()
     delete fileHandle_;
 }
 
-void AudioRecord::setSndSamplingRate(int smplRate)
+void AudioRecord::setSndFormat(AudioFormat format)
 {
-    sndSmplRate_ = smplRate;
+    sndFormat_ = format;
 }
 
-void AudioRecord::setRecordingOptions(int sndSmplRate, const std::string &path)
+void AudioRecord::setRecordingOptions(AudioFormat format, const std::string &path)
 {
     std::string filePath;
 
@@ -120,26 +118,23 @@ void AudioRecord::setRecordingOptions(int sndSmplRate, const std::string &path)
         filePath = path;
     }
 
-    channels_ = 1;
-    sndSmplRate_ = sndSmplRate;
+    sndFormat_ = format;
     savePath_ = (*filePath.rbegin() == DIR_SEPARATOR_CH) ? filePath : filePath + DIR_SEPARATOR_STR;
 }
 
-namespace {
-bool
+static bool
 nonFilenameCharacter(char c)
 {
     return not(std::isalnum(c) or c == '_' or c == '.');
 }
 
 // Replace any character that is inappropriate for a filename with '_'
-std::string
+static std::string
 sanitize(std::string s)
 {
     std::replace_if(s.begin(), s.end(), nonFilenameCharacter, '_');
     return s;
 }
-}
 
 void AudioRecord::initFilename(const std::string &peerNumber)
 {
@@ -166,7 +161,8 @@ bool AudioRecord::openFile()
     const bool doAppend = fileExists();
     const int access = doAppend ? SFM_RDWR : SFM_WRITE;
 
-    fileHandle_ = new SndfileHandle(savePath_.c_str(), access, SF_FORMAT_WAV | SF_FORMAT_PCM_16, channels_, sndSmplRate_);
+    DEBUG("Opening file %s with format %s", savePath_.c_str(), sndFormat_.toString().c_str());
+    fileHandle_ = new SndfileHandle(savePath_.c_str(), access, SF_FORMAT_WAV | SF_FORMAT_PCM_16, sndFormat_.nb_channels, sndFormat_.sample_rate);
 
     // check overloaded boolean operator
     if (!*fileHandle_) {
@@ -231,10 +227,9 @@ void AudioRecord::recData(AudioBuffer& buffer)
         return;
     }
 
-    const int nSamples = buffer.frames();
-
-    // FIXME: mono only
-    if (fileHandle_->write(buffer.getChannel(0)->data(), nSamples) != nSamples) {
+    auto interleaved = buffer.interleave();
+    const int nSamples = interleaved.size();
+    if (fileHandle_->write(interleaved.data(), nSamples) != nSamples) {
         WARN("Could not record data!");
     } else {
         fileHandle_->writeSync();
