@@ -28,11 +28,12 @@
  *  as that of the covered work.
  */
 
+#include "audio_rtp_factory.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "audio_rtp_factory.h"
 #if HAVE_ZRTP
 #include "audio_zrtp_session.h"
 #endif
@@ -48,11 +49,7 @@ namespace sfl {
 
 AudioRtpFactory::AudioRtpFactory(SIPCall *ca) : rtpSession_(),
     audioRtpThreadMutex_(), srtpEnabled_(false), helloHashEnabled_(false),
-    cachedLocalMasterKey_(MAX_MASTER_KEY_LENGTH),
-    cachedLocalMasterSalt_(MAX_MASTER_SALT_LENGTH),
-    cachedRemoteMasterKey_(MAX_MASTER_KEY_LENGTH),
-    cachedRemoteMasterSalt_(MAX_MASTER_SALT_LENGTH),
-    remoteOfferIsSet_(false), ca_(ca),
+    remoteOfferIsSet_(false), call_(ca),
     keyExchangeProtocol_(NONE)
 {
     // FIXME: workaround for uncatchable ost::Socket "exceptions"
@@ -66,7 +63,7 @@ void AudioRtpFactory::initConfig()
 {
     stop();
 
-    const std::string accountId(ca_->getAccountId());
+    const std::string accountId(call_->getAccountId());
 
     SIPAccount *account = Manager::instance().getSipAccount(accountId);
 
@@ -97,60 +94,28 @@ void AudioRtpFactory::initConfig()
     }
 }
 
-// XXX: This wrapper exists because CCRTP may directly call abort() upon socket failure
-//      when calling UDPSocket::UDPSocket
-namespace {
-    jmp_buf env;
-
-    // SIGABRT handler must not return or the program will terminate.
-    // Therefore we call longjmp.
-    void on_sigabrt(int /*signum*/)
-    {
-        signal(SIGABRT, SIG_DFL); // restore default SIGABRT handler
-        longjmp(env, 1);
-    }
-
-    AudioRtpSession *
-    callConstructor(const std::function<AudioRtpSession* ()> &func)
-    {
-        AudioRtpSession *session = nullptr;
-        if (setjmp(env) == 0) {
-            signal(SIGABRT, &on_sigabrt);
-            session = func();
-            signal(SIGABRT, SIG_DFL); // restore default SIGABRT handler
-        } else {
-            ERROR("aborted");
-            throw AudioRtpFactoryException("Socket failure");
-        }
-        return session;
-    }
-}
-
 void AudioRtpFactory::initSession()
 {
     std::lock_guard<std::mutex> lock(audioRtpThreadMutex_);
 
     if (srtpEnabled_) {
-        const std::string zidFilename(Manager::instance().voipPreferences.getZidFile());
 
         switch (keyExchangeProtocol_) {
 #if HAVE_ZRTP
-
             case ZRTP: {
-                const auto ctor = [&] () {return new AudioZrtpSession(*ca_, zidFilename);};
-                rtpSession_.reset(callConstructor(ctor));
+                const std::string zidFilename(Manager::instance().voipPreferences.getZidFile());
+                rtpSession_.reset(new AudioZrtpSession(*call_, zidFilename, call_->getLocalIp()));
 
                 // TODO: be careful with that. The hello hash is computed asynchronously. Maybe it's
                 // not even available at that point.
                 if (helloHashEnabled_)
-                    ca_->getLocalSDP()->setZrtpHash(static_cast<AudioZrtpSession *>(rtpSession_.get())->getHelloHash());
+                    call_->getLocalSDP()->setZrtpHash(static_cast<AudioZrtpSession *>(rtpSession_.get())->getHelloHash());
                 break;
             }
 #endif
 
             case SDES: {
-                const auto ctor = [&] () {return new AudioSrtpSession(*ca_);};
-                rtpSession_.reset(callConstructor(ctor));
+                rtpSession_.reset(new AudioSrtpSession(*call_));
                 break;
             }
 
@@ -158,8 +123,14 @@ void AudioRtpFactory::initSession()
                 throw UnsupportedRtpSessionType("Unsupported Rtp Session Exception Type!");
         }
     } else {
-        const auto ctor = [&] () {return new AudioSymmetricRtpSession(*ca_);};
-        rtpSession_.reset(callConstructor(ctor));
+#if HAVE_IPV6
+        if (call_->getLocalIp().isIpv6()) {
+            rtpSession_.reset(new AudioSymmetricRtpSessionIPv6(*call_));
+        } else
+#endif
+        {
+            rtpSession_.reset(new AudioSymmetricRtpSession(*call_));
+        }
     }
 }
 
@@ -187,25 +158,6 @@ void AudioRtpFactory::stop()
     rtpSession_.reset();
 }
 
-int AudioRtpFactory::getSessionMedia()
-{
-    std::lock_guard<std::mutex> lock(audioRtpThreadMutex_);
-    if (!rtpSession_)
-        throw AudioRtpFactoryException("RTP session was null when trying to get session media type");
-
-    return rtpSession_->getEncoderPayloadType();
-}
-
-std::string
-AudioRtpFactory::getCurrentAudioCodecNames()
-{
-    std::lock_guard<std::mutex> lock(audioRtpThreadMutex_);
-    if (!rtpSession_)
-        throw AudioRtpFactoryException("RTP session was null when trying to get session media type");
-
-    return rtpSession_->getCurrentAudioCodecNames();
-}
-
 void AudioRtpFactory::updateSessionMedia(const std::vector<AudioCodec*> &audioCodecs)
 {
     std::lock_guard<std::mutex> lock(audioRtpThreadMutex_);
@@ -243,7 +195,7 @@ void AudioRtpFactory::initLocalCryptoInfo()
         AudioSrtpSession *srtp = static_cast<AudioSrtpSession*>(rtpSession_.get());
         // the context is invalidated and deleted by the call to initLocalCryptoInfo
         srtp->initLocalCryptoInfo();
-        ca_->getLocalSDP()->setLocalSdpCrypto(srtp->getLocalCryptoInfo());
+        call_->getLocalSDP()->setLocalSdpCrypto(srtp->getLocalCryptoInfo());
     }
 }
 
@@ -255,7 +207,7 @@ void AudioRtpFactory::initLocalCryptoInfoOnOffHold()
         AudioSrtpSession *srtp = static_cast<AudioSrtpSession*>(rtpSession_.get());
         // the context is invalidated and deleted by the call to initLocalCryptoInfo
         srtp->initLocalCryptoInfoOnOffhold();
-        ca_->getLocalSDP()->setLocalSdpCrypto(srtp->getLocalCryptoInfo());
+        call_->getLocalSDP()->setLocalSdpCrypto(srtp->getLocalCryptoInfo());
     }
 }
 
@@ -268,7 +220,11 @@ void AudioRtpFactory::setRemoteCryptoInfo(SdesNegotiator& nego)
 
     if (keyExchangeProtocol_ == SDES) {
         AudioSrtpSession *srtp = static_cast<AudioSrtpSession *>(rtpSession_.get());
-        srtp->setRemoteCryptoInfo(nego);
+        try {
+            srtp->setRemoteCryptoInfo(nego);
+        } catch (const AudioSrtpException &e) {
+            throw AudioRtpFactoryException(e.what());
+        }
     } else {
         ERROR("Should not store remote crypto info for non-SDES sessions");
     }
@@ -291,18 +247,14 @@ void AudioRtpFactory::sendDtmfDigit(int digit)
 void AudioRtpFactory::saveLocalContext()
 {
     std::lock_guard<std::mutex> lock(audioRtpThreadMutex_);
-    if (rtpSession_ and keyExchangeProtocol_ == SDES) {
-        cachedLocalMasterKey_ = rtpSession_->getLocalMasterKey();
-        cachedLocalMasterSalt_ = rtpSession_->getLocalMasterSalt();
-    }
+    if (rtpSession_ and keyExchangeProtocol_ == SDES)
+        cachedAudioRtpState_.reset(rtpSession_->saveState());
 }
 
 void AudioRtpFactory::restoreLocalContext()
 {
     std::lock_guard<std::mutex> lock(audioRtpThreadMutex_);
-    if (rtpSession_ and keyExchangeProtocol_ == SDES) {
-        rtpSession_->setLocalMasterKey(cachedLocalMasterKey_);
-        rtpSession_->setLocalMasterSalt(cachedLocalMasterSalt_);
-    }
+    if (rtpSession_ and keyExchangeProtocol_ == SDES)
+        rtpSession_->restoreState(*cachedAudioRtpState_);
 }
 }
