@@ -31,7 +31,6 @@
 #ifndef __AUDIO_RTP_FACTORY_H__
 #define __AUDIO_RTP_FACTORY_H__
 
-#include <ccrtp/CryptoContext.h>
 #include <stdexcept>
 #include <mutex>
 #include <memory>
@@ -39,8 +38,6 @@
 #include "audio_srtp_session.h"
 #include "noncopyable.h"
 
-#include "sip/sdes_negotiator.h"
-
 class SdesNegotiator;
 class SIPCall;
 
@@ -93,11 +90,6 @@ class AudioRtpFactory {
          */
         void stop();
 
-        /**
-         * Return the RTP payload currently used for this session
-         */
-        int getSessionMedia();
-
         /**
          * Dynamically update session media
          */
@@ -152,13 +144,11 @@ class AudioRtpFactory {
 
         void restoreLocalContext();
 
-        std::string
-        getCurrentAudioCodecNames();
-
     private:
         NON_COPYABLE(AudioRtpFactory);
         enum KeyExchangeProtocol { NONE, SDES, ZRTP };
         std::unique_ptr<AudioRtpSession> rtpSession_;
+        std::unique_ptr<CachedAudioRtpState> cachedAudioRtpState_;
         std::mutex audioRtpThreadMutex_;
 
         // Field used when initializing audio rtp session
@@ -169,22 +159,10 @@ class AudioRtpFactory {
         // May be set manually or from config using initAudioRtpConfig
         bool helloHashEnabled_;
 
-        /** local master key for outgoing packet encryption **/
-        std::vector<uint8> cachedLocalMasterKey_;
-
-        /** local master salt for outgoing packet encryption **/
-        std::vector<uint8> cachedLocalMasterSalt_;
-
-        /** remote master key for incoming packet decryption **/
-        std::vector<uint8> cachedRemoteMasterKey_;
-
-        /** remote master salt for incoming packet decryption **/
-        std::vector<uint8> cachedRemoteMasterSalt_;
-
         /** Used to make sure remote crypto context not initialized twice. */
         bool remoteOfferIsSet_;
 
-        SIPCall *ca_;
+        SIPCall *call_;
         KeyExchangeProtocol keyExchangeProtocol_;
 };
 }
