@@ -34,21 +34,19 @@
 #ifndef AUDIO_SYMMETRIC_RTP_SESSION_H_
 #define AUDIO_SYMMETRIC_RTP_SESSION_H_
 
-#include <exception>
-#include <cassert>
-#include <cstddef>
-
 #include "global.h"
 #include "audio_rtp_session.h"
+#include "ip_utils.h"
 #include "noncopyable.h"
 
-using std::ptrdiff_t;
-
-
 #pragma GCC diagnostic ignored "-Weffc++"
 #include <ccrtp/rtp.h>
 #include <ccrtp/iqueue.h>
 
+#include <exception>
+#include <cassert>
+#include <cstddef>
+
 class SIPCall;
 
 namespace sfl {
@@ -60,6 +58,7 @@ class AudioSymmetricRtpSession : public ost::SymmetricRTPSession, public AudioRt
         * @param call The SIP call
         */
         AudioSymmetricRtpSession(SIPCall &call);
+        ~AudioSymmetricRtpSession();
 
         std::vector<long>
         getSocketDescriptors() const;
@@ -68,24 +67,48 @@ class AudioSymmetricRtpSession : public ost::SymmetricRTPSession, public AudioRt
             return AudioRtpSession::onRTPPacketRecv(pkt);
         }
 
-        virtual void setLocalMasterKey(const std::vector<uint8>& key UNUSED) {}
-
-        virtual void setLocalMasterSalt(const std::vector<uint8>& key UNUSED) {};
-
-        /** Not used for symmetric rtp session, return an empty vector */
-        virtual std::vector<uint8> getLocalMasterKey() const { std::vector<uint8> vec; return vec; }
-
-        /** Not used for symmetric rtp session, return an empty vector */
-        virtual std::vector<uint8> getLocalMasterSalt() const { std::vector<uint8> vec; return vec; }
-
     private:
         void onGotRR(ost::SyncSource& source, ost::RTCPCompoundHandler::RecvReport& RR, uint8 blocks);
         void onGotSR(ost::SyncSource& source, ost::RTCPCompoundHandler::SendReport& SR, uint8 blocks);
 
         NON_COPYABLE(AudioSymmetricRtpSession);
-        void startReceiveThread();
+
+        void startRTPLoop();
 };
 
+#if HAVE_IPV6
+
+class AudioSymmetricRtpSessionIPv6 : public ost::SymmetricRTPSessionIPV6, public AudioRtpSession {
+    public:
+        /**
+        * Constructor
+        * @param call The SIP call
+        */
+        AudioSymmetricRtpSessionIPv6(SIPCall &call);
+        ~AudioSymmetricRtpSessionIPv6();
+
+        std::vector<long>
+        getSocketDescriptors() const;
+
+        virtual bool onRTPPacketRecv(ost::IncomingRTPPkt& pkt) {
+            return AudioRtpSession::onRTPPacketRecv(pkt);
+        }
+
+    protected:
+        virtual size_t recvData(unsigned char* buffer, size_t len, ost::IPV4Host&, ost::tpport_t& port);
+        virtual size_t recvControl(unsigned char* buffer, size_t len, ost::IPV4Host&, ost::tpport_t& port);
+
+    private:
+        void onGotRR(ost::SyncSource& source, ost::RTCPCompoundHandler::RecvReport& RR, uint8 blocks);
+        void onGotSR(ost::SyncSource& source, ost::RTCPCompoundHandler::SendReport& SR, uint8 blocks);
+
+        NON_COPYABLE(AudioSymmetricRtpSessionIPv6);
+
+        void startRTPLoop();
+};
+
+#endif // HAVE_IPV6
+
 }
 #pragma GCC diagnostic warning "-Weffc++"
 #endif // AUDIO_SYMMETRIC_RTP_SESSION_H__
