@@ -41,15 +41,22 @@
 
 namespace sfl {
 
-AudioZrtpSession::AudioZrtpSession(SIPCall &call, const std::string &zidFilename) :
-    ost::SymmetricZRTPSession(ost::InetHostAddress(call.getLocalIp().c_str()), call.getLocalAudioPort())
+AudioZrtpSession::AudioZrtpSession(SIPCall &call, const std::string &zidFilename,
+                                   const std::string &localIP) :
+    ost::SymmetricZRTPSession(ost::InetHostAddress(localIP.c_str()), call.getLocalAudioPort())
     , AudioRtpSession(call, *this)
     , zidFilename_(zidFilename)
+#if HAVE_ZRTP_CONFIGURE
+    , zrtpConfigure_()
 {
+    setEnrollmentMode(false);
+    setSignSas(false);
+#else
+{
+#endif
     initializeZid();
     DEBUG("Setting new RTP session with destination %s:%d",
-          call_.getLocalIp().c_str(), call_.getLocalAudioPort());
-    audioRtpRecord_.callId_ = call_.getCallId();
+          localIP.c_str(), call_.getLocalAudioPort());
 }
 
 std::vector<long>
@@ -66,26 +73,30 @@ void AudioZrtpSession::initializeZid()
     if (zidFilename_.empty())
         throw ZrtpZidException("zid filename empty");
 
-    const std::string cache_home(XDG_CACHE_HOME);
-    std::string zidDirName;
-    std::string zidCompleteFilename;
-
-    if (not cache_home.empty()) {
-        zidDirName = cache_home;
-    } else {
-        zidDirName = fileutils::get_home_dir() + DIR_SEPARATOR_STR +
-                              ".cache" + DIR_SEPARATOR_STR + PACKAGE;
-    }
-
-    zidCompleteFilename = zidDirName + DIR_SEPARATOR_STR + zidFilename_;
+    const std::string zidDirName(fileutils::get_cache_dir());
+    const std::string zidCompleteFilename(zidDirName + DIR_SEPARATOR_STR + zidFilename_);
 
     fileutils::check_dir(zidDirName.c_str());
 
+#if HAVE_ZRTP_CONFIGURE
+    // workaround buggy libzrtpcpp that can't parse EC25, see:
+    // https://projects.savoirfairelinux.com/issues/40216
+    zrtpConfigure_.reset(new ZrtpConfigure);
+    // FIXME: perhaps this should be only done for libzrtpcpp < 3?
+    zrtpConfigure_->setMandatoryOnly();
+
+    if (initialize(zidCompleteFilename.c_str(), true, zrtpConfigure_.get()) >= 0) {
+        setEnableZrtp(true);
+        setUserCallback(new ZrtpSessionCallback(call_));
+        return;
+    }
+#else
     if (initialize(zidCompleteFilename.c_str()) >= 0) {
         setEnableZrtp(true);
         setUserCallback(new ZrtpSessionCallback(call_));
         return;
     }
+#endif
 
     DEBUG("Initialization from ZID file failed. Trying to remove...");
 
@@ -94,26 +105,6 @@ void AudioZrtpSession::initializeZid()
 
     if (initialize(zidCompleteFilename.c_str()) < 0)
         throw ZrtpZidException("zid initialization failed");
-
-    return;
-}
-
-void AudioZrtpSession::sendMicData()
-{
-    int compSize = processDataEncode();
-
-    // if no data return
-    if (compSize == 0)
-        return;
-
-    // Increment timestamp for outgoing packet
-    timestamp_ += timestampIncrement_;
-
-    // this step is only needed for ZRTP
-    queue_.putData(timestamp_, getMicDataEncoded(), compSize);
-
-    // putData puts the data on RTP queue, sendImmediate bypasses this queue
-    queue_.sendImmediate(timestamp_, getMicDataEncoded(), compSize);
 }
 
 int AudioZrtpSession::getIncrementForDTMF() const
@@ -121,9 +112,9 @@ int AudioZrtpSession::getIncrementForDTMF() const
     return 160;
 }
 
-void AudioZrtpSession::startReceiveThread()
+void AudioZrtpSession::startRTPLoop()
 {
-    ost::SymmetricZRTPSession::start();
+    ost::SymmetricZRTPSession::startRunning();
 }
 
 }
