@@ -37,8 +37,10 @@
 using std::ptrdiff_t;
 #include <ccrtp/rtp.h>
 #include <libzrtpcpp/zrtpccrtp.h>
-#include <libzrtpcpp/ZrtpQueue.h>
-#include <libzrtpcpp/ZrtpUserCallback.h>
+
+#if HAVE_ZRTP_CONFIGURE
+#include <memory>
+#endif
 
 #include "global.h"
 #include "audio_rtp_session.h"
@@ -58,7 +60,7 @@ class AudioZrtpSession :
     public ost::SymmetricZRTPSession,
     public AudioRtpSession {
     public:
-        AudioZrtpSession(SIPCall &call, const std::string& zidFilename);
+        AudioZrtpSession(SIPCall &call, const std::string& zidFilename, const std::string &localIP);
 
         std::vector<long>
         getSocketDescriptors() const;
@@ -67,21 +69,15 @@ class AudioZrtpSession :
             return AudioRtpSession::onRTPPacketRecv(pkt);
         }
 
-        virtual std::vector<uint8> getLocalMasterKey() const { std::vector<uint8> vec; return vec; }
-
-        virtual std::vector<uint8> getLocalMasterSalt() const { std::vector<uint8> vec; return vec; }
-
-        virtual void setLocalMasterKey(const std::vector<unsigned char>& vec UNUSED) {}
-
-        virtual void setLocalMasterSalt(const std::vector<unsigned char>& vec UNUSED) {}
-
     private:
         NON_COPYABLE(AudioZrtpSession);
 
-        void sendMicData();
         void initializeZid();
         std::string zidFilename_;
-        void startReceiveThread();
+#if HAVE_ZRTP_CONFIGURE
+        std::unique_ptr<ZrtpConfigure> zrtpConfigure_;
+#endif
+        void startRTPLoop();
         virtual int getIncrementForDTMF() const;
 };
 
