@@ -31,6 +31,7 @@
 
 #include "sfl_types.h"
 #include "audiocodec.h"
+#include "g711.h"
 
 class Alaw : public sfl::AudioCodec {
 
@@ -48,80 +49,31 @@ class Alaw : public sfl::AudioCodec {
             return new Alaw;
         }
 
-        int decode(SFLAudioSample *dst, unsigned char *src, size_t buf_size)
+        int decode(SFLAudioSample *pcm, unsigned char *data, size_t len)
         {
-            for (unsigned char* end = src + buf_size; src < end; ++src, ++dst)
-                *dst = ALawDecode(*src);
+            for (unsigned char* end = data + len; data < end; ++data, ++pcm)
+                *pcm = ALawDecode(*data);
 
-            return buf_size;
+            return len;
         }
 
-        int encode(unsigned char *dst, SFLAudioSample *src, size_t buf_size)
+        int encode(unsigned char *data, SFLAudioSample *pcm, size_t max_data_bytes)
         {
-            for (unsigned char *end = dst + buf_size; dst < end; ++src, ++dst)
-                *dst = ALawEncode(*src);
+            unsigned char *end = std::min(data + frameSize_, data + max_data_bytes);
+            unsigned char *tmp = data;
 
-            return buf_size;
+            for (; tmp < end; ++pcm, ++tmp)
+                *tmp = ALawEncode(*pcm);
+
+            return end - data;
         }
 
-        int ALawDecode(uint8_t alaw) {
-            alaw ^= 0x55;  // A-law has alternate bits inverted for transmission
-            uint8_t sign = alaw & 0x80;
-            int linear = alaw & 0x1f;
-            linear <<= 4;
-            linear += 8;  // Add a 'half' bit (0x08) to place PCM value in middle of range
-
-            alaw &= 0x7f;
-
-            if (alaw >= 0x20) {
-                linear |= 0x100;  // Put in MSB
-                uint8_t shift = (alaw >> 4) - 1;
-                linear <<= shift;
-            }
-
-            if (!sign)
-                return -linear;
-            else
-                return linear;
+        static SFLAudioSample ALawDecode(unsigned char alaw) {
+            return alaw_to_linear(alaw);
         }
 
-        uint8_t ALawEncode(SFLAudioSample pcm16) {
-            int p = pcm16;
-            uint8_t a;  // u-law value we are forming
-
-            if (p < 0) {
-                p = ~p;
-                a = 0x00; // sign = 0
-            } else {
-                //+ve value
-                a = 0x80; //sign = 1
-            }
-
-            //calculate segment and interval numbers
-            p >>= 4;
-
-            if (p >= 0x20) {
-                if (p >= 0x100) {
-                    p >>= 4;
-                    a += 0x40;
-                }
-
-                if (p >= 0x40) {
-                    p >>= 2;
-                    a += 0x20;
-                }
-
-                if (p >= 0x20) {
-                    p >>= 1;
-                    a += 0x10;
-                }
-            }
-
-            // a&0x70 now holds segment value and 'p' the interval number
-            a += p; // a now equal to encoded A-law value
-
-            // A-law has alternate bits inverted for transmission
-            return a ^ 0x55;
+        static unsigned char ALawEncode(SFLAudioSample pcm16) {
+            return linear_to_alaw(pcm16);
         }
 };
 
