@@ -31,15 +31,17 @@
  */
 
 #include "audiocodec.h"
+#include <cassert>
 using std::ptrdiff_t;
 
 namespace sfl {
 
-AudioCodec::AudioCodec(uint8_t payload, const std::string &codecName,
-                       int clockRate, int frameSize, unsigned channels) :
+AudioCodec::AudioCodec(uint8_t payload, const std::string &codecName, uint32_t clockRate, unsigned frameSize, uint8_t channels) :
     codecName_(codecName),
     clockRate_(clockRate),
-    channel_(channels),
+    clockRateCur_(clockRate),
+    channels_(channels),
+    channelsCur_(channels),
     frameSize_(frameSize),
     bitrate_(0.0),
     payload_(payload),
@@ -49,23 +51,46 @@ AudioCodec::AudioCodec(uint8_t payload, const std::string &codecName,
 AudioCodec::AudioCodec(const AudioCodec& c) :
     codecName_(c.codecName_),
     clockRate_(c.clockRate_),
-    channel_(c.channel_),
+    clockRateCur_(c.clockRateCur_),
+    channels_(c.channels_),
+    channelsCur_(c.channelsCur_),
     frameSize_(c.frameSize_),
     bitrate_(c.bitrate_),
     payload_(c.payload_),
     hasDynamicPayload_(c.hasDynamicPayload_)
 {}
 
-// Mono only, subclasses must implement multichannel support
-int AudioCodec::decode(std::vector<std::vector<SFLAudioSample> > &dst, unsigned char *buf, size_t buffer_size)
+int AudioCodec::decode(SFLAudioSample *, unsigned char *, size_t)
 {
-    return decode(dst[0].data(), buf, buffer_size);
+    // Unimplemented!
+    assert(false);
+    return 0;
+}
+
+int AudioCodec::encode(unsigned char *, SFLAudioSample *, size_t)
+{
+    // Unimplemented!
+    assert(false);
+    return 0;
+}
+
+
+// Mono only, subclasses must implement multichannel support
+int AudioCodec::decode(std::vector<std::vector<SFLAudioSample> > &pcm, const uint8_t* data, size_t len)
+{
+    return decode(pcm[0].data(), const_cast<uint8_t*>(data), len);
 }
 
 // Mono only, subclasses must implement multichannel support
-int AudioCodec::encode(unsigned char *dst, std::vector<std::vector<SFLAudioSample> > &src, size_t buffer_size)
+size_t AudioCodec::encode(const std::vector<std::vector<SFLAudioSample> > &pcm, uint8_t *data, size_t len)
 {
-    return encode(dst, src[0].data(), buffer_size);
+    return encode(data, const_cast<SFLAudioSample*>(pcm[0].data()), len);
+}
+
+int AudioCodec::decode(std::vector<std::vector<SFLAudioSample> > &pcm)
+{
+    pcm.clear();
+    return frameSize_;
 }
 
 std::string AudioCodec::getMimeSubtype() const
@@ -88,6 +113,11 @@ uint32_t AudioCodec::getClockRate() const
     return clockRate_;
 }
 
+uint32_t AudioCodec::getCurrentClockRate() const
+{
+    return clockRateCur_;
+}
+
 uint32_t AudioCodec::getSDPClockRate() const
 {
     return clockRate_;
@@ -103,9 +133,14 @@ double AudioCodec::getBitRate() const
     return bitrate_;
 }
 
-unsigned AudioCodec::getChannels() const
+uint8_t AudioCodec::getChannels() const
 {
-    return channel_;
+    return channels_;
+}
+
+uint8_t AudioCodec::getCurrentChannels() const
+{
+    return channelsCur_;
 }
 
 const char *
