@@ -39,20 +39,20 @@
 #define XSTR(s) STR(s)
 #define STR(s) #s
 
-/* bump when codec binary interface changes */
-#define AUDIO_CODEC_ENTRY create_1_3_0
+/* bump for each release, and bump the 0.0.0.xth digit for each API change */
+#define AUDIO_CODEC_ENTRY create_1_4_1_0
 #define AUDIO_CODEC_ENTRY_SYMBOL XSTR(AUDIO_CODEC_ENTRY)
 
-// We assume all decoders will be fed 20ms of audio or less
-// And we'll resample them to 44.1kHz or less
-// Also assume mono
-#define DEC_BUFFER_SIZE ((44100 * 20) / 1000)
+// Opus documentation:
+// "If this is less than the maximum packet duration (120ms; 5760 for 48kHz),
+// opus_decode will not be capable of decoding some packets."
+#define RAW_BUFFER_SIZE (120 * 48000 / 1000)
 
 namespace sfl {
 
 class AudioCodec {
     public:
-        AudioCodec(uint8_t payload, const std::string &codecName, int clockRate, int frameSize, unsigned channels);
+        AudioCodec(uint8_t payload, const std::string &codecName, uint32_t clockRate, unsigned frameSize, uint8_t channels);
 
         /**
          * Copy constructor.
@@ -65,31 +65,24 @@ class AudioCodec {
 
         std::string getMimeSubtype() const;
 
-        /**
-         * Decode an input buffer and fill the output buffer with the decoded data
-         * @param buffer_size : the size of the input buffer
-         * @return the number of samples decoded
-         */
-        virtual int decode(SFLAudioSample *dst, unsigned char *buf, size_t buffer_size) = 0;
-
-        /**
-         * Encode an input buffer and fill the output buffer with the encoded data
-         * @param buffer_size : the maximum size of encoded data buffer (dst)
-         * @return the number of bytes encoded
-         */
-        virtual int encode(unsigned char *dst, SFLAudioSample *src, size_t buffer_size) = 0;
-
         /**
          * Multichannel version of decode().
-         * Default implementation decode(short *, unsigned char *, size_t) to the first channel (assume 1 channel).
+         * Default implementation calls mono version
          */
-        virtual int decode(std::vector<std::vector<SFLAudioSample> > &dst, unsigned char *buf, size_t buffer_size);
+        virtual int decode(std::vector<std::vector<SFLAudioSample> > &pcm, const uint8_t* data, size_t len);
+
+        /**
+         * Inform the codec of a lost packet and perform packet loss concealment.
+         * Default implementation fills dst with 0.
+         */
+        virtual int decode(std::vector<std::vector<SFLAudioSample> > &pcm);
 
         /**
          * Multichannel version of encode().
          * Default implementation calls encode() on the first channel (assume 1 channel).
+         * @return the number of bytes encoded
          */
-        virtual int encode(unsigned char *dst, std::vector<std::vector<SFLAudioSample> > &src, size_t buffer_size);
+        virtual size_t encode(const std::vector<std::vector<SFLAudioSample> > &pcm, uint8_t *data, size_t len);
 
         uint8_t getPayloadType() const;
 
@@ -102,32 +95,93 @@ class AudioCodec {
          */
         bool hasDynamicPayload() const;
 
+        /**
+         * Having Packet Loss Concealment (PLC) supported means decode(dst) won't
+         * fill the buffer with 0 but will rather try to conceal the loss.
+         * @return true if the codec supports PLC, false otherwise.
+         */
+        virtual inline bool supportsPacketLossConcealment() const {
+            return false;
+        }
+
+        /**
+         * @returns maximum supported clock rate (sample rate).
+         */
         uint32_t getClockRate() const;
-        // clock-rate in SDP MAY be different than actual clock-rate (in derived classes)
+
+        /**
+         * @returns currently configured clock rate (sample rate).
+         */
+        uint32_t getCurrentClockRate() const;
+
+        /**
+         * Clock-rate in SDP MAY be different than actual clock-rate
+         * (in derived classes).
+         * @returns Clock-rate as it should be reported by SDP
+         *          during negociation.
+         */
         virtual uint32_t getSDPClockRate() const;
 
         double getBitRate() const;
 
-        unsigned getChannels() const;
-        // channels in SDP MAY be different than actual channels (in derived classes)
-        // Should be an empty string EXCEPT for Opus which returns "2".
-        virtual const char *
-        getSDPChannels() const;
+        /**
+         * @returns maximum supported channel number.
+         */
+        uint8_t getChannels() const;
+
+        /**
+         * @returns currently configured channel number.
+         */
+        uint8_t getCurrentChannels() const;
+
+        /**
+         * Channels in SDP MAY be different than actual channels.
+         * Should be an empty string EXCEPT for Opus which returns "2".
+         * @returns Supported channel number as it should be reported by SDP
+         *          during negociation.
+         */
+        virtual const char * getSDPChannels() const;
 
         /**
          * @return the framing size for this codec.
          */
         unsigned int getFrameSize() const;
 
+        /**
+         * Set the sampling rate and channel number preferred by the core.
+         * May or may not be considered by the codec.
+         * Use getCurrentClockRate() and getCurrentChannels() to get the format
+         * used by the codec.
+         */
+        virtual void setOptimalFormat(uint32_t /* sample_rate */ , uint8_t /* channels */ ) {}
+
     protected:
+        /**
+         * Decode an input buffer and fill the output buffer with the decoded data
+         * @param pcm: output signal
+         * @param data: input buffer to decode
+         * @param len: length of input buffer
+         * @return the number of samples decoded
+         */
+        virtual int decode(SFLAudioSample *pcm, unsigned char *data, size_t len);
+
+        /**
+         * Encode an input buffer and fill the output buffer with the encoded data
+         * @param data: output buffer, must be at least as big as max_data_bytes
+         * @param pcm: input signal
+         * @param max_data_bytes: the maximum size of the encoded data buffer (data)
+         * @return the number of bytes encoded
+         */
+        virtual int encode(unsigned char *data, SFLAudioSample *pcm, size_t max_data_bytes);
+
         /** Holds SDP-compliant codec name */
         std::string codecName_; // what we put inside sdp
 
         /** Clock rate or sample rate of the codec, in Hz */
-        uint32_t clockRate_;
+        uint32_t clockRate_, clockRateCur_;
 
-        /** Number of channel 1 = mono, 2 = stereo */
-        uint8_t channel_;
+        /** Number of channels 1 = mono, 2 = stereo */
+        uint8_t channels_, channelsCur_;
 
         /** codec frame size in samples*/
         unsigned frameSize_;
@@ -135,12 +189,12 @@ class AudioCodec {
         /** Bitrate */
         double bitrate_;
 
+        uint8_t payload_;
+        bool hasDynamicPayload_;
+
     private:
         AudioCodec& operator=(const AudioCodec&);
 
-protected:
-        uint8_t payload_;
-        bool hasDynamicPayload_;
 };
 } // end namespace sfl
 
