@@ -45,8 +45,8 @@ public:
    ~G729();
 private:
    AudioCodec * clone();
-   virtual int decode(SFLAudioSample *dst, unsigned char *buf, size_t buffer_size);
-   virtual int encode(unsigned char *dst, SFLAudioSample *src, size_t buffer_size);
+   virtual int decode(SFLAudioSample *pcm, unsigned char *data, size_t len);
+   virtual int encode(unsigned char *data, SFLAudioSample *pcm, size_t max_data_bytes);
 
    NON_COPYABLE(G729);
    //Attributes
