@@ -28,120 +28,47 @@
  *  as that of the covered work.
  */
 
+#include "opensllayer.h"
+
+#include "client/configurationmanager.h"
+
+#include "manager.h"
+#include "mainbuffer.h"
+#include "audio/dcblocker.h"
+#include "logger.h"
+#include "array_size.h"
+
+#include <thread>
+#include <chrono>
 #include <cstdio>
 #include <cassert>
 #include <unistd.h>
 
-#include "logger.h"
-#include "array_size.h"
-#include "manager.h"
-#include "mainbuffer.h"
-#include "audio/dcblocker.h"
-
-#include "opensllayer.h"
-
 const int OpenSLLayer::NB_BUFFER_PLAYBACK_QUEUE = ANDROID_BUFFER_QUEUE_LENGTH;
 const int OpenSLLayer::NB_BUFFER_CAPTURE_QUEUE = ANDROID_BUFFER_QUEUE_LENGTH;
 
-class OpenSLThread {
-    public:
-        OpenSLThread(OpenSLLayer *opensl);
-        ~OpenSLThread();
-        void initAudioLayer();
-        void start();
-        bool isRunning() const;
-
-    private:
-        void run();
-        static void *runCallback(void *context);
-
-        NON_COPYABLE(OpenSLThread);
-        pthread_t thread_;
-        OpenSLLayer* opensl_;
-        bool running_;
-};
-
-OpenSLThread::OpenSLThread(OpenSLLayer *opensl)
-    : thread_(0), opensl_(opensl), running_(false)
-{}
-
-bool
-OpenSLThread::isRunning() const
-{
-    return running_;
-}
-
-OpenSLThread::~OpenSLThread()
-{
-    running_ = false;
-    opensl_->shutdownAudioEngine();
-
-    if (thread_)
-        pthread_join(thread_, nullptr);
-}
-
-void
-OpenSLThread::start()
-{
-    running_ = true;
-    pthread_create(&thread_, nullptr, &runCallback, this);
-}
-
-void *
-OpenSLThread::runCallback(void *data)
-{
-    OpenSLThread *context = static_cast<OpenSLThread*>(data);
-    context->run();
-    return nullptr;
-}
-
-void
-OpenSLThread::initAudioLayer()
-{
-    opensl_->initAudioEngine();
-    opensl_->initAudioPlayback();
-    opensl_->initAudioCapture();
-
-    opensl_->flushMain();
-    opensl_->flushUrgent();
-}
-
-/**
- * Reimplementation of run()
- */
-void
-OpenSLThread::run()
-{
-    initAudioLayer();
-
-    opensl_->startAudioPlayback();
-    opensl_->startAudioCapture();
-    opensl_->isStarted_ = true;
-
-    while (opensl_->isStarted_)
-        usleep(20000); // 20 ms
-}
-
 // Constructor
 OpenSLLayer::OpenSLLayer(const AudioPreference &pref)
     : AudioLayer(pref)
     , indexIn_(0)
     , indexOut_(0)
     , indexRing_(0)
-    , audioThread_(0)
-    , engineObject_(0)
-    , engineInterface_(0)
-    , outputMixer_(0)
-    , playerObject_(0)
-    , recorderObject_(0)
-    , playerInterface_(0)
-    , recorderInterface_(0)
-    , playbackBufferQueue_(0)
-    , recorderBufferQueue_(0)
+    , audioThread_(nullptr)
+    , engineObject_(nullptr)
+    , engineInterface_(nullptr)
+    , outputMixer_(nullptr)
+    , playerObject_(nullptr)
+    , recorderObject_(nullptr)
+    , playerInterface_(nullptr)
+    , recorderInterface_(nullptr)
+    , playbackBufferQueue_(nullptr)
+    , recorderBufferQueue_(nullptr)
     , playbackBufferIndex_(0)
     , recordBufferIndex_(0)
-    , playbackBufferStack_(ANDROID_BUFFER_QUEUE_LENGTH, AudioBuffer(BUFFER_SIZE, 1, 8000))
-    , recordBufferStack_(ANDROID_BUFFER_QUEUE_LENGTH, AudioBuffer(BUFFER_SIZE, 1, 8000))
+    , hardwareFormat_(AudioFormat::MONO)
+    , hardwareBuffSize_(BUFFER_SIZE)
+    , playbackBufferStack_(ANDROID_BUFFER_QUEUE_LENGTH, AudioBuffer(hardwareBuffSize_, AudioFormat::MONO))
+    , recordBufferStack_(ANDROID_BUFFER_QUEUE_LENGTH, AudioBuffer(hardwareBuffSize_, AudioFormat::MONO))
 {
 }
 
@@ -149,13 +76,23 @@ OpenSLLayer::OpenSLLayer(const AudioPreference &pref)
 OpenSLLayer::~OpenSLLayer()
 {
     isStarted_ = false;
-    delete audioThread_;
 
     /* Then close the audio devices */
     stopAudioPlayback();
     stopAudioCapture();
 }
 
+void
+OpenSLLayer::init()
+{
+    initAudioEngine();
+    initAudioPlayback();
+    initAudioCapture();
+
+    flushMain();
+    flushUrgent();
+}
+
 void
 OpenSLLayer::startStream()
 {
@@ -166,13 +103,24 @@ OpenSLLayer::startStream()
 
     DEBUG("Start OpenSL audio layer");
 
-    if (audioThread_ == nullptr) {
-        audioThread_ = new OpenSLThread(this);
-        audioThread_->start();
-    } else if (!audioThread_->isRunning()) {
-        audioThread_->start();
-    }
+    std::vector<int32_t> hw_infos = Manager::instance().getClient()->getConfigurationManager()->getHardwareAudioFormat();
+    hardwareFormat_ = AudioFormat(hw_infos[0], 1);  // Mono on Android
+    hardwareBuffSize_ = hw_infos[1];
 
+    for(auto& buf : playbackBufferStack_)
+        buf.resize(hardwareBuffSize_);
+    for(auto& buf : recordBufferStack_)
+        buf.resize(hardwareBuffSize_);
+
+    hardwareFormatAvailable(hardwareFormat_);
+
+    std::thread launcher([this](){
+        init();
+        startAudioPlayback();
+        startAudioCapture();
+        isStarted_ = true;
+    });
+    launcher.detach();
 }
 
 void
@@ -188,9 +136,6 @@ OpenSLLayer::stopStream()
 
     isStarted_ = false;
 
-    delete audioThread_;
-    audioThread_ = nullptr;
-
     flushMain();
     flushUrgent();
 }
@@ -245,12 +190,12 @@ OpenSLLayer::shutdownAudioEngine()
         outputMixer_ = nullptr;
     }
 
-	if(recorderObject_ != nullptr){
-		(*recorderObject_)->Destroy(recorderObject_);
-		recorderObject_ = nullptr;
-		recorderInterface_ = nullptr;
-		recorderBufferQueue_ = nullptr;
-	}
+    if (recorderObject_ != nullptr) {
+        (*recorderObject_)->Destroy(recorderObject_);
+        recorderObject_ = nullptr;
+        recorderInterface_ = nullptr;
+        recorderBufferQueue_ = nullptr;
+    }
 
     // destroy engine object, and invalidate all associated interfaces
     DEBUG("Shutdown audio engine\n");
@@ -280,7 +225,7 @@ OpenSLLayer::initAudioPlayback()
     DEBUG("Setting audio format\n");
     SLDataFormat_PCM audioFormat = {SL_DATAFORMAT_PCM,
                                     1,
-                                    sampleRate_ * 1000,
+                                    audioFormat_.sample_rate * 1000,
                                     SL_PCMSAMPLEFORMAT_FIXED_16,
                                     SL_PCMSAMPLEFORMAT_FIXED_16,
                                     SL_SPEAKER_FRONT_CENTER,
@@ -301,10 +246,9 @@ OpenSLLayer::initAudioPlayback()
     SLDataSink audioSink = {&mixerLocation, nullptr};
 
     const SLInterfaceID ids[] = {SL_IID_ANDROIDSIMPLEBUFFERQUEUE,
-                                SL_IID_VOLUME,
-                                SL_IID_ANDROIDCONFIGURATION,
-                                SL_IID_PLAY
-                                };
+                                 SL_IID_VOLUME,
+                                 SL_IID_ANDROIDCONFIGURATION,
+                                 SL_IID_PLAY};
     const SLboolean req[] = {SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE};
 
     const unsigned nbInterface = ARRAYSIZE(ids);
@@ -328,7 +272,7 @@ OpenSLLayer::initAudioPlayback()
                      &streamType, sizeof(SLint32));
     }
 
-	DEBUG("Realize audio player\n");
+    DEBUG("Realize audio player\n");
     result = (*playerObject_)->Realize(playerObject_, SL_BOOLEAN_FALSE);
     assert(SL_RESULT_SUCCESS == result);
 
@@ -378,10 +322,10 @@ OpenSLLayer::initAudioCapture()
                                            NB_BUFFER_CAPTURE_QUEUE
                                                            };
 
-	DEBUG("Capture-> Sampling Rate: %d", sampleRate_);
-	DEBUG("Capture-> getInternalSamplingRate: %d", Manager::instance().getMainBuffer().getInternalSamplingRate());
+    DEBUG("Capture-> Sampling Rate: %d", audioFormat_.sample_rate);
+    DEBUG("Capture-> getInternalSamplingRate: %d", Manager::instance().getMainBuffer().getInternalSamplingRate());
     SLDataFormat_PCM audioFormat = {SL_DATAFORMAT_PCM, 1,
-                                    sampleRate_ * 1000,
+                                    audioFormat_.sample_rate * 1000,
                                     SL_PCMSAMPLEFORMAT_FIXED_16,
                                     SL_PCMSAMPLEFORMAT_FIXED_16,
                                     SL_SPEAKER_FRONT_CENTER,
@@ -395,12 +339,15 @@ OpenSLLayer::initAudioCapture()
     // create audio recorder
     // (requires the RECORD_AUDIO permission)
     DEBUG("Create audio recorder\n");
-    const SLInterfaceID id[] = {SL_IID_ANDROIDSIMPLEBUFFERQUEUE};
-    const SLboolean req[] = {SL_BOOLEAN_TRUE};
+    const SLInterfaceID id[2] = {SL_IID_ANDROIDSIMPLEBUFFERQUEUE,
+                                SL_IID_ANDROIDCONFIGURATION};
+    const SLboolean req[2] ={SL_BOOLEAN_TRUE,
+                             SL_BOOLEAN_TRUE};
+    SLAndroidConfigurationItf recorderConfig;
 
     if (engineInterface_ != nullptr) {
         result = (*engineInterface_)->CreateAudioRecorder(engineInterface_,
-                 &recorderObject_, &audioSource, &audioSink, 1, id, req);
+                 &recorderObject_, &audioSource, &audioSink, 2, id, req);
     }
 
     if (SL_RESULT_SUCCESS != result) {
@@ -408,6 +355,22 @@ OpenSLLayer::initAudioCapture()
         return;
     }
 
+    /* Set Android configuration */
+    result = (*recorderObject_)->GetInterface(recorderObject_,
+                                                SL_IID_ANDROIDCONFIGURATION,
+                                                &recorderConfig);
+    if (result == SL_RESULT_SUCCESS) {
+       SLint32 streamType = SL_ANDROID_RECORDING_PRESET_VOICE_COMMUNICATION;
+       result = (*recorderConfig)->SetConfiguration(
+                recorderConfig, SL_ANDROID_KEY_RECORDING_PRESET,
+                &streamType, sizeof(SLint32));
+    }
+
+    if (result != SL_RESULT_SUCCESS) {
+        DEBUG("Warning: Unable to set android recorder configuration");
+        return;
+    }
+
     // realize the audio recorder
     DEBUG("Realize the audio recorder\n");
     result = (*recorderObject_)->Realize(recorderObject_, SL_BOOLEAN_FALSE);
@@ -542,7 +505,7 @@ OpenSLLayer::stopAudioCapture()
 std::vector<std::string>
 OpenSLLayer::getCaptureDeviceList() const
 {
-	std::vector<std::string> captureDeviceList;
+    std::vector<std::string> captureDeviceList;
 
 
 
@@ -551,64 +514,60 @@ OpenSLLayer::getCaptureDeviceList() const
 // obtain or select proper one (SLAudioIODeviceCapabilitiesItf, the official interface
 // to obtain such an information)-> SL_FEATURE_UNSUPPORTED
 
-	SLuint32 InputDeviceIDs[MAX_NUMBER_INPUT_DEVICES];
-	SLint32 numInputs = 0;
-	SLboolean mic_available = SL_BOOLEAN_FALSE;
-	SLuint32 mic_deviceID = 0;
+    SLuint32 InputDeviceIDs[MAX_NUMBER_INPUT_DEVICES];
+    SLint32 numInputs = 0;
+    SLboolean mic_available = SL_BOOLEAN_FALSE;
+    SLuint32 mic_deviceID = 0;
 
-	SLresult res;
+    SLresult res;
 
-	initAudioEngine();
+    initAudioEngine();
     initAudioPlayback();
     initAudioCapture();
 
 
-	// Get the Audio IO DEVICE CAPABILITIES interface, implicit
-	DEBUG("Get the Audio IO DEVICE CAPABILITIES interface, implicit");
+    // Get the Audio IO DEVICE CAPABILITIES interface, implicit
+    DEBUG("Get the Audio IO DEVICE CAPABILITIES interface, implicit");
 
     res = (*engineObject_)->GetInterface(engineObject_, SL_IID_AUDIOIODEVICECAPABILITIES, (void*)&AudioIODeviceCapabilitiesItf);
     CheckErr(res);
 
-	DEBUG("Get the Audio IO DEVICE CAPABILITIES interface, implicit");
-	numInputs = MAX_NUMBER_INPUT_DEVICES;
+    DEBUG("Get the Audio IO DEVICE CAPABILITIES interface, implicit");
+    numInputs = MAX_NUMBER_INPUT_DEVICES;
 
-	res = (*AudioIODeviceCapabilitiesItf)->GetAvailableAudioInputs(AudioIODeviceCapabilitiesItf, &numInputs, InputDeviceIDs);
-	CheckErr(res);
+    res = (*AudioIODeviceCapabilitiesItf)->GetAvailableAudioInputs(AudioIODeviceCapabilitiesItf, &numInputs, InputDeviceIDs);
+    CheckErr(res);
 
-	int i;
-	// Search for either earpiece microphone or headset microphone input
-	// device - with a preference for the latter
-	for (i=0;i<numInputs; i++)
-	{
-		res = (*AudioIODeviceCapabilitiesItf)->QueryAudioInputCapabilities(AudioIODeviceCapabilitiesItf,
-																				InputDeviceIDs[i],
-																				&AudioInputDescriptor);
-		CheckErr(res);
+    // Search for either earpiece microphone or headset microphone input
+    // device - with a preference for the latter
+    for (int i = 0; i < numInputs; i++) {
+        res = (*AudioIODeviceCapabilitiesItf)->QueryAudioInputCapabilities(AudioIODeviceCapabilitiesItf,
+                                                                           InputDeviceIDs[i],
+                                                                           &AudioInputDescriptor);
+        CheckErr(res);
 
-		if((AudioInputDescriptor.deviceConnection == SL_DEVCONNECTION_ATTACHED_WIRED) &&
-			(AudioInputDescriptor.deviceScope == SL_DEVSCOPE_USER) &&
-			(AudioInputDescriptor.deviceLocation == SL_DEVLOCATION_HEADSET))
-		{
-			DEBUG("SL_DEVCONNECTION_ATTACHED_WIRED : mic_deviceID: %d", InputDeviceIDs[i] );
-			mic_deviceID = InputDeviceIDs[i];
-			mic_available = SL_BOOLEAN_TRUE;
-			break;
-		} else if((AudioInputDescriptor.deviceConnection == SL_DEVCONNECTION_INTEGRATED) &&
-					(AudioInputDescriptor.deviceScope == SL_DEVSCOPE_USER) &&
-					(AudioInputDescriptor.deviceLocation == SL_DEVLOCATION_HANDSET))
-		{
-			DEBUG("SL_DEVCONNECTION_INTEGRATED : mic_deviceID: %d", InputDeviceIDs[i] );
-			mic_deviceID = InputDeviceIDs[i];
-			mic_available = SL_BOOLEAN_TRUE;
-			break;
-		}
-	}
+        if (AudioInputDescriptor.deviceConnection == SL_DEVCONNECTION_ATTACHED_WIRED and
+            AudioInputDescriptor.deviceScope == SL_DEVSCOPE_USER and
+            AudioInputDescriptor.deviceLocation == SL_DEVLOCATION_HEADSET) {
+            DEBUG("SL_DEVCONNECTION_ATTACHED_WIRED : mic_deviceID: %d", InputDeviceIDs[i] );
+            mic_deviceID = InputDeviceIDs[i];
+            mic_available = SL_BOOLEAN_TRUE;
+            break;
+        } else if (AudioInputDescriptor.deviceConnection == SL_DEVCONNECTION_INTEGRATED and
+                   AudioInputDescriptor.deviceScope == SL_DEVSCOPE_USER and
+                   AudioInputDescriptor.deviceLocation == SL_DEVLOCATION_HANDSET) {
+            DEBUG("SL_DEVCONNECTION_INTEGRATED : mic_deviceID: %d", InputDeviceIDs[i] );
+            mic_deviceID = InputDeviceIDs[i];
+            mic_available = SL_BOOLEAN_TRUE;
+            break;
+        }
+    }
 
-	if (!mic_available) {
-		// Appropriate error message here
-		ERROR("No mic available quitting");
-		exit(1);
-	}
+    if (!mic_available) {
+        // Appropriate error message here
+        ERROR("No mic available quitting");
+        exit(1);
+    }
 
 
     return captureDeviceList;
@@ -618,11 +577,10 @@ OpenSLLayer::getCaptureDeviceList() const
 void
 OpenSLLayer::CheckErr( SLresult res )
 {
-	if ( res != SL_RESULT_SUCCESS )
-	{
-		// Debug printing to be placed here
-		exit(1);
-	}
+    if (res != SL_RESULT_SUCCESS) {
+        // Debug printing to be placed here
+        exit(1);
+    }
 }
 
 std::vector<std::string>
@@ -637,46 +595,45 @@ void
 OpenSLLayer::audioPlaybackCallback(SLAndroidSimpleBufferQueueItf queue, void *context)
 {
     assert(nullptr != context);
+    //auto start = std::chrono::high_resolution_clock::now();
     static_cast<OpenSLLayer*>(context)->playback(queue);
+    //auto end = std::chrono::high_resolution_clock::now();
+    //auto elapsed = std::chrono::duration_cast<std::chrono::nanoseconds>(end - start);
+    //DEBUG("Took %d us\n", elapsed/1000);
 }
 
 void
 OpenSLLayer::playback(SLAndroidSimpleBufferQueueItf queue)
 {
     assert(nullptr != queue);
-
-	notifyIncomingCall();
+    notifyIncomingCall();
 
     AudioBuffer &buffer = getNextPlaybackBuffer();
-
-    size_t samplesToGet = Manager::instance().getMainBuffer().availableForGet(MainBuffer::DEFAULT_ID);
     size_t urgentSamplesToGet = urgentRingBuffer_.availableForGet(MainBuffer::DEFAULT_ID);
 
-	bufferIsFilled_ = false;
-	//DEBUG("samplesToGet:%d", samplesToGet);
-	//DEBUG("urgentSamplesToGet:%d", urgentSamplesToGet);
-
-    if (urgentSamplesToGet > 0)
-        bufferIsFilled_ = audioPlaybackFillWithUrgent(buffer, urgentSamplesToGet);
-    else {
-		if (samplesToGet > 0)
-			bufferIsFilled_ = audioPlaybackFillWithVoice(buffer, samplesToGet);
-        else {
+    bufferIsFilled_ = false;
+    if (urgentSamplesToGet > 0) {
+        bufferIsFilled_ = audioPlaybackFillWithUrgent(buffer, std::min(urgentSamplesToGet, hardwareBuffSize_));
+    } else {
+        auto& main_buffer = Manager::instance().getMainBuffer();
+        buffer.resize(hardwareBuffSize_);
+        size_t samplesToGet = audioPlaybackFillWithVoice(buffer);
+        if (samplesToGet == 0) {
             bufferIsFilled_ = audioPlaybackFillWithToneOrRingtone(buffer);
+        } else {
+            bufferIsFilled_ = true;
         }
-	}
+    }
 
     if (bufferIsFilled_) {
         SLresult result = (*queue)->Enqueue(queue, buffer.getChannel(0)->data(), buffer.frames()*sizeof(SFLAudioSample));
-
         if (SL_RESULT_SUCCESS != result) {
             DEBUG("Error could not enqueue buffers in playback callback\n");
         }
-
         incrementPlaybackIndex();
     } else {
-		DEBUG("Error buffer not filled in audio playback\n");
-	}
+        DEBUG("Error buffer not filled in audio playback\n");
+    }
 }
 
 void
@@ -699,7 +656,6 @@ OpenSLLayer::capture(SLAndroidSimpleBufferQueueItf queue)
     // enqueue an empty buffer to be filled by the recorder
     // (for streaming recording, we enqueue at least 2 empty buffers to start things off)
     result = (*recorderBufferQueue_)->Enqueue(recorderBufferQueue_, buffer.getChannel(0)->data(), buffer.frames()*sizeof(SFLAudioSample));
-    
 
     audioCaptureFillBuffer(old_buffer);
 
@@ -711,21 +667,18 @@ OpenSLLayer::capture(SLAndroidSimpleBufferQueueItf queue)
 
 
 void
-OpenSLLayer::updatePreference(AudioPreference &preference, int index, PCMType type)
+OpenSLLayer::updatePreference(AudioPreference &preference, int index, DeviceType type)
 {
 #ifdef OUTSIDE_TESTING
 
     switch (type) {
-        case SFL_PCM_PLAYBACK:
+        case Device::PLAYBACK:
             break;
 
-        case SFL_PCM_CAPTURE:
+        case Device::CAPTURE:
             break;
 
-        case SFL_PCM_RINGTONE:
-            break;
-
-        default:
+        case Device::RINGTONE:
             break;
     }
 
@@ -736,15 +689,16 @@ void OpenSLLayer::audioCaptureFillBuffer(AudioBuffer &buffer)
 {
     MainBuffer &mbuffer = Manager::instance().getMainBuffer();
 
-    const unsigned mainBufferSampleRate = mbuffer.getInternalSamplingRate();
-    const bool resample = mainBufferSampleRate != sampleRate_;
+    //const unsigned mainBufferSampleRate = mbuffer.getInternalSamplingRate();
+    const AudioFormat mainBufferFormat = mbuffer.getInternalAudioFormat();
+    const bool resample = mainBufferFormat.sample_rate != audioFormat_.sample_rate;
 
     buffer.applyGain(isCaptureMuted_ ? 0.0 : captureGain_);
 
     if (resample) {
-		int outSamples = buffer.frames() * (static_cast<double>(sampleRate_) / mainBufferSampleRate);
-        AudioBuffer out(outSamples, 1, mainBufferSampleRate);
-        converter_.resample(buffer, out);
+        int outSamples = buffer.frames() * (static_cast<double>(audioFormat_.sample_rate) / mainBufferFormat.sample_rate);
+        AudioBuffer out(outSamples, mainBufferFormat);
+        resampler_.resample(buffer, out);
         dcblocker_.process(out);
         mbuffer.putData(out, MainBuffer::DEFAULT_ID);
     } else {
@@ -755,7 +709,7 @@ void OpenSLLayer::audioCaptureFillBuffer(AudioBuffer &buffer)
 
 bool OpenSLLayer::audioPlaybackFillWithToneOrRingtone(AudioBuffer &buffer)
 {
-    buffer.resize(BUFFER_SIZE);
+    buffer.resize(hardwareBuffSize_);
     AudioLoop *tone = Manager::instance().getTelephoneTone();
     AudioLoop *file_tone = Manager::instance().getTelephoneFile();
 
@@ -775,7 +729,7 @@ bool OpenSLLayer::audioPlaybackFillWithToneOrRingtone(AudioBuffer &buffer)
 bool OpenSLLayer::audioPlaybackFillWithUrgent(AudioBuffer &buffer, size_t samplesToGet)
 {
     // Urgent data (dtmf, incoming call signal) come first.
-    samplesToGet = std::min(samplesToGet, BUFFER_SIZE);
+    samplesToGet = std::min(samplesToGet, hardwareBuffSize_);
     buffer.resize(samplesToGet);
     urgentRingBuffer_.get(buffer, MainBuffer::DEFAULT_ID);
     buffer.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
@@ -786,23 +740,20 @@ bool OpenSLLayer::audioPlaybackFillWithUrgent(AudioBuffer &buffer, size_t sample
     return true;
 }
 
-bool OpenSLLayer::audioPlaybackFillWithVoice(AudioBuffer &buffer, size_t samplesAvail)
+size_t OpenSLLayer::audioPlaybackFillWithVoice(AudioBuffer &buffer)
 {
     MainBuffer &mainBuffer = Manager::instance().getMainBuffer();
-
-    buffer.resize(samplesAvail);
-    mainBuffer.getData(buffer, MainBuffer::DEFAULT_ID);
+    size_t got = mainBuffer.getAvailableData(buffer, MainBuffer::DEFAULT_ID);
+    buffer.resize(got);
     buffer.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
-
-    if (sampleRate_ != mainBuffer.getInternalSamplingRate()) {
-        DEBUG("OpenSLLayer::audioPlaybackFillWithVoice sampleRate_ != mainBuffer.getInternalSamplingRate() \n");
+    if (audioFormat_.sample_rate != mainBuffer.getInternalSamplingRate()) {
+        DEBUG("OpenSLLayer::audioPlaybackFillWithVoice sample_rate != mainBuffer.getInternalSamplingRate() \n");
         AudioBuffer out(buffer, false);
-        out.setSampleRate(sampleRate_);
-        converter_.resample(buffer, out);
+        out.setSampleRate(audioFormat_.sample_rate);
+        resampler_.resample(buffer, out);
         buffer = out;
     }
-
-    return true;
+    return buffer.size();
 }
 
 void dumpAvailableEngineInterfaces()
