@@ -36,7 +36,7 @@
 
 #include "audiostream.h"
 #include "pulselayer.h"
-#include "audio/samplerateconverter.h"
+#include "audio/resampler.h"
 #include "audio/dcblocker.h"
 #include "logger.h"
 #include "manager.h"
@@ -45,30 +45,30 @@
 #include <cstdlib>
 #include <fstream>
 
-namespace {
-
-void playback_callback(pa_stream * /*s*/, size_t /*bytes*/, void* userdata)
+static void
+playback_callback(pa_stream * /*s*/, size_t /*bytes*/, void* userdata)
 {
     static_cast<PulseLayer*>(userdata)->writeToSpeaker();
 }
 
-void capture_callback(pa_stream * /*s*/, size_t /*bytes*/, void* userdata)
+static void
+capture_callback(pa_stream * /*s*/, size_t /*bytes*/, void* userdata)
 {
     static_cast<PulseLayer*>(userdata)->readFromMic();
 }
 
-void ringtone_callback(pa_stream * /*s*/, size_t /*bytes*/, void* userdata)
+static void
+ringtone_callback(pa_stream * /*s*/, size_t /*bytes*/, void* userdata)
 {
     static_cast<PulseLayer*>(userdata)->ringtoneToSpeaker();
 }
 
-void stream_moved_callback(pa_stream *s, void *userdata UNUSED)
+static void
+stream_moved_callback(pa_stream *s, void *userdata UNUSED)
 {
     DEBUG("stream %d to %d", pa_stream_get_index(s), pa_stream_get_device_index(s));
 }
 
-} // end anonymous namespace
-
 PulseMainLoopLock::PulseMainLoopLock(pa_threaded_mainloop *loop) : loop_(loop), destroyLoop_(false)
 {
     pa_threaded_mainloop_lock(loop_);
@@ -91,13 +91,13 @@ PulseMainLoopLock::~PulseMainLoopLock()
 
 PulseLayer::PulseLayer(AudioPreference &pref)
     : AudioLayer(pref)
-    , playback_(0)
-    , record_(0)
-    , ringtone_(0)
+    , playback_(nullptr)
+    , record_(nullptr)
+    , ringtone_(nullptr)
     , sinkList_()
     , sourceList_()
-    , micBuffer_(0, 1, 8000)
-    , context_(0)
+    , micBuffer_(0, AudioFormat::MONO)
+    , context_(nullptr)
     , mainloop_(pa_threaded_mainloop_new())
     , enumeratingSinks_(false)
     , enumeratingSources_(false)
@@ -166,6 +166,7 @@ PulseLayer::~PulseLayer()
         PulseMainLoopLock lock(mainloop_);
 
         pa_context_set_state_callback(context_, NULL, NULL);
+        pa_context_set_subscribe_callback(context_, NULL, NULL);
         pa_context_disconnect(context_);
         pa_context_unref(context_);
     }
@@ -231,7 +232,7 @@ void PulseLayer::updateSourceList()
 
 bool PulseLayer::inSinkList(const std::string &deviceName)
 {
-    const bool found = std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::nameComparator(deviceName)) != sinkList_.end();
+    const bool found = std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::NameComparator(deviceName)) != sinkList_.end();
 
     DEBUG("seeking for %s in sinks. %s found", deviceName.c_str(), found ? "" : "NOT");
     return found;
@@ -239,7 +240,7 @@ bool PulseLayer::inSinkList(const std::string &deviceName)
 
 bool PulseLayer::inSourceList(const std::string &deviceName)
 {
-    const bool found = std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::nameComparator(deviceName)) != sourceList_.end();
+    const bool found = std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::NameComparator(deviceName)) != sourceList_.end();
 
     DEBUG("seeking for %s in sources. %s found", deviceName.c_str(), found ? "" : "NOT");
     return found;
@@ -251,7 +252,7 @@ std::vector<std::string> PulseLayer::getCaptureDeviceList() const
     std::vector<std::string> names(n);
 
     for (unsigned i = 0; i < n; i++)
-        names[i] = sourceList_[i].name;
+        names[i] = sourceList_[i].description;
 
     return names;
 }
@@ -262,37 +263,53 @@ std::vector<std::string> PulseLayer::getPlaybackDeviceList() const
     std::vector<std::string> names(n);
 
     for (unsigned i = 0; i < n; i++)
-        names[i] = sinkList_[i].name;
+        names[i] = sinkList_[i].description;
 
     return names;
 }
 
-int PulseLayer::getAudioDeviceIndex(const std::string& name) const
+int PulseLayer::getAudioDeviceIndex(const std::string& descr, DeviceType type) const
 {
-    int index = std::distance(sourceList_.begin(), std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::nameComparator(name)));
-
-    if (index == std::distance(sourceList_.begin(), sourceList_.end())) {
-        index = std::distance(sinkList_.begin(), std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::nameComparator(name)));
+    switch (type) {
+    case DeviceType::PLAYBACK:
+    case DeviceType::RINGTONE:
+        return std::distance(sinkList_.begin(), std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::DescriptionComparator(descr)));
+    case DeviceType::CAPTURE:
+        return std::distance(sourceList_.begin(), std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::DescriptionComparator(descr)));
+    default:
+        ERROR("Unexpected device type");
+        return 0;
     }
+}
 
-    return index;
+int PulseLayer::getAudioDeviceIndexByName(const std::string& name, DeviceType type) const
+{
+    switch (type) {
+    case DeviceType::PLAYBACK:
+    case DeviceType::RINGTONE:
+        return std::distance(sinkList_.begin(), std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::NameComparator(name)));
+    case DeviceType::CAPTURE:
+        return std::distance(sourceList_.begin(), std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::NameComparator(name)));
+    default:
+        ERROR("Unexpected device type");
+        return 0;
+    }
 }
 
 const PaDeviceInfos* PulseLayer::getDeviceInfos(const std::vector<PaDeviceInfos>& list, const std::string& name) const
 {
-    std::vector<PaDeviceInfos>::const_iterator dev_info = std::find_if(list.begin(), list.end(), PaDeviceInfos::nameComparator(name));
+    std::vector<PaDeviceInfos>::const_iterator dev_info = std::find_if(list.begin(), list.end(), PaDeviceInfos::NameComparator(name));
 
     if (dev_info == list.end()) return nullptr;
 
     return &(*dev_info);
 }
 
-std::string PulseLayer::getAudioDeviceName(int index, PCMType type) const
+std::string PulseLayer::getAudioDeviceName(int index, DeviceType type) const
 {
-
     switch (type) {
-        case SFL_PCM_PLAYBACK:
-        case SFL_PCM_RINGTONE:
+        case DeviceType::PLAYBACK:
+        case DeviceType::RINGTONE:
             if (index < 0 or static_cast<size_t>(index) >= sinkList_.size()) {
                 ERROR("Index %d out of range", index);
                 return "";
@@ -300,15 +317,16 @@ std::string PulseLayer::getAudioDeviceName(int index, PCMType type) const
 
             return sinkList_[index].name;
 
-        case SFL_PCM_CAPTURE:
+        case DeviceType::CAPTURE:
             if (index < 0 or static_cast<size_t>(index) >= sourceList_.size()) {
                 ERROR("Index %d out of range", index);
                 return "";
             }
 
             return sourceList_[index].name;
-
         default:
+            // Should never happen
+            ERROR("Unexpected type");
             return "";
     }
 }
@@ -323,18 +341,19 @@ void PulseLayer::createStreams(pa_context* c)
     std::string ringtoneDevice(preference_.getPulseDeviceRingtone());
     std::string defaultDevice = "";
 
-    DEBUG("Devices: playback: %s record: %s ringtone: %s",
-          playbackDevice.c_str(), captureDevice.c_str(), ringtoneDevice.c_str());
+    DEBUG("playback: %s record: %s ringtone: %s", playbackDevice.c_str(),
+          captureDevice.c_str(), ringtoneDevice.c_str());
 
     // Create playback stream
     const PaDeviceInfos* dev_infos = getDeviceInfos(sinkList_, playbackDevice);
 
     if (dev_infos == nullptr) {
         dev_infos = &sinkList_[0];
-        DEBUG("Prefered playback device not found in device list, selecting %s instead.", dev_infos->name.c_str());
+        WARN("Prefered playback device %s not found in device list, selecting %s instead.",
+             playbackDevice.c_str(), dev_infos->name.c_str());
     }
 
-    playback_ = new AudioStream(c, mainloop_, "SFLphone playback", PLAYBACK_STREAM, sampleRate_, dev_infos);
+    playback_ = new AudioStream(c, mainloop_, "SFLphone playback", PLAYBACK_STREAM, audioFormat_.sample_rate, dev_infos);
 
     pa_stream_set_write_callback(playback_->pulseStream(), playback_callback, this);
     pa_stream_set_moved_callback(playback_->pulseStream(), stream_moved_callback, this);
@@ -344,10 +363,11 @@ void PulseLayer::createStreams(pa_context* c)
 
     if (dev_infos == nullptr) {
         dev_infos = &sourceList_[0];
-        DEBUG("Prefered capture device not found in device list, selecting %s instead.", dev_infos->name.c_str());
+        WARN("Prefered capture device %s not found in device list, selecting %s instead.",
+             captureDevice.c_str(), dev_infos->name.c_str());
     }
 
-    record_ = new AudioStream(c, mainloop_, "SFLphone capture", CAPTURE_STREAM, sampleRate_, dev_infos);
+    record_ = new AudioStream(c, mainloop_, "SFLphone capture", CAPTURE_STREAM, audioFormat_.sample_rate, dev_infos);
 
     pa_stream_set_read_callback(record_->pulseStream() , capture_callback, this);
     pa_stream_set_moved_callback(record_->pulseStream(), stream_moved_callback, this);
@@ -357,10 +377,13 @@ void PulseLayer::createStreams(pa_context* c)
 
     if (dev_infos == nullptr) {
         dev_infos = &sinkList_[0];
-        DEBUG("Prefered ringtone device not found in device list, selecting %s instead.", dev_infos->name.c_str());
+        WARN("Prefered ringtone device %s not found in device list, selecting %s instead.",
+             ringtoneDevice.c_str(), dev_infos->name.c_str());
     }
 
-    ringtone_ = new AudioStream(c, mainloop_, "SFLphone ringtone", RINGTONE_STREAM, sampleRate_, dev_infos);
+    ringtone_ = new AudioStream(c, mainloop_, "SFLphone ringtone", RINGTONE_STREAM, audioFormat_.sample_rate, dev_infos);
+
+    hardwareFormatAvailable(playback_->getFormat());
 
     pa_stream_set_write_callback(ringtone_->pulseStream(), ringtone_callback, this);
     pa_stream_set_moved_callback(ringtone_->pulseStream(), stream_moved_callback, this);
@@ -371,16 +394,13 @@ void PulseLayer::createStreams(pa_context* c)
     flushUrgent();
 }
 
-namespace {
 // Delete stream and zero out its pointer
-void
+static void
 cleanupStream(AudioStream *&stream)
 {
     delete stream;
     stream = 0;
 }
-}
-
 
 void PulseLayer::disconnectAudioStream()
 {
@@ -426,7 +446,7 @@ void PulseLayer::writeToSpeaker()
     pa_stream *s = playback_->pulseStream();
     const pa_sample_spec* sample_spec = pa_stream_get_sample_spec(s);
     size_t sample_size = pa_frame_size(sample_spec);
-    const unsigned n_channels = sample_spec->channels;
+    const AudioFormat format(playback_->getFormat());
 
     // available bytes to be written in pulseaudio internal buffer
     int ret = pa_stream_writable_size(s);
@@ -453,7 +473,7 @@ void PulseLayer::writeToSpeaker()
     SFLAudioSample *data = 0;
 
     if (urgentBytes) {
-        AudioBuffer linearbuff(urgentSamples, n_channels, 8000);
+        AudioBuffer linearbuff(urgentSamples, format);
         pa_stream_begin_write(s, (void**)&data, &urgentBytes);
         urgentRingBuffer_.get(linearbuff, MainBuffer::DEFAULT_ID); // retrive only the first sample_spec->channels channels
         linearbuff.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
@@ -471,7 +491,7 @@ void PulseLayer::writeToSpeaker()
     if (toneToPlay) {
         if (playback_->isReady()) {
             pa_stream_begin_write(s, (void**)&data, &writableBytes);
-            AudioBuffer linearbuff(writableSamples, n_channels, 8000);
+            AudioBuffer linearbuff(writableSamples, format);
             toneToPlay->getNext(linearbuff, playbackGain_); // retrive only n_channels
             linearbuff.interleave(data);
             pa_stream_write(s, data, writableBytes, nullptr, 0, PA_SEEK_RELATIVE);
@@ -496,11 +516,11 @@ void PulseLayer::writeToSpeaker()
 
     double resampleFactor = 1.;
 
-    unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
-    bool resample = sampleRate_ != mainBufferSampleRate;
+    AudioFormat mainBufferAudioFormat = Manager::instance().getMainBuffer().getInternalAudioFormat();
+    bool resample = audioFormat_.sample_rate != mainBufferAudioFormat.sample_rate;
 
     if (resample) {
-        resampleFactor = (double) sampleRate_ / mainBufferSampleRate;
+        resampleFactor = (double) audioFormat_.sample_rate / mainBufferAudioFormat.sample_rate;
         readableSamples = (double) readableSamples / resampleFactor;
     }
 
@@ -510,12 +530,12 @@ void PulseLayer::writeToSpeaker()
 
     pa_stream_begin_write(s, (void**)&data, &resampledBytes);
 
-    AudioBuffer linearbuff(readableSamples, n_channels, 8000);
+    AudioBuffer linearbuff(readableSamples, format);
     Manager::instance().getMainBuffer().getData(linearbuff, MainBuffer::DEFAULT_ID);
 
     if (resample) {
-        AudioBuffer rsmpl_out(nResampled, 1, sampleRate_);
-        converter_.resample(linearbuff, rsmpl_out);
+        AudioBuffer rsmpl_out(nResampled, format);
+        resampler_.resample(linearbuff, rsmpl_out);
         rsmpl_out.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
         rsmpl_out.interleave(data);
         pa_stream_write(s, data, resampledBytes, nullptr, 0, PA_SEEK_RELATIVE);
@@ -535,20 +555,20 @@ void PulseLayer::readFromMic()
     size_t bytes;
 
     const size_t sample_size = record_->sampleSize();
-    const uint8_t channels = record_->channels();
+    const AudioFormat format(record_->getFormat());
 
     if (pa_stream_peek(record_->pulseStream() , (const void**) &data , &bytes) < 0 or !data)
         return;
 
-    assert(channels);
+    assert(format.nb_channels);
     assert(sample_size);
-    const size_t samples = bytes / sample_size / channels;
+    const size_t samples = bytes / sample_size / format.nb_channels;
 
-    AudioBuffer in(samples, channels, sampleRate_);
-    in.deinterleave((SFLAudioSample*)data, samples, channels);
+    AudioBuffer in(samples, format);
+    in.deinterleave((SFLAudioSample*)data, samples, format.nb_channels);
 
     unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
-    bool resample = sampleRate_ != mainBufferSampleRate;
+    bool resample = audioFormat_.sample_rate != mainBufferSampleRate;
 
     in.applyGain(isCaptureMuted_ ? 0.0 : captureGain_);
 
@@ -556,7 +576,7 @@ void PulseLayer::readFromMic()
 
     if (resample) {
         micBuffer_.setSampleRate(mainBufferSampleRate);
-        converter_.resample(in, micBuffer_);
+        resampler_.resample(in, micBuffer_);
         out = &micBuffer_;
     }
 
@@ -593,7 +613,7 @@ void PulseLayer::ringtoneToSpeaker()
 
     if (fileToPlay) {
         const unsigned samples = (bytes / sample_size) / ringtone_->channels();
-        AudioBuffer tmp(samples, ringtone_->channels(), 8000);
+        AudioBuffer tmp(samples, ringtone_->getFormat());
         fileToPlay->getNext(tmp, playbackGain_);
         tmp.interleave((SFLAudioSample*) data);
     } else {
@@ -690,8 +710,7 @@ void PulseLayer::source_input_info_callback(pa_context *c UNUSED, const pa_sourc
           i->flags & PA_SOURCE_HARDWARE ? "HARDWARE" : "");
 
     if (not context->inSourceList(i->name)) {
-        PaDeviceInfos ep_infos(i->index, i->name, i->sample_spec, i->channel_map);
-        context->sourceList_.push_back(ep_infos);
+        context->sourceList_.push_back(*i);
     }
 }
 
@@ -731,47 +750,43 @@ void PulseLayer::sink_input_info_callback(pa_context *c UNUSED, const pa_sink_in
           i->flags & PA_SINK_HARDWARE ? "HARDWARE" : "");
 
     if (not context->inSinkList(i->name)) {
-        PaDeviceInfos ep_infos(i->index, i->name, i->sample_spec, i->channel_map);
-        context->sinkList_.push_back(ep_infos);
+        context->sinkList_.push_back(*i);
     }
 }
 
-void PulseLayer::updatePreference(AudioPreference &preference, int index, PCMType type)
+void PulseLayer::updatePreference(AudioPreference &preference, int index, DeviceType type)
 {
     const std::string devName(getAudioDeviceName(index, type));
 
     switch (type) {
-        case SFL_PCM_PLAYBACK:
+        case DeviceType::PLAYBACK:
             DEBUG("setting %s for playback", devName.c_str());
             preference.setPulseDevicePlayback(devName);
             break;
 
-        case SFL_PCM_CAPTURE:
+        case DeviceType::CAPTURE:
             DEBUG("setting %s for capture", devName.c_str());
             preference.setPulseDeviceRecord(devName);
             break;
 
-        case SFL_PCM_RINGTONE:
+        case DeviceType::RINGTONE:
             DEBUG("setting %s for ringer", devName.c_str());
             preference.setPulseDeviceRingtone(devName);
             break;
-
-        default:
-            break;
     }
 }
 
 int PulseLayer::getIndexCapture() const
 {
-    return getAudioDeviceIndex(preference_.getPulseDeviceRecord());
+    return getAudioDeviceIndexByName(preference_.getPulseDeviceRecord(), DeviceType::CAPTURE);
 }
 
 int PulseLayer::getIndexPlayback() const
 {
-    return getAudioDeviceIndex(preference_.getPulseDevicePlayback());
+    return getAudioDeviceIndexByName(preference_.getPulseDevicePlayback(), DeviceType::PLAYBACK);
 }
 
 int PulseLayer::getIndexRingtone() const
 {
-    return getAudioDeviceIndex(preference_.getPulseDeviceRingtone());
+    return getAudioDeviceIndexByName(preference_.getPulseDeviceRingtone(), DeviceType::RINGTONE);
 }
