@@ -34,7 +34,7 @@
 Recordable::Recordable() : recAudio_(), recorder_(&recAudio_, Manager::instance().getMainBuffer())
 {
     DEBUG("Set recording options: %s", Manager::instance().audioPreference.getRecordPath().c_str());
-    recAudio_.setRecordingOptions(8000, Manager::instance().audioPreference.getRecordPath());
+    recAudio_.setRecordingOptions(AudioFormat::MONO, Manager::instance().audioPreference.getRecordPath());
 }
 
 Recordable::~Recordable()
@@ -53,7 +53,7 @@ std::string Recordable::getFilename() const
     return recAudio_.getFilename();
 }
 
-void Recordable::setRecordingSmplRate(int smplRate)
+void Recordable::setRecordingFormat(AudioFormat format)
 {
-    recAudio_.setSndSamplingRate(smplRate);
+    recAudio_.setSndFormat(format);
 }
