@@ -50,8 +50,13 @@ class Recordable {
         /**
          * This method must be implemented for this interface as calls and conferences
          * have different behavior.
+         * Implementations must call the super method.
          */
-        virtual bool toggleRecording() = 0;
+        virtual bool toggleRecording() {
+            if (not isRecording())
+                recorder_.init();
+            return recAudio_.toggleRecording();
+        }
 
         /**
          * Stop recording
@@ -73,7 +78,7 @@ class Recordable {
         /**
          * Set recording sampling rate.
          */
-        void setRecordingSmplRate(int smplRate);
+        void setRecordingFormat(AudioFormat format);
 
     protected:
         AudioRecord recAudio_;
