@@ -33,23 +33,24 @@
  *  as that of the covered work.
  */
 
+#include "ringbuffer.h"
+#include "logger.h"
+
+#include <chrono>
+#include <utility> // for std::pair
 #include <cstdlib>
 #include <cstring>
-#include <utility> // for std::pair
 
-#include "logger.h"
-#include "ringbuffer.h"
-
-namespace {
 // corresponds to 160 ms (about 5 rtp packets)
-const size_t MIN_BUFFER_SIZE = 1280;
-}
+static const size_t MIN_BUFFER_SIZE = 1024;
 
 // Create  a ring buffer with 'size' bytes
-RingBuffer::RingBuffer(size_t size, const std::string &call_id) :
+RingBuffer::RingBuffer(size_t size, const std::string &call_id, AudioFormat format /* = MONO */) :
     endPos_(0)
-    , buffer_(std::max(size, MIN_BUFFER_SIZE), 1, 8000)
-    , readpointers_()
+    , buffer_(std::max(size, MIN_BUFFER_SIZE), format)
+    , lock_()
+    , not_empty_()
+    , readoffsets_()
     , buffer_id_(call_id)
 {
 }
@@ -57,16 +58,16 @@ RingBuffer::RingBuffer(size_t size, const std::string &call_id) :
 void
 RingBuffer::flush(const std::string &call_id)
 {
-    storeReadPointer(endPos_, call_id);
+    storeReadOffset(endPos_, call_id);
 }
 
 
 void
 RingBuffer::flushAll()
 {
-    ReadPointer::iterator iter;
+    ReadOffset::iterator iter;
 
-    for (iter = readpointers_.begin(); iter != readpointers_.end(); ++iter)
+    for (iter = readoffsets_.begin(); iter != readoffsets_.end(); ++iter)
         iter->second = endPos_;
 }
 
@@ -76,7 +77,7 @@ RingBuffer::putLength() const
     const size_t buffer_size = buffer_.frames();
     if (buffer_size == 0)
         return 0;
-    const size_t startPos = (not readpointers_.empty()) ? getSmallestReadPointer() : 0;
+    const size_t startPos = (not readoffsets_.empty()) ? getSmallestReadOffset() : 0;
     return (endPos_ + buffer_size - startPos) % buffer_size;
 }
 
@@ -85,81 +86,76 @@ size_t RingBuffer::getLength(const std::string &call_id) const
     const size_t buffer_size = buffer_.frames();
     if (buffer_size == 0)
         return 0;
-    return (endPos_ + buffer_size - getReadPointer(call_id)) % buffer_size;
+    return (endPos_ + buffer_size - getReadOffset(call_id)) % buffer_size;
 }
 
 void
 RingBuffer::debug()
 {
-    DEBUG("Start=%d; End=%d; BufferSize=%d", getSmallestReadPointer(), endPos_, buffer_.frames());
+    DEBUG("Start=%d; End=%d; BufferSize=%d", getSmallestReadOffset(), endPos_, buffer_.frames());
 }
 
-size_t RingBuffer::getReadPointer(const std::string &call_id) const
+size_t RingBuffer::getReadOffset(const std::string &call_id) const
 {
-    if (hasNoReadPointers())
+    if (hasNoReadOffsets())
         return 0;
-
-    ReadPointer::const_iterator iter = readpointers_.find(call_id);
-    return (iter != readpointers_.end()) ? iter->second : 0;
+    ReadOffset::const_iterator iter = readoffsets_.find(call_id);
+    return (iter != readoffsets_.end()) ? iter->second : 0;
 }
 
 size_t
-RingBuffer::getSmallestReadPointer() const
+RingBuffer::getSmallestReadOffset() const
 {
-    if (hasNoReadPointers())
+    if (hasNoReadOffsets())
         return 0;
-
     size_t smallest = buffer_.frames();
-
-    ReadPointer::const_iterator iter;
-
-    for (iter = readpointers_.begin(); iter != readpointers_.end(); ++iter)
-        if (iter->second < smallest)
-            smallest = iter->second;
-
+    for(auto const& iter : readoffsets_)
+        smallest = std::min(smallest, iter.second);
     return smallest;
 }
 
 void
-RingBuffer::storeReadPointer(size_t pointer_value, const std::string &call_id)
+RingBuffer::storeReadOffset(size_t offset, const std::string &call_id)
 {
-    ReadPointer::iterator iter = readpointers_.find(call_id);
+    ReadOffset::iterator iter = readoffsets_.find(call_id);
 
-    if (iter != readpointers_.end())
-        iter->second = pointer_value;
+    if (iter != readoffsets_.end())
+        iter->second = offset;
     else
-        DEBUG("Cannot find \"%s\" readPointer in \"%s\" ringbuffer", call_id.c_str(), buffer_id_.c_str());
+        DEBUG("Cannot find \"%s\" readOffset in \"%s\" ringbuffer", call_id.c_str(), buffer_id_.c_str());
 }
 
 
 void
-RingBuffer::createReadPointer(const std::string &call_id)
+RingBuffer::createReadOffset(const std::string &call_id)
 {
-    if (!hasThisReadPointer(call_id))
-        readpointers_.insert(std::pair<std::string, int> (call_id, endPos_));
+    std::lock_guard<std::mutex> l(lock_);
+    if (!hasThisReadOffset(call_id))
+        readoffsets_.insert(std::pair<std::string, int> (call_id, endPos_));
 }
 
 
 void
-RingBuffer::removeReadPointer(const std::string &call_id)
+RingBuffer::removeReadOffset(const std::string &call_id)
 {
-    ReadPointer::iterator iter = readpointers_.find(call_id);
+    std::lock_guard<std::mutex> l(lock_);
+    ReadOffset::iterator iter = readoffsets_.find(call_id);
 
-    if (iter != readpointers_.end())
-        readpointers_.erase(iter);
+    if (iter != readoffsets_.end())
+        readoffsets_.erase(iter);
 }
 
 
 bool
-RingBuffer::hasThisReadPointer(const std::string &call_id) const
+RingBuffer::hasThisReadOffset(const std::string &call_id) const
 {
-    return readpointers_.find(call_id) != readpointers_.end();
+    return readoffsets_.find(call_id) != readoffsets_.end();
 }
 
 
-bool RingBuffer::hasNoReadPointers() const
+bool RingBuffer::hasNoReadOffsets() const
 {
-    return readpointers_.empty();
+    return readoffsets_.empty();
 }
 
 //
@@ -169,21 +165,21 @@ bool RingBuffer::hasNoReadPointers() const
 // This one puts some data inside the ring buffer.
 void RingBuffer::put(AudioBuffer& buf)
 {
-    const size_t len = putLength();
+    std::lock_guard<std::mutex> l(lock_);
     const size_t sample_num = buf.frames();
     const size_t buffer_size = buffer_.frames();
     if (buffer_size == 0)
         return;
 
+    size_t len = putLength();
+    if (buffer_size - len < sample_num)
+        discard(sample_num);
     size_t toCopy = sample_num;
 
     // Add more channels if the input buffer holds more channels than the ring.
     if (buffer_.channels() < buf.channels())
         buffer_.setChannelNum(buf.channels());
 
-    if (toCopy > buffer_size - len)
-        toCopy = buffer_size - len;
-
     size_t in_pos = 0;
     size_t pos = endPos_;
 
@@ -200,6 +196,7 @@ void RingBuffer::put(AudioBuffer& buf)
     }
 
     endPos_ = pos;
+    not_empty_.notify_all();
 }
 
 //
@@ -213,26 +210,31 @@ RingBuffer::availableForGet(const std::string &call_id) const
     return getLength(call_id);
 }
 
-// Get will move 'toCopy' bytes from the internal FIFO to 'buffer'
 size_t RingBuffer::get(AudioBuffer& buf, const std::string &call_id)
 {
-    if (hasNoReadPointers())
+    std::lock_guard<std::mutex> l(lock_);
+
+    if (hasNoReadOffsets())
         return 0;
 
-    if (not hasThisReadPointer(call_id))
+    if (not hasThisReadOffset(call_id))
         return 0;
 
-    const size_t len = getLength(call_id);
-    const size_t sample_num = buf.frames();
     const size_t buffer_size = buffer_.frames();
     if (buffer_size == 0)
         return 0;
+
+    size_t len = getLength(call_id);
+    const size_t sample_num = buf.frames();
     size_t toCopy = std::min(sample_num, len);
+    if (toCopy and toCopy != sample_num) {
+        DEBUG("Partial get: %d/%d", toCopy, sample_num);
+    }
 
     const size_t copied = toCopy;
 
     size_t dest = 0;
-    size_t startPos = getReadPointer(call_id);
+    size_t startPos = getReadOffset(call_id);
 
     while (toCopy > 0) {
         size_t block = toCopy;
@@ -247,24 +249,64 @@ size_t RingBuffer::get(AudioBuffer& buf, const std::string &call_id)
         toCopy -= block;
     }
 
-    storeReadPointer(startPos, call_id);
+    storeReadOffset(startPos, call_id);
     return copied;
 }
 
+
+size_t RingBuffer::waitForDataAvailable(const std::string &call_id, const size_t min_data_length, const std::chrono::high_resolution_clock::time_point& deadline) const
+{
+    std::unique_lock<std::mutex> l(lock_);
+    const size_t buffer_size = buffer_.frames();
+    if (buffer_size < min_data_length) return 0;
+    ReadOffset::const_iterator read_ptr = readoffsets_.find(call_id);
+    if (read_ptr == readoffsets_.end()) return 0;
+    size_t getl = 0;
+    if (deadline == std::chrono::high_resolution_clock::time_point()) {
+        not_empty_.wait(l, [=, &getl] {
+                getl =  (endPos_ + buffer_size - read_ptr->second) % buffer_size;
+                return getl >= min_data_length;
+        });
+    } else {
+        not_empty_.wait_until(l, deadline, [=, &getl]{
+                getl = (endPos_ + buffer_size - read_ptr->second) % buffer_size;
+                return getl >= min_data_length;
+        });
+    }
+    return getl;
+}
+
 size_t
 RingBuffer::discard(size_t toDiscard, const std::string &call_id)
 {
-    size_t len = getLength(call_id);
+    std::lock_guard<std::mutex> l(lock_);
 
+    const size_t buffer_size = buffer_.frames();
+    if (buffer_size == 0)
+        return 0;
+
+    size_t len = getLength(call_id);
     if (toDiscard > len)
         toDiscard = len;
 
-    size_t buffer_size = buffer_.frames();
+    size_t startPos = (getReadOffset(call_id) + toDiscard) % buffer_size;
+    storeReadOffset(startPos, call_id);
+    return toDiscard;
+}
+
+size_t
+RingBuffer::discard(size_t toDiscard)
+{
+    const size_t buffer_size = buffer_.frames();
     if (buffer_size == 0)
         return 0;
-    size_t startPos = (getReadPointer(call_id) + toDiscard) % buffer_size;
-
-    storeReadPointer(startPos, call_id);
 
+    for (auto & r : readoffsets_) {
+        size_t dst = (r.second + buffer_size - endPos_) % buffer_size;
+        if (dst < toDiscard) {
+            DEBUG("%s : discarding: %d frames", r.first.c_str(), toDiscard - dst);
+            r.second = (r.second + toDiscard - dst) % buffer_size;
+        }
+    }
     return toDiscard;
 }
