@@ -23,14 +23,17 @@
 #ifndef __RING_BUFFER__
 #define __RING_BUFFER__
 
-#include <fstream>
-#include <vector>
-#include <map>
-
-#include "noncopyable.h"
 #include "audiobuffer.h"
+#include "noncopyable.h"
 
-typedef std::map<std::string, size_t> ReadPointer;
+#include <condition_variable>
+#include <mutex>
+#include <chrono>
+#include <map>
+#include <vector>
+#include <fstream>
+
+typedef std::map<std::string, size_t> ReadOffset;
 
 /**
  * A ring buffer for mutichannel audio samples
@@ -41,57 +44,40 @@ class RingBuffer {
          * Constructor
          * @param size  Size of the buffer to create
          */
-        RingBuffer(size_t size, const std::string &call_id);
+        RingBuffer(size_t size, const std::string &call_id, AudioFormat format=AudioFormat::MONO);
 
         std::string getBufferId() const {
             return buffer_id_;
         }
 
         /**
-         * Reset the counters to 0 for this read pointer
+         * Reset the counters to 0 for this read offset
          */
         void flush(const std::string &call_id);
 
         void flushAll();
 
-        /**
-         * Get read pointer coresponding to this call
-         */
-        size_t getReadPointer(const std::string &call_id) const;
+        inline  AudioFormat getFormat() const {
+            return buffer_.getFormat();
+        }
 
-        /**
-         * Get the whole readpointer list for this ringbuffer
-         */
-        ReadPointer* getReadPointerList() {
-            return &readpointers_;
+        inline void setFormat(AudioFormat format) {
+            buffer_.setFormat(format);
         }
 
         /**
-         * Return the smalest readpointer. Usefull to evaluate if ringbuffer is full
+         * Add a new readoffset for this ringbuffer
          */
-        size_t getSmallestReadPointer() const;
+        void createReadOffset(const std::string &call_id);
 
         /**
-         * Move readpointer forward by pointer_value
+         * Remove a readoffset for this ringbuffer
          */
-        void storeReadPointer(size_t pointer_value, const std::string &call_id);
+        void removeReadOffset(const std::string &call_id);
 
-        /**
-         * Add a new readpointer for this ringbuffer
-         */
-        void createReadPointer(const std::string &call_id);
+        size_t readOffsetCount() const { return readoffsets_.size(); }
 
-        /**
-         * Remove a readpointer for this ringbuffer
-         */
-        void removeReadPointer(const std::string &call_id);
-
-        /**
-         * Test if readpointer coresponding to this call is still active
-         */
-        bool hasThisReadPointer(const std::string &call_id) const;
-
-        bool hasNoReadPointers() const;
+        bool hasNoReadOffsets() const;
 
         /**
          * Write data in the ring buffer
@@ -129,24 +115,68 @@ class RingBuffer {
 
         size_t getLength(const std::string &call_id) const;
 
+        inline bool isFull() const {
+            return putLength() == buffer_.frames();
+        }
+
+        inline bool isEmpty() const {
+            return putLength() == 0;
+        }
+
+
+        /**
+         * Blocks until min_data_length samples of data is available, or until deadline is missed.
+         *
+         * @param call_id The read offset for which data should be available.
+         * @param min_data_length Minimum number of samples that should be vailable for the call to return
+         * @param deadline The call is garenteed to end after this time point. If no deadline is provided, the the call blocks indefinitely.
+         * @return available data for call_id after the call returned (same as calling getLength(call_id) ).
+         */
+        size_t waitForDataAvailable(const std::string &call_id, const size_t min_data_length, const std::chrono::high_resolution_clock::time_point& deadline) const;
+
         /**
          * Debug function print mEnd, mStart, mBufferSize
          */
         void debug();
 
     private:
-        /* POULET! */
         NON_COPYABLE(RingBuffer);
 
-        /** Pointer on the last data */
+        /**
+         * Return the smalest readoffset. Usefull to evaluate if ringbuffer is full
+         */
+        size_t getSmallestReadOffset() const;
+
+        /**
+         * Get read offset coresponding to this call
+         */
+        size_t getReadOffset(const std::string &call_id) const;
+
+        /**
+         * Move readoffset forward by offset
+         */
+        void storeReadOffset(size_t offset, const std::string &call_id);
+
+        /**
+         * Test if readoffset coresponding to this call is still active
+         */
+        bool hasThisReadOffset(const std::string &call_id) const;
+
+        /**
+         * Discard data from all read offsets to make place for new data.
+         */
+        size_t discard(size_t toDiscard);
+
+        /** Offset on the last data */
         size_t endPos_;
         /** Data */
         AudioBuffer buffer_;
 
-        ReadPointer readpointers_;
-        std::string buffer_id_;
+        mutable std::mutex lock_;
+        mutable std::condition_variable not_empty_;
 
-        friend class MainBufferTest;
+        ReadOffset readoffsets_;
+        std::string buffer_id_;
 };
 
 
