@@ -113,7 +113,6 @@ void IAXAccount::setAccountDetails(const std::map<std::string, std::string> &det
     parseString(details, CONFIG_ACCOUNT_PASSWORD, password_);
     parseBool(details, CONFIG_ACCOUNT_ENABLE, enabled_);
     parseString(details, CONFIG_ACCOUNT_MAILBOX, mailBox_);
-    parseString(details, CONFIG_DISPLAY_NAME, displayName_);
     parseString(details, CONFIG_ACCOUNT_USERAGENT, userAgent_);
 }
 
@@ -141,21 +140,23 @@ void IAXAccount::registerVoIPLink()
 {
     try {
         link_.init();
-        link_.sendRegister(this);
+        link_.sendRegister(*this);
     } catch (const VoipLinkException &e) {
         ERROR("IAXAccount: %s", e.what());
     }
 }
 
 void
-IAXAccount::unregisterVoIPLink()
+IAXAccount::unregisterVoIPLink(std::function<void(bool)> cb)
 {
     try {
-        link_.sendUnregister(this);
+        link_.sendUnregister(*this);
         link_.terminate();
     } catch (const VoipLinkException &e) {
         ERROR("IAXAccount: %s", e.what());
     }
+    if (cb)
+        cb(true);
 }
 
 void
