@@ -32,14 +32,15 @@
 #include <cstring>
 #include <sys/socket.h>
 #include <iax-client.h>
+#include <frame.h>
+
 #include "iaxcall.h"
-#include "iax2/frame.h"
 #include "logger.h"
 #include "account.h"
 #include "manager.h"
 
-namespace {
-int codecToASTFormat(int c)
+static int
+codecToASTFormat(int c)
 {
     switch (c) {
         case PAYLOAD_CODEC_ULAW:
@@ -58,7 +59,6 @@ int codecToASTFormat(int c)
             return 0;
     }
 }
-}
 
 IAXCall::IAXCall(const std::string& id, Call::CallType type, const std::string &account_id) : Call(id, type, account_id),
     format(0), session(NULL)
