@@ -34,14 +34,13 @@
 #include <algorithm>
 
 #include "iaxcall.h"
-#include "eventthread.h"
 #include "im/instant_messaging.h"
 #include "iaxaccount.h"
 #include "logger.h"
 #include "manager.h"
 #include "hooks/urlhook.h"
 #include "audio/audiolayer.h"
-#include "audio/samplerateconverter.h"
+#include "audio/resampler.h"
 #include "array_size.h"
 #include "map_utils.h"
 
@@ -53,13 +52,12 @@ IAXVoIPLink::IAXVoIPLink(const std::string& accountID) :
     regSession_(NULL)
     , nextRefreshStamp_(0)
     , mutexIAX_()
-    , decData_(DEC_BUFFER_SIZE, 1, 8000)
-    , resampledData_(DEC_BUFFER_SIZE * 4, 1, 8000)
+    , rawBuffer_(RAW_BUFFER_SIZE, AudioFormat::MONO)
+    , resampledData_(RAW_BUFFER_SIZE * 4, AudioFormat::MONO)
     , encodedData_()
-    , converter_(44100)
+    , resampler_(44100)
     , initDone_(false)
     , accountID_(accountID)
-    , evThread_(this)
 {
     srand(time(NULL));    // to get random number for RANDOM_PORT
 }
@@ -84,7 +82,6 @@ IAXVoIPLink::init()
     for (int port = IAX_DEFAULT_PORTNO, nbTry = 0; nbTry < 3 ; port = rand() % 64000 + 1024, nbTry++) {
         if (iax_init(port) >= 0) {
             handlingEvents_ = true;
-            evThread_.start();
             initDone_ = true;
             break;
         }
@@ -101,23 +98,21 @@ IAXVoIPLink::terminate()
     handlingEvents_ = false;
 
     for (auto & item : iaxCallMap_) {
-        IAXCall *call = static_cast<IAXCall*>(item.second);
-
+        auto& call = item.second;
         if (call) {
             std::lock_guard<std::mutex> lock(mutexIAX_);
             iax_hangup(call->session, const_cast<char*>("Dumped Call"));
-            delete call;
+            call.reset();
         }
     }
 
     iaxCallMap_.clear();
-    evThread_.join();
 
     initDone_ = false;
 }
 
 bool
-IAXVoIPLink::getEvent()
+IAXVoIPLink::handleEvents()
 {
     iax_event *event = NULL;
 
@@ -155,8 +150,11 @@ IAXVoIPLink::getEvent()
         }
     }
 
-    if (nextRefreshStamp_ and nextRefreshStamp_ < time(NULL))
-        sendRegister(Manager::instance().getIaxAccount(accountID_));
+    if (nextRefreshStamp_ and nextRefreshStamp_ < time(NULL)) {
+        auto account = Manager::instance().getIaxAccount(accountID_);
+        if (account)
+            sendRegister(*account);
+    }
 
     sendAudioFromMic();
 
@@ -175,11 +173,22 @@ IAXVoIPLink::getCallIDs()
     return v;
 }
 
+std::vector<std::shared_ptr<Call> >
+IAXVoIPLink::getCalls(const std::string &account_id) const
+{
+    std::vector<std::shared_ptr<Call> > calls;
+    for (const auto & item : iaxCallMap_) {
+        if (item.second->getAccountId() == account_id)
+            calls.push_back(item.second);
+    }
+    return calls;
+}
+
 void
 IAXVoIPLink::sendAudioFromMic()
 {
     for (const auto & item : iaxCallMap_) {
-        IAXCall *currentCall = static_cast<IAXCall*>(item.second);
+        auto& currentCall = item.second;
 
         if (!currentCall or currentCall->getState() != Call::ACTIVE)
             continue;
@@ -202,8 +211,8 @@ IAXVoIPLink::sendAudioFromMic()
             continue;
 
         // Get bytes from micRingBuffer to data_from_mic
-        decData_.resize(samples);
-        samples = Manager::instance().getMainBuffer().getData(decData_, currentCall->getCallId());
+        rawBuffer_.resize(samples);
+        samples = Manager::instance().getMainBuffer().getData(rawBuffer_, currentCall->getCallId());
 
         int compSize;
         unsigned int audioRate = audioCodec->getClockRate();
@@ -211,17 +220,17 @@ IAXVoIPLink::sendAudioFromMic()
         AudioBuffer *in;
 
         if (audioRate != mainBufferSampleRate) {
-            decData_.setSampleRate(audioRate);
+            rawBuffer_.setSampleRate(audioRate);
             resampledData_.setSampleRate(mainBufferSampleRate);
-            converter_.resample(decData_, resampledData_);
+            resampler_.resample(rawBuffer_, resampledData_);
             in = &resampledData_;
             outSamples = 0;
         } else {
             outSamples = samples;
-            in = &decData_;
+            in = &rawBuffer_;
         }
 
-        compSize = audioCodec->encode(encodedData_, in->getData(), DEC_BUFFER_SIZE);
+        compSize = audioCodec->encode(in->getData(), encodedData_, RAW_BUFFER_SIZE);
 
         if (currentCall->session and samples > 0) {
             std::lock_guard<std::mutex> lock(mutexIAX_);
@@ -232,28 +241,25 @@ IAXVoIPLink::sendAudioFromMic()
     }
 }
 
-IAXCall*
+std::shared_ptr<IAXCall>
 IAXVoIPLink::getIAXCall(const std::string& id)
 {
     return getIaxCall(id);
 }
 
 void
-IAXVoIPLink::sendRegister(Account *a)
+IAXVoIPLink::sendRegister(Account& a)
 {
-    IAXAccount *account = static_cast<IAXAccount*>(a);
-
-    if (!account)
-        throw VoipLinkException("Account is NULL");
-    else if (not account->isEnabled()) {
+    IAXAccount& account = static_cast<IAXAccount&>(a);
+    if (not account.isEnabled()) {
         WARN("Account must be enabled to register, ignoring");
         return;
     }
 
-    if (account->getHostname().empty())
+    if (account.getHostname().empty())
         throw VoipLinkException("Account hostname is empty");
 
-    if (account->getUsername().empty())
+    if (account.getUsername().empty())
         throw VoipLinkException("Account username is empty");
 
     std::lock_guard<std::mutex> lock(mutexIAX_);
@@ -264,14 +270,14 @@ IAXVoIPLink::sendRegister(Account *a)
     regSession_ = iax_session_new();
 
     if (regSession_) {
-        iax_register(regSession_, account->getHostname().data(), account->getUsername().data(), account->getPassword().data(), 120);
+        iax_register(regSession_, account.getHostname().data(), account.getUsername().data(), account.getPassword().data(), 120);
         nextRefreshStamp_ = time(NULL) + 10;
-        account->setRegistrationState(TRYING);
+        account.setRegistrationState(RegistrationState::TRYING);
     }
 }
 
 void
-IAXVoIPLink::sendUnregister(Account *a)
+IAXVoIPLink::sendUnregister(Account& a, std::function<void(bool)> cb)
 {
     if (regSession_) {
         std::lock_guard<std::mutex> lock(mutexIAX_);
@@ -281,18 +287,21 @@ IAXVoIPLink::sendUnregister(Account *a)
 
     nextRefreshStamp_ = 0;
 
-    static_cast<IAXAccount*>(a)->setRegistrationState(UNREGISTERED);
+    static_cast<IAXAccount&>(a).setRegistrationState(RegistrationState::UNREGISTERED);
+
+    if (cb)
+        cb(true);
 }
 
-Call*
+std::shared_ptr<Call>
 IAXVoIPLink::newOutgoingCall(const std::string& id, const std::string& toUrl, const std::string &account_id)
 {
-    IAXCall* call = new IAXCall(id, Call::OUTGOING, account_id);
+    std::shared_ptr<IAXCall> call(new IAXCall(id, Call::OUTGOING, account_id));
 
     call->setPeerNumber(toUrl);
     call->initRecFilename(toUrl);
 
-    iaxOutgoingInvite(call);
+    iaxOutgoingInvite(call.get());
     call->setConnectionState(Call::PROGRESSING);
     call->setState(Call::ACTIVE);
     addIaxCall(call);
@@ -324,8 +333,8 @@ IAXVoIPLink::hangup(const std::string& id, int reason UNUSED)
 
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == nullptr)
+        auto call = getIAXCall(id);
+        if (!call)
             throw VoipLinkException("Could not find call");
 
         {
@@ -346,8 +355,8 @@ IAXVoIPLink::peerHungup(const std::string& id)
 
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == NULL)
+        auto call = getIAXCall(id);
+        if (!call)
             throw VoipLinkException("Could not find call");
 
         call->session = NULL;
@@ -365,8 +374,8 @@ IAXVoIPLink::onhold(const std::string& id)
 
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == NULL)
+        auto call = getIAXCall(id);
+        if (!call)
             throw VoipLinkException("Could not find call");
         {
             std::lock_guard<std::mutex> lock(mutexIAX_);
@@ -384,8 +393,8 @@ IAXVoIPLink::offhold(const std::string& id)
 
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == NULL)
+        auto call = getIAXCall(id);
+        if (!call)
             throw VoipLinkException("Could not find call");
 
         {
@@ -407,8 +416,8 @@ IAXVoIPLink::transfer(const std::string& id, const std::string& to)
 
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == NULL)
+        auto call = getIAXCall(id);
+        if (!call)
             return;
         {
             std::lock_guard<std::mutex> lock(mutexIAX_);
@@ -428,8 +437,8 @@ IAXVoIPLink::refuse(const std::string& id)
 {
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == NULL)
+        auto call = getIAXCall(id);
+        if (!call)
             return;
         {
             std::lock_guard<std::mutex> lock(mutexIAX_);
@@ -444,8 +453,8 @@ void
 IAXVoIPLink::carryingDTMFdigits(const std::string& id, char code)
 {
     std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-    IAXCall* call = getIAXCall(id);
-    if (call == NULL)
+    auto call = getIAXCall(id);
+    if (!call)
         return;
 
     {
@@ -461,8 +470,8 @@ IAXVoIPLink::sendTextMessage(const std::string& callID,
                              const std::string& /*from*/)
 {
     std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-    IAXCall* call = getIAXCall(callID);
-    if (call == nullptr)
+    auto call = getIAXCall(callID);
+    if (!call)
         return;
 
     {
@@ -476,23 +485,18 @@ void
 IAXVoIPLink::clearIaxCallMap()
 {
     std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-
-    for (const auto & item : iaxCallMap_)
-        delete item.second;
-
     iaxCallMap_.clear();
-
 }
 
 void
-IAXVoIPLink::addIaxCall(IAXCall* call)
+IAXVoIPLink::addIaxCall(std::shared_ptr<IAXCall>& call)
 {
     if (call == nullptr)
         return;
 
     std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-    if (getIaxCall(call->getCallId()) == NULL)
-        iaxCallMap_[call->getCallId()] = call;
+    if (!getIaxCall(call->getCallId()))
+        iaxCallMap_[call->getCallId()] = std::shared_ptr<IAXCall>(call);
 }
 
 void
@@ -501,40 +505,18 @@ IAXVoIPLink::removeIaxCall(const std::string& id)
     std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
 
     DEBUG("Removing call %s from list", id.c_str());
-
-    delete iaxCallMap_[id];
     iaxCallMap_.erase(id);
 }
 
 
-IAXCall*
+std::shared_ptr<IAXCall>
 IAXVoIPLink::getIaxCall(const std::string& id)
 {
     IAXCallMap::iterator iter = iaxCallMap_.find(id);
     if (iter != iaxCallMap_.end())
         return iter->second;
     else
-        return NULL;
-}
-
-
-std::string
-IAXVoIPLink::getCurrentVideoCodecName(Call * /*call*/) const
-{
-    // FIXME: Video not supported for IAX yet
-    return "";
-}
-
-std::string
-IAXVoIPLink::getCurrentAudioCodecNames(Call *c) const
-{
-    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-    IAXCall *call = static_cast<IAXCall*>(c);
-    if (call == nullptr)
-        return "";
-
-    sfl::AudioCodec *audioCodec = Manager::instance().audioCodecFactory.getCodec(call->getAudioCodec());
-    return audioCodec ? audioCodec->getMimeSubtype() : "";
+        return nullptr;
 }
 
 void
@@ -563,7 +545,7 @@ IAXVoIPLink::iaxFindCallIDBySession(iax_session* session)
     std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
 
     for (const auto & item : iaxCallMap_) {
-        IAXCall* call = static_cast<IAXCall*>(item.second);
+        auto& call = item.second;
 
         if (call and call->session == session)
             return call->getCallId();
@@ -577,8 +559,7 @@ IAXVoIPLink::handleReject(const std::string &id)
 {
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall *call = getIAXCall(id);
-        if (call) {
+        if (auto call = getIAXCall(id)) {
             call->setConnectionState(Call::CONNECTED);
             call->setState(Call::ERROR);
         }
@@ -592,8 +573,7 @@ IAXVoIPLink::handleAccept(iax_event* event, const std::string &id)
 {
     if (event->ies.format) {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall *call = getIAXCall(id);
-        if (call)
+        if (auto call = getIAXCall(id))
             call->format = event->ies.format;
     }
 }
@@ -603,7 +583,7 @@ IAXVoIPLink::handleAnswerTransfer(iax_event* event, const std::string &id)
 {
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall *call = getIAXCall(id);
+        auto call = getIAXCall(id);
         if (!call or call->getConnectionState() == Call::CONNECTED)
             return;
 
@@ -626,8 +606,8 @@ IAXVoIPLink::handleBusy(const std::string &id)
 {
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall *call = getIAXCall(id);
-        if (call == nullptr)
+        auto call = getIAXCall(id);
+        if (!call)
             return;
         call->setConnectionState(Call::CONNECTED);
         call->setState(Call::BUSY);
@@ -642,8 +622,8 @@ IAXVoIPLink::handleMessage(iax_event* event, const std::string &id)
     std::string peerNumber;
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall *call = getIAXCall(id);
-        if (call == nullptr)
+        auto call = getIAXCall(id);
+        if (!call)
             return;
         peerNumber = call->getPeerNumber();
     }
@@ -656,8 +636,8 @@ IAXVoIPLink::handleRinging(const std::string &id)
 {
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall *call = getIAXCall(id);
-        if (call == nullptr)
+        auto call = getIAXCall(id);
+        if (!call)
             return;
         call->setConnectionState(Call::RINGING);
     }
@@ -731,8 +711,8 @@ void IAXVoIPLink::iaxHandleVoiceEvent(iax_event* event, const std::string &id)
 
     {
         std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
-        IAXCall* call = getIAXCall(id);
-        if (call == nullptr)
+        auto call = getIAXCall(id);
+        if (!call)
             throw VoipLinkException("Could not find call");
 
         sfl::AudioCodec *audioCodec = static_cast<sfl::AudioCodec *>(Manager::instance().audioCodecFactory.getCodec(call->getAudioCodec()));
@@ -754,14 +734,14 @@ void IAXVoIPLink::iaxHandleVoiceEvent(iax_event* event, const std::string &id)
         if (size > max)
             size = max;
 
-        audioCodec->decode(decData_.getData(), data , size);
-        out = &decData_;
+        audioCodec->decode(rawBuffer_.getData(), data , size);
+        out = &rawBuffer_;
         unsigned int audioRate = audioCodec->getClockRate();
 
         if (audioRate != mainBufferSampleRate) {
-            decData_.setSampleRate(mainBufferSampleRate);
+            rawBuffer_.setSampleRate(mainBufferSampleRate);
             resampledData_.setSampleRate(audioRate);
-            converter_.resample(decData_, resampledData_);
+            resampler_.resample(rawBuffer_, resampledData_);
             out = &resampledData_;
         }
     }
@@ -785,7 +765,7 @@ void IAXVoIPLink::iaxHandleRegReply(iax_event* event)
         regSession_ = NULL;
     }
 
-    account->setRegistrationState((event->etype == IAX_EVENT_REGREJ) ? ERROR_AUTH : REGISTERED);
+    account->setRegistrationState((event->etype == IAX_EVENT_REGREJ) ? RegistrationState::ERROR_AUTH : RegistrationState::REGISTERED);
 
     if (event->etype == IAX_EVENT_REGACK)
         nextRefreshStamp_ = time(NULL) + (event->ies.refresh ? event->ies.refresh : 60);
@@ -793,7 +773,7 @@ void IAXVoIPLink::iaxHandleRegReply(iax_event* event)
 
 void IAXVoIPLink::iaxHandlePrecallEvent(iax_event* event)
 {
-    IAXCall *call;
+    std::shared_ptr<IAXCall> call;
     std::string id;
     int format;
 
@@ -801,7 +781,8 @@ void IAXVoIPLink::iaxHandlePrecallEvent(iax_event* event)
         case IAX_EVENT_CONNECT:
             id = Manager::instance().getNewCallID();
 
-            call = new IAXCall(id, Call::INCOMING, accountID_);
+            call = std::make_shared<IAXCall>(id, Call::INCOMING, accountID_);
+
             call->session = event->session;
             call->setConnectionState(Call::PROGRESSING);
 
