@@ -40,21 +40,22 @@
 #include "account.h"
 #include "voiplink.h"
 #include "audio/audiobuffer.h"
-#include "audio/codecs/audiocodec.h" // for DEC_BUFFER_SIZE
+#include "audio/codecs/audiocodec.h" // for RAW_BUFFER_SIZE
 #include "sfl_types.h"
 #include "noncopyable.h"
-#include "audio/samplerateconverter.h"
-#include "eventthread.h"
+#include "audio/resampler.h"
 
 #include <iax-client.h>
 
+#include <memory>
+
 class IAXCall;
 class IAXAccount;
 
 class AudioCodec;
 class AudioLayer;
 
-typedef std::map<std::string, IAXCall*> IAXCallMap;
+typedef std::map<std::string, std::shared_ptr<IAXCall> > IAXCallMap;
 
 /**
  * @file iaxvoiplink.h
@@ -71,12 +72,13 @@ class IAXVoIPLink : public VoIPLink {
         /**
          *	Listen to events sent by the call manager ( asterisk, etc .. )
          */
-        virtual bool getEvent();
+        virtual bool handleEvents();
 
 
         /* Returns a list of all callIDs */
-        static std::vector<std::string>
-        getCallIDs();
+        static std::vector<std::string> getCallIDs();
+
+        virtual std::vector<std::shared_ptr<Call> > getCalls(const std::string &account_id) const;
 
         /**
          * Return the internal account map for all VOIP links
@@ -101,14 +103,14 @@ class IAXVoIPLink : public VoIPLink {
         /**
          * Send out registration
          */
-        virtual void sendRegister(Account *a);
+        virtual void sendRegister(Account& a);
 
         /**
          * Destroy registration session
          * @todo Send an IAX_COMMAND_REGREL to force unregistration upstream.
          *       Urgency: low
          */
-        virtual void sendUnregister(Account *a);
+        virtual void sendUnregister(Account& a, std::function<void(bool)> cb = std::function<void(bool)>());
 
         /**
          * Create a new outgoing call
@@ -116,7 +118,7 @@ class IAXVoIPLink : public VoIPLink {
          * @param toUrl The address to call
          * @return Call*  A pointer on the call
          */
-        virtual Call* newOutgoingCall(const std::string& id, const std::string& toUrl, const std::string &account_id);
+        virtual std::shared_ptr<Call> newOutgoingCall(const std::string& id, const std::string& toUrl, const std::string &account_id);
 
         /**
          * Answer a call
@@ -191,18 +193,11 @@ class IAXVoIPLink : public VoIPLink {
         virtual void sendTextMessage(const std::string& callID, const std::string& message, const std::string& from);
 #endif
         static void clearIaxCallMap();
-        static void addIaxCall(IAXCall* call);
+        static void addIaxCall(std::shared_ptr<IAXCall>& call);
         // must be called while holding iaxCallMapMutex
-        static IAXCall* getIaxCall(const std::string& id);
+        static std::shared_ptr<IAXCall> getIaxCall(const std::string& id);
         static void removeIaxCall(const std::string &id);
 
-        /**
-         * Return the codec protocol used for this call
-         * @param id The call identifier
-         */
-        virtual std::string getCurrentVideoCodecName(Call *c) const;
-        virtual std::string getCurrentAudioCodecNames(Call *c) const;
-
     private:
         NON_COPYABLE(IAXVoIPLink);
 
@@ -236,7 +231,7 @@ class IAXVoIPLink : public VoIPLink {
          * @param id CallId
          * @return IAXCall pointer or 0
          */
-        IAXCall* getIAXCall(const std::string& id);
+        std::shared_ptr<IAXCall> getIAXCall(const std::string& id);
 
         /**
          * Find a iaxcall by iax session number
@@ -295,14 +290,11 @@ class IAXVoIPLink : public VoIPLink {
         std::mutex mutexIAX_;
 
         /** encoder/decoder/resampler buffers */
-        AudioBuffer decData_;
+        AudioBuffer rawBuffer_;
         AudioBuffer resampledData_;
-        //SFLAudioSample decData_[DEC_BUFFER_SIZE];
-        //SFLAudioSample resampledData_[DEC_BUFFER_SIZE * 4];
-        unsigned char encodedData_[DEC_BUFFER_SIZE];
+        unsigned char encodedData_[RAW_BUFFER_SIZE];
 
-        /** Sample rate converter object */
-        SamplerateConverter converter_;
+        Resampler resampler_;
 
         /** Whether init() was called already or not
          * This should be used in init() and terminate(), to
@@ -311,11 +303,6 @@ class IAXVoIPLink : public VoIPLink {
         bool initDone_;
 
         const std::string accountID_;
-
-        /**
-         * Threading object
-         */
-        EventThread evThread_;
 };
 
 #endif
