@@ -33,8 +33,7 @@
 #include "logger.h"
 #include <expat.h>
 
-namespace {
-void XMLCALL
+static void XMLCALL
 startElementCallback(void *userData, const char *name, const char **atts)
 {
     if (strcmp(name, "entry"))
@@ -48,16 +47,16 @@ startElementCallback(void *userData, const char *name, const char **atts)
     static_cast<sfl::InstantMessaging::UriList *>(userData)->push_back(entry);
 }
 
-void XMLCALL endElementCallback(void * /*userData*/, const char * /*name*/)
+static void XMLCALL
+endElementCallback(void * /*userData*/, const char * /*name*/)
 {}
-} // end anonymous namespace
 
 namespace sfl {
 bool InstantMessaging::saveMessage(const std::string &message, const std::string &author, const std::string &id, int mode)
 {
     std::ofstream File;
     std::string filename = "im:" + id;
-    File.open(filename.c_str(), (std::_Ios_Openmode) mode);
+    File.open(filename.c_str(), static_cast<std::ios_base::openmode>(mode));
 
     if (!File.good() || !File.is_open())
         return false;
