@@ -76,10 +76,6 @@ class History;
 class TelephoneTone;
 class VoIPLink;
 
-#ifdef USE_ZEROCONF
-class DNSService;
-#endif
-
 class Account;
 class SIPAccount;
 class IAXAccount;
@@ -123,6 +119,10 @@ class ManagerImpl {
          */
         ShortcutPreferences shortcutPreferences;
 
+        // Manager should not be accessed until initialized.
+        // FIXME this is an evil hack!
+        static std::atomic_bool initialized;
+
         /**
          * Initialisation of thread (sound) and map.
          * Init a new VoIPLink, audio codec and audio driver
@@ -131,14 +131,14 @@ class ManagerImpl {
 
         void setPath(const std::string &path);
 
-#ifdef HAVE_DBUS
         /**
-         * Enter Dbus mainloop
+         * Enter mainloop
          */
         int run();
-        // Interrupt Dbus mainloop
+        /**
+         * Interrupt mainloop
+         */
         int interrupt();
-#endif
 
         /*
          * Terminate all threads and exit DBus loop
@@ -419,16 +419,6 @@ class ManagerImpl {
          */
         void sendRegister(const std::string& accountId, bool enable);
 
-        /**
-         * Register all account in accountMap_
-         */
-        void registerAllAccounts();
-
-        /**
-         * Unregister all account in accountMap_
-         */
-        void unregisterAllAccounts();
-
         /**
          * Get account list
          * @return std::vector<std::string> A list of accoundIDs
@@ -480,6 +470,12 @@ class ManagerImpl {
          */
         std::vector<std::string> getParticipantList(const std::string& confID) const;
 
+        /**
+         * Get a list of the display names for everyone in a conference
+         * @return std::vector<std::string> A list of display names
+         */
+        std::vector<std::string> getDisplayNames(const std::string& confID) const;
+
         std::string getConferenceId(const std::string& callID);
 
         /**
@@ -506,14 +502,6 @@ class ManagerImpl {
          */
         void removeAccount(const std::string& accountID);
 
-        /**
-         * Get current codec name
-         * @param call id
-         * @return std::string The codec name
-         */
-        std::string getCurrentAudioCodecName(const std::string& id);
-        std::string getCurrentVideoCodecName(const std::string& id);
-
         /**
          * Set input audio plugin
          * @param audioPlugin The audio plugin
@@ -523,9 +511,9 @@ class ManagerImpl {
         /**
              * Set audio device
              * @param index The index of the soundcard
-             * @param the type of stream, either SFL_PCM_PLAYBACK, SFL_PCM_CAPTURE, SFL_PCM_RINGTONE
+             * @param the type of stream, either PLAYBACK, CAPTURE, RINGTONE
              */
-        void setAudioDevice(int index, AudioLayer::PCMType streamType);
+        void setAudioDevice(int index, DeviceType streamType);
 
         /**
          * Get list of supported audio output device
@@ -550,7 +538,8 @@ class ManagerImpl {
          * @param name The string description of an audio device
          * @return int  His index
          */
-        int getAudioDeviceIndex(const std::string &name);
+        int getAudioInputDeviceIndex(const std::string &name);
+        int getAudioOutputDeviceIndex(const std::string &name);
 
         /**
          * Get current alsa plugin
@@ -660,16 +649,22 @@ class ManagerImpl {
 
         /**
          * Set the audio manager
+         * @return true if api is now in use, false otherwise
          */
-        void setAudioManager(const std::string &api);
-
-        void switchAudioManager();
+        bool setAudioManager(const std::string &api);
 
         /**
-         * Set the internal audio sampling rate change. Should close the audio layer and
-         * reopen stream at different rate,
+         * Callback called when the audio layer initialised with its
+         * preferred format.
          */
-        void audioSamplingRateChanged(int);
+        void hardwareAudioFormatChanged(AudioFormat format);
+
+        /**
+         * Should be called by any component dealing with an external
+         * audio source, indicating the format used so the mixer format
+         * can be eventually adapted.
+         */
+        void audioFormatUsed(AudioFormat format);
 
         /**
          * Change a specific value in the configuration tree.
@@ -787,7 +782,7 @@ class ManagerImpl {
         /**
          * Get the Call referred to by callID. If the Call does not exist, return NULL
          */
-        Call *getCallFromCallID(const std::string &callID);
+        std::shared_ptr<Call> getCallFromCallID(const std::string &callID);
 
         /**
          * Process remaining participant given a conference and the current call id.
@@ -802,11 +797,6 @@ class ManagerImpl {
          */
         std::string retrieveConfigPath() const;
 
-        /*
-         * Initialize zeroconf module and scanning
-         */
-        void initZeroconf();
-
         /**
          * Set current call ID to empty string
          */
@@ -841,6 +831,9 @@ class ManagerImpl {
         // Main thread
         std::unique_ptr<DTMF> dtmfKey_;
 
+        /** Buffer to generate DTMF */
+        AudioBuffer dtmfBuf_;
+
         /////////////////////
         // Protected by Mutex
         /////////////////////
@@ -885,12 +878,6 @@ class ManagerImpl {
          */
         std::string path_;
 
-#ifdef USE_ZEROCONF
-        // DNSService contain every zeroconf services
-        //  configuration detected on the network
-        DNSService *DNSService_;
-#endif
-
         std::map<std::string, bool> IPToIPMap_;
 
         bool isIPToIP(const std::string& callID) const;
@@ -899,10 +886,6 @@ class ManagerImpl {
          * Load the account map from configuration
          */
         int loadAccountMap(Conf::YamlParser &parser);
-        /**
-         * Load default account map (no configuration)
-         */
-        void loadDefaultAccountMap();
 
         /**
          * Instance of the MainBuffer for the whole application
@@ -942,7 +925,7 @@ class ManagerImpl {
          */
         Client* getClient();
 #ifdef SFL_VIDEO
-        VideoControls * getVideoControls();
+        VideoManager * getVideoManager();
 #endif
 
         /**
@@ -979,7 +962,7 @@ class ManagerImpl {
         IAXAccount *getIaxAccount(const std::string& accountID) const;
 #endif
 
-	/**
+        /**
          * Get a pointer to the IP2IP account
          * @return SIPAccount * Pointer to the IP2IP account
          */
@@ -1000,9 +983,16 @@ class ManagerImpl {
         VoIPLink* getAccountLink(const std::string& accountID);
 
         /**
-         * Send registration to all enabled accounts
+         * Free all ressources related to this account.
+         *   ***Current calls using this account are HANG-UP***
+         */
+        void freeAccount(const std::string& accountID);
+
+        /**
+         * Send registration for all enabled accounts
          */
         void registerAccounts();
+
         void saveHistory();
 
         /**
@@ -1014,9 +1004,19 @@ class ManagerImpl {
         void
         checkAudio();
 
+        /**
+         * Call periodically to poll for VoIP events */
+        void
+        pollEvents();
+
     private:
         NON_COPYABLE(ManagerImpl);
 
+        /**
+         * Send unregister for all enabled accounts
+         */
+        void unregisterAccounts();
+
         /**
          * Play the dtmf-associated sound
          * @param code  The pressed key
