@@ -41,6 +41,9 @@
 #if HAVE_ALSA
 #include "audio/alsa/alsalayer.h"
 #endif
+#if HAVE_JACK
+#include "audio/jack/jacklayer.h"
+#endif
 #if HAVE_PULSE
 #include "audio/pulseaudio/pulselayer.h"
 #endif
@@ -57,7 +60,6 @@
 const char * const Preferences::DFT_ZONE = "North America";
 const char * const Preferences::REGISTRATION_EXPIRE_KEY = "registrationexpire";
 
-namespace {
 // general preferences
 static const char * const ORDER_KEY = "order";
 static const char * const AUDIO_API_KEY = "audioApi";
@@ -66,7 +68,6 @@ static const char * const HISTORY_MAX_CALLS_KEY = "historyMaxCalls";
 static const char * const ZONE_TONE_CHOICE_KEY = "zoneToneChoice";
 static const char * const PORT_NUM_KEY = "portNum";
 static const char * const SEARCH_BAR_DISPLAY_KEY = "searchBarDisplay";
-static const char * const ZEROCONF_ENABLE_KEY = "zeroConfenable";
 static const char * const MD5_HASH_KEY = "md5Hash";
 
 // voip preferences
@@ -118,7 +119,6 @@ static const char * const TOGGLE_PICKUP_HANGUP_SHORT_KEY = "togglePickupHangup";
 static const char * const DFT_PULSE_LENGTH_STR = "250"; /** Default DTMF lenght */
 static const char * const ZRTP_ZIDFILE = "zidFile";     /** The filename used for storing ZIDs */
 static const char * const ALSA_DFT_CARD    = "0";          /** Default sound card index */
-} // end anonymous namespace
 
 Preferences::Preferences() :
     accountOrder_("")
@@ -128,7 +128,6 @@ Preferences::Preferences() :
     , registrationExpire_(180)
     , portNum_(5060)
     , searchBarDisplay_(true)
-    , zeroConfenable_(false)
     , md5Hash_(false)
 {}
 
@@ -197,7 +196,6 @@ void Preferences::serialize(Conf::YamlEmitter &emiter)
     portstr << portNum_;
     Conf::ScalarNode portNum(portstr.str());
     Conf::ScalarNode searchBarDisplay(searchBarDisplay_);
-    Conf::ScalarNode zeroConfenable(zeroConfenable_);
     Conf::ScalarNode md5Hash(md5Hash_);
 
     preferencemap.setKeyValue(ORDER_KEY, &order);
@@ -207,7 +205,6 @@ void Preferences::serialize(Conf::YamlEmitter &emiter)
     preferencemap.setKeyValue(REGISTRATION_EXPIRE_KEY, &registrationExpire);
     preferencemap.setKeyValue(PORT_NUM_KEY, &portNum);
     preferencemap.setKeyValue(SEARCH_BAR_DISPLAY_KEY, &searchBarDisplay);
-    preferencemap.setKeyValue(ZEROCONF_ENABLE_KEY, &zeroConfenable);
     preferencemap.setKeyValue(MD5_HASH_KEY, &md5Hash);
 
     emiter.serializePreference(&preferencemap, "preferences");
@@ -222,7 +219,6 @@ void Preferences::unserialize(const Conf::YamlNode &map)
     map.getValue(REGISTRATION_EXPIRE_KEY, &registrationExpire_);
     map.getValue(PORT_NUM_KEY, &portNum_);
     map.getValue(SEARCH_BAR_DISPLAY_KEY, &searchBarDisplay_);
-    map.getValue(ZEROCONF_ENABLE_KEY, &zeroConfenable_);
     map.getValue(MD5_HASH_KEY, &md5Hash_);
 }
 
@@ -354,31 +350,54 @@ AudioPreference::AudioPreference() :
     , playbackMuted_(false)
 {}
 
-namespace {
 #if HAVE_ALSA
-void checkSoundCard(int &card, AudioLayer::PCMType stream)
+static void
+checkSoundCard(int &card, DeviceType type)
 {
-    if (not AlsaLayer::soundCardIndexExists(card, stream)) {
+    if (not AlsaLayer::soundCardIndexExists(card, type)) {
         WARN(" Card with index %d doesn't exist or is unusable.", card);
         card = ALSA_DFT_CARD_ID;
     }
 }
 #endif
+
+#ifdef __ANDROID__
+AudioLayer* AudioPreference::createAudioLayer()
+{
+    return new OpenSLLayer(*this);
 }
+#else
 
 AudioLayer* AudioPreference::createAudioLayer()
 {
-#ifdef __ANDROID__
-    return new OpenSLLayer(*this);
+#if HAVE_JACK
+    if (audioApi_ == JACK_API_STR) {
+        if (system("jack_lsp > /dev/null") == 0) {
+            try {
+                return new JackLayer(*this);
+            } catch (const std::runtime_error &e) {
+                ERROR("%s", e.what());
+#if HAVE_PULSE
+                WARN("falling back to pulseaudio");
+                audioApi_ = PULSEAUDIO_API_STR;
+#elif HAVE_ALSA
+                audioApi_ = ALSA_API_STR;
+#else
+                throw;
+#endif
+            }
+        }
+    }
 #endif
 
 #if HAVE_PULSE
 
     if (audioApi_ == PULSEAUDIO_API_STR) {
-        if (system("pactl info > /dev/null") == 0)
+        try {
             return new PulseLayer(*this);
-        else
-            WARN("pulseaudio daemon not running, falling back to ALSA");
+        } catch (const std::runtime_error &e) {
+            WARN("Could not create pulseaudio layer, falling back to ALSA");
+        }
     }
 
 #endif
@@ -386,9 +405,9 @@ AudioLayer* AudioPreference::createAudioLayer()
 #if HAVE_ALSA
 
     audioApi_ = ALSA_API_STR;
-    checkSoundCard(alsaCardin_, AudioLayer::SFL_PCM_CAPTURE);
-    checkSoundCard(alsaCardout_, AudioLayer::SFL_PCM_PLAYBACK);
-    checkSoundCard(alsaCardring_, AudioLayer::SFL_PCM_RINGTONE);
+    checkSoundCard(alsaCardin_, DeviceType::CAPTURE);
+    checkSoundCard(alsaCardout_, DeviceType::PLAYBACK);
+    checkSoundCard(alsaCardring_, DeviceType::RINGTONE);
 
     return new AlsaLayer(*this);
 #else
@@ -396,15 +415,7 @@ AudioLayer* AudioPreference::createAudioLayer()
 #endif
 }
 
-AudioLayer* AudioPreference::switchAndCreateAudioLayer()
-{
-    if (audioApi_ == PULSEAUDIO_API_STR)
-        audioApi_ = ALSA_API_STR;
-    else
-        audioApi_ = PULSEAUDIO_API_STR;
-
-    return createAudioLayer();
-}
+#endif // __ANDROID__
 
 void AudioPreference::serialize(Conf::YamlEmitter &emitter)
 {
@@ -479,11 +490,12 @@ void AudioPreference::serialize(Conf::YamlEmitter &emitter)
 bool
 AudioPreference::setRecordPath(const std::string &r)
 {
-    if (fileutils::isDirectoryWritable(r)) {
-        recordpath_ = r;
+    std::string path = fileutils::expand_path(r);
+    if (fileutils::isDirectoryWritable(path)) {
+        recordpath_ = path;
         return true;
     } else {
-        ERROR("%s is not writable, cannot be the recording path", r.c_str());
+        ERROR("%s is not writable, cannot be the recording path", path.c_str());
         return false;
     }
 }
