@@ -30,22 +30,22 @@
  *  as that of the covered work.
  */
 
+#include "sdp.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "sdp.h"
-#include "logger.h"
-#include "manager.h"
-
-#include <algorithm>
 #include "sipaccount.h"
-
+#include "manager.h"
+#include "logger.h"
 
 #ifdef SFL_VIDEO
 #include "video/libav_utils.h"
 #endif
 
+#include <algorithm>
+
 using std::string;
 using std::map;
 using std::vector;
@@ -53,16 +53,18 @@ using std::stringstream;
 
 Sdp::Sdp(pj_pool_t *pool)
     : memPool_(pool)
-    , negotiator_(NULL)
-    , localSession_(NULL)
-    , remoteSession_(NULL)
-    , activeLocalSession_(NULL)
-    , activeRemoteSession_(NULL)
+    , negotiator_(nullptr)
+    , localSession_(nullptr)
+    , remoteSession_(nullptr)
+    , activeLocalSession_(nullptr)
+    , activeRemoteSession_(nullptr)
     , audio_codec_list_()
     , video_codec_list_()
-    , sessionAudioMedia_()
+    , sessionAudioMediaLocal_()
+    , sessionAudioMediaRemote_()
     , sessionVideoMedia_()
     , publishedIpAddr_()
+    , publishedIpAddrType_()
     , remoteIpAddr_()
     , localAudioDataPort_(0)
     , localAudioControlPort_(0)
@@ -83,26 +85,46 @@ Sdp::~Sdp()
 #endif
 }
 
-namespace {
-    bool hasPayload(const std::vector<sfl::AudioCodec*> &codecs, int pt)
-    {
-        for (const auto &i : codecs)
-            if (i and i->getPayloadType() == pt)
-                return true;
-        return false;
-    }
-
-    bool hasCodec(const std::vector<std::string> &codecs, const std::string &codec)
-    {
-        return std::find(codecs.begin(), codecs.end(), codec) != codecs.end();
-    }
+static bool
+hasPayload(const std::vector<sfl::AudioCodec*> &codecs, int pt)
+{
+    for (const auto &i : codecs)
+        if (i and i->getPayloadType() == pt)
+            return true;
+    return false;
 }
 
+static bool
+hasCodec(const std::vector<std::string> &codecs, const std::string &codec)
+{
+    return std::find(codecs.begin(), codecs.end(), codec) != codecs.end();
+}
+
+static std::string
+rtpmapToString(pjmedia_sdp_rtpmap *rtpmap)
+{
+    std::ostringstream os;
+    const std::string enc(rtpmap->enc_name.ptr, rtpmap->enc_name.slen);
+    const std::string param(rtpmap->param.ptr, rtpmap->param.slen);
+    os << enc << "/" << rtpmap->clock_rate;
+    if (not param.empty())
+        os << "/" << param;
+    return os.str();
+}
+
+static sfl::AudioCodec *
+findCodecByName(const std::string &codec)
+{
+    // try finding by name
+    return Manager::instance().audioCodecFactory.getCodec(codec);
+}
 
 void Sdp::setActiveLocalSdpSession(const pjmedia_sdp_session *sdp)
 {
     activeLocalSession_ = (pjmedia_sdp_session*) sdp;
 
+    sessionAudioMediaLocal_.clear();
+
     for (unsigned i = 0; i < activeLocalSession_->media_count; ++i) {
         pjmedia_sdp_media *current = activeLocalSession_->media[i];
 
@@ -120,12 +142,17 @@ void Sdp::setActiveLocalSdpSession(const pjmedia_sdp_session *sdp)
 
             if (!pj_stricmp2(&current->desc.media, "audio")) {
                 const unsigned long pt = pj_strtoul(&current->desc.fmt[fmt]);
-                if (pt != telephoneEventPayload_ and not hasPayload(sessionAudioMedia_, pt)) {
+                if (pt != telephoneEventPayload_ and not hasPayload(sessionAudioMediaLocal_, pt)) {
                     sfl::AudioCodec *codec = Manager::instance().audioCodecFactory.getCodec(pt);
                     if (codec)
-                        sessionAudioMedia_.push_back(codec);
-                    else
-                        ERROR("Could not get codec for payload type %lu", pt);
+                        sessionAudioMediaLocal_.push_back(codec);
+                    else {
+                        codec = findCodecByName(rtpmapToString(rtpmap));
+                        if (codec)
+                            sessionAudioMediaLocal_.push_back(codec);
+                        else
+                            ERROR("Could not get codec for name %.*s", rtpmap->enc_name.slen, rtpmap->enc_name.ptr);
+                    }
                 }
             } else if (!pj_stricmp2(&current->desc.media, "video")) {
                 const string codec(rtpmap->enc_name.ptr, rtpmap->enc_name.slen);
@@ -146,6 +173,8 @@ void Sdp::setActiveRemoteSdpSession(const pjmedia_sdp_session *sdp)
 
     activeRemoteSession_ = (pjmedia_sdp_session*) sdp;
 
+    sessionAudioMediaRemote_.clear();
+
     bool parsedTelelphoneEvent = false;
     for (unsigned i = 0; i < sdp->media_count; i++) {
         pjmedia_sdp_media *r_media = sdp->media[i];
@@ -165,20 +194,35 @@ void Sdp::setActiveRemoteSdpSession(const pjmedia_sdp_session *sdp)
 
             // add audio codecs from remote as needed
             for (unsigned fmt = 0; fmt < r_media->desc.fmt_count; ++fmt) {
+                static const pj_str_t STR_RTPMAP = { (char*) "rtpmap", 6 };
+                pjmedia_sdp_attr *rtpMapAttribute = pjmedia_sdp_media_find_attr(r_media, &STR_RTPMAP, &r_media->desc.fmt[fmt]);
+
+                if (!rtpMapAttribute) {
+                    ERROR("Could not find rtpmap attribute");
+                    break;
+                }
+
+                pjmedia_sdp_rtpmap *rtpmap;
+                pjmedia_sdp_attr_to_rtpmap(memPool_, rtpMapAttribute, &rtpmap);
 
                 const unsigned long pt = pj_strtoul(&r_media->desc.fmt[fmt]);
-                if (pt != telephoneEventPayload_ and not hasPayload(sessionAudioMedia_, pt)) {
+                if (pt != telephoneEventPayload_ and not hasPayload(sessionAudioMediaRemote_, pt)) {
                     sfl::AudioCodec *codec = Manager::instance().audioCodecFactory.getCodec(pt);
                     if (codec) {
                         DEBUG("Adding codec with new payload type %d", pt);
-                        sessionAudioMedia_.push_back(codec);
-                    } else
-                        DEBUG("Could not get codec for payload type %lu", pt);
+                        sessionAudioMediaRemote_.push_back(codec);
+                    } else {
+                        // Search by codec name, clock rate and param (channel count)
+                        codec = findCodecByName(rtpmapToString(rtpmap));
+                        if (codec)
+                            sessionAudioMediaRemote_.push_back(codec);
+                        else
+                            ERROR("Could not get codec for name %.*s", rtpmap->enc_name.slen, rtpmap->enc_name.ptr);
+                    }
                 }
             }
         }
     }
-    DEBUG("Ready to decode %u audio codecs", sessionAudioMedia_.size());
 }
 
 string Sdp::getSessionVideoCodec() const
@@ -190,24 +234,31 @@ string Sdp::getSessionVideoCodec() const
     return sessionVideoMedia_[0];
 }
 
-string Sdp::getAudioCodecNames() const
-{
-    std::string result;
-    char sep = ' ';
-    for (std::vector<sfl::AudioCodec*>::const_iterator i = sessionAudioMedia_.begin();
-         i != sessionAudioMedia_.end(); ++i) {
-        if (i == sessionAudioMedia_.end() - 1)
-            sep = '\0';
-        if (*i)
-            result += (*i)->getMimeSubtype() + sep;
-    }
-    return result;
-}
-
 std::vector<sfl::AudioCodec*>
 Sdp::getSessionAudioMedia() const
 {
-    return sessionAudioMedia_;
+    vector<sfl::AudioCodec*> codecs;
+
+    // Common codecs first
+    for (auto c : sessionAudioMediaLocal_) {
+        if (std::find(sessionAudioMediaRemote_.begin(), sessionAudioMediaRemote_.end(), c) != sessionAudioMediaRemote_.end())
+            codecs.push_back(c);
+    }
+    DEBUG("%u common audio codecs", codecs.size());
+
+    // Next, the other codecs we declared to be able to encode
+    for (auto c : sessionAudioMediaLocal_) {
+        if (std::find(codecs.begin(), codecs.end(), c) == codecs.end())
+            codecs.push_back(c);
+    }
+    // Finally, the remote codecs so we can decode them
+    for (auto c : sessionAudioMediaRemote_) {
+        if (std::find(codecs.begin(), codecs.end(), c) == codecs.end())
+            codecs.push_back(c);
+    }
+    DEBUG("Ready to decode %u audio codecs", codecs.size());
+
+    return codecs;
 }
 
 
@@ -300,26 +351,23 @@ Sdp::setMediaDescriptorLines(bool audio)
 
 void Sdp::addRTCPAttribute(pjmedia_sdp_media *med)
 {
-    std::ostringstream os;
-    os << publishedIpAddr_ << ":" << localAudioControlPort_;
-    const std::string str(os.str());
-    pj_str_t input_str = pj_str((char*) str.c_str());
-    pj_sockaddr outputAddr;
-    pj_status_t status = pj_sockaddr_parse(PJ_AF_UNSPEC, 0, &input_str, &outputAddr);
-    if (status != PJ_SUCCESS) {
-        ERROR("Could not parse address %s", str.c_str());
-        return;
-    }
-    pjmedia_sdp_attr *attr = pjmedia_sdp_attr_create_rtcp(memPool_, &outputAddr);
+    IpAddr outputAddr = publishedIpAddr_;
+    outputAddr.setPort(localAudioControlPort_);
+    pjmedia_sdp_attr *attr = pjmedia_sdp_attr_create_rtcp(memPool_, outputAddr.pjPtr());
     if (attr)
         pjmedia_sdp_attr_add(&med->attr_count, med->attr, attr);
 }
 
 void
-Sdp::setPublishedIP(const std::string &ip_addr)
+Sdp::setPublishedIP(const std::string &addr, pj_uint16_t addr_type)
 {
-    publishedIpAddr_ = ip_addr;
+    publishedIpAddr_ = addr;
+    publishedIpAddrType_ = addr_type;
     if (localSession_) {
+        if (addr_type == pj_AF_INET6())
+            localSession_->origin.addr_type = pj_str((char*) "IP6");
+        else
+            localSession_->origin.addr_type = pj_str((char*) "IP4");
         localSession_->origin.addr = pj_str((char*) publishedIpAddr_.c_str());
         localSession_->conn->addr = localSession_->origin.addr;
         if (pjmedia_sdp_validate(localSession_) != PJ_SUCCESS)
@@ -328,12 +376,23 @@ Sdp::setPublishedIP(const std::string &ip_addr)
 }
 
 void
-Sdp::updatePorts(const std::vector<pj_sockaddr_in> &sockets)
+Sdp::setPublishedIP(const IpAddr& ip_addr)
 {
-    localAudioDataPort_     = pj_ntohs(sockets[0].sin_port);
-    localAudioControlPort_  = pj_ntohs(sockets[1].sin_port);
-    localVideoDataPort_     = pj_ntohs(sockets[2].sin_port);
-    localVideoControlPort_  = pj_ntohs(sockets[3].sin_port);
+    setPublishedIP(ip_addr, ip_addr.getFamily());
+}
+
+void
+Sdp::updatePorts(const std::vector<pj_sockaddr> &sockets)
+{
+    localAudioDataPort_     = pj_sockaddr_get_port(&sockets[0]);
+    localAudioControlPort_  = pj_sockaddr_get_port(&sockets[1]);
+    if (sockets.size() > 3) {
+        localVideoDataPort_     = pj_sockaddr_get_port(&sockets[2]);
+        localVideoControlPort_  = pj_sockaddr_get_port(&sockets[3]);
+    } else {
+        localVideoDataPort_ = 0;
+        localVideoControlPort_ = 0;
+    }
 
     if (localSession_) {
         if (localSession_->media[0]) {
@@ -400,14 +459,13 @@ void Sdp::setLocalMediaAudioCapabilities(const vector<int> &selectedCodecs)
     }
 }
 
-namespace {
-    void printSession(const pjmedia_sdp_session *session)
-    {
-        char buffer[2048];
-        size_t size = pjmedia_sdp_print(session, buffer, sizeof(buffer));
-        string sessionStr(buffer, std::min(size, sizeof(buffer)));
-        DEBUG("%s", sessionStr.c_str());
-    }
+static void
+printSession(const pjmedia_sdp_session *session)
+{
+    char buffer[2048];
+    size_t size = pjmedia_sdp_print(session, buffer, sizeof(buffer));
+    string sessionStr(buffer, std::min(size, sizeof(buffer)));
+    DEBUG("%s", sessionStr.c_str());
 }
 
 int Sdp::createLocalSession(const vector<int> &selectedAudioCodecs, const vector<map<string, string> > &selectedVideoCodecs)
@@ -427,7 +485,10 @@ int Sdp::createLocalSession(const vector<int> &selectedAudioCodecs, const vector
     // Use Network Time Protocol format timestamp to ensure uniqueness.
     localSession_->origin.id = tv.sec + 2208988800UL;
     localSession_->origin.net_type = pj_str((char*) "IN");
-    localSession_->origin.addr_type = pj_str((char*) "IP4");
+    if (publishedIpAddrType_ == pj_AF_INET6())
+        localSession_->origin.addr_type = pj_str((char*) "IP6");
+    else
+        localSession_->origin.addr_type = pj_str((char*) "IP4");
     localSession_->origin.addr = pj_str((char*) publishedIpAddr_.c_str());
 
     localSession_->name = pj_str((char*) PACKAGE);
@@ -510,8 +571,10 @@ void Sdp::startNegotiation()
     const pjmedia_sdp_session *active_local;
     const pjmedia_sdp_session *active_remote;
 
-    if (pjmedia_sdp_neg_get_state(negotiator_) != PJMEDIA_SDP_NEG_STATE_WAIT_NEGO)
+    if (pjmedia_sdp_neg_get_state(negotiator_) != PJMEDIA_SDP_NEG_STATE_WAIT_NEGO) {
         WARN("Negotiator not in right state for negotiation");
+        return;
+    }
 
     if (pjmedia_sdp_neg_negotiate(memPool_, negotiator_, 0) != PJ_SUCCESS)
         return;
@@ -606,17 +669,15 @@ std::string Sdp::getOutgoingVideoCodec() const
     return string(codec_buf);
 }
 
-namespace {
-    vector<map<string, string> >::const_iterator
-        findCodecInList(const vector<map<string, string> > &codecs, const string &codec)
-        {
-            for (vector<map<string, string> >::const_iterator i = codecs.begin(); i != codecs.end(); ++i) {
-                map<string, string>::const_iterator name = i->find("name");
-                if (name != i->end() and (codec == name->second))
-                    return i;
-            }
-            return codecs.end();
-        }
+static vector<map<string, string> >::const_iterator
+findCodecInList(const vector<map<string, string> > &codecs, const string &codec)
+{
+    for (vector<map<string, string> >::const_iterator i = codecs.begin(); i != codecs.end(); ++i) {
+        map<string, string>::const_iterator name = i->find("name");
+        if (name != i->end() and (codec == name->second))
+            return i;
+    }
+    return codecs.end();
 }
 
 std::string
@@ -682,23 +743,22 @@ void Sdp::addZrtpAttribute(pjmedia_sdp_media* media, std::string hash)
         throw SdpException("Could not add zrtp attribute to media");
 }
 
-namespace {
-    // Returns index of desired media attribute, or -1 if not found */
-    int getIndexOfAttribute(const pjmedia_sdp_session * const session, const char * const type)
-    {
-        if (!session) {
-            ERROR("Session is NULL when looking for \"%s\" attribute", type);
-            return -1;
-        }
-        size_t i = 0;
-        while (i < session->media_count and pj_stricmp2(&session->media[i]->desc.media, type) != 0)
-            ++i;
-
-        if (i == session->media_count)
-            return -1;
-        else
-            return i;
+// Returns index of desired media attribute, or -1 if not found */
+static int
+getIndexOfAttribute(const pjmedia_sdp_session * const session, const char * const type)
+{
+    if (!session) {
+        ERROR("Session is NULL when looking for \"%s\" attribute", type);
+        return -1;
     }
+    size_t i = 0;
+    while (i < session->media_count and pj_stricmp2(&session->media[i]->desc.media, type) != 0)
+        ++i;
+
+    if (i == session->media_count)
+        return -1;
+    else
+        return i;
 }
 
 void Sdp::addAttributeToLocalAudioMedia(const char *attr)
