@@ -32,6 +32,10 @@
 #ifndef SDP_H_
 #define SDP_H_
 
+#include "global.h"
+#include "noncopyable.h"
+#include "ip_utils.h"
+
 #include <pjmedia/sdp.h>
 #include <pjmedia/sdp_neg.h>
 #include <pjsip/sip_transport.h>
@@ -40,13 +44,11 @@
 #include <pjmedia/errno.h>
 #include <pj/pool.h>
 #include <pj/assert.h>
+
 #include <vector>
 #include <string>
 #include <stdexcept>
 
-#include "global.h"
-#include "noncopyable.h"
-
 namespace sfl {
 class AudioCodec;
 }
@@ -153,11 +155,16 @@ class Sdp {
         /*
          * Write accessor. Set the local IP address that will be used in the sdp session
          */
-        void setPublishedIP(const std::string &ip_addr);
+        void setPublishedIP(const std::string &addr, pj_uint16_t addr_type =  pj_AF_UNSPEC());
+        void setPublishedIP(const IpAddr& addr);
 
         /*
          * Read accessor. Get the local IP address
          */
+        IpAddr getPublishedIPAddr() const {
+            return publishedIpAddr_;
+        }
+
         std::string getPublishedIP() const {
             return publishedIpAddr_;
         }
@@ -172,7 +179,7 @@ class Sdp {
             localVideoControlPort_ = port + 1;
         }
 
-        void updatePorts(const std::vector<pj_sockaddr_in> &sockets);
+        void updatePorts(const std::vector<pj_sockaddr> &sockets);
 
         /**
          * Return IP of destination
@@ -246,7 +253,6 @@ class Sdp {
 
         void setMediaTransportInfoFromRemoteSdp();
 
-        std::string getAudioCodecNames() const;
         std::string getSessionVideoCodec() const;
         std::vector<sfl::AudioCodec*> getSessionAudioMedia() const;
         // Sets @param settings with appropriate values and returns true if
@@ -306,10 +312,13 @@ class Sdp {
         /**
          * The codecs that will be used by the session (after the SDP negotiation)
          */
-        std::vector<sfl::AudioCodec *> sessionAudioMedia_;
+        std::vector<sfl::AudioCodec *> sessionAudioMediaLocal_;
+        std::vector<sfl::AudioCodec *> sessionAudioMediaRemote_;
         std::vector<std::string> sessionVideoMedia_;
 
         std::string publishedIpAddr_;
+        pj_uint16_t publishedIpAddrType_;
+
         std::string remoteIpAddr_;
 
         int localAudioDataPort_;
