@@ -29,27 +29,27 @@
  *  as that of the covered work.
  */
 
+#include "sip_utils.h"
+#include "logger.h"
+#include "utf8_utils.h"
+
 #include <pjsip.h>
-#include <pjlib.h>
 #include <pjsip_ua.h>
 #include <pjlib-util.h>
 #include <pjnath.h>
 #include <pjnath/stun_config.h>
 #include <pj/string.h>
-#include <pjsip/sip_msg.h>
 #include <pjsip/sip_types.h>
 #include <pjsip/sip_uri.h>
 #include <pj/list.h>
-#include "sip_utils.h"
 
-#include <vector>
-#include <algorithm>
 #include <netdb.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include "logger.h"
+#include <vector>
+#include <algorithm>
 
 std::string
 sip_utils::fetchHeaderValue(pjsip_msg *msg, const std::string &field)
@@ -125,6 +125,9 @@ sip_utils::parseDisplayName(const char * buffer)
       end_displayName = temp.find("<");
       if (end_displayName != std::string::npos) {
           begin_displayName = temp.find_first_not_of(" ", temp.find(":"));
+          if (begin_displayName == std::string::npos)
+              return "";
+
           // omit trailing/leading spaces
           begin_displayName++;
           end_displayName--;
@@ -138,9 +141,10 @@ sip_utils::parseDisplayName(const char * buffer)
     std::string displayName = temp.substr(begin_displayName + 1,
                                           end_displayName - begin_displayName - 1);
 
-    static const size_t MAX_DISPLAY_NAME_SIZE = 25;
-    if (displayName.size() > MAX_DISPLAY_NAME_SIZE)
-        return displayName.substr(0, MAX_DISPLAY_NAME_SIZE);
+    // Filter out invalid UTF-8 characters to avoid getting kicked from D-Bus
+    if (not utf8_validate(displayName)) {
+        return utf8_make_valid(displayName);
+    }
 
     return displayName;
 }
@@ -192,30 +196,31 @@ sip_utils::getIPList(const std::string &name)
     if (name.empty())
         return ipList;
 
+    //ERROR("sip_utils::getIPList %s", name.c_str());
+
     struct addrinfo *result;
     struct addrinfo hints;
-    memset(&hints, '\0', sizeof(hints));
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_DGRAM;
     hints.ai_flags = AI_ADDRCONFIG;
     /* resolve the domain name into a list of addresses */
-    const int error = getaddrinfo(name.c_str(), NULL, &hints, &result);
+    const int error = getaddrinfo(name.c_str(), nullptr, &hints, &result);
     if (error != 0) {
         DEBUG("getaddrinfo on \"%s\" failed: %s", name.c_str(), gai_strerror(error));
         return ipList;
     }
 
-    for (struct addrinfo *res = result; res != NULL; res = res->ai_next) {
-
+    for (struct addrinfo *res = result; res != nullptr; res = res->ai_next) {
         void *ptr = 0;
         std::vector<char> addrstr;
-        static const int AF_INET_STRLEN = 16, AF_INET6_STRLEN = 40;
         switch (res->ai_family) {
             case AF_INET:
-                addrstr.resize(AF_INET_STRLEN);
+                addrstr.resize(INET_ADDRSTRLEN);
                 ptr = &((struct sockaddr_in *) res->ai_addr)->sin_addr;
                 break;
             case AF_INET6:
-                addrstr.resize(AF_INET6_STRLEN);
+                addrstr.resize(INET6_ADDRSTRLEN);
                 ptr = &((struct sockaddr_in6 *) res->ai_addr)->sin6_addr;
                 break;
             default:
@@ -226,8 +231,9 @@ sip_utils::getIPList(const std::string &name)
         // don't add duplicates, and don't use an std::set because
         // we want this order preserved.
         const std::string tmp(addrstr.begin(), addrstr.end());
-        if (std::find(ipList.begin(), ipList.end(), tmp) == ipList.end())
+        if (std::find(ipList.begin(), ipList.end(), tmp) == ipList.end()) {
             ipList.push_back(tmp);
+        }
     }
 
     freeaddrinfo(result);
@@ -244,3 +250,12 @@ sip_utils::addContactHeader(const pj_str_t *contact_str, pjsip_tx_data *tdata)
     pjsip_msg_find_remove_hdr(tdata->msg, PJSIP_H_CONTACT, NULL);
     pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr*) contact);
 }
+
+
+void
+sip_utils::sip_strerror(pj_status_t code)
+{
+    char err_msg[PJ_ERR_MSG_SIZE];
+    pj_strerror(code, err_msg, sizeof err_msg);
+    ERROR("%d: %s", code, err_msg);
+}
