@@ -32,11 +32,13 @@
 #ifndef SIP_UTILS_H_
 #define SIP_UTILS_H_
 
+#include <pjsip/sip_msg.h>
+#include <pjlib.h>
+
+#include <utility>
 #include <string>
 #include <vector>
 
-#include <pjsip/sip_msg.h>
-
 struct pjsip_msg;
 
 namespace sip_utils {
@@ -58,6 +60,8 @@ namespace sip_utils {
     std::vector<std::string> getIPList(const std::string &name);
 
     void addContactHeader(const pj_str_t *contactStr, pjsip_tx_data *tdata);
+
+    void sip_strerror(pj_status_t code);
 }
 
 #endif // SIP_UTILS_H_
