@@ -39,16 +39,18 @@
 #include "config.h"
 #endif
 
+#include "siptransport.h"
+#include "account.h"
+#include "noncopyable.h"
+#include "ip_utils.h"
+
+#include <pjsip/sip_transport_tls.h>
+#include <pjsip/sip_types.h>
+#include <pjsip-ua/sip_regc.h>
+
 #include <vector>
 #include <map>
 
-#include "pjsip/sip_transport_tls.h"
-#include "pjsip/sip_types.h"
-#include "pjsip-ua/sip_regc.h"
-
-#include "noncopyable.h"
-#include "account.h"
-
 typedef std::vector<pj_ssl_cipher> CipherArray;
 
 namespace Conf {
@@ -203,9 +205,9 @@ class SIPAccount : public Account {
         void registerVoIPLink();
 
         /**
-         * Send unregistration and clean all related stuff ( calls , thread )
+         * Send unregistration.
          */
-        void unregisterVoIPLink();
+        void unregisterVoIPLink(std::function<void(bool)> cb = std::function<void(bool)>());
 
         /**
          * Start the keep alive function, once started, the account will be registered periodically
@@ -278,12 +280,12 @@ class SIPAccount : public Account {
 
         /**
          * Registration flag
-	 */
+         */
         bool isRegistered() const {
             return bRegister_;
         }
 
-	/**
+        /**
          * Set registration flag
          */
         void setRegister(bool result) {
@@ -307,6 +309,7 @@ class SIPAccount : public Account {
          * @return void
          */
         void setRegistrationInfo(pjsip_regc *regc) {
+            if (regc_) destroyRegistrationInfo();
             regc_ = regc;
         }
 
@@ -351,7 +354,7 @@ class SIPAccount : public Account {
          * account is set to TLS.
          */
         bool isTlsEnabled() const {
-            return transportType_ == PJSIP_TRANSPORT_TLS;
+            return tlsEnable_;
         }
 
         /**
@@ -362,7 +365,7 @@ class SIPAccount : public Account {
             return stunEnabled_;
         }
 
-        /*
+        /**
          * @return pj_str_t "From" uri based on account information.
          * From RFC3261: "The To header field first and foremost specifies the desired
          * logical" recipient of the request, or the address-of-record of the
@@ -373,7 +376,7 @@ class SIPAccount : public Account {
          */
         std::string getFromUri() const;
 
-        /*
+        /**
          * This method adds the correct scheme, hostname and append
          * the ;transport= parameter at the end of the uri, in accordance with RFC3261.
          * It is expected that "port" is present in the internal hostname_.
@@ -383,7 +386,7 @@ class SIPAccount : public Account {
          */
         std::string getToUri(const std::string& username) const;
 
-        /*
+        /**
          * In the current version of SFLPhone, "srv" uri is obtained in the preformated
          * way: hostname:port. This method adds the correct scheme and append
          * the ;transport= parameter at the end of the uri, in accordance with RFC3261.
@@ -402,7 +405,7 @@ class SIPAccount : public Account {
         /**
          * Get the local interface name on which this account is bound.
          */
-        std::string getLocalInterface() const {
+        const std::string& getLocalInterface() const {
             return interface_;
         }
 
@@ -462,16 +465,20 @@ class SIPAccount : public Account {
          * Get the public IP address set by the user for this account.
          * If this setting is not provided, the local bound adddress
          * will be used.
-         * @return std::string The public IPV4 address formatted in the standard dot notation.
+         * @return std::string The public IPv4 or IPv6 address formatted in standard notation.
          */
         std::string getPublishedAddress() const {
             return publishedIpAddress_;
         }
 
-        void setPublishedAddress(const std::string &ip_addr) {
-            publishedIpAddress_ = ip_addr;
+        IpAddr getPublishedIpAddress() const {
+            return publishedIp_;
         }
 
+        void setPublishedAddress(const IpAddr& ip_addr) {
+            publishedIp_ = ip_addr;
+            publishedIpAddress_ = ip_addr.toString();
+        }
 
         std::string getServiceRoute() const {
             return serviceRoute_;
@@ -536,10 +543,22 @@ class SIPAccount : public Account {
             return keepAliveEnabled_;
         }
 
+        inline pjsip_transport* getTransport() {
+            return transport_;
+        }
+
+        void setTransport(pjsip_transport* transport = nullptr, pjsip_tpfactory* lis = nullptr);
+
+        inline pjsip_transport_type_e getTransportType() const {
+            return transportType_;
+        }
+
         /**
-         * Pointer to the transport used by this acccount
+         * Shortcut for SipTransport::getTransportSelector(account.getTransport()).
          */
-        pjsip_transport* transport_;
+        inline pjsip_tpselector getTransportSelector() {
+            return SipTransport::getTransportSelector(transport_);
+        }
 
         /* Returns true if the username and/or hostname match this account */
         MatchRank matches(const std::string &username, const std::string &hostname, pjsip_endpoint *endpt, pj_pool_t *pool) const;
@@ -600,6 +619,16 @@ class SIPAccount : public Account {
          */
         std::vector< std::map<std::string, std::string > > credentials_;
 
+        /**
+         * Pointer to the transport used by this acccount
+         */
+        pjsip_transport* transport_;
+
+        /**
+         * If a TLS tranport, pointer to the tls listener.
+         */
+        pjsip_tpfactory* tlsListener_;
+
 #if HAVE_TLS
         /**
          * Maps a string description of the SSL method
@@ -619,6 +648,7 @@ class SIPAccount : public Account {
          * great, so this function forces our cipher list to fit this constraint.
          */
         void trimCiphers();
+
 #endif
 
         /**
@@ -661,9 +691,10 @@ class SIPAccount : public Account {
         bool publishedSameasLocal_;
 
         /**
-         * Published IP address, ued only if defined by the user in account
+         * Published IP address, used only if defined by the user in account
          * configuration
          */
+        IpAddr publishedIp_;
         std::string publishedIpAddress_;
 
         /**
@@ -744,7 +775,6 @@ class SIPAccount : public Account {
         bool tlsVerifyClient_;
         bool tlsRequireClientCertificate_;
         std::string tlsNegotiationTimeoutSec_;
-        std::string tlsNegotiationTimeoutMsec_;
 
         /**
          * The stun server hostname (optional), used to provide the public IP address in case the softphone
@@ -818,7 +848,7 @@ class SIPAccount : public Account {
         /**
          * Voice over IP Link contains a listener thread and calls
          */
-        SIPVoIPLink* link_;
+        SIPVoIPLink& link_;
 
         /**
          * Optional: "received" parameter from VIA header
@@ -841,6 +871,7 @@ class SIPAccount : public Account {
         bool allowViaRewrite_;
         /* Undocumented feature in pjsip, this can == 2 */
         int allowContactRewrite_;
+        bool contactOverwritten_;
         pjsip_transport *via_tp_;
 
         /*
