@@ -112,10 +112,10 @@ class SIPCall : public Call {
 
         void setContactHeader(pj_str_t *contact);
 
+        void onhold();
+        void offhold(const std::function<void()> &SDPUpdateFunc);
+
     private:
-        // override of Call::getDetails
-        std::map<std::string, std::string>
-        getDetails();
 
         // override of Call::createHistoryEntry
         std::map<std::string, std::string>
