@@ -2,6 +2,7 @@
  *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
+ *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,50 +33,34 @@
 #ifndef SIPTRANSPORT_H_
 #define SIPTRANSPORT_H_
 
-#include <string>
-#include <vector>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "noncopyable.h"
+#include "logger.h"
 
 #include <pjsip.h>
-#include <pjlib.h>
 #include <pjsip_ua.h>
+#include <pjlib.h>
 #include <pjlib-util.h>
 #include <pjnath.h>
 #include <pjnath/stun_config.h>
-#include "noncopyable.h"
 
-#include "config.h"
+#include <functional>
+#include <mutex>
+#include <condition_variable>
+#include <map>
+#include <string>
+#include <vector>
+#include <memory>
 
 class SIPAccount;
 
+
 class SipTransport {
     public:
-        SipTransport(pjsip_endpoint *endpt, pj_caching_pool *cp, pj_pool_t *pool);
-        static std::string getSIPLocalIP();
-
-        /**
-         * Get the IP for the network interface named ifaceName
-         */
-        static std::string getInterfaceAddrFromName(const std::string &ifaceName);
-
-        /**
-        * List all the interfaces on the system and return
-        * a vector list containing their name (eth0, eth0:1 ...).
-        * @param void
-        * @return std::vector<std::string> A std::string vector
-        * of interface name available on all of the interfaces on
-        * the system.
-        */
-        static std::vector<std::string> getAllIpInterfaceByName();
-
-        /**
-         * List all the interfaces on the system and return
-         * a vector list containing their IPV4 address.
-         * @param void
-         * @return std::vector<std::string> A std::string vector
-         * of IPV4 address available on all of the interfaces on
-         * the system.
-         */
-        static std::vector<std::string> getAllIpInterface();
+        SipTransport(pjsip_endpoint *endpt, pj_caching_pool& cp, pj_pool_t& pool);
+        ~SipTransport();
 
         /**
          * General Sip transport creation method according to the
@@ -86,19 +71,18 @@ class SipTransport {
 
         /**
          * Initialize the transport selector
-         * @param transport		A transport associated with an account
-         *
-         * @return          	A pointer to the transport selector structure
+         * @param transport     A transport associated with an account
+         * @return          	A transport selector structure
          */
-        pjsip_tpselector *
-        createTransportSelector(pjsip_transport *transport, pj_pool_t *tp_pool) const;
+        static inline pjsip_tpselector getTransportSelector(pjsip_transport *transport) {
+            pjsip_tpselector tp = {PJSIP_TPSELECTOR_TRANSPORT, {transport}};
+            return tp;
+        }
 
         /**
          * This function returns a list of STUN mapped sockets for
          * a given set of socket file descriptors */
-        std::vector<pj_sockaddr_in>
-        getSTUNAddresses(const SIPAccount &account,
-                         std::vector<long> &socks) const;
+        std::vector<pj_sockaddr> getSTUNAddresses(const SIPAccount &account, std::vector<long> &socks) const;
 
         /**
          * Get the correct address to use (ie advertised) from
@@ -108,10 +92,23 @@ class SipTransport {
          * @param uri The uri from which we want to discover the address to use
          * @param transport The transport to use to discover the address
          */
-        void findLocalAddressFromTransport(pjsip_transport *transport, pjsip_transport_type_e transportType, std::string &address, std::string &port) const;
+        void findLocalAddressFromTransport(pjsip_transport *transport, pjsip_transport_type_e transportType, const std::string &host, std::string &address, pj_uint16_t &port) const;
 
         void findLocalAddressFromSTUN(pjsip_transport *transport, pj_str_t *stunServerName,
-                int stunPort, std::string &address, std::string &port) const;
+                int stunPort, std::string &address, pj_uint16_t &port) const;
+
+        /**
+         * Go through the transport list and remove unused ones.
+         */
+        void cleanupTransports();
+
+        /**
+         * Call released_cb(success) when transport tp is destroyed, making the
+         * socket available for a new similar transport.
+         * success is true if the transport is actually released.
+         * TODO: make this call non-blocking.
+         */
+        void waitForReleased(pjsip_transport* tp, std::function<void(bool)> released_cb);
 
     private:
         NON_COPYABLE(SipTransport);
@@ -131,30 +128,46 @@ class SipTransport {
          * Create The default TLS listener which is global to the application. This means that
          * only one TLS connection can be established for the momment.
          * @param the SIPAccount for which we are creating the TLS listener
+         * @param IP protocol version to use, can be pj_AF_INET() or pj_AF_INET6()
          * @return a pointer to the new listener
          */
         pjsip_tpfactory *
-        createTlsListener(SIPAccount &account);
+        createTlsListener(SIPAccount &account, pj_uint16_t family = pj_AF_UNSPEC());
 #endif
 
         /**
         * Create SIP UDP transport from account's setting
         * @param account The account for which a transport must be created.
+        * @param IP protocol version to use, can be pj_AF_INET() or pj_AF_INET6()
+        * @return a pointer to the new transport
         */
         pjsip_transport *createUdpTransport(const std::string &interface,
-                                            unsigned int port);
+                                            pj_uint16_t port, pj_uint16_t family = pj_AF_UNSPEC());
+
+        /**
+         * Go through the transport list and remove unused ones.
+         * Returns a list of LOCKED transports that have to be processed and unlocked.
+         */
+        std::vector<pjsip_transport*> _cleanupTransports();
+
+        static void tp_state_callback(pjsip_transport *, pjsip_transport_state, const pjsip_transport_state_info *);
+
+        void transportStateChanged(pjsip_transport* tp, pjsip_transport_state state);
 
         /**
          * UDP Transports are stored in this map in order to retreive them in case
          * several accounts would share the same port number.
          */
         std::map<std::string, pjsip_transport*> transportMap_;
+        std::mutex transportMapMutex_;
+        std::condition_variable transportDestroyedCv_;
 
-        pj_caching_pool *cp_;
-
-        pj_pool_t *pool_;
+        pj_caching_pool& cp_;
+        pj_pool_t& pool_;
 
         pjsip_endpoint *endpt_;
 };
 
+void sip_strerror(pj_status_t code);
+
 #endif // SIPTRANSPORT_H_
