@@ -40,28 +40,28 @@
 #include "config.h"
 #endif
 
-#include <map>
-#include <mutex>
+#include "voiplink.h"
+#include "sipaccount.h"
+#include "siptransport.h"
+
+#include <pjsip.h>
+#include <pjlib.h>
+#include <pjsip_ua.h>
+#include <pjlib-util.h>
+#include <pjnath.h>
+#include <pjnath/stun_config.h>
 
-#include "pjsip.h"
-#include "pjlib.h"
-#include "pjsip_ua.h"
-#include "pjlib-util.h"
-#include "pjnath.h"
-#include "pjnath/stun_config.h"
 #ifdef SFL_VIDEO
 #include <queue>
 #endif
-
-#include "sipaccount.h"
-#include "voiplink.h"
-#include "siptransport.h"
-#include "eventthread.h"
+#include <map>
+#include <mutex>
+#include <memory>
 
 class SIPCall;
 class SIPAccount;
 
-typedef std::map<std::string, SIPCall*> SipCallMap;
+typedef std::map<std::string, std::shared_ptr<SIPCall> > SipCallMap;
 /**
  * @file sipvoiplink.h
  * @brief Specific VoIPLink for SIP (SIP core for incoming and outgoing events).
@@ -77,7 +77,7 @@ class SIPVoIPLink : public VoIPLink {
          * Singleton method. Enable to retrieve the unique static instance
          * @return SIPVoIPLink* A pointer on the object
          */
-        static SIPVoIPLink* instance();
+        static SIPVoIPLink& instance();
 
         /**
          * Destroy the singleton instance
@@ -98,7 +98,7 @@ class SIPVoIPLink : public VoIPLink {
         /**
          * Event listener. Each event send by the call manager is received and handled from here
          */
-        virtual bool getEvent();
+        virtual bool handleEvents();
 
         /* Returns a list of all callIDs */
         std::vector<std::string>
@@ -110,15 +110,18 @@ class SIPVoIPLink : public VoIPLink {
         AccountMap &
         getAccounts() { return sipAccountMap_; }
 
+        virtual std::vector<std::shared_ptr<Call> > getCalls(const std::string &account_id) const;
+
         /**
          * Build and send SIP registration request
          */
-        virtual void sendRegister(Account *a);
+        virtual void sendRegister(Account& a);
 
         /**
          * Build and send SIP unregistration request
+         * @param destroy_transport If true, attempt to destroy the transport.
          */
-        virtual void sendUnregister(Account *a);
+        virtual void sendUnregister(Account& a, std::function<void(bool)> cb = std::function<void(bool)>());
 
         /**
          * Register a new keepalive registration timer to this endpoint
@@ -136,21 +139,23 @@ class SIPVoIPLink : public VoIPLink {
          * @param toUrl  The Sip address of the recipient of the call
          * @return Call* The current call
          */
-        virtual Call* newOutgoingCall(const std::string& id, const std::string& toUrl, const std::string &account_id);
+        virtual std::shared_ptr<Call> newOutgoingCall(const std::string& id, const std::string& toUrl, const std::string &account_id);
 
         /**
          * Start a new SIP call using the IP2IP profile
          * @param The call id
          * @param The target sip uri
          */
-        Call *SIPNewIpToIpCall(const std::string& id, const std::string& to);
+        std::shared_ptr<Call> SIPNewIpToIpCall(const std::string& id, const std::string& to);
 
         /**
          * Place a call using the currently selected account
          * @param The call id
          * @param The target sip uri
          */
-        Call *newRegisteredAccountCall(const std::string& id, const std::string& toUrl, const std::string &account_id);
+        std::shared_ptr<Call> newRegisteredAccountCall(const std::string& id,
+                                                       const std::string& toUrl,
+                                                       const std::string &account_id);
 
         /**
          * Answer the call
@@ -240,23 +245,6 @@ class SIPVoIPLink : public VoIPLink {
          */
         pj_caching_pool *getMemoryPoolFactory();
 
-        /**
-         * A non-blocking SIPCall accessor
-         *
-         * Will return NULL if the callMapMutex could not be locked
-         *
-         * @param id  The call identifier
-         * @return SIPCall* A pointer to the SIPCall object
-         */
-        SIPCall* tryGetSIPCall(const std::string &id);
-
-        /**
-         * Return the codec protocol used for this call
-         * @param c The call identifier
-         */
-        std::string getCurrentVideoCodecName(Call *c) const;
-        std::string getCurrentAudioCodecNames(Call *c) const;
-
         /**
          * Retrive useragent name from account
          */
@@ -275,10 +263,20 @@ class SIPVoIPLink : public VoIPLink {
                              const std::string& from);
 #endif
         void clearSipCallMap();
-        void addSipCall(SIPCall* call);
+        void addSipCall(std::shared_ptr<SIPCall>& call);
+
+        std::shared_ptr<SIPCall> getSipCall(const std::string& id);
+
+        /**
+         * A non-blocking SIPCall accessor
+         *
+         * Will return NULL if the callMapMutex could not be locked
+         *
+         * @param id  The call identifier
+         * @return SIPCall* A pointer to the SIPCall object
+         */
+        std::shared_ptr<SIPCall> tryGetSIPCall(const std::string &id);
 
-        SIPCall* getSipCall(const std::string& id);
-        SIPCall* tryGetSipCall(const std::string& id);
         void removeSipCall(const std::string &id);
 
         /**
@@ -287,6 +285,8 @@ class SIPVoIPLink : public VoIPLink {
         void createDefaultSipUdpTransport();
 
     public:
+        void loadIP2IPSettings();
+
         /**
          * Instance that maintain and manage transport (UDP, TLS)
          */
@@ -314,7 +314,7 @@ class SIPVoIPLink : public VoIPLink {
          */
         AccountMap sipAccountMap_;
 
-        std::mutex sipCallMapMutex_;
+        mutable std::mutex sipCallMapMutex_;
         SipCallMap sipCallMap_;
 
         /**
@@ -322,12 +322,7 @@ class SIPVoIPLink : public VoIPLink {
          * @param call  The current call
          * @return true if all is correct
          */
-        bool SIPStartCall(SIPCall* call);
-
-        /**
-         * Threading object
-         */
-        EventThread evThread_;
+        bool SIPStartCall(std::shared_ptr<SIPCall>& call);
 
 #ifdef SFL_VIDEO
         void dequeKeyframeRequests();
@@ -336,7 +331,7 @@ class SIPVoIPLink : public VoIPLink {
         std::queue<std::string> keyframeRequests_;
 #endif
 
-        static SIPVoIPLink *instance_;
+        static SIPVoIPLink * instance_;
 
         friend class SIPTest;
 };
