@@ -43,18 +43,24 @@
 #include <sys/socket.h>
 #include <netdb.h>
 
+#if defined(__ANDROID__) && !defined(SOCK_NONBLOCK)
+#include <asm-generic/fcntl.h>
+# define SOCK_NONBLOCK O_NONBLOCK
+#endif
 
-namespace {
+static const int NET_POLL_TIMEOUT = 100; /* poll() timeout in ms */
 
-int ff_network_wait_fd(int fd)
+static int
+ff_network_wait_fd(int fd)
 {
     struct pollfd p = { fd, POLLOUT, 0 };
     int ret;
-    ret = poll(&p, 1, 100);
+    ret = poll(&p, 1, NET_POLL_TIMEOUT);
     return ret < 0 ? errno : p.revents & (POLLOUT | POLLERR | POLLHUP) ? 0 : -EAGAIN;
 }
 
-struct addrinfo* udp_resolve_host(const char *node, int service)
+static struct
+addrinfo* udp_resolve_host(const char *node, int service)
 {
     struct addrinfo hints, *res = 0;
     int error;
@@ -74,8 +80,8 @@ struct addrinfo* udp_resolve_host(const char *node, int service)
     return res;
 }
 
-unsigned udp_set_url(struct sockaddr_storage *addr,
-            const char *hostname, int port)
+static unsigned
+udp_set_url(struct sockaddr_storage *addr, const char *hostname, int port)
 {
     struct addrinfo *res0;
     int addr_len;
@@ -89,8 +95,8 @@ unsigned udp_set_url(struct sockaddr_storage *addr,
     return addr_len;
 }
 
-int udp_socket_create(sockaddr_storage *addr, socklen_t *addr_len,
-                  int local_port)
+static int
+udp_socket_create(sockaddr_storage *addr, socklen_t *addr_len, int local_port)
 {
     int udp_fd = -1;
     struct addrinfo *res0 = NULL, *res = NULL;
@@ -117,7 +123,7 @@ int udp_socket_create(sockaddr_storage *addr, socklen_t *addr_len,
     // on local port
     if (bind(udp_fd, reinterpret_cast<sockaddr*>(addr), *addr_len) < 0) {
         ERROR("Bind failed");
-        Logger::strErr();
+        strErr();
         close(udp_fd);
         udp_fd = -1;
     }
@@ -128,12 +134,10 @@ int udp_socket_create(sockaddr_storage *addr, socklen_t *addr_len,
     return udp_fd;
 }
 
-const int RTP_BUFFER_SIZE = 1472;
-
-}
-
 namespace sfl_video {
 
+static const int RTP_BUFFER_SIZE = 1472;
+
 SocketPair::SocketPair(const char *uri, int localPort) :
            rtcpWriteMutex_(),
            rtpHandle_(0),
@@ -161,9 +165,9 @@ void SocketPair::interrupt()
 void SocketPair::closeSockets()
 {
     if (rtcpHandle_ > 0 and close(rtcpHandle_))
-        Logger::strErr();
+        strErr();
     if (rtpHandle_ > 0 and close(rtpHandle_))
-        Logger::strErr();
+        strErr();
 }
 
 void SocketPair::openSockets(const char *uri, int local_rtp_port)
@@ -224,8 +228,11 @@ int SocketPair::readCallback(void *opaque, uint8_t *buf, int buf_size)
         }
 
         /* build fdset to listen to RTP and RTCP packets */
-        n = poll(p, 2, 100);
+        // FIXME:WORKAROUND: reduce to RTP handle until RTCP is fixed
+        n = poll(p, 1, NET_POLL_TIMEOUT);
         if (n > 0) {
+// FIXME:WORKAROUND: prevent excessive packet loss
+#if 0
             /* first try RTCP */
             if (p[1].revents & POLLIN) {
                 from_len = sizeof(from);
@@ -242,6 +249,7 @@ int SocketPair::readCallback(void *opaque, uint8_t *buf, int buf_size)
                 }
                 break;
             }
+#endif
             /* then RTP */
             if (p[0].revents & POLLIN) {
                 from_len = sizeof(from);
@@ -284,20 +292,27 @@ int SocketPair::writeCallback(void *opaque, uint8_t *buf, int buf_size)
 
     int ret;
 
+retry:
     if (RTP_PT_IS_RTCP(buf[1])) {
         /* RTCP payload type */
         std::lock_guard<std::mutex> lock(context->rtcpWriteMutex_);
         ret = ff_network_wait_fd(context->rtcpHandle_);
-        if (ret < 0)
+        if (ret < 0) {
+            if (ret == -EAGAIN)
+                goto retry;
             return ret;
+        }
 
         ret = sendto(context->rtcpHandle_, buf, buf_size, 0,
                      (sockaddr*) &context->rtcpDestAddr_, context->rtcpDestAddrLen_);
     } else {
         /* RTP payload type */
         ret = ff_network_wait_fd(context->rtpHandle_);
-        if (ret < 0)
+        if (ret < 0) {
+            if (ret == -EAGAIN)
+                goto retry;
             return ret;
+        }
 
         ret = sendto(context->rtpHandle_, buf, buf_size, 0,
                      (sockaddr*) &context->rtpDestAddr_, context->rtpDestAddrLen_);
