@@ -35,7 +35,10 @@
 #define __VOIP_LINK_H__
 
 #include <stdexcept>
+#include <functional>
 #include <string>
+#include <vector>
+#include <memory>
 
 class Call;
 class Account;
@@ -59,19 +62,20 @@ class VoIPLink {
          * Virtual method
          * Event listener. Each event send by the call manager is received and handled from here
          */
-        virtual bool getEvent() = 0;
+        virtual bool handleEvents() = 0;
 
         /**
          * Virtual method
          * Build and send account registration request
          */
-        virtual void sendRegister(Account *a) = 0;
+        virtual void sendRegister(Account& a) = 0;
 
         /**
          * Virtual method
          * Build and send account unregistration request
+         * Underlying ressources are released after unregistration
          */
-        virtual void sendUnregister(Account *a) = 0;
+        virtual void sendUnregister(Account& a, std::function<void(bool)> cb = std::function<void(bool)>()) = 0;
 
         /**
          * Place a new call
@@ -79,9 +83,15 @@ class VoIPLink {
          * @param toUrl  The address of the recipient of the call
          * @return Call* The current call
          */
-        virtual Call* newOutgoingCall(const std::string &id,
-                                      const std::string &toUrl,
-                                      const std::string &account_id) = 0;
+        virtual std::shared_ptr<Call> newOutgoingCall(const std::string &id,
+                                                      const std::string &toUrl,
+                                                      const std::string &account_id) = 0;
+
+        /**
+         * Virtual method
+         * Returns calls involving this account.
+         */
+        virtual std::vector<std::shared_ptr<Call> > getCalls(const std::string &account_id) const = 0;
 
         /**
          * Answer the call
@@ -143,13 +153,6 @@ class VoIPLink {
          */
         virtual void carryingDTMFdigits(const std::string &id, char code) = 0;
 
-        /**
-         * Return the codec protocol used for this call
-         * @param call The call
-         */
-        virtual std::string getCurrentVideoCodecName(Call *call) const = 0;
-        virtual std::string getCurrentAudioCodecNames(Call *call) const = 0;
-
         /**
          * Send a message to a call identified by its callid
          *
