@@ -51,19 +51,9 @@ class MainBufferTest : public CppUnit::TestCase {
          * Use cppunit library macros to add unit test the factory
          */
         CPPUNIT_TEST_SUITE(MainBufferTest);
-        CPPUNIT_TEST(testRingBufferCreation);
-        CPPUNIT_TEST(testRingBufferReadPointer);
-        CPPUNIT_TEST(testCallIDSet);
-        CPPUNIT_TEST(testRingBufferInt);
-        CPPUNIT_TEST(testRingBufferNonDefaultID);
-        //CPPUNIT_TEST(testRingBufferFloat);
-        CPPUNIT_TEST(testTwoPointer);
         CPPUNIT_TEST(testBindUnbindBuffer);
-        CPPUNIT_TEST(testGetPutDataByID);
         CPPUNIT_TEST(testGetPutData);
         CPPUNIT_TEST(testDiscardFlush);
-        CPPUNIT_TEST(testReadPointerInit);
-        CPPUNIT_TEST(testRingBufferSeveralPointers);
         CPPUNIT_TEST(testConference);
         CPPUNIT_TEST_SUITE_END();
 
@@ -71,34 +61,14 @@ class MainBufferTest : public CppUnit::TestCase {
 
         MainBufferTest();
 
-        void testRingBufferCreation();
-
-        void testRingBufferReadPointer();
-
-        void testCallIDSet();
-
-        void testRingBufferInt();
-
-        void testRingBufferNonDefaultID();
-
-        void testRingBufferFloat();
-
-        void testTwoPointer();
-
         void testBindUnbindBuffer();
 
-        void testGetPutDataByID();
-
         void testGetPutData();
 
-        void testAvailForGetPut();
+        void testGetAvailableData();
 
         void testDiscardFlush();
 
-        void testReadPointerInit();
-
-        void testRingBufferSeveralPointers();
-
         void testConference();
 
     private:
