@@ -179,12 +179,12 @@ void account_list_set_current(account_t *current)
 const gchar * account_state_name(account_state_t s)
 {
     switch (s) {
-        case ACCOUNT_STATE_REGISTERED:
-            return _("Registered");
         case ACCOUNT_STATE_UNREGISTERED:
             return _("Not Registered");
         case ACCOUNT_STATE_TRYING:
             return _("Trying...");
+        case ACCOUNT_STATE_REGISTERED:
+            return _("Registered");
         case ACCOUNT_STATE_ERROR:
             return _("Error");
         case ACCOUNT_STATE_ERROR_AUTH:
@@ -195,13 +195,14 @@ const gchar * account_state_name(account_state_t s)
             return _("Host unreachable");
         case ACCOUNT_STATE_ERROR_SERVICE_UNAVAILABLE:
             return _("Service unavailable");
-        case ACCOUNT_STATE_ERROR_NOT_ACCEPTABLE:
-            return _("Not acceptable");
         case ACCOUNT_STATE_ERROR_EXIST_STUN:
             return _("Stun server invalid");
+        case ACCOUNT_STATE_ERROR_NOT_ACCEPTABLE:
+            return _("Not acceptable");
         case ACCOUNT_STATE_IP2IP_READY:
             return _("Ready");
         default:
+            g_warning("Unexpected state %d", s);
             return _("Invalid");
     }
 }
@@ -235,7 +236,7 @@ account_list_move_up(guint account_index)
 void
 account_list_move_down(guint account_index)
 {
-    if (account_index != accountQueue->length) {
+    if (account_index < accountQueue->length-1) {
         gpointer acc = g_queue_pop_nth(accountQueue, account_index);
         g_queue_push_nth(accountQueue, acc, account_index + 1);
     }
@@ -331,6 +332,11 @@ gboolean current_account_has_new_message(void)
     return current && current->_messages_number > 0;
 }
 
+gboolean account_has_custom_user_agent(const account_t *account)
+{
+    return g_strcmp0(account_lookup(account, CONFIG_ACCOUNT_HAS_CUSTOM_USERAGENT), "true") == 0;
+}
+
 gboolean account_has_autoanswer_on(const account_t *account)
 {
     return g_strcmp0(account_lookup(account, CONFIG_ACCOUNT_AUTOANSWER), "true") == 0;
