@@ -66,9 +66,6 @@
 #include "sliders.h"
 #include "messaging/message_tab.h"
 #include "history_loader.h"
-#ifdef SFL_VIDEO
-#include "video/video_callbacks.h"
-#endif
 
 static GHashTable * ip2ip_profile;
 
@@ -154,9 +151,6 @@ void
 sflphone_quit(gboolean force_quit, SFLPhoneClient *client)
 {
     if (force_quit || calllist_get_size(current_calls_tab) == 0 || main_window_ask_quit(client)) {
-#ifdef SFL_VIDEO
-        video_cleanup();
-#endif
         dbus_unregister(getpid());
         dbus_clean();
         account_list_free();
@@ -177,14 +171,14 @@ void
 sflphone_hold(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_state = CALL_STATE_HOLD;
-    calltree_update_call(current_calls_tab, c, client, FALSE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
 sflphone_ringing(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_state = CALL_STATE_RINGING;
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
@@ -225,22 +219,24 @@ void sflphone_fill_account_list(void)
             acc->state = ACCOUNT_STATE_UNREGISTERED;
         else if (g_strcmp0(status, "TRYING") == 0)
             acc->state = ACCOUNT_STATE_TRYING;
-        else if (g_strcmp0(status, "g_warning") == 0)
+        else if (g_strcmp0(status, "ERROR_GENERIC") == 0)
             acc->state = ACCOUNT_STATE_ERROR;
-        else if (g_strcmp0(status, "g_warning_AUTH") == 0)
+        else if (g_strcmp0(status, "ERROR_AUTH") == 0)
             acc->state = ACCOUNT_STATE_ERROR_AUTH;
-        else if (g_strcmp0(status, "g_warning_NETWORK") == 0)
+        else if (g_strcmp0(status, "ERROR_NETWORK") == 0)
             acc->state = ACCOUNT_STATE_ERROR_NETWORK;
-        else if (g_strcmp0(status, "g_warning_HOST") == 0)
+        else if (g_strcmp0(status, "ERROR_HOST") == 0)
             acc->state = ACCOUNT_STATE_ERROR_HOST;
-        else if (g_strcmp0(status, "g_warning_NOT_ACCEPTABLE") == 0)
+        else if (g_strcmp0(status, "ERROR_NOT_ACCEPTABLE") == 0)
             acc->state = ACCOUNT_STATE_ERROR_NOT_ACCEPTABLE;
-        else if (g_strcmp0(status, "g_warning_EXIST_STUN") == 0)
+        else if (g_strcmp0(status, "ERROR_EXIST_STUN") == 0)
             acc->state = ACCOUNT_STATE_ERROR_EXIST_STUN;
-        else if (g_strcmp0(status, "ACCOUNT_STATE_IP2IP_READY") == 0)
+        else if (g_strcmp0(status, "READY") == 0)
             acc->state = ACCOUNT_STATE_IP2IP_READY;
-        else
+        else {
+            g_warning("Unexpected status %s", status);
             acc->state = ACCOUNT_STATE_INVALID;
+        }
 
         gchar * code = account_lookup(acc, CONFIG_ACCOUNT_REGISTRATION_STATE_CODE);
         if (code != NULL)
@@ -282,7 +278,7 @@ gboolean sflphone_init(GError **error, SFLPhoneClient *client)
 
 void sflphone_fill_ip2ip_profile(void)
 {
-    ip2ip_profile = (GHashTable *) dbus_get_ip2_ip_details();
+    ip2ip_profile = dbus_get_ip2ip_details();
 }
 
 GHashTable *sflphone_get_ip2ip_properties(void)
@@ -341,7 +337,7 @@ sflphone_hang_up(SFLPhoneClient *client)
         }
     }
 
-    calltree_update_call(history_tab, selectedCall, client, FALSE);
+    calltree_update_call(history_tab, selectedCall, client);
 
     statusbar_update_clock("");
 
@@ -370,7 +366,7 @@ sflphone_pick_up(SFLPhoneClient *client)
             break;
         case CALL_STATE_INCOMING:
             selectedCall->_history_state = g_strdup(INCOMING_STRING);
-            calltree_update_call(history_tab, selectedCall, client, TRUE);
+            calltree_update_call(history_tab, selectedCall, client);
 
             dbus_accept(selectedCall);
             break;
@@ -432,14 +428,14 @@ void
 sflphone_fail(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_state = CALL_STATE_FAILURE;
-    calltree_update_call(current_calls_tab, c, client, FALSE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
 sflphone_busy(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_state = CALL_STATE_BUSY;
-    calltree_update_call(current_calls_tab, c, client, FALSE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
@@ -449,7 +445,7 @@ sflphone_current(callable_obj_t * c, SFLPhoneClient *client)
         time(&c->_time_start);
 
     c->_state = CALL_STATE_CURRENT;
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
@@ -461,7 +457,7 @@ sflphone_set_transfer(SFLPhoneClient *client)
         c->_state = CALL_STATE_TRANSFER;
         g_free(c->_trsft_to);
         c->_trsft_to = g_strdup("");
-        calltree_update_call(current_calls_tab, c, client, FALSE);
+        calltree_update_call(current_calls_tab, c, client);
     } else {
         update_actions(client);
     }
@@ -476,7 +472,7 @@ sflphone_unset_transfer(SFLPhoneClient *client)
         c->_state = CALL_STATE_CURRENT;
         g_free(c->_trsft_to);
         c->_trsft_to = g_strdup("");
-        calltree_update_call(current_calls_tab, c, client, FALSE);
+        calltree_update_call(current_calls_tab, c, client);
     } else {
         update_actions(client);
     }
@@ -536,7 +532,7 @@ process_dialing(callable_obj_t *c, guint keyval, const gchar *key, SFLPhoneClien
             if (len) {
                 len--; // delete one character
                 num[len] = '\0';
-                calltree_update_call(current_calls_tab, c, client, FALSE);
+                calltree_update_call(current_calls_tab, c, client);
 
                 /* If number is now empty, hang up immediately */
                 if (c->_state != CALL_STATE_TRANSFER && len == 0)
@@ -570,7 +566,7 @@ process_dialing(callable_obj_t *c, guint keyval, const gchar *key, SFLPhoneClien
                     c->_peer_number = new_peer_number;
                 }
 
-                calltree_update_call(current_calls_tab, c, client, FALSE);
+                calltree_update_call(current_calls_tab, c, client);
             }
 
             break;
@@ -589,11 +585,11 @@ sflphone_new_call(SFLPhoneClient *client)
     callable_obj_t *selected_call = calllist_empty(current_calls_tab) ? NULL :
         calltab_get_selected_call(current_calls_tab);
 
-    if (selected_call != NULL) {
+    if (selected_call != NULL && selected_call->_callID) {
         gchar *confID = dbus_get_conference_id(selected_call->_callID);
-        if(g_strcmp0(confID, "") != 0) {
+        if (g_strcmp0(confID, "") != 0)
             sflphone_on_hold();
-        }
+        g_free(confID);
     }
 
     // Play a tone when creating a new call
@@ -642,7 +638,7 @@ sflphone_keypad(guint keyval, const gchar * key, SFLPhoneClient *client)
                     case GDK_KEY_Escape:
                         dbus_hang_up(c);
                         time(&c->_time_stop);
-                        calltree_update_call(history_tab, c, client, FALSE);
+                        calltree_update_call(history_tab, c, client);
                         break;
                     default:
                         // To play the dtmf when calling mail box for instance
@@ -657,7 +653,7 @@ sflphone_keypad(guint keyval, const gchar * key, SFLPhoneClient *client)
                     case GDK_KEY_Return:
                     case GDK_KEY_KP_Enter:
                         c->_history_state = g_strdup(INCOMING_STRING);
-                        calltree_update_call(history_tab, c, client, TRUE);
+                        calltree_update_call(history_tab, c, client);
                         dbus_accept(c);
                         break;
                     case GDK_KEY_Escape:
@@ -705,7 +701,7 @@ sflphone_keypad(guint keyval, const gchar * key, SFLPhoneClient *client)
                 switch (keyval) {
                     case GDK_KEY_Escape:
                         dbus_hang_up(c);
-                        calltree_update_call(history_tab, c, client, FALSE);
+                        calltree_update_call(history_tab, c, client);
                         break;
                 }
 
@@ -827,7 +823,7 @@ sflphone_rec_call(SFLPhoneClient *client)
 
     if (selectedCall) {
         result = dbus_toggle_recording(selectedCall->_callID);
-        calltree_update_call(current_calls_tab, selectedCall, client, TRUE);
+        calltree_update_call(current_calls_tab, selectedCall, client);
     } else if (selectedConf) {
         result = dbus_toggle_recording(selectedConf->_confID);
 
@@ -867,21 +863,30 @@ sflphone_fill_audio_codec_list_per_account(account_t *account)
     else
         g_queue_clear(account->acodecs);
 
+    GQueue *system_acodecs = get_audio_codecs_list();
+    if (!system_acodecs) {  // should never happen
+        g_warning("Couldn't get codec list in %s at %d", __FILE__, __LINE__);
+        return;
+    }
+
     /* First add the active codecs for this account */
     GArray *order = dbus_get_active_audio_codec_list(account->accountID);
-    GQueue *system_acodecs = get_audio_codecs_list();
-    for (guint i = 0; i < order->len; i++) {
-        gint payload = g_array_index(order, gint, i);
-        codec_t *orig = codec_list_get_by_payload(payload, system_acodecs);
-        codec_t *c = codec_create_new_from_caps(orig);
+    if (order) {
+        for (guint i = 0; i < order->len; i++) {
+            gint payload = g_array_index(order, gint, i);
+            codec_t *orig = codec_list_get_by_payload(payload, system_acodecs);
+            codec_t *c = codec_create_new_from_caps(orig);
 
-        if (c) {
-            c->is_active = TRUE;
-            g_queue_push_tail(account->acodecs, c);
-        } else
-            g_warning("Couldn't find codec %d %p", payload, orig);
+            if (c) {
+                c->is_active = TRUE;
+                g_queue_push_tail(account->acodecs, c);
+            } else
+                g_warning("Couldn't find codec %d %p", payload, orig);
+        }
+        g_array_unref(order);
+    } else {
+        g_warning("SFLphone: Error: order obj is NULL in %s at %d", __FILE__, __LINE__);
     }
-    g_array_unref(order);
 
     /* Here we add installed codecs that aren't active for the account */
     guint caps_size = g_queue_get_length(system_acodecs);
@@ -904,16 +909,17 @@ void sflphone_fill_call_list(void)
     gchar **call_list = dbus_get_call_list();
 
     for (gchar **callp = call_list; callp && *callp; ++callp) {
-        gchar *callID = *callp;
+        const gchar *callID = *callp;
         if (!calllist_get_call(current_calls_tab, callID)) {
-            callable_obj_t *call = create_new_call_from_details(*callp, dbus_get_call_details(*callp));
+            callable_obj_t *call = create_new_call_from_details(callID, dbus_get_call_details(callID));
             call->_zrtp_confirmed = FALSE;
             calllist_add_call(current_calls_tab, call);
 
-            // add in treeview only if does not participate to a conference
+            // only add in treeview if this call is NOT in a conference
             gchar *confID = dbus_get_conference_id(call->_callID);
-            if(g_strcmp0(confID, "") == 0)
+            if (g_strcmp0(confID, "") == 0)
                 calltree_add_call(current_calls_tab, call, NULL);
+            g_free(confID);
         }
     }
 
@@ -950,14 +956,14 @@ sflphone_srtp_sdes_on(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_srtp_state = SRTP_STATE_SDES_SUCCESS;
 
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
 sflphone_srtp_sdes_off(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_srtp_state = SRTP_STATE_UNLOCKED;
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 
@@ -966,14 +972,14 @@ sflphone_srtp_zrtp_on(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_srtp_state = SRTP_STATE_ZRTP_SAS_UNCONFIRMED;
 
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
 sflphone_srtp_zrtp_off(callable_obj_t * c, SFLPhoneClient *client)
 {
     c->_srtp_state = SRTP_STATE_UNLOCKED;
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
@@ -982,7 +988,7 @@ sflphone_srtp_zrtp_show_sas(callable_obj_t * c, const gchar* sas, const gboolean
     c->_sas = g_strdup(sas);
     c->_srtp_state = verified ? SRTP_STATE_ZRTP_SAS_CONFIRMED : SRTP_STATE_ZRTP_SAS_UNCONFIRMED;
 
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
 void
@@ -1008,6 +1014,76 @@ sflphone_call_state_changed(callable_obj_t * c, const gchar * description, const
     c->_state_code_description = g_strdup(description);
     c->_state_code = code;
 
-    calltree_update_call(current_calls_tab, c, client, TRUE);
+    calltree_update_call(current_calls_tab, c, client);
 }
 
+#ifdef SFL_VIDEO
+gchar *
+sflphone_get_display(void)
+{
+    int width = gdk_screen_width();
+    int height = gdk_screen_height();
+    char *display = getenv("DISPLAY");
+
+    return g_strdup_printf("display://%s %dx%d", display, width, height);
+}
+
+gchar *
+sflphone_get_active_video(void)
+{
+    gchar *device = dbus_video_get_default_device();
+    gchar *resource;
+
+    if (strlen(device) > 0)
+        resource = g_strconcat("v4l2://", device, NULL);
+    else
+        resource = sflphone_get_video_none();
+
+    g_free(device);
+
+    return resource;
+}
+
+gchar *
+sflphone_get_video_none(void)
+{
+    static const gchar * const logo = "file://" ICONS_DIR "/sflphone.png";
+    const gchar *none = access(logo, R_OK) ? "" : logo;
+
+    return g_strdup(none);
+}
+
+/*
+ * URI of previous resource
+ * FIXME when we'll validate the toggle alternatives, we might get rid of this.
+ * FIXME this will be leaked at exit
+ */
+static gchar *last_uri;
+
+void
+sflphone_toggle_screenshare(void)
+{
+    gchar *resource = last_uri && g_str_has_prefix(last_uri, "display://") ?
+        sflphone_get_active_video() :
+        sflphone_get_display();
+
+    sflphone_switch_video_input(resource);
+    g_free(resource);
+}
+
+void
+sflphone_switch_video_input(const gchar *resource)
+{
+    gchar *decoded = g_uri_unescape_string(resource, NULL);
+    g_debug("MRL: '%s'", decoded);
+
+    if (dbus_switch_video_input(decoded)) {
+        g_free(last_uri);
+        last_uri = g_strdup(resource);
+    } else {
+        g_warning("Failed to switch to MRL '%s'\n", resource);
+    }
+
+    g_free(decoded);
+}
+#endif
