@@ -261,4 +261,14 @@ void sflphone_srtp_sdes_off(callable_obj_t * c, SFLPhoneClient *client);
 
 void sflphone_fill_conference_list(SFLPhoneClient *client);
 
+gchar * sflphone_get_display(void);
+
+gchar * sflphone_get_active_video(void);
+
+gchar * sflphone_get_video_none(void);
+
+void sflphone_toggle_screenshare(void);
+
+void sflphone_switch_video_input(const gchar *resource);
+
 #endif
