@@ -101,7 +101,6 @@ typedef struct  {
     time_t _time_stop;              // The timestamp the call was over
     gchar *_history_state;          // The history state if necessary
     srtp_state_t _srtp_state;       // The state of security on the call
-    gchar* _srtp_cipher;            // Cipher used for the srtp session
     gchar* _sas;                    // The Short Authentication String that should be displayed
     gboolean _zrtp_confirmed;       // Override real state. Used for hold/unhold
     // since rtp session is killed each time and
@@ -198,10 +197,8 @@ gchar* get_call_duration(callable_obj_t *obj);
 
 gchar* get_formatted_start_timestamp(time_t);
 
-gchar* call_get_audio_codec(callable_obj_t *obj);
+void format_duration(callable_obj_t *obj, time_t end, char *timestr, size_t timestr_sz);
 
-gchar* call_get_audio_codec(callable_obj_t *obj);
-gchar* call_get_video_codec(callable_obj_t *obj);
 gboolean call_was_outgoing(callable_obj_t * obj);
 
 void restore_call(const gchar *id);
