@@ -100,6 +100,7 @@ static GtkWidget *ringtone_seekslider;
 static GtkWidget *audio_port_min_spin_box;
 static GtkWidget *audio_port_max_spin_box;
 #ifdef SFL_VIDEO
+static GtkWidget *enable_video_button;
 static GtkWidget *video_port_min_spin_box;
 static GtkWidget *video_port_max_spin_box;
 #endif
@@ -145,6 +146,15 @@ auto_answer_cb(GtkToggleButton *widget, account_t *account)
                     gtk_toggle_button_get_active(widget) ? "true" : "false");
 }
 
+static void
+user_agent_checkbox_cb(GtkToggleButton *widget, account_t *account)
+{
+    const gboolean is_active = gtk_toggle_button_get_active(widget);
+    account_replace(account, CONFIG_ACCOUNT_HAS_CUSTOM_USERAGENT,
+                    is_active ? "true" : "false");
+    gtk_widget_set_sensitive(entry_user_agent, is_active);
+}
+
 /*
  * Display / Hide the password
  */
@@ -244,12 +254,21 @@ create_auto_answer_checkbox(const account_t *account)
     return auto_answer_checkbox;
 }
 
+static GtkWidget*
+create_user_agent_checkbox(const account_t *account)
+{
+    GtkWidget *user_agent_checkbox = gtk_check_button_new_with_mnemonic(_("_Use custom user-agent"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_agent_checkbox), account_has_custom_user_agent(account));
+    g_signal_connect(user_agent_checkbox, "toggled", G_CALLBACK(user_agent_checkbox_cb), (gpointer) account);
+    return user_agent_checkbox;
+}
+
 static void
 alias_changed_cb(GtkEditable *editable, gpointer data)
 {
     const gchar *alias = gtk_entry_get_text(GTK_ENTRY(editable));
     GtkDialog *dialog = GTK_DIALOG(data);
-    gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, strlen(alias));
+    gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, !!strlen(alias));
 }
 
 
@@ -358,9 +377,9 @@ create_account_parameters(account_t *account, gboolean is_new, GtkWidget *dialog
     gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     entry_password = gtk_entry_new();
-    gtk_entry_set_icon_from_stock(GTK_ENTRY(entry_password),
-                                  GTK_ENTRY_ICON_PRIMARY,
-                                  GTK_STOCK_DIALOG_AUTHENTICATION);
+    gtk_entry_set_icon_from_icon_name(GTK_ENTRY(entry_password),
+            GTK_ENTRY_ICON_PRIMARY, "dialog-password");
+
     gtk_entry_set_visibility(GTK_ENTRY(entry_password), FALSE);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry_password);
     password = password ? password : "";
@@ -408,6 +427,12 @@ create_account_parameters(account_t *account, gboolean is_new, GtkWidget *dialog
     gtk_entry_set_text(GTK_ENTRY(entry_user_agent), user_agent);
     gtk_grid_attach(GTK_GRID(grid), entry_user_agent, 1, row, 1, 1);
 
+    gtk_widget_set_sensitive(entry_user_agent, account_has_custom_user_agent(account));
+
+    row++;
+    GtkWidget *user_agent_checkbox = create_user_agent_checkbox(account);
+    gtk_grid_attach(GTK_GRID(grid), user_agent_checkbox, 0, row, 1, 1);
+
     row++;
     GtkWidget *auto_answer_checkbox = create_auto_answer_checkbox(account);
     gtk_grid_attach(GTK_GRID(grid), auto_answer_checkbox, 0, row, 1, 1);
@@ -704,11 +729,15 @@ static void set_published_addr_manually_cb(GtkWidget * widget, G_GNUC_UNUSED gpo
     }
 }
 
-static void use_stun_cb(GtkWidget *widget, G_GNUC_UNUSED gpointer data)
+static void
+use_stun_cb(GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
     /* Widgets have not been created yet */
-    if (!stun_server_label)
-        return;
+    g_return_if_fail(stun_server_label || stun_server_entry ||
+                     same_as_local_radio_button ||
+                     published_addr_radio_button || published_address_label ||
+                     published_port_label || published_address_entry ||
+                     published_port_spin_box);
 
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
         g_debug("Showing stun options, hiding Local/Published info");
@@ -814,11 +843,11 @@ static GtkWidget* create_credential_widget(const account_t *account)
     /* 2x1 */
     gtk_grid_attach(GTK_GRID(grid), hbox, 0, 1, 2, 1);
 
-    GtkWidget *addButton = gtk_button_new_from_stock(GTK_STOCK_ADD);
+    GtkWidget *addButton = gtk_button_new_with_label(_("Add"));
     g_signal_connect(addButton, "clicked", G_CALLBACK(add_credential_cb), credential_store);
     gtk_box_pack_start(GTK_BOX(hbox), addButton, FALSE, FALSE, 0);
 
-    delete_cred_button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
+    delete_cred_button = gtk_button_new_with_label(_("Remove"));
     g_signal_connect(delete_cred_button, "clicked", G_CALLBACK(delete_credential_cb), treeview_credential);
     gtk_box_pack_start(GTK_BOX(hbox), delete_cred_button, FALSE, FALSE, 0);
 
@@ -862,7 +891,7 @@ create_security_widget(account_t *account, SFLPhoneClient *client)
     OptionsData *options = g_new0(OptionsData, 1);
     options->account = account;
     options->client = client;
-    GtkWidget *sip_tls_advanced_button = gtk_button_new_from_stock(GTK_STOCK_EDIT);
+    GtkWidget *sip_tls_advanced_button = gtk_button_new_with_label(_("Edit"));
     gtk_grid_attach(GTK_GRID(grid), sip_tls_advanced_button, 2, 0, 1, 1);
     gtk_widget_set_sensitive(sip_tls_advanced_button, FALSE);
     g_signal_connect(G_OBJECT(sip_tls_advanced_button), "clicked",
@@ -885,7 +914,7 @@ create_security_widget(account_t *account, SFLPhoneClient *client)
     gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(key_exchange_combo), "SDES");
     gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(key_exchange_combo), _("Disabled"));
 
-    zrtp_button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
+    zrtp_button = gtk_button_new_with_label(_("Preferences"));
     g_signal_connect(G_OBJECT(zrtp_button), "clicked",
                      G_CALLBACK(show_advanced_zrtp_options_cb),
                      options);
@@ -923,9 +952,13 @@ create_security_tab(account_t *account, SFLPhoneClient *client)
     GtkWidget * ret = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
     gtk_container_set_border_width(GTK_CONTAINER(ret), 10);
 
-    // Credentials frame
-    GtkWidget * frame = create_credential_widget(account);
-    gtk_box_pack_start(GTK_BOX(ret), frame, FALSE, FALSE, 0);
+    GtkWidget *frame;
+
+    if (!account_is_IP2IP(account)) {
+        // Credentials frame
+        frame = create_credential_widget(account);
+        gtk_box_pack_start(GTK_BOX(ret), frame, FALSE, FALSE, 0);
+    }
 
     // Security frame
     frame = create_security_widget(account, client);
@@ -1065,9 +1098,6 @@ GtkWidget* create_published_address(const account_t *account)
     use_stun_check_box = gtk_check_button_new_with_mnemonic(_("Using STUN"));
     gtk_grid_attach(GTK_GRID(grid), use_stun_check_box, 0, 0, 1, 1);
     g_signal_connect(use_stun_check_box, "toggled", G_CALLBACK(use_stun_cb), NULL);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_stun_check_box),
-                                 utf8_case_equal(stun_enable, "true"));
-    gtk_widget_set_sensitive(use_stun_check_box, !utf8_case_equal(use_tls, "true"));
 
     stun_server_label = gtk_label_new_with_mnemonic(_("STUN server URL"));
     gtk_grid_attach(GTK_GRID(grid), stun_server_label, 0, 1, 1, 1);
@@ -1117,6 +1147,11 @@ GtkWidget* create_published_address(const account_t *account)
     g_signal_connect(same_as_local_radio_button, "toggled", G_CALLBACK(same_as_local_cb), same_as_local_radio_button);
     g_signal_connect(published_addr_radio_button, "toggled", G_CALLBACK(set_published_addr_manually_cb), published_addr_radio_button);
 
+    /* Now that widgets have been initialized it's safe to invoke this signal */
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_stun_check_box),
+                                 utf8_case_equal(stun_enable, "true"));
+    gtk_widget_set_sensitive(use_stun_check_box, !utf8_case_equal(use_tls, "true"));
+
     set_published_addr_manually_cb(published_addr_radio_button, NULL);
 
     return frame;
@@ -1165,8 +1200,12 @@ GtkWidget* create_advanced_tab(const account_t *account)
 
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
 
-    GtkWidget *frame = create_registration_expire(account);
-    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+    GtkWidget *frame;
+
+    if (!account_is_IP2IP(account)) {
+        frame = create_registration_expire(account);
+        gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+    }
 
     frame = create_network(account);
     gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
@@ -1314,6 +1353,13 @@ create_videocodecs_configuration(const account_t *a)
     gtk_widget_show(videocodecs);
     gtk_container_add(GTK_CONTAINER (videocodecs), box);
 
+    /* Check button to enable/disable video for an account */
+    gpointer ptr = account_lookup(a, CONFIG_VIDEO_ENABLED);
+    enable_video_button = gtk_check_button_new_with_mnemonic(_("_Enable video"));
+    const gboolean video_enabled = g_strcmp0(ptr, "true") == 0;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_video_button), video_enabled);
+    gtk_box_pack_start(GTK_BOX(vbox), enable_video_button, FALSE, FALSE, 0);
+
     gtk_widget_show_all(vbox);
 
     return vbox;
@@ -1338,9 +1384,6 @@ static GtkWidget* create_direct_ip_calls_tab(account_t *account)
     gtk_widget_set_size_request(label, 350, -1);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 
-    GtkWidget *frame = create_network(account);
-    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
-
     GtkWidget *auto_answer_checkbox = create_auto_answer_checkbox(account);
     gtk_box_pack_start(GTK_BOX(vbox), auto_answer_checkbox, FALSE, FALSE, 0);
 
@@ -1355,6 +1398,8 @@ static const gchar *bool_to_string(gboolean v)
 
 static void update_account_from_basic_tab(account_t *account)
 {
+    const gboolean IS_IP2IP = account_is_IP2IP(account);
+
     // Update protocol in case it changed
     gchar *proto;
     if (protocol_combo)
@@ -1363,11 +1408,14 @@ static void update_account_from_basic_tab(account_t *account)
         proto = g_strdup("SIP");
 
     if (g_strcmp0(proto, "SIP") == 0) {
-        account_replace(account, CONFIG_ACCOUNT_REGISTRATION_EXPIRE,
-                gtk_entry_get_text(GTK_ENTRY(expire_spin_box)));
 
-        account_replace(account, CONFIG_ACCOUNT_ROUTESET,
-                gtk_entry_get_text(GTK_ENTRY(entry_route_set)));
+        if (!IS_IP2IP) {
+            account_replace(account, CONFIG_ACCOUNT_REGISTRATION_EXPIRE,
+                    gtk_entry_get_text(GTK_ENTRY(expire_spin_box)));
+
+            account_replace(account, CONFIG_ACCOUNT_ROUTESET,
+                    gtk_entry_get_text(GTK_ENTRY(entry_route_set)));
+        }
 
         gboolean v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_stun_check_box));
         account_replace(account, CONFIG_STUN_ENABLE,
@@ -1391,9 +1439,11 @@ static void update_account_from_basic_tab(account_t *account)
 #endif
 
 #ifdef SFL_PRESENCE
-        v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(presence_check_box));
-        account_replace(account, CONFIG_PRESENCE_ENABLED, bool_to_string(v));
-        // TODO enable/disable the presence window view
+        if (!IS_IP2IP) {
+            v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(presence_check_box));
+            account_replace(account, CONFIG_PRESENCE_ENABLED, bool_to_string(v));
+            // TODO enable/disable the presence window view
+        }
 #endif
 
         if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(overrtp))) {
@@ -1430,13 +1480,17 @@ static void update_account_from_basic_tab(account_t *account)
     }
 
     account_replace(account, CONFIG_ACCOUNT_USERAGENT,
-                    gtk_entry_get_text(GTK_ENTRY(entry_user_agent)));
-    account_replace(account, CONFIG_ACCOUNT_ALIAS, gtk_entry_get_text(GTK_ENTRY(entry_alias)));
-    account_replace(account, CONFIG_ACCOUNT_TYPE, proto);
-    account_replace(account, CONFIG_ACCOUNT_HOSTNAME, gtk_entry_get_text(GTK_ENTRY(entry_hostname)));
-    account_replace(account, CONFIG_ACCOUNT_USERNAME, gtk_entry_get_text(GTK_ENTRY(entry_username)));
-    account_replace(account, CONFIG_ACCOUNT_PASSWORD, gtk_entry_get_text(GTK_ENTRY(entry_password)));
-    account_replace(account, CONFIG_ACCOUNT_MAILBOX, gtk_entry_get_text(GTK_ENTRY(entry_mailbox)));
+            gtk_entry_get_text(GTK_ENTRY(entry_user_agent)));
+
+    if (!IS_IP2IP) {
+        account_replace(account, CONFIG_ACCOUNT_ALIAS, gtk_entry_get_text(GTK_ENTRY(entry_alias)));
+        account_replace(account, CONFIG_ACCOUNT_TYPE, proto);
+        account_replace(account, CONFIG_ACCOUNT_HOSTNAME, gtk_entry_get_text(GTK_ENTRY(entry_hostname)));
+        account_replace(account, CONFIG_ACCOUNT_USERNAME, gtk_entry_get_text(GTK_ENTRY(entry_username)));
+        account_replace(account, CONFIG_ACCOUNT_PASSWORD, gtk_entry_get_text(GTK_ENTRY(entry_password)));
+        account_replace(account, CONFIG_ACCOUNT_MAILBOX, gtk_entry_get_text(GTK_ENTRY(entry_mailbox)));
+    }
+
     g_free(proto);
 }
 
@@ -1450,8 +1504,7 @@ void update_account_from_dialog(GtkWidget *dialog, const gchar *accountID)
         return;
 
     const gboolean IS_IP2IP = account_is_IP2IP(account);
-    if (!IS_IP2IP)
-        update_account_from_basic_tab(account);
+    update_account_from_basic_tab(account);
 
     if (account_is_SIP(account)) {
         if (IS_IP2IP ||
@@ -1478,6 +1531,12 @@ void update_account_from_dialog(GtkWidget *dialog, const gchar *accountID)
     const gboolean tone_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_tone));
     account_replace(account, CONFIG_RINGTONE_ENABLED, bool_to_string(tone_enabled));
 
+#ifdef SFL_VIDEO
+    const gboolean video_enabled =
+        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_video_button));
+    account_replace(account, CONFIG_VIDEO_ENABLED, bool_to_string(video_enabled));
+#endif
+
     gchar *ringtone_path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
     account_replace(account, CONFIG_RINGTONE_PATH, ringtone_path);
     g_free(ringtone_path);
@@ -1534,7 +1593,7 @@ show_account_window(const gchar *accountID, GtkDialog *parent, SFLPhoneClient *c
 
     account_t *account = account_list_get_by_id(accountID);
     if (!account) {
-        g_error("Invalid account %s", accountID);
+        g_warning("Invalid account %s", accountID);
         return NULL;
     }
 
@@ -1557,21 +1616,20 @@ show_account_window(const gchar *accountID, GtkDialog *parent, SFLPhoneClient *c
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), videocodecs_tab, gtk_label_new(_("Video")));
 #endif
 
-    // Do not need advanced or security one for the IP2IP account
-    if (!IS_IP2IP) {
-        /* Advanced */
-        advanced_tab = create_advanced_tab(account);
-        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced_tab, gtk_label_new(_("Advanced")));
-
-        /* Security */
-        security_tab = create_security_tab(account, client);
-        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), security_tab, gtk_label_new(_("Security")));
-    } else {
+    if (IS_IP2IP) {
         /* Custom tab for the IP to IP profile */
         GtkWidget *ip_tab = create_direct_ip_calls_tab(account);
-        gtk_notebook_prepend_page(GTK_NOTEBOOK(notebook), ip_tab, gtk_label_new(_("Network")));
+        gtk_notebook_prepend_page(GTK_NOTEBOOK(notebook), ip_tab, gtk_label_new(_("General")));
     }
 
+    /* Advanced */
+    advanced_tab = create_advanced_tab(account);
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced_tab, gtk_label_new(_("Advanced")));
+
+    /* Security */
+    security_tab = create_security_tab(account, client);
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), security_tab, gtk_label_new(_("Security")));
+
     // Emit signal to hide advanced and security tabs in case of IAX
     if (protocol_combo)
         g_signal_emit_by_name(protocol_combo, "changed", NULL);
