@@ -39,6 +39,7 @@
 #include "mainwindow.h"
 #include "utils.h"
 #include "presence.h"
+#include "presencewindow.h"
 #include <glib/gi18n.h>
 #include <string.h>
 
@@ -307,7 +308,11 @@ enable_account_cb(G_GNUC_UNUSED GtkCellRendererToggle *rend, gchar* path,
                        COLUMN_ACCOUNT_ID, &id, -1);
 
     account_t *account = account_list_get_by_id(id);
-    g_assert(account);
+    if (!account) {
+        g_warning("Invalid account %s", id);
+        return;
+    }
+
     enable = !enable;
 
     // Store value
@@ -349,7 +354,7 @@ account_move(gboolean move_up, gpointer data)
 
     GtkTreePath *tree_path = gtk_tree_path_new_from_string(path);
     gint *indices = gtk_tree_path_get_indices(tree_path);
-    const gint pos = indices[0];
+    const gint pos = indices[0] - 1; /* black magic : gtk tree order is not account queue order */
 
     // Depending on button direction get new path
     if (move_up)
@@ -558,24 +563,24 @@ create_account_list(SFLPhoneClient *client)
     gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), GTK_BUTTONBOX_START);
     gtk_grid_attach(GTK_GRID(grid), button_box, 1, 0, 1, 1);
 
-    move_up_button = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+    move_up_button = gtk_button_new_with_label(_("Up"));
     gtk_widget_set_sensitive(move_up_button, FALSE);
     gtk_box_pack_start(GTK_BOX(button_box), move_up_button, FALSE, FALSE, 0);
     g_signal_connect(G_OBJECT(move_up_button), "clicked",
                      G_CALLBACK(move_up_cb), tree_view);
 
-    move_down_button = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
+    move_down_button = gtk_button_new_with_label(_("Down"));
     gtk_widget_set_sensitive(move_down_button, FALSE);
     gtk_box_pack_start(GTK_BOX(button_box), move_down_button, FALSE, FALSE, 0);
     g_signal_connect(G_OBJECT(move_down_button), "clicked",
                      G_CALLBACK(move_down_cb), tree_view);
 
-    GtkWidget *add_button = gtk_button_new_from_stock(GTK_STOCK_ADD);
+    GtkWidget *add_button = gtk_button_new_with_label(_("Add"));
     g_signal_connect_swapped(G_OBJECT(add_button), "clicked",
                              G_CALLBACK(add_account_cb), client);
     gtk_box_pack_start(GTK_BOX(button_box), add_button, FALSE, FALSE, 0);
 
-    edit_button = gtk_button_new_from_stock(GTK_STOCK_EDIT);
+    edit_button = gtk_button_new_with_label(_("Edit"));
     gtk_widget_set_sensitive(edit_button, FALSE);
     EditData *edit_data = g_new0(EditData, 1);
     edit_data->view = GTK_TREE_VIEW(tree_view);
@@ -583,7 +588,7 @@ create_account_list(SFLPhoneClient *client)
     g_signal_connect(G_OBJECT(edit_button), "clicked", G_CALLBACK(edit_account_cb), edit_data);
     gtk_box_pack_start(GTK_BOX(button_box), edit_button, FALSE, FALSE, 0);
 
-    delete_button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
+    delete_button = gtk_button_new_with_label(_("Remove"));
     gtk_widget_set_sensitive(delete_button, FALSE);
     g_object_set_data(G_OBJECT(delete_button), "window", client->win);
     g_signal_connect(G_OBJECT(delete_button), "clicked",
@@ -595,12 +600,12 @@ create_account_list(SFLPhoneClient *client)
     /* this element will be 2x1 cells */
     gtk_grid_attach(GTK_GRID(grid), buttonHbox, 0, 1, 2, 1);
 
-    GtkWidget * helpButton = gtk_button_new_from_stock(GTK_STOCK_HELP);
+    GtkWidget * helpButton = gtk_button_new_with_label(_("Help"));
     g_signal_connect_swapped(G_OBJECT(helpButton), "clicked",
                              G_CALLBACK(help_contents_cb), NULL);
     gtk_box_pack_start(GTK_BOX(buttonHbox), helpButton, FALSE, FALSE, 0);
 
-    GtkWidget * closeButton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+    GtkWidget * closeButton = gtk_button_new_with_label(_("Close"));
     g_signal_connect_swapped(G_OBJECT(closeButton), "clicked",
                              G_CALLBACK(close_dialog_cb), NULL);
     gtk_box_pack_start(GTK_BOX(buttonHbox), closeButton, FALSE, FALSE, 0);
@@ -651,11 +656,8 @@ void update_account_list_status_bar(account_t *account)
 
     GtkTreeModel *model = GTK_TREE_MODEL(account_store);
     GtkTreeIter iter;
-    if (find_account_in_account_store(account->accountID, model, &iter)) {
+    if (find_account_in_account_store(account->accountID, model, &iter))
         gtk_list_store_set(account_store, &iter, COLUMN_ACCOUNT_STATUS, state_name, -1);
-        gtk_list_store_set(account_store, &iter, COLUMN_ACCOUNT_ACTIVE,
-                           account->state == ACCOUNT_STATE_REGISTERED, -1);
-    }
 }
 
 void dialog_destroy_cb()
@@ -705,7 +707,7 @@ void show_account_list_config_dialog(SFLPhoneClient *client)
         g_free(message);
     } else {
         gtk_statusbar_push(GTK_STATUSBAR(account_list_status_bar), CONTEXT_ID_REGISTRATION,
-                           _("You have no active account"));
+                           _("You have no active accounts"));
     }
 
     gtk_dialog_run(account_list_dialog);
