@@ -108,12 +108,14 @@ fill_codec_list(const account_t *account)
 }
 
 static GtkListStore *
-create_device_list_store(gchar **list)
+create_device_list_store(gchar **list, gboolean output)
 {
     GtkListStore *list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
 
     for (gchar **tmp = list; tmp && *tmp; ++tmp) {
-        gint device_index = dbus_get_audio_device_index(*tmp);
+        gint device_index = output
+            ? dbus_get_audio_output_device_index(*tmp)
+            : dbus_get_audio_input_device_index(*tmp);
         GtkTreeIter iter;
         gtk_list_store_append(list_store, &iter);
         gtk_list_store_set(list_store, &iter, 0, *tmp, 1, device_index, -1);
@@ -143,118 +145,58 @@ create_alsa_plugin_list_store()
     return list_store;
 }
 
+static const guint OUTPUT_INDEX = 0;
+static const guint INPUT_INDEX = 1;
+static const guint RINGTONE_INDEX = 2;
+
 static void
-select_active_output_audio_device(GtkWidget *output)
+select_active_audio_device(GtkWidget *widget, guint index)
 {
-    // Select active output device on server
+    // Select active device
     gchar **devices = dbus_get_current_audio_devices_index();
-    if (devices && devices[0]) {
+    if (!devices || !devices[index])
+        goto error;
 
-        int currentDeviceIndex = atoi(devices[0]);
-        g_strfreev(devices);
-        g_debug("audio device index for output = %d", currentDeviceIndex);
-        GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(output));
+    const gint currentDeviceIndex = atoi(devices[index]);
+    g_strfreev(devices);
+    GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
 
-        // Find the currently set output device
-        GtkTreeIter iter;
-        gtk_tree_model_get_iter_first(model, &iter);
-
-        do {
-            int deviceIndex;
-            gtk_tree_model_get(model, &iter, 1, &deviceIndex, -1);
-
-            if (deviceIndex == currentDeviceIndex) {
-                // Set current iteration the active one
-                gtk_combo_box_set_active_iter(GTK_COMBO_BOX(output), &iter);
-                return;
-            }
-        } while (gtk_tree_model_iter_next(model, &iter));
+    // Find the currently set device
+    GtkTreeIter iter;
+    if (!gtk_tree_model_get_iter_first(model, &iter)) {
+        g_warning("Tree is empty");
+        goto error;
     }
 
+    do {
+        int deviceIndex;
+        gtk_tree_model_get(model, &iter, 1, &deviceIndex, -1);
+
+        if (deviceIndex == currentDeviceIndex) {
+            // Set current iteration the active one
+            gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget), &iter);
+            return;
+        }
+    } while (gtk_tree_model_iter_next(model, &iter));
+
+error:
     // No index was found, select first one
-    g_warning("No active output device found");
-    gtk_combo_box_set_active(GTK_COMBO_BOX(output), 0);
-}
-
-
-/**
- * Select active output audio device
- */
-static void
-select_active_ringtone_audio_device(GtkWidget *ringtone)
-{
-    // Select active ringtone device on server
-    gchar **devices = dbus_get_current_audio_devices_index();
-    if (devices && devices[2]) {
-        const gint currentDeviceIndex = atoi(devices[2]);
-        g_strfreev(devices);
-        g_debug("audio device index for ringtone = %d", currentDeviceIndex);
-        GtkTreeModel* model = gtk_combo_box_get_model(GTK_COMBO_BOX(ringtone));
-
-        // Find the currently set ringtone device
-        GtkTreeIter iter;
-        gtk_tree_model_get_iter_first(model, &iter);
-
-        do {
-            int deviceIndex;
-            gtk_tree_model_get(model, &iter, 1, &deviceIndex, -1);
-
-            if (deviceIndex == currentDeviceIndex) {
-                // Set current iteration the active one
-                gtk_combo_box_set_active_iter(GTK_COMBO_BOX(ringtone), &iter);
-                return;
-            }
-        } while (gtk_tree_model_iter_next(model, &iter));
-    }
-
-    // No index was found, select first one
-    g_warning("No active ringtone device found");
-    gtk_combo_box_set_active(GTK_COMBO_BOX(ringtone), 0);
+    g_warning("No active device found");
+    gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);
 }
 
 static GtkListStore*
 create_output_list_store()
 {
     gchar **list = dbus_get_audio_output_device_list();
-    return create_device_list_store(list);
+    return create_device_list_store(list, TRUE);
 }
 
 static GtkListStore*
 create_input_list_store()
 {
     gchar **list = dbus_get_audio_input_device_list();
-    return create_device_list_store(list);
-}
-
-static void
-select_active_input_audio_device(GtkWidget *input)
-{
-    // Select active input device on server
-    gchar **devices = dbus_get_current_audio_devices_index();
-    if (devices && devices[1]) {
-        int currentDeviceIndex = atoi(devices[1]);
-        g_strfreev(devices);
-        GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(input));
-
-        // Find the currently set input device
-        GtkTreeIter iter;
-        gtk_tree_model_get_iter_first(model, &iter);
-
-        do {
-            int deviceIndex;
-            gtk_tree_model_get(model, &iter, 1, &deviceIndex, -1);
-
-            if (deviceIndex == currentDeviceIndex) {
-                // Set current iteration the active one
-                gtk_combo_box_set_active_iter(GTK_COMBO_BOX(input), &iter);
-                return;
-            }
-        } while (gtk_tree_model_iter_next(model, &iter));
-    }
-
-    // No index was found, select first one
-    g_warning("No active input device found");
-    gtk_combo_box_set_active(GTK_COMBO_BOX(input), 0);
+    return create_device_list_store(list, FALSE);
 }
 
 static void
@@ -266,10 +208,21 @@ update_device_widget(const gchar *pluginName, GtkWidget *output, GtkWidget *inpu
     gtk_widget_set_sensitive(ringtone, !is_default);
 }
 
+
+static gboolean
+audio_api_in_use(const gchar *api)
+{
+    gchar *current_api = dbus_get_audio_manager();
+    int ret = g_strcmp0(current_api, api);
+    g_free(current_api);
+    return ret == 0;
+}
+
+
 static void
 select_output_alsa_plugin(GtkComboBox* widget, G_GNUC_UNUSED gpointer data)
 {
-    if (!must_show_alsa_conf())
+    if (!audio_api_in_use(ALSA_API_STR))
         return;
     const gint comboBoxIndex = gtk_combo_box_get_active(widget);
 
@@ -287,14 +240,17 @@ select_output_alsa_plugin(GtkComboBox* widget, G_GNUC_UNUSED gpointer data)
 static void
 select_active_output_alsa_plugin(GtkWidget *alsa_plugin)
 {
-    if (!must_show_alsa_conf())
+    if (!audio_api_in_use(ALSA_API_STR))
         return;
     // Select active output device on server
     GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(alsa_plugin));
 
     // Find the current alsa plugin
     GtkTreeIter iter;
-    gtk_tree_model_get_iter_first(model, &iter);
+    if (!gtk_tree_model_get_iter_first(model, &iter)) {
+        g_warning("Tree is empty");
+        goto error;
+    }
 
     gchar *alsa_pluginname = dbus_get_current_audio_output_plugin();
     gchar *tmp = alsa_pluginname;
@@ -311,6 +267,7 @@ select_active_output_alsa_plugin(GtkWidget *alsa_plugin)
     } while (gtk_tree_model_iter_next(model, &iter));
     g_free(alsa_pluginname);
 
+error:
     // No index was found, select first one
     g_warning("No active output device found");
     gtk_combo_box_set_active(GTK_COMBO_BOX(alsa_plugin), 0);
@@ -564,13 +521,13 @@ audiocodecs_box(const account_t *account)
     gtk_container_set_border_width(GTK_CONTAINER(buttonBox), 10);
     gtk_box_pack_start(GTK_BOX(audiocodecs_hbox), buttonBox, FALSE, FALSE, 0);
 
-    codecMoveUpButton = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+    codecMoveUpButton = gtk_button_new_with_label(_("Up"));
     gtk_widget_set_sensitive(GTK_WIDGET(codecMoveUpButton), FALSE);
     gtk_box_pack_start(GTK_BOX(buttonBox), codecMoveUpButton, FALSE, FALSE, 0);
     g_signal_connect(G_OBJECT(codecMoveUpButton), "clicked",
                      G_CALLBACK(codec_move_up), (gpointer) account);
 
-    codecMoveDownButton = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
+    codecMoveDownButton = gtk_button_new_with_label(_("Down"));
     gtk_widget_set_sensitive(GTK_WIDGET(codecMoveDownButton), FALSE);
     gtk_box_pack_start(GTK_BOX(buttonBox), codecMoveDownButton, FALSE, FALSE, 0);
     g_signal_connect(G_OBJECT(codecMoveDownButton), "clicked",
@@ -625,7 +582,7 @@ static GtkWidget* alsa_box()
     // Set choices of output devices
     GtkListStore *outputlist = create_output_list_store();
     alsa_output = gtk_combo_box_new_with_model(GTK_TREE_MODEL(outputlist));
-    select_active_output_audio_device(alsa_output);
+    select_active_audio_device(alsa_output, OUTPUT_INDEX);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), alsa_output);
     g_signal_connect(G_OBJECT(alsa_output), "changed", G_CALLBACK(select_audio_output_device), NULL);
 
@@ -647,7 +604,7 @@ static GtkWidget* alsa_box()
     // Set choices of input devices
     GtkListStore *inputlist = create_input_list_store();
     alsa_input = gtk_combo_box_new_with_model(GTK_TREE_MODEL(inputlist));
-    select_active_input_audio_device(alsa_input);
+    select_active_audio_device(alsa_input, INPUT_INDEX);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), alsa_input);
     g_signal_connect(G_OBJECT(alsa_input), "changed", G_CALLBACK(select_audio_input_device), NULL);
 
@@ -666,7 +623,7 @@ static GtkWidget* alsa_box()
     // set choices of ringtone devices
     GtkListStore *ringtonelist = create_output_list_store();
     alsa_ringtone = gtk_combo_box_new_with_model(GTK_TREE_MODEL(ringtonelist));
-    select_active_ringtone_audio_device(alsa_ringtone);
+    select_active_audio_device(alsa_ringtone, RINGTONE_INDEX);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), alsa_output);
     g_signal_connect(G_OBJECT(alsa_ringtone), "changed", G_CALLBACK(select_audio_ringtone_device), NULL);
 
@@ -707,7 +664,7 @@ static GtkWidget* pulse_box()
     // Set choices of output devices
     GtkListStore *outputlist = create_output_list_store();
     GtkWidget *pulse_output = gtk_combo_box_new_with_model(GTK_TREE_MODEL(outputlist));
-    select_active_output_audio_device(pulse_output);
+    select_active_audio_device(pulse_output, OUTPUT_INDEX);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), pulse_output);
     g_signal_connect(G_OBJECT(pulse_output), "changed", G_CALLBACK(select_audio_output_device), NULL);
 
@@ -729,7 +686,7 @@ static GtkWidget* pulse_box()
     // Set choices of output devices
     GtkListStore *inputlist = create_input_list_store();
     GtkWidget *pulse_input = gtk_combo_box_new_with_model(GTK_TREE_MODEL(inputlist));
-    select_active_input_audio_device(pulse_input);
+    select_active_audio_device(pulse_input, INPUT_INDEX);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), pulse_input);
     g_signal_connect(G_OBJECT(pulse_input), "changed", G_CALLBACK(select_audio_input_device), NULL);
 
@@ -748,7 +705,7 @@ static GtkWidget* pulse_box()
     // set choices of ringtone devices
     GtkListStore *ringtonelist = create_output_list_store();
     GtkWidget *ringtone = gtk_combo_box_new_with_model(GTK_TREE_MODEL(ringtonelist));
-    select_active_ringtone_audio_device(ringtone);
+    select_active_audio_device(ringtone, RINGTONE_INDEX);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), pulse_output);
     g_signal_connect(G_OBJECT(ringtone), "changed", G_CALLBACK(select_audio_ringtone_device), NULL);
 
@@ -764,44 +721,82 @@ static GtkWidget* pulse_box()
     return pulse_hbox;
 }
 
+static void
+show_audio_conf(GtkWidget *conf, GtkWidget *box)
+{
+    if (!GTK_IS_WIDGET(conf))
+        return;
+
+    if (!GTK_IS_WIDGET(box))
+        return;
+
+    gtk_container_add(GTK_CONTAINER(conf), box);
+    gtk_widget_show(conf);
+    gtk_widget_set_sensitive(conf, TRUE);
+
+}
 
 static void
-select_audio_manager(GtkWidget *alsa_button, SFLPhoneClient *client)
+hide_audio_conf(GtkWidget *conf, GtkWidget *box)
 {
-    if (!must_show_alsa_conf() && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(alsa_button))) {
-        dbus_set_audio_manager(ALSA_API_STR);
-        gtk_container_remove(GTK_CONTAINER(pulse_conf), pulsebox);
-        gtk_widget_hide(pulse_conf);
+    if (!GTK_IS_WIDGET(conf))
+        return;
 
-        alsabox = alsa_box();
-        gtk_container_add(GTK_CONTAINER(alsa_conf), alsabox);
-        gtk_widget_show(alsa_conf);
-        gtk_widget_set_sensitive(alsa_conf, TRUE);
+    if (GTK_IS_WIDGET(box))
+        gtk_container_remove(GTK_CONTAINER(conf), box);
+    gtk_widget_hide(conf);
+}
 
-        if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(volumeToggle_))) {
-            main_window_volume_controls(FALSE);
-            g_settings_set_boolean(client->settings, "show-volume-controls", FALSE);
-            gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), FALSE);
-        }
+static void
+alsa_toggled(GtkToggleButton *alsa_button, SFLPhoneClient *client)
+{
+    if (!gtk_toggle_button_get_active(alsa_button))
+        return;
 
-    } else if (must_show_alsa_conf() && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(alsa_button))) {
-        dbus_set_audio_manager(PULSEAUDIO_API_STR);
-        gtk_container_remove(GTK_CONTAINER(alsa_conf), alsabox);
-        gtk_widget_hide(alsa_conf);
+    dbus_set_audio_manager(ALSA_API_STR);
 
-        pulsebox = pulse_box();
-        gtk_container_add(GTK_CONTAINER(pulse_conf), pulsebox);
-        gtk_widget_show(pulse_conf);
-        gtk_widget_set_sensitive(pulse_conf, TRUE);
+    hide_audio_conf(pulse_conf, pulsebox);
 
-        if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(volumeToggle_))) {
-            main_window_volume_controls(FALSE);
-            g_settings_set_boolean(client->settings, "show-volume-controls", FALSE);
-            gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), FALSE);
-        }
+    alsabox = alsa_box();
+    show_audio_conf(alsa_conf, alsabox);
 
-    } else
-        g_warning("Unexpected audio API state");
+    if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(volumeToggle_))) {
+        main_window_volume_controls(FALSE);
+        g_settings_set_boolean(client->settings, "show-volume-controls", FALSE);
+        gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), FALSE);
+    }
+}
+
+static void
+pulse_toggled(GtkToggleButton *pulse_button, SFLPhoneClient *client)
+{
+    if (!gtk_toggle_button_get_active(pulse_button))
+        return;
+
+    dbus_set_audio_manager(PULSEAUDIO_API_STR);
+
+    hide_audio_conf(alsa_conf, alsabox);
+
+    pulsebox = pulse_box();
+    show_audio_conf(pulse_conf, pulsebox);
+
+    if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(volumeToggle_))) {
+        main_window_volume_controls(FALSE);
+        g_settings_set_boolean(client->settings, "show-volume-controls", FALSE);
+        gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), FALSE);
+    }
+}
+
+static void
+jack_toggled(GtkToggleButton *jack_button)
+{
+    if (!gtk_toggle_button_get_active(jack_button))
+        return;
+
+    dbus_set_audio_manager(JACK_API_STR);
+
+    hide_audio_conf(pulse_conf, pulsebox);
+    hide_audio_conf(alsa_conf, alsabox);
 }
 
 static void
@@ -843,15 +838,29 @@ static void record_path_changed(GtkFileChooser *chooser, G_GNUC_UNUSED gpointer
 }
 
 static
-void
+gboolean
 mute_dtmf_toggled(GtkToggleButton *tb, G_GNUC_UNUSED gpointer data)
 {
     if (gtk_toggle_button_get_active(tb))
         dbus_mute_dtmf(TRUE);
     else
         dbus_mute_dtmf(FALSE);
+    return TRUE;
 }
 
+static gboolean
+is_jack_supported()
+{
+    gchar **backends = dbus_get_supported_audio_managers();
+    for (gchar **tmp = backends; tmp && *tmp; ++tmp) {
+        if (!g_strcmp0(*tmp, JACK_API_STR)) {
+            g_strfreev(backends);
+            return TRUE;
+        }
+    }
+    g_strfreev(backends);
+    return FALSE;
+}
 
 GtkWidget* create_audio_configuration(SFLPhoneClient *client)
 {
@@ -864,12 +873,13 @@ GtkWidget* create_audio_configuration(SFLPhoneClient *client)
     gnome_main_section_new_with_grid(_("Sound Manager"), &frame, &grid);
     gtk_box_pack_start(GTK_BOX(audio_vbox), frame, FALSE, FALSE, 0);
 
-    gchar *audio_manager = dbus_get_audio_manager();
-    const gboolean using_pulse = g_strcmp0(audio_manager, PULSEAUDIO_API_STR) == 0;
-    g_free(audio_manager);
+    const gboolean using_pulse = audio_api_in_use(PULSEAUDIO_API_STR);
+    const gboolean using_alsa = !using_pulse && audio_api_in_use(ALSA_API_STR);
+    const gboolean using_jack = !using_pulse && !using_alsa && audio_api_in_use(JACK_API_STR);
 
     GtkWidget *pulse_button = gtk_radio_button_new_with_mnemonic(NULL , _("_Pulseaudio"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pulse_button), using_pulse);
+    g_signal_connect(G_OBJECT(pulse_button), "toggled", G_CALLBACK(pulse_toggled), client);
     gtk_grid_attach(GTK_GRID(grid), pulse_button, 0, 0, 1, 1);
 
     // Box for the Pulse configuration
@@ -878,21 +888,28 @@ GtkWidget* create_audio_configuration(SFLPhoneClient *client)
     gtk_widget_show(pulse_conf);
 
     GtkWidget *alsa_button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(pulse_button), _("_ALSA"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alsa_button), !using_pulse);
-    g_signal_connect(G_OBJECT(alsa_button), "clicked", G_CALLBACK(select_audio_manager), client);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alsa_button), using_alsa);
+    g_signal_connect(G_OBJECT(alsa_button), "toggled", G_CALLBACK(alsa_toggled), client);
     gtk_grid_attach(GTK_GRID(grid), alsa_button, 1, 0, 1, 1);
 
+    if (is_jack_supported()) {
+        GtkWidget *jack_button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(pulse_button), _("_JACK"));
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(jack_button), using_jack);
+        g_signal_connect(G_OBJECT(jack_button), "toggled", G_CALLBACK(jack_toggled), NULL);
+        gtk_grid_attach(GTK_GRID(grid), jack_button, 2, 0, 1, 1);
+    }
+
     // Box for the ALSA configuration
     alsa_conf = gnome_main_section_new(_("ALSA settings"));
     gtk_box_pack_start(GTK_BOX(audio_vbox), alsa_conf, FALSE, FALSE, 0);
     gtk_widget_show(alsa_conf);
 
-    if (must_show_alsa_conf()) {
+    if (using_alsa) {
         // Box for the ALSA configuration
         alsabox = alsa_box();
         gtk_container_add(GTK_CONTAINER(alsa_conf), alsabox);
         gtk_widget_hide(alsa_conf);
-    } else {
+    } else if (using_pulse) {
         pulsebox = pulse_box();
         gtk_container_add(GTK_CONTAINER(pulse_conf), pulsebox);
         gtk_widget_hide(pulse_conf);
@@ -947,28 +964,21 @@ GtkWidget* create_audio_configuration(SFLPhoneClient *client)
     GtkWidget *muteDtmf = gtk_check_button_new_with_mnemonic(_("_Mute DTMF"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(muteDtmf), dbus_is_dtmf_muted());
 
-    g_signal_connect(G_OBJECT(muteDtmf), "toggled", mute_dtmf_toggled, NULL);
+    g_signal_connect(G_OBJECT(muteDtmf), "toggled", G_CALLBACK(mute_dtmf_toggled), NULL);
     gtk_grid_attach(GTK_GRID(grid), muteDtmf, 0, 1, 1, 1);
 
     gtk_widget_show_all(audio_vbox);
 
-    if (!using_pulse)
+    if (using_alsa) {
         gtk_widget_show(alsa_conf);
-    else
+        gtk_widget_hide(pulse_conf);
+    } else {
         gtk_widget_hide(alsa_conf);
+    }
 
     return audio_vbox;
 }
 
-/** Show/Hide the alsa configuration panel */
-gboolean must_show_alsa_conf()
-{
-    gchar *api = dbus_get_audio_manager();
-    int ret = g_strcmp0(api, ALSA_API_STR);
-    g_free(api);
-    return ret == 0;
-}
-
 gboolean
 must_show_volume(SFLPhoneClient *client)
 {
