@@ -43,7 +43,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
 
 #include "dbus.h"
 #include "statusicon.h"
@@ -326,17 +325,17 @@ static GtkTreeModel* create_model(GtkWidget *widget)
         gchar* icon_name;
         gint page_number;
     } browser_entries_full[] = {
-        {"General", GTK_STOCK_PREFERENCES, 0},
-        {"Audio", GTK_STOCK_AUDIO_CARD, 1},
+        {"General", GTK_STOCK_PREFS, 0},
+        {"Audio", GTK_STOCK_PREFS_AUDIO, 1},
 #ifdef SFL_VIDEO
-        {"Video", "camera-web", 2},
-        {"Hooks", "applications-development", 3},
-        {"Shortcuts", "preferences-desktop-keyboard", 4},
-        {"Address Book", GTK_STOCK_ADDRESSBOOK, 5},
+        {"Video", GTK_STOCK_PREFS_VIDEO, 2},
+        {"Hooks", GTK_STOCK_PREFS_HOOK, 3},
+        {"Shortcuts", GTK_STOCK_PREFS_SHORTCUT, 4},
+        {"Address Book", GTK_STOCK_PREFS_ADDRESSBOOK, 5},
 #else
-        {"Hooks", "applications-development", 2},
-        {"Shortcuts", "preferences-desktop-keyboard", 3},
-        {"Address Book", GTK_STOCK_ADDRESSBOOK, 4},
+        {"Hooks", GTK_STOCK_PREFS_HOOK, 2},
+        {"Shortcuts", GTK_STOCK_PREFS_SHORTCUT, 3},
+        {"Address Book", GTK_STOCK_PREFS_ADDRESSBOOK, 4},
 #endif
     };
     GdkPixbuf *pixbuf;
@@ -363,6 +362,23 @@ static GtkTreeModel* create_model(GtkWidget *widget)
     return GTK_TREE_MODEL(store);
 }
 
+/* Callback used to catch the destroy event send by pressing escape key or the
+ * close button in the preference dialog */
+static void
+dialog_destroy_cb(G_GNUC_UNUSED GtkWidget *widget,
+        G_GNUC_UNUSED gpointer user_data)
+{
+#ifdef SFL_VIDEO
+    gchar ** str = dbus_get_call_list();
+
+    /* we stop the video only if we are not currently in a call */
+    if (str == NULL || *str == NULL) {
+        dbus_stop_video_camera();
+    }
+
+    g_strfreev(str);
+#endif
+}
 
 /**
  * Show configuration window with tabs
@@ -393,6 +409,11 @@ show_preferences_dialog(SFLPhoneClient *client)
                   "margin", 10,
                   NULL);
 
+
+    /* Connect the callback when the dialog is destroy, ie: pressing escape key or the
+     * close button */
+    g_signal_connect(GTK_WIDGET(dialog), "destroy", G_CALLBACK(dialog_destroy_cb), NULL);
+
     // Connect the callback when clicking on an item
     g_signal_connect(G_OBJECT(iconview), "selection-changed", G_CALLBACK(selection_changed_cb), NULL);
     gtk_box_pack_start(GTK_BOX(hbox), iconview, TRUE, TRUE, 0);
