@@ -44,12 +44,65 @@ const gchar *toggle_to_string(GtkWidget *toggle)
     return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle)) ? "true" : "false";
 }
 
+/* Caller must free returned string */
 static
-const gchar *get_filename(GtkWidget *chooser)
+gchar *
+get_filename(GtkWidget *chooser)
 {
     return gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 }
 
+static gboolean
+confirm_certificate_use(GtkWidget *window)
+{
+    gchar *msg = g_markup_printf_escaped(_("Are you sure want to use this certificate?"));
+
+    /* Create the widgets */
+    GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(window),
+            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+            GTK_MESSAGE_WARNING,
+            GTK_BUTTONS_CANCEL,
+            "%s", msg);
+
+    gtk_dialog_add_buttons(GTK_DIALOG(dialog), _("Use anyway"), GTK_RESPONSE_OK, NULL);
+
+    gtk_window_set_title(GTK_WINDOW(dialog), _("Invalid certificate detected"));
+    const gint response = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+
+    g_free(msg);
+
+    return response == GTK_RESPONSE_OK;
+}
+
+static void
+certificate_set_cb(GtkFileChooserButton *widget, gpointer user_data)
+{
+    gchar *filename = get_filename(GTK_WIDGET(widget));
+    gchar *caname = get_filename(GTK_WIDGET(g_object_get_data((GObject *)GTK_WIDGET(widget), "ca")));
+
+    const gboolean is_valid = dbus_check_certificate(caname, filename);
+
+    gboolean contains_key = FALSE;
+    GtkWidget *private_key_chooser = user_data;
+
+    /* If certificate is invalid, check if user really wants to use it */
+    if (!is_valid && !confirm_certificate_use(gtk_widget_get_toplevel(GTK_WIDGET(widget)))) {
+        gtk_file_chooser_unselect_filename(GTK_FILE_CHOOSER(widget), filename);
+    } else {
+
+        /* disable private key file chooser if certificate contains key */
+        contains_key = dbus_certificate_contains_private_key(filename);
+        if (contains_key)
+            gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(private_key_chooser));
+    }
+
+    /* Defaults to sensitive if no key was found */
+    gtk_widget_set_sensitive(private_key_chooser, !contains_key);
+
+    g_free(filename);
+}
+
 void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
 {
     GtkDialog *tlsDialog = GTK_DIALOG(gtk_dialog_new_with_buttons(_("Advanced options for TLS"),
@@ -98,7 +151,6 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
     gchar * verify_client = NULL;
     gchar * require_client_certificate = NULL;
     gchar * negotiation_timeout_sec = NULL;
-    gchar * negotiation_timeout_msec = NULL;
 
     if (account->properties != NULL) {
         tls_listener_port = account_lookup(account, CONFIG_TLS_LISTENER_PORT);
@@ -113,11 +165,10 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
         verify_client = account_lookup(account, CONFIG_TLS_VERIFY_CLIENT);
         require_client_certificate = account_lookup(account, CONFIG_TLS_REQUIRE_CLIENT_CERTIFICATE);
         negotiation_timeout_sec = account_lookup(account, CONFIG_TLS_NEGOTIATION_TIMEOUT_SEC);
-        negotiation_timeout_msec = account_lookup(account, CONFIG_TLS_NEGOTIATION_TIMEOUT_MSEC);
     }
 
 
-    label = gtk_label_new(_("Global TLS listener (all accounts)"));
+    label = gtk_label_new(_("TLS listener port"));
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);
     GtkWidget * hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
@@ -147,6 +198,11 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
     GtkWidget * certificateFileChooser = gtk_file_chooser_button_new(_("Choose a public endpoint certificate (optional)"), GTK_FILE_CHOOSER_ACTION_OPEN);
     gtk_grid_attach(GTK_GRID(grid), certificateFileChooser, 1, 4, 1, 1);
 
+    gboolean contains_key = FALSE;
+
+    /* save the ca path in order to use it when validating the certificate */
+    g_object_set_data((GObject *)certificateFileChooser, "ca", GTK_WIDGET(caListFileChooser));
+
     if (!tls_certificate_file) {
         gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(caListFileChooser));
     } else {
@@ -156,6 +212,7 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
             GFile * file = g_file_new_for_path(tls_certificate_file);
             gtk_file_chooser_set_file(GTK_FILE_CHOOSER(certificateFileChooser), file, NULL);
             g_object_unref(file);
+            contains_key = dbus_certificate_contains_private_key(tls_certificate_file);
         }
     }
 
@@ -165,10 +222,14 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
     GtkWidget *privateKeyFileChooser = gtk_file_chooser_button_new(_("Choose a private key file (optional)"), GTK_FILE_CHOOSER_ACTION_OPEN);
     gtk_grid_attach(GTK_GRID(grid), privateKeyFileChooser, 1, 5, 1, 1);
 
-    if (!tls_private_key_file) {
+    /* if certificate contains private key file, disallow private
+     * key file selection */
+    g_signal_connect(GTK_FILE_CHOOSER(certificateFileChooser), "file-set", G_CALLBACK(certificate_set_cb), privateKeyFileChooser);
+
+    if (!tls_private_key_file || contains_key) {
         gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(privateKeyFileChooser));
     } else {
-        if (!*tls_private_key_file) {
+        if (!*tls_private_key_file || contains_key) {
             gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(privateKeyFileChooser));
         } else {
             GFile * file = g_file_new_for_path(tls_private_key_file);
@@ -176,13 +237,16 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
             g_object_unref(file);
         }
     }
+    gtk_widget_set_sensitive(privateKeyFileChooser, !contains_key);
 
     label = gtk_label_new_with_mnemonic(_("Password for the private key"));
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     gtk_grid_attach(GTK_GRID(grid), label, 0, 6, 1, 1);
     GtkWidget * privateKeyPasswordEntry;
     privateKeyPasswordEntry = gtk_entry_new();
-    gtk_entry_set_icon_from_stock(GTK_ENTRY(privateKeyPasswordEntry), GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_DIALOG_AUTHENTICATION);
+    gtk_entry_set_icon_from_icon_name(GTK_ENTRY(privateKeyPasswordEntry),
+            GTK_ENTRY_ICON_PRIMARY, "dialog-password");
+
     gtk_entry_set_visibility(GTK_ENTRY(privateKeyPasswordEntry), FALSE);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), privateKeyPasswordEntry);
     gtk_entry_set_text(GTK_ENTRY(privateKeyPasswordEntry), tls_password);
@@ -230,7 +294,7 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
     gtk_entry_set_text(GTK_ENTRY(serverNameInstance), tls_server_name);
     gtk_grid_attach(GTK_GRID(grid), serverNameInstance, 1, 9, 1, 1);
 
-    label = gtk_label_new(_("Negotiation timeout (sec:msec)"));
+    label = gtk_label_new(_("Negotiation timeout (seconds)"));
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     gtk_grid_attach(GTK_GRID(grid), label, 0, 10, 1, 1);
     GtkWidget * tlsTimeOutSec;
@@ -240,11 +304,6 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), tlsTimeOutSec);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(tlsTimeOutSec), g_ascii_strtod(negotiation_timeout_sec, NULL));
     gtk_box_pack_start(GTK_BOX(hbox), tlsTimeOutSec, TRUE, TRUE, 0);
-    GtkWidget * tlsTimeOutMSec;
-    tlsTimeOutMSec = gtk_spin_button_new_with_range(0, pow(2,sizeof(long)), 1);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(label), tlsTimeOutMSec);
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(tlsTimeOutMSec), g_ascii_strtod(negotiation_timeout_msec, NULL));
-    gtk_box_pack_start(GTK_BOX(hbox), tlsTimeOutMSec, TRUE, TRUE, 0);
 
     GtkWidget * verifyCertificateServer;
     verifyCertificateServer = gtk_check_button_new_with_mnemonic(_("Verify incoming certificates, as a server"));
@@ -269,13 +328,19 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
     if (gtk_dialog_run(GTK_DIALOG(tlsDialog)) == GTK_RESPONSE_ACCEPT) {
         account_replace(account, CONFIG_TLS_LISTENER_PORT,
                         gtk_entry_get_text(GTK_ENTRY(tlsListenerPort)));
-        account_replace(account, CONFIG_TLS_CA_LIST_FILE, get_filename(caListFileChooser));
 
-        account_replace(account, CONFIG_TLS_CERTIFICATE_FILE,
-                        get_filename(certificateFileChooser));
+        gchar *ca_list_file = get_filename(caListFileChooser);
+        account_replace(account, CONFIG_TLS_CA_LIST_FILE, ca_list_file);
+        g_free(ca_list_file);
 
-        account_replace(account, CONFIG_TLS_PRIVATE_KEY_FILE,
-                        get_filename(privateKeyFileChooser));
+
+        gchar *certificate_file = get_filename(certificateFileChooser);
+        account_replace(account, CONFIG_TLS_CERTIFICATE_FILE, certificate_file);
+        g_free(certificate_file);
+
+        gchar *private_key_file = get_filename(privateKeyFileChooser);
+        account_replace(account, CONFIG_TLS_PRIVATE_KEY_FILE, private_key_file);
+        g_free(private_key_file);
 
         account_replace(account, CONFIG_TLS_PASSWORD, gtk_entry_get_text(GTK_ENTRY(privateKeyPasswordEntry)));
 
@@ -294,8 +359,6 @@ void show_advanced_tls_options(account_t *account, SFLPhoneClient *client)
         account_replace(account, CONFIG_TLS_REQUIRE_CLIENT_CERTIFICATE, toggle_to_string(requireCertificate));
 
         account_replace(account, CONFIG_TLS_NEGOTIATION_TIMEOUT_SEC, gtk_entry_get_text(GTK_ENTRY(tlsTimeOutSec)));
-
-        account_replace(account, CONFIG_TLS_NEGOTIATION_TIMEOUT_MSEC, gtk_entry_get_text(GTK_ENTRY(tlsTimeOutMSec)));
     }
 
     gtk_widget_destroy(GTK_WIDGET(tlsDialog));
