@@ -1,6 +1,7 @@
 /*
- *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2014 Savoir-Faire Linux Inc.
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
+ *  Author: Vivien Didelot <vivien.didelot@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,16 +35,22 @@
 #include "utils.h"
 #include "dbus.h"
 #include "codeclist.h"
+#include "video/video_capabilities.h"
 
-static GtkWidget *v4l2Device;
-static GtkWidget *v4l2Channel;
-static GtkWidget *v4l2Size;
-static GtkWidget *v4l2Rate;
+typedef struct {
+    VideoCapabilities *cap;
 
-static GtkListStore *v4l2DeviceList;
-static GtkListStore *v4l2ChannelList;
-static GtkListStore *v4l2SizeList;
-static GtkListStore *v4l2RateList;
+    /* Video choices */
+    gchar *name;
+    gchar *chan;
+    gchar *size;
+    gchar *rate;
+} VideoDevice;
+
+static GtkComboBoxText *v4l2Device;
+static GtkComboBoxText *v4l2Channel;
+static GtkComboBoxText *v4l2Size;
+static GtkComboBoxText *v4l2Rate;
 
 static GtkWidget *v4l2_hbox;
 static GtkWidget *v4l2_nodev;
@@ -101,11 +108,20 @@ static const gchar *const CAMERA_STOP_STR = "_Stop";
 static void
 camera_button_toggled(GtkButton *button, G_GNUC_UNUSED gpointer data)
 {
-    camera_button = GTK_WIDGET(button);
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
-        dbus_start_video_camera();
-    else
-        dbus_stop_video_camera();
+    gchar ** str = dbus_get_call_list();
+
+    /* we can toggle only if there is no call */
+    if (str == NULL || *str == NULL) {
+
+        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
+            dbus_start_video_camera();
+        } else {
+            dbus_stop_video_camera();
+        }
+
+    }
+
+    g_strfreev(str);
 
     update_camera_button_label();
 }
@@ -513,12 +529,12 @@ videocodecs_box(account_t *acc)
     gtk_container_set_border_width(GTK_CONTAINER(buttonBox), 10);
     gtk_box_pack_start(GTK_BOX(ret), buttonBox, FALSE, FALSE, 0);
 
-    codecMoveUpButton = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+    codecMoveUpButton = gtk_button_new_with_label(_("Up"));
     gtk_widget_set_sensitive(GTK_WIDGET(codecMoveUpButton), FALSE);
     gtk_box_pack_start(GTK_BOX(buttonBox), codecMoveUpButton, FALSE, FALSE, 0);
     g_signal_connect(G_OBJECT(codecMoveUpButton), "clicked", G_CALLBACK(codec_move_up), acc);
 
-    codecMoveDownButton = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
+    codecMoveDownButton = gtk_button_new_with_label(_("Down"));
     gtk_widget_set_sensitive(GTK_WIDGET(codecMoveDownButton), FALSE);
     gtk_box_pack_start(GTK_BOX(buttonBox), codecMoveDownButton, FALSE, FALSE, 0);
     g_signal_connect(G_OBJECT(codecMoveDownButton), "clicked", G_CALLBACK(codec_move_down), acc);
@@ -528,293 +544,156 @@ videocodecs_box(account_t *acc)
     return ret;
 }
 
-/* Gets a newly allocated string with the active text, the caller must
- * free this string */
-static gchar *
-get_active_text(GtkComboBox *box)
-{
-    gchar *text = NULL;
-    int comboBoxIndex = gtk_combo_box_get_active(box);
-    if (comboBoxIndex >= 0) {
-        GtkTreeIter iter;
-        gtk_combo_box_get_active_iter(box, &iter);
-        gtk_tree_model_get(gtk_combo_box_get_model(box), &iter, 0, &text, -1);
-    }
-    return text;
-}
-
-/* Return 0 if string was found in the combo box, != 0 if the string was not found */
-static int
-set_combo_index_from_str(GtkComboBox *box, const gchar *str, size_t max)
-{
-    g_assert(str);
-
-    GtkTreeModel *model = gtk_combo_box_get_model(box);
-    GtkTreeIter iter;
-    unsigned idx = 0;
-    gtk_tree_model_get_iter_first(model, &iter);
-    do {
-        gchar *boxstr = 0;
-        gtk_tree_model_get(model, &iter, 0, &boxstr, -1);
-        if (boxstr && !g_strcmp0(boxstr, str))
-            break;
-    } while (idx++ < max && gtk_tree_model_iter_next(model, &iter));
-
-    if (idx >= max)
-        return 1;
-
-    gtk_combo_box_set_active(box, idx);
-    return 0;
-}
-
-
-/**
- * Fill video input device rate store
- */
 static void
-preferences_dialog_fill_video_input_device_rate_list()
+reset_combo_box(GtkComboBoxText *combo, gchar **entries, const gchar *preferred)
 {
-    GtkTreeIter iter;
-    gchar** list = NULL;
+    g_assert(GTK_IS_WIDGET(combo));
 
-    if (v4l2RateList)
-        gtk_list_store_clear(v4l2RateList);
+    guint index = 0; /* first one if not found */
 
-    gchar *dev  = get_active_text(GTK_COMBO_BOX(v4l2Device));
-    gchar *chan = get_active_text(GTK_COMBO_BOX(v4l2Channel));
-    gchar *size = get_active_text(GTK_COMBO_BOX(v4l2Size));
+    /* Temporarily deactivate the "changed" signal to clear the list */
+    const guint signal_id = g_signal_lookup("changed", G_OBJECT_TYPE(combo));
+    const gulong handler_id = g_signal_handler_find(combo, G_SIGNAL_MATCH_ID, signal_id, 0, NULL, NULL, NULL);
+    g_signal_handler_block(combo, handler_id);
+    gtk_combo_box_text_remove_all(combo);
+    g_signal_handler_unblock(combo, handler_id);
 
-    // Call dbus to retreive list
-    if (dev && chan && size) {
-        list = dbus_get_video_device_rate_list(dev, chan, size);
-        g_free(size);
-        g_free(chan);
-        g_free(dev);
+    for (guint i = 0; entries[i]; ++i) {
+        gtk_combo_box_text_insert_text(combo, i, entries[i]);
+        if (g_strcmp0(entries[i], preferred) == 0)
+            index = i;
     }
 
-    // For each device name included in list
-    if (list && *list) {
-        gint c = 0;
-        for (gchar **tmp = list; *tmp; c++, tmp++) {
-            gtk_list_store_append(v4l2RateList, &iter);
-            gtk_list_store_set(v4l2RateList, &iter, 0, *tmp, 1, c, -1);
-        }
-        g_strfreev(list);
-
-        gchar *rate = dbus_get_active_video_device_rate();
-        if (!rate || !*rate || set_combo_index_from_str(GTK_COMBO_BOX(v4l2Rate), rate, c)) {
-            // if setting is invalid, choose first entry
-            gtk_combo_box_set_active(GTK_COMBO_BOX(v4l2Rate), 0);
-            gchar *selected = get_active_text(GTK_COMBO_BOX(v4l2Rate));
-            if (selected) {
-                dbus_set_active_video_device_rate(selected);
-                g_free(selected);
-            }
-        }
-        g_free(rate);
-    } else
-        g_warning("No video rate list found for device");
-}
-
-
-/**
- * Set the video input device rate on the server
- */
-static void
-select_video_input_device_rate_cb(GtkComboBox* comboBox, G_GNUC_UNUSED gpointer data)
-{
-    gchar *str = get_active_text(comboBox);
-    if (str)
-        dbus_set_active_video_device_rate(str);
-    g_free(str);
-}
-
-/**
- * Fill video input device size store
- */
-static void
-preferences_dialog_fill_video_input_device_size_list()
-{
-    if (v4l2SizeList)
-        gtk_list_store_clear(v4l2SizeList);
-
-    gchar *dev  = get_active_text(GTK_COMBO_BOX(v4l2Device));
-    gchar *chan = get_active_text(GTK_COMBO_BOX(v4l2Channel));
-
-    gchar** list = NULL;
-    // Call dbus to retrieve list
-    if (dev && chan) {
-        list = dbus_get_video_device_size_list(dev, chan);
-        g_free(chan);
-        g_free(dev);
-    }
-
-    if (list && *list) {
-        // For each device name included in list
-        gint c = 0;
-        for (gchar **tmp = list; *tmp; c++, tmp++) {
-            GtkTreeIter iter;
-            gtk_list_store_append(v4l2SizeList, &iter);
-            gtk_list_store_set(v4l2SizeList, &iter, 0, *tmp, 1, c, -1);
-        }
-        g_strfreev(list);
-        gchar *size = dbus_get_active_video_device_size();
-        if (!size || !*size || set_combo_index_from_str(GTK_COMBO_BOX(v4l2Size), size, c)) {
-            // if setting is invalid, choose first entry
-            gtk_combo_box_set_active(GTK_COMBO_BOX(v4l2Size), 0);
-            gchar *selected = get_active_text(GTK_COMBO_BOX(v4l2Size));
-            if (selected) {
-                dbus_set_active_video_device_size(selected);
-                g_free(selected);
-            }
-        }
-        g_free(size);
-    } else
-        g_warning("No device size list found");
-}
-
-/**
- * Set the video input device size on the server
- */
-static void
-select_video_input_device_size_cb(GtkComboBox* comboBox, G_GNUC_UNUSED gpointer data)
-{
-    gchar *str = get_active_text(comboBox);
-    if (str) {
-        dbus_set_active_video_device_size(str);
-        preferences_dialog_fill_video_input_device_rate_list();
-        g_free(str);
-    }
-}
-
-/**
- * Fill video input device input store
- */
-static void
-preferences_dialog_fill_video_input_device_channel_list()
-{
-    if (v4l2ChannelList)
-        gtk_list_store_clear(v4l2ChannelList);
-
-    gchar *dev = get_active_text(GTK_COMBO_BOX(v4l2Device));
-
-    gchar **list = NULL;
-    // Call dbus to retrieve list
-    if (dev) {
-        list = dbus_get_video_device_channel_list(dev);
-        g_free(dev);
-    }
-
-    if (list && *list) {
-        // For each device name included in list
-        int c = 0;
-        for (gchar **tmp = list; *tmp; c++, tmp++) {
-            GtkTreeIter iter;
-            gtk_list_store_append(v4l2ChannelList, &iter);
-            gtk_list_store_set(v4l2ChannelList, &iter, 0, *tmp, 1, c, -1);
-        }
-        g_strfreev(list);
-        gchar *channel = dbus_get_active_video_device_channel();
-        if (!channel || !*channel || set_combo_index_from_str(GTK_COMBO_BOX(v4l2Channel), channel, c)) {
-            // if setting is invalid, choose first entry
-            gtk_combo_box_set_active(GTK_COMBO_BOX(v4l2Channel), 0);
-            gchar *selected = get_active_text(GTK_COMBO_BOX(v4l2Channel));
-            if (selected) {
-                dbus_set_active_video_device_channel(selected);
-                g_free(selected);
-            }
-        }
-        g_free(channel);
-    } else
-        g_warning("No channel list found");
-}
-
-/**
- * Set the video input device input on the server
- */
-static void
-select_video_input_device_channel_cb(GtkComboBox* comboBox, G_GNUC_UNUSED gpointer data)
-{
-    gchar *str = get_active_text(comboBox);
-    if (str) {
-        dbus_set_active_video_device_channel(str);
-        preferences_dialog_fill_video_input_device_size_list();
-        g_free(str);
-    }
-}
-
-/**
- * Fill video input device store
- */
-static gboolean
-preferences_dialog_fill_video_input_device_list()
-{
-    gtk_list_store_clear(v4l2DeviceList);
-
-    // Call dbus to retrieve list
-    gchar **list = dbus_get_video_device_list();
-    if (!list || !*list) {
-        g_warning("No device list found");
-        return FALSE;
-    } else {
-        // For each device name included in list
-        gint c = 0;
-        for (gchar **tmp = list; *tmp; c++, tmp++) {
-            GtkTreeIter iter;
-            gtk_list_store_append(v4l2DeviceList, &iter);
-            gtk_list_store_set(v4l2DeviceList, &iter, 0, *tmp, 1, c, -1);
-        }
-        g_strfreev(list);
-        gchar *dev = dbus_get_active_video_device();
-        if (!dev || !*dev || set_combo_index_from_str(GTK_COMBO_BOX(v4l2Device), dev, c)) {
-            // if setting is invalid, choose first entry
-            gtk_combo_box_set_active(GTK_COMBO_BOX(v4l2Device), 0);
-            gchar *selected = get_active_text(GTK_COMBO_BOX(v4l2Device));
-            if (selected) {
-                dbus_set_active_video_device(selected);
-                g_free(selected);
-            }
-        }
-        g_free(dev);
-        return TRUE;
-    }
-}
-
-/**
- * Set the video input device on the server
- */
-static void
-select_video_input_device_cb(GtkComboBox* comboBox, G_GNUC_UNUSED gpointer data)
-{
-    gchar *str = get_active_text(comboBox);
-    if (str) {
-        g_debug("Setting video input device to %s", str);
-        dbus_set_active_video_device(str);
-        preferences_dialog_fill_video_input_device_channel_list();
-        g_free(str);
-    }
+    /* NOTE This will invoke the combo box's "changed" callback */
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), index);
 }
 
 static void
-fill_devices()
+fill_devices(void)
 {
-    if (preferences_dialog_fill_video_input_device_list()) {
+    gchar **devices = dbus_video_get_device_list();
+    g_assert(devices);
+
+    if (*devices) {
+        gchar *default_device = dbus_video_get_default_device();
+        reset_combo_box(v4l2Device, devices, default_device);
+        g_free(default_device);
+
         gtk_widget_show_all(v4l2_hbox);
         gtk_widget_hide(v4l2_nodev);
         gtk_widget_set_sensitive(camera_button, TRUE);
     } else if (GTK_IS_WIDGET(v4l2_hbox)) {
+        g_warning("No video devices");
         gtk_widget_hide(v4l2_hbox);
         gtk_widget_show(v4l2_nodev);
         gtk_widget_set_sensitive(camera_button, FALSE);
     }
+
+    g_strfreev(devices);
+}
+
+static void
+combo_box_device_changed_cb(G_GNUC_UNUSED GtkComboBoxText* combo, gpointer data)
+{
+    VideoDevice *dev = (VideoDevice *) data;
+    gchar *name = gtk_combo_box_text_get_active_text(v4l2Device);
+    g_assert(name);
+
+    /* Clear the video device */
+    if (dev->cap)
+        video_capabilities_free(dev->cap);
+    g_free(dev->name);
+    g_free(dev->chan);
+    g_free(dev->size);
+    g_free(dev->rate);
+
+    dev->name = name;
+    dev->cap = video_capabilities_new(name);
+
+    /* Fetch user preferences for this device */
+    GHashTable *hash = dbus_video_get_settings(name);
+    dev->chan = g_strdup(g_hash_table_lookup(hash, "channel"));
+    dev->size = g_strdup(g_hash_table_lookup(hash, "size"));
+    dev->rate = g_strdup(g_hash_table_lookup(hash, "rate"));
+    g_hash_table_destroy(hash);
+
+    /* Start the cascade into the channel combo box */
+    gchar **channels = video_capabilities_get_channels(dev->cap);
+    reset_combo_box(v4l2Channel, channels, dev->chan);
+    g_strfreev(channels);
+}
+
+static void
+combo_box_channel_changed_cb(G_GNUC_UNUSED GtkComboBoxText *combo, gpointer data)
+{
+    VideoDevice *dev = (VideoDevice *) data;
+    gchar *chan = gtk_combo_box_text_get_active_text(v4l2Channel);
+    g_assert(chan);
+    g_free(dev->chan);
+    dev->chan = chan;
+
+    /* Cascade into the size combo box */
+    gchar** sizes = video_capabilities_get_sizes(dev->cap, dev->chan);
+    reset_combo_box(v4l2Size, sizes, dev->size);
+    g_strfreev(sizes);
+}
+
+static void
+combo_box_size_changed_cb(G_GNUC_UNUSED GtkComboBoxText *combo, gpointer data)
+{
+    VideoDevice *dev = (VideoDevice *) data;
+    gchar *size = gtk_combo_box_text_get_active_text(v4l2Size);
+    g_assert(size);
+    g_free(dev->size);
+    dev->size = size;
+
+    /* Cascade into the rate combo box */
+    gchar **rates = video_capabilities_get_rates(dev->cap, dev->chan, dev->size);
+    reset_combo_box(v4l2Rate, rates, dev->rate);
+    g_strfreev(rates);
+}
+
+static void
+combo_box_rate_changed_cb(G_GNUC_UNUSED GtkComboBoxText *combo, gpointer data)
+{
+    VideoDevice *dev = (VideoDevice *) data;
+    gchar *rate = gtk_combo_box_text_get_active_text(v4l2Rate);
+    g_assert(rate);
+    g_free(dev->rate);
+    dev->rate = rate;
+
+    /* End of the cascade, save the default device and its settings */
+    dbus_video_set_default_device(dev->name);
+    GHashTable *hash = g_hash_table_new(g_str_hash, g_str_equal);
+    g_hash_table_insert(hash, "channel", dev->chan);
+    g_hash_table_insert(hash, "size", dev->size);
+    g_hash_table_insert(hash, "rate", dev->rate);
+    dbus_video_apply_settings(dev->name, hash);
+    g_hash_table_destroy(hash);
 }
 
 void
 video_device_event_cb(G_GNUC_UNUSED DBusGProxy *proxy, G_GNUC_UNUSED gpointer foo)
 {
+    /* FIXME: get rid of these global widgets */
+    g_return_if_fail(GTK_IS_WIDGET(v4l2Device));
     fill_devices();
 }
 
+static GtkComboBoxText *
+attach_combo_box(GtkGrid *grid, const guint row, const gchar *label, GCallback callback, gpointer data)
+{
+    /* Attach the label on left */
+    GtkWidget *item = gtk_label_new(_(label));
+    gtk_grid_attach(grid, item, 0, row, 1, 1);
+
+    /* Attach the combo combo on right */
+    GtkWidget *combo = gtk_combo_box_text_new();
+    g_signal_connect(G_OBJECT(combo), "changed", callback, data);
+    gtk_grid_attach(grid, combo, 1, row, 1, 1);
+
+    return GTK_COMBO_BOX_TEXT(combo);
+}
 
 static GtkWidget *
 v4l2_box()
@@ -831,67 +710,16 @@ v4l2_box()
     gtk_grid_set_column_spacing(GTK_GRID(grid), 40);
     gtk_box_pack_start(GTK_BOX(v4l2_hbox), grid, TRUE, TRUE, 1);
 
-    // Set choices of input devices
-    GtkWidget *item = gtk_label_new(_("Device"));
-    v4l2DeviceList = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
-    v4l2Device = gtk_combo_box_new_with_model(GTK_TREE_MODEL(v4l2DeviceList));
-    gtk_label_set_mnemonic_widget(GTK_LABEL(item), v4l2Device);
+    VideoDevice *dev = g_new0(VideoDevice, 1);
 
-    g_signal_connect(G_OBJECT(v4l2Device), "changed", G_CALLBACK(select_video_input_device_cb), NULL);
-    gtk_grid_attach(GTK_GRID(grid), item, 0, 0, 1, 1);
-
-    // Set rendering
-    GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(v4l2Device), renderer, TRUE);
-    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(v4l2Device), renderer, "text", 0, NULL);
-    gtk_grid_attach(GTK_GRID(grid), v4l2Device, 1, 0, 1, 1);
-
-    // Set choices of input
-    item = gtk_label_new(_("Channel"));
-    v4l2ChannelList = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
-    v4l2Channel = gtk_combo_box_new_with_model(GTK_TREE_MODEL(v4l2ChannelList));
-    gtk_label_set_mnemonic_widget(GTK_LABEL(item), v4l2Channel);
-    g_signal_connect(G_OBJECT(v4l2Channel), "changed", G_CALLBACK(select_video_input_device_channel_cb), NULL);
-    gtk_grid_attach(GTK_GRID(grid), item, 0, 1, 1, 1);
-
-    // Set rendering
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(v4l2Channel), renderer, TRUE);
-    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(v4l2Channel), renderer, "text", 0, NULL);
-    gtk_grid_attach(GTK_GRID(grid), v4l2Channel, 1, 1, 1, 1);
-
-    // Set choices of sizes
-    item = gtk_label_new(_("Size"));
-    v4l2SizeList = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
-    v4l2Size = gtk_combo_box_new_with_model(GTK_TREE_MODEL(v4l2SizeList));
-    gtk_label_set_mnemonic_widget(GTK_LABEL(item), v4l2Size);
-    g_signal_connect(G_OBJECT(v4l2Size), "changed", G_CALLBACK(select_video_input_device_size_cb), NULL);
-    gtk_grid_attach(GTK_GRID(grid), item, 0, 2, 1, 1);
-
-    // Set rendering
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(v4l2Size), renderer, TRUE);
-    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(v4l2Size), renderer, "text", 0, NULL);
-    gtk_grid_attach(GTK_GRID(grid), v4l2Size, 1, 2, 1, 1);
-
-    // Set choices of rates
-    item = gtk_label_new(_("Rate"));
-    v4l2RateList = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
-    v4l2Rate = gtk_combo_box_new_with_model(GTK_TREE_MODEL(v4l2RateList));
-    gtk_label_set_mnemonic_widget(GTK_LABEL(item), v4l2Rate);
-    g_signal_connect(G_OBJECT(v4l2Rate), "changed", G_CALLBACK(select_video_input_device_rate_cb), NULL);
-    gtk_grid_attach(GTK_GRID(grid), item, 0, 3, 1, 1);
-
-    // Set rendering
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(v4l2Rate), renderer, TRUE);
-    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(v4l2Rate), renderer, "text", 0, NULL);
-    gtk_grid_attach(GTK_GRID(grid), v4l2Rate, 1, 3, 1, 1);
+    v4l2Device = attach_combo_box(GTK_GRID(grid), 0, "Device", G_CALLBACK(combo_box_device_changed_cb), dev);
+    v4l2Channel = attach_combo_box(GTK_GRID(grid), 1, "Channel", G_CALLBACK(combo_box_channel_changed_cb), dev);
+    v4l2Size = attach_combo_box(GTK_GRID(grid), 2, "Size", G_CALLBACK(combo_box_size_changed_cb), dev);
+    v4l2Rate = attach_combo_box(GTK_GRID(grid), 3, "Rate", G_CALLBACK(combo_box_rate_changed_cb), dev);
 
     return ret;
 }
 
-
 GtkWidget *
 create_video_configuration()
 {
