@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,7 @@
 GtkWidget* create_video_configuration();
 GtkWidget* videocodecs_box();
 void video_device_event_cb(DBusGProxy *proxy, void * foo);
-void update_preview_button_label();
-void set_preview_button_sensitivity(gboolean sensitive);
+void update_camera_button_label();
+void set_camera_button_sensitivity(gboolean sensitive);
 
 #endif // __VIDEO_CONF_H__
