@@ -38,8 +38,10 @@
 #include "calltree.h"
 #include "str_utils.h"
 #include "account_schema.h"
+#include <glib/gi18n.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <gtk/gtk.h>
 
 #include "dbus.h"
@@ -70,9 +72,8 @@ typedef struct {
 
 static PopupData *popup_data = NULL;
 
-// Messages used in menu item
-static const gchar * const SFL_CREATE_CONFERENCE = "Create conference";
-static const gchar * const SFL_TRANSFER_CALL = "Transfer call to";
+#define SFL_CREATE_CONFERENCE _("Create conference")
+#define SFL_TRANSFER_CALL _("Transfer call to")
 
 static GtkWidget *calltree_popupmenu = NULL;
 static GtkWidget *calltree_menu_items = NULL;
@@ -272,12 +273,12 @@ row_single_click(G_GNUC_UNUSED GtkTreeView *tree_view, SFLPhoneClient *client)
                             selectedCall->_zrtp_confirmed = TRUE;
 
                         dbus_confirm_sas(selectedCall);
-                        calltree_update_call(current_calls_tab, selectedCall, client, TRUE);
+                        calltree_update_call(current_calls_tab, selectedCall, client);
                         break;
                     case SRTP_STATE_ZRTP_SAS_CONFIRMED:
                         selectedCall->_srtp_state = SRTP_STATE_ZRTP_SAS_UNCONFIRMED;
                         dbus_reset_sas(selectedCall);
-                        calltree_update_call(current_calls_tab, selectedCall, client, TRUE);
+                        calltree_update_call(current_calls_tab, selectedCall, client);
                         break;
                     default:
                         g_debug("Single click but no action");
@@ -324,9 +325,7 @@ static gchar *clean_display_number(gchar *name)
 }
 
 static gchar *
-calltree_display_call_info(callable_obj_t * call, CallDisplayType display_type,
-                           const gchar *const audio_codec,
-                           const gchar *const video_codec)
+calltree_display_call_info(callable_obj_t * call, CallDisplayType display_type)
 {
     gchar display_number[strlen(call->_peer_number) + 1];
     strcpy(display_number, call->_peer_number);
@@ -355,33 +354,22 @@ calltree_display_call_info(callable_obj_t * call, CallDisplayType display_type,
 
     gchar *desc = g_markup_printf_escaped("<b>%s</b>   <i>%s</i>   ", name, details);
     gchar *suffix = NULL;
-    gchar *codec = NULL;
 
     switch (display_type) {
         case DISPLAY_TYPE_CALL:
             if (call->_state_code)
                 suffix = g_markup_printf_escaped("\n<i>%s (%d)</i>", call->_state_code_description, call->_state_code);
             break;
-        case DISPLAY_TYPE_STATE_CODE :
-            if (video_codec && *video_codec)
-                codec = g_strconcat(audio_codec, " ", video_codec, NULL);
-            else
-                codec = g_strdup(audio_codec);
 
             if (call->_state_code)
-                suffix = g_markup_printf_escaped("\n<i>%s (%d)</i>  <i>%s</i>",
-                                                 call->_state_code_description, call->_state_code,
-                                                 codec);
-            else
-                suffix = g_markup_printf_escaped("\n<i>%s</i>", codec);
-
-            g_free(codec);
+                suffix = g_markup_printf_escaped("\n<i>%s (%d)</i>",
+                                                 call->_state_code_description, call->_state_code);
             break;
         case DISPLAY_TYPE_CALL_TRANSFER:
-            suffix = g_markup_printf_escaped("\n<i>Transfer to:%s</i> ", call->_trsft_to);
+            suffix = g_markup_printf_escaped(_("\n<i>Transfer to:%s</i> "), call->_trsft_to);
             break;
         case DISPLAY_TYPE_SAS:
-            suffix = g_markup_printf_escaped("\n<i>Confirm SAS <b>%s</b> ?</i>", call->_sas);
+            suffix = g_markup_printf_escaped(_("\n<i>Confirm SAS <b>%s</b> ?</i>"), call->_sas);
             break;
         case DISPLAY_TYPE_HISTORY :
         default:
@@ -632,7 +620,6 @@ static GdkPixbuf *history_state_to_pixbuf(callable_obj_t *call)
 typedef struct {
     calltab_t *tab;
     callable_obj_t *call;
-    gboolean update_codecs;
 } CallUpdateCtx;
 
 typedef struct {
@@ -679,32 +666,13 @@ update_call(GtkTreeModel *model, G_GNUC_UNUSED GtkTreePath *path, GtkTreeIter *i
     /* Update text */
     gchar *description = NULL;
 
-    gchar *audio_codec;
-    gchar *video_codec;
-
-    /* Don't get codec names if call does not exist */
-    if (ctx->update_codecs) {
-        audio_codec = call_get_audio_codec(call);
-#ifdef SFL_VIDEO
-        video_codec = call_get_video_codec(call);
-#else
-        video_codec = g_strdup("");
-#endif
-    } else {
-        audio_codec = g_strdup("");
-        video_codec = g_strdup("");
-    }
-
     if (call->_state == CALL_STATE_TRANSFER)
-        description = calltree_display_call_info(call, DISPLAY_TYPE_CALL_TRANSFER, "", "");
+        description = calltree_display_call_info(call, DISPLAY_TYPE_CALL_TRANSFER);
     else
         if (call->_sas && display_sas && call->_srtp_state == SRTP_STATE_ZRTP_SAS_UNCONFIRMED && !call->_zrtp_confirmed)
-            description = calltree_display_call_info(call, DISPLAY_TYPE_SAS, "", "");
+            description = calltree_display_call_info(call, DISPLAY_TYPE_SAS);
         else
-            description = calltree_display_call_info(call, DISPLAY_TYPE_STATE_CODE, audio_codec, video_codec);
-
-    g_free(video_codec);
-    g_free(audio_codec);
+            description = calltree_display_call_info(call, DISPLAY_TYPE_STATE_CODE);
 
     /* Update icons */
     if (calltab_has_name(tab, CURRENT_CALLS)) {
@@ -769,7 +737,7 @@ update_call(GtkTreeModel *model, G_GNUC_UNUSED GtkTreePath *path, GtkTreeIter *i
         pixbuf = history_state_to_pixbuf(call);
 
         g_free(description);
-        description = calltree_display_call_info(call, DISPLAY_TYPE_HISTORY, "", "");
+        description = calltree_display_call_info(call, DISPLAY_TYPE_HISTORY);
         gchar *date = get_formatted_start_timestamp(call->_time_start);
         gchar *duration = get_call_duration(call);
         gchar *full_duration = g_strconcat(date , duration , NULL);
@@ -800,13 +768,13 @@ update_call(GtkTreeModel *model, G_GNUC_UNUSED GtkTreePath *path, GtkTreeIter *i
 }
 
 void
-calltree_update_call(calltab_t* tab, callable_obj_t * call, SFLPhoneClient *client, gboolean update_codecs)
+calltree_update_call(calltab_t* tab, callable_obj_t * call, SFLPhoneClient *client)
 {
     if (!call) {
         g_warning("Call is NULL, ignoring");
         return;
     }
-    CallUpdateCtx ctx = {tab, call, update_codecs};
+    CallUpdateCtx ctx = {tab, call};
     GtkTreeStore *store = tab->store;
     GtkTreeModel *model = GTK_TREE_MODEL(store);
     gtk_tree_model_foreach(model, update_call, (gpointer) &ctx);
@@ -828,7 +796,7 @@ void calltree_add_call(calltab_t* tab, callable_obj_t * call, GtkTreeIter *paren
 
     // New call in the list
 
-    gchar *description = calltree_display_call_info(call, DISPLAY_TYPE_CALL, "", "");
+    gchar *description = calltree_display_call_info(call, DISPLAY_TYPE_CALL);
 
     gtk_tree_store_prepend(tab->store, &iter, parent);
 
@@ -915,7 +883,7 @@ void calltree_add_call(calltab_t* tab, callable_obj_t * call, GtkTreeIter *paren
 void calltree_add_history_entry(callable_obj_t *call)
 {
     // New call in the list
-    gchar * description = calltree_display_call_info(call, DISPLAY_TYPE_HISTORY, "", "");
+    gchar * description = calltree_display_call_info(call, DISPLAY_TYPE_HISTORY);
 
     GtkTreeIter iter;
     gtk_tree_store_prepend(history_tab->store, &iter, NULL);
@@ -1202,18 +1170,16 @@ void calltree_display(calltab_t *tab, SFLPhoneClient *client)
     update_actions(client);
 }
 
-
 gboolean calltree_update_clock(G_GNUC_UNUSED gpointer data)
 {
     if (calllist_empty(current_calls_tab))
         return TRUE;
 
-    char timestr[20];
+    char timestr[32];
     const gchar *msg = "";
-    long duration;
     callable_obj_t *call = calltab_get_selected_call(current_calls_tab);
 
-    if (call)
+    if (call) {
         switch (call->_state) {
             case CALL_STATE_INVALID:
             case CALL_STATE_INCOMING:
@@ -1223,15 +1189,11 @@ gboolean calltree_update_clock(G_GNUC_UNUSED gpointer data)
             case CALL_STATE_BUSY:
                 break;
             default:
-                duration = difftime(time(NULL), call->_time_start);
-
-                if (duration < 0)
-                    duration = 0;
-
-                g_snprintf(timestr, sizeof(timestr), "%.2ld:%.2ld", duration / 60, duration % 60);
+                format_duration(call, time(NULL), timestr, sizeof(timestr));
                 msg = timestr;
                 break;
         }
+    }
 
     statusbar_update_clock(msg);
     return TRUE;
@@ -1321,6 +1283,7 @@ handle_drop_into(GtkTreeModel *model, GtkTreeIter *source_iter, GtkTreeIter *des
                 sflphone_add_participant(source_ID, conf_ID);
                 result = TRUE;
             }
+            g_free(conf_ID);
         }
     }
     g_value_unset(&source_val);
