@@ -222,7 +222,7 @@ update_search_entry(GtkEntry *entry, const gchar *search_str,
 static void
 search_all(G_GNUC_UNUSED GtkWidget *item, GtkEntry *entry)
 {
-    gtk_entry_set_icon_from_stock(entry, GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_FIND);
+    gtk_entry_set_icon_from_icon_name(entry, GTK_ENTRY_ICON_PRIMARY, "edit-find-symbolic");
     update_search_entry(entry, _("Search all"),
                         _("Click here to change the search type"), SEARCH_ALL);
 }
@@ -309,7 +309,7 @@ history_searchbar_new(GSettings *settings)
     GtkWidget *ret = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
     searchbox = gtk_entry_new();
-    gtk_entry_set_icon_from_stock(GTK_ENTRY(searchbox), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+    gtk_entry_set_icon_from_icon_name(GTK_ENTRY(searchbox), GTK_ENTRY_ICON_SECONDARY, "edit-clear-symbolic");
 
     missed_pixbuf = gdk_pixbuf_new_from_file(ICONS_DIR "/missed.svg", NULL);
     incoming_pixbuf = gdk_pixbuf_new_from_file(ICONS_DIR "/incoming.svg", NULL);
@@ -328,25 +328,25 @@ history_searchbar_new(GSettings *settings)
     menu = gtk_menu_new();
     gtk_menu_attach_to_widget(GTK_MENU(menu), searchbox, NULL);
 
-    GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
+    GtkWidget *image = gtk_image_new_from_icon_name("edit-find-symbolic", GTK_ICON_SIZE_MENU);
     GtkWidget *item = gtk_image_menu_item_new_with_label("Search all");
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
     g_signal_connect(item, "activate", G_CALLBACK(search_all), searchbox);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
-    item = gtk_image_menu_item_new_with_label("Search by missed call");
+    item = gtk_image_menu_item_new_with_label(_("Search by missed call"));
     image = gtk_image_new_from_file(ICONS_DIR "/missed.svg");
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
     g_signal_connect(item, "activate", G_CALLBACK(search_by_missed), searchbox);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
-    item = gtk_image_menu_item_new_with_label("Search by incoming call");
+    item = gtk_image_menu_item_new_with_label(_("Search by incoming call"));
     image = gtk_image_new_from_file(ICONS_DIR "/incoming.svg");
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
     g_signal_connect(item, "activate", G_CALLBACK(search_by_incoming), searchbox);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
-    item = gtk_image_menu_item_new_with_label("Search by outgoing call");
+    item = gtk_image_menu_item_new_with_label(_("Search by outgoing call"));
     image = gtk_image_new_from_file(ICONS_DIR "/outgoing.svg");
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
     g_signal_connect(item, "activate", G_CALLBACK(search_by_outgoing), searchbox);
@@ -427,8 +427,8 @@ contacts_searchbar_new(GSettings *settings)
     gchar *tooltip_text = g_strdup("Search is");
 
     addressbookentry = gtk_entry_new();
-    gtk_entry_set_icon_from_stock(GTK_ENTRY(addressbookentry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
-    gtk_entry_set_icon_from_stock(GTK_ENTRY(addressbookentry), GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_FIND);
+    gtk_entry_set_icon_from_icon_name(GTK_ENTRY(addressbookentry), GTK_ENTRY_ICON_SECONDARY, "edit-clear-symbolic");
+    gtk_entry_set_icon_from_icon_name(GTK_ENTRY(addressbookentry), GTK_ENTRY_ICON_PRIMARY, "edit-find-symbolic");
     gtk_entry_set_icon_tooltip_text(GTK_ENTRY(addressbookentry), GTK_ENTRY_ICON_PRIMARY,
                                     tooltip_text);
 
