@@ -39,7 +39,7 @@
 #include "callmanager-glue.h"
 #include "configurationmanager-glue.h"
 #ifdef SFL_VIDEO
-#include "video_controls-glue.h"
+#include "videomanager-glue.h"
 #endif
 #include "instance-glue.h"
 #include "preferencesdialog.h"
@@ -203,7 +203,7 @@ process_existing_call_state_change(callable_obj_t *c, const gchar *state, SFLPho
         if (c->_state == CALL_STATE_CURRENT)
             time(&c->_time_stop);
 
-        calltree_update_call(history_tab, c, client, FALSE);
+        calltree_update_call(history_tab, c, client);
         status_bar_display_account();
         sflphone_hung_up(c, client);
     } else if (g_strcmp0(state, "UNHOLD") == 0 || g_strcmp0(state, "CURRENT") == 0)
@@ -515,11 +515,10 @@ static void
 secure_zrtp_on_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *callID,
                   const gchar *cipher, SFLPhoneClient *client)
 {
-    g_debug("SRTP using ZRTP is ON secure_on_cb");
+    g_debug("SRTP using ZRTP is ON with cipher %s", cipher);
     callable_obj_t *c = calllist_get_call(current_calls_tab, callID);
 
     if (c) {
-        c->_srtp_cipher = g_strdup(cipher);
         sflphone_srtp_zrtp_on(c, client);
         notify_secure_on(c, client);
     }
@@ -570,6 +569,27 @@ zrtp_not_supported_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *callID, SFLP
     }
 }
 
+
+#ifdef RTCP_DEBUG
+static void
+print_rtcp_stats(const gchar *key, gint value, G_GNUC_UNUSED gpointer data)
+{
+    g_debug("%s: %d", key, value);
+}
+#endif
+
+static void
+on_rtcp_report_received_cb(G_GNUC_UNUSED DBusGProxy *proxy,
+        const gchar *callID,
+        G_GNUC_UNUSED const GHashTable *stats,
+        G_GNUC_UNUSED SFLPhoneClient *client)
+{
+    g_debug("Daemon notification of new RTCP report for %s", callID);
+#ifdef RTCP_DEBUG
+    g_hash_table_foreach(stats, print_rtcp_stats, NULL);
+#endif
+}
+
 static void
 sip_call_state_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *callID,
                   const gchar *description, guint code, SFLPhoneClient *client)
@@ -822,10 +842,11 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
         g_cclosure_user_marshal_VOID__STRING_STRING_STRING, G_TYPE_NONE,
         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
 
-    /* Register STRING STRING INT INT Marshaller */
+    /* Register STRING STRING INT INT BOOL Marshaller */
     dbus_g_object_register_marshaller(
-        g_cclosure_user_marshal_VOID__STRING_STRING_INT_INT, G_TYPE_NONE,
-        G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID);
+        g_cclosure_user_marshal_VOID__STRING_STRING_INT_INT_BOOLEAN, G_TYPE_NONE,
+        G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_BOOLEAN,
+        G_TYPE_INVALID);
 
     /* Register STRING STRING BOOLEAN STRING Marshaller */
     dbus_g_object_register_marshaller(
@@ -955,6 +976,20 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
     dbus_g_proxy_connect_signal(call_proxy, "sipCallStateChanged",
                                 G_CALLBACK(sip_call_state_cb), client, NULL);
 
+
+    /* Manually register this marshaller as we need to declare that the boxed type is in fact
+     * a GHashTable */
+    dbus_g_object_register_marshaller(g_cclosure_user_marshal_VOID__STRING_BOXED, G_TYPE_NONE,
+            G_TYPE_STRING, G_TYPE_HASH_TABLE, G_TYPE_INVALID);
+
+    dbus_g_proxy_add_signal(call_proxy, "onRtcpReportReceived",
+                            G_TYPE_STRING,
+                            dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_INT),
+                            G_TYPE_INVALID);
+
+    dbus_g_proxy_connect_signal(call_proxy, "onRtcpReportReceived",
+                                G_CALLBACK(on_rtcp_report_received_cb), client, NULL);
+
     g_debug("Adding configurationmanager Dbus signals");
 
     dbus_g_proxy_add_signal(config_proxy, "accountsChanged", G_TYPE_INVALID);
@@ -998,12 +1033,12 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
 #endif
 
 #ifdef SFL_VIDEO
-    const gchar *videocontrols_object_instance = "/org/sflphone/SFLphone/VideoControls";
-    const gchar *videocontrols_interface = "org.sflphone.SFLphone.VideoControls";
+    const gchar *videomanager_object_instance = "/org/sflphone/SFLphone/VideoManager";
+    const gchar *videomanager_interface = "org.sflphone.SFLphone.VideoManager";
     video_proxy = dbus_g_proxy_new_for_name(connection, dbus_message_bus_name,
-            videocontrols_object_instance, videocontrols_interface);
+            videomanager_object_instance, videomanager_interface);
     if (video_proxy == NULL) {
-        g_warning("Error: Failed to connect to %s", videocontrols_object_instance);
+        g_warning("Error: Failed to connect to %s", videomanager_object_instance);
         return FALSE;
     }
     /* Video related signals */
@@ -1012,16 +1047,15 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
             G_CALLBACK(video_device_event_cb), NULL, NULL);
 
     dbus_g_proxy_add_signal(video_proxy, "startedDecoding", G_TYPE_STRING,
-            G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID);
+            G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_BOOLEAN,
+            G_TYPE_INVALID);
     dbus_g_proxy_connect_signal(video_proxy, "startedDecoding",
-            G_CALLBACK(started_decoding_video_cb), NULL,
-            NULL);
+            G_CALLBACK(started_decoding_video_cb), client, NULL);
 
     dbus_g_proxy_add_signal(video_proxy, "stoppedDecoding",
-            G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
+            G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INVALID);
     dbus_g_proxy_connect_signal(video_proxy, "stoppedDecoding",
-            G_CALLBACK(stopped_decoding_video_cb),
-            NULL, NULL);
+            G_CALLBACK(stopped_decoding_video_cb), client, NULL);
 #endif
 
     /* Defines a default timeout for the proxies */
@@ -1048,7 +1082,7 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
 void dbus_clean()
 {
 #ifdef SFL_VIDEO
-        g_object_unref(video_proxy);
+    g_object_unref(video_proxy);
 #endif
     g_object_unref(call_proxy);
     g_object_unref(config_proxy);
@@ -1115,10 +1149,11 @@ dbus_stop_recorded_file_playback(const gchar *filepath)
 }
 
 static void
-hang_up_reply_cb(G_GNUC_UNUSED DBusGProxy *proxy, gboolean is_hung_up, GError *error, G_GNUC_UNUSED gpointer userdata)
+hang_up_reply_cb(G_GNUC_UNUSED DBusGProxy *proxy,
+        G_GNUC_UNUSED gboolean is_hung_up,
+        GError *error,
+        G_GNUC_UNUSED gpointer userdata)
 {
-    if (!is_hung_up)
-        g_warning("Did not hang up properly");
     check_error(error);
 }
 
@@ -1241,7 +1276,7 @@ dbus_get_credentials(account_t *a)
 }
 
 GHashTable *
-dbus_get_ip2_ip_details(void)
+dbus_get_ip2ip_details(void)
 {
     GError *error = NULL;
     GHashTable *details = NULL;
@@ -1412,7 +1447,7 @@ dbus_get_video_codecs(const gchar *accountID)
 {
     GError *error = NULL;
     GPtrArray *array = NULL;
-    org_sflphone_SFLphone_VideoControls_get_codecs(video_proxy, accountID, &array, &error);
+    org_sflphone_SFLphone_VideoManager_get_codecs(video_proxy, accountID, &array, &error);
     check_error(error);
     return array;
 }
@@ -1421,9 +1456,19 @@ void
 dbus_set_video_codecs(const gchar *accountID, const GPtrArray *list)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_VideoControls_set_codecs(video_proxy, accountID, list, &error);
+    org_sflphone_SFLphone_VideoManager_set_codecs(video_proxy, accountID, list, &error);
     check_error(error);
 }
+
+gboolean
+dbus_switch_video_input(const gchar *resource)
+{
+    GError *error = NULL;
+    gboolean switched;
+    org_sflphone_SFLphone_VideoManager_switch_input(video_proxy, resource, &switched, &error);
+    check_error(error);
+    return switched;
+}
 #endif
 
 gchar **
@@ -1436,38 +1481,6 @@ dbus_audio_codec_details(int payload)
     return array;
 }
 
-#ifdef SFL_VIDEO
-
-gchar *
-dbus_get_current_video_codec_name(const callable_obj_t *c)
-{
-    gchar *codecName = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_VideoControls_get_current_codec_name(video_proxy,
-            c->_callID, &codecName, &error);
-
-    if (check_error(error)) {
-        g_free(codecName);
-        codecName = g_strdup("");
-    }
-
-    return codecName;
-}
-#endif
-
-gchar *
-dbus_get_current_audio_codec_name(const callable_obj_t *c)
-{
-    gchar *codecName = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_CallManager_get_current_audio_codec_name(call_proxy, c->_callID, &codecName,
-                                      &error);
-    check_error(error);
-    return codecName;
-}
-
 GArray *
 dbus_get_active_audio_codec_list(const gchar *accountID)
 {
@@ -1589,11 +1602,25 @@ dbus_get_current_audio_devices_index()
  * Get index
  */
 int
-dbus_get_audio_device_index(const gchar *name)
+dbus_get_audio_output_device_index(const gchar *name)
 {
     int device_index = 0;
     GError *error = NULL;
-    org_sflphone_SFLphone_ConfigurationManager_get_audio_device_index(config_proxy, name, &device_index, &error);
+    org_sflphone_SFLphone_ConfigurationManager_get_audio_output_device_index(config_proxy, name, &device_index, &error);
+    check_error(error);
+
+    return device_index;
+}
+
+/**
+ * Get index
+ */
+int
+dbus_get_audio_input_device_index(const gchar *name)
+{
+    int device_index = 0;
+    GError *error = NULL;
+    org_sflphone_SFLphone_ConfigurationManager_get_audio_input_device_index(config_proxy, name, &device_index, &error);
     check_error(error);
 
     return device_index;
@@ -1639,12 +1666,7 @@ dbus_set_noise_suppress_state(gboolean state)
 {
     GError *error = NULL;
     org_sflphone_SFLphone_ConfigurationManager_set_noise_suppress_state(config_proxy, state, &error);
-
-    if (error) {
-        g_warning("Failed to call set_noise_suppress_state() on "
-              "ConfigurationManager: %s", error->message);
-        g_error_free(error);
-    }
+    check_error(error);
 }
 
 /**
@@ -1671,12 +1693,7 @@ dbus_set_agc_state(gboolean state)
 {
     GError *error = NULL;
     org_sflphone_SFLphone_ConfigurationManager_set_agc_state(config_proxy, state, &error);
-
-    if (error) {
-        g_warning("Failed to call set_agc_state() on "
-              "ConfigurationManager: %s", error->message);
-        g_error_free(error);
-    }
+    check_error(error);
 }
 
 int
@@ -1690,36 +1707,58 @@ dbus_is_iax2_enabled()
     return res;
 }
 
+static void
+dbus_join_participant_async_cb(G_GNUC_UNUSED DBusGProxy *proxy,
+                      gboolean result,
+                      GError *error,
+                      G_GNUC_UNUSED gpointer data)
+{
+    check_error(error);
+    if (!result)
+        g_warning("Failed to join participant");
+}
+
 void
 dbus_join_participant(const gchar *sel_callID, const gchar *drag_callID)
 {
-    g_debug("Join participant %s and %s\n", sel_callID, drag_callID);
-    GError *error = NULL;
-    gboolean result;
-    org_sflphone_SFLphone_CallManager_join_participant(call_proxy, sel_callID,
-            drag_callID, &result, &error);
+    org_sflphone_SFLphone_CallManager_join_participant_async(call_proxy, sel_callID,
+            drag_callID, dbus_join_participant_async_cb, NULL);
+}
+
+static void
+dbus_add_participant_async_cb(G_GNUC_UNUSED DBusGProxy *proxy,
+                              gboolean result,
+                              GError *error,
+                              G_GNUC_UNUSED gpointer data)
+{
     check_error(error);
+    if (!result)
+        g_warning("Failed to add participant");
 }
 
 void
 dbus_add_participant(const gchar *callID, const gchar *confID)
 {
-    g_debug("Add participant %s to %s\n", callID, confID);
-    GError *error = NULL;
-    gboolean result;
-    org_sflphone_SFLphone_CallManager_add_participant(call_proxy, callID,
-            confID, &result, &error);
+    org_sflphone_SFLphone_CallManager_add_participant_async(call_proxy, callID,
+            confID, dbus_add_participant_async_cb, NULL);
+}
+
+static void
+dbus_add_main_participant_async_cb(G_GNUC_UNUSED DBusGProxy *proxy,
+                                   gboolean result,
+                                   GError *error,
+                                   G_GNUC_UNUSED gpointer data)
+{
     check_error(error);
+    if (!result)
+        g_warning("Failed to add main participant");
 }
 
 void
 dbus_add_main_participant(const gchar *confID)
 {
-    GError *error = NULL;
-    gboolean result;
-    org_sflphone_SFLphone_CallManager_add_main_participant(call_proxy, confID,
-            &result, &error);
-    check_error(error);
+    org_sflphone_SFLphone_CallManager_add_main_participant_async(call_proxy,
+            confID, dbus_add_main_participant_async_cb, NULL);
 }
 
 void
@@ -1832,12 +1871,14 @@ dbus_clear_history(void)
     check_error(error);
 }
 
-void
+gboolean
 dbus_set_audio_manager(const gchar *api)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_ConfigurationManager_set_audio_manager(config_proxy, api, &error);
+    gboolean result;
+    org_sflphone_SFLphone_ConfigurationManager_set_audio_manager(config_proxy, api, &result, &error);
     check_error(error);
+    return result;
 }
 
 gchar *
@@ -1851,137 +1892,79 @@ dbus_get_audio_manager(void)
     return api;
 }
 
+gchar **
+dbus_get_supported_audio_managers()
+{
+    GError *error = NULL;
+    gchar **array = NULL;
+    org_sflphone_SFLphone_ConfigurationManager_get_supported_audio_managers(config_proxy, &array, &error);
+    check_error(error);
+
+    return array;
+}
+
 #ifdef SFL_VIDEO
 gchar *
-dbus_get_active_video_device_channel()
+dbus_video_get_default_device()
 {
     gchar *str = NULL;
     GError *error = NULL;
 
-    org_sflphone_SFLphone_VideoControls_get_active_device_channel(video_proxy, &str, &error);
-    check_error(error);
-
-    return str;
-}
-
-gchar *
-dbus_get_active_video_device_size()
-{
-    gchar *str = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_VideoControls_get_active_device_size(video_proxy, &str, &error);
-    check_error(error);
-
-    return str;
-}
-
-gchar *
-dbus_get_active_video_device_rate()
-{
-    gchar *str = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_VideoControls_get_active_device_rate(video_proxy, &str, &error);
-    check_error(error);
-
-    return str;
-}
-
-gchar *
-dbus_get_active_video_device()
-{
-    gchar *str = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_VideoControls_get_active_device(video_proxy, &str, &error);
+    org_sflphone_SFLphone_VideoManager_get_default_device(video_proxy, &str, &error);
     check_error(error);
 
     return str;
 }
 
 void
-dbus_set_active_video_device(const gchar *device)
+dbus_video_set_default_device(const gchar *name)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_VideoControls_set_active_device(video_proxy, device, &error);
+    org_sflphone_SFLphone_VideoManager_set_default_device(video_proxy, name, &error);
     check_error(error);
 }
 
+GHashTable *
+dbus_video_get_settings(const gchar *name)
+{
+    GError *error = NULL;
+    GHashTable *settings = NULL;
+
+    org_sflphone_SFLphone_VideoManager_get_settings(video_proxy, name, &settings, &error);
+    check_error(error);
+
+    return settings;
+}
+
 void
-dbus_set_active_video_device_channel(const gchar *channel)
+dbus_video_apply_settings(const gchar *name, GHashTable *settings)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_VideoControls_set_active_device_channel(video_proxy, channel, &error);
-    check_error(error);
-}
-
-void
-dbus_set_active_video_device_size(const gchar *size)
-{
-    GError *error = NULL;
-    org_sflphone_SFLphone_VideoControls_set_active_device_size(video_proxy, size, &error);
-    check_error(error);
-}
-
-void
-dbus_set_active_video_device_rate(const gchar *rate)
-{
-    GError *error = NULL;
-    org_sflphone_SFLphone_VideoControls_set_active_device_rate(video_proxy, rate, &error);
+    org_sflphone_SFLphone_VideoManager_apply_settings(video_proxy, name, settings, &error);
     check_error(error);
 }
 
 gchar **
-dbus_get_video_device_list()
+dbus_video_get_device_list()
 {
     gchar **array = NULL;
     GError *error = NULL;
 
-    org_sflphone_SFLphone_VideoControls_get_device_list(video_proxy, &array, &error);
+    org_sflphone_SFLphone_VideoManager_get_device_list(video_proxy, &array, &error);
     check_error(error);
     return array;
 }
 
-/**
- * Get the list of channels supported by the given device
- */
-gchar **
-dbus_get_video_device_channel_list(const gchar *dev)
+GHashTable *
+dbus_video_get_capabilities(const gchar *name)
 {
-    gchar **array = NULL;
     GError *error = NULL;
-    org_sflphone_SFLphone_VideoControls_get_device_channel_list(video_proxy, dev, &array, &error);
+    GHashTable *cap = NULL;
+
+    org_sflphone_SFLphone_VideoManager_get_capabilities(video_proxy, name, &cap, &error);
     check_error(error);
-    return array;
-}
 
-/**
- * Get the list of resolutions supported by the given channel of the given device
- */
-gchar **
-dbus_get_video_device_size_list(const gchar *dev, const gchar *channel)
-{
-    gchar **array = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_VideoControls_get_device_size_list(video_proxy, dev, channel, &array, &error);
-    check_error(error);
-    return array;
-}
-
-/**
- * Get the list of frame rates supported by the given resolution of the given channel of the given device
- */
-gchar **
-dbus_get_video_device_rate_list(const gchar *dev, const gchar *channel, const gchar *size)
-{
-    gchar **array = NULL;
-    GError *error = NULL;
-
-    org_sflphone_SFLphone_VideoControls_get_device_rate_list(video_proxy, dev, channel, size, &array, &error);
-    check_error(error);
-    return array;
+    return cap;
 }
 #endif
 
@@ -2039,13 +2022,24 @@ dbus_get_conference_list(void)
     return list;
 }
 
+gchar **
+dbus_get_display_names(const gchar *confID)
+{
+    GError *error = NULL;
+    gchar **list = NULL;
+
+    org_sflphone_SFLphone_CallManager_get_display_names(call_proxy, confID, &list, &error);
+    check_error(error);
+
+    return list;
+}
+
 gchar **
 dbus_get_participant_list(const gchar *confID)
 {
     GError *error = NULL;
     gchar **list = NULL;
 
-    g_debug("Get conference %s participant list", confID);
     org_sflphone_SFLphone_CallManager_get_participant_list(call_proxy, confID, &list, &error);
     check_error(error);
 
@@ -2150,6 +2144,28 @@ dbus_get_tls_settings_default(void)
     return results;
 }
 
+gboolean
+dbus_check_certificate(const gchar *capath, const gchar *certpath)
+{
+    GError *error = NULL;
+    gboolean result;
+    org_sflphone_SFLphone_ConfigurationManager_check_certificate_validity(config_proxy, capath, certpath, &result, &error);
+    check_error(error);
+
+    return result;
+}
+
+gboolean
+dbus_certificate_contains_private_key(const gchar *filepath)
+{
+    GError *error = NULL;
+    gboolean result;
+    org_sflphone_SFLphone_ConfigurationManager_check_for_private_key(config_proxy, filepath, &result, &error);
+    check_error(error);
+
+    return result;
+}
+
 gchar *
 dbus_get_address_from_interface_name(const gchar *interface)
 {
@@ -2226,14 +2242,14 @@ void
 dbus_start_video_camera()
 {
     set_camera_button_sensitivity(FALSE);
-    org_sflphone_SFLphone_VideoControls_start_camera_async(video_proxy, video_camera_async_cb, NULL);
+    org_sflphone_SFLphone_VideoManager_start_camera_async(video_proxy, video_camera_async_cb, NULL);
 }
 
 void
 dbus_stop_video_camera()
 {
     set_camera_button_sensitivity(FALSE);
-    org_sflphone_SFLphone_VideoControls_stop_camera_async(video_proxy, video_camera_async_cb, NULL);
+    org_sflphone_SFLphone_VideoManager_stop_camera_async(video_proxy, video_camera_async_cb, NULL);
 }
 
 gboolean
@@ -2241,7 +2257,7 @@ dbus_has_video_camera_started()
 {
     GError *error = NULL;
     gboolean started = FALSE;
-    org_sflphone_SFLphone_VideoControls_has_camera_started(video_proxy, &started, &error);
+    org_sflphone_SFLphone_VideoManager_has_camera_started(video_proxy, &started, &error);
     check_error(error);
     return started;
 }
@@ -2319,12 +2335,10 @@ dbus_screensaver_uninhibit(void)
 {
     if (cookie == 0)
         return;
-    g_debug("uninhibit");
 
     GVariant *parameters = g_variant_new("(u)", cookie);
     if (parameters == NULL) {
-        g_warning("Could not create session manager uninhibit "
-               "parameters");
+        g_warning("Could not create session manager uninhibit parameters");
         return;
     }
 
@@ -2339,7 +2353,6 @@ void
 dbus_presence_publish(const gchar *accountID, gboolean status)
 {
     GError *error = NULL;
-    g_debug("DBus: publish presence status.");
     org_sflphone_SFLphone_PresenceManager_publish(presence_proxy, accountID,status, "Tout va bien.", NULL);
     check_error(error);
 }
@@ -2347,7 +2360,6 @@ dbus_presence_publish(const gchar *accountID, gboolean status)
 void
 dbus_presence_subscribe(const gchar *accountID, const gchar *uri, gboolean flag)
 {
-    g_debug("DBus: subscrbe presence status %s:%s.", uri, flag? "true" : "false");
     GError *error = NULL;
     org_sflphone_SFLphone_PresenceManager_subscribe_buddy(presence_proxy, accountID, uri, flag, NULL);
     check_error(error);
