@@ -153,7 +153,7 @@ void dbus_get_credentials(account_t *a);
 /**
  * ConfigurationManager - Get the details for the ip2ip profile
  */
-GHashTable *dbus_get_ip2_ip_details(void);
+GHashTable *dbus_get_ip2ip_details(void);
 
 /**
  * ConfigurationManager - Send registration request
@@ -258,16 +258,11 @@ void
 dbus_set_video_codecs(const gchar *id, const GPtrArray *list);
 
 /**
- * CallManager - return the video codec name
- * @param callable_obj_t* current call
+ * ConfigurationManager - Switch the video input
+ * @param resource A media resource locator (MRL) to switch to
  */
-gchar *dbus_get_current_video_codec_name(const callable_obj_t *c);
-
-/**
- * CallManager - return the audio codec name
- * @param callable_obj_t* current call
- */
-gchar *dbus_get_current_audio_codec_name(const callable_obj_t *c);
+gboolean
+dbus_switch_video_input(const gchar *resource);
 
 /**
  * ConfigurationManager - Get the list of available output audio plugins
@@ -313,11 +308,18 @@ void dbus_set_audio_input_device(int index);
 gchar **dbus_get_current_audio_devices_index();
 
 /**
- * ConfigurationManager - Get the index of the specified audio device
+ * ConfigurationManager - Get the index of the specified output audio device
  * @param name The string description of the audio device
  * @return int The index of the device
  */
-int dbus_get_audio_device_index(const gchar *name);
+int dbus_get_audio_output_device_index(const gchar *name);
+
+/**
+ * ConfigurationManager - Get the index of the specified input audio device
+ * @param name The string description of the audio device
+ * @return int The index of the device
+ */
+int dbus_get_audio_input_device_index(const gchar *name);
 
 /**
  * ConfigurationManager - Get the current output audio plugin
@@ -377,25 +379,21 @@ void dbus_set_history_limit(guint days);
  */
 gchar *dbus_get_audio_manager(void);
 
+gchar **dbus_get_supported_audio_managers(void);
+
 /**
  * ConfigurationManager - Set the audio manager
  * @param api	"alsa"
  *		"pulseaudio"
  */
-void dbus_set_audio_manager(const gchar *api);
+gboolean dbus_set_audio_manager(const gchar *api);
 
-void dbus_set_active_video_device(const gchar *dev);
-void dbus_set_active_video_device_channel(const gchar *channel);
-void dbus_set_active_video_device_size(const gchar *size);
-void dbus_set_active_video_device_rate(const gchar *rate);
-gchar *dbus_get_active_video_device();
-gchar *dbus_get_active_video_device_channel();
-gchar *dbus_get_active_video_device_size();
-gchar *dbus_get_active_video_device_rate();
-gchar **dbus_get_video_device_list();
-gchar **dbus_get_video_device_channel_list(const gchar *dev);
-gchar **dbus_get_video_device_size_list(const gchar *dev, const gchar *channel);
-gchar **dbus_get_video_device_rate_list(const gchar *dev, const gchar *channel, const gchar *size);
+gchar *dbus_video_get_default_device();
+void dbus_video_set_default_device(const gchar *name);
+GHashTable *dbus_video_get_settings(const gchar *name);
+void dbus_video_apply_settings(const gchar *name, GHashTable *settings);
+gchar **dbus_video_get_device_list();
+GHashTable *dbus_video_get_capabilities(const gchar *name);
 
 /**
  * ConfigurationManager - Start a tone when a new call is open and no numbers have been dialed
@@ -431,14 +429,21 @@ gint dbus_get_sip_address(void);
  */
 void dbus_add_participant(const gchar *callID, const gchar *confID);
 
+/**
+ * Return a list of display names for this conference (confID)
+ */
+gchar **
+dbus_get_display_names(const gchar *confID);
+
 /**
  * Return a list of participant for this conference (confID)
  */
 gchar **dbus_get_participant_list(const gchar *confID);
 
 /**
- * If thsi call participate to a conference, return the conference id
- * Return an empty string elsewhere
+ * If this call is part of a conference, return the conference id,
+ * otherwise return an empty string.
+ * Result must be freed by caller.
  */
 gchar *dbus_get_conference_id(const gchar *callID);
 
@@ -506,10 +511,14 @@ gboolean dbus_get_is_recording(const callable_obj_t *);
 
 GHashTable *dbus_get_call_details(const gchar *callID);
 
+/* Returns a newly allocated list callIDs.
+ * Caller is responsible for freeing it with g_strfreev */
 gchar **dbus_get_call_list(void);
 
 GHashTable* dbus_get_conference_details(const gchar *confID);
 
+/* Returns a newly allocated list conferenceIDs.
+ * Caller is responsible for freeing it with g_strfreev */
 gchar **dbus_get_conference_list(void);
 
 void dbus_set_accounts_order(const gchar *order);
@@ -559,6 +568,10 @@ void dbus_set_confirm_go_clear(const callable_obj_t *c);
  */
 gchar **dbus_get_supported_tls_method();
 
+gboolean dbus_certificate_contains_private_key(const gchar *filepath);
+gboolean dbus_check_certificate(const gchar *capath,
+                                const gchar *certpath);
+
 GHashTable* dbus_get_shortcuts(void);
 
 void dbus_set_shortcuts(GHashTable *shortcuts);
