@@ -31,22 +31,30 @@
 #ifndef ICON_FACTORY_H
 #define ICON_FACTORY_H
 
-#define GTK_STOCK_PICKUP				"gnome-stock-pickup"
-#define GTK_STOCK_HANGUP				"gnome-stock-hangup"
-#define GTK_STOCK_ONHOLD				"gnome-stock-onhold"
-#define GTK_STOCK_OFFHOLD				"gnome-stock-offhold"
-#define GTK_STOCK_IM					"gnome-stock-im"
+#define GTK_STOCK_PICKUP                "gnome-stock-pickup"
+#define GTK_STOCK_HANGUP                "gnome-stock-hangup"
+#define GTK_STOCK_ONHOLD                "gnome-stock-onhold"
+#define GTK_STOCK_OFFHOLD               "gnome-stock-offhold"
+#define GTK_STOCK_IM                    "gnome-stock-im"
 #define GTK_STOCK_TRANSFER              "gnome-stock-transfer"
 #define GTK_STOCK_DIAL                  "gnome-stock-dial"
 #define GTK_STOCK_CALL_CURRENT          "gnome-stock-call-current"
 #define GTK_STOCK_ADDRESSBOOK           "gnome-stock-addressbook"
-#define GTK_STOCK_CALLS					"gnome-stock-calls"
-#define GTK_STOCK_SFLPHONE				"gnome-stock-sflphone"
-#define GTK_STOCK_FAIL					"gnome-stock-fail"
-#define GTK_STOCK_USER					"gnome-stock-user"
-#define GTK_STOCK_AUDIO_CARD            "audio-card"
-
-#define GTK_STOCK_AUDIO_CARD			"audio-card"
+#define GTK_STOCK_CALLS                 "gnome-stock-calls"
+#define GTK_STOCK_SFLPHONE              "gnome-stock-sflphone"
+#define GTK_STOCK_FAIL                  "gnome-stock-fail"
+#define GTK_STOCK_USER                  "gnome-stock-user"
+#define GTK_STOCK_SCREENSHARING         "gnome-stock-screensharing"
+#define GTK_STOCK_RECORD                "gnome-stock-record"
+#define GTK_STOCK_HISTORY               "gnome-stock-history"
+#define GTK_STOCK_VOICEMAIL             "gnome-stock-voicemail"
+#define GTK_STOCK_NEWVOICEMAIL          "gnome-stock-newvoicemail"
+#define GTK_STOCK_PREFS                 "gnome-stock-prefs"
+#define GTK_STOCK_PREFS_AUDIO           "gnome-stock-prefs-audio"
+#define GTK_STOCK_PREFS_VIDEO           "gnome-stock-prefs-video"
+#define GTK_STOCK_PREFS_SHORTCUT        "gnome-stock-prefs-shortcut"
+#define GTK_STOCK_PREFS_HOOK            "gnome-stock-prefs-hook"
+#define GTK_STOCK_PREFS_ADDRESSBOOK     "gnome-stock-prefs-addressbook"
 
 void init_icon_factory();
 
