@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -83,9 +83,6 @@ main(int argc, char *argv[])
     signal(SIGHUP, signal_handler);
     signal(SIGTERM, signal_handler);
 
-    /* Must be called prior to any GLib calls (i.e. in update_schema_dir) */
-    g_type_init();
-
     /* Tell glib to look for our schema in gnome/data in case SFLphone is not
      * installed. We have to do this early for it to work properly for older
      * versions of GLib. */
