@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -81,6 +81,8 @@ void main_window_volume_controls (gboolean state);
  */
 void statusbar_push_message (const gchar * const left_hand_message, const gchar * const right_hand_message, guint id);
 
+void statusbar_enable_presence();
+
 /**
  * Pop a message from the statusbar stack
  * @param id  The identifier of the message
@@ -107,7 +109,7 @@ void focus_on_searchbar_in();
  * if the size is 0 or if the current value is larger than the size, the cursor position
  * is moved at the end of the scale.
  */
-void main_window_update_playback_scale(guint current, guint size);
+gboolean main_window_update_playback_scale(const gchar *file_path, guint current, guint size);
 
 void main_window_set_playback_scale_sensitive();
 
@@ -131,4 +133,7 @@ void main_window_pause_keygrabber(gboolean value);
 void main_window_reset_playback_scale();
 
 void main_window_bring_to_front(SFLPhoneClient *client, guint32 timestamp);
+
+void main_window_update_seekslider(const gchar *recordfile);
+
 #endif
