@@ -474,7 +474,7 @@ static gboolean
 field_error_dialog(const gchar *error_string)
 {
     gchar *msg;
-    msg = g_markup_printf_escaped("Field error %s.",error_string);// TODO: use _()
+    msg = g_markup_printf_escaped(_("Field error %s."), error_string);
 
     /* Create the widgets */
     GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(presence_window),
@@ -517,7 +517,7 @@ show_buddy_info_dialog(const gchar *title, buddy_t *b)
     // TODO: this is ugly but temporary
     guint group_index = 0;
     guint group_count = 0;
-    GtkWidget * combo_group;
+    GtkWidget * combo_group = NULL;
     if (g_strcmp0(title, "Add new buddy") != 0)
     {
         label = gtk_label_new_with_mnemonic("_Group");
@@ -654,7 +654,7 @@ static gboolean
 confirm_buddy_deletion(buddy_t *b)
 {
     gchar *msg;
-    msg = g_markup_printf_escaped("Are you sure want to delete \"%s\"", b->alias);
+    msg = g_markup_printf_escaped(_("Are you sure want to delete \"%s\""), b->alias);
 
     /* Create the widgets */
     GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(presence_window),
@@ -678,7 +678,7 @@ static gboolean
 confirm_group_deletion(gchar *group)
 {
     gchar *msg;
-    msg = g_markup_printf_escaped("Do you really want to delete the group %s", group);
+    msg = g_markup_printf_escaped(_("Do you really want to delete the group %s"), group);
 
     /* Create the widgets */
     GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(presence_window),
@@ -711,7 +711,7 @@ presence_view_popup_menu_onCallBuddy(G_GNUC_UNUSED GtkWidget *menuitem, gpointer
     g_free(c->_accountID);
     c->_accountID = g_strdup(b->acc);
 
-    calltree_update_call(current_calls_tab, c, presence_client, FALSE);
+    calltree_update_call(current_calls_tab, c, presence_client);
     sflphone_place_call(c, presence_client);
 
     /* Legacy system tray option, requires TopIcons GNOME extension */
