@@ -137,10 +137,10 @@ sfl_seekslider_init(SFLSeekSlider *seekslider)
     seekslider->priv->hscale = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, adjustment);
     seekslider->priv->hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
 
-    seekslider->priv->playRecordImage = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON);
+    seekslider->priv->playRecordImage = gtk_image_new_from_icon_name("media-playback-start-symbolic", GTK_ICON_SIZE_BUTTON);
     seekslider->priv->playRecordWidget = gtk_button_new();
     gtk_button_set_image(GTK_BUTTON(seekslider->priv->playRecordWidget), seekslider->priv->playRecordImage);
-    seekslider->priv->stopRecordImage = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_BUTTON);
+    seekslider->priv->stopRecordImage = gtk_image_new_from_icon_name("media-playback-pause-symbolic", GTK_ICON_SIZE_BUTTON);
 
     seekslider->priv->stopRecordWidget = gtk_button_new();
     gtk_button_set_image(GTK_BUTTON(seekslider->priv->stopRecordWidget), seekslider->priv->stopRecordImage);
@@ -197,6 +197,19 @@ sfl_seekslider_init(SFLSeekSlider *seekslider)
     seekslider->priv->file_path = NULL;
 }
 
+
+static void
+ensure_stop(SFLSeekSliderPrivate *priv)
+{
+    g_return_if_fail(priv && priv->file_path && strlen(priv->file_path) != 0);
+
+    if (priv->is_playing) {
+        dbus_stop_recorded_file_playback(priv->file_path);
+        g_debug("Stop file playback %s", priv->file_path);
+        priv->is_playing = FALSE;
+    }
+}
+
 static void
 sfl_seekslider_finalize(GObject *object)
 {
@@ -206,10 +219,9 @@ sfl_seekslider_finalize(GObject *object)
     g_return_if_fail(SFL_IS_SEEKSLIDER(object));
 
     seekslider = SFL_SEEKSLIDER(object);
-    g_return_if_fail(seekslider->priv != NULL);
 
     /* Ensure that we've stopped playback */
-    sfl_seekslider_stop_playback_record_cb(NULL, seekslider);
+    ensure_stop(seekslider->priv);
 
     G_OBJECT_CLASS(sfl_seekslider_parent_class)->finalize(object);
 }
@@ -365,14 +377,7 @@ static void sfl_seekslider_stop_playback_record_cb(GtkButton *button G_GNUC_UNUS
 {
     SFLSeekSlider *self = SFL_SEEKSLIDER(user_data);
 
-    if (self->priv->file_path == NULL || (*self->priv->file_path == 0))
-        return;
-
-    if (self->priv->is_playing) {
-        dbus_stop_recorded_file_playback(self->priv->file_path);
-        g_debug("Stop file playback %s", self->priv->file_path);
-        self->priv->is_playing = FALSE;
-    }
+    ensure_stop(self->priv);
 
     sfl_seekslider_set_display(self, SFL_SEEKSLIDER_DISPLAY_PLAY);
 }
