@@ -29,6 +29,10 @@
  *  as that of the covered work.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <glib/gi18n.h>
 #include "sflphone_client.h"
 #include "sflphone_options.h"
@@ -36,6 +40,9 @@
 #include "statusicon.h"
 #include "shortcuts.h"
 #include "mainwindow.h"
+#ifdef SFL_VIDEO
+#include "video/video_widget.h"
+#endif
 
 G_DEFINE_TYPE(SFLPhoneClient, sflphone_client, GTK_TYPE_APPLICATION);
 
@@ -59,12 +66,17 @@ sflphone_client_command_line_handler(G_GNUC_UNUSED GApplication *application,
 
     g_option_context_free(context);
 
+    /* Override theme since we don't have appropriate icons for a dark them (yet) */
+	GtkSettings *gtk_settings = gtk_settings_get_default();
+	g_object_set(G_OBJECT(gtk_settings), "gtk-application-prefer-dark-theme",
+                 FALSE, NULL);
+
     if (!sflphone_init(&error, client)) {
         g_warning("%s", error->message);
         GtkWidget *dialog = gtk_message_dialog_new(NULL,
                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                "Unable to initialize.\nMake sure the daemon is running.\nError: %s",
+                                _("Unable to initialize.\nMake sure the daemon is running.\nError: %s"),
                                 error->message);
 
         gtk_window_set_title(GTK_WINDOW(dialog), _("SFLphone Error"));
@@ -102,7 +114,7 @@ sflphone_client_new()
 {
     SFLPhoneClient *client = g_object_new(sflphone_client_get_type(),
             "application-id", "org.sfl.SFLphone",
-            "flags", G_APPLICATION_HANDLES_COMMAND_LINE | G_APPLICATION_IS_SERVICE, NULL);
+            "flags", G_APPLICATION_HANDLES_COMMAND_LINE, NULL);
     return client;
 }
 
@@ -110,6 +122,11 @@ static void
 sflphone_client_init(SFLPhoneClient *self)
 {
     self->settings = g_settings_new(SFLPHONE_GSETTINGS_SCHEMA);
+
+#ifdef SFL_VIDEO
+    self->video = video_widget_new();
+#endif
+
     self->win = 0;
     g_signal_connect(G_OBJECT(self), "command-line", G_CALLBACK(sflphone_client_command_line_handler), self);
 }
