@@ -33,6 +33,7 @@
 #endif
 
 #include "str_utils.h"
+#include "actions.h"
 #include "preferencesdialog.h"
 #include "icons/icon_factory.h"
 #include "dbus/dbus.h"
@@ -88,6 +89,9 @@ static GtkAction * copyAction_;
 static GtkAction * pasteAction_;
 static GtkAction * recordAction_;
 static GtkAction * imAction_;
+#ifdef SFL_VIDEO
+static GtkAction * screenshareAction_;
+#endif
 
 static GtkWidget * pickUpWidget_;
 static GtkWidget * newCallWidget_;
@@ -99,6 +103,7 @@ static GtkWidget * transferToolbar_;
 static GtkWidget * recordWidget_;
 static GtkWidget * voicemailToolbar_;
 static GtkWidget * imToolbar_;
+static GtkWidget * screenshareToolbar_;
 
 static GtkWidget * editable_num_;
 static GtkWidget * edit_dialog_;
@@ -129,6 +134,22 @@ static void add_to_toolbar(GtkWidget *toolbar, GtkWidget *item, gint pos)
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(item), pos);
 }
 
+#ifdef SFL_VIDEO
+static gboolean
+is_video_call(callable_obj_t *call)
+{
+    gboolean video_enabled = FALSE;
+    account_t *account = account_list_get_by_id(call->_accountID);
+
+    if (account) {
+        gchar *ptr = (gchar *) account_lookup(account, CONFIG_VIDEO_ENABLED);
+        video_enabled = g_strcmp0(ptr, "true") == 0;
+    }
+
+    return video_enabled;
+}
+#endif
+
 static void
 update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging_enabled, SFLPhoneClient *client)
 {
@@ -221,6 +242,10 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
         }
         case CALL_STATE_CURRENT:
         {
+#ifdef SFL_VIDEO
+            const gboolean video_enabled = is_video_call(selectedCall);
+#endif
+
                 g_debug("Call State Current");
                 g_signal_handler_block(transferToolbar_, transferButtonConnId_);
                 g_signal_handler_block(recordWidget_, recordButtonConnId_);
@@ -232,6 +257,10 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
                 gtk_widget_set_sensitive(transferToolbar_, TRUE);
                 if (instant_messaging_enabled)
                     gtk_action_set_sensitive(imAction_, TRUE);
+#ifdef SFL_VIDEO
+                if (video_enabled)
+                    gtk_action_set_sensitive(screenshareAction_, TRUE);
+#endif
 
                 pos = 1;
                 add_to_toolbar(toolbar, hangUpWidget_, pos++);
@@ -240,6 +269,10 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
                 add_to_toolbar(toolbar, recordWidget_, pos++);
                 if (instant_messaging_enabled)
                     add_to_toolbar(toolbar, imToolbar_, pos++);
+#ifdef SFL_VIDEO
+                if (video_enabled)
+                    add_to_toolbar(toolbar, screenshareToolbar_, pos++);
+#endif
 
                 gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(transferToolbar_), FALSE);
                 gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(recordWidget_), dbus_get_is_recording(selectedCall));
@@ -376,6 +409,9 @@ update_actions(SFLPhoneClient *client)
     gtk_action_set_sensitive(recordAction_, FALSE);
     gtk_action_set_sensitive(copyAction_, FALSE);
     gtk_action_set_sensitive(imAction_, FALSE);
+#ifdef SFL_VIDEO
+    gtk_action_set_sensitive(screenshareAction_, FALSE);
+#endif
 
     gtk_widget_set_sensitive(holdMenu_, FALSE);
     gtk_widget_set_sensitive(holdToolbar_, FALSE);
@@ -392,6 +428,9 @@ update_actions(SFLPhoneClient *client)
     g_object_ref(transferToolbar_);
     g_object_ref(voicemailToolbar_);
     g_object_ref(imToolbar_);
+#ifdef SFL_VIDEO
+    g_object_ref(screenshareToolbar_);
+#endif
 
     if (addrbook)
         g_object_ref(contactButton_);
@@ -406,6 +445,7 @@ update_actions(SFLPhoneClient *client)
     remove_from_toolbar(toolbar, historyButton_);
     remove_from_toolbar(toolbar, voicemailToolbar_);
     remove_from_toolbar(toolbar, imToolbar_);
+    remove_from_toolbar(toolbar, screenshareToolbar_);
     remove_from_toolbar(toolbar, holdToolbar_);
     remove_from_toolbar(toolbar, offHoldToolbar_);
     remove_from_toolbar(toolbar, newCallWidget_);
@@ -470,11 +510,11 @@ update_voicemail_status()
                       current_account_get_message_number());
 
     if (current_account_has_new_message())
-        gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(voicemailToolbar_),
-                                      "mail-message-new");
+        gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(voicemailToolbar_),
+                                      GTK_STOCK_NEWVOICEMAIL);
     else
-        gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(voicemailToolbar_),
-                                      "mail-read");
+        gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(voicemailToolbar_),
+                                      GTK_STOCK_VOICEMAIL);
 
     gtk_tool_button_set_label(GTK_TOOL_BUTTON(voicemailToolbar_), messages);
     g_free(messages);
@@ -661,6 +701,55 @@ call_im(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
     }
 }
 
+static void
+call_screenshare(G_GNUC_UNUSED GtkAction *action, G_GNUC_UNUSED SFLPhoneClient *client)
+{
+#ifdef SFL_VIDEO
+    sflphone_toggle_screenshare();
+#endif
+}
+
+static gchar *
+choose_file(void)
+{
+    gchar *uri = NULL;
+    GtkWidget *dialog = gtk_file_chooser_dialog_new(NULL, NULL,
+            GTK_FILE_CHOOSER_ACTION_OPEN,
+            _("_Cancel"), GTK_RESPONSE_CANCEL,
+            _("_Open"), GTK_RESPONSE_ACCEPT,
+            NULL);
+
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
+        GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
+        uri = gtk_file_chooser_get_uri(chooser);
+    }
+
+    gtk_widget_destroy(dialog);
+    return uri;
+}
+
+#ifdef SFL_VIDEO
+static void
+call_switch_video_input(G_GNUC_UNUSED GtkWidget *widget, gchar *device)
+{
+    gchar *resource;
+
+    if (g_strcmp0(device, _("Screen")) == 0) {
+        resource = sflphone_get_display();
+    } else if (g_strcmp0(device, _("Choose file...")) == 0) {
+        resource = choose_file();
+        if (!resource)
+            return;
+    } else {
+        dbus_video_set_default_device(device);
+        resource = g_strconcat("v4l2://", device, NULL);
+    }
+
+    sflphone_switch_video_input(resource);
+    g_free(resource);
+}
+#endif
+
 static void
 conference_hold(G_GNUC_UNUSED gpointer foo)
 {
@@ -846,7 +935,7 @@ edit_paste(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
                     selectedCall->_peer_info = g_strconcat("\"\" <",
                                                            selectedCall->_peer_number, ">", NULL);
 
-                calltree_update_call(current_calls_tab, selectedCall, client, FALSE);
+                calltree_update_call(current_calls_tab, selectedCall, client);
             }
             break;
             case CALL_STATE_RINGING:
@@ -865,7 +954,7 @@ edit_paste(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
                 selectedCall->_peer_info = g_strconcat("\"\" <",
                                                        selectedCall->_peer_number, ">", NULL);
 
-                calltree_update_call(current_calls_tab, selectedCall, client, FALSE);
+                calltree_update_call(current_calls_tab, selectedCall, client);
             }
             break;
             case CALL_STATE_CURRENT:
@@ -881,7 +970,7 @@ edit_paste(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
                     selectedCall->_peer_info = get_peer_info(temp, selectedCall->_display_name);
                     g_free(temp);
                     g_free(oneNo);
-                    calltree_update_call(current_calls_tab, selectedCall, client, TRUE);
+                    calltree_update_call(current_calls_tab, selectedCall, client);
                 }
             }
             break;
@@ -897,7 +986,7 @@ edit_paste(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
         g_free(selectedCall->_peer_info);
         selectedCall->_peer_info = g_strconcat("\"\" <",
                                                selectedCall->_peer_number, ">", NULL);
-        calltree_update_call(current_calls_tab, selectedCall, client, FALSE);
+        calltree_update_call(current_calls_tab, selectedCall, client);
     }
 
     g_free(no);
@@ -1009,6 +1098,10 @@ static const GtkActionEntry menu_entries[] = {
         "InstantMessaging", GTK_STOCK_IM, N_("Send _message"), "<control>M",
         N_("Send message"), G_CALLBACK(call_im)
     },
+    {
+        "ScreenSharing", GTK_STOCK_SCREENSHARING, N_("Share screen"), "<control>X",
+        N_("Share screen"), G_CALLBACK(call_screenshare)
+    },
     {
         "AccountAssistant", NULL, N_("Configuration _Assistant"), NULL,
         N_("Run the configuration assistant"), G_CALLBACK(call_configuration_assistant)
@@ -1062,11 +1155,11 @@ static const GtkActionEntry menu_entries[] = {
 
 static const GtkToggleActionEntry toggle_menu_entries[] = {
     { "Transfer", GTK_STOCK_TRANSFER, N_("_Transfer"), "<control>T", N_("Transfer the call"), NULL, TRUE },
-    { "Record", GTK_STOCK_MEDIA_RECORD, N_("_Record"), "<control>R", N_("Record the current conversation"), NULL, TRUE },
+    { "Record", GTK_STOCK_RECORD, N_("_Record"), "<control>R", N_("Record the current conversation"), NULL, TRUE },
     { "Toolbar", NULL, N_("_Show toolbar"), "<control>T", N_("Show the toolbar"), NULL, TRUE },
     { "Dialpad", NULL, N_("_Dialpad"), "<control>D", N_("Show the dialpad"), G_CALLBACK(dialpad_bar_cb), TRUE },
     { "VolumeControls", NULL, N_("_Volume controls"), "<control>V", N_("Show the volume controls"), G_CALLBACK(volume_bar_cb), TRUE },
-    { "History", "appointment-soon", N_("_History"), NULL, N_("Calls history"), G_CALLBACK(toggle_history_cb), FALSE },
+    { "History", GTK_STOCK_HISTORY, N_("_History"), NULL, N_("Call history"), G_CALLBACK(toggle_history_cb), FALSE },
     { "Addressbook", GTK_STOCK_ADDRESSBOOK, N_("_Address book"), NULL, N_("Address book"), G_CALLBACK(toggle_addressbook_cb), FALSE },
 #ifdef SFL_PRESENCE
     { "Buddies", NULL, N_("_Buddy list"), NULL, N_("Display the buddy list"), G_CALLBACK(toggle_presence_window_cb), FALSE},
@@ -1208,11 +1301,26 @@ static void menu_popup_wrapper(GtkWidget *menu, GtkWidget *my_widget, GdkEventBu
                        gtk_get_current_event_time());
 }
 
+#ifdef SFL_VIDEO
+static void
+append_video_input_to_submenu(GtkWidget *submenu, const gchar *device)
+{
+    GtkWidget *item = gtk_image_menu_item_new_with_mnemonic(_(device));
+    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item);
+    g_signal_connect(G_OBJECT(item), "activate",
+            G_CALLBACK(call_switch_video_input), (gpointer) device);
+    gtk_widget_show(item);
+}
+#endif
+
 void
 show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *client)
 {
     // TODO update the selection to make sure the call under the mouse is the call selected
     gboolean pickup = FALSE, hangup = FALSE, hold = FALSE, copy = FALSE, record = FALSE, im = FALSE;
+#ifdef SFL_VIDEO
+    gboolean video_sources = FALSE;
+#endif
     gboolean accounts = FALSE;
 
     // conference type boolean
@@ -1250,6 +1358,9 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
                     hold = TRUE;
                     record = TRUE;
                     im = TRUE;
+#ifdef SFL_VIDEO
+                    video_sources = is_video_call(selectedCall);
+#endif
                     break;
                 case CALL_STATE_BUSY:
                 case CALL_STATE_FAILURE:
@@ -1287,21 +1398,20 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
     }
 
     GtkWidget *menu = gtk_menu_new();
+    gtk_menu_set_accel_group(GTK_MENU(menu), get_accel_group());
 
     if (calltab_get_selected_type(current_calls_tab) == A_CALL) {
         g_debug("Build call menu");
         if (copy) {
-            GtkWidget *menu_items = gtk_image_menu_item_new_from_stock(GTK_STOCK_COPY,
-                                    get_accel_group());
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);
-            g_signal_connect(G_OBJECT(menu_items), "activate",
+            GtkWidget *copy_item = gtk_menu_item_new_with_mnemonic(_("_Copy"));
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), copy_item);
+            g_signal_connect(G_OBJECT(copy_item), "activate",
                              G_CALLBACK(edit_copy),
                              NULL);
-            gtk_widget_show(menu_items);
+            gtk_widget_show(copy_item);
         }
 
-        GtkWidget *paste = gtk_image_menu_item_new_from_stock(GTK_STOCK_PASTE,
-                           get_accel_group());
+        GtkWidget *paste = gtk_menu_item_new_with_mnemonic(_("_Paste"));
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), paste);
         g_signal_connect(G_OBJECT(paste), "activate", G_CALLBACK(edit_paste),
                          client);
@@ -1314,14 +1424,15 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
         }
 
         if (pickup) {
-            GtkWidget *menu_items = gtk_image_menu_item_new_with_mnemonic(_("_Pick up"));
+            GtkWidget *pickup_item = gtk_image_menu_item_new_with_mnemonic(_("_Pick up"));
             GtkWidget *image = gtk_image_new_from_file(ICONS_DIR "/icon_accept.svg");
-            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_items), image);
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);
-            g_signal_connect(G_OBJECT(menu_items), "activate",
+            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(pickup_item), image);
+            gtk_image_menu_item_set_always_show_image(GTK_IMAGE_MENU_ITEM(pickup_item), TRUE);
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), pickup_item);
+            g_signal_connect(G_OBJECT(pickup_item), "activate",
                              G_CALLBACK(call_pick_up),
                              client);
-            gtk_widget_show(menu_items);
+            gtk_widget_show(pickup_item);
         }
 
         if (hangup) {
@@ -1348,7 +1459,7 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
 
         if (record) {
             GtkWidget *menu_items = gtk_image_menu_item_new_with_mnemonic(_("_Record"));
-            GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_RECORD,
+            GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_RECORD,
                                GTK_ICON_SIZE_MENU);
             gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_items), image);
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);
@@ -1377,6 +1488,35 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
                 gtk_widget_show(menu_items);
             }
         }
+
+#ifdef SFL_VIDEO
+        if (video_sources) {
+            GtkWidget *video_sep, *video_item, *video_menu;
+
+            video_sep = gtk_separator_menu_item_new();
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), video_sep);
+            gtk_widget_show(video_sep);
+
+            /* Add a video sources menu */
+            video_item = gtk_image_menu_item_new_with_mnemonic(_("Video sources"));
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), video_item);
+            gtk_widget_show(video_item);
+
+            video_menu = gtk_menu_new();
+            gtk_menu_item_set_submenu(GTK_MENU_ITEM(video_item), video_menu);
+
+            /* Append sources to the submenu */
+            gchar **video_list = dbus_video_get_device_list();
+            while (*video_list) {
+                append_video_input_to_submenu(video_menu, *video_list);
+                //g_free(*video_list);
+                video_list++;
+            }
+            /* Add the special X11 device */
+            append_video_input_to_submenu(video_menu, _("Screen"));
+            append_video_input_to_submenu(video_menu, _("Choose file..."));
+        }
+#endif
     } else {
         g_debug("Build conf menus");
 
@@ -1639,6 +1779,9 @@ create_menus(GtkUIManager *ui, SFLPhoneClient *client)
     holdMenu_ = get_widget(ui, "/MenuBar/CallMenu/OnHoldMenu");
     recordAction_ = get_action(ui, "/MenuBar/CallMenu/Record");
     imAction_ = get_action(ui, "/MenuBar/CallMenu/InstantMessaging");
+#ifdef SFL_VIDEO
+    screenshareAction_ = get_action(ui, "/MenuBar/CallMenu/ScreenSharing");
+#endif
     copyAction_ = get_action(ui, "/MenuBar/EditMenu/Copy");
     pasteAction_ = get_action(ui, "/MenuBar/EditMenu/Paste");
     volumeToggle_ = get_action(ui, "/MenuBar/ViewMenu/VolumeControls");
@@ -1672,6 +1815,11 @@ create_toolbar_actions(GtkUIManager *ui, SFLPhoneClient *client)
     hangUpWidget_ = get_widget(ui, "/ToolbarActions/HangUpToolbar");
     recordWidget_ = get_widget(ui, "/ToolbarActions/RecordToolbar");
     imToolbar_ = get_widget(ui, "/ToolbarActions/InstantMessagingToolbar");
+    screenshareToolbar_ = get_widget(ui, "/ToolbarActions/ScreenSharingToolbar");
+/* Hide this widget if video support is disabled */
+#ifndef SFL_VIDEO
+    remove_from_toolbar(client->toolbar, screenshareToolbar_);
+#endif
     historyButton_ = get_widget(ui, "/ToolbarActions/HistoryToolbar");
     if (addrbook)
         contactButton_ = get_widget(ui, "/ToolbarActions/AddressbookToolbar");
