@@ -31,6 +31,8 @@
 #include <KShortcutsDialog>
 #include <KActionCollection>
 #include <KEditToolBar>
+#include <KGlobal>
+#include <KSharedConfig>
 
 //SFLPhone
 #include "extendedaction.h"
@@ -42,12 +44,12 @@
 #include "klib/kcfg_settings.h"
 #include "klib/helperfunctions.h"
 #include "klib/macromodel.h"
-#include "klib/akonadibackend.h"
 #include <lib/call.h>
 #include <lib/account.h>
 #include <lib/accountlistmodel.h>
 #include <lib/callmodel.h>
 #include <lib/audiosettingsmodel.h>
+#include <lib/contactmodel.h>
 
 
 ActionCollection* ActionCollection::m_spInstance = nullptr;
@@ -58,7 +60,12 @@ ActionCollection* ActionCollection::instance() {
    return m_spInstance;
 }
 
-ActionCollection::ActionCollection(QObject* parent) : QObject(parent),m_pWizard(nullptr)
+ActionCollection::ActionCollection(QObject* parent) : QObject(parent),m_pWizard(nullptr),
+action_mailBox(nullptr), action_close(nullptr), action_quit(nullptr), action_displayVolumeControls(nullptr),
+action_displayDialpad(nullptr), action_displayMessageBox(nullptr), action_configureSflPhone(nullptr),
+action_configureShortcut(nullptr), action_accountCreationWizard(nullptr), action_pastenumber(nullptr),
+action_showContactDock(nullptr), action_showHistoryDock(nullptr), action_showBookmarkDock(nullptr),
+action_editToolBar(nullptr), action_addContact(nullptr), action_screen(nullptr)
 {
    action_accept   = new ExtendedAction(this);
    action_refuse   = new ExtendedAction(this);
@@ -99,18 +106,39 @@ ActionCollection::ActionCollection(QObject* parent) : QObject(parent),m_pWizard(
    action_video_flip_vertical   = new ExtendedAction(this);
    action_video_mute            = new ExtendedAction(this);
    action_video_preview         = new ExtendedAction(this);
+   action_video_scale           = new ExtendedAction(this);
+   action_video_fullscreen      = new ExtendedAction(this);
    action_video_rotate_left     ->setText ( i18n( "Rotate left"  ) );
    action_video_rotate_right    ->setText ( i18n( "Rotate right" ) );
    action_video_flip_horizontal ->setText ( i18n( "Flip"         ) );
    action_video_flip_vertical   ->setText ( i18n( "Flip"         ) );
    action_video_mute            ->setText ( i18n( "Mute"         ) );
    action_video_preview         ->setText ( i18n( "Preview"      ) );
+   action_video_scale           ->setText ( i18n( "Keep aspect ratio"      ));
+   action_video_fullscreen      ->setText ( i18n( "Fullscreen"             ));
    action_video_rotate_left     ->setAltIcon(KIcon("object-rotate-left"    ));
    action_video_rotate_right    ->setAltIcon(KIcon("object-rotate-right"   ));
    action_video_flip_horizontal ->setAltIcon(KIcon("object-flip-horizontal"));
    action_video_flip_vertical   ->setAltIcon(KIcon("object-flip-vertical"  ));
    action_video_mute            ->setAltIcon(KIcon("camera-web"            ));
    action_video_preview         ->setAltIcon(KIcon("view-preview"          ));
+   action_video_scale           ->setAltIcon(KIcon("transform-scale"       ));
+   action_video_fullscreen      ->setAltIcon(KIcon("view-fullscreen"       ));
+   action_video_rotate_left     ->setIcon(KIcon("object-rotate-left"    ));
+   action_video_rotate_right    ->setIcon(KIcon("object-rotate-right"   ));
+   action_video_flip_horizontal ->setIcon(KIcon("object-flip-horizontal"));
+   action_video_flip_vertical   ->setIcon(KIcon("object-flip-vertical"  ));
+   action_video_mute            ->setIcon(KIcon("camera-web"            ));
+   action_video_preview         ->setIcon(KIcon("view-preview"          ));
+   action_video_scale           ->setIcon(KIcon("transform-scale"       ));
+   action_video_fullscreen      ->setIcon(KIcon("view-fullscreen"       ));
+   action_video_preview         ->setCheckable(true);
+   action_video_scale           ->setCheckable(true);
+   action_video_fullscreen      ->setCheckable(true);
+   action_video_mute            ->setCheckable(true);
+
+   action_video_scale           ->setChecked(ConfigurationSkeleton::keepVideoAspectRatio());
+   action_video_preview         ->setChecked(ConfigurationSkeleton::displayVideoPreview());
    #endif
 }
 
@@ -137,6 +165,9 @@ ActionCollection::~ActionCollection()
    delete action_editToolBar           ;
    delete action_mute_playback         ;
    delete action_mute_capture          ;
+#ifdef ENABLE_VIDEO
+   delete action_video_fullscreen      ;
+#endif
 }
 
 void ActionCollection::setupAction()
@@ -158,7 +189,7 @@ void ActionCollection::setupAction()
    action_quit  = KStandardAction::quit(SFLPhone::app(), SLOT(quitButton()), SFLPhone::app());
 
    action_configureSflPhone = KStandardAction::preferences(this, SLOT(configureSflPhone()), SFLPhone::app());
-   action_configureSflPhone->setText(i18n("Configure SFLphone"));
+   action_configureSflPhone->setText(i18n("Configure SFLPhone-KDE"));
 
    action_displayDialpad        = new KAction(KIcon(QIcon(ICON_DISPLAY_DIALPAD)), i18n("Display dialpad")                 , this);
    action_displayMessageBox     = new KAction(KIcon("mail-message-new"), i18n("Display text message box")                 , this);
@@ -228,6 +259,7 @@ void ActionCollection::setupAction()
    connect(AudioSettingsModel::instance(),SIGNAL(captureVolumeChanged(int)),this,SLOT(updateRecordButton()));
    connect(AudioSettingsModel::instance(),SIGNAL(playbackVolumeChanged(int)),this,SLOT(updateVolumeButton()));
 
+//    SFLPhone::app()->actionCollection()->setConfigGlobal(true);
    SFLPhone::app()->actionCollection()->addAction("action_accept"                , action_accept                );
    SFLPhone::app()->actionCollection()->addAction("action_refuse"                , action_refuse                );
    SFLPhone::app()->actionCollection()->addAction("action_hold"                  , action_hold                  );
@@ -259,9 +291,12 @@ void ActionCollection::setupAction()
    foreach(KAction* ac,acList) {
       SFLPhone::app()->actionCollection()->addAction(ac->objectName() , ac);
    }
+//    qDebug() << "\n\n\nGlobal" << KGlobal::config()->groupList();
+//    KConfigGroup g = KGlobal::config()->group("KShortcutsDialog Settings");
+//    SFLPhone::app()->actionCollection()->exportGlobalShortcuts(&g);
 
-   void updateRecordButton();
-   void updateVolumeButton();
+   updateRecordButton();
+   updateVolumeButton();
 }
 
 ///Call
@@ -273,9 +308,11 @@ void ActionCollection::accept() //TODO dead code?
       CallModel::instance()->dialingCall();
       SFLPhone::view()->selectDialingCall();
       SFLPhone::view()->updateWindowCallState();
+      SFLPhone::app()->selectCallTab();
    }
    else {
       const Call::State state = call->state();
+      //TODO port to lifeCycle code
       if (state == Call::State::RINGING || state == Call::State::CURRENT || state == Call::State::HOLD
          || state == Call::State::BUSY || state == Call::State::FAILURE || state == Call::State::ERROR) {
          kDebug() << "Calling when item currently ringing, current, hold or busy. Opening an item.";
@@ -571,7 +608,7 @@ KAction* ActionCollection::addContact()
 void ActionCollection::slotAddContact()
 {
    Contact* aContact = new Contact();
-   AkonadiBackend::instance()->addNewContact(aContact);
+   ContactModel::instance()->addNewContact(aContact);
 }
 
 ///Change icon of the record button
@@ -582,6 +619,7 @@ void ActionCollection::updateRecordButton()
       QIcon(":/images/icons/mic_50.svg"),QIcon(":/images/icons/mic_75.svg")};
    const int idx = (recVol/26 < 0 || recVol/26 >= 4)?0:recVol/26;
    ActionCollection::instance()->muteCaptureAction()->setIcon(icons[idx]);
+   SFLPhone::view()->updateVolumeControls();
 }
 
 ///Update the colunm button icon
@@ -592,6 +630,7 @@ void ActionCollection::updateVolumeButton()
       QIcon(":/images/icons/speaker_50.svg"),QIcon(":/images/icons/speaker_75.svg")};
    const int idx = (sndVol/26 < 0 || sndVol/26 >= 4)?0:sndVol/26;
    ActionCollection::instance()->mutePlaybackAction()->setIcon(icons[idx]);
+   SFLPhone::view()->updateVolumeControls();
 }
 
 //Video actions
@@ -625,4 +664,14 @@ ExtendedAction* ActionCollection::videoPreviewAction        ()
 {
    return action_video_preview;
 }
+
+ExtendedAction* ActionCollection::videoScaleAction          ()
+{
+   return action_video_scale;
+}
+
+ExtendedAction* ActionCollection::videoFullscreenAction     ()
+{
+   return action_video_fullscreen;
+}
 #endif
