@@ -38,7 +38,7 @@
 #include "sflphone.h"
 
 
-CanvasObjectManager::Object CanvasObjectManager::m_slEvents[EVENT_COUNT] = { CanvasObjectManager::Object::NoObject };
+CanvasObjectManager::Object CanvasObjectManager::m_slEvents[EVENT_COUNT+1] = { CanvasObjectManager::Object::NoObject };
 
 #define OBJ_DEF(obj) elements[static_cast<int>(obj)]
 #define _(NAME) CanvasObjectManager::Object##NAME::
@@ -57,7 +57,8 @@ const CanvasObjectManager::CanvasElement CanvasObjectManager::elements[ELEMENT_C
    /*5 AutoComplete */ {_(Type)WIDGET , _(LifeCycle)EVENT  , _(Priority)HIGH       , E::CALL_DIALING_CHANGED    , E::CALL_STATE_CHANGED
                                                                                                                  | E::CALL_ENDED
                                                                                                                  | E::CALL_RINGING
-                                                                                                                 | E::CALL_BUSY        , false , false  , true },
+                                                                                                                 | E::CALL_BUSY
+                                                                                                                 | E::USER_CANCEL      , false , false  , true },
    /*6 DropInfo     */ {_(Type)OBJECT , _(LifeCycle)EVENT  , _(Priority)MEDIUM     , E::DRAG_ENTER|E::DRAG_MOVE , E::DRAG_LEAVE|E::DROP, false , false  , false},
    /*7 ConfInfo     */ {_(Type)OBJECT , _(LifeCycle)EVENT  , _(Priority)LOW        , E::CALL_COUNT_CHANGED      , E::ANY               , false , false  , false},
    /*8 AccountDown  */ {_(Type)OBJECT , _(LifeCycle)EVENT  , _(Priority)HIGH       , E::UNREGISTERED_ACCOUNT    , E::ANY               , false , true   , false},
@@ -80,7 +81,7 @@ void CanvasObjectManager::hInitEvents() {
 
 CanvasObjectManager::CanvasObjectManager(QObject* parent) : QObject(parent),m_DisableTransition(false),
 m_CurrentObject(CanvasObjectManager::Object::NoObject),m_NextObject(CanvasObjectManager::Object::NoObject)
-,m_pTimer(nullptr),m_CurrentState(ObjectState::NO_OBJECT),m_Minimized(false)
+,m_pTimer(nullptr),m_CurrentState(ObjectState::NO_OBJECT),m_Minimized(false),m_CurrentLifeCycle(ObjectLifeCycle::NONE)
 {
    hInitEvents();
 
@@ -178,6 +179,7 @@ void CanvasObjectManager::initiateOutTransition(bool skipAnimation)
                case ObjectType::WIDGET: {
 //                   qDebug() << "OUT WIDGET";
                   QWidget* w = TipCollection::canvasWidgetsToTip(m_CurrentObject);
+//                   qDebug() << "\n\n\nFOO" << w;
                   if (w) {
                      w->setVisible(false);
                      m_CurrentObject = m_NextObject;
@@ -397,7 +399,7 @@ void CanvasObjectManager::slotMinimized(bool isMinimized)
 
 bool CanvasObjectManager::testEventToEvent() const
 {
-   /* This test is to validate the the event table is correctly generated
+   /* This test is to validate the event table is correctly generated
     * from the structured elements transition table ("elements"). Each
     * event can be mapped to ONE AND ONLY ONE canvas object
     *
