@@ -45,10 +45,13 @@
 #include "lib/sflphone_const.h"
 #include "lib/credentialmodel.h"
 #include "lib/audiocodecmodel.h"
+#include "lib/securityvalidationmodel.h"
 #include "lib/accountlistmodel.h"
 #include "lib/keyexchangemodel.h"
 #include "lib/ringtonemodel.h"
 #include "lib/tlsmethodmodel.h"
+#include "lib/certificate.h"
+#include "lib/securityvalidationmodel.h"
 #include "lib/categorizedaccountmodel.h"
 #include "../delegates/ringtonedelegate.h"
 #include "../delegates/categorizeddelegate.h"
@@ -66,7 +69,7 @@
 
 ///Constructor
 DlgAccounts::DlgAccounts(KConfigDialog* parent)
- : QWidget(parent),m_IsLoading(false)
+ : QWidget(parent),m_IsLoading(false),m_pPKII(nullptr)
 {
    m_IsLoading++;
    setupUi(this);
@@ -87,8 +90,20 @@ DlgAccounts::DlgAccounts(KConfigDialog* parent)
    treeView_accountList->setItemDelegate(m_pCategoryDelegate);
    treeView_accountList->expandAll();
 
+   //BEGIN Temporarely disable advanced security widgets
+   m_pSecurityIssues->setVisible(false);
+   frame->setVisible(false);
+   frame_2->setVisible(false);
+   //END
+
    m_pInfoIconL->setPixmap(KIcon("dialog-information").pixmap(QSize(32,32)));
    label_message_icon->setPixmap(KIcon("dialog-information").pixmap(QSize(24,24)));
+   m_pCancelFix->setIcon(KIcon("dialog-close"));
+   m_pCancelMove->setIcon(KIcon("dialog-close"));
+   m_pMoveCertPB->setIcon(KIcon("folder-sync"));
+   m_pFixCertPB->setIcon(KIcon("configure"));
+   label_3->setPixmap(KIcon("dialog-information").pixmap(QSize(22,22)));
+   label_2->setPixmap(KIcon("dialog-information").pixmap(QSize(22,22)));
 
    //Add an info tip in the account list
    m_pTipManager = new TipManager(treeView_accountList);
@@ -103,8 +118,33 @@ DlgAccounts::DlgAccounts(KConfigDialog* parent)
    file_tls_authority->lineEdit()->setPlaceholderText(i18n("Usually called \"ca.crt\" or \"cacert.pem\""));
    file_tls_endpoint->lineEdit()->setPlaceholderText(i18n("A .pem or .crt"));
    file_tls_private_key->lineEdit()->setPlaceholderText(i18n("A .key file"));
+   file_tls_authority->lineEdit()->setClearButtonShown(false);
+   file_tls_endpoint->lineEdit()->setClearButtonShown(false);
+   file_tls_private_key->lineEdit()->setClearButtonShown(false);
 
-   loadAccountList();
+#if false
+   //BEGIN Temporarely disable advanced security widgets
+   //Authority
+   m_pAuthorityII = new IssuesIcon(file_tls_authority->lineEdit());
+   m_pAuthorityII->setupForLineEdit(file_tls_authority->lineEdit());
+   connect(m_pAuthorityII,SIGNAL(selectFlaw(QModelIndex)),m_pSecurityIssues->view(),SLOT(setCurrentIndex(QModelIndex)));
+
+   //End point
+   m_pEndCertII = new IssuesIcon(file_tls_endpoint->lineEdit());
+   m_pEndCertII->setupForLineEdit(file_tls_endpoint->lineEdit());
+   connect(m_pEndCertII,SIGNAL(selectFlaw(QModelIndex)),m_pSecurityIssues->view(),SLOT(setCurrentIndex(QModelIndex)));
+
+   //Private key
+   m_pPKII = new IssuesIcon(file_tls_private_key->lineEdit());
+   m_pPKII->setupForLineEdit(file_tls_private_key->lineEdit());
+   connect(m_pPKII,SIGNAL(selectFlaw(QModelIndex)),m_pSecurityIssues->view(),SLOT(setCurrentIndex(QModelIndex)));
+
+   connect(m_pVerifyServer,SIGNAL(selectFlaw(QModelIndex)),m_pSecurityIssues->view(),SLOT(setCurrentIndex(QModelIndex)));
+   connect(m_pVerifyClient,SIGNAL(selectFlaw(QModelIndex)),m_pSecurityIssues->view(),SLOT(setCurrentIndex(QModelIndex)));
+   connect(m_pReqTLS,SIGNAL(selectFlaw(QModelIndex)),m_pSecurityIssues->view(),SLOT(setCurrentIndex(QModelIndex)));
+#endif
+//END
+//    loadAccountList();
    accountListHasChanged = false;
 
    combo_tls_method->setModel(TlsMethodModel::instance());
@@ -129,6 +169,7 @@ DlgAccounts::DlgAccounts(KConfigDialog* parent)
    /**/connect(m_pDTMFOverRTP,                    SIGNAL(clicked(bool))                  , this   , SLOT(changedAccountList())              );
    /**/connect(m_pDTMFOverSIP,                    SIGNAL(clicked(bool))                  , this   , SLOT(changedAccountList())              );
    /**/connect(m_pAutoAnswer,                     SIGNAL(clicked(bool))                  , this   , SLOT(changedAccountList())              );
+   /**/connect(m_pEnableVideo,                    SIGNAL(clicked(bool))                  , this   , SLOT(changedAccountList())              );
    /**/connect(spinbox_regExpire,                 SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
    /**/connect(spinBox_pa_published_port,         SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
    /**/connect(comboBox_ni_local_address,         SIGNAL(currentIndexChanged(int))       , this   , SLOT(changedAccountList())              );
@@ -144,6 +185,7 @@ DlgAccounts::DlgAccounts(KConfigDialog* parent)
    /**/connect(lineEdit_pa_published_address,     SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
    /**/connect(lineEdit_pa_published_address,     SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
    /**/connect(edit_tls_private_key_password,     SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
+   /**/connect(m_pUserAgent,                      SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
    /**/connect(spinbox_tls_listener,              SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
    /**/connect(m_pBitrateSB,                      SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
    /**/connect(file_tls_authority,                SIGNAL(textChanged(QString))           , this   , SLOT(changedAccountList())              );
@@ -167,6 +209,10 @@ DlgAccounts::DlgAccounts(KConfigDialog* parent)
    /**/connect(edit_credential_auth,              SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
    /**/connect(edit_credential_password,          SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
    /**/connect(edit_tls_private_key_password,     SIGNAL(textEdited(QString))            , this   , SLOT(changedAccountList())              );
+   /**/connect(m_pMinAudioPort,                   SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
+   /**/connect(m_pMaxAudioPort,                   SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
+   /**/connect(m_pMinVideoPort,                   SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
+   /**/connect(m_pMaxVideoPort,                   SIGNAL(valueChanged(int))              , this   , SLOT(changedAccountList())              );
    /**/connect(this,                              SIGNAL(updateButtons())                , parent , SLOT(updateButtons())                   );
    /**/connect(combo_security_STRP,               SIGNAL(currentIndexChanged(int))       , this   , SLOT(updateCombo(int))                  );
    /**/connect(button_add_credential,             SIGNAL(clicked())                      , this   , SLOT(addCredential())                   );
@@ -234,7 +280,7 @@ void DlgAccounts::saveAccount(const QModelIndex& item)
    m_IsLoading++;
 
    if (!m_pProxyCK->isChecked()) {
-      m_pProxyLE->setText("");
+      m_pProxyLE->setText(QString());
    }
 
    const KeyExchangeModel::Type currentKeyExchange = (!groupbox_STRP_keyexchange->isChecked())?KeyExchangeModel::Type::NONE:static_cast<KeyExchangeModel::Type>(combo_security_STRP->currentIndex());
@@ -255,9 +301,6 @@ void DlgAccounts::saveAccount(const QModelIndex& item)
    /*                                            Security                                                             */
    /**/ ACC setTlsPassword                 ( edit_tls_private_key_password->text()                                    );
    /**/ ACC setTlsListenerPort             ( spinbox_tls_listener->value()                                            );
-   /**/ ACC setTlsCaListFile               ( file_tls_authority->text()                                               );
-   /**/ ACC setTlsCertificateFile          ( file_tls_endpoint->text()                                                );
-   /**/ ACC setTlsPrivateKeyFile           ( file_tls_private_key->text()                                             );
    /**/ ACC setTlsMethod                   ( static_cast<TlsMethodModel::Type>(combo_tls_method->currentIndex())      );
    /**/ ACC setTlsCiphers                  ( edit_tls_cipher->text()                                                  );
    /**/ ACC setTlsServerName               ( edit_tls_outgoing->text()                                                );
@@ -267,7 +310,7 @@ void DlgAccounts::saveAccount(const QModelIndex& item)
    /**/ ACC setTlsVerifyServer             ( check_tls_incoming->isChecked()                                          );
    /**/ ACC setTlsVerifyClient             ( check_tls_answer->isChecked()                                            );
    /**/ ACC setTlsRequireClientCertificate ( check_tls_requier_cert->isChecked()                                      );
-   /**/ ACC setTlsEnable                   ( group_security_tls->isChecked()                                          );
+   /**/ ACC setTlsEnabled                  ( group_security_tls->isChecked()                                          );
    /**/ ACC setDisplaySasOnce              ( checkbox_ZRTP_Ask_user->isChecked()                                      );
    /**/ ACC setSrtpRtpFallback             ( checkbox_SDES_fallback_rtp->isChecked()                                  );
    /**/ ACC setZrtpDisplaySas              ( checkbox_ZRTP_display_SAS->isChecked()                                   );
@@ -284,8 +327,18 @@ void DlgAccounts::saveAccount(const QModelIndex& item)
    /**/ ACC setAutoAnswer                  ( m_pAutoAnswer->isChecked()                                               );
    /**/ ACC setLocalPort                   ( spinBox_ni_local_port->value()                                           );
    /**/ ACC setSrtpEnabled                 ( groupbox_STRP_keyexchange->isChecked()                                   );
+   /**/ ACC setAudioPortMax                ( m_pMaxAudioPort->value()                                                 );
+   /**/ ACC setAudioPortMin                ( m_pMinAudioPort->value()                                                 );
+   /**/ ACC setVideoPortMax                ( m_pMaxVideoPort->value()                                                 );
+   /**/ ACC setVideoPortMin                ( m_pMinVideoPort->value()                                                 );
+   /**/ ACC setUserAgent                   ( m_pUserAgent->text()                                                     );
+   /**/ ACC setVideoEnabled                ( m_pEnableVideo->isChecked()                                              );
    //                                                                                                                  /
 
+   /**/ ACC tlsCaListCertificate()->setPath( file_tls_authority->text()                                               );
+   /**/ ACC tlsCertificate ()->setPath     ( file_tls_endpoint->text()                                                );
+   /**/ ACC tlsPrivateKeyCertificate()->setPath( file_tls_private_key->text()                                         );
+
 
    if (m_pDefaultAccount->isChecked()) {
       ConfigurationSkeleton::setDefaultAccountId(ACC id());
@@ -350,39 +403,45 @@ void DlgAccounts::loadAccount(QModelIndex item)
 
    //         WIDGET VALUE                                          VALUE                     /
    /**/edit2_protocol->setCurrentIndex          ( (protocolIndex < 0) ? 0 : protocolIndex    );
-   /**/edit3_server->setText                    (  ACC hostname                       ());
-   /**/edit4_user->setText                      (  ACC username                       ());
-   /**/edit6_mailbox->setText                   (  ACC mailbox                        ());
-   /**/m_pProxyLE->setText                      (  ACC proxy                          ());
-   /**/m_pPresenceCK->setChecked                (  ACC presenceEnabled                ());
+   /**/edit3_server->setText                    (  ACC hostname                        ());
+   /**/edit4_user->setText                      (  ACC username                        ());
+   /**/edit6_mailbox->setText                   (  ACC mailbox                         ());
+   /**/m_pProxyLE->setText                      (  ACC proxy                           ());
+   /**/m_pPresenceCK->setChecked                (  ACC presenceEnabled                 ());
    /**/m_pPresenceCK->setEnabled                (  ACC supportPresencePublish() || ACC supportPresenceSubscribe());
-   /**/checkbox_ZRTP_Ask_user->setChecked       (  ACC isDisplaySasOnce               ());
-   /**/checkbox_SDES_fallback_rtp->setChecked   (  ACC isSrtpRtpFallback              ());
-   /**/checkbox_ZRTP_display_SAS->setChecked    (  ACC isZrtpDisplaySas               ());
-   /**/checkbox_ZRTP_warn_supported->setChecked (  ACC isZrtpNotSuppWarning           ());
-   /**/checkbox_ZTRP_send_hello->setChecked     (  ACC isZrtpHelloHash                ());
-   /**/checkbox_stun->setChecked                (  ACC isSipStunEnabled               ());
-   /**/line_stun->setText                       (  ACC sipStunServer                  ());
-   /**/spinbox_regExpire->setValue              (  ACC registrationExpire             ());
-   /**/radioButton_pa_same_as_local->setChecked (  ACC isPublishedSameAsLocal         ());
-   /**/radioButton_pa_custom->setChecked        ( !account->isPublishedSameAsLocal    ());
-   /**/lineEdit_pa_published_address->setText   (  ACC publishedAddress               ());
-   /**/spinBox_pa_published_port->setValue      (  ACC publishedPort                  ());
+   /**/checkbox_ZRTP_Ask_user->setChecked       (  ACC isDisplaySasOnce                ());
+   /**/checkbox_SDES_fallback_rtp->setChecked   (  ACC isSrtpRtpFallback               ());
+   /**/checkbox_ZRTP_display_SAS->setChecked    (  ACC isZrtpDisplaySas                ());
+   /**/checkbox_ZRTP_warn_supported->setChecked (  ACC isZrtpNotSuppWarning            ());
+   /**/checkbox_ZTRP_send_hello->setChecked     (  ACC isZrtpHelloHash                 ());
+   /**/checkbox_stun->setChecked                (  ACC isSipStunEnabled                ());
+   /**/m_pEnableVideo->setChecked               (  ACC isVideoEnabled                  ());
+   /**/line_stun->setText                       (  ACC sipStunServer                   ());
+   /**/spinbox_regExpire->setValue              (  ACC registrationExpire              ());
+   /**/radioButton_pa_same_as_local->setChecked (  ACC isPublishedSameAsLocal          ());
+   /**/radioButton_pa_custom->setChecked        ( !account->isPublishedSameAsLocal     ());
+   /**/lineEdit_pa_published_address->setText   (  ACC publishedAddress                ());
+   /**/spinBox_pa_published_port->setValue      (  ACC publishedPort                   ());
    /*                                                  Security                             **/
-   /**/edit_tls_private_key_password->setText   (  ACC tlsPassword                    ());
-   /**/spinbox_tls_listener->setValue           (  ACC tlsListenerPort                ());
-   /**/file_tls_authority->setText              (  ACC tlsCaListFile                  ());
-   /**/file_tls_endpoint->setText               (  ACC tlsCertificateFile             ());
-   /**/file_tls_private_key->setText            (  ACC tlsPrivateKeyFile              ());
-   /**/edit_tls_cipher->setText                 (  ACC tlsCiphers                     ());
-   /**/edit_tls_outgoing->setText               (  ACC tlsServerName                  ());
-   /**/spinbox_tls_timeout_sec->setValue        (  ACC tlsNegotiationTimeoutSec       ());
-   /**/spinbox_tls_timeout_msec->setValue       (  ACC tlsNegotiationTimeoutMsec      ());
-   /**/check_tls_incoming->setChecked           (  ACC isTlsVerifyServer              ());
-   /**/check_tls_answer->setChecked             (  ACC isTlsVerifyClient              ());
-   /**/check_tls_requier_cert->setChecked       (  ACC isTlsRequireClientCertificate  ());
-   /**/group_security_tls->setChecked           (  ACC isTlsEnable                    ());
-   /**/m_pAutoAnswer->setChecked                (  ACC isAutoAnswer                   ());
+   /**/edit_tls_private_key_password->setText   (  ACC tlsPassword                     ());
+   /**/spinbox_tls_listener->setValue           (  ACC tlsListenerPort                 ());
+   /**/file_tls_authority->setText              (  ACC tlsCaListCertificate    ()->path().toLocalFile());
+   /**/file_tls_endpoint->setText               (  ACC tlsCertificate          ()->path().toLocalFile());
+   /**/file_tls_private_key->setText            (  ACC tlsPrivateKeyCertificate()->path().toLocalFile());
+   /**/edit_tls_cipher->setText                 (  ACC tlsCiphers                      ());
+   /**/edit_tls_outgoing->setText               (  ACC tlsServerName                   ());
+   /**/spinbox_tls_timeout_sec->setValue        (  ACC tlsNegotiationTimeoutSec        ());
+   /**/spinbox_tls_timeout_msec->setValue       (  ACC tlsNegotiationTimeoutMsec       ());
+   /**/check_tls_incoming->setChecked           (  ACC isTlsVerifyServer               ());
+   /**/check_tls_answer->setChecked             (  ACC isTlsVerifyClient               ());
+   /**/check_tls_requier_cert->setChecked       (  ACC isTlsRequireClientCertificate   ());
+   /**/group_security_tls->setChecked           (  ACC isTlsEnabled                    ());
+   /**/m_pAutoAnswer->setChecked                (  ACC isAutoAnswer                    ());
+   /**/m_pMinVideoPort->setValue                (  ACC videoPortMin                    ());
+   /**/m_pMaxVideoPort->setValue                (  ACC videoPortMax                    ());
+   /**/m_pMaxAudioPort->setValue                (  ACC audioPortMax                    ());
+   /**/m_pMinAudioPort->setValue                (  ACC audioPortMin                    ());
+   /**/m_pUserAgent->setText                    (  ACC userAgent                       ());
    /*                                                                                        */
 
    combo_security_STRP->setModel(ACC keyExchangeModel());
@@ -391,6 +450,11 @@ void DlgAccounts::loadAccount(QModelIndex item)
       /**/combo_security_STRP->setCurrentIndex     (  ACC keyExchangeModel()->toIndex( ACC keyExchange()).row());
    }
 
+   //BUG the daemon doesn't support changing account type after
+   edit2_protocol->setEnabled(ACC state() == Account::AccountEditState::NEW || ACC id().isEmpty());
+
+   m_pCodecsLW->setEnabled(ACC isVideoEnabled ());
+
    updateCombo(0);
 
    m_pDTMFOverRTP->setChecked( ACC DTMFType()==DtmfType::OverRtp);
@@ -430,12 +494,15 @@ void DlgAccounts::loadAccount(QModelIndex item)
    connect(m_pCodecsLW->selectionModel()       ,SIGNAL(currentChanged(QModelIndex,QModelIndex)), this, SLOT(loadVidCodecDetails(QModelIndex,QModelIndex))  );
    #endif
 
+   ACC securityValidationModel()->update();
+   m_pSecurityIssues->setModel(ACC securityValidationModel());
+
 
    //Enable tabs
-   bool isntIP2IP = ! ( ACC alias() == "IP2IP" );
+   bool isntIP2IP = ! ( ACC alias() == Account::ProtocolName::IP2IP );
    bool isIAX     = ACC protocol()  == Account::Protocol::IAX;
-   bool enableTab[6] = {isntIP2IP,isntIP2IP,true,isntIP2IP && !isIAX,isntIP2IP,true};
-   for (int i=0;i<6;i++)
+   bool enableTab[7] = {isntIP2IP,isntIP2IP,isntIP2IP,true,isntIP2IP && !isIAX,isntIP2IP,true};
+   for (int i=0;i<7;i++)
       frame2_editAccounts->setTabEnabled( i, enableTab[i] );
 
    //Setup ringtone
@@ -491,6 +558,8 @@ void DlgAccounts::loadAccount(QModelIndex item)
       line_stun->setText( ACC sipStunServer() );
    }
 
+   updateSecurityValidation();
+
    updateStatusLabel(account);
    enablePublished();
    frame2_editAccounts->setEnabled(true);
@@ -502,7 +571,7 @@ void DlgAccounts::loadAccount(QModelIndex item)
 ///Load an account
 void DlgAccounts::loadAccountList()
 {
-   AccountListModel::instance()->updateAccounts();
+//    AccountListModel::instance()->updateAccounts();
    if (treeView_accountList->model()->rowCount() > 0 && !treeView_accountList->currentIndex().isValid()) {
       if (CategorizedAccountModel::instance()->index(0,0,CategorizedAccountModel::instance()->index(0,0)).isValid())
          treeView_accountList->setCurrentIndex(CategorizedAccountModel::instance()->index(0,0,CategorizedAccountModel::instance()->index(0,0)));
@@ -570,11 +639,10 @@ void DlgAccounts::on_button_accountDown_clicked()
 ///Add new account
 void DlgAccounts::on_button_accountAdd_clicked()
 {
-   const QString newAlias = i18n("New account%1",AccountListModel::getSimilarAliasIndex("New account"));
-   AccountListModel::instance()->addAccount(newAlias);
-   int r = treeView_accountList->model()->rowCount(treeView_accountList->model()->index(0,0)) - 1;
-   const QModelIndex index = treeView_accountList->model()->index(r,0,treeView_accountList->model()->index(0,0));
-   treeView_accountList->setCurrentIndex(index);
+   const QString newAlias = i18n("New account%1",AccountListModel::getSimilarAliasIndex(i18n("New account")));
+   Account* newAcc = AccountListModel::instance()->addAccount(newAlias);
+   const QModelIndex& newIndex = CategorizedAccountModel::instance()->mapFromSource(newAcc->index());
+   treeView_accountList->setCurrentIndex(newIndex);
 
    frame2_editAccounts->setEnabled(true);
    frame2_editAccounts->setCurrentIndex(0);
@@ -725,7 +793,9 @@ void DlgAccounts::updateStatusLabel(Account* account)
    if(!account || currentAccount() != account)
       return;
    const QString status = ACC toHumanStateName();
-   edit7_state->setText( "<FONT COLOR=\"" + ACC stateColorName() + "\">" + status + "</FONT>" );
+   const QString extra = ACC protocol() == Account::Protocol::SIP && ACC lastErrorCode()>=0
+      ? (QString(" (%1)").arg(ACC lastErrorCode())) :QString();
+   edit7_state->setText( "<FONT COLOR=\"" + ACC stateColorName() + "\">" + status + "</FONT>"+extra );
 }
 
 void DlgAccounts::updateStatusLabel()
@@ -750,7 +820,7 @@ bool DlgAccounts::hasIncompleteRequiredFields()
    }
 
    bool fields[4] = {edit1_alias->text().isEmpty(),edit3_server->text().isEmpty(),edit4_user->text().isEmpty(),edit5_password->text().isEmpty()};
-   bool isIncomplete = acc && (acc->alias() != "IP2IP") && (fields[0]|fields[1]|fields[2]|fields[3]);
+   bool isIncomplete = acc && (acc->alias() != Account::ProtocolName::IP2IP) && (fields[0]|fields[1]|fields[2]|fields[3]);
    //Add visual feedback for missing fields
    for (int i=0;i<requiredFieldsLabels.size();i++) {
       static KStatefulBrush errorBrush( KColorScheme::Window, KColorScheme::NegativeText );
@@ -884,4 +954,107 @@ Account* DlgAccounts::currentAccount() const
    return AccountListModel::instance()->getAccountByModelIndex(CategorizedAccountModel::instance()->mapToSource(treeView_accountList->currentIndex()));
 }
 
+
+void DlgAccounts::addFlawToCertificateField(const Flaw* flaw)
+{
+Q_UNUSED(flaw)
+//BEGIN Temporarely disable advanced security widgets
+#if false
+   switch(flaw->type()) {
+      case Certificate::Type::AUTHORITY:
+         m_pAuthorityII->addFlaw(flaw);
+         break;
+      case Certificate::Type::USER:
+         m_pEndCertII->addFlaw(flaw);
+         break;
+      case Certificate::Type::PRIVATE_KEY:
+         m_pPKII->addFlaw(flaw);
+         break;
+      case Certificate::Type::NONE:
+         qDebug() << "Invalid certificate type";
+         break;
+   };
+#endif
+//END
+}
+
+
+void DlgAccounts::updateSecurityValidation()
+{
+#if false
+   Account* a = currentAccount();
+   //Create the widgets
+   m_pVerifyServer->setModel(a->securityValidationModel());
+   m_pVerifyClient->setModel(a->securityValidationModel());
+   m_pReqTLS->setModel(a->securityValidationModel());
+   m_pAuthorityII->setModel(a->securityValidationModel());
+   m_pEndCertII->setModel(a->securityValidationModel());
+   m_pPKII->setModel(a->securityValidationModel());
+
+//    IssuesIcon*
+
+//    newWidget->addFlaw(new Flaw(SecurityValidationModel::SecurityFlaw::TLS_DISABLED,Certificate::Type::NONE));
+
+   //Add the flaws
+   foreach(const Flaw* flaw, currentAccount()->securityValidationModel()->currentFlaws()) {
+      switch (flaw->flaw()) {
+         case SecurityValidationModel::SecurityFlaw::SRTP_DISABLED:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::TLS_DISABLED:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::CERTIFICATE_EXPIRED:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::CERTIFICATE_SELF_SIGNED:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::CA_CERTIFICATE_MISSING:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::END_CERTIFICATE_MISSING:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::PRIVATE_KEY_MISSING:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::CERTIFICATE_MISMATCH:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::CERTIFICATE_STORAGE_PERMISSION:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::CERTIFICATE_STORAGE_FOLDER:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::CERTIFICATE_STORAGE_LOCATION:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::OUTGOING_SERVER_MISMATCH:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::VERIFY_INCOMING_DISABLED:
+            m_pVerifyServer->addFlaw(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::VERIFY_ANSWER_DISABLED:
+            m_pVerifyClient->addFlaw(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::REQUIRE_CERTIFICATE_DISABLED:
+
+            break;
+         case SecurityValidationModel::SecurityFlaw::MISSING_CERTIFICATE:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::MISSING_AUTHORITY:
+            addFlawToCertificateField(flaw);
+            break;
+         case SecurityValidationModel::SecurityFlaw::__COUNT:
+         default:
+            qDebug() << "Invalid flaw";
+      }
+   }
+#endif
+}
+
 #undef ACC_
