@@ -23,37 +23,34 @@
 #include <KLocale>
 
 //SFLPhone
-#include "../lib/videodevice.h"
-#include "../lib/videocodecmodel.h"
-#include "../lib/videomodel.h"
+#include "../lib/video/videodevice.h"
+#include "../lib/video/videocodecmodel.h"
+#include "../lib/video/videomodel.h"
+#include "../lib/video/videoresolution.h"
+#include "../lib/video/videochannel.h"
+#include "../lib/video/videorate.h"
+#include "../lib/video/videodevicemodel.h"
 
 ///Constructor
 DlgVideo::DlgVideo(KConfigDialog* parent)
- : QWidget(parent),m_pDevice(nullptr),m_IsChanged(false),m_IsLoading(true)
+ : QWidget(parent),m_pDevice(nullptr),m_IsChanged(false),m_IsLoading(true),m_pChannel(nullptr),m_pResolution(nullptr)
 {
    setupUi(this);
-   
-   const QList<VideoDevice*> devices =  VideoDevice::deviceList();
-   foreach(VideoDevice* dev,devices) {
-      m_pDeviceCB->addItem(dev->deviceId());
-   }
 
-   connect(m_pDeviceCB    ,SIGNAL(currentIndexChanged(QString)), this   , SLOT(loadDevice(QString))    );
-   connect(m_pChannelCB   ,SIGNAL(currentIndexChanged(QString)), this   , SLOT(loadResolution(QString)));
-   connect(m_pResolutionCB,SIGNAL(currentIndexChanged(QString)), this   , SLOT(loadRate(QString))      );
-   connect(m_pRateCB      ,SIGNAL(currentIndexChanged(QString)), this   , SLOT(changeRate(QString))    );
+   //updateWidgets();
+
    connect(m_pPreviewPB   ,SIGNAL(clicked())                   , this   , SLOT(startStopPreview())     );
    connect( this          ,SIGNAL(updateButtons())             , parent , SLOT(updateButtons())        );
+   connect(VideoModel::instance(),SIGNAL(previewStateChanged(bool)),this,SLOT(startStopPreview(bool))  );
 
+   connect(VideoModel::instance(),SIGNAL(previewStarted(VideoRenderer*)),m_pPreviewGV,SLOT(addRenderer(VideoRenderer*))   );
+   connect(VideoModel::instance(),SIGNAL(previewStopped(VideoRenderer*)),m_pPreviewGV,SLOT(removeRenderer(VideoRenderer*)));
 
-   m_pConfGB->setEnabled(devices.size());
-
-   if (devices.size())
-      loadDevice(devices[0]->deviceId());
 
    if (VideoModel::instance()->isPreviewing()) {
       m_pPreviewPB->setText(i18n("Stop preview"));
    }
+//    m_pVideoSettings->slotReloadDevices();
    m_IsChanged = false;
    m_IsLoading = false;
 }
@@ -70,80 +67,51 @@ bool DlgVideo::hasChanged()
    return m_IsChanged;
 }
 
-///Load the device list
-void DlgVideo::loadDevice(QString device)
-{
-   if (!m_IsLoading) {
-      m_IsChanged = true;
-      emit updateButtons();
-   }
-   m_pDevice = VideoDevice::getDevice(device);
-   const QString curChan = m_pDevice->channel();
-   if (m_pDevice) {
-      m_pChannelCB->clear();
-      foreach(const VideoChannel& channel,m_pDevice->channelList()) {
-         m_pChannelCB->addItem(channel);
-         if (channel == curChan)
-            m_pChannelCB->setCurrentIndex(m_pChannelCB->count()-1);
-      }
-   }
-}
-
-///Load resolution
-void DlgVideo::loadResolution(QString channel)
-{
-   if (!m_IsLoading) {
-      m_IsChanged = true;
-      emit updateButtons();
-   }
-   Resolution current = m_pDevice->resolution();
-   m_pResolutionCB->clear();
-   foreach(const Resolution& res,m_pDevice->resolutionList(channel)) {
-      m_pResolutionCB->addItem(res.toString());
-      if (current == res) {
-         m_pResolutionCB->setCurrentIndex(m_pResolutionCB->count()-1);
-      }
-   }
-   m_pDevice->setChannel(channel);
-}
-
-///Load the rate
-void DlgVideo::loadRate(QString resolution)
-{
-   if (!m_IsLoading) {
-      m_IsChanged = true;
-      emit updateButtons();
-   }
-   m_pRateCB->clear();
-   const QString rate = m_pDevice->rate();
-   foreach(const QString& r,m_pDevice->rateList(m_pChannelCB->currentText(),resolution)) {
-      m_pRateCB->addItem(r);
-      if (r == rate)
-         m_pRateCB->setCurrentIndex(m_pRateCB->count()-1);
-   }
-   m_pDevice->setResolution(resolution);
-}
-
-///Changes the rate
-void DlgVideo::changeRate(QString rate)
-{
-   if (!m_IsLoading) {
-      m_IsChanged = true;
-      emit updateButtons();
-   }
-   m_pDevice->setRate(rate);
-}
-
 ///Start or stop preview
 void DlgVideo::startStopPreview()
 {
    //TODO check if the preview is already running
    if (VideoModel::instance()->isPreviewing()) {
-      m_pPreviewPB->setText(i18n("Start preview"));
       VideoModel::instance()->stopPreview();
    }
    else {
-      m_pPreviewPB->setText(i18n("Stop preview"));
       VideoModel::instance()->startPreview();
    }
 }
+
+void DlgVideo::startStopPreview(bool state)
+{
+   if (state) {
+      m_pPreviewPB->setText(i18n("Stop preview"));
+   }
+   else {
+      m_pPreviewPB->setText(i18n("Start preview"));
+   }
+}
+
+void DlgVideo::updateWidgets ()
+{
+   //The models should take care of that
+}
+
+void DlgVideo::updateSettings()
+{
+   VideoDeviceModel::instance()->setActive(m_pVideoSettings->device());
+   m_IsChanged = false;
+}
+
+void DlgVideo::slotReloadPreview()
+{
+   if (VideoModel::instance()->isPreviewing()) {
+      VideoModel::instance()->stopPreview();
+      VideoModel::instance()->startPreview();
+   }
+}
+
+
+void DlgVideo::slotSettingsChanged()
+{
+   m_IsChanged = true;
+   emit updateButtons();
+   slotReloadPreview();
+}
