@@ -62,7 +62,8 @@ int AccountListNoCheckProxyModel::rowCount(const QModelIndex& parentIdx ) const
 
 ///Constructors
 ///@param fill Whether to fill the list with accounts from configurationManager or not.
-AccountListModel::AccountListModel() : QAbstractListModel(QCoreApplication::instance()),m_pColorVisitor(nullptr),m_pDefaultAccount(nullptr)
+AccountListModel::AccountListModel() : QAbstractListModel(QCoreApplication::instance()),
+m_pColorVisitor(nullptr),m_pDefaultAccount(nullptr),m_pIP2IP(nullptr)
 {
    setupRoleName();
 }
@@ -100,9 +101,9 @@ void AccountListModel::setupRoleName()
    roles.insert(Account::Role::Mailbox                  ,QByteArray("mailbox"                       ));
    roles.insert(Account::Role::Proxy                    ,QByteArray("proxy"                         ));
    roles.insert(Account::Role::TlsPassword              ,QByteArray("tlsPassword"                   ));
-   roles.insert(Account::Role::TlsCaListFile            ,QByteArray("tlsCaListFile"                 ));
-   roles.insert(Account::Role::TlsCertificateFile       ,QByteArray("tlsCertificateFile"            ));
-   roles.insert(Account::Role::TlsPrivateKeyFile        ,QByteArray("tlsPrivateKeyFile"             ));
+   roles.insert(Account::Role::TlsCaListCertificate     ,QByteArray("tlsCaListCertificate"          ));
+   roles.insert(Account::Role::TlsCertificate           ,QByteArray("tlsCertificate"                ));
+   roles.insert(Account::Role::TlsPrivateKeyCertificate ,QByteArray("tlsPrivateKeyCertificate"      ));
    roles.insert(Account::Role::TlsCiphers               ,QByteArray("tlsCiphers"                    ));
    roles.insert(Account::Role::TlsServerName            ,QByteArray("tlsServerName"                 ));
    roles.insert(Account::Role::SipStunServer            ,QByteArray("sipStunServer"                 ));
@@ -121,7 +122,7 @@ void AccountListModel::setupRoleName()
    roles.insert(Account::Role::TlsVerifyServer          ,QByteArray("tlsVerifyServer"               ));
    roles.insert(Account::Role::TlsVerifyClient          ,QByteArray("tlsVerifyClient"               ));
    roles.insert(Account::Role::TlsRequireClientCertificate,QByteArray("tlsRequireClientCertificate" ));
-   roles.insert(Account::Role::TlsEnable                ,QByteArray("tlsEnable"                     ));
+   roles.insert(Account::Role::TlsEnabled               ,QByteArray("tlsEnabled"                    ));
    roles.insert(Account::Role::DisplaySasOnce           ,QByteArray("displaySasOnce"                ));
    roles.insert(Account::Role::SrtpRtpFallback          ,QByteArray("srtpRtpFallback"               ));
    roles.insert(Account::Role::ZrtpDisplaySas           ,QByteArray("zrtpDisplaySas"                ));
@@ -140,6 +141,18 @@ void AccountListModel::setupRoleName()
    setRoleNames(roles);
 }
 
+///Get the IP2IP account
+Account* AccountListModel::ip2ip() const
+{
+   if (!m_pIP2IP) {
+      foreach(Account* a,m_lAccounts) {
+         if (a->id() == Account::ProtocolName::IP2IP)
+            const_cast<AccountListModel*>(this)->m_pIP2IP = a;
+      }
+   }
+   return m_pIP2IP;
+}
+
 ///Singleton
 AccountListModel* AccountListModel::instance()
 {
@@ -175,6 +188,7 @@ void AccountListModel::accountChanged(const QString& account,const QString& stat
             Account* acc = Account::buildExistingAccountFromId(accountIds[i]);
             m_lAccounts.insert(i, acc);
             connect(acc,SIGNAL(changed(Account*)),this,SLOT(accountChanged(Account*)));
+            connect(acc,SIGNAL(presenceEnabledChanged(bool)),this,SLOT(slotAccountPresenceEnabledChanged(bool)));
             emit dataChanged(index(i,0),index(size()-1));
             emit layoutChanged();
          }
@@ -237,6 +251,13 @@ void AccountListModel::slotVoiceMailNotify(const QString &accountID, int count)
    }
 }
 
+///Propagate account presence state
+void AccountListModel::slotAccountPresenceEnabledChanged(bool state)
+{
+   Q_UNUSED(state)
+   emit presenceEnabledChanged(isPresenceEnabled());
+}
+
 ///Update accounts
 void AccountListModel::update()
 {
@@ -260,6 +281,7 @@ void AccountListModel::update()
          m_lAccounts.insert(i, a);
          emit dataChanged(index(i,0),index(size()-1,0));
          connect(a,SIGNAL(changed(Account*)),this,SLOT(accountChanged(Account*)));
+         connect(a,SIGNAL(presenceEnabledChanged(bool)),this,SLOT(slotAccountPresenceEnabledChanged(bool)));
          emit layoutChanged();
       }
    }
@@ -278,6 +300,7 @@ void AccountListModel::updateAccounts()
          Account* a = Account::buildExistingAccountFromId(accountIds[i]);
          m_lAccounts += a;
          connect(a,SIGNAL(changed(Account*)),this,SLOT(accountChanged(Account*)));
+         connect(a,SIGNAL(presenceEnabledChanged(bool)),this,SLOT(slotAccountPresenceEnabledChanged(bool)));
          emit dataChanged(index(size()-1,0),index(size()-1,0));
       }
       else {
@@ -441,7 +464,7 @@ Account* AccountListModel::currentAccount()
    else {
       Account* a = instance()->firstRegisteredAccount();
       if (!a)
-         a = instance()->getAccountById("IP2IP");
+         a = instance()->getAccountById(Account::ProtocolName::IP2IP);
       instance()->setPriorAccount(a);
       return a;
    }
@@ -523,6 +546,33 @@ QString AccountListModel::getSimilarAliasIndex(const QString& alias)
    return QString();
 }
 
+bool AccountListModel::isPresenceEnabled() const
+{
+   foreach(Account* a,m_lAccounts) {
+      if (a->presenceEnabled())
+         return true;
+   }
+   return false;
+}
+
+bool AccountListModel::isPresencePublishSupported() const
+{
+   foreach(Account* a,m_lAccounts) {
+      if (a->supportPresencePublish())
+         return true;
+   }
+   return false;
+}
+
+bool AccountListModel::isPresenceSubscribeSupported() const
+{
+   foreach(Account* a,m_lAccounts) {
+      if (a->supportPresenceSubscribe())
+         return true;
+   }
+   return false;
+}
+
 
 /*****************************************************************************
  *                                                                           *
@@ -536,6 +586,7 @@ Account* AccountListModel::addAccount(const QString& alias)
    Account* a = Account::buildNewAccountFromAlias(alias);
    connect(a,SIGNAL(changed(Account*)),this,SLOT(accountChanged(Account*)));
    m_lAccounts += a;
+   connect(a,SIGNAL(presenceEnabledChanged(bool)),this,SLOT(slotAccountPresenceEnabledChanged(bool)));
 
    emit dataChanged(index(m_lAccounts.size()-1,0), index(m_lAccounts.size()-1,0));
    return a;
@@ -550,7 +601,7 @@ void AccountListModel::removeAccount(Account* account)
    m_lAccounts.remove(aindex);
    m_lDeletedAccounts << account->id();
    if (currentAccount() == account)
-      setPriorAccount(getAccountById("IP2IP"));
+      setPriorAccount(getAccountById(Account::ProtocolName::IP2IP));
    emit dataChanged(index(aindex,0), index(m_lAccounts.size()-1,0));
    emit layoutChanged();
    //delete account;
