@@ -20,6 +20,7 @@
 #define CALL_H
 
 #include "categorizedcompositenode.h"
+#include <time.h>
 
 //Qt
 #include <QtCore/QDebug>
@@ -30,13 +31,13 @@ class QTimer;
 #include "sflphone_const.h"
 #include "typedefs.h"
 #include "historytimecategorymodel.h"
-class AbstractContactBackend;
 class Account;
 class VideoRenderer;
 class InstantMessagingModel;
 class UserActionModel;
 class PhoneNumber;
 class TemporaryPhoneNumber;
+class AbstractHistoryBackend;
 
 class Call;
 
@@ -93,8 +94,6 @@ public:
       Department    = 112,
       Email         = 113,
       Organisation  = 114,
-      Codec         = 115,
-      IsConference  = 116,
       Object        = 117,
       PhotoPtr      = 118,
       CallState     = 119,
@@ -110,6 +109,7 @@ public:
       CallCount     = 129, /* The number of calls made with the same phone number */
       TotalSpentTime= 130, /* The total time spent speaking to with this phone number*/
       Missed        = 131,
+      CallLifeCycleState= 132,
       DropState     = 300,
       DTMFAnimState = 400,
       LastDTMFidx   = 401,
@@ -136,7 +136,8 @@ public:
       ERROR           = 10,/** This state should never be reached */
       CONFERENCE      = 11,/** This call is the current conference*/
       CONFERENCE_HOLD = 12,/** This call is a conference on hold*/
-      COUNT,
+      INITIALIZATION  = 13,/** The call have been placed, but the peer hasn't confirmed yet */
+      __COUNT,
    };
    Q_ENUMS(State)
 
@@ -157,8 +158,8 @@ public:
 
    ///@enum Direction If the user have been called or have called
    enum class Direction : int {
-      INCOMING,
-      OUTGOING,
+      INCOMING, /** Someone has called      */
+      OUTGOING, /** The user called someone */
    };
    Q_ENUMS(Direction)
 
@@ -211,7 +212,9 @@ public:
       constexpr static const char* TIMESTAMP_STOP    = "timestamp_stop" ;
       constexpr static const char* MISSED            = "missed"         ;
       constexpr static const char* DIRECTION         = "direction"      ;
-
+      constexpr static const char* CONTACT_USED      = "contact_used"   ;
+      constexpr static const char* CONTACT_UID       = "contact_uid"    ;
+      constexpr static const char* NUMBER_TYPE       = "number_type"    ;
    };
 
    ///"getCallDetails()" fields
@@ -226,14 +229,28 @@ public:
       constexpr static const char* CONF_ID           = "CONF_ID"        ;
    };
 
+   ///"getConferenceDetails()" fields
+   class ConfDetailsMapFields {
+   public:
+      constexpr static const char* CONF_STATE        = "CONF_STATE"     ;
+      constexpr static const char* CONFID            = "CONFID"         ;
+   };
+
    ///If the call is incoming or outgoing
-   class CallType {
+   class CallDirection {
    public:
       constexpr static const char* INCOMING = "0";
       constexpr static const char* OUTGOING = "1";
    };
 
-   /** @enum Call::DaemonState 
+   ///Is the call between one or more participants
+   enum class Type {
+      CALL      , /** A simple call                  */
+      CONFERENCE, /** A composition of other calls   */
+      HISTORY   , /** A call from a previous session */
+   };
+
+   /** @enum Call::DaemonState
    * This enum have all the states a call can take for the daemon.
    */
    enum class DaemonState : unsigned int
@@ -244,7 +261,7 @@ public:
       HOLD    = 3, /** Call is on hold */
       HUNG_UP = 4, /** Call is over  */
       FAILURE = 5, /** Call has failed */
-      COUNT,
+      __COUNT,
    };
 
    /** @enum Call::Action
@@ -257,7 +274,20 @@ public:
       TRANSFER = 2, /** Put into or out of transfer mode*/
       HOLD     = 3, /** Hold or unhold the call */
       RECORD   = 4, /** Enable or disable recording */
-      COUNT,
+      __COUNT,
+   };
+
+   /** @enum Call::LifeCycleState
+    * This enum help track the call meta state
+    * @todo Eventually add a meta state between progress and finished for
+    *  calls that are still relevant enough to be in the main UI, such
+    *  as BUSY OR FAILURE while also finished
+    */
+   enum class LifeCycleState {
+      INITIALIZATION = 0, /** Anything before the media transfer start   */
+      PROGRESS       = 1, /** The peers are in communication (or hold)   */
+      FINISHED       = 2, /** Everything is over, there is no going back */
+      __COUNT
    };
 
    //Read only properties
@@ -267,10 +297,7 @@ public:
    Q_PROPERTY( bool               isHistory        READ isHistory                                 )
    Q_PROPERTY( uint               stopTimeStamp    READ stopTimeStamp                             )
    Q_PROPERTY( uint               startTimeStamp   READ startTimeStamp                            )
-   Q_PROPERTY( QString            currentCodecName READ currentCodecName                          )
    Q_PROPERTY( bool               isSecure         READ isSecure                                  )
-   Q_PROPERTY( bool               isConference     READ isConference                              )
-   Q_PROPERTY( QString            confId           READ confId                                    )
    Q_PROPERTY( VideoRenderer*     videoRenderer    READ videoRenderer                             )
    Q_PROPERTY( QString            formattedName    READ formattedName                             )
    Q_PROPERTY( QString            length           READ length                                    )
@@ -280,6 +307,7 @@ public:
    Q_PROPERTY( QString            toHumanStateName READ toHumanStateName                          )
    Q_PROPERTY( bool               missed           READ isMissed                                  )
    Q_PROPERTY( Direction          direction        READ direction                                 )
+   Q_PROPERTY( bool               hasVideo         READ hasVideo                                  )
    Q_PROPERTY( Call::LegacyHistoryState historyState     READ historyState                        )
 
    //Read/write properties
@@ -295,9 +323,7 @@ public:
    static Call* buildIncomingCall (const QString& callId                                                       );
    static Call* buildRingingCall  (const QString& callId                                                       );
    static Call* buildHistoryCall  (const QMap<QString,QString>& hc                                             );
-   static Call* buildExistingCall (QString callId                                                              );
-   static void  setContactBackend (AbstractContactBackend* be                                                  );
-   static AbstractContactBackend* contactBackend ();
+   static Call* buildExistingCall (const QString& callId                                                       );
 
    //Static getters
    static Call::LegacyHistoryState historyStateFromType    ( const QString& type                                           );
@@ -315,10 +341,7 @@ public:
    bool                     isHistory        () const;
    time_t                   stopTimeStamp    () const;
    time_t                   startTimeStamp   () const;
-   QString                  currentCodecName () const;
    bool                     isSecure         () const;
-   bool                     isConference     () const;
-   const QString            confId           () const;
    const QString            transferNumber   () const;
    const QString            dialNumber       () const;
    const QString            recordingPath    () const;
@@ -332,33 +355,38 @@ public:
    bool                     isHistory        ()      ;
    bool                     isMissed         () const;
    Call::Direction          direction        () const;
+   AbstractHistoryBackend*  backend          () const;
+   bool                     hasVideo         () const;
+   Call::LifeCycleState     lifeCycleState   () const;
+   Call::Type               type             () const;
 
    //Automated function
    Call::State stateChanged(const QString & newState);
    Call::State performAction(Call::Action action);
 
    //Setters
-   void setConference     ( bool               value      );
-   void setConfId         ( const QString&     value      );
    void setTransferNumber ( const QString&     number     );
    void setDialNumber     ( const QString&     number     );
    void setDialNumber     ( const PhoneNumber* number     );
    void setRecordingPath  ( const QString&     path       );
    void setPeerName       ( const QString&     name       );
    void setAccount        ( Account*           account    );
+   void setBackend        ( AbstractHistoryBackend* backend);
 
    //Mutators
    void appendText(const QString& str);
    void backspaceItemText();
    void reset();
-   void sendTextMessage(QString message);
+   void sendTextMessage(const QString& message);
+
+   //syntactic sugar
+   Call* operator<<( Call::Action& c);
 
 private:
 
    //Attributes
    Account*                 m_Account         ;
    QString                  m_CallId          ;
-   QString                  m_ConfId          ;
    PhoneNumber*             m_pPeerPhoneNumber;
    QString                  m_PeerName        ;
    QString                  m_RecordingPath   ;
@@ -367,23 +395,20 @@ private:
    time_t                   m_pStopTimeStamp  ;
    TemporaryPhoneNumber*    m_pTransferNumber ;
    TemporaryPhoneNumber*    m_pDialNumber     ;
-   bool                     m_isConference    ;
    Call::State              m_CurrentState    ;
    bool                     m_Recording       ;
-   static Call*             m_sSelectedCall   ;
    InstantMessagingModel*   m_pImModel        ;
    QTimer*                  m_pTimer          ;
    UserActionModel*         m_pUserActionModel;
    bool                     m_History         ;
    bool                     m_Missed          ;
    Call::Direction          m_Direction       ;
+   Call::Type               m_Type            ;
+   AbstractHistoryBackend*  m_pBackend        ;
 
    //Cache
    HistoryTimeCategoryModel::HistoryConst m_HistoryConst;
 
-   //Static attribute
-   static AbstractContactBackend* m_pContactBackend;
-
    //State machine
    /**
     *  actionPerformedStateMap[orig_state][action]
@@ -415,6 +440,19 @@ private:
    **/
    static const TypedStateMachine< TypedStateMachine< function , Call::DaemonState > , Call::State > stateChangedFunctionMap;
 
+   /**
+    * metaStateTransitionValidationMap help validate if a state transition violate the lifecycle logic.
+    * it should technically never happen, but this is an easy additional safety to implement
+    * and prevent human (developer) errors.
+    */
+   static const TypedStateMachine< TypedStateMachine< bool , Call::LifeCycleState > , Call::State > metaStateTransitionValidationMap;
+
+   /**
+    * Convert the call state into its meta state (life cycle state). The meta state is a flat,
+    * forward only progression from creating to archiving of a call.
+    */
+   static const TypedStateMachine< Call::LifeCycleState , Call::State > metaStateMap;
+
    explicit Call(const QString& confId, const QString& account);
    Call(Call::State startState, const QString& callId, const QString& peerName = QString(), PhoneNumber* number = nullptr, Account* account = nullptr);
    static Call::DaemonState toDaemonCallState   (const QString& stateName);
@@ -425,6 +463,7 @@ private:
    // to know when it is called.
    void nothing      () __attribute__ ((const));
    void error        () __attribute__ ((noreturn));
+   void failure      ();
    void accept       ();
    void refuse       ();
    void acceptTransf ();
@@ -447,6 +486,7 @@ private:
    //Helpers
    void changeCurrentState(Call::State newState);
    void setStartTimeStamp(time_t stamp);
+   void initTimer();
 
 public Q_SLOTS:
    void playRecording();
