@@ -23,6 +23,7 @@
 
 //SFLPhone library
 #include "call.h"
+#include "uri.h"
 #include "phonedirectorymodel.h"
 #include "phonenumber.h"
 #include "accountlistmodel.h"
@@ -32,10 +33,11 @@
 #include "dbus/instancemanager.h"
 #include "sflphone_const.h"
 #include "typedefs.h"
-#include "abstractcontactbackend.h"
+#include "abstractitembackend.h"
 #include "dbus/videomanager.h"
 #include "historymodel.h"
 #include "visitors/phonenumberselector.h"
+#include "contactmodel.h"
 
 //Define
 ///InternalStruct: internal representation of a call
@@ -81,7 +83,7 @@ void CallModel::init()
    if (!dbusInit) {
       CallManagerInterface& callManager = DBus::CallManager::instance();
       #ifdef ENABLE_VIDEO
-      VideoInterface& interface = DBus::VideoManager::instance();
+      VideoManagerInterface& interface = DBus::VideoManager::instance();
       #endif
 
       //SLOTS
@@ -91,12 +93,11 @@ void CallModel::init()
       /**/connect(&callManager, SIGNAL(conferenceCreated(QString))              , this , SLOT(slotIncomingConference(QString))         );
       /**/connect(&callManager, SIGNAL(conferenceChanged(QString,QString))      , this , SLOT(slotChangingConference(QString,QString)) );
       /**/connect(&callManager, SIGNAL(conferenceRemoved(QString))              , this , SLOT(slotConferenceRemoved(QString))          );
-      /**/connect(&callManager, SIGNAL(volumeChanged(QString,double))           , this , SLOT(slotVolumeChanged(QString,double))       );
       /**/connect(&callManager, SIGNAL(recordPlaybackFilepath(QString,QString)) , this , SLOT(slotNewRecordingAvail(QString,QString))  );
       /**/connect(&callManager, SIGNAL(recordingStateChanged(QString,bool))     , this,  SLOT(slotRecordStateChanged(QString,bool)));
       #ifdef ENABLE_VIDEO
-      /**/connect(&interface  , SIGNAL(startedDecoding(QString,QString,int,int)), this , SLOT(slotStartedDecoding(QString,QString))    );
-      /**/connect(&interface  , SIGNAL(stoppedDecoding(QString,QString))        , this , SLOT(slotStoppedDecoding(QString,QString))    );
+      /**/connect(&interface  , SIGNAL(startedDecoding(QString,QString,int,int,bool)), this , SLOT(slotStartedDecoding(QString,QString))    );
+      /**/connect(&interface  , SIGNAL(stoppedDecoding(QString,QString,bool))        , this , SLOT(slotStoppedDecoding(QString,QString))    );
       #endif
       /*                                                                                                                           */
 
@@ -160,8 +161,6 @@ void CallModel::initRoles()
    roles.insert(Call::Role::Department    ,QByteArray("department"));
    roles.insert(Call::Role::Email         ,QByteArray("email"));
    roles.insert(Call::Role::Organisation  ,QByteArray("organisation"));
-   roles.insert(Call::Role::Codec         ,QByteArray("codec"));
-   roles.insert(Call::Role::IsConference  ,QByteArray("isConference"));
    roles.insert(Call::Role::Object        ,QByteArray("object"));
    roles.insert(Call::Role::PhotoPtr      ,QByteArray("photoPtr"));
    roles.insert(Call::Role::CallState     ,QByteArray("callState"));
@@ -267,25 +266,48 @@ Call* CallModel::getCall( const QString& callId ) const
 ///Add a call in the model structure, the call must exist before being added to the model
 Call* CallModel::addCall(Call* call, Call* parentCall)
 {
-   if (!call || call->state() == Call::State::OVER || (parentCall && parentCall->state() == Call::State::OVER))
-      return new Call(QString(),QString()); //Invalid, but better than managing NULL everywhere
+   //The current History implementation doesn't support conference
+   //if something try to add an history conference, something went wrong
+   if (!call
+    || ((parentCall && parentCall->lifeCycleState() == Call::LifeCycleState::FINISHED)
+    && (call->lifeCycleState() == Call::LifeCycleState::FINISHED))) {
 
+      qWarning() << "Trying to add an invalid call to the tree" << call;
+      Q_ASSERT(false);
+
+      //WARNING this will trigger an assert later on, but isn't critical enough in release mode.
+      //HACK This return an invalid object that should be equivalent to NULL but wont require
+      //nullptr check everywhere in the code. It is safer to use an invalid object rather than
+      //causing a NULL dereference
+      return new Call(QString(),QString());
+   }
+   if (m_sPrivateCallList_call[call]) {
+      qWarning() << "Trying to add a call that already have been added" << call;
+      Q_ASSERT(false);
+   }
+
+   //Even history call currently need to be tracked in CallModel, this may change
    InternalStruct* aNewStruct = new InternalStruct;
    aNewStruct->call_real  = call;
    aNewStruct->conference = false;
 
    m_sPrivateCallList_call  [ call       ] = aNewStruct;
-   if (call->state() != Call::State::OVER)
+   if (call->lifeCycleState() != Call::LifeCycleState::FINISHED) {
+      beginInsertRows(QModelIndex(),m_lInternalModel.size(),m_lInternalModel.size());
       m_lInternalModel << aNewStruct;
+      endInsertRows();
+   }
    m_sPrivateCallList_callId[ call->id() ] = aNewStruct;
 
-   if (call->state() != Call::State::OVER)
+   //If the call is already finished, there is no point to track it here
+   if (call->lifeCycleState() != Call::LifeCycleState::FINISHED) {
       emit callAdded(call,parentCall);
-   const QModelIndex idx = index(m_lInternalModel.size()-1,0,QModelIndex());
-   emit dataChanged(idx, idx);
-   connect(call,SIGNAL(changed(Call*)),this,SLOT(slotCallChanged(Call*)));
-   connect(call,SIGNAL(dtmfPlayed(QString)),this,SLOT(slotDTMFPlayed(QString)));
-   emit layoutChanged();
+      const QModelIndex idx = index(m_lInternalModel.size()-1,0,QModelIndex());
+      emit dataChanged(idx, idx);
+      connect(call,SIGNAL(changed(Call*)),this,SLOT(slotCallChanged(Call*)));
+      connect(call,SIGNAL(dtmfPlayed(QString)),this,SLOT(slotDTMFPlayed(QString)));
+      emit layoutChanged();
+   }
    return call;
 } //addCall
 
@@ -327,32 +349,55 @@ Call* CallModel::addRingingCall(const QString& callId)
    return addCall(Call::buildRingingCall(callId));
 }
 
+///Properly remove an internal from the Qt model
+void CallModel::removeInternal(InternalStruct* internal)
+{
+   if (!internal) return;
+
+   const int idx = m_lInternalModel.indexOf(internal);
+   //Exit if the call is not found
+   if (idx == -1) {
+      qDebug() << "Cannot remove " << internal->call_real << ": call not found in tree";
+      return;
+   }
+
+   //Using layoutChanged would SEGFAULT when an editor is open
+   beginRemoveRows(QModelIndex(),idx,idx);
+   m_lInternalModel.removeAt(idx);
+   endRemoveRows();
+}
+
 ///Remove a call and update the internal structure
 void CallModel::removeCall(Call* call, bool noEmit)
 {
+   Q_UNUSED(noEmit)
    InternalStruct* internal = m_sPrivateCallList_call[call];
 
    if (!internal || !call) {
-      qDebug() << "Cannot remove call: call not found";
+      qDebug() << "Cannot remove " << internal->call_real << ": call not found";
       return;
    }
 
    if (m_sPrivateCallList_call[call] != nullptr) {
-      m_lInternalModel.removeAll(m_sPrivateCallList_call[call]);
+      removeInternal(m_sPrivateCallList_call[call]);
       //NOTE Do not free the memory, it can still be used elsewhere or in modelindexes
    }
 
+   //TODO DEAD CODE Is this really required?, so far multi conference fail without
    if (m_sPrivateCallList_callId[m_sPrivateCallList_callId.key(internal)] == internal) {
       m_sPrivateCallList_callId.remove(m_sPrivateCallList_callId.key(internal));
    }
 
-   m_lInternalModel.removeAll(internal);
+   removeInternal(internal);
 
-   //Restore calls to the main list if they are not rely over
+   //Restore calls to the main list if they are not really over
    if (internal->m_lChildren.size()) {
       foreach(InternalStruct* child,internal->m_lChildren) {
-         if (child->call_real->state() != Call::State::OVER)
+         if (child->call_real->state() != Call::State::OVER && child->call_real->state() != Call::State::ERROR) {
+            beginInsertRows(QModelIndex(),m_lInternalModel.size(),m_lInternalModel.size());
             m_lInternalModel << child;
+            endInsertRows();
+         }
       }
    }
 
@@ -362,10 +407,14 @@ void CallModel::removeCall(Call* call, bool noEmit)
 
    //The daemon often fail to emit the right signal, cleanup manually
    foreach(InternalStruct* topLevel, m_lInternalModel) {
-      if (topLevel->call_real->isConference() && !topLevel->m_lChildren.size())
-         removeConference(topLevel->call_real);
+      if (topLevel->call_real->type() == Call::Type::CONFERENCE &&
+         (!topLevel->m_lChildren.size()
+            //HACK Make a simple validation to prevent ERROR->ERROR->ERROR state loop for conferences
+            || topLevel->m_lChildren.first()->call_real->state() == Call::State::ERROR
+            || topLevel->m_lChildren.last() ->call_real->state() == Call::State::ERROR))
+            removeConference(topLevel->call_real);
    }
-   if (!noEmit)
+//    if (!noEmit)
       emit layoutChanged();
 } //removeCall
 
@@ -445,14 +494,16 @@ Call* CallModel::addConference(const QString& confID)
 
       m_sPrivateCallList_call[newConf]  = aNewStruct;
       m_sPrivateCallList_callId[confID] = aNewStruct;
+      beginInsertRows(QModelIndex(),m_lInternalModel.size(),m_lInternalModel.size());
       m_lInternalModel << aNewStruct;
+      endInsertRows();
 
       foreach(const QString& callId,callList) {
          InternalStruct* callInt = m_sPrivateCallList_callId[callId];
          if (callInt) {
             if (callInt->m_pParent && callInt->m_pParent != aNewStruct)
                callInt->m_pParent->m_lChildren.removeAll(callInt);
-            m_lInternalModel.removeAll(callInt);
+            removeInternal(callInt);
             callInt->m_pParent = aNewStruct;
             callInt->call_real->setProperty("dropState",0);
             if (aNewStruct->m_lChildren.indexOf(callInt) == -1)
@@ -483,8 +534,8 @@ bool CallModel::createConferenceFromCall(Call* call1, Call* call2)
 ///Add a new participant to a conference
 bool CallModel::addParticipant(Call* call2, Call* conference)
 {
-   if (conference->isConference()) {
-      Q_NOREPLY DBus::CallManager::instance().addParticipant(call2->id(), conference->confId());
+   if (conference->type() == Call::Type::CONFERENCE) {
+      Q_NOREPLY DBus::CallManager::instance().addParticipant(call2->id(), conference->id());
       return true;
    }
    else {
@@ -503,7 +554,7 @@ bool CallModel::detachParticipant(Call* call)
 ///Merge two conferences
 bool CallModel::mergeConferences(Call* conf1, Call* conf2)
 {
-   Q_NOREPLY DBus::CallManager::instance().joinConference(conf1->confId(),conf2->confId());
+   Q_NOREPLY DBus::CallManager::instance().joinConference(conf1->id(),conf2->id());
    return true;
 }
 
@@ -604,10 +655,6 @@ QVariant CallModel::data( const QModelIndex& idx, int role) const
       if (intList->conference == true && intList->m_lChildren.size() > idx.row() && intList->m_lChildren[idx.row()])
          call = intList->m_lChildren[idx.row()]->call_real;
    }
-   if (!call) {
-//       qWarning() << "Call not found" << idx.row() << idx.isValid() << m_lInternalModel.size() << idx.parent().isValid() << idx.parent().row() << rowCount(idx.parent());
-      return QVariant();
-   }
    return call?call->roleData((Call::Role)role):QVariant();
 }
 
@@ -628,12 +675,12 @@ int CallModel::rowCount( const QModelIndex& parentIdx ) const
 
    const InternalStruct* modelItem = static_cast<InternalStruct*>(parentIdx.internalPointer());
    if (modelItem) {
-      if (modelItem->call_real->isConference() && modelItem->m_lChildren.size() > 0)
+      if (modelItem->call_real->type() == Call::Type::CONFERENCE && modelItem->m_lChildren.size() > 0)
          return modelItem->m_lChildren.size();
-      else if (modelItem->call_real->isConference())
+      else if (modelItem->call_real->type() == Call::Type::CONFERENCE)
          qWarning() << modelItem->call_real << "have"
             << modelItem->m_lChildren.size() << "and"
-            << (modelItem->call_real->isConference()?"is":"is not") << "a conference";
+            << ((modelItem->call_real->type() == Call::Type::CONFERENCE)?"is":"is not") << "a conference";
    }
    return 0;
 }
@@ -643,10 +690,16 @@ Qt::ItemFlags CallModel::flags( const QModelIndex& idx ) const
 {
    if (!idx.isValid())
       return Qt::NoItemFlags;
-   return Qt::ItemIsEnabled|Qt::ItemIsSelectable 
-      | Qt::ItemIsDragEnabled 
-      | ((!idx.data(Call::Role::IsConference).toBool())?(Qt::ItemIsDropEnabled):Qt::ItemIsEnabled)
-      | ((idx.data(Call::Role::CallState) == static_cast<int>(Call::State::DIALING))?Qt::ItemIsEditable:Qt::NoItemFlags);
+
+   const InternalStruct* modelItem = static_cast<InternalStruct*>(idx.internalPointer());
+   if (modelItem ) {
+      const Call* c = modelItem->call_real;
+      return Qt::ItemIsEnabled|Qt::ItemIsSelectable 
+         | Qt::ItemIsDragEnabled 
+         | ((c->type() != Call::Type::CONFERENCE)?(Qt::ItemIsDropEnabled):Qt::ItemIsEnabled)
+         | ((c->state() == Call::State::DIALING)?Qt::ItemIsEditable:Qt::NoItemFlags);
+   }
+   return Qt::NoItemFlags;
 }
 
 ///Return valid payload types
@@ -660,7 +713,7 @@ int CallModel::columnCount ( const QModelIndex& parentIdx) const
 {
    const InternalStruct* modelItem = static_cast<InternalStruct*>(parentIdx.internalPointer());
    if (modelItem) {
-      return modelItem->call_real->isConference()?1:0;
+      return (modelItem->call_real->type() == Call::Type::CONFERENCE)?1:0;
    }
    else if (parentIdx.isValid())
       return 0;
@@ -762,22 +815,22 @@ bool CallModel::dropMimeData(const QMimeData* mimedata, Qt::DropAction action, i
                return false;
             }
             //Conference dropped on a conference -> merge both conferences
-            else if (call && target && call->isConference() && target->isConference()) {
-               qDebug() << "Merge conferences" << call->confId() << "and" << target->confId();
+            else if (call && target && call->type() == Call::Type::CONFERENCE && target->type() == Call::Type::CONFERENCE) {
+               qDebug() << "Merge conferences" << call->id() << "and" << target->id();
                mergeConferences(call,target);
                return true;
             }
             //Conference dropped on a call part of a conference -> merge both conferences
-            else if (call && call->isConference() && targetIdx.parent().isValid()) {
-               qDebug() << "Merge conferences" << call->confId() << "and" << targetIdx.parent().data(Call::Role::Id).toString();
+            else if (call && call->type() == Call::Type::CONFERENCE && targetIdx.parent().isValid()) {
+               qDebug() << "Merge conferences" << call->id() << "and" << targetIdx.parent().data(Call::Role::Id).toString();
                mergeConferences(call,getCall(targetIdx.parent()));
                return true;
             }
             //Drop a call on a conference -> add it to the conference
-            else if (target && (targetIdx.parent().isValid() || target->isConference())) {
-               Call* conf = target->isConference()?target:qvariant_cast<Call*>(targetIdx.parent().data(Call::Role::Object));
+            else if (target && (targetIdx.parent().isValid() || target->type() == Call::Type::CONFERENCE)) {
+               Call* conf = target->type() == Call::Type::CONFERENCE?target:qvariant_cast<Call*>(targetIdx.parent().data(Call::Role::Object));
                if (conf) {
-                  qDebug() << "Adding call " << call->id() << "to conference" << conf->confId();
+                  qDebug() << "Adding call " << call->id() << "to conference" << conf->id();
                   addParticipant(call,conf);
                return true;
                }
@@ -819,7 +872,7 @@ bool CallModel::dropMimeData(const QMimeData* mimedata, Qt::DropAction action, i
       qDebug() << "Contact" << encodedContact << "on call" << target;
       if (PhoneNumberSelector::defaultVisitor()) {
          const PhoneNumber* number = PhoneNumberSelector::defaultVisitor()->getNumber(
-            Call::contactBackend()->getContactByUid(encodedContact));
+         ContactModel::instance()->getContactByUid(encodedContact));
          if (!number->uri().isEmpty()) {
             Call* newCall = dialingCall();
             newCall->setDialNumber(number);
@@ -844,16 +897,16 @@ bool CallModel::dropMimeData(const QMimeData* mimedata, Qt::DropAction action, i
  ****************************************************************************/
 
 ///When a call state change
-void CallModel::slotCallStateChanged(const QString& callID, const QString& state)
+void CallModel::slotCallStateChanged(const QString& callID, const QString& stateName)
 {
    //This code is part of the CallModel interface too
-   qDebug() << "Call State Changed for call  " << callID << " . New state : " << state;
+   qDebug() << "Call State Changed for call  " << callID << " . New state : " << stateName;
    InternalStruct* internal = m_sPrivateCallList_callId[callID];
    Call* call = nullptr;
    Call::State previousState = Call::State::RINGING;
    if(!internal) {
       qDebug() << "Call not found";
-      if(state == Call::StateChange::RINGING) {
+      if(stateName == Call::StateChange::RINGING) {
          call = addRingingCall(callID);
       }
       else {
@@ -865,18 +918,19 @@ void CallModel::slotCallStateChanged(const QString& callID, const QString& state
       call = internal->call_real;
       previousState = call->state();
       qDebug() << "Call found" << call << call->state();
-      const Call::State oldState = call->state();
-      call->stateChanged(state);
-      //The second clause isn't required, but it does happen when errors happen
-      //if the error is reported afterwards, the call may be forgotten 2in this list
-      if (state == Call::StateChange::HUNG_UP
-         || (oldState != Call::State::OVER && call->state() == Call::State::OVER)
-         || call->state() == Call::State::ERROR) {
+      const Call::LifeCycleState oldLifeCycleState = call->lifeCycleState();
+      const Call::State          oldState          = call->state();
+      call->stateChanged(stateName);
+      //Remove call when they end normally, keep errors and failure one
+      if ((stateName == Call::StateChange::HUNG_UP)
+         || ((oldState == Call::State::OVER) && (call->state() == Call::State::OVER))
+         || (oldLifeCycleState != Call::LifeCycleState::FINISHED && call->state() == Call::State::OVER)) {
          removeCall(call);
       }
    }
 
-   if (call->state() == Call::State::OVER) {
+   //Add to history
+   if (call->lifeCycleState() == Call::LifeCycleState::FINISHED) {
       HistoryModel::instance()->add(call);
    }
 
@@ -926,10 +980,12 @@ void CallModel::slotChangingConference(const QString &confID, const QString& sta
 
       //First remove old participants, add them back to the top level list
       foreach(InternalStruct* child,confInt->m_lChildren) {
-         if (participants.indexOf(child->call_real->id()) == -1 && child->call_real->state() != Call::State::OVER) {
+         if (participants.indexOf(child->call_real->id()) == -1 && child->call_real->lifeCycleState() != Call::LifeCycleState::FINISHED) {
             qDebug() << "Remove" << child->call_real << "from" << conf;
             child->m_pParent = nullptr;
+            beginInsertRows(QModelIndex(),m_lInternalModel.size(),m_lInternalModel.size());
             m_lInternalModel << child;
+            endInsertRows();
             const QModelIndex idx = getIndex(child->call_real);
          }
       }
@@ -939,7 +995,7 @@ void CallModel::slotChangingConference(const QString &confID, const QString& sta
          if (callInt) {
             if (callInt->m_pParent && callInt->m_pParent != confInt)
                callInt->m_pParent->m_lChildren.removeAll(callInt);
-            m_lInternalModel.removeAll(callInt);
+            removeInternal(callInt);
             callInt->m_pParent = confInt;
             confInt->m_lChildren << callInt;
          }
@@ -950,7 +1006,7 @@ void CallModel::slotChangingConference(const QString &confID, const QString& sta
 
       //The daemon often fail to emit the right signal, cleanup manually
       foreach(InternalStruct* topLevel, m_lInternalModel) {
-         if (topLevel->call_real->isConference() && !topLevel->m_lChildren.size()) {
+         if (topLevel->call_real->type() == Call::Type::CONFERENCE && !topLevel->m_lChildren.size()) {
             removeConference(topLevel->call_real);
          }
       }
@@ -963,7 +1019,7 @@ void CallModel::slotChangingConference(const QString &confID, const QString& sta
          if (callInt) {
             const QString confId = callDetails[Call::DetailsMapFields::CONF_ID];
             if (callInt->m_pParent) {
-               if (!confId.isEmpty()  && callInt->m_pParent->call_real->confId() != confId) {
+               if (!confId.isEmpty()  && callInt->m_pParent->call_real->id() != confId) {
                   qWarning() << "Conference parent mismatch";
                }
                else if (confId.isEmpty() ){
@@ -974,8 +1030,9 @@ void CallModel::slotChangingConference(const QString &confID, const QString& sta
             else if (!confId.isEmpty()) {
                qWarning() << "Found an orphan call";
                InternalStruct* confInt2 = m_sPrivateCallList_callId[confId];
-               if (confInt2 && confInt2->call_real->isConference() && !callInt->call_real->isConference()) {
-                  m_lInternalModel.removeAll(callInt);
+               if (confInt2 && confInt2->call_real->type() == Call::Type::CONFERENCE 
+                && (callInt->call_real->type() != Call::Type::CONFERENCE)) {
+                  removeInternal(callInt);
                   if (confInt2->m_lChildren.indexOf(callInt) == -1)
                      confInt2->m_lChildren << callInt;
                }
@@ -1007,12 +1064,6 @@ void CallModel::slotConferenceRemoved(const QString &confId)
    emit conferenceRemoved(conf);
 }
 
-///When the daemon change the volume
-void CallModel::slotVolumeChanged(const QString & device, double value)
-{
-   emit volumeChanged(device,value);
-}
-
 ///Make the call aware it has a recording
 void CallModel::slotNewRecordingAvail( const QString& callId, const QString& filePath)
 {
@@ -1038,23 +1089,48 @@ void CallModel::slotStoppedDecoding(const QString& callId, const QString& shmKey
 ///Update model if the data change
 void CallModel::slotCallChanged(Call* call)
 {
-   //Transfer is "local" state, it doesn't require the daemon, so it need to be
-   //handled "manually" instead of relying on the backend signals
-   if (call->state() == Call::State::TRANSFERRED) {
-      emit callStateChanged(call, Call::State::TRANSFERRED);
-   }
+   switch(call->state()) {
+      //Transfer is "local" state, it doesn't require the daemon, so it need to be
+      //handled "manually" instead of relying on the backend signals
+      case Call::State::TRANSFERRED:
+         emit callStateChanged(call, Call::State::TRANSFERRED);
+         break;
+      //Same goes for some errors
+      case Call::State::__COUNT:
+      case Call::State::ERROR:
+         removeCall(call);
+         break;
+      //Over can be caused by local events
+      case Call::State::OVER:
+         removeCall(call);
+         break;
+      //Let the daemon handle the others
+      case Call::State::INCOMING:
+      case Call::State::RINGING:
+      case Call::State::INITIALIZATION:
+      case Call::State::CURRENT:
+      case Call::State::DIALING:
+      case Call::State::HOLD:
+      case Call::State::FAILURE:
+      case Call::State::BUSY:
+      case Call::State::TRANSF_HOLD:
+      case Call::State::CONFERENCE:
+      case Call::State::CONFERENCE_HOLD:
+         break;
+   };
+
    InternalStruct* callInt = m_sPrivateCallList_call[call];
    if (callInt) {
       const QModelIndex idx = getIndex(call);
-      emit dataChanged(idx,idx);
+      if (idx.isValid())
+         emit dataChanged(idx,idx);
    }
-   /*if (call->state() == Call::State::ERROR && !call->isConference()) {
-      removeCall(call);
-   }*/
 }
 
 ///Add call slot
 void CallModel::slotAddPrivateCall(Call* call) {
+   if (m_sPrivateCallList_call[call])
+      return;
    addCall(call,nullptr);
 }
 
