@@ -29,6 +29,8 @@
 #include "callmodel.h"
 #include "historytimecategorymodel.h"
 #include "lastusednumbermodel.h"
+#include "abstractitembackend.h"
+#include "visitors/itemmodelstateserializationvisitor.h"
 
 /*****************************************************************************
  *                                                                           *
@@ -50,7 +52,8 @@ HistoryModel* HistoryModel::m_spInstance    = nullptr;
 CallMap       HistoryModel::m_sHistoryCalls          ;
 
 HistoryModel::TopLevelItem::TopLevelItem(const QString& name, int index) : 
-   CategorizedCompositeNode(CategorizedCompositeNode::Type::TOP_LEVEL),QObject(nullptr),m_Index(index),m_NameStr(name)
+   CategorizedCompositeNode(CategorizedCompositeNode::Type::TOP_LEVEL),QObject(nullptr),m_Index(index),m_NameStr(name),
+   m_AbsoluteIndex(-1),modelRow(-1)
 {}
 
 HistoryModel::TopLevelItem::~TopLevelItem() {
@@ -67,7 +70,8 @@ QObject* HistoryModel::TopLevelItem::getSelf() const
    return const_cast<HistoryModel::TopLevelItem*>(this);
 }
 
-HistoryModel::HistoryItem::HistoryItem(Call* call) : CategorizedCompositeNode(CategorizedCompositeNode::Type::CALL),m_pCall(call)
+HistoryModel::HistoryItem::HistoryItem(Call* call) : CategorizedCompositeNode(CategorizedCompositeNode::Type::CALL),m_pCall(call),
+m_Index(0),m_pParent(nullptr),m_pNode(nullptr)
 {
    
 }
@@ -96,24 +100,9 @@ Call* HistoryModel::HistoryItem::call() const
  ****************************************************************************/
 
 ///Constructor
-HistoryModel::HistoryModel():QAbstractItemModel(QCoreApplication::instance()),m_HistoryInit(false),m_Role(Call::Role::FuzzyDate),m_HaveContactModel(false)
+HistoryModel::HistoryModel():QAbstractItemModel(QCoreApplication::instance()),m_Role(Call::Role::FuzzyDate)
 {
-   ConfigurationManagerInterface& configurationManager = DBus::ConfigurationManager::instance();
-   const QVector< QMap<QString, QString> > history = configurationManager.getHistory();
-   beginResetModel();
-   for(int i = history.size()-1;i>=0;i--) {
-      const MapStringString& hc = history[i];
-      Call* pastCall = Call::buildHistoryCall(hc);
-      if (pastCall->peerName().isEmpty()) {
-         pastCall->setPeerName(tr("Unknown"));
-      }
-      pastCall->setRecordingPath(hc[ Call::HistoryMapFields::RECORDING_PATH ]);
-      add(pastCall);
-   }
-   endResetModel();
-   m_HistoryInit = true;
    m_spInstance  = this;
-//    reloadCategories();
    m_lMimes << MIME_PLAIN_TEXT << MIME_PHONENUMBER << MIME_HISTORYID;
    QHash<int, QByteArray> roles = roleNames();
    roles.insert(Call::Role::Name          ,QByteArray("name"          ));
@@ -131,8 +120,6 @@ HistoryModel::HistoryModel():QAbstractItemModel(QCoreApplication::instance()),m_
    roles.insert(Call::Role::Department    ,QByteArray("department"    ));
    roles.insert(Call::Role::Email         ,QByteArray("email"         ));
    roles.insert(Call::Role::Organisation  ,QByteArray("organisation"  ));
-   roles.insert(Call::Role::Codec         ,QByteArray("codec"         ));
-   roles.insert(Call::Role::IsConference  ,QByteArray("isConference"  ));
    roles.insert(Call::Role::Object        ,QByteArray("object"        ));
    roles.insert(Call::Role::PhotoPtr      ,QByteArray("photoPtr"      ));
    roles.insert(Call::Role::CallState     ,QByteArray("callState"     ));
@@ -204,17 +191,23 @@ HistoryModel::TopLevelItem* HistoryModel::getCategory(const Call* call)
    return category;
 }
 
+
+const CallMap HistoryModel::getHistoryCalls() const
+{
+   return m_sHistoryCalls;
+}
+
 ///Add to history
 void HistoryModel::add(Call* call)
 {
-   if (!call || call->state() != Call::State::OVER || !call->startTimeStamp()) {
+   if (!call || call->lifeCycleState() != Call::LifeCycleState::FINISHED || !call->startTimeStamp()) {
       return;
    }
 
-   if (!m_HaveContactModel && call->contactBackend()) {
-      connect(((QObject*)call->contactBackend()),SIGNAL(collectionChanged()),this,SLOT(reloadCategories()));
-      m_HaveContactModel = true;
-   }
+//    if (!m_HaveContactModel && call->contactBackend()) {
+//       connect(((QObject*)call->contactBackend()),SIGNAL(collectionChanged()),this,SLOT(reloadCategories()));
+//       m_HaveContactModel = true;
+//    }//TODO implement reordering
 
    emit newHistoryCall(call);
    emit layoutAboutToBeChanged();
@@ -235,6 +228,19 @@ void HistoryModel::add(Call* call)
    emit layoutChanged();
    LastUsedNumberModel::instance()->addCall(call);
    emit historyChanged();
+
+   // Loop until it find a compatible backend
+   //HACK only support a single active history backend
+   if (!call->backend()) {
+      foreach (AbstractHistoryBackend* backend,m_lBackends) {
+         if (backend->supportedFeatures() & AbstractHistoryBackend::ADD) {
+            if (backend->append(call)) {
+               call->setBackend(backend);
+               break;
+            }
+         }
+      }
+   }
 }
 
 ///Set if the history has a limit
@@ -271,8 +277,6 @@ int HistoryModel::historyLimit() const
 
 void HistoryModel::reloadCategories()
 {
-   if (!m_HistoryInit)
-      return;
    beginResetModel();
    m_hCategories.clear();
    m_hCategoryByName.clear();
@@ -280,7 +284,6 @@ void HistoryModel::reloadCategories()
       delete item;
    }
    m_lCategoryCounter.clear();
-   m_isContactDateInit = false;
    foreach(Call* call, m_sHistoryCalls) {
       TopLevelItem* category = getCategory(call);
       if (category) {
@@ -497,8 +500,66 @@ bool HistoryModel::dropMimeData(const QMimeData *mime, Qt::DropAction action, in
    return false;
 }
 
+
+bool HistoryModel::hasBackends() const
+{
+   return m_lBackends.size();
+}
+
+bool HistoryModel::hasEnabledBackends() const
+{
+   return m_lBackends.size();
+}
+
+void HistoryModel::addBackend(AbstractHistoryBackend* backend, LoadOptions options)
+{
+   m_lBackends << backend;
+   connect(backend,SIGNAL(newHistoryCallAdded(Call*)),this,SLOT(add(Call*)));
+   if (options & LoadOptions::FORCE_ENABLED || ItemModelStateSerializationVisitor::instance()->isChecked(backend))
+      backend->load();
+   emit newBackendAdded(backend);
+}
+
+///Call all backends that support clearing
+void HistoryModel::clearAllBackends() const
+{
+   foreach (AbstractHistoryBackend* backend,m_lBackends) {
+      if (backend->supportedFeatures() & AbstractHistoryBackend::ADD) {
+         backend->clear();
+      }
+   }
+
+   //TODO Remove this
+   //Clear the daemon history backend as apparently the Gnome client wont
+   //Use its native backend anytime soon
+   DBus::ConfigurationManager::instance().clearHistory();
+}
+
+
+bool HistoryModel::enableBackend(AbstractHistoryBackend* backend, bool enabled)
+{
+   Q_UNUSED(backend)
+   Q_UNUSED(enabled)
+   return false;//TODO
+}
+
+CommonItemBackendModel* HistoryModel::backendModel() const
+{
+   return nullptr; //TODO
+}
+
+const QVector<AbstractHistoryBackend*> HistoryModel::backends() const
+{
+   return m_lBackends;
+}
+
+const QVector<AbstractHistoryBackend*> HistoryModel::enabledBackends() const
+{
+   return m_lBackends;
+}
+
 ///Return valid payload types
-int HistoryModel::acceptedPayloadTypes()
+int HistoryModel::acceptedPayloadTypes() const
 {
    return CallModel::DropPayloadType::CALL;
 }
