@@ -23,14 +23,16 @@
 //SFLPhone
 #include "phonenumber.h"
 #include "call.h"
+#include "uri.h"
 #include "account.h"
 #include "contact.h"
 #include "accountlistmodel.h"
 #include "numbercategory.h"
 #include "numbercategorymodel.h"
-#include "abstractcontactbackend.h"
+#include "abstractitembackend.h"
 #include "dbus/presencemanager.h"
 #include "visitors/pixmapmanipulationvisitor.h"
+#include "contactmodel.h"
 
 PhoneDirectoryModel* PhoneDirectoryModel::m_spInstance = nullptr;
 
@@ -45,6 +47,7 @@ PhoneDirectoryModel::PhoneDirectoryModel(QObject* parent) :
 PhoneDirectoryModel::~PhoneDirectoryModel()
 {
    QList<NumberWrapper*> vals = m_hNumbersByNames.values();
+   //Used by indexes
    m_hNumbersByNames.clear();
    m_lSortedNames.clear();
    while (vals.size()) {
@@ -52,6 +55,8 @@ PhoneDirectoryModel::~PhoneDirectoryModel()
       vals.removeAt(0);
       delete w;
    }
+
+   //Used by auto completion
    vals = m_hSortedNumbers.values();
    m_hSortedNumbers.clear();
    m_hDirectory.clear();
@@ -278,13 +283,134 @@ QVariant PhoneDirectoryModel::headerData(int section, Qt::Orientation orientatio
    return QVariant();
 }
 
+/**
+ * This helper method make sure that number without an account get registered
+ * correctly with their alternate URIs. In case there is an obvious duplication,
+ * it will try to merge both numbers.
+ */
+void PhoneDirectoryModel::setAccount(PhoneNumber* number, Account* account ) {
+   const URI& strippedUri = number->uri();
+   const bool hasAtSign = strippedUri.hasHostname();
+   number->setAccount(account);
+
+   if (!hasAtSign) {
+      NumberWrapper* wrap = m_hDirectory[strippedUri];
+
+      //Let make sure none is created in the future for nothing
+      if (!wrap) {
+         //It wont be a duplicate as none exist for this URI
+         const QString extendedUri = strippedUri+'@'+account->hostname();
+         wrap = new NumberWrapper();
+         m_hDirectory    [extendedUri] = wrap;
+         m_hSortedNumbers[extendedUri] = wrap;
+
+      }
+      else {
+         //After all this, it is possible the number is now a duplicate
+         foreach(PhoneNumber* n, wrap->numbers) {
+            if (n != number && n->account() && n->account() == number->account()) {
+               number->merge(n);
+            }
+         }
+      }
+      wrap->numbers << number;
+
+   }
+}
+
+/**
+ * This version of getNumber() try to get a phone number with a contact from an URI and account
+ * It will also try to attach an account to existing numbers. This is not 100% reliable, but
+ * it is correct often enough to do it.
+ */
+PhoneNumber* PhoneDirectoryModel::getNumber(const QString& uri, Account* account, const QString& type)
+{
+   return getNumber(uri,nullptr,account,type);
+}
+
+///Add new information to existing numbers and try to merge
+PhoneNumber* PhoneDirectoryModel::fillDetails(NumberWrapper* wrap, const URI& strippedUri, Account* account, Contact* contact, const QString& type)
+{
+   //TODO pick the best URI
+   //TODO the account hostname change corner case
+   //TODO search for account that has the same hostname as the URI
+   if (wrap) {
+      foreach(PhoneNumber* number, wrap->numbers) {
+
+         //BEGIN Check if contact can be set
+
+         //Check if the contact is compatible
+         const bool hasCompatibleContact = contact && (
+               (!number->contact())
+            || (
+                  (number->contact()->uid() == contact->uid())
+               && number->contact() != contact
+            )
+         );
+
+         //Check if the URI match
+         const bool hasCompatibleURI =  hasCompatibleContact && (number->uri().hasHostname()?(
+               /* Has hostname */
+                   strippedUri == number->uri()
+                   /* Something with an hostname can be used with IP2IP */ //TODO support DHT here
+               || (account && account->id() == Account::ProtocolName::IP2IP)
+            ) : ( /* Has no hostname */
+                  number->account() && number->uri()+'@'+number->account()->hostname() == strippedUri
+            ));
+
+         //Check if the account is compatible
+         const bool hasCompatibleAccount = hasCompatibleURI && ((!account)
+            || (!number->account())
+            /* Direct match, this is always valid */
+            || (account == number->account())
+            /* IP2IP is a special case */ //TODO support DHT here
+            || (
+                  account->id() == Account::ProtocolName::IP2IP
+                  && strippedUri.hasHostname()
+               ));
+
+         //TODO the Display name could be used to influence the choice
+         //It would need to ignore all possible translated values of unknown
+         //and only be available when another information match
+
+         //If everything match, set the contact
+         if (hasCompatibleAccount)
+            number->setContact(contact);
+         //END Check if the contact can be set
+
+
+         //BEGIN Check is account can be set
+         // Try to match the account
+         // Not perfect, but better than ignoring the high probabilities
+         //TODO only do it is hostname match
+         if ((!account) || account != number->account()) {
+
+            if (account && (!contact) && !number->account())
+               setAccount(number,account);
+
+            //Set a type, this has low probabilities of being invalid
+            if ((!number->hasType()) && (!type.isEmpty())) {
+               number->setCategory(NumberCategoryModel::instance()->getCategory(type));
+            }
+
+            //We already have enough information to confirm the choice
+            if (contact && number->contact() &&((contact->uid()) == number->contact()->uid()))
+               return number;
+         }
+         //END Check is account can be set
+      }
+   }
+   return nullptr;
+}
+
+///Return/create a number when no information is available
 PhoneNumber* PhoneDirectoryModel::getNumber(const QString& uri, const QString& type)
 {
-   const QString strippedUri =  PhoneNumber::stripUri(uri);
+   const URI strippedUri(uri);
    NumberWrapper* wrap = m_hDirectory[strippedUri];
    if (wrap) {
       PhoneNumber* nb = wrap->numbers[0];
-      if ((!nb->m_hasType) && (!type.isEmpty())) {
+      if ((!nb->hasType()) && (!type.isEmpty())) {
          nb->setCategory(NumberCategoryModel::instance()->getCategory(type));
       }
       return nb;
@@ -292,17 +418,12 @@ PhoneNumber* PhoneDirectoryModel::getNumber(const QString& uri, const QString& t
 
    //Too bad, lets create one
    PhoneNumber* number = new PhoneNumber(strippedUri,NumberCategoryModel::instance()->getCategory(type));
+   number->setIndex(m_lNumbers.size());
+   m_lNumbers << number;
    connect(number,SIGNAL(callAdded(Call*)),this,SLOT(slotCallAdded(Call*)));
    connect(number,SIGNAL(changed()),this,SLOT(slotChanged()));
-   number->m_Index = m_lNumbers.size();
-   m_lNumbers << number;
 
-   const QString hn = number->hostname();
-
-   //Check if we are lucky enough to have a single registered account with the same hostname
-   /*if (!hn.isEmpty()) {
-      
-   }*/
+   const QString hn = number->uri().hostname();
 
    emit layoutChanged();
    if (!wrap) {
@@ -314,83 +435,123 @@ PhoneNumber* PhoneDirectoryModel::getNumber(const QString& uri, const QString& t
    return number;
 }
 
-PhoneNumber* PhoneDirectoryModel::getNumber(const QString& uri, Account* account, const QString& type)
-{
-   const QString strippedUri =  PhoneNumber::stripUri(uri);
-   //See if the number is already loaded
-   NumberWrapper* wrap = m_hDirectory[strippedUri];
-   if ((!wrap) && account && uri.indexOf('@') == -1) {
-      //Append the account hostname
-      wrap = m_hDirectory[strippedUri+'@'+account->hostname()];
-   }
-   if (wrap) {
-      foreach(PhoneNumber* number, wrap->numbers) {
-         //Not perfect, but better than ignoring the high probabilities
-         if (!number->account())
-            number->setAccount(account);
-         if ((!number->m_hasType) && (!type.isEmpty())) {
-            number->setCategory(NumberCategoryModel::instance()->getCategory(type));
-         }
-         if ((!account) || number->account() == account)
-            return number;
-      }
-   }
-
-   //Create the number
-   PhoneNumber* number = new PhoneNumber(strippedUri,NumberCategoryModel::instance()->getCategory(type));
-   connect(number,SIGNAL(callAdded(Call*)),this,SLOT(slotCallAdded(Call*)));
-   connect(number,SIGNAL(changed()),this,SLOT(slotChanged()));
-   number->setAccount(account);
-   number->m_Index = m_lNumbers.size();
-   m_lNumbers << number;
-   if (!wrap) {
-      wrap = new NumberWrapper();
-      m_hDirectory[strippedUri] = wrap;
-      m_hSortedNumbers[strippedUri] = wrap;
-   }
-   wrap->numbers << number;
-   emit layoutChanged();
-   return number;
-}
-
+///Create a number when a more information is available duplicated ones
 PhoneNumber* PhoneDirectoryModel::getNumber(const QString& uri, Contact* contact, Account* account, const QString& type)
 {
-   const QString strippedUri =  PhoneNumber::stripUri(uri);
+   //Remove extra data such as "<sip:" from the main URI
+   const URI strippedUri(uri);
+
    //See if the number is already loaded
-   NumberWrapper* wrap = m_hDirectory[strippedUri];
+   NumberWrapper* wrap  = m_hDirectory[strippedUri];
+   NumberWrapper* wrap2 = nullptr;
+   NumberWrapper* wrap3 = nullptr;
+
+   //Check if the URI is complete or short
+   const bool hasAtSign = strippedUri.hasHostname();
+
+   //Try to see if there is a better candidate with a suffix (LAN only)
+   if ( !hasAtSign && account ) {
+      //Append the account hostname
+      wrap2 = m_hDirectory[strippedUri+'@'+account->hostname()];
+   }
+
+   //Check
+   PhoneNumber* confirmedCandidate = fillDetails(wrap,strippedUri,account,contact,type);
+
+   //URIs can be represented in multiple way, check if a more verbose version
+   //already exist
+   PhoneNumber* confirmedCandidate2 = nullptr;
+
+   //Try to use a PhoneNumber with a contact when possible, work only after the
+   //contact are loaded
+   if (confirmedCandidate && confirmedCandidate->contact())
+      confirmedCandidate2 = fillDetails(wrap2,strippedUri,account,contact,type);
+
+   PhoneNumber* confirmedCandidate3 = nullptr;
+
+   //Else, try to see if the hostname correspond to the account and flush it
+   //This have to be done after the parent if as the above give "better"
+   //results. It cannot be merged with wrap2 as this check only work if the
+   //candidate has an account.
+   if (hasAtSign && account && strippedUri.hostname() == account->hostname()) {
+     wrap3 = m_hDirectory[strippedUri.userinfo()];
+     if (wrap3) {
+         foreach(PhoneNumber* number, wrap3->numbers) {
+            if (number->account() == account) {
+               if (contact && ((!number->contact()) || (contact->uid() == number->contact()->uid())))
+                  number->setContact(contact); //TODO Check all cases from fillDetails()
+               //TODO add alternate URI
+               confirmedCandidate3 = number;
+               break;
+            }
+         }
+     }
+   }
+
+   //If multiple PhoneNumber are confirmed, then they are the same, merge them
+   if (confirmedCandidate3 && (confirmedCandidate || confirmedCandidate2)) {
+      confirmedCandidate3->merge(confirmedCandidate?confirmedCandidate:confirmedCandidate2);
+   }
+   else if (confirmedCandidate && confirmedCandidate2) {
+      if (confirmedCandidate->contact() && !confirmedCandidate2->contact())
+         confirmedCandidate2->merge(confirmedCandidate);
+      else if (confirmedCandidate2->contact() && !confirmedCandidate->contact())
+         confirmedCandidate->merge(confirmedCandidate2);
+   }
+
+   //Empirical testing resulted in this as the best return order
+   //The merge may have failed either in the "if" above or in the merging code
+   if (confirmedCandidate2)
+      return confirmedCandidate2;
+   if (confirmedCandidate)
+      return confirmedCandidate;
+   if (confirmedCandidate3)
+      return confirmedCandidate3;
+
+   //No better candidates were found than the original assumption, use it
    if (wrap) {
-      //TODO find something better, it is prone to collisions
       foreach(PhoneNumber* number, wrap->numbers) {
-         if (!number->contact()) {
-            if (!number->account())
-               number->setAccount(account);
-            number->setContact(contact);
-         }
-      }
-      foreach(PhoneNumber* number, wrap->numbers) {
-         if ((!number->m_hasType) && (!type.isEmpty())) {
-            number->setCategory(NumberCategoryModel::instance()->getCategory(type));
-         }
-         if (((!contact) || number->contact() == contact) && ((!account) || number->account() == account))
+         if (((!account) || number->account() == account) && ((!contact) || ((*contact) == number->contact()) || (!number->contact()))) {
+            //Assume this is valid until a smarter solution is implemented to merge both
+            //For a short time, a placeholder contact and a contact can coexist, drop the placeholder
+            if (contact && (!number->contact() || (contact->uid() == number->contact()->uid())))
+               number->setContact(contact);
+
             return number;
+         }
       }
    }
 
    //Create the number
    PhoneNumber* number = new PhoneNumber(strippedUri,NumberCategoryModel::instance()->getCategory(type));
+   number->setAccount(account);
+   number->setIndex( m_lNumbers.size());
+   if (contact)
+      number->setContact(contact);
+   m_lNumbers << number;
    connect(number,SIGNAL(callAdded(Call*)),this,SLOT(slotCallAdded(Call*)));
    connect(number,SIGNAL(changed()),this,SLOT(slotChanged()));
-   number->setAccount(account);
-   number->setContact(contact);
-   number->m_Index = m_lNumbers.size();
-   m_lNumbers << number;
    if (!wrap) {
       wrap = new NumberWrapper();
-      m_hDirectory[strippedUri] = wrap;
+      m_hDirectory    [strippedUri] = wrap;
       m_hSortedNumbers[strippedUri] = wrap;
+
+      //Also add its alternative URI, it should be safe to do
+      if ( !hasAtSign && account && !account->hostname().isEmpty() ) {
+         const QString extendedUri = strippedUri+'@'+account->hostname();
+         //Also check if it hasn't been created by setAccount
+         if ((!wrap2) && (!m_hDirectory[extendedUri])) {
+            wrap2 = new NumberWrapper();
+            m_hDirectory    [extendedUri] = wrap2;
+            m_hSortedNumbers[extendedUri] = wrap2;
+         }
+         wrap2->numbers << number;
+      }
+
    }
    wrap->numbers << number;
    emit layoutChanged();
+
    return number;
 }
 
@@ -405,7 +566,7 @@ PhoneNumber* PhoneDirectoryModel::fromHash(const QString& hash)
    if (fields.size() == 3) {
       const QString uri = fields[0];
       Account* account = AccountListModel::instance()->getAccountById(fields[1]);
-      Contact* contact = Call::contactBackend()?Call::contactBackend()->getContactByUid(fields[2]):nullptr;
+      Contact* contact = ContactModel::instance()->getContactByUid(fields[2].toUtf8());
       return getNumber(uri,contact,account);
    }
    else if (fields.size() == 1) {
@@ -434,24 +595,24 @@ void PhoneDirectoryModel::slotCallAdded(Call* call)
             PhoneNumber* tmp = m_lPopularityIndex[currentIndex-1];
             m_lPopularityIndex[currentIndex-1] = number;
             m_lPopularityIndex[currentIndex  ] = tmp   ;
-            tmp->m_PopularityIndex++;
+            tmp->setPopularityIndex(tmp->popularityIndex()+1);
             currentIndex--;
          } while (currentIndex && m_lPopularityIndex[currentIndex-1]->callCount() < number->callCount());
-         number->m_PopularityIndex = currentIndex;
+         number->setPopularityIndex(currentIndex);
          emit layoutChanged();
       }
       //The top 10 is not complete, a call count of "1" is enough to make it
       else if (m_lPopularityIndex.size() < 10 && currentIndex == -1) {
          m_lPopularityIndex << number;
-         number->m_PopularityIndex = m_lPopularityIndex.size()-1;
+         number->setPopularityIndex(m_lPopularityIndex.size()-1);
          emit layoutChanged();
       }
       //The top 10 is full, but this number just made it to the top 10
       else if (currentIndex == -1 && m_lPopularityIndex.size() >= 10 && m_lPopularityIndex[9] != number && m_lPopularityIndex[9]->callCount() < number->callCount()) {
          PhoneNumber* tmp = m_lPopularityIndex[9];
-         tmp->m_PopularityIndex    = -1;
+         tmp->setPopularityIndex(-1);
          m_lPopularityIndex[9]     = number;
-         number->m_PopularityIndex = 9;
+         number->setPopularityIndex(9);
          emit tmp->changed();
          emit number->changed();
       }
@@ -467,8 +628,12 @@ void PhoneDirectoryModel::slotChanged()
 {
    PhoneNumber* number = qobject_cast<PhoneNumber*>(sender());
    if (number) {
-      const int idx = number->m_Index;
-      emit dataChanged(index(idx,0),index(idx,5));
+      const int idx = number->index();
+#ifndef NDEBUG
+      if (idx<0)
+         qDebug() << "Invalid slotChanged() index!" << idx;
+#endif
+      emit dataChanged(index(idx,0),index(idx,static_cast<int>(Columns::UID)));
    }
 }
 
