@@ -48,18 +48,25 @@
 //sflphone library
 #include "lib/sflphone_const.h"
 #include "lib/dbus/instancemanager.h"
-#include "lib/dbus/configurationmanager.h"
 #include "lib/contact.h"
 #include "lib/accountlistmodel.h"
 #include "lib/instantmessagingmodel.h"
 #include "lib/numbercategorymodel.h"
+#include "lib/legacyhistorybackend.h"
+#include "klib/minimalhistorybackend.h"
 #include "lib/visitors/numbercategoryvisitor.h"
 #include "klib/macromodel.h"
 #include "klib/akonadibackend.h"
 #include "klib/kcfg_settings.h"
+#include "klib/akonadicontactcollectionmodel.h"
 #include "lib/presencestatusmodel.h"
-#include "lib/videomodel.h"
+#include "lib/video/videomodel.h"
 #include "lib/phonenumber.h"
+#include "lib/contactmodel.h"
+#include "lib/itembackendmodel.h"
+#include "lib/visitors/itemmodelstateserializationvisitor.h"
+#include "klib/itemmodelserialization.h"
+#include "lib/extensions/presenceitembackendmodelextension.h"
 
 //sflphone
 #include "accountwizard.h"
@@ -100,21 +107,21 @@ class ConcreteNumberCategoryVisitor :public NumberCategoryVisitor {
       const bool isEmpty = !list.size();
 #define IS_ENABLED(name) (list.indexOf(name) != -1) || isEmpty
 #define ICN(name) new QPixmap(KStandardDirs::locate("data" , QString("sflphone-client-kde/mini/%1.png").arg(name)))
-      model->addCategory("Home"     ,ICN("home")     , KABC::PhoneNumber::Home ,IS_ENABLED( KABC::PhoneNumber::Home     ));
-      model->addCategory("Work"     ,ICN("work")     , KABC::PhoneNumber::Work ,IS_ENABLED( KABC::PhoneNumber::Work     ));
-      model->addCategory("Msg"      ,ICN("mail")     , KABC::PhoneNumber::Msg  ,IS_ENABLED( KABC::PhoneNumber::Msg      ));
-      model->addCategory("Pref"     ,ICN("call")     , KABC::PhoneNumber::Pref ,IS_ENABLED( KABC::PhoneNumber::Pref     ));
-      model->addCategory("Voice"    ,ICN("video")    , KABC::PhoneNumber::Voice,IS_ENABLED( KABC::PhoneNumber::Voice    ));
-      model->addCategory("Fax"      ,ICN("call")     , KABC::PhoneNumber::Fax  ,IS_ENABLED( KABC::PhoneNumber::Fax      ));
-      model->addCategory("Cell"     ,ICN("mobile")   , KABC::PhoneNumber::Cell ,IS_ENABLED( KABC::PhoneNumber::Cell     ));
-      model->addCategory("Video"    ,ICN("call")     , KABC::PhoneNumber::Video,IS_ENABLED( KABC::PhoneNumber::Video    ));
-      model->addCategory("Bbs"      ,ICN("call")     , KABC::PhoneNumber::Bbs  ,IS_ENABLED( KABC::PhoneNumber::Bbs      ));
-      model->addCategory("Modem"    ,ICN("call")     , KABC::PhoneNumber::Modem,IS_ENABLED( KABC::PhoneNumber::Modem    ));
-      model->addCategory("Car"      ,ICN("car")      , KABC::PhoneNumber::Car  ,IS_ENABLED( KABC::PhoneNumber::Car      ));
-      model->addCategory("Isdn"     ,ICN("call")     , KABC::PhoneNumber::Isdn ,IS_ENABLED( KABC::PhoneNumber::Isdn     ));
-      model->addCategory("Pcs"      ,ICN("call")     , KABC::PhoneNumber::Pcs  ,IS_ENABLED( KABC::PhoneNumber::Pcs      ));
-      model->addCategory("Pager"    ,ICN("pager")    , KABC::PhoneNumber::Pager,IS_ENABLED( KABC::PhoneNumber::Pager    ));
-      model->addCategory("Preferred",ICN("preferred"), 10000                   ,IS_ENABLED( 10000                       ));
+      model->addCategory(i18n("Home")     ,ICN("home")     , KABC::PhoneNumber::Home ,IS_ENABLED( KABC::PhoneNumber::Home     ));
+      model->addCategory(i18n("Work")     ,ICN("work")     , KABC::PhoneNumber::Work ,IS_ENABLED( KABC::PhoneNumber::Work     ));
+      model->addCategory(i18n("Msg")      ,ICN("mail")     , KABC::PhoneNumber::Msg  ,IS_ENABLED( KABC::PhoneNumber::Msg      ));
+      model->addCategory(i18n("Pref")     ,ICN("call")     , KABC::PhoneNumber::Pref ,IS_ENABLED( KABC::PhoneNumber::Pref     ));
+      model->addCategory(i18n("Voice")    ,ICN("video")    , KABC::PhoneNumber::Voice,IS_ENABLED( KABC::PhoneNumber::Voice    ));
+      model->addCategory(i18n("Fax")      ,ICN("call")     , KABC::PhoneNumber::Fax  ,IS_ENABLED( KABC::PhoneNumber::Fax      ));
+      model->addCategory(i18n("Cell")     ,ICN("mobile")   , KABC::PhoneNumber::Cell ,IS_ENABLED( KABC::PhoneNumber::Cell     ));
+      model->addCategory(i18n("Video")    ,ICN("call")     , KABC::PhoneNumber::Video,IS_ENABLED( KABC::PhoneNumber::Video    ));
+      model->addCategory(i18n("Bbs")      ,ICN("call")     , KABC::PhoneNumber::Bbs  ,IS_ENABLED( KABC::PhoneNumber::Bbs      ));
+      model->addCategory(i18n("Modem")    ,ICN("call")     , KABC::PhoneNumber::Modem,IS_ENABLED( KABC::PhoneNumber::Modem    ));
+      model->addCategory(i18n("Car")      ,ICN("car")      , KABC::PhoneNumber::Car  ,IS_ENABLED( KABC::PhoneNumber::Car      ));
+      model->addCategory(i18n("Isdn")     ,ICN("call")     , KABC::PhoneNumber::Isdn ,IS_ENABLED( KABC::PhoneNumber::Isdn     ));
+      model->addCategory(i18n("Pcs")      ,ICN("call")     , KABC::PhoneNumber::Pcs  ,IS_ENABLED( KABC::PhoneNumber::Pcs      ));
+      model->addCategory(i18n("Pager")    ,ICN("pager")    , KABC::PhoneNumber::Pager,IS_ENABLED( KABC::PhoneNumber::Pager    ));
+      model->addCategory(i18n("Preferred"),ICN("preferred"), 10000                   ,IS_ENABLED( 10000                       ));
 #undef ICN
 #undef IS_ENABLED
    }
@@ -137,14 +144,31 @@ SFLPhone::SFLPhone(QWidget *parent)
    }
    static bool init = false;
    if (!init) {
-      Call::setContactBackend(AkonadiBackend::instance());
+
+      //Start the Akonadi collection backend (contact loader)
+      AkonadiContactCollectionModel::instance();
+      HistoryModel::instance()->addBackend(new MinimalHistoryBackend(this),LoadOptions::FORCE_ENABLED);
+
+      // Import all calls from the legacy backend
+      if (ConfigurationSkeleton::requireLegacyHistoryImport()) {
+         HistoryModel::instance()->addBackend(new LegacyHistoryBackend(this),LoadOptions::FORCE_ENABLED);
+         ConfigurationSkeleton::setRequireLegacyHistoryImport(false);
+
+         //In case the client is not quitted correctly, save now
+         ConfigurationSkeleton::self()->writeConfig();
+      }
       NumberCategoryModel::instance()->setVisitor(new ConcreteNumberCategoryVisitor());
+      ItemModelStateSerializationVisitor::setInstance(new ItemModelStateSerialization());
+      ContactModel::instance()->backendModel()->load();
       InstantMessagingModelManager::init();
       AccountListModel::instance()->setDefaultAccount(AccountListModel::instance()->getAccountById(ConfigurationSkeleton::defaultAccountId()));
       #ifdef ENABLE_VIDEO
       VideoModel::instance();
       #endif
       init = true;
+
+      PresenceItemBackendModelExtension* ext = new PresenceItemBackendModelExtension(this);
+      ContactModel::instance()->backendModel()->addExtension(ext);
    }
 
    //Belong to setupActions(), but is needed now
@@ -161,7 +185,7 @@ SFLPhone::SFLPhone(QWidget *parent)
    m_pCentralDW = new QDockWidget(this);
    m_pCentralDW->setSizePolicy  ( QSizePolicy::Expanding,QSizePolicy::Expanding );
    m_pCentralDW->setWidget      ( m_pView                                       );
-   m_pCentralDW->setWindowTitle ( i18n("Call")                                  );
+   m_pCentralDW->setWindowTitle ( i18nc("Call tab","Call")                                  );
    m_pCentralDW->setFeatures    ( QDockWidget::NoDockWidgetFeatures             );
    m_pView->setSizePolicy       ( QSizePolicy::Expanding,QSizePolicy::Expanding );
    m_pCentralDW->setStyleSheet  ( "\
@@ -186,10 +210,10 @@ SFLPhone::SFLPhone(QWidget *parent)
    //System tray
    m_pTrayIcon        = new SFLPhoneTray ( this->windowIcon(), this );
 
-   addDockWidget( Qt::TopDockWidgetArea, m_pCentralDW  );
-   addDockWidget( Qt::TopDockWidgetArea, m_pContactCD  );
-   addDockWidget( Qt::TopDockWidgetArea, m_pHistoryDW  );
-   addDockWidget( Qt::TopDockWidgetArea, m_pBookmarkDW );
+   addDockWidget( Qt::BottomDockWidgetArea, m_pCentralDW  );
+   addDockWidget( Qt::BottomDockWidgetArea, m_pContactCD  );
+   addDockWidget( Qt::BottomDockWidgetArea, m_pHistoryDW  );
+   addDockWidget( Qt::BottomDockWidgetArea, m_pBookmarkDW );
 
    tabifyDockWidget(m_pCentralDW,m_pHistoryDW );
    tabifyDockWidget(m_pCentralDW,m_pContactCD );
@@ -243,7 +267,7 @@ SFLPhone::SFLPhone(QWidget *parent)
 
    move(QCursor::pos().x() - geometry().width()/2, QCursor::pos().y() - geometry().height()/2);
 
-   if (AccountListModel::instance()->size() <= 1)
+   if (AccountListModel::instance()->size() <= 1 && ConfigurationSkeleton::displayAccountWizard())
       (new AccountWizard())->show();
 
    m_pInitialized = true ;
@@ -262,19 +286,21 @@ SFLPhone::SFLPhone(QWidget *parent)
 
    m_pPresent = new QToolButton(bar);
    m_pPresent->setAutoRaise(true);
-   m_pPresent->setText("Online");
+   m_pPresent->setText(i18nc("The presence state is \"Online\"","Online"));
    m_pPresent->setCheckable(true);
+   m_pPresent->setVisible(AccountListModel::instance()->isPresenceEnabled() && AccountListModel::instance()->isPresencePublishSupported());
 //    m_pPresent->setStyleSheet("background-color:red;");
    bar->addWidget(m_pPresent);
    connect(PresenceStatusModel::instance(),SIGNAL(currentNameChanged(QString)),this,SLOT(updatePresence(QString)));
    connect(PresenceStatusModel::instance(),SIGNAL(currentNameChanged(QString)),this,SLOT(hidePresenceDock()));
+   connect(AccountListModel::instance(),SIGNAL(presenceEnabledChanged(bool)),this,SLOT(slotPresenceEnabled(bool)));
 
    m_pPresenceDock = new QDockWidget(this);
    m_pPresenceDock->setObjectName("presence-dock");
    m_pPresenceDock->setWidget(new Presence(m_pPresenceDock));
-   m_pPresenceDock->setAllowedAreas(Qt::BottomDockWidgetArea);
+   m_pPresenceDock->setAllowedAreas(Qt::TopDockWidgetArea);
    m_pPresenceDock->setFeatures(QDockWidget::NoDockWidgetFeatures);
-   addDockWidget( Qt::BottomDockWidgetArea, m_pPresenceDock );
+   addDockWidget( Qt::TopDockWidgetArea, m_pPresenceDock );
    m_pPresenceDock->setVisible(false);
    if (m_pPresenceDock->titleBarWidget())
       m_pPresenceDock->titleBarWidget()->setVisible(false);
@@ -336,8 +362,8 @@ SFLPhone::~SFLPhone()
    delete m_pPresent         ;
    delete m_pPresenceDock    ;
 
-   delete AkonadiBackend::instance();
    delete CallModel::instance();
+   delete ContactModel::instance();
    //saveState();
 }
 
@@ -492,6 +518,12 @@ void SFLPhone::on_m_pView_incomingCall(const Call* call)
    }
 }
 
+///Hide or show the statusbar presence widget
+void SFLPhone::slotPresenceEnabled(bool state)
+{
+   m_pPresent->setVisible(state && AccountListModel::instance()->isPresencePublishSupported());
+}
+
 ///Change current account
 void SFLPhone::currentAccountIndexChanged(int newIndex)
 {
@@ -558,6 +590,8 @@ void SFLPhone::displayVideoDock(VideoRenderer* r)
 {
    if (!m_pVideoDW) {
       m_pVideoDW = new VideoDock(this);
+      addDockWidget( Qt::TopDockWidgetArea, m_pVideoDW  );
+      m_pVideoDW->setFloating(true);
    }
    m_pVideoDW->addRenderer(r);
    m_pVideoDW->show();
