@@ -64,8 +64,6 @@
 //sflphone library
 #include "klib/kcfg_settings.h"
 #include "lib/phonenumber.h"
-#include "lib/typedefs.h"
-#include "lib/dbus/configurationmanager.h"
 #include "lib/dbus/callmanager.h"
 #include "lib/dbus/instancemanager.h"
 #include "lib/sflphone_const.h"
@@ -80,6 +78,18 @@
 
 #define IM_ACTIVE m_pMessageTabBox->isVisible()
 
+#define ACTION_LABEL_CALL                 i18n("New call")
+#define ACTION_LABEL_PLACE_CALL           i18n("Place call")
+#define ACTION_LABEL_HANG_UP              i18n("Hang up")
+#define ACTION_LABEL_HOLD                 i18n("Hold on")
+#define ACTION_LABEL_TRANSFER             i18n("Transfer")
+#define ACTION_LABEL_RECORD               i18n("Record")
+#define ACTION_LABEL_ACCEPT               i18n("Pick up")
+#define ACTION_LABEL_REFUSE               i18n("Hang up")
+#define ACTION_LABEL_UNHOLD               i18n("Hold off")
+#define ACTION_LABEL_GIVE_UP_TRANSF       i18n("Give up transfer")
+#define ACTION_LABEL_MAILBOX              i18n("Voicemail")
+
 ///Constructor
 SFLPhoneView::SFLPhoneView(QWidget *parent)
    : QWidget(parent),m_pTransferOverlay(nullptr),m_pAutoCompletion(nullptr)
@@ -140,6 +150,9 @@ SFLPhoneView::SFLPhoneView(QWidget *parent)
    /**/connect(m_pView                      , SIGNAL(itemDoubleClicked(QModelIndex)), m_pEventManager, SLOT(enter())                          );
    /*                                                                                                                                        */
 
+   //Volume controls
+//    connect(AudioSettingsModel::instance(),SIGNAL(playbackVolumeChanged(int)),slider_sndVol,SLOT(setValue(int)));
+
    //Auto completion
    loadAutoCompletion();
 
@@ -167,8 +180,6 @@ void SFLPhoneView::loadWindow()
    updateWindowCallState ();
    updateVolumeControls  ();
    loadAutoCompletion    ();
-   slider_recVol->setValue(AudioSettingsModel::instance()->captureVolume());
-   slider_sndVol->setValue(AudioSettingsModel::instance()->playbackVolume());
    widget_dialpad->setVisible(ConfigurationSkeleton::displayDialpad());
    AudioSettingsModel::instance()->setEnableRoomTone(ConfigurationSkeleton::enableRoomTone());
 }
@@ -273,7 +284,7 @@ void SFLPhoneView::updateWindowCallState()
       enabledActions[ SFLPhone::CallAction::Record   ] = false;
       m_pMessageBoxW->setVisible(false);
    }
-   else if (call->isConference()) {
+   else if (call->type() == Call::Type::CONFERENCE) {
       //TODO Something to do?
    }
    else {
@@ -290,6 +301,7 @@ void SFLPhoneView::updateWindowCallState()
             break;
 
          case Call::State::RINGING:
+         case Call::State::INITIALIZATION:
             enabledActions  [ SFLPhone::CallAction::Hold     ] = false                       ;
             enabledActions  [ SFLPhone::CallAction::Transfer ] = false                       ;
             m_pMessageBoxW->setVisible(false)                                    ;
@@ -372,7 +384,7 @@ void SFLPhoneView::updateWindowCallState()
             enabledActions  [ SFLPhone::CallAction::Transfer ] = false                       ;
             m_pMessageBoxW->setVisible(false)                                    ;
             break;
-         case Call::State::COUNT:
+         case Call::State::__COUNT:
          default:
             kDebug() << "Error : Reached unexisting state for call "  << call->id() << "(" << call->state() << "!";
             break;
@@ -383,8 +395,8 @@ void SFLPhoneView::updateWindowCallState()
          int activeCallCounter=0;
          foreach (Call* call2, CallModel::instance()->getCallList()) {
             if (dynamic_cast<Call*>(call2)) {
-               activeCallCounter += (call2->state() == Call::State::CURRENT || call2->state() == Call::State::HOLD);
-               activeCallCounter -= (call2->state() == Call::State::INCOMING || call2->state() ==Call::State::RINGING)*1000;
+               activeCallCounter += (call2->lifeCycleState() == Call::LifeCycleState::PROGRESS)?1:0;
+               activeCallCounter -= (call2->lifeCycleState() == Call::LifeCycleState::INITIALIZATION)*1000;
             }
          }
          if (activeCallCounter >= 2 && !CallModel::instance()->getConferenceList().size()) {
@@ -408,7 +420,7 @@ void SFLPhoneView::loadAutoCompletion()
          m_pAutoCompletion = new AutoCompletion(m_pView);
          PhoneDirectoryModel::instance()->setCallWithAccount(ConfigurationSkeleton::autoCompleteUseAccount());
          m_pAutoCompletion->setUseUnregisteredAccounts(ConfigurationSkeleton::autoCompleteMergeNumbers());
-         connect(m_pAutoCompletion, SIGNAL(requestVisibility(bool)), m_pEventManager, SLOT(slotAutoCompletionVisibility(bool)));
+         connect(m_pAutoCompletion, SIGNAL(requestVisibility(bool,bool)), m_pEventManager, SLOT(slotAutoCompletionVisibility(bool,bool)));
          connect(m_pAutoCompletion,SIGNAL(doubleClicked(PhoneNumber*)),this,SLOT(slotAutoCompleteClicked(PhoneNumber*)));
       }
    }
@@ -425,6 +437,16 @@ void SFLPhoneView::updateVolumeControls()
    toolButton_sndVol->setVisible ( ActionCollection::instance()->displayVolumeControlsAction()->isChecked() && ConfigurationSkeleton::displayVolume() );
    slider_recVol->setVisible     ( ActionCollection::instance()->displayVolumeControlsAction()->isChecked() && ConfigurationSkeleton::displayVolume() );
    slider_sndVol->setVisible     ( ActionCollection::instance()->displayVolumeControlsAction()->isChecked() && ConfigurationSkeleton::displayVolume() );
+
+   //Prevent an infinite update loop
+   slider_recVol->blockSignals(true);
+   slider_sndVol->blockSignals(true);
+
+   slider_recVol->setValue(AudioSettingsModel::instance()->captureVolume());
+   slider_sndVol->setValue(AudioSettingsModel::instance()->playbackVolume());
+
+   slider_recVol->blockSignals(false);
+   slider_sndVol->blockSignals(false);
 }
 
 
