@@ -38,6 +38,7 @@
 #include <KInputDialog>
 #include <KAction>
 #include <KLocale>
+#include <KMessageBox>
 
 //SFLPhone
 #include "sflphone.h"
@@ -50,12 +51,12 @@
 #include "lib/historymodel.h"
 #include "lib/call.h"
 #include "lib/contact.h"
+#include "lib/contactmodel.h"
 #include "lib/numbercategory.h"
 #include "lib/phonedirectorymodel.h"
 #include "lib/phonenumber.h"
 #include "lib/accountlistmodel.h"
 #include "klib/helperfunctions.h"
-#include "klib/akonadibackend.h"
 #include "klib/kcfg_settings.h"
 #include "../lib/contactproxymodel.h"
 #include "../delegates/categorizeddelegate.h"
@@ -80,7 +81,7 @@ bool KeyPressEaterC::eventFilter(QObject *obj, QEvent *event)
 
 bool ContactSortFilterProxyModel::filterAcceptsRow ( int source_row, const QModelIndex & source_parent ) const
 {
-   const bool status = sourceModel()->index(source_row,0,source_parent).data(AbstractContactBackend::Role::Active).toBool();
+   const bool status = sourceModel()->index(source_row,0,source_parent).data(ContactModel::Role::Active).toBool();
    if (!status)
       return false;
    else if (!source_parent.isValid() || source_parent.parent().isValid())
@@ -119,12 +120,12 @@ ContactDock::ContactDock(QWidget* parent) : QDockWidget(parent),m_pCallAgain(nul
    m_pView->setDelegate(m_pCategoryDelegate);
    m_pView->setViewType(CategorizedTreeView::ViewType::Contact);
 
-   m_pSourceModel = new ContactProxyModel(AkonadiBackend::instance(),Qt::DisplayRole,false);
+   m_pSourceModel = new ContactProxyModel(Qt::DisplayRole,false);
    m_pProxyModel = new ContactSortFilterProxyModel(this);
    m_pProxyModel->setSourceModel(m_pSourceModel);
    m_pProxyModel->setSortRole(Qt::DisplayRole);
    m_pProxyModel->setSortLocaleAware(true);
-   m_pProxyModel->setFilterRole(AbstractContactBackend::Role::Filter);
+   m_pProxyModel->setFilterRole(ContactModel::Role::Filter);
    m_pProxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    m_pProxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    m_pView->setModel(m_pProxyModel);
@@ -132,7 +133,7 @@ ContactDock::ContactDock(QWidget* parent) : QDockWidget(parent),m_pCallAgain(nul
    m_pView->setSortingEnabled(true);
    m_pView->sortByColumn(0,Qt::AscendingOrder);
    connect(m_pView,SIGNAL(contextMenuRequest(QModelIndex))     , this , SLOT(slotContextMenu(QModelIndex)));
-   connect(m_pProxyModel ,SIGNAL(layoutChanged()), this , SLOT(expandTree())                );
+   connect(m_pProxyModel ,SIGNAL(layoutChanged()), this , SLOT(expandTree()));
    connect(m_pFilterLE ,SIGNAL(filterStringChanged(QString)), m_pProxyModel , SLOT(setFilterRegExp(QString)));
    connect(m_pFilterLE ,SIGNAL(textChanged(QString)), this , SLOT(expandTree()));
 
@@ -143,7 +144,7 @@ ContactDock::ContactDock(QWidget* parent) : QDockWidget(parent),m_pCallAgain(nul
    connect(m_pSortByCBB                  ,SIGNAL(currentIndexChanged(int)),                             this, SLOT(setCategory(int))                    );
 //    connect(ConfigurationSkeleton::self() ,SIGNAL(configChanged()),                                      this, SLOT(reloadContact())                     );
    connect(m_pView                       ,SIGNAL(doubleClicked(QModelIndex)),                           this, SLOT(slotDoubleClick(QModelIndex))        );
-   setWindowTitle(i18n("Contact"));
+   setWindowTitle(i18nc("Contact tab","Contact"));
 } //ContactDock
 
 ///Destructor
@@ -157,6 +158,7 @@ ContactDock::~ContactDock()
       delete m_pEmail       ;
       delete m_pAddPhone    ;
       delete m_pBookmark    ;
+      delete m_pRemove      ;
    }
    //Delegates
    delete m_pCategoryDelegate;
@@ -262,12 +264,18 @@ void ContactDock::showContext(const QModelIndex& index)
       m_pBookmark->setText        ( i18n("Bookmark")           );
       m_pBookmark->setIcon        ( KIcon("bookmarks")         );
 
+      m_pRemove      = new KAction(this);
+      m_pRemove->setShortcut    ( Qt::CTRL + Qt::Key_Shift + Qt::Key_Delete );
+      m_pRemove->setText        ( i18n("Delete")               );
+      m_pRemove->setIcon        ( KIcon("edit-delete")         );
+
       connect(m_pCallAgain    , SIGNAL(triggered()) , this,SLOT(callAgain())  );
       connect(m_pEditContact  , SIGNAL(triggered()) , this,SLOT(editContact()));
       connect(m_pCopy         , SIGNAL(triggered()) , this,SLOT(copy())       );
       connect(m_pEmail        , SIGNAL(triggered()) , this,SLOT(sendEmail())  );
       connect(m_pAddPhone     , SIGNAL(triggered()) , this,SLOT(addPhone())   );
       connect(m_pBookmark     , SIGNAL(triggered()) , this,SLOT(bookmark())   );
+      connect(m_pRemove       , SIGNAL(triggered()) , this,SLOT(slotDelete()) );
    }
    if (index.parent().isValid()  && !index.parent().parent().isValid()) {
       Contact* ct = (Contact*)((CategorizedCompositeNode*)(static_cast<const QSortFilterProxyModel*>(index.model()))->mapToSource(index).internalPointer())->getSelf();
@@ -293,6 +301,7 @@ void ContactDock::showContext(const QModelIndex& index)
       m_pMenu->addAction( m_pEditContact );
       m_pMenu->addAction( m_pAddPhone    );
       m_pMenu->addAction( m_pCopy        );
+      m_pMenu->addAction( m_pRemove      );
       m_pMenu->addAction( m_pEmail       );
       m_pMenu->addAction( m_pBookmark    );
    }
@@ -312,7 +321,7 @@ void ContactDock::sendEmail()
 void ContactDock::callAgain(const PhoneNumber* n)
 {
    kDebug() << "Calling ";
-   bool ok;
+   bool ok = false;
    const PhoneNumber* number = n?n:showNumberSelector(ok);
    if ( (n || ok) && number) {
       const QString name = n?n->contact()->formattedName() : m_pCurrentContact->formattedName();
@@ -334,7 +343,7 @@ void ContactDock::copy()
 {
    kDebug() << "Copying contact";
    QMimeData* mimeData = new QMimeData();
-   mimeData->setData(MIME_CONTACT, m_pCurrentContact->uid().toUtf8());
+   mimeData->setData(MIME_CONTACT, m_pCurrentContact->uid());
    QString numbers(m_pCurrentContact->formattedName()+": ");
    QString numbersHtml("<b>"+m_pCurrentContact->formattedName()+"</b><br />");
    foreach (PhoneNumber* number, m_pCurrentContact->phoneNumbers()) {
@@ -351,7 +360,7 @@ void ContactDock::copy()
 void ContactDock::editContact()
 {
    kDebug() << "Edit contact";
-   AkonadiBackend::instance()->editContact(m_pCurrentContact);
+   m_pCurrentContact->edit();
 }
 
 ///Add a new phone number for this contact
@@ -359,10 +368,11 @@ void ContactDock::editContact()
 void ContactDock::addPhone()
 {
    kDebug() << "Adding to contact";
-   bool ok;
+   bool ok = false;
    const QString text = KInputDialog::getText( i18n("Enter a new number"), i18n("New number:"), QString(), &ok,this);
    if (ok && !text.isEmpty()) {
-      AkonadiBackend::instance()->addPhoneNumber(m_pCurrentContact,text,"work");
+      PhoneNumber* n = PhoneDirectoryModel::instance()->getNumber(text,"work");
+      m_pCurrentContact->addPhoneNumber(n);
    }
 }
 
@@ -375,11 +385,22 @@ void ContactDock::bookmark()
    }
 }
 
+void ContactDock::slotDelete()
+{
+   if (!m_pCurrentContact)
+      return;
+   const int ret = KMessageBox::questionYesNo(this, i18n("Are you sure you want to permanently delete %1?",
+      m_pCurrentContact->formattedName()), i18n("Delete contact"));
+   if (ret == KMessageBox::Yes) {
+      m_pCurrentContact->remove();
+   }
+}
+
 ///Called when a call is dropped on transfer
 void ContactDock::transferEvent(QMimeData* data)
 {
    if (data->hasFormat( MIME_CALLID)) {
-      bool ok;
+      bool ok = false;
       const PhoneNumber* result = showNumberSelector(ok);
       if (ok && result) {
          Call* call = CallModel::instance()->getCall(data->data(MIME_CALLID));
@@ -466,24 +487,24 @@ void ContactDock::setCategory(int index)
          m_pProxyModel->setSortRole(Qt::DisplayRole);
          break;
       case SortingCategory::Organization:
-         m_pProxyModel->setSortRole(AbstractContactBackend::Role::Organization);
-         m_pSourceModel->setRole(AbstractContactBackend::Role::Organization);
+         m_pProxyModel->setSortRole(ContactModel::Role::Organization);
+         m_pSourceModel->setRole(ContactModel::Role::Organization);
          m_pSourceModel->setShowAll(true);
          break;
       case SortingCategory::RecentlyUsed:
-         m_pSourceModel->setRole(AbstractContactBackend::Role::FormattedLastUsed);
+         m_pSourceModel->setRole(ContactModel::Role::FormattedLastUsed);
          m_pSourceModel->setShowAll(true);
-         m_pProxyModel->setSortRole(AbstractContactBackend::Role::IndexedLastUsed);
+         m_pProxyModel->setSortRole(ContactModel::Role::IndexedLastUsed);
          break;
       case SortingCategory::Group:
-         m_pSourceModel->setRole(AbstractContactBackend::Role::Group);
+         m_pSourceModel->setRole(ContactModel::Role::Group);
          m_pSourceModel->setShowAll(true);
-         m_pProxyModel->setSortRole(AbstractContactBackend::Role::Group);
+         m_pProxyModel->setSortRole(ContactModel::Role::Group);
          break;
       case SortingCategory::Department:
-         m_pSourceModel->setRole(AbstractContactBackend::Role::Department);
+         m_pSourceModel->setRole(ContactModel::Role::Department);
          m_pSourceModel->setShowAll(true);
-         m_pProxyModel->setSortRole(AbstractContactBackend::Role::Department);
+         m_pProxyModel->setSortRole(ContactModel::Role::Department);
          break;
    };
 }
@@ -524,5 +545,5 @@ void ContactDock::keyPressEvent(QKeyEvent* event) {
 ///Expand the tree according to the user preferences
 void ContactDock::expandTree()
 {
-   m_pView->expandToDepth( 1 );
+   m_pView->expandToDepth( 2 );
 }
