@@ -48,13 +48,13 @@
 #include "sflphone.h"
 #include "widgets/categorizedtreeview.h"
 #include "widgets/bookmarkdock.h"
-#include "klib/akonadibackend.h"
 #include "klib/kcfg_settings.h"
 #include "lib/historymodel.h"
 #include "lib/accountlistmodel.h"
 #include "lib/callmodel.h"
 #include "lib/phonenumber.h"
 #include "lib/phonedirectorymodel.h"
+#include "lib/contactmodel.h"
 #include "../delegates/categorizeddelegate.h"
 #include "../delegates/historydelegate.h"
 
@@ -195,7 +195,7 @@ HistoryDock::HistoryDock(QWidget* parent) : QDockWidget(parent),m_pMenu(nullptr)
    m_pBottomWidget->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    m_pBottomWidget->layout()->addItem(mainLayout);
 
-   setWindowTitle(i18n("History"));
+   setWindowTitle(i18nc("History tab","History"));
 
    QDate date(2000,1,1);
    m_pFromDW->setDate(date);
@@ -437,7 +437,7 @@ void HistoryDock::slotAddContact()
    numbers << PhoneDirectoryModel::instance()->getNumber(m_pCurrentCall->peerPhoneNumber()->uri(),aContact,nullptr, "Home");//new PhoneNumber(m_pCurrentCall->peerPhoneNumber(), "Home");
    aContact->setPhoneNumbers(numbers);
    aContact->setFormattedName(m_pCurrentCall->peerName());
-   AkonadiBackend::instance()->addNewContact(aContact);
+   ContactModel::instance()->addNewContact(aContact);
 }
 
 void HistoryDock::slotAddToContact()
@@ -453,6 +453,8 @@ void HistoryDock::slotBookmark()
 
 void HistoryDock::slotDoubleClick(const QModelIndex& index)
 {
+   if (!index.isValid())
+      return;
    QModelIndex idx = (static_cast<const HistorySortFilterProxyModel*>(index.model()))->mapToSource(index);
    if (!idx.isValid() || !idx.parent().isValid())
       return;
